package com.diagbot.aggregate;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.facade.TransferFacade;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/11/14 16:11
 */
@Component
public class SearchItemAggregate {
    @Autowired
    private TransferFacade transferFacade;

    @DataProvider("assembleSearchData")
    public SearchVO assembleSearchData(@InvokeParameter("searchVO") SearchVO searchVO,
                                       @InvokeParameter("isConnect") Boolean isConnect,
                                       @DataConsumer("setLis") List<Lis> lis,
                                       @DataConsumer("setPacs") List<Pacs> pacs,
                                       @DataConsumer("setDiag") List<Item> diag,
                                       @DataConsumer("setDrug") List<Item> drug,
                                       @DataConsumer("setLisOrder") List<Lis> lisOrder,
                                       @DataConsumer("setPacsOrder") List<Pacs> pacsOrder,
                                       @DataConsumer("setDiagOrder") List<Item> diagOrder,
                                       @DataConsumer("setDrugOrder") List<Item> drugOrder,
                                       @DataConsumer("setOperationOrder") List<Item> operationOrder,
                                       @DataConsumer("setOtherOrder") List<Item> otherOrder,
                                       @DataConsumer("setDiseaseName") Item diseaseName
    ) {
        if (isConnect) {
            //化验数据映射公表（化验公表项）
            searchVO.setLis(lis);
            //辅检数据映射公表（辅检项目）
            searchVO.setPacs(pacs);
            //诊断数据映射公表（诊断）
            searchVO.setDiag(diag);
            //药品数据映射公表（药品通用名）
            searchVO.setDrug(drug);
            //化验开单项映射
            searchVO.setLisOrder(lisOrder);
            //辅检开单项映射
            searchVO.setPacsOrder(pacsOrder);
            //诊断开单项映射
            searchVO.setDiagOrder(diagOrder);
            //药品开单项映射
            searchVO.setDrugOrder(drugOrder);
            //手术开单项映射
            searchVO.setOperationOrder(operationOrder);
            //其他开单项映射
            searchVO.setOtherOrder(otherOrder);
            //选中诊断映射
            searchVO.setDiseaseName(diseaseName);
        }
        return searchVO;
    }

    /**
     * 化验增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setLis")
    public List<Lis> setLis(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Lis> lis = searchVO.getLis();
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            lis = transferFacade.addLisUniqueName(lis, searchVO.getHosCode());
        }
        return lis;
    }

    /**
     * 辅检增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setPacs")
    public List<Pacs> setPacs(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Pacs> pacs = searchVO.getPacs();
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            pacs = transferFacade.addPacsUniqueName(pacs, searchVO.getHosCode());
        }
        return pacs;
    }

    /**
     * 诊断增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setDiag")
    public List<Item> setDiag(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Item> diag = searchVO.getDiag();
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            diag = transferFacade.addItemUniqueName(diag, ConceptTypeEnum.Disease.getKey(), searchVO.getHosCode());
        }
        return diag;
    }

    /**
     * 药品增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setDrug")
    public List<Item> setDrug(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Item> drug = searchVO.getDrug();
        drug = transferFacade.addItemUniqueName(drug, ConceptTypeEnum.Drug.getKey(), searchVO.getHosCode());
        return drug;
    }

    /**
     * 化验开单项增加公表名称
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setLisOrder")
    public List<Lis> setLisOrder(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Lis> lisOrder = searchVO.getLisOrder();
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            lisOrder = transferFacade.addLisUniqueName(lisOrder, searchVO.getHosCode());
        }
        return lisOrder;
    }

    /**
     * 辅检开单项增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setPacsOrder")
    public List<Pacs> setPacsOrder(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Pacs> pacsOrder = searchVO.getPacsOrder();
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            pacsOrder = transferFacade.addPacsUniqueName(pacsOrder, searchVO.getHosCode());
        }
        return pacsOrder;
    }

    /**
     * 诊断开单项增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setDiagOrder")
    public List<Item> setDiagOrder(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Item> diagOrder = searchVO.getDiagOrder();
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            diagOrder
                    = transferFacade.addItemUniqueName(diagOrder, ConceptTypeEnum.Disease.getKey(), searchVO.getHosCode());
        }
        return diagOrder;
    }

    /**
     * 药品开单项增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setDrugOrder")
    public List<Item> setDrugOrder(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Item> drugOrder = searchVO.getDrugOrder();
        drugOrder = transferFacade.addItemUniqueName(drugOrder, ConceptTypeEnum.Drug.getKey(), searchVO.getHosCode());
        return drugOrder;
    }

    /**
     * 手术开单项增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setOperationOrder")
    public List<Item> setOperationOrder(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Item> operationOrder = searchVO.getOperationOrder();
        //数据引擎模式和页面推送模式对手术开单项入参做公表映射
        if (searchVO.getSysType().intValue() == SysTypeEnum.DATA_SERVICE.getKey()
                || searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            operationOrder
                    = transferFacade.addItemUniqueName(operationOrder, ConceptTypeEnum.OPERATION.getKey(), searchVO.getHosCode());
        }
        return operationOrder;
    }

    /**
     * 其他开单项增加公表项（暂未处理）
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setOtherOrder")
    public List<Item> setOtherOrder(@InvokeParameter("searchVO") SearchVO searchVO) {
        List<Item> operationOrder = searchVO.getOperationOrder();
        return operationOrder;
    }

    /**
     * 诊断开单项增加公表项
     *
     * @param searchVO
     * @return
     */
    @DataProvider("setDiseaseName")
    public Item setDiseaseName(@InvokeParameter("searchVO") SearchVO searchVO) {
        if (searchVO.getDiseaseName() == null) {
            return null;
        }
        List<Item> diseaseItem = Lists.newLinkedList();
        diseaseItem.add(searchVO.getDiseaseName());
        //数据引擎模式映射数据
        if (searchVO.getSysType().intValue() == SysTypeEnum.LTAPI_SERVICE.getKey()) {
            diseaseItem = transferFacade.addItemUniqueName(diseaseItem, ConceptTypeEnum.Disease.getKey(), searchVO.getHosCode());
        }
        if (ListUtil.isNotEmpty(diseaseItem)) {
            return diseaseItem.get(0);
        } else {
            return null;
        }
    }
}