package com.diagbot.client;

import com.diagbot.client.bean.Response;
import com.diagbot.client.hystrix.NLPServiceHystrix;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.Lexeme;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description:调用NLP对接层服务
 * @Author:zhaops
 * @time: 2019/4/2 15:31
 */
@FeignClient(name = "NLP", url = "${nlp.server.address}", fallback = NLPServiceHystrix.class)
public interface NLPServiceClient {
    /**
     * 症状特征提取
     *
     * @param text
     * @return
     */
    @GetMapping(value = "/nlp-web/feature/find_symptom_feature")
    Response<List<FeatureConceptDTO>> symptomFeaturePageData(@RequestParam("text") String text);

    /**
     * 分词
     *
     * @param text
     * @return
     */
    @PostMapping(value = "/nlp-web/participle/split")
    Response<List<Lexeme>> split(@RequestParam("content") String text);
}