package com.diagbot.client;

import com.diagbot.client.bean.HosCodeVO;
import com.diagbot.client.hystrix.TranServiceHystrix;
import com.diagbot.dto.GetDiseaseIcdDTO;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.GetTopPatientInfoDjDTO;
import com.diagbot.dto.HospitalDeptInfoDTO;
import com.diagbot.dto.IndexDataDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.vo.DiseaseIcdVO;
import com.diagbot.vo.GetDiseaseIcdVO;
import com.diagbot.vo.GetTopPatientInfoDjVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.HospitalDeptInfoVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.IndexDataSaveVO;
import com.diagbot.vo.IndexDataiIds;
import com.diagbot.vo.LisConfigVO;
import com.diagbot.vo.OperationConfigVO;
import com.diagbot.vo.PacsConfigVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: 调用信息对接层服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "tran-service", fallback = TranServiceHystrix.class)
public interface TranServiceClient {
    /**
     * 获取科室信息
     *
     * @param hospitalDeptInfoVO
     * @return
     */
    @PostMapping("/hospitalDeptInfo/getHospitalDeptInfo")
    RespDTO<HospitalDeptInfoDTO> getHospitalDeptInfo(@RequestBody HospitalDeptInfoVO hospitalDeptInfoVO);

    /**
     * 根据医院编号疾病概念名称获取各医院对应的诊断icd
     *
     * @param getDiseaseIcdVO
     * @return
     */
    @PostMapping("/diseaseIcd/getDiseaseIcds")
    RespDTO<List<GetDiseaseIcdDTO>> getDiseaseIcds(@RequestBody GetDiseaseIcdVO getDiseaseIcdVO);

    /**
     * 获取页面顶部病人医生科室信息
     *
     * @param getTopPatientInfoVO
     * @return
     */
    @PostMapping("/patientInfo/getTopPatientInfo")
    RespDTO<GetTopPatientInfoDTO> getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO);

    /**
     * 对接-获取页面顶部病人医生科室信息
     *
     * @param getTopPatientInfoDjVO
     * @return
     */
    @PostMapping("/patientInfo/getTopPatientInfoDj")
    RespDTO<GetTopPatientInfoDjDTO> getTopPatientInfoDj(@RequestBody GetTopPatientInfoDjVO getTopPatientInfoDjVO);

    /**
     * 根据医院编码和套餐名称获取化验公表映射关系
     *
     * @param lisConfigVO
     * @return
     */
    @PostMapping("/tranLisConfig/getLisConfigByMealNameAndHosCode")
    RespDTO<Map<String, Map<String, List<String>>>> getLisConfigByMealNameAndHosCode(@RequestBody LisConfigVO lisConfigVO);

    /**
     * 根据医院编码获取辅检公表映射关系
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/pacsConfig/getPacsConfigByHosCode")
    RespDTO<Map<String, String>> getPacsConfigByHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 获取辅检映射关系
     *
     * @param pacsConfigVO
     * @return
     */
    @PostMapping("/pacsConfig/getPacsConfig")
    RespDTO<Map<String, List<String>>> getPacsConfig(@RequestBody PacsConfigVO pacsConfigVO);

    /**
     * 获取手术映射关系
     *
     * @param operationConfigVO
     * @return
     */
    @PostMapping("/operationConfig/getOperationConfig")
    RespDTO<Map<String, List<String>>> getOperationConfig(@RequestBody OperationConfigVO operationConfigVO);

    /**
     * 根据医院编码获取提示信息标题映射关系
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/introduceTitleConfig/getTitleMappingHosCode")
    RespDTO<Map<String, String>> getTitleMappingHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 根据医院编码查询诊断icd映射
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/diseaseIcd/getDiseaseIcdByHosCode")
    RespDTO<Map<String, String>> getDiseaseIcdByHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 获取诊断名称映射map
     *
     * @param diseaseIcdVO
     * @return
     */
    @PostMapping("/diseaseIcd/getDiseaseIcdMap")
    RespDTO<Map<String, String>> getDiseaseIcdMap(DiseaseIcdVO diseaseIcdVO);

    /**
     * 数据服务模式是否对外对接
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/dataService/isConnect")
    RespDTO<Boolean> isConnect(HosCodeVO hosCodeVO);

    /**
     * 根据医院编码获取化验公表映射关系，公表项做key
     *
     * @param lisConfigVO
     * @return
     */
    @PostMapping("/tranLisConfig/getLisConfigByUniqueNameAndHosCode")
    RespDTO<Map<String, List<String>>> getLisConfigByUniqueNameAndHosCode(@RequestBody LisConfigVO lisConfigVO);

    /**
     * 根据医院编码查询辅检公表映射，公表项做key
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/pacsConfig/getPacsConfigByUniqueNameAndHosCode")
    RespDTO<Map<String, List<String>>> getPacsConfigByUniqueNameAndHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 指标数据获取
     *
     * @param indexDataFindVO
     * @return
     */
    @PostMapping(value = "/indexData/getIndexDatas")
    RespDTO<List<IndexDataDTO>> getIndexDatas(@RequestBody IndexDataiIds indexDataFindVO);

    /**
     * 指标数据保存
     *
     * @param indexDataSaveVO
     * @return
     */
    @PostMapping(value = "/indexData/saveIndexDatas")
    RespDTO<Boolean> saveIndexDatas(@RequestBody IndexDataSaveVO indexDataSaveVO);

    @PostMapping("/sysSet/getSysSetInfoDatas")
    RespDTO<List<SysSetInfoDTO>> getSysSetInfoDatas(@Valid @RequestBody HospitalSetVO hospitalSetVO);
}