package com.diagbot.facade;

import com.alibaba.fastjson.JSON;
import com.diagbot.biz.push.entity.ResponseData;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.AIServiceClient;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.Response;
import com.diagbot.client.bean.SearchData;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.entity.ScaleContent;
import com.diagbot.enums.ScaleTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.ScaleContentService;
import com.diagbot.util.CryptUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 数据处理业务层
 * </p>
 *
 * @author Mark Huang
 * @since 2019-03-04
 */
@Component
public class ClinicalFacade {

    @Autowired
    private AIServiceClient aiServiceClient;
    @Autowired
    private ScaleContentService scaleContentService;
    @Autowired
    private AssembleFacade assembleFacade;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 处理临床数据
     *
     * @param searchVO
     * @return ResponseData
     */
    public ResponseData processClinicalData(@RequestBody SearchVO searchVO) {
        SearchData searchData = assembleFacade.assembleData(searchVO);
        Response<ResponseData> res = aiServiceClient.bayesPageData(searchData);
        if (res == null || res.getData() == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "AI没有返回结果");
        }
        return res.getData();
    }

    /**
     * 获取量表内容
     *
     * @param searchVO
     * @return
     */
    public List<ScaleContent> getScale(SearchVO searchVO) {
        SearchData searchData = assembleFacade.assembleData(searchVO);
        Map<String, Object> response = aiServiceClient.scale(searchData);
        if (null == response) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "量表没有返回结果");
        }
        String scaleName = searchData.getScaleName();
        //加密
        if (cryptConfiguer.cryptFlag) {
            scaleName = CryptUtil.encrypt_char(searchData.getScaleName());
        }
        List<ScaleContent> list = getContent(scaleName, JSON.toJSONString(response));
        return list;
    }

    /**
     * 获取量表内容
     *
     * @param scaleName
     * @param pushContent
     * @return
     */
    public List<ScaleContent> getContent(String scaleName, String pushContent) {
        List<ScaleContent> list = scaleContentService.getContentByName(scaleName);
        for (ScaleContent scaleContent : list) {
            if (ScaleTypeEnum.PUSH.getKey() == scaleContent.getType()) {
                scaleContent.setContent(pushContent);
                break;
            }
        }
        return list;
    }

    /**
     * 计算接口
     *
     * @param calculateData
     * @return
     */
    public Map<String, Object> scaleCalc(CalculateData calculateData) {
        Map<String, Object> response = aiServiceClient.scaleCalc(calculateData);
        if (response == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "计算没有返回结构");
        }
        return response;
    }
}