package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.TranServiceClient;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.enums.PositionTypeEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ConceptDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.ParamConvertUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ConnectIntroduceVO;
import com.diagbot.vo.DiseaseIcdVO;
import com.diagbot.vo.ExistListByConceptIdsVO;
import com.diagbot.vo.PacsConfigVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:提示信息
 * @Author:zhaops
 * @time: 2019/5/6 15:03
 */
@Component
public class ConceptDetailFacade extends ConceptDetailServiceImpl {
    @Autowired
    private ConceptFacade conceptFacade;
    @Autowired
    private TransferFacade transferFacade;
    @Autowired
    private TranServiceClient tranServiceClient;

    /**
     * 单条获取静态知识
     *
     * @param conceptIntroduceVO
     * @return
     */
    public ConceptIntroduceDTO getConceptDetail(ConceptIntroduceVO conceptIntroduceVO) {
        if (StringUtil.isBlank(conceptIntroduceVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入概念名称");
        }
        conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
        //验证存在并且获取概念基本信息
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        BeanUtil.copyProperties(conceptIntroduceVO, conceptBaseVO);
        Concept concept = conceptFacade.exist(conceptBaseVO);
        //提示信息明细
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .orderByAsc("order_no");
        if (conceptIntroduceVO.getPosition() == null) {
            if (conceptIntroduceVO.getLibType().equals(LexiconTypeEnum.DRUGS.getKey())) {
                conceptIntroduceVO.setPosition(PositionTypeEnum.T5.getKey());
            } else if (conceptIntroduceVO.getLibType().equals(LexiconTypeEnum.SIDE_EFFECTS.getKey())) {
                conceptIntroduceVO.setPosition(PositionTypeEnum.T6.getKey());
            } else {
                conceptIntroduceVO.setPosition(PositionTypeEnum.T2.getKey());
            }
        }
        //position=0，返回所有位置的内容
        if (!conceptIntroduceVO.getPosition().equals(0)) {
            conceptDetailQueryWrapper.apply("find_in_set({0},position)", conceptIntroduceVO.getPosition());
        }
        if (ListUtil.isNotEmpty(conceptIntroduceVO.getTitles())) {
            conceptDetailQueryWrapper.in("title", conceptIntroduceVO.getTitles());
        }
        List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
        List<ConceptDetailDTO> conceptDetailDTOList = ListUtil.newArrayList();
        if (ListUtil.isNotEmpty(conceptDetailList)) {
            conceptDetailDTOList = BeanUtil.listCopyTo(conceptDetailList, ConceptDetailDTO.class);
        }
        ConceptIntroduceDTO conceptIntroduceDTO = new ConceptIntroduceDTO();
        conceptIntroduceDTO.setConceptId(concept.getId());
        conceptIntroduceDTO.setName(concept.getLibName());
        conceptIntroduceDTO.setLibType(concept.getLibType());
        conceptIntroduceDTO.setType(ParamConvertUtil.libConvert2Concept(concept.getLibType().intValue()));
        conceptIntroduceDTO.setDetails(conceptDetailDTOList);
        return conceptIntroduceDTO;
    }

    /**
     * 批量取静态知识
     *
     * @param conceptIntroduceVOList
     * @return
     */
    public List<ConceptIntroduceDTO> getConceptDetails(List<ConceptIntroduceVO> conceptIntroduceVOList) {
        List<ConceptIntroduceDTO> conceptIntroduceDTOList = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(conceptIntroduceVOList)) {
            for (ConceptIntroduceVO conceptIntroduceVO : conceptIntroduceVOList) {
                ConceptIntroduceDTO conceptIntroduceDTO = getConceptDetail(conceptIntroduceVO);
                if (conceptIntroduceDTO != null) {
                    conceptIntroduceDTOList.add(conceptIntroduceDTO);
                }

            }
        }
        return conceptIntroduceDTOList;
    }

    /**
     * 是否存在提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    public Boolean exist(ConceptIntroduceVO conceptIntroduceVO) {
        conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
        //验证存在并且获取概念基本信息
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        BeanUtil.copyProperties(conceptIntroduceVO, conceptBaseVO);
        Concept concept = conceptFacade.exist(conceptBaseVO);
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId());
        Integer res = this.count(conceptDetailQueryWrapper);
        if (res > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 是否存在提示信息-列表
     *
     * @param conceptList
     * @return
     */
    public Map<String, Map<Long, List<ConceptDetail>>> hasConDetail(List<Concept> conceptList) {
        Map<String, Map<Long, List<ConceptDetail>>> retMap = new LinkedHashMap<>();
        Map<Long, List<ConceptDetail>> retSubMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(conceptList)) {
            List<Long> conceptIds = conceptList
                    .stream()
                    .map(concept -> concept.getId())
                    .collect(Collectors.toList());
            QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
            conceptDetailQueryWrapper
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds);
            if (ListUtil.isEmpty(conceptIds)) {
                return retMap;
            }
            List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
            if (ListUtil.isNotEmpty(conceptDetailList)) {
                Map<Long, List<ConceptDetail>> conceptDetailMap
                        = EntityUtil.makeEntityListMap(conceptDetailList, "conceptId");
                for (Concept concept : conceptList) {
                    retSubMap = new LinkedHashMap<>();
                    if (retMap.containsKey(concept.getLibName())) {
                        retSubMap = retMap.get(concept.getLibName());
                    }
                    if (conceptDetailMap.containsKey(concept.getId())
                            && ListUtil.isNotEmpty(conceptDetailMap.get(concept.getId()))) {
                        retSubMap.put(concept.getLibType(), conceptDetailMap.get(concept.getId()));
                        retMap.put(concept.getLibName(), retSubMap);
                    }
                }
            }
        }
        return retMap;
    }

    /**
     * 根据概念id验证提示信息是否存在
     *
     * @param existListByConceptIdsVO
     * @return
     */
    public List<Long> existListByConceptIds(ExistListByConceptIdsVO existListByConceptIdsVO) {
        if (ListUtil.isNotEmpty(existListByConceptIdsVO.getConceptIds())) {
            QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
            conceptDetailQueryWrapper
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", existListByConceptIdsVO.getConceptIds());
            List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
            if (ListUtil.isNotEmpty(conceptDetailList)) {
                List<Long> retConceptIds = conceptDetailList
                        .stream()
                        .map(conceptDetail -> conceptDetail.getConceptId())
                        .distinct()
                        .collect(Collectors.toList());
                return retConceptIds;
            }
        }
        return null;
    }

    /**
     * 获取对接静态知识
     *
     * @param connectIntroduceVO
     * @return
     */
    public List<ConceptIntroduceDTO> getConnectConceptDetails(ConnectIntroduceVO connectIntroduceVO) {
        //入参预处理
        List<ConceptIntroduceVO> paramList = convertParam(connectIntroduceVO);
        return this.getConceptDetails(paramList);
    }

    /**
     * 静态知识入参预处理-对接
     *
     * @param connectIntroduceVO
     * @return
     */
    public List<ConceptIntroduceVO> convertParam(ConnectIntroduceVO connectIntroduceVO) {
        List<ConceptIntroduceVO> retList = Lists.newLinkedList();
        List<String> nameList = Lists.newLinkedList();
        nameList.add(CryptUtil.decrypt_char(connectIntroduceVO.getName()));
        if (connectIntroduceVO.getSysType() == null) {
            connectIntroduceVO.setSysType(SysTypeEnum.AIPT_SERVICE.getKey());
        }
        if (!connectIntroduceVO.getSysType().equals(SysTypeEnum.LTAPI_SERVICE.getKey())) {
            ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
            BeanUtil.copyProperties(connectIntroduceVO, conceptIntroduceVO);
            conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
            retList.add(conceptIntroduceVO);
            return retList;
        }

        //是否对接
        Boolean isConnect = transferFacade.isConnect(connectIntroduceVO.getHosCode(), connectIntroduceVO.getSysType());
        if (isConnect) {
            if (connectIntroduceVO.getType().equals(ConceptTypeEnum.Lis.getKey())) {
                Map<String, Map<String, List<String>>> lisConfigMap
                        = transferFacade.getLisConfigByMealNameAndHosCode(nameList, connectIntroduceVO.getHosCode());
                if (lisConfigMap.get(CryptUtil.decrypt_char(connectIntroduceVO.getName())) != null) {
                    Map<String, List<String>> itemMap = lisConfigMap.get(CryptUtil.decrypt_char(connectIntroduceVO.getName()));
                    if (StringUtil.isBlank(connectIntroduceVO.getDetailName())) {
                        connectIntroduceVO.setDetailName(CryptUtil.encrypt_char(""));
                    }
                    List<String> uniqueNameList = itemMap.get(CryptUtil.decrypt_char(connectIntroduceVO.getDetailName()));
                    if (ListUtil.isNotEmpty(uniqueNameList)) {
                        for (String uniqueName : uniqueNameList) {
                            ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
                            BeanUtil.copyProperties(connectIntroduceVO, conceptIntroduceVO);
                            conceptIntroduceVO.setName(CryptUtil.encrypt_char(uniqueName));
                            conceptIntroduceVO.setType(ConceptTypeEnum.LIS_TABLES.getKey());
                            conceptIntroduceVO.setLibType(LexiconTypeEnum.LIS_TABLES.getKey());
                            retList.add(conceptIntroduceVO);
                        }
                    }
                }
            } else if (connectIntroduceVO.getType().equals(ConceptTypeEnum.Pacs.getKey())) {
                PacsConfigVO pacsConfigVO = new PacsConfigVO();
                pacsConfigVO.setHosCode(connectIntroduceVO.getHosCode());
                pacsConfigVO.setMealNameList(nameList);
                RespDTO<Map<String, List<String>>> pacsRespDTO = tranServiceClient.getPacsConfig(pacsConfigVO);
                if (RespDTOUtil.respIsOK(pacsRespDTO)) {
                    Map<String, List<String>> pacsMap = pacsRespDTO.data;
                    for (String pacsStr : pacsMap.get(CryptUtil.decrypt_char(connectIntroduceVO.getName()))) {
                        ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
                        BeanUtil.copyProperties(connectIntroduceVO, conceptIntroduceVO);
                        conceptIntroduceVO.setName(CryptUtil.encrypt_char(pacsStr));
                        conceptIntroduceVO.setType(ConceptTypeEnum.Pacs.getKey());
                        conceptIntroduceVO.setLibType(LexiconTypeEnum.PACS_ITEMS.getKey());
                        retList.add(conceptIntroduceVO);
                    }
                }
            } else if (connectIntroduceVO.getType().equals(ConceptTypeEnum.Disease.getKey())) {
                DiseaseIcdVO diseaseIcdVO = new DiseaseIcdVO();
                diseaseIcdVO.setHosCode(connectIntroduceVO.getHosCode());
                diseaseIcdVO.setDiseaseNameList(nameList);

                Map<String, String> disMap = new LinkedHashMap<>();
                RespDTO<Map<String, String>> disRespDTO = tranServiceClient.getDiseaseIcdMap(diseaseIcdVO);
                if (RespDTOUtil.respIsOK(disRespDTO)) {
                    disMap = disRespDTO.data;
                    if (disMap.get(CryptUtil.decrypt_char(connectIntroduceVO.getName())) != null) {
                        ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
                        BeanUtil.copyProperties(connectIntroduceVO, conceptIntroduceVO);
                        conceptIntroduceVO.setName(CryptUtil.encrypt_char(disMap.get(CryptUtil.decrypt_char(connectIntroduceVO.getName()))));
                        conceptIntroduceVO.setType(ConceptTypeEnum.Disease.getKey());
                        conceptIntroduceVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
                        retList.add(conceptIntroduceVO);
                    }
                }
            } else {
                ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
                BeanUtil.copyProperties(connectIntroduceVO, conceptIntroduceVO);
                conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
                retList.add(conceptIntroduceVO);
            }
        } else {
            ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
            BeanUtil.copyProperties(connectIntroduceVO, conceptIntroduceVO);
            conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
            retList.add(conceptIntroduceVO);
        }
        return retList;
    }
}