package com.diagbot.facade;

import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.dto.DisScaleDTO;
import com.diagbot.dto.ScaleIndexDTO;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.service.impl.DisScaleServiceImpl;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.DisScaleVO;
import com.diagbot.vo.ScaleIndexVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 诊断量表
 * @author: zhoutg
 * @time: 2015/05/05 13:19
 */
@Component
public class DisScaleFacade extends DisScaleServiceImpl {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 返回诊断量表列表
     *
     * @return
     */
    public List<DisScaleDTO> getDisScaleFac(DisScaleVO disScaleVO) {
        // 入参数据封装
        ConceptWrapper conceptWrapper = new ConceptWrapper();
        conceptWrapper.setStartName(disScaleVO.getDisName());
        conceptWrapper.setStartType(LexiconTypeEnum.DIAGNOSIS.getKey());
        conceptWrapper.setEndType(LexiconTypeEnum.GAUGE.getKey());
        conceptWrapper.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        List<ConceptWithOrderRes> list = conceptFacade.getConceptWithOrder(conceptWrapper);

        // 出参数据封装
        List<DisScaleDTO> res = new ArrayList<>();
        for(ConceptWithOrderRes bean : list) {
            DisScaleDTO dto = new DisScaleDTO();
            dto.setName(bean.getEndName());
            dto.setConceptId(bean.getEndId());
            res.add(dto);
        }
        return res;
    }


    /**
     * 量表搜索
     *
     * @param scaleIndexVO 参数
     * @return
     */
    public List<ScaleIndexDTO> indexFac(ScaleIndexVO scaleIndexVO) {
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        conceptBaseVO.setName(scaleIndexVO.getName());
        conceptBaseVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
        List<ConceptBaseDTO> conceptBaseDTOS = conceptFacade.indexConceptFac(conceptBaseVO);
        // 设置诊断名称
        if (ListUtil.isNotEmpty(conceptBaseDTOS)) {
            scaleIndexVO.setDisName(conceptBaseDTOS.stream().map(row -> row.getName()).collect(Collectors.toList()));
        }
        scaleIndexVO.setStartType(LexiconTypeEnum.DIAGNOSIS.getKey());
        scaleIndexVO.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        scaleIndexVO.setEndType(LexiconTypeEnum.GAUGE.getKey());
        if (cryptConfiguer.cryptFlag) {
            CryptUtil.encryptList(scaleIndexVO.getFilterName());
        }
        List<ScaleIndexDTO> data = this.indexScale(scaleIndexVO);
        List<ScaleIndexDTO> res = new ArrayList<>();
        List<Long> selfList = new ArrayList<>(); // 本体
        for (ScaleIndexDTO bean : data) {
            if (bean.getShowType().intValue() == 1) { // 本体
                selfList.add(bean.getConceptId());
            }
        }
        for (ScaleIndexDTO bean : data) {
            if (bean.getShowType().intValue() == 1) { // 本体
                res.add(bean);
            } else if (bean.getShowType().intValue() == 0) { //同义词或者诊断关联的量表，只取一个
                if(!selfList.contains(bean.getConceptId())) {
                    selfList.add(bean.getConceptId());
                    res.add(bean);
                }
            }
        }
        return res;
    }
}
