package com.diagbot.facade;

import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.ConceptRes;
import com.diagbot.dto.DisDTO;
import com.diagbot.dto.DisTypeDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.service.impl.DisTypeServiceImpl;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.DisTypeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description: 诊断类型
 * @author: zhoutg
 * @time: 2018/11/19 13:19
 */
@Component
public class DisTypeFacade extends DisTypeServiceImpl {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 返回诊断类型
     *
     * @return
     */
    public DisTypeDTO getDisTypeFac() {
        DisTypeDTO res = new DisTypeDTO();

        // 入参数据封装
        ConceptWrapper conceptWrapper = new ConceptWrapper();
        conceptWrapper.setStartType(LexiconTypeEnum.DIAGNOSIS.getKey());
        conceptWrapper.setEndType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
        conceptWrapper.setRelationType(LexiconRSTypeEnum.INCLUDE_OF.getKey());
        List<ConceptRes> list = conceptFacade.getConcept(conceptWrapper);

        // 出参数据封装
        List<DisDTO> data = new ArrayList<>();
        for (ConceptRes bean : list) {
            DisDTO disDTO = new DisDTO();
            disDTO.setName(bean.getStartName());
            disDTO.setConceptId(bean.getStartId());
            disDTO.setTypeName(bean.getEndName());
            data.add(disDTO);
        }
        Map<String, List<DisDTO>> map = EntityUtil.makeEntityListMap(data, "typeName");
        if (map != null) {
            String typeName = "";
            if (cryptConfiguer.cryptFlag) {
                typeName = CryptUtil.encrypt_char(DisTypeEnum.CHRONIC.getName());
            } else {
                typeName = DisTypeEnum.CHRONIC.getName();
            }
            List<DisDTO> chr = map.get(typeName); //放入慢病列表
            if (ListUtil.isNotEmpty(chr)) {
                res.setChronic(chr);
            }
            if (cryptConfiguer.cryptFlag) {
                typeName = CryptUtil.encrypt_char(DisTypeEnum.EMERGENCY.getName());
            } else {
                typeName = DisTypeEnum.EMERGENCY.getName();
            }
            List<DisDTO> eme = map.get(typeName); //放入急诊列表
            if (ListUtil.isNotEmpty(eme)) {
                res.setEmergency(eme);
            }
        }
        return res;
    }

    /**
     * 诊断类型验证
     *
     * @param disTypeVO
     * @return
     */
    public Boolean verifyDisType(DisTypeVO disTypeVO) {
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        conceptBaseVO.setName(disTypeVO.getName());
        conceptBaseVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
        Concept disease = conceptFacade.exist(conceptBaseVO);
        String disTypeName = DisTypeEnum.getName(disTypeVO.getDisType());
        if (cryptConfiguer.cryptFlag) {
            disTypeName = CryptUtil.encrypt_char(disTypeName);
        }
        conceptBaseVO.setName(disTypeName);
        conceptBaseVO.setLibType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
        Concept disTypeConcept = conceptFacade.exist(conceptBaseVO);
        ConceptWrapper conceptWrapper = new ConceptWrapper();
        conceptWrapper.setStartId(disease.getId());
        conceptWrapper.setStartName(disease.getLibName());
        conceptWrapper.setStartType(LexiconTypeEnum.DIAGNOSIS.getKey());
        conceptWrapper.setEndId(disTypeConcept.getId());
        conceptWrapper.setEndName(disTypeName);
        conceptWrapper.setEndType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
        conceptWrapper.setRelationType(LexiconRSTypeEnum.INCLUDE_OF.getKey());
        List<ConceptRes> conceptResList = conceptFacade.getConcept(conceptWrapper);
        if (ListUtil.isNotEmpty(conceptResList)) {
            return true;
        } else {
            return false;
        }
    }
}