package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.TranServiceClient;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.DiseaseIcdDTO;
import com.diagbot.dto.GetDiseaseIcdDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.Concept;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.GetDiseaseIcdVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: wangyu
 * @time: 2019/2/14 15:25
 */
@Component
public class DiseaseIcdFacade {

    @Autowired
    TranServiceClient tranServiceClient;
    @Autowired
    ConceptFacade conceptFacade;

    /**
     * 根据医院编号和诊断ids获取各自的icd编码
     *
     * @param getDiseaseIcdVO
     * @return
     */
    public List<DiseaseIcdDTO> getDiseaseIcds(GetDiseaseIcdVO getDiseaseIcdVO) {
        RespDTO<List<GetDiseaseIcdDTO>> getDiseaseIcdsList = tranServiceClient.getDiseaseIcds(getDiseaseIcdVO);
        RespDTOUtil.respNGDealCover(getDiseaseIcdsList, "获取icd编码失败");

        List<DiseaseIcdDTO> res = new ArrayList<>();

        List<String> disNames
                = getDiseaseIcdsList.data.stream()
                .map(g -> g.getConceptDisName())
                .distinct()
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(disNames)) {
            return res;
        }

        QueryWrapper<Concept> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_type", LexiconTypeEnum.DIAGNOSIS.getKey())
                .in("lib_name", disNames);

        List<Concept> conceptDisNames = conceptFacade.list(queryWrapper);

        Map<String, Long> map
                = conceptDisNames.stream()
                .collect(Collectors.toMap(c -> c.getLibName(), c -> c.getId()));
        if (null != map) {
            DiseaseIcdDTO diseaseIcdDTO = null;
            for (GetDiseaseIcdDTO getDiseaseIcdDTO : getDiseaseIcdsList.data) {
                if (null != map.get(getDiseaseIcdDTO.getConceptDisName())) {
                    diseaseIcdDTO = new DiseaseIcdDTO();
                    BeanUtil.copyProperties(getDiseaseIcdDTO, diseaseIcdDTO);
                    diseaseIcdDTO.setConceptId(map.get(getDiseaseIcdDTO.getConceptDisName()));
                    res.add(diseaseIcdDTO);
                }
            }
        }
        return res;
    }
}
