package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.EvaluationDTO;
import com.diagbot.service.impl.EvaluationServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.EvaluationDiseaseNameVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component
public class EvaluationFacade extends EvaluationServiceImpl {

    @Autowired
    PushFacade pushFacade;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 获取管理与评估模板接口
     *
     * @param searchVO
     * @return
     */
    public List<EvaluationDTO> getEvaluationModule(SearchVO searchVO) {
        if (searchVO.getDiseaseName() == null || StringUtil.isBlank(searchVO.getDiseaseName().getUniqueName())) {
            return Lists.newArrayList();
        }

        Map<String, JSONObject> jsonobject = pushFacade.getManagementEvaluationContent(searchVO);
        EvaluationDiseaseNameVO evaluationdiseaseNameVO = new EvaluationDiseaseNameVO();
        BeanUtil.copyProperties(searchVO, evaluationdiseaseNameVO);

        if (cryptConfiguer.cryptFlag) {
            evaluationdiseaseNameVO.getDiseaseName().setUniqueName(CryptUtil.encrypt_char(evaluationdiseaseNameVO.getDiseaseName().getUniqueName()));
        }

        List<EvaluationDTO> evaluationList = this.getEvaluationModule(evaluationdiseaseNameVO);
        if (jsonobject == null) {
            return evaluationList;
        }
        for (EvaluationDTO evaluadata : evaluationList) {
            if (jsonobject.containsKey(evaluadata.getRegionName())) {
                evaluadata.setData(jsonobject.get(evaluadata.getRegionName()));
            }
        }
        return evaluationList;
    }
}