package com.diagbot.facade;

import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.ParamConvertUtil;
import com.diagbot.vo.ConceptNameVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/13 15:38
 */
@Component
public class FeatureFacade {
    @Autowired
    private NLPFacade nlpFacade;
    @Autowired
    private ConceptFacade conceptFacade;

    /**
     * 症状关键词提取
     *
     * @param text
     * @return
     */
    public List<FeatureConceptDTO> getSymptomFeature(String text) {
        List<FeatureConceptDTO> symptomFeatureList = nlpFacade.symptomFeaturePageData(text);
        if (!ListUtil.isNotEmpty(symptomFeatureList)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "未提取到特征词");
        }

        Map<Integer, List<FeatureConceptDTO>> symptomMap
                = EntityUtil.makeEntityListMap(symptomFeatureList, "libType");
        Map<Integer, Map<String, FeatureConceptDTO>> map = new LinkedHashMap<>();
        for (Map.Entry<Integer, List<FeatureConceptDTO>> entry : symptomMap.entrySet()) {
            List<String> names = entry.getValue()
                    .stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            ConceptNameVO conceptNameVO = new ConceptNameVO();
            conceptNameVO.setNames(names);
            conceptNameVO.setType(entry.getKey());
            List<FeatureConceptDTO> featureConcepts = conceptFacade.getFeatureConcept(conceptNameVO);
            Map<String, FeatureConceptDTO> conceptMap
                    = EntityUtil.makeEntityMap(featureConcepts, "origin");
            map.put(entry.getKey(), conceptMap);
        }

        for (FeatureConceptDTO featureConceptDTO : symptomFeatureList) {
            if (null != featureConceptDTO.getLibType()) {
                featureConceptDTO.setType(ParamConvertUtil.libConvert2Concept(featureConceptDTO.getLibType()));
                if (map.containsKey(featureConceptDTO.getLibType())
                        && map.get(featureConceptDTO.getLibType()).containsKey(featureConceptDTO.getName())) {
                    featureConceptDTO.setConceptId(map
                            .get(featureConceptDTO.getLibType())
                            .get(featureConceptDTO.getName())
                            .getConceptId());
                    featureConceptDTO.setOrigin(map
                            .get(featureConceptDTO.getLibType())
                            .get(featureConceptDTO.getName())
                            .getOrigin());
                    featureConceptDTO.setName(map
                            .get(featureConceptDTO.getLibType())
                            .get(featureConceptDTO.getName())
                            .getName());
                }
            }
        }
        return symptomFeatureList;
    }
}