package com.diagbot.facade;

import com.diagbot.client.TranServiceClient;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.GetTopPatientInfoDjDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.Concept;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.GetTopPatientInfoDjVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @Description: 患者业务逻辑
 * @author: wangyu
 * @time: 2018/11/19 13:19
 */
@Component
public class PatientInfoFacade {

    @Autowired
    private ConceptFacade conceptFacade;
    @Autowired
    TranServiceClient tranServiceClient;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 页面顶部病人医生科室信息查询
     *
     * @param getTopPatientInfoVO
     * @return
     */
    public GetTopPatientInfoDTO getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO) {
        RespDTO<GetTopPatientInfoDTO> respDTO = tranServiceClient.getTopPatientInfo(getTopPatientInfoVO);
        RespDTOUtil.respNGDealCover(respDTO, respDTO.msg);

        GetTopPatientInfoDTO getTopPatientInfoDTO = respDTO.data;

        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        conceptBaseVO.setLibType(LexiconTypeEnum.DEPARTMENT.getKey());
        if (StringUtil.isNotBlank(getTopPatientInfoDTO.getSelfDeptName())) {
            if (cryptConfiguer.cryptFlag) {
                conceptBaseVO.setName(CryptUtil.encrypt_char(getTopPatientInfoDTO.getSelfDeptName()));
            } else {
                conceptBaseVO.setName(getTopPatientInfoDTO.getSelfDeptName());
            }
        }

        Concept concept = conceptFacade.exist(conceptBaseVO);
        getTopPatientInfoDTO.setSelfDeptId(concept.getId());

        return getTopPatientInfoDTO;
    }

    /**
     * 对接-页面顶部病人医生科室信息查询
     *
     * @param getTopPatientInfoDjVO
     * @return
     */
    public GetTopPatientInfoDjDTO getTopPatientInfoDj(@RequestBody GetTopPatientInfoDjVO getTopPatientInfoDjVO) {
        RespDTO<GetTopPatientInfoDjDTO> respDTO = tranServiceClient.getTopPatientInfoDj(getTopPatientInfoDjVO);
        RespDTOUtil.respNGDealCover(respDTO, "获取页面顶部病人医生科室信息失败");

        GetTopPatientInfoDjDTO getTopPatientInfoDjDTO = respDTO.data;

        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        conceptBaseVO.setLibType(LexiconTypeEnum.DEPARTMENT.getKey());
        if (StringUtil.isNotBlank(getTopPatientInfoDjDTO.getSelfDeptName())) {
            if (cryptConfiguer.cryptFlag) {
                conceptBaseVO.setName(CryptUtil.encrypt_char(getTopPatientInfoDjDTO.getSelfDeptName()));
            } else {
                conceptBaseVO.setName(getTopPatientInfoDjDTO.getSelfDeptName());
            }
        }
        Concept concept = conceptFacade.exist(conceptBaseVO);
        getTopPatientInfoDjDTO.setSelfDeptId(concept.getId());

        return getTopPatientInfoDjDTO;
    }

}
