package com.diagbot.facade;

import com.diagbot.biz.push.entity.FeatureRate;
import com.diagbot.biz.push.entity.FeatureRateV2;
import com.diagbot.biz.push.entity.FeatureRateWithExplain;
import com.diagbot.biz.push.entity.FeatureRateWithExplainV2;
import com.diagbot.biz.push.entity.ResponseData;
import com.diagbot.biz.push.entity.ResponseDataV2;
import com.diagbot.biz.push.entity.ResponseDataWithExplain;
import com.diagbot.biz.push.entity.ResponseDataWithExplainV2;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.AINeoServiceClient;
import com.diagbot.client.bean.Response;
import com.diagbot.client.bean.SearchData;
import com.diagbot.entity.KlSymptom;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/4/16 16:32
 */
@Component
public class PushV2Facade {
    @Autowired
    private AINeoServiceClient aiNeoServiceClient;
    @Autowired
    private AssembleFacade assembleFacade;
    @Autowired
    private ConceptFacade conceptFacade;

    /**
     * 大数据原始推理结果
     *
     * @param searchVO
     * @return
     */
    public ResponseDataV2 originalPush(SearchVO searchVO) {
        SearchData searchData = assembleFacade.assembleData(searchVO);
        Response<ResponseDataV2> res = aiNeoServiceClient.push(searchData);
        if (res == null || res.getData() == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "AI没有返回结果");
        }
        return res.getData();
    }

    /**
     * 推理结果带名词解释
     *
     * @param searchVO
     * @return
     */
    public ResponseDataWithExplainV2 pushWithExplain(SearchVO searchVO) {
        ResponseDataWithExplainV2 responseData = new ResponseDataWithExplainV2();
        ResponseDataV2 data = this.originalPush(searchVO);
        List<FeatureRateV2> symptom = data.getSymptom();
        if (ListUtil.isNotEmpty(symptom)
                && (searchVO.getSysType().equals(SysTypeEnum.PREC_SERVICE.getKey())
                || searchVO.getSysType().equals(SysTypeEnum.TRIAGE_SERVICE.getKey()))) {
            List<String> names = symptom.stream().map(item -> item.getFeatureName()).collect(Collectors.toList());
            Map<String, KlSymptom> klSymptomMap
                    = conceptFacade.getSymptomWithExplain(names, LexiconTypeEnum.SYMPTOM.getKey());
            List<FeatureRateWithExplainV2> featureRateWithExplains
                    = BeanUtil.listCopyTo(symptom, FeatureRateWithExplainV2.class);
            if (klSymptomMap != null) {
                for (FeatureRateWithExplainV2 featureRateWithExplain : featureRateWithExplains) {
                    if (klSymptomMap.containsKey(featureRateWithExplain.getFeatureName())) {
                        featureRateWithExplain
                                .setDescription(klSymptomMap.get(featureRateWithExplain.getFeatureName()).getDescription());
                        featureRateWithExplain
                                .setExplains(klSymptomMap.get(featureRateWithExplain.getFeatureName()).getExplains());
                    }
                }
            }
            responseData.setSymptom(featureRateWithExplains);
        }
        responseData.setDis(data.getDis());
        return responseData;
    }
}
