package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.client.TranServiceClient;
import com.diagbot.client.bean.HosCodeVO;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.LisSourceEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.DiseaseIcdVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.LisConfigVO;
import com.diagbot.vo.OperationConfigVO;
import com.diagbot.vo.PacsConfigVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/11/11 14:14
 */
@Component
public class TransferFacade {
    @Autowired
    private TranServiceClient tranServiceClient;
    @Autowired
    CryptConfiguer cryptConfiguer;

    /**
     * 查询对接状态
     *
     * @param hosCode
     * @param sysType
     * @return
     */
    public Boolean isConnect(String hosCode, Integer sysType) {
        Boolean isConnect = false;
        if (StringUtil.isNotBlank(hosCode)) {
            HosCodeVO hosCodeVO = new HosCodeVO();
            hosCodeVO.setHosCode(hosCode);
            HospitalSetVO hospitalSetVO = new HospitalSetVO();
            hospitalSetVO.setHospitalCode(hosCode);
            hospitalSetVO.setCode("connect");
            if (sysType != null) {
                hospitalSetVO.setSysType(sysType);
            } else {
                hospitalSetVO.setSysType(SysTypeEnum.AIPT_SERVICE.getKey());
            }
            RespDTO<List<SysSetInfoDTO>> sysSetInfoListRes = tranServiceClient.getSysSetInfoDatas(hospitalSetVO);
            if (RespDTOUtil.respIsOK(sysSetInfoListRes)) {
                List<SysSetInfoDTO> sysSetInfoList = sysSetInfoListRes.data;
                if (ListUtil.isNotEmpty(sysSetInfoList)) {
                    if (sysSetInfoList.get(0).getValue().equals("1")) {
                        isConnect = true;
                    } else {
                        isConnect = false;
                    }
                } else {
                    isConnect = false;
                }
            }
        }
        return isConnect;
    }

    /**
     * 化验结果增加公表名
     *
     * @param lis
     * @param hosCode
     * @return
     */
    public List<Lis> addLisUniqueName(List<Lis> lis, String hosCode) {
        if (ListUtil.isEmpty(lis)) {
            return Lists.newLinkedList();
        }
        List<Lis> retLis = Lists.newLinkedList();
        //化验项转公表内容,参数处理
        List<String> mealNameList = lis
                .stream()
                .map(lisResult -> lisResult.getName()).collect(Collectors.toList());
        LisConfigVO lisConfigVO = new LisConfigVO();
        lisConfigVO.setHosCode(hosCode);
        lisConfigVO.setMealNameList(mealNameList);
        Map<String, Map<String, List<String>>> lisConfigMap = new LinkedHashMap<>();
        RespDTO<Map<String, Map<String, List<String>>>> respDTO
                = tranServiceClient.getLisConfigByMealNameAndHosCode(lisConfigVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            lisConfigMap = respDTO.data;
        }
        for (Lis lisResult : lis) {
            //内部数据，获取标签时映射， 不二次映射
            if (lisResult.getSource() == null || lisResult.getSource().equals(LisSourceEnum.Inner.getKey())) {
                retLis.add(lisResult);
                continue;
            }
            if (lisResult.getDetailName() == null) {
                lisResult.setDetailName("");
            }
            Map<String, List<String>> itemMap = lisConfigMap.get(lisResult.getName());
            if (itemMap != null) {
                List<String> uniqueNameList = itemMap.get(lisResult.getDetailName());
                if (ListUtil.isNotEmpty(uniqueNameList)) {
                    //化验公表映射出多项时，拆分成多个公表项数据，指标值保持一致
                    for (String uniqueName : uniqueNameList) {
                        Lis lisResult_new = new Lis();
                        BeanUtil.copyProperties(lisResult, lisResult_new);
                        lisResult_new.setUniqueName(uniqueName);
                        retLis.add(lisResult_new);
                    }
                } else {
                    retLis.add(lisResult);
                }
            }
        }
        return retLis;
    }

    /**
     * 辅检结果增加公表名（辅检项目名称）
     *
     * @param pacs
     * @param hosCode
     * @return
     */
    public List<Pacs> addPacsUniqueName(List<Pacs> pacs, String hosCode) {
        if (ListUtil.isEmpty(pacs)) {
            return Lists.newLinkedList();
        }
        List<Pacs> convertPacs = Lists.newLinkedList();
        PacsConfigVO pacsConfigVO = new PacsConfigVO();
        pacsConfigVO.setHosCode(hosCode);
        pacsConfigVO.setMealNameList(pacs
                .stream()
                .map(pacsResult -> pacsResult.getName())
                .distinct()
                .collect(Collectors.toList()));
        Map<String, List<String>> pacsMap = new LinkedHashMap<>();
        RespDTO<Map<String, List<String>>> respDTO = tranServiceClient.getPacsConfig(pacsConfigVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            pacsMap = respDTO.data;
        }
        for (Pacs pacsResult : pacs) {
            if (StringUtil.isBlank(pacsResult.getName())) {
                convertPacs.add(pacsResult);
                continue;
            } else {
                if (ListUtil.isNotEmpty(pacsMap.get(pacsResult.getName()))) {
                    for (String uniqueName : pacsMap.get(pacsResult.getName())) {
                        Pacs convertPacsResult = new Pacs();
                        BeanUtil.copyProperties(pacsResult, convertPacsResult);
                        convertPacsResult.setUniqueName(uniqueName);
                        convertPacs.add(convertPacsResult);
                    }
                } else {
                    convertPacs.add(pacsResult);
                }
            }
        }
        return convertPacs;
    }

    /**
     * 获取化验公表映射关系
     *
     * @param nameList
     * @param hosCode
     * @return
     */
    public Map<String, Map<String, List<String>>> getLisConfigByMealNameAndHosCode(List<String> nameList, String hosCode) {
        Map<String, Map<String, List<String>>> lisConfigMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(nameList) && StringUtil.isNotBlank(hosCode)) {
            LisConfigVO lisConfigVO = new LisConfigVO();
            lisConfigVO.setHosCode(hosCode);
            lisConfigVO.setMealNameList(nameList);
            RespDTO<Map<String, Map<String, List<String>>>> respDTO = tranServiceClient.getLisConfigByMealNameAndHosCode(lisConfigVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                lisConfigMap = respDTO.data;
            }
        }
        return lisConfigMap;
    }

    /**
     * 其他类型增加公表名（诊断、药品、手术等）
     *
     * @param items
     * @param hosCode
     * @return
     */
    public List<Item> addItemUniqueName(List<Item> items, Integer type, String hosCode) {
        if (ListUtil.isEmpty(items)) {
            return Lists.newLinkedList();
        }
        List<Item> convertItems = Lists.newLinkedList();
        List<String> nameList = items
                .stream().map(item -> item.getName())
                .distinct()
                .collect(Collectors.toList());
        if (type.equals(ConceptTypeEnum.Disease.getKey())) {
            DiseaseIcdVO diseaseIcdVO = new DiseaseIcdVO();
            diseaseIcdVO.setHosCode(hosCode);
            diseaseIcdVO.setDiseaseNameList(nameList);
            Map<String, String> disMap = new LinkedHashMap<>();
            RespDTO<Map<String, String>> respDTO = tranServiceClient.getDiseaseIcdMap(diseaseIcdVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                disMap = respDTO.data;
            }
            for (Item disease : items) {
                if (StringUtil.isBlank(disease.getName())) {
                    continue;
                } else {
                    if (StringUtil.isNotBlank(disMap.get(disease.getName()))) {
                        disease.setUniqueName(disMap.get(disease.getName()));
                    }
                }
            }
            convertItems = BeanUtil.listCopyTo(items, Item.class);
        } else if (type.equals(ConceptTypeEnum.Drug.getKey())) {
            //暂无映射关系
            convertItems.addAll(items);
        } else if (type.equals(ConceptTypeEnum.OPERATION.getKey())) {
            OperationConfigVO operationConfigVO = new OperationConfigVO();
            operationConfigVO.setHosCode(hosCode);
            operationConfigVO.setNameList(items
                    .stream()
                    .map(operation -> operation.getName())
                    .distinct()
                    .collect(Collectors.toList()));
            Map<String, List<String>> operationMap = new LinkedHashMap<>();
            RespDTO<Map<String, List<String>>> respDTO = tranServiceClient.getOperationConfig(operationConfigVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                operationMap = respDTO.data;
            }
            for (Item item : items) {
                if (StringUtil.isBlank(item.getName())) {
                    convertItems.add(item);
                    continue;
                } else {
                    if (ListUtil.isNotEmpty(operationMap.get(item.getName()))) {
                        //外部一个项目对应到内部多个项目的数据，都保留
                        for (String uniqueName : operationMap.get(item.getName())) {
                            Item convertItem = new Item();
                            BeanUtil.copyProperties(item, convertItem);
                            convertItem.setUniqueName(uniqueName);
                            convertItems.add(convertItem);
                        }
                    } else {
                        convertItems.add(item);
                    }
                }
            }
        }
        return convertItems;
    }

    /**
     * 外部项目名称转内部名称-字符串格式
     *
     * @param originalStr 原始数据
     * @param hosCode     医院编码
     * @param type        5-化验（仅限套餐），6-辅检，7-诊断
     * @return
     */
    public String strConvert(String originalStr, String hosCode, Integer type) {
        String retStr = "";
        if (StringUtil.isBlank(originalStr)) {
            return retStr;
        }
        String[] strArr = originalStr
                .replace(" ", "")
                .split(",|，|;|；");
        List<String> strList = Arrays.asList(strArr);
        strList = strList
                .stream()
                .filter(str -> StringUtil.isNotBlank(str))
                .distinct()
                .collect(Collectors.toList());
        if (type.equals(FeatureTypeEnum.Feature_Type_Lis.getKey())) {
            Map<String, Map<String, List<String>>> map = getLisConfigByMealNameAndHosCode(strList, hosCode);
            if (map != null) {
                for (String lis : strList) {
                    Map<String, List<String>> itemMap = map.get(lis);
                    if (itemMap != null && itemMap.get("") != null) {
                        retStr += String.join(",", itemMap.get("")) + ",";
                    }
                }
            }

        } else if (type.equals(FeatureTypeEnum.Feature_Type_Pacs.getKey())) {
            PacsConfigVO pacsConfigVO = new PacsConfigVO();
            pacsConfigVO.setHosCode(hosCode);
            pacsConfigVO.setMealNameList(strList);
            Map<String, List<String>> pacsMap = new LinkedHashMap<>();
            RespDTO<Map<String, List<String>>> respDTO = tranServiceClient.getPacsConfig(pacsConfigVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                pacsMap = respDTO.data;
            }
            for (Map.Entry<String, List<String>> entry : pacsMap.entrySet()) {
                if (ListUtil.isNotEmpty(entry.getValue())) {
                    retStr += String.join(",", entry.getValue()) + ",";
                }
            }
        } else if (type.equals(FeatureTypeEnum.Feature_Type_Disease.getKey())) {
            DiseaseIcdVO diseaseIcdVO = new DiseaseIcdVO();
            diseaseIcdVO.setHosCode(hosCode);
            diseaseIcdVO.setDiseaseNameList(strList);
            Map<String, String> disMap = new LinkedHashMap<>();
            RespDTO<Map<String, String>> respDTO = tranServiceClient.getDiseaseIcdMap(diseaseIcdVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                disMap = respDTO.data;
            }
            for (String diseaseName : strList) {
                if (StringUtil.isNotBlank(disMap.get(diseaseName))) {
                    retStr += disMap.get(diseaseName) + ",";
                }
            }
        }
        if (retStr.endsWith(",")) {
            retStr = retStr.substring(0, retStr.length() - 1);
        }
        return retStr;
    }
}