package com.diagbot.facade;

import com.diagbot.biz.push.entity.AdverseReaction;
import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Medicition;
import com.diagbot.biz.push.entity.MedicitionClass;
import com.diagbot.biz.push.entity.MeditionDetail;
import com.diagbot.biz.push.entity.Treat;
import com.diagbot.biz.push.entity.TreatmentPlan;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.ConceptRes;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:治疗方案业务处理
 * @Author:zhaops
 * @time: 2019/5/8 17:16
 */
@Component
public class TreatmentFacade {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    ConceptDetailFacade conceptDetailFacade;
    @Autowired
    DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;
    @Autowired
    CryptConfiguer cryptConfiguer;

    /**
     * 获取治疗方案
     *
     * @param
     * @return
     */
    public Treat getTreatment(Treat treat, Item diseaseName, Integer disType) {
        String disUniqueName = diseaseName.getUniqueName();
        if (cryptConfiguer.cryptFlag) {
            //加密诊断名称
            disUniqueName = CryptUtil.encrypt_char(disUniqueName);
        }
        List<MedicitionClass> drugsList = Lists.newLinkedList();
        List<String> conceptNameList = Lists.newLinkedList();
        conceptNameList.add(disUniqueName);
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        conceptBaseVO.setName(disUniqueName);
        conceptBaseVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
        Concept disease = conceptFacade.exist(conceptBaseVO);
        if (disease == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "诊断不存在");
        }
        if (treat == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "未找到治疗方案");
        }

        //治疗方案对应诊断
        treat.setDiseaseName(diseaseName);

        //推荐治疗方案
        List<TreatmentPlan> treatmentPlanList = treat.getTreatmentPlan();
        //不良反应
        List<AdverseReaction> adverseReactionList = treat.getAdverseEvent();

        if (ListUtil.isNotEmpty(treatmentPlanList)) {
            for (int index = 0; index < treatmentPlanList.size(); index++) {
                TreatmentPlan treatmentPlan = treatmentPlanList.get(index);
                List<MeditionDetail> meditionDetailList = treatmentPlan.getMeditionDetails();
                if (ListUtil.isNotEmpty(meditionDetailList)) {
                    for (MeditionDetail meditionDetail : meditionDetailList) {
                        //药品推荐
                        List<MedicitionClass> treatmentDrugList = meditionDetail.getTreatment();
                        if (ListUtil.isNotEmpty(treatmentDrugList)) {
                            drugsList.addAll(treatmentDrugList);
                            //药品大类
                            List<String> cateBigNameList = treatmentDrugList
                                    .stream()
                                    .map(drugs -> drugs.getBigdrugsName())
                                    .collect(Collectors.toList());
                            conceptNameList.addAll(cateBigNameList);
                            //药品小类
                            List<String> cateSmallNameList = treatmentDrugList
                                    .stream()
                                    .map(drugs -> drugs.getSubdrugsName())
                                    .collect(Collectors.toList());
                            conceptNameList.addAll(cateSmallNameList);
                            //药品
                            List<Medicition> medicitionTotalList = Lists.newLinkedList();
                            for (MedicitionClass medicitionClass : treatmentDrugList) {
                                if (medicitionClass.getMedicitionsList() != null) {
                                    medicitionTotalList.addAll(medicitionClass.getMedicitionsList());
                                }
                            }
                            List<String> drugNameList = medicitionTotalList
                                    .stream()
                                    .map(m -> m.getMedicitionName())
                                    .collect(Collectors.toList());
                            conceptNameList.addAll(drugNameList);
                        } else {
                            if (StringUtil.isBlank(meditionDetail.getDescription())) {
                                treatmentPlanList.remove(index);
                                index--;
                                break;
                            }
                        }
                    }
                }
            }
        }
        if (ListUtil.isNotEmpty(adverseReactionList)) {
            List<String> adNameList = adverseReactionList
                    .stream()
                    .map(a -> a.getName())
                    .collect(Collectors.toList());
            conceptNameList.addAll(adNameList);
        }
        //概念
        List<Concept> conceptList = conceptFacade.getListByNames(conceptNameList);
        //概念map
        Map<String, Map<Long, Concept>> conceptMap = conceptList2Map(conceptList);
        //说明信息是否存在
        Map<String, Map<Long, List<ConceptDetail>>> hasConDetailMap = conceptDetailFacade.hasConDetail(conceptList);

        //判断是否慢病
        Boolean isChronic = false;
        if (disType != null && disType.equals(DisTypeEnum.CHRONIC.getKey())) {
            //验证是否慢病
            if (cryptConfiguer.cryptFlag) {
                conceptBaseVO.setName(CryptUtil.encrypt_char("慢病"));
            } else {
                conceptBaseVO.setName("慢病");
            }
            conceptBaseVO.setLibType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
            Concept chronicConcept = conceptFacade.exist(conceptBaseVO);
            ConceptWrapper conceptWrapper = new ConceptWrapper();
            conceptWrapper.setStartId(disease.getId());
            conceptWrapper.setStartName(disUniqueName);
            conceptWrapper.setStartType(LexiconTypeEnum.DIAGNOSIS.getKey());
            conceptWrapper.setEndId(chronicConcept.getId());
            conceptWrapper.setEndName(chronicConcept.getLibName());
            conceptWrapper.setEndType(LexiconTypeEnum.TYPES_OF_DISEASE.getKey());
            conceptWrapper.setRelationType(LexiconRSTypeEnum.INCLUDE_OF.getKey());
            List<ConceptRes> conceptResList = conceptFacade.getConcept(conceptWrapper);
            isChronic = ListUtil.isNotEmpty(conceptResList);
        }

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("treat", treat);
            invokeParams.put("diseaseName", disUniqueName);
            invokeParams.put("drugsList", drugsList);
            invokeParams.put("treatmentPlan", treatmentPlanList);
            invokeParams.put("conceptMap", conceptMap);
            invokeParams.put("hasConDetailMap", hasConDetailMap);
            invokeParams.put("adverseReactionList", adverseReactionList);
            invokeParams.put("isChronic", isChronic);

            treat = dataBeanAggregateQueryFacade.get("setTreatmentAll", invokeParams, Treat.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        //判断是否慢病，慢病增加回访时间，上次用药，不良反应
        if (isChronic) {
            if (treat == null) {
                treat = new Treat();
                treat.setDiseaseName(diseaseName);
            }
            treat.setDisType(1);
            //慢病，增加回访时间，暂时前端写死， 以后由只是图谱返回
            treat.setFollowUp(null);
            //上次用药-icss层处理
            treat.setDrugHistory(null);
        }
        return treat;
    }

    /**
     * 转换概念list到map  Map<概念名称,Map<词性,概念对象>>
     *
     * @param conceptList
     * @return
     */
    public Map<String, Map<Long, Concept>> conceptList2Map(List<Concept> conceptList) {
        Map<String, Map<Long, Concept>> retMap = new LinkedHashMap<>();
        Map<Long, Concept> retSubMap = new LinkedHashMap<>();
        for (Concept concept : conceptList) {
            if (retMap.containsKey(concept.getLibName())) {
                retSubMap = retMap.get(concept.getLibName());
            }
            if (retSubMap == null) {
                retSubMap = new LinkedHashMap<>();
            }
            retSubMap.put(concept.getLibType(), concept);
            retMap.put(concept.getLibName(), retSubMap);
            retSubMap = new LinkedHashMap<>();
        }
        return retMap;
    }
}