package com.diagbot.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRes;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptFindVO;
import com.diagbot.vo.ConceptNameVO;
import com.diagbot.vo.RetrievalVO;

import java.util.List;

/**
 * <p>
 * 概念表 服务类
 * </p>
 *
 * @author zhaops
 * @since 2019-05-06
 */
public interface ConceptService extends IService<Concept> {

    /**
     * 获取概念-关系-概念
     *
     * @param conceptWrapper 查询参数
     * @return 起点术语和终点术语
     */
    List<ConceptRes> getConcept(ConceptWrapper conceptWrapper);

    /**
     * 获取带排序概念-关系-概念
     *
     * @param conceptWrapper 查询参数
     * @return 起点术语和终点术语以及排序号
     */
    List<ConceptWithOrderRes> getConceptWithOrder(ConceptWrapper conceptWrapper);

    /**
     * 概念检索
     *
     * @param retrievalVO
     * @return
     */
    List<ConceptRetrievalDTO> retrievalConcept(RetrievalVO retrievalVO);

    /**
     * 静态知识检索
     *
     * @param retrievalVO
     * @return
     */
    List<RetrievalDTO> staticKnowledge(RetrievalVO retrievalVO);

    /**
     * 静态知识检索
     *
     * @param retrievalVO
     * @return
     */
    List<RetrievalDTO> staticKnowledgeHaiNan(RetrievalVO retrievalVO);

    List<ConceptBaseDTO> getConceptConMap(ConceptFindVO conceptFindVO);

    /**
     * 获得诊断依据问题词的标准词
     *
     * @return
     */
    List<String> getStandWord();


    /**
     * 检索词库获取标准词
     *
     * @param conceptBaseVO 入参
     * @return
     */
    public List<ConceptBaseDTO> indexConcept(ConceptBaseVO conceptBaseVO);

    /**
     * 特征词提取相关
     *
     * @param conceptNameVO
     * @return
     */
    public List<FeatureConceptDTO> getFeatureConcept(ConceptNameVO conceptNameVO);

}
