package com.diagbot.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRes;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.mapper.ConceptMapper;
import com.diagbot.service.ConceptService;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptFindVO;
import com.diagbot.vo.ConceptNameVO;
import com.diagbot.vo.RetrievalVO;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 概念表 服务实现类
 * </p>
 *
 * @author zhaops
 * @since 2019-05-06
 */
@Service
public class ConceptServiceImpl extends ServiceImpl<ConceptMapper, Concept> implements ConceptService {

    /**
     * 获取概念-关系-概念
     *
     * @param conceptWrapper 查询参数
     * @return 起点术语和终点术语
     */
    @Override
    public List<ConceptRes> getConcept(ConceptWrapper conceptWrapper) {
        return baseMapper.getConcept(conceptWrapper);
    }

    /**
     * 概念检索
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public List<ConceptRetrievalDTO> retrievalConcept(RetrievalVO retrievalVO) {
        return baseMapper.retrievalConcept(retrievalVO);
    }

    /**
     * 静态知识检索
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public List<RetrievalDTO> staticKnowledge(RetrievalVO retrievalVO){
        List<RetrievalDTO> staticList = baseMapper.staticKnowledge(retrievalVO);
        List<RetrievalDTO> res = new ArrayList<>();
        if (ListUtil.isNotEmpty(staticList)){
            List<Long> ids = new ArrayList<>();
            for (RetrievalDTO retrievalDTO : staticList){
                if (!ids.contains(retrievalDTO.getConceptId())){
                    ids.add(retrievalDTO.getConceptId());
                    res.add(retrievalDTO);
                }
            }
        }
        return res;
    }

    /**
     * 静态知识检索【海南】
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public List<RetrievalDTO> staticKnowledgeHaiNan(RetrievalVO retrievalVO){
        List<RetrievalDTO> staticList = baseMapper.staticKnowledgeHaiNan(retrievalVO);
        List<RetrievalDTO> res = new ArrayList<>();
        if (ListUtil.isNotEmpty(staticList)){
            List<Long> ids = new ArrayList<>();
            for (RetrievalDTO retrievalDTO : staticList){
                if (!ids.contains(retrievalDTO.getConceptId())){
                    ids.add(retrievalDTO.getConceptId());
                    res.add(retrievalDTO);
                }
            }
        }
        return res;
    }

    /**
     * 获取带排序概念-关系-概念
     *
     * @param conceptWrapper 查询参数
     * @return 起点术语和终点术语以及排序号
     */
    @Override
    public List<ConceptWithOrderRes> getConceptWithOrder(ConceptWrapper conceptWrapper) {
        return baseMapper.getConceptWithOrder(conceptWrapper);
    }

    @Override
    public List<ConceptBaseDTO> getConceptConMap(ConceptFindVO conceptFindVO){
        return baseMapper.getConceptConMap(conceptFindVO);
    }
    /**
     * 获得诊断依据问题词的标准词
     *
     * @return
     */
    @Override
    public List<String> getStandWord() {
        return baseMapper.getStandWord();
    }

    @Override
    public List<ConceptBaseDTO> indexConcept(ConceptBaseVO conceptBaseVO) {
        return baseMapper.indexConcept(conceptBaseVO);
    }

    /**
     * 特征词提取相关
     *
     * @param conceptNameVO
     * @return
     */
    public List<FeatureConceptDTO> getFeatureConcept(ConceptNameVO conceptNameVO){
        return baseMapper.getFeatureConcept(conceptNameVO);
    }
}
