package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.biz.push.entity.ResponseData;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ScaleContent;
import com.diagbot.facade.ClinicalFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Mark Huang
 * @since 2019-03-04
 */
@RestController
@RequestMapping("/clinicaldata")
@Api(value = "临床数据处理API", tags = { "知识库标准化-临床数据处理API" })
@SuppressWarnings("unchecked")
public class ClinicalController {
    @Autowired
    private ClinicalFacade clinicalFacade;

    @ApiOperation(value = "知识库标准化-临床数据处理(只是用大数据推理)", notes = "")
    @PostMapping("/processData")
    @SysLogger("processData")
    public RespDTO<ResponseData> processData(@Valid @RequestBody SearchVO searchVO) {
        ResponseData data = clinicalFacade.processClinicalData(searchVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-获取量表内容", notes = "")
    @PostMapping("/getScale")
    @SysLogger("getScale")
    public RespDTO<List<ScaleContent>> getScale(@RequestBody SearchVO searchVO) {
        List<ScaleContent> data = clinicalFacade.getScale(searchVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-计算接口", notes = "")
    @PostMapping("/scaleCalc")
    @SysLogger("scaleCalc")
    public RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData) {
        Map<String, Object> data = clinicalFacade.scaleCalc(calculateData);
        return RespDTO.onSuc(data);
    }
}