package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConceptDetailFacade;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ConnectIntroduceVO;
import com.diagbot.vo.ExistListByConceptIdsVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 提示信息明细表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2019-05-06
 */
@RestController
@RequestMapping("/conceptDetail")
@Api(value = "提示信息相关API", tags = { "知识库标准化-提示信息相关API" })
@SuppressWarnings("unchecked")
public class ConceptDetailController {
    @Autowired
    private ConceptDetailFacade conceptDetailFacade;

    @ApiOperation(value = "知识库标准化-获取提示信息[by:zhaops]",
            notes = "name: 标签名称，必填<br>" +
                    "titles: 标题，数组选填<br>" +
                    "type：标签类型（1-症状，5-化验，6-辅检，7-诊断，8-药品，9-药品大类，10-不良反应，11-药品小类，12-化验公表项，22-指标，25-手术），单选必填<br>" +
                    "position：1-摘要，2-全文，5-药品说明书，6-不良反应，单选选填")
    @PostMapping("/getConceptDetail")
    @SysLogger("getConceptDetail")
    public RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        ConceptIntroduceDTO data = conceptDetailFacade.getConceptDetail(conceptIntroduceVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-批量获取提示信息[by:zhaops]",
            notes = "name: 标签名称，必填<br>" +
                    "titles: 标题，数组选填<br>" +
                    "type：标签类型（1-症状，5-化验，6-辅检，7-诊断，8-药品，9-药品大类，10-不良反应，11-药品小类，12-化验公表项，22-指标，25-手术），单选必填<br>" +
                    "position：1-摘要，2-全文，5-药品说明书，6-不良反应，单选选填")
    @PostMapping("/getConceptDetails")
    @SysLogger("getConceptDetails")
    public RespDTO<List<ConceptIntroduceDTO>> getConceptDetails(@Valid @RequestBody List<ConceptIntroduceVO> conceptIntroduceVOList) {
        List<ConceptIntroduceDTO> data = conceptDetailFacade.getConceptDetails(conceptIntroduceVOList);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-获取静态知识-对接模式[by:zhaops]",
            notes = "name: 名称，必填<br>" +
                    "hosCode: 医院编码，必填<br>" +
                    "detailName: 明细名称，仅针对化验明细项<br>" +
                    "titles: 标题，数组选填<br>" +
                    "type：标签类型（5-化验，6-辅检，7-诊断,8-药品），单选必填<br>" +
                    "position：1-摘要，2-全文，5-药品说明书，6-不良反应，单选选填")
    @PostMapping("/getConnectConceptDetails")
    @SysLogger("getConnectConceptDetails")
    public RespDTO<List<ConceptIntroduceDTO>> getConnectConceptDetails(@Valid @RequestBody ConnectIntroduceVO connectIntroduceVO) {
        List<ConceptIntroduceDTO> data = conceptDetailFacade.getConnectConceptDetails(connectIntroduceVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-获取提示信息存在性[by:zhaops]",
            notes = "name: 标签名称，必填<br>" +
                    "type：标签类型（1-症状，5-化验，6-辅检，7-诊断，8-药品，9-药品大类，10-不良反应，11-药品小类，12-化验公表项，22-指标），单选必填<br>")
    @PostMapping("/exist")
    @SysLogger("exist")
    @ApiIgnore
    public RespDTO<Boolean> exist(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        Boolean data = conceptDetailFacade.exist(conceptIntroduceVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-获取提示信息存在性[by:zhaops]",
            notes = "conceptIds: 概念id，必填<br>")
    @PostMapping("/existListByConceptIds")
    @SysLogger("existListByConceptIds")
    @ApiIgnore
    public RespDTO<List<Long>> existListByConceptIds(@Valid @RequestBody ExistListByConceptIdsVO existListByConceptIdsVO) {
        List<Long> data = conceptDetailFacade.existListByConceptIds(existListByConceptIdsVO);
        return RespDTO.onSuc(data);
    }
}