package com.diagbot.client;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.annotation.SysLogger;
import com.diagbot.client.hystrix.CdssCoreHystrix;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.KlRuleByIdParDTO;
import com.diagbot.dto.KlRuleInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.KlRuleMenuWrapper;
import com.diagbot.enums.StatusEnum;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.DictionaryInfoVO;
import com.diagbot.vo.HasStaticKnowledgeVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.KlRuleByIdVO;
import com.diagbot.vo.KlRuleInfoClearVO;
import com.diagbot.vo.KlRuleInfoSaveVO;
import com.diagbot.vo.KlRuleInfoVO;
import com.diagbot.vo.KlRuleMenuVO;
import com.diagbot.vo.KlRuleSatartOrdisaVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.diagbot.vo.StaticKnowledgeVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/25 10:59
 */
@FeignClient(name = "cdss-core", url = "${cdss-core.url}", fallback = CdssCoreHystrix.class)
public interface CdssCoreClient {
    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    @PostMapping("/retrieval/staticKnowledgeIndex")
    RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@RequestBody @Valid StaticKnowledgeIndexVO staticKnowledgeIndexVO);

    /**
     * 更新是否有静态知识状态
     *
     * @param hasStaticKnowledgeVO
     * @return
     */
    @PostMapping("/staticKnowledge/updateHasInfoStatus")
    RespDTO<Boolean> updateHasInfoStatus(@Valid @RequestBody HasStaticKnowledgeVO hasStaticKnowledgeVO);

    /**
     * 术语批量校验
     *
     * @param conceptVO
     * @return
     */
    @PostMapping("/retrieval/getConceptNames")
    RespDTO<List<String>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO);

    /**
     * 化验大项、化验小项、辅检、诊断、药品、手术等检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/retrieval/index")
    RespDTO<RetrievalDTO> index(@RequestBody @Valid RetrievalVO retrievalVO);

    /**
     * 开单合理性推理
     *
     * @param indicationPushVO
     * @return
     */
    @PostMapping("/core/indication")
    RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO);

    /**
     * 根据名称和类型获取静态知识内容
     *
     * @param staticKnowledgeVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getStaticKnowledge")
    RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody StaticKnowledgeVO staticKnowledgeVO);

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getPage")
    RespDTO<Page<KlConceptStaticDTO>> getPage(@Valid @RequestBody KlConceptStaticPageVO klConceptStaticPageVO);

    /**
     * 保存静态知识
     *
     * @param klConceptStaticVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/saveOrUpdateRecord")
    RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO);

    /**
     * 静态知识启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/changeStatus")
    RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO);

    /**
     * 根据id获取静态知识
     *
     * @param idVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getRecordById")
    RespDTO<KlConceptStaticDTO> getRecordById(@Valid @RequestBody IdVO idVO);

    /**
     * 静态知识是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/isExist")
    RespDTO<Boolean> isExist(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO);

    /**
     *获取规则下拉菜单信息
     * @param klRuleMenuVO
     * @return
     */
    @PostMapping("/klRulePlan/getMenu")
    RespDTO<List<KlRuleMenuWrapper>> getMenus(@RequestBody KlRuleMenuVO klRuleMenuVO);

   //"分页获取规则维护列表
    @PostMapping("/klRule/getKlRuleInfoPage")
    RespDTO<Page<KlRuleInfoDTO>> getKlRuleInfoPages(@RequestBody KlRuleInfoVO klRuleInfoVO);


    //根据规则Id获取规则详情
    @PostMapping("/klRule/getByIdRuleInfo")
    RespDTO<KlRuleByIdParDTO> getByIdRuleInfoAll(@RequestBody @Valid KlRuleByIdVO klRuleByIdVO);

    //保存规则详情[
    @PostMapping("/klRule/saveRuleInfo")
    RespDTO<Boolean> saveRuleInfoAll(@RequestBody @Valid KlRuleInfoSaveVO klRuleInfoSaveVO);

    //刪除规则详情
    @PostMapping("/klRule/clearRuleInfo")
    RespDTO<Boolean> clearRuleInfoAll(@RequestBody @Valid KlRuleInfoClearVO klRuleInfoClearVO);

    //停用规则
    @PostMapping("/klRule/disableRuleInfo")
    RespDTO<Boolean> disableRuleInfos(@RequestBody @Valid KlRuleSatartOrdisaVO klRuleSatartOrdisaVO);

   //启用规则
    @PostMapping("/klRule/startRuleInfo")
    RespDTO<Boolean> startRuleInfos(@RequestBody @Valid KlRuleSatartOrdisaVO klRuleSatartOrdisaVO);

    @PostMapping("/cache/clear")
    RespDTO<Boolean> clearAlls();

    @PostMapping("/cache/clearDrug")
    RespDTO<Boolean> clearDrugloadDrug();

    @PostMapping("/cache/clearRuleInfoAll")
    RespDTO<Boolean> clearRuleAll();

    @PostMapping("/cache/clearStandRuleDrug")
    RespDTO<Boolean> clearStandRuleDrugAll();

    @PostMapping("/kl/dictionary/getDictionaryInfo")
    public RespDTO<List<DictionaryInfoDTO>> getDictionaryAll(@RequestBody DictionaryInfoVO dictionaryInfoVO);
}