package com.diagbot.client.hystrix;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.KlRuleByIdParDTO;
import com.diagbot.dto.KlRuleInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.KlRuleMenuWrapper;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.DictionaryInfoVO;
import com.diagbot.vo.HasStaticKnowledgeVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.KlRuleByIdVO;
import com.diagbot.vo.KlRuleInfoClearVO;
import com.diagbot.vo.KlRuleInfoSaveVO;
import com.diagbot.vo.KlRuleInfoVO;
import com.diagbot.vo.KlRuleMenuVO;
import com.diagbot.vo.KlRuleSatartOrdisaVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.diagbot.vo.StaticKnowledgeVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/25 10:59
 */
@Component
@Slf4j
public class CdssCoreHystrix implements CdssCoreClient {
    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    @Override
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@RequestBody @Valid StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        log.error("【hystrix】调用{}异常", "staticKnowledgeIndex");
        return null;
    }

    /**
     * 更新是否有静态知识状态
     *
     * @param hasStaticKnowledgeVO
     * @return
     */
    @Override
    public RespDTO<Boolean> updateHasInfoStatus(@Valid @RequestBody HasStaticKnowledgeVO hasStaticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "updateHasInfoStatus");
        return null;
    }

    /**
     * 术语批量校验
     *
     * @param conceptVO
     * @return
     */
    @Override
    public RespDTO<List<String>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO) {
        log.error("【hystrix】调用{}异常", "getConceptNames");
        return null;
    }

    /**
     * 化验大项、化验小项、辅检、诊断、药品、手术等检索
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public RespDTO<RetrievalDTO> index(@RequestBody @Valid RetrievalVO retrievalVO) {
        log.error("【hystrix】调用{}异常", "index");
        return null;
    }

    /**
     * 开单合理性推理
     *
     * @param indicationPushVO
     * @return
     */
    @Override
    public RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO) {
        log.error("【hystrix】调用{}异常", "indication");
        return null;
    }

    /**
     * 根据名称和类型获取静态知识内容
     *
     * @param staticKnowledgeVO
     * @return
     */
    @Override
    public RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody StaticKnowledgeVO staticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "getStaticKnowledge");
        return null;
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    @Override
    public RespDTO<Page<KlConceptStaticDTO>> getPage(@Valid @RequestBody KlConceptStaticPageVO klConceptStaticPageVO) {
        log.error("【hystrix】调用{}异常", "getPage");
        return null;
    }

    /**
     * 保存静态知识
     *
     * @param klConceptStaticVO
     * @return
     */
    @Override
    public RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        log.error("【hystrix】调用{}异常", "saveOrUpdateRecord");
        return null;
    }

    /**
     * 静态知识启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    @Override
    public RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        log.error("【hystrix】调用{}异常", "changeStatus");
        return null;
    }

    /**
     * 根据id获取静态知识
     *
     * @param idVO
     * @return
     */
    @Override
    public RespDTO<KlConceptStaticDTO> getRecordById(@Valid @RequestBody IdVO idVO) {
        log.error("【hystrix】调用{}异常", "getRecordById");
        return null;
    }

    /**
     * 静态知识是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    @Override
    public RespDTO<Boolean> isExist(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        log.error("【hystrix】调用{}异常", "isExist");
        return null;
    }

    @Override
    public RespDTO<List<KlRuleMenuWrapper>> getMenus(@RequestBody KlRuleMenuVO klRuleMenuVO) {
        log.error("【hystrix】调用{}异常", "getMenus");
        return null;
    }

    @Override
    public RespDTO<Page<KlRuleInfoDTO>> getKlRuleInfoPages(@RequestBody KlRuleInfoVO klRuleInfoVO) {
        log.error("【hystrix】调用{}异常", "getKlRuleInfoPages");
        return null;
    }

    @Override
    public RespDTO<KlRuleByIdParDTO> getByIdRuleInfoAll(@Valid @RequestBody KlRuleByIdVO klRuleByIdVO) {
        log.error("【hystrix】调用{}异常", "getByIdRuleInfoAll");
        return null;
    }

    @Override
    public RespDTO<Boolean> saveRuleInfoAll(@Valid @RequestBody KlRuleInfoSaveVO klRuleInfoSaveVO) {
        log.error("【hystrix】调用{}异常", "saveRuleInfoAll");
        return null;
    }

    @Override
    public RespDTO<Boolean> clearRuleInfoAll(@Valid @RequestBody KlRuleInfoClearVO klRuleInfoClearVO) {
        log.error("【hystrix】调用{}异常", "clearRuleInfoAll");
        return null;
    }

    @Override
    public RespDTO<Boolean> disableRuleInfos(@Valid @RequestBody KlRuleSatartOrdisaVO klRuleSatartOrdisaVO) {
        log.error("【hystrix】调用{}异常", "disableRuleInfos");
        return null;
    }

    @Override
    public RespDTO<Boolean> startRuleInfos(@Valid @RequestBody KlRuleSatartOrdisaVO klRuleSatartOrdisaVO) {
        log.error("【hystrix】调用{}异常", "startRuleInfos");
        return null;
    }

    @Override
    public RespDTO<Boolean> clearAlls() {
        log.error("【hystrix】调用{}异常", "clearAlls");
        return null;
    }

    @Override
    public RespDTO<Boolean> clearDrugloadDrug() {
        log.error("【hystrix】调用{}异常", "clearDrugloadDrug");
        return null;
    }

    @Override
    public RespDTO<Boolean> clearRuleAll() {
        log.error("【hystrix】调用{}异常", "clearRuleAll");
        return null;
    }

    @Override
    public RespDTO<Boolean> clearStandRuleDrugAll() {
        log.error("【hystrix】调用{}异常", "clearStandRuleDrugAll");
        return null;
    }

    @Override
    public RespDTO<List<DictionaryInfoDTO>> getDictionaryAll(DictionaryInfoVO dictionaryInfoVO) {
        log.error("【hystrix】调用{}异常", "getDictionaryAll");
        return null;
    }
}
