package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.HospitalInfoDTO;
import com.diagbot.entity.DeptConfig;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.DrugConfig;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.entity.LisConfig;
import com.diagbot.entity.OperationConfig;
import com.diagbot.entity.PacsConfig;
import com.diagbot.entity.Plan;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.DeptConfigService;
import com.diagbot.service.DiseaseConfigService;
import com.diagbot.service.DrugConfigService;
import com.diagbot.service.HospitalInfoService;
import com.diagbot.service.LisConfigService;
import com.diagbot.service.OperationConfigService;
import com.diagbot.service.PacsConfigService;
import com.diagbot.service.TransfusionConfigService;
import com.diagbot.service.impl.HospitalInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.Cn2SpellUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.HospitalInfoListVO;
import com.diagbot.vo.HospitalInfoPageVO;
import com.diagbot.vo.HospitalSaveVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-08-06 18:37
 */
@Component
public class HospitalInfoFacade extends HospitalInfoServiceImpl {
    @Autowired
    @Qualifier("hospitalInfoServiceImpl")
    private HospitalInfoService hospitalInfoService;
    @Autowired
    private DeptConfigService deptConfigService;
    @Autowired
    private DiseaseConfigService diseaseConfigService;
    @Autowired
    private DrugConfigService drugConfigService;
    @Autowired
    private LisConfigService lisConfigService;
    @Autowired
    private OperationConfigService operationConfigService;
    @Autowired
    private PacsConfigService pacsConfigService;
    @Autowired
    private TransfusionConfigService transfusionConfigService;
    @Autowired
    private PlanFacade planFacade;

    public List<HospitalInfoDTO> getHospitalInfo() {
        QueryWrapper<HospitalInfo> hospitalInfo = new QueryWrapper<>();
        hospitalInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
        List<HospitalInfo> list = list(hospitalInfo);
        List<HospitalInfoDTO> data = new ArrayList<>();
        data = BeanUtil.listCopyTo(list, HospitalInfoDTO.class);
        return data;
    }

    /**
     * 保存记录-单条
     *
     * @param hospitalInfo
     * @return
     */
    public Boolean saveOrUpdateRecord(HospitalInfo hospitalInfo) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        hospitalInfo.setModifier(userId);
        hospitalInfo.setGmtModified(now);
        QueryWrapper<HospitalInfo> queryWrapper = new QueryWrapper<>();
        if (StringUtil.isNotBlank(hospitalInfo.getName())) {
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
            queryWrapper.eq("name", hospitalInfo.getName());
            HospitalInfo oldRecord = this.getOne(queryWrapper, false);
            if (hospitalInfo.getId() == null
                    && oldRecord != null) {
                throw new CommonException(CommonErrorCode.IS_EXISTS, "该医院名称已存在");
            }
            if (hospitalInfo.getId() != null
                    && oldRecord != null
                    && !hospitalInfo.getId().equals(oldRecord.getId())) {
                throw new CommonException(CommonErrorCode.IS_EXISTS, "该医院名称已存在");
            }
        }
        /*if (StringUtil.isNotBlank(hospitalInfo.getCode())) {
            queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
            queryWrapper.eq("code", hospitalInfo.getCode());
            HospitalInfo oldRecord = this.getOne(queryWrapper, false);
            if (hospitalInfo.getId() == null
                    && oldRecord != null) {
                throw new CommonException(CommonErrorCode.IS_EXISTS, "该医院编码已存在");
            }
            if (hospitalInfo.getId() != null
                    && oldRecord != null
                    && !hospitalInfo.getId().equals(oldRecord.getId())) {
                throw new CommonException(CommonErrorCode.IS_EXISTS, "该医院编码已存在");
            }
        }*/

        //新增数据
        if (hospitalInfo.getId() == null) {
            hospitalInfo.setCreator(userId);
            hospitalInfo.setGmtCreate(now);
            hospitalInfo.setConnect(1);
        }
        if (StringUtil.isBlank(hospitalInfo.getSpell())) {
            hospitalInfo.setSpell(Cn2SpellUtil.converterToFirstSpell(hospitalInfo.getName()));
        }
        if (hospitalInfo.getIsDeleted() == null) {
            hospitalInfo.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        this.saveOrUpdate(hospitalInfo);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param hospitalInfoListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(HospitalInfoListVO hospitalInfoListVO) {
        if (ListUtil.isEmpty(hospitalInfoListVO.getHospitalInfoList())) {
            return false;
        }
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        hospitalInfoListVO.getHospitalInfoList().forEach(item -> {
            item.setModifier(userId);
            item.setGmtModified(now);
            if (item.getId() == null) {
                item.setCreator(userId);
                item.setGmtCreate(now);
            }
        });
        hospitalInfoService.saveOrUpdateBatch(hospitalInfoListVO.getHospitalInfoList());
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        HospitalInfo hospitalInfo = hospitalInfoService.getById(idVO.getId());
        if (hospitalInfo == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "数据不存在");
        }
        if (hospitalInfo.getIsDeleted().equals(IsDeleteEnum.Y.getKey())) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "数据已删除");
        }

        //校验关联数据
        List<DeptConfig> deptConfigList
                = deptConfigService.list(new QueryWrapper<DeptConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(deptConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<DiseaseConfig> diseaseConfigList
                = diseaseConfigService.list(new QueryWrapper<DiseaseConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(diseaseConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<DrugConfig> drugConfigList
                = drugConfigService.list(new QueryWrapper<DrugConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(drugConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<LisConfig> lisConfigList
                = lisConfigService.list(new QueryWrapper<LisConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(lisConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<OperationConfig> operationConfigList
                = operationConfigService.list(new QueryWrapper<OperationConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(operationConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<PacsConfig> pacsConfigList
                = pacsConfigService.list(new QueryWrapper<PacsConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(pacsConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<TransfusionConfig> transfusionConfigList
                = transfusionConfigService.list(new QueryWrapper<TransfusionConfig>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(transfusionConfigList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }
        List<Plan> planList
                = planFacade.list(new QueryWrapper<Plan>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", idVO.getId()));
        if (ListUtil.isNotEmpty(planList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医院存在关联数据，无法删除");
        }


        hospitalInfo.setIsDeleted(IsDeleteEnum.Y.getKey());
        hospitalInfo.setModifier(userId);
        hospitalInfo.setGmtModified(now);
        hospitalInfoService.updateById(hospitalInfo);
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        List<HospitalInfo> hospitalInfoList = new ArrayList<>(hospitalInfoService.listByIds(idListVO.getIds()));

        if (ListUtil.isEmpty(hospitalInfoList)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "数据不存在");
        }

        hospitalInfoList.forEach(hospitalInfo -> {
            hospitalInfo.setIsDeleted(IsDeleteEnum.Y.getKey());
            hospitalInfo.setModifier(userId);
            hospitalInfo.setGmtModified(now);
        });
        hospitalInfoService.updateBatchById(hospitalInfoList);
        return true;
    }

    /**
     * 分页查询
     *
     * @param hospitalInfoPageVO
     * @return
     */
    public IPage<DiseaseConfig> getPage(HospitalInfoPageVO hospitalInfoPageVO) {
        return hospitalInfoService.getPage(hospitalInfoPageVO);
    }

    /**
     * 保存医院信息--医院端只能修改对接状态
     *
     * @param hospitalSaveVO
     * @return
     */
    public Boolean modifyConnectStatus(HospitalSaveVO hospitalSaveVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        UpdateWrapper<HospitalInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("connect", hospitalSaveVO.getConnect())
                .set("modifier", userId)
                .set("gmt_modified", now)
                .eq("id", hospitalSaveVO.getId());
        return this.update(updateWrapper);
    }

    public Map<Long, String> getHospitalInfoData(List<Long> hospitalId) {
        QueryWrapper<HospitalInfo> hospitalInfo = new QueryWrapper<>();
        hospitalInfo.eq("is_deleted", IsDeleteEnum.N.getKey()).in("id", hospitalId);
        List<HospitalInfo> list = list(hospitalInfo);
        List<HospitalInfoDTO> data = new ArrayList<>();
        data = BeanUtil.listCopyTo(list, HospitalInfoDTO.class);
        Map<Long, String> hospitalInfoData = data.stream().collect(Collectors.toMap(k -> k.getId(), v -> v.getName()));
        return hospitalInfoData;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        UpdateWrapper<HospitalInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", changeStatusVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("modifier", userId)
                .set("status", changeStatusVO.getStatus());
        this.update(updateWrapper);
        return true;
    }
}
