package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.KlConceptAllDTO;
import com.diagbot.dto.KlConceptAllSubDTO;
import com.diagbot.dto.KlConceptInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlConceptCommon;
import com.diagbot.entity.KlDisease;
import com.diagbot.entity.KlLibraryInfo;
import com.diagbot.entity.KlLis;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.KlConceptCommonService;
import com.diagbot.service.KlLisService;
import com.diagbot.service.impl.KlConceptServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.GetAllForRelationVO;
import com.diagbot.vo.KlConceptAllVO;
import com.diagbot.vo.KlConceptClearVO;
import com.diagbot.vo.KlConceptInfoVO;
import com.diagbot.vo.KlConceptSatarOrdisaVO;
import com.diagbot.vo.KlConceptSaveSubVO;
import com.diagbot.vo.KlConceptSaveVO;
import com.diagbot.vo.SearchConceptVO;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2021-03-01 13:21
 */
@Component
public class KlConceptFacade extends KlConceptServiceImpl {
    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private KlLibraryInfoFacade klLibraryInfoFacade;
    @Autowired
    private KlConceptCommonService klConceptCommonService;
    @Autowired
    KlLisService klLisService;
    @Autowired
    KlDiseaseFacade klDiseaseFacade;

    /**
     * @param klConceptInfoVO
     * @return
     */
    public IPage<KlConceptInfoDTO> getConceptInfoPage(KlConceptInfoVO klConceptInfoVO) {
        IPage<KlConceptInfoDTO> conceptPage = getConceptPage(klConceptInfoVO);
        List<String> userIds = conceptPage.getRecords().stream()
                .map(i -> i.getModifierId()).distinct().collect(Collectors.toList());
        RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(userIds);
        RespDTOUtil.respNGDealCover(respDTO, "获取用户信息失败");
        conceptPage.getRecords().forEach(i -> {
            i.setModifierName(respDTO.data.get(i.getModifierId()));
        });
        return conceptPage;
    }

    /**
     * @param klConceptAllVO
     * @return
     */
    public KlConceptAllDTO getConceptAlls(KlConceptAllVO klConceptAllVO) {
        List<KlConceptAllDTO> conceptAll = getConceptAll(klConceptAllVO);
        KlConceptAllDTO klConceptAll = new KlConceptAllDTO();
        List<KlConceptAllSubDTO> klConceptSub = new ArrayList<>();
        if (ListUtil.isNotEmpty(conceptAll)) {
            for (KlConceptAllDTO data : conceptAll) {
                if (data.getIsConcept().equals(1)) {
                    BeanUtil.copyProperties(data, klConceptAll);
                } else {
                    KlConceptAllSubDTO klConceptAllSubDTO = new KlConceptAllSubDTO();
                    BeanUtil.copyProperties(data, klConceptAllSubDTO);
                    klConceptSub.add(klConceptAllSubDTO);
                }
            }
        }
        klConceptAll.setKlConceptSub(klConceptSub);
        return klConceptAll;
    }

    /**
     * @param klConceptSaveVO
     * @return
     */
    public Boolean saveConceptInfo(KlConceptSaveVO klConceptSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
        // 校验名称是否相同
        int count = this.count(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_type", klConceptSaveVO.getLibType())
                .eq("lib_name", klConceptSaveVO.getLibName())
                .ne("id", klConceptSaveVO.getConceptId() == null ? -1 : klConceptSaveVO.getConceptId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该基础术语名称已存在");
        }
        //校验
        KlConceptSaveSubVO paesubVO = new KlConceptSaveSubVO();
        paesubVO.setConceptId(klConceptSaveVO.getConceptId());
        paesubVO.setIsConcept(klConceptSaveVO.getIsConcept());
        paesubVO.setLibId(klConceptSaveVO.getLibId());
        paesubVO.setSynonymName(klConceptSaveVO.getSynonymName());
        klLibraryInfoFacade.checkLibraryInfoData(paesubVO);
        //先判断是新加的,还是修改
        if (null != klConceptSaveVO.getConceptId() && null != klConceptSaveVO.getLibId()) {
            //更新标准词
            res = updateAll(klConceptSaveVO);
            //更新同义词
            List<KlConceptSaveSubVO> klConceptAllSub = klConceptSaveVO.getKlConceptSub();
            if (klConceptAllSub.size() > 0) {
                for (KlConceptSaveSubVO subVO : klConceptAllSub) {
                    if (subVO.getLibId() == null) {
                        //校验
                        klLibraryInfoFacade.checkLibraryInfoData(subVO);
                        //保存LibraryInfo
                        res = klLibraryInfoFacade.savekLibraryInfoData(klConceptSaveVO.getConceptId(),subVO);
                    } else {
                        UpdateWrapper<KlLibraryInfo> klLibraryInfoQuerySub = new UpdateWrapper<>();
                        klLibraryInfoQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                                .eq("id", klConceptSaveVO.getLibId())
                                .set("gmt_modified", now)
                                .set("modifier", UserUtils.getCurrentPrincipleID())
                                .set("name", klConceptSaveVO.getLibName())
                                .set("spell", klConceptSaveVO.getLibName())
                                .set("remark", klConceptSaveVO.getLibName());
                        res = klLibraryInfoFacade.update(new KlLibraryInfo(), klLibraryInfoQuerySub);
                    }
                }
            }
        } else {
            //保存新数据
            //1.先保存标准词，生成conceptId
            KlConcept klConceptPare = new KlConcept();
            Long libId = klConceptSaveVO.getLibId();
            if (null == libId) {
                libId = Long.valueOf(-(new Random().nextInt(100000)));
            }
            klConceptPare.setLibName(klConceptSaveVO.getLibName());
            klConceptPare.setLibType(klConceptSaveVO.getLibType());
            klConceptPare.setStatus(klConceptSaveVO.getStatus());
            klConceptPare.setLibId(libId);
            klConceptPare.setCreator(UserUtils.getCurrentPrincipleID());
            klConceptPare.setModifier(UserUtils.getCurrentPrincipleID());
            klConceptPare.setGmtCreate(now);
            klConceptPare.setGmtModified(now);
            res = save(klConceptPare);
            if (res) {
                //2.再把标准词d的conceptId保存到kl_library_info表 生成libId，
                Long conceptId = klConceptPare.getId();
                KlLibraryInfo klLibraryInfoSub = new KlLibraryInfo();
                klLibraryInfoSub.setConceptId(conceptId);
                klLibraryInfoSub.setName(klConceptSaveVO.getSynonymName());
                klLibraryInfoSub.setSpell(klConceptSaveVO.getSpell());
                klLibraryInfoSub.setIsConcept(klConceptSaveVO.getIsConcept());
                klLibraryInfoSub.setRemark(klConceptSaveVO.getRemark());
                klLibraryInfoSub.setCreator(UserUtils.getCurrentPrincipleID());
                klLibraryInfoSub.setModifier(UserUtils.getCurrentPrincipleID());
                klLibraryInfoSub.setGmtCreate(now);
                klLibraryInfoSub.setGmtModified(now);
                res = klLibraryInfoFacade.save(klLibraryInfoSub);
                if (res) {
                    //3.最后更新回concept表里得lid_id字段
                    UpdateWrapper<KlConcept> klConceptQuerySub = new UpdateWrapper<>();
                    klConceptQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                            .eq("id", conceptId)
                            .set("lib_Id", klLibraryInfoSub.getId());
                    res = update(new KlConcept(), klConceptQuerySub);
                }
            }
            //KlConceptCommon
            if (null != klConceptSaveVO.getIsHasCommon() && klConceptSaveVO.getIsHasCommon().equals(1)) {
                KlConceptCommon conceptCommon = new KlConceptCommon();
                conceptCommon.setSexType(klConceptSaveVO.getSexType());
                conceptCommon.setMaxAge(klConceptSaveVO.getMaxAge());
                conceptCommon.setMinAge(klConceptSaveVO.getMinAge());
                conceptCommon.setConceptId(klConceptPare.getId());
                conceptCommon.setCreator(UserUtils.getCurrentPrincipleID());
                conceptCommon.setModifier(UserUtils.getCurrentPrincipleID());
                conceptCommon.setGmtCreate(now);
                conceptCommon.setGmtModified(now);
                klConceptCommonService.save(conceptCommon);
            }
            //更新kl_disease
            KlDisease kllDiseaseSub = new KlDisease();
            kllDiseaseSub.setConceptId(klConceptPare.getId());
            kllDiseaseSub.setGmtModified(now);
            kllDiseaseSub.setIcd10Code(klConceptSaveVO.getIcdCode());
            kllDiseaseSub.setCreator(UserUtils.getCurrentPrincipleID());
            kllDiseaseSub.setModifier(UserUtils.getCurrentPrincipleID());
            kllDiseaseSub.setGmtCreate(now);
            kllDiseaseSub.setGmtModified(now);
            res = klDiseaseFacade.saveOrUpdate(kllDiseaseSub);
            //更新kl_lis
            KlLis klLisSub = new KlLis();
            klLisSub.setConceptId(klConceptPare.getId());
            klLisSub.setMinValue(klConceptSaveVO.getMinVal());
            klLisSub.setMaxValue(klConceptSaveVO.getMaxVal());
            klLisSub.setType(klConceptSaveVO.getScopeType());
            klLisSub.setCreator(UserUtils.getCurrentPrincipleID());
            klLisSub.setModifier(UserUtils.getCurrentPrincipleID());
            klLisSub.setGmtCreate(now);
            klLisSub.setGmtModified(now);
            res = klLisService.saveOrUpdate(klLisSub);
            //增加同义词
            List<KlConceptSaveSubVO> klConceptSub = klConceptSaveVO.getKlConceptSub();
            if (ListUtil.isNotEmpty(klConceptSub)) {
                for (KlConceptSaveSubVO subVO : klConceptSub) {
                    if (subVO.getLibId() == null) {

                        klLibraryInfoFacade.checkLibraryInfoData(subVO);

                        res = klLibraryInfoFacade.savekLibraryInfoData(klConceptPare.getId(),subVO);
                    }
                }
            }


        }
        return res;
    }

    public Boolean updateAll(KlConceptSaveVO klConceptSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
        UpdateWrapper<KlConcept> klConceptQuerySub = new UpdateWrapper<>();
        klConceptQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSaveVO.getConceptId())
                .set("gmt_modified", now)
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("lib_name", klConceptSaveVO.getLibName());
        res = update(new KlConcept(), klConceptQuerySub);

        UpdateWrapper<KlLibraryInfo> klLibraryInfoQuerySub = new UpdateWrapper<>();
        klLibraryInfoQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSaveVO.getLibId())
                .set("gmt_modified", now)
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("name", klConceptSaveVO.getLibName())
                .set("spell", klConceptSaveVO.getSpell())
                .set("remark", klConceptSaveVO.getRemark());

        res = klLibraryInfoFacade.update(new KlLibraryInfo(), klLibraryInfoQuerySub);
        //更新kl_concept_common表
        if (null != klConceptSaveVO.getIsHasCommon() && klConceptSaveVO.getIsHasCommon().equals(1)) {
            UpdateWrapper<KlConceptCommon> klConceptCommonQuerySub = new UpdateWrapper<>();
            klConceptCommonQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("concept_id", klConceptSaveVO.getConceptId())
                    .set("gmt_modified", now)
                    .set("modifier", UserUtils.getCurrentPrincipleID())
                    .set("sex_type", klConceptSaveVO.getLibName())
                    .set("min_age", klConceptSaveVO.getLibName())
                    .set("max_age", klConceptSaveVO.getLibName());
            KlConceptCommon klConceptCommonSub = new KlConceptCommon();
            klConceptCommonSub.setConceptId(klConceptSaveVO.getConceptId());
            klConceptCommonSub.setMaxAge(klConceptSaveVO.getMaxAge());
            klConceptCommonSub.setMinAge(klConceptSaveVO.getMinAge());
            klConceptCommonSub.setGmtModified(now);
            klConceptCommonSub.setModifier(UserUtils.getCurrentPrincipleID());
            res = klConceptCommonService.saveOrUpdate(klConceptCommonSub);
        }
        //更新kl_disease
        if (null != klConceptSaveVO.getIcdCode()) {
            KlDisease kllDiseaseSub = new KlDisease();
            kllDiseaseSub.setConceptId(klConceptSaveVO.getConceptId());
            kllDiseaseSub.setGmtModified(now);
            kllDiseaseSub.setModifier(UserUtils.getCurrentPrincipleID());
            kllDiseaseSub.setIcd10Code(klConceptSaveVO.getIcdCode());
            res = klDiseaseFacade.saveOrUpdate(kllDiseaseSub);
        }
        //更新kl_lis
        if (null != klConceptSaveVO.getScopeType()) {
            KlLis klLisSub = new KlLis();
            klLisSub.setConceptId(klConceptSaveVO.getConceptId());
            klLisSub.setGmtModified(now);
            klLisSub.setModifier(UserUtils.getCurrentPrincipleID());
            klLisSub.setMinValue(klConceptSaveVO.getMinVal());
            klLisSub.setMaxValue(klConceptSaveVO.getMaxVal());
            klLisSub.setType(klConceptSaveVO.getScopeType());
            res = klLisService.saveOrUpdate(klLisSub);
        }
        return res;
    }

    /**
     * 获取所有概念(术语关系维护时筛选使用)
     *
     * @param getAllForRelationVO
     * @return
     */
    public List<GetAllForRelationDTO> getAllForRelation(GetAllForRelationVO getAllForRelationVO) {
        List<GetAllForRelationDTO> retList = Lists.newArrayList();

        if (StringUtil.isNotBlank(getAllForRelationVO.getName())
                || null != getAllForRelationVO.getTypeId()) {
            List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like(StringUtil.isNotBlank(getAllForRelationVO.getName()), "lib_name", getAllForRelationVO.getName().trim())
                    .eq(getAllForRelationVO.getTypeId() != null, "lib_type", getAllForRelationVO.getTypeId())
                    .notIn(ListUtil.isNotEmpty(getAllForRelationVO.getExcludedConceptIds()), "id", getAllForRelationVO.getExcludedConceptIds())
            );
            //            retList = BeanUtil.listCopyTo(conceptList, GetAllForRelationDTO.class);
            if (ListUtil.isNotEmpty(conceptList)) {
                retList = conceptList.stream().map(x -> {
                    GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                    getAllForRelationDTO.setConceptId(x.getId());
                    getAllForRelationDTO.setConceptName(x.getLibName());
                    getAllForRelationDTO.setConceptNameType(String.valueOf(x.getLibType()));
                    getAllForRelationDTO.setLibType(x.getLibType());
                    return getAllForRelationDTO;
                }).collect(Collectors.toList());
            }

        }
        return retList;
    }

    public List<GetAllForRelationDTO> searchConceptByNameAndLibType(SearchConceptVO searchConceptVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = Lists.newArrayList();
        String name = searchConceptVO.getName();
        Integer libType = searchConceptVO.getLibType();
        List<Long> excludedConceptIds = searchConceptVO.getExcludedConceptIds();
        if (StringUtils.isNotBlank(name) && libType != null) {
            List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like(StringUtil.isNotBlank(name), "lib_name", name.trim())
                    .eq("lib_type", libType)
                    .notIn(ListUtil.isNotEmpty(excludedConceptIds), "id", excludedConceptIds));
            if (ListUtil.isNotEmpty(conceptList)) {
                getAllForRelationDTOS = conceptList.stream().map(x -> {
                    GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                    getAllForRelationDTO.setConceptNameType(x.getLibName());
                    getAllForRelationDTO.setConceptName(x.getLibName());
                    getAllForRelationDTO.setConceptId(x.getId());
                    getAllForRelationDTO.setLibType(x.getLibType());
                    return getAllForRelationDTO;
                }).collect(Collectors.toList());
            }
        }
        return getAllForRelationDTOS;
    }

    /**
     * 筛选符合类型的概念id
     *
     * @param libTypeId
     * @param sourceConceptIds
     * @return
     */
    public List<Long> getCompatibleTypeConceptIds(Long libTypeId, List<Long> sourceConceptIds) {
        if (libTypeId == null || ListUtil.isEmpty(sourceConceptIds)) {
            return null;
        }
        QueryWrapper<KlConcept> conceptQe = new QueryWrapper<>();
        conceptQe.in("id", sourceConceptIds);
        conceptQe.eq("lib_type", libTypeId);
        return list(conceptQe).stream().map(i -> i.getId()).collect(Collectors.toList());
    }

    public Boolean clearConceptInfo(KlConceptClearVO klConceptClearVO) {
        Date now = DateUtil.now();
        int sum = klLibraryInfoFacade.count(new QueryWrapper<KlLibraryInfo>().eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptClearVO.getLibId()).eq("is_concept", 0));
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存在或删除的不是同义词！");
        }
        UpdateWrapper<KlLibraryInfo> klLibraryUpdate = new UpdateWrapper<>();
        klLibraryUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptClearVO.getLibId()).set("is_deleted", IsDeleteEnum.Y.getKey()).set("gmt_modified", now);
        return klLibraryInfoFacade.update(klLibraryUpdate);
    }

    public Boolean startOrDisableRule(KlConceptSatarOrdisaVO klConceptSatarOrdisaVO, int status) {
        Date now = DateUtil.now();
        int sum = this.count(new QueryWrapper<KlConcept>().eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSatarOrdisaVO.getConceptId()));
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存！");
        }
        UpdateWrapper<KlConcept> klLibraryUpdate = new UpdateWrapper<>();
        klLibraryUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSatarOrdisaVO.getConceptId()).set("status", status).set("gmt_modified", now);
        return this.update(klLibraryUpdate);

    }
}
