package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.*;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlLexiconRelationship;
import com.diagbot.enums.DiseaseRelationTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.RelationLibTypeEnum;
import com.diagbot.service.KlDiseaseService;
import com.diagbot.service.KlLexiconRelationshipService;
import com.diagbot.service.impl.KlDiseaseServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.DiseaseDeleteVO;
import com.diagbot.vo.DiseasePageVO;
import com.diagbot.vo.GetDiseaseDetailVO;
import com.google.common.collect.Maps;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2021-03-09 15:33
 */
@Component
public class KlDiseaseFacade extends KlDiseaseServiceImpl {
    @Autowired
    KlLexiconRelationshipFacade klLexiconRelationshipFacade;
    @Autowired
    KlConceptFacade klConceptFacade;

    /**
     * 诊断依据分页查询
     *
     * @param klConceptInfoVO
     * @return
     */
    public IPage<DiseasePageDTO> getDiseasePageAll(DiseasePageVO klConceptInfoVO) {
        return this.getDiseasePage(klConceptInfoVO);
    }

    public List<KlConcept> getDiseaseAll(DiseaseDeleteVO diseaseDeleteVO) {
        List<KlConcept> klConceptss = new ArrayList<>();
        List<DiseasePageDTO> diseasePage = this.getDiseaseAlls(diseaseDeleteVO);
        List<String> diseases = diseasePage.stream().map(x -> x.getDisName()).collect(Collectors.toList());
        String disName = diseaseDeleteVO.getDisName();
        if (StringUtil.isNotEmpty(disName)) {
            List<KlConcept> klConcepts = klConceptFacade.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like("lib_name", disName.trim())
                    .eq("lib_type", 100));
            Map<String, List<KlConcept>> libNameMap = EntityUtil.makeEntityListMap(klConcepts, "libName");
            if (MapUtils.isNotEmpty(libNameMap) && ListUtil.isNotEmpty(diseases)) {
                libNameMap.forEach((x, y) -> {
                    if (!diseases.contains(x)) {
                        klConceptss.addAll(y);
                    }
                });
            }
        }

        return klConceptss;
    }


    /**
     * 某个诊断依据的具体数据
     *
     * @param getDiseaseDetailVO
     * @return
     */
    public DiseaseTreeDetailDTO getDetail(GetDiseaseDetailVO getDiseaseDetailVO) {
        List<Integer> relations = Arrays.asList(501,502,503,504,505,506,507,508);
        List<KlLexiconRelationship> klLexiconRelationships = klLexiconRelationshipFacade.list(new QueryWrapper<KlLexiconRelationship>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("code", relations)
        .orderByAsc("code"));
        Map<Integer, String> relation_name = klLexiconRelationships.stream().collect(Collectors.toMap(KlLexiconRelationship::getCode, KlLexiconRelationship::getName, (key1, key2) -> key2));
        DiseaseTreeDetailDTO diseaseTreeDetailDTO = new DiseaseTreeDetailDTO();

        DiseaseDetailDTO diseaseDetailDTO = new DiseaseDetailDTO();
        Map<Integer, DiseaseSubDetailDTO> disSubDetailMap = Maps.newHashMap();
        List<DiseaseAllDetailDTO> diseaseAllDetailDTOS = this.getDiseaseDetail(getDiseaseDetailVO);
        if (ListUtil.isNotEmpty(diseaseAllDetailDTOS)) {
            diseaseAllDetailDTOS.forEach(x -> {
                diseaseDetailDTO.setSid(x.getSid());
                diseaseDetailDTO.setDisName(x.getDisName());
                if (disSubDetailMap.containsKey(x.getRid())) {
                    DiseaseSubDetailDTO diseaseSubDetailDTO = disSubDetailMap.get(x.getRid());

                    diseaseSubDetailDTO.setRid(x.getRid());
                    diseaseSubDetailDTO.setRname(x.getRname());
                    List<KlConcept> details = diseaseSubDetailDTO.getDetails();
                    diseaseSubDetailDTO.setDetails(details);
                    KlConcept klConcept = new KlConcept();
                    klConcept.setId(x.getEid());
                    klConcept.setLibName(x.getItemName());
                    details.add(klConcept);

                    disSubDetailMap.put(x.getRid(), diseaseSubDetailDTO);
                } else {
                    DiseaseSubDetailDTO diseaseSubDetailDTO = new DiseaseSubDetailDTO();

                    diseaseSubDetailDTO.setRid(x.getRid());
                    diseaseSubDetailDTO.setRname(x.getRname());
                    List<KlConcept> details = Lists.newArrayList();
                    diseaseSubDetailDTO.setDetails(details);
                    KlConcept klConcept = new KlConcept();
                    klConcept.setId(x.getEid());
                    klConcept.setLibName(x.getItemName());
                    details.add(klConcept);

                    disSubDetailMap.put(x.getRid(), diseaseSubDetailDTO);
                }

            });
        }
        List<DiseaseSubDetailDTO> datas = Lists.newArrayList();
        List<List<String>> types = Lists.newArrayList();
        relation_name.forEach((x,y)->{
            List<String> split = Arrays.asList(DiseaseRelationTypeEnum.getName(x).split("&"));
            List<Integer> collect = Arrays.stream(RelationLibTypeEnum.getName(x).split("&")).map(z -> Integer.parseInt(z)).collect(Collectors.toList());
            types.add(split);
            if(disSubDetailMap.containsKey(x)){
                DiseaseSubDetailDTO diseaseSubDetailDTO = disSubDetailMap.get(x);
                diseaseSubDetailDTO.setLibType(collect);
                datas.add(diseaseSubDetailDTO);
            }else {
                DiseaseSubDetailDTO diseaseSubDetailDTO = new DiseaseSubDetailDTO();
                diseaseSubDetailDTO.setLibType(collect);
                diseaseSubDetailDTO.setRname(relation_name.get(x));
                diseaseSubDetailDTO.setRid(x);
                datas.add(diseaseSubDetailDTO);
            }
        });
        diseaseDetailDTO.setDatas(datas);
        diseaseTreeDetailDTO.setDiseaseDetailDTO(diseaseDetailDTO);
        diseaseTreeDetailDTO.setTypes(types);
        return diseaseTreeDetailDTO;
    }


}

