package com.diagbot.facade;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.MedClassMedDTO;
import com.diagbot.dto.RuleDTO;
import com.diagbot.entity.ResultRule;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleBaseTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RuleVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.stream.Collectors;


/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/3/1 16:49
 */
@Component
public class MedRuleConvertFacade {
    @Autowired
    private KlRuleFacade klRuleFacade;
    @Autowired
    private KlRelationFacade klRelationFacade;

    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;

    /**
     * 开单规则处理
     *
     * @param hospitalId
     * @param caseId
     * @return
     */
    public List<ResultRule> billConvert(Long hospitalId, Long caseId) {
        List<ResultRule> retList = Lists.newLinkedList();
        List<RuleDTO> groupList = Lists.newLinkedList();
        List<ResultRule> inputList = Lists.newLinkedList();
        List<ResultRule> medicineList = Lists.newLinkedList();
        List<ResultRule> containsDiseaseList = Lists.newLinkedList();
        List<ResultRule> rePacsBillList = Lists.newLinkedList();
        List<ResultRule> billMappingList = Lists.newLinkedList();
        List<ResultRule> conflictMappingList = Lists.newLinkedList();

        RuleVO ruleVO = new RuleVO();
        if (caseId != null) {
            if (caseId.equals(1L)) {
                ruleVO.setRuleType(1);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.Medicine.getKey(),
                        LexiconEnum.Operation.getKey(),
                        LexiconEnum.LisName.getKey(),
                        LexiconEnum.PacsName.getKey(),
                        LexiconEnum.PacsSubName.getKey() }));
            } else if (caseId.equals(2L)) {
                ruleVO.setRuleType(1);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.Transfusion.getKey() }));
            } else if (caseId.equals(3L)) {
                ruleVO.setRuleType(3);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.LisSubName.getKey()
                }));
            } else if (caseId.equals(4L)) {
                ruleVO.setRuleType(3);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.PacsResult.getKey(),
                        LexiconEnum.PacsSubName.getKey() }));
            } else if (caseId.equals(5L)) {
                ruleVO.setRuleType(2);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.Medicine.getKey()
                }));
            } else if (caseId.equals(6L)) {
                ruleVO.setRuleType(2);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.Operation.getKey() }));
            } else if (caseId.equals(7L)) {
                ruleVO.setRuleType(4);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.LisSubName.getKey()
                }));
            } else if (caseId.equals(8L)) {
                ruleVO.setRuleType(4);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.PacsResult.getKey() }));
            } else if (caseId.equals(9L)) {
                ruleVO.setRuleType(5);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.LisSubName.getKey() }));
            } else if (caseId.equals(15L)) {
                ruleVO.setRuleType(6);
                ruleVO.setLibTypes(Arrays.asList(new Integer[] { LexiconEnum.PacsName.getKey(),
                        LexiconEnum.PacsSubName.getKey()
                }));
            }
        }
        List<RuleDTO> list = klRuleFacade.getRules(ruleVO);

        if (ListUtil.isEmpty(list)) {
            return retList;
        }

        Map<String, Map<String, Map<String, List<Long>>>> lisConfigMap = lisConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> pacsConfigMap = pacsConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, Map<String, List<Long>>>> drugConfigMap = drugConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> diseaseConfigMap = diseaseConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> operationConfigMap = operationConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> transfusionConfigMap = transfusionConfigFacade.getUniqueConfigMap(hospitalId, null, null);

        //高危手术结果去重
        if (caseId.equals(6L)) {
            list = highrisk_operation_distinct(list);
        }

        Map<String, List<RuleDTO>> map = list
                .stream()
                .collect(Collectors.groupingBy(k -> k.getRuleId() + "_" + k.getGroupType()));
        Long ruleOrderNo = 1L;
        for (Map.Entry<String, List<RuleDTO>> entry : map.entrySet()) {
            //多个条件的痛ruleOrderNo
            for (RuleDTO item : entry.getValue()) {
                item.setRuleOrderNo(ruleOrderNo);
                groupList.add(item);
            }
            ruleOrderNo++;
        }
        groupList = groupList.stream()
                .sorted(Comparator.comparing(RuleDTO::getRuleOrderNo).thenComparing(RuleDTO::getGroupType))
                .collect(Collectors.toList());

        Map<Long, List<RuleDTO>> groupMap = groupList
                .stream()
                .collect(Collectors.groupingBy(RuleDTO::getRuleOrderNo));
        ruleOrderNo = 0L;
        for (Map.Entry<Long, List<RuleDTO>> entry : groupMap.entrySet()) {
            Map<Integer, List<ResultRule>> tempMap = new HashMap<>();
            Integer num = 1;
            for (RuleDTO record : entry.getValue()) {
                List<ResultRule> tempList = Lists.newLinkedList();

                ResultRule result = new ResultRule();
                result.setRuleOrderNo(ruleOrderNo);
                result.setRuleType(ruleVO.getRuleType());

                result.setRuleId(record.getRuleId());
                result.setRuleLibType(record.getRuleLibType());
                result.setRuleLibTypeName(record.getRuleLibTypeName());
                result.setRuleLibName(record.getRuleLibName());
                result.setRuleHisName("");
                result.setRuleHisDetailName("");

                result.setHasSubCond(record.getHasSubCond());

                if (result.getHasSubCond() != null && result.getHasSubCond().equals(1)) {
                    result.setRuleBaseId(record.getRuleBaseId());
                    result.setRuleBaseType(record.getRuleBaseType());
                    result.setRuleBaseLibType(record.getRuleBaseLibType());
                    result.setRuleBaseLibTypeName(record.getRuleBaseLibTypeName());
                    result.setRuleBaseLibName(record.getRuleBaseLibName());
                    result.setRuleBaseHisName("");
                    result.setRuleBaseHisDetailName("");
                    result.setGroupType(record.getGroupType());

                    //年龄转换数值
                    switch (result.getRuleBaseLibName()) {
                        case "新生儿":
                            result.setRuleBaseInputValue(random(BigDecimal.ZERO, BigDecimal.ONE, false, true).toString());
                            break;
                        case "幼儿":
                            result.setRuleBaseInputValue(random_int(1, 6, true, false).toString());
                            break;
                        case "儿童":
                            result.setRuleBaseInputValue(random_int(1, 6, true, false).toString());
                            break;
                        case "成人":
                            result.setRuleBaseInputValue(random_int(18, 200, true, false).toString());
                            break;
                        case "月经期":
                            result.setRuleBaseInputValue(random_int(0, 99, true, true).toString());
                            break;
                        default:
                            break;
                    }
                }

                result.setMessage("");
                result.setReferenceValue("");
                result.setRuleMsg(record.getRuleMsg());

                //其他值提醒辅检
                if (result.getRuleType().equals(4)) {
                    result.setExpectedOutput(record.getRuleMsg());
                }

                //高危药品
                if (result.getRuleType().equals(2)
                        && result.getRuleLibType().equals(LexiconEnum.Medicine.getKey())) {
                    result.setReferenceValue(record.getRuleMsg());
                }

                //输血其他值提醒
                if (result.getRuleType().equals(5)) {
                    result.setReferenceValue(record.getRuleMsg());
                }
                //比较关系
                if (result.getHasSubCond() != null && result.getHasSubCond().equals(1)) {
                    if (StringUtil.isNotBlank(record.getRuleBaseMinOperator())
                            && StringUtil.isNotBlank(record.getRuleBaseMaxOperator())) {
                        //上下限比较
                        if (record.getRuleBaseMaxValue() >= record.getRuleBaseMinValue()) {
                            ResultRule valResult = new ResultRule();
                            BeanUtil.copyProperties(result, valResult);
                            valResult.setRuleBaseMinValue(new BigDecimal(record.getRuleBaseMinValue()));
                            valResult.setRuleBaseMaxValue(new BigDecimal(record.getRuleBaseMaxValue()));
                            if (valResult.getRuleBaseLibName().equals("年龄")
                                    && valResult.getRuleBaseMinValue().doubleValue() > 1D) {
                                valResult.setRuleBaseInputValue(random_int(record.getRuleBaseMinValue().intValue(),
                                        record.getRuleBaseMaxValue().intValue(),
                                        (record.getRuleBaseMaxOperator().equals("<=") ? true : false),
                                        (record.getRuleBaseMinOperator().equals(">=") ? true : false)).toString());
                            } else {
                                valResult.setRuleBaseInputValue(random(new BigDecimal(record.getRuleBaseMinValue()),
                                        new BigDecimal(record.getRuleBaseMaxValue()),
                                        (record.getRuleBaseMaxOperator().equals("<=") ? true : false),
                                        (record.getRuleBaseMinOperator().equals(">=") ? true : false)).toString());
                            }
                            valResult.setRuleBaseUnit(StringUtil.isNotBlank(record.getRuleBaseMinUnit()) ? record.getRuleBaseMinUnit()
                                    : (StringUtil.isNotBlank(record.getRuleBaseMaxUnit()) ? record.getRuleBaseMaxUnit() : ""));
                            valResult.setReferenceValue(record.getRuleBaseMinValue()
                                    + record.getRuleBaseMinUnit()
                                    + record.getRuleBaseMinOperator()
                                    + record.getRuleBaseMaxOperator()
                                    + record.getRuleBaseMaxValue()
                                    + record.getRuleBaseMaxUnit());
                            tempList.add(valResult);
                        } else {
                            //大于最小值inputValue>minValue
                            ResultRule gtResult = new ResultRule();
                            BeanUtil.copyProperties(result, gtResult);
                            gtResult.setRuleBaseMinValue(new BigDecimal(record.getRuleBaseMinValue()));
                            if (gtResult.getRuleBaseLibName().equals("年龄")
                                    && gtResult.getRuleBaseMinValue().doubleValue() > 1D) {
                                gtResult.setRuleBaseInputValue(random_int(record.getRuleBaseMinValue().intValue(),
                                        null,
                                        (record.getRuleBaseMinOperator().equals(">=") ? true : false),
                                        false).toString());
                            } else {
                                gtResult.setRuleBaseInputValue(random(new BigDecimal(record.getRuleBaseMinValue()),
                                        null,
                                        (record.getRuleBaseMinOperator().equals(">=") ? true : false),
                                        false).toString());
                            }
                            gtResult.setRuleBaseUnit(StringUtil.isNotBlank(record.getRuleBaseMinUnit()) ? record.getRuleBaseMinUnit() : "");
                            gtResult.setReferenceValue(record.getRuleBaseMinOperator()
                                    + record.getRuleBaseMinValue()
                                    + record.getRuleBaseMinUnit());
                            tempList.add(gtResult);

                            //小于最大值inputValue<maxValue
                            ResultRule ltResult = new ResultRule();
                            BeanUtil.copyProperties(result, ltResult);
                            //inputValue>minValue
                            ltResult.setRuleBaseMaxValue(new BigDecimal(record.getRuleBaseMaxValue()));
                            if (ltResult.getRuleBaseLibName().equals("年龄")
                                    && ltResult.getRuleBaseMaxValue().doubleValue() > 1D) {
                                ltResult.setRuleBaseInputValue(random_int(null,
                                        record.getRuleBaseMaxValue().intValue(),
                                        false,
                                        (record.getRuleBaseMaxOperator().equals("<=") ? true : false)).toString());
                            } else {
                                ltResult.setRuleBaseInputValue(random(null,
                                        new BigDecimal(record.getRuleBaseMaxValue()),
                                        false,
                                        (record.getRuleBaseMaxOperator().equals("<=") ? true : false)).toString());
                            }
                            ltResult.setRuleBaseUnit(StringUtil.isNotBlank(record.getRuleBaseMaxUnit()) ? record.getRuleBaseMaxUnit() : "");
                            ltResult.setReferenceValue(record.getRuleBaseMaxOperator()
                                    + record.getRuleBaseMaxValue()
                                    + record.getRuleBaseMaxUnit());
                            tempList.add(ltResult);
                        }
                    } else if (StringUtil.isNotBlank(record.getRuleBaseMinOperator())
                            && StringUtil.isBlank(record.getRuleBaseMaxOperator())) {
                        ResultRule gtResult = new ResultRule();
                        BeanUtil.copyProperties(result, gtResult);
                        gtResult.setRuleBaseMinValue(new BigDecimal(record.getRuleBaseMinValue()));
                        if (gtResult.getRuleBaseLibName().equals("年龄")
                                && gtResult.getRuleBaseMinValue().doubleValue() > 1D) {
                            gtResult.setRuleBaseInputValue(random_int(record.getRuleBaseMinValue().intValue(),
                                    null,
                                    (record.getRuleBaseMinOperator().equals(">=") ? true : false),
                                    false).toString());
                        } else {
                            gtResult.setRuleBaseInputValue(random(new BigDecimal(record.getRuleBaseMinValue()),
                                    null,
                                    (record.getRuleBaseMinOperator().equals(">=") ? true : false),
                                    false).toString());
                        }
                        gtResult.setRuleBaseUnit(StringUtil.isNotBlank(record.getRuleBaseMinUnit()) ? record.getRuleBaseMinUnit() : "");
                        gtResult.setReferenceValue(record.getRuleBaseMinOperator()
                                + record.getRuleBaseMinValue()
                                + record.getRuleBaseMinUnit());
                        tempList.add(gtResult);
                    } else if (StringUtil.isBlank(record.getRuleBaseMinOperator())
                            && StringUtil.isNotBlank(record.getRuleBaseMaxOperator())) {
                        ResultRule ltResult = new ResultRule();
                        BeanUtil.copyProperties(result, ltResult);
                        //inputValue>minValue
                        ltResult.setRuleBaseMaxValue(new BigDecimal(record.getRuleBaseMaxValue()));
                        if (ltResult.getRuleBaseLibName().equals("年龄")
                                && ltResult.getRuleBaseMaxValue().doubleValue() > 1D) {
                            ltResult.setRuleBaseInputValue(random_int(null,
                                    record.getRuleBaseMaxValue().intValue(),
                                    false,
                                    (record.getRuleBaseMaxOperator().equals("<=") ? true : false)).toString());
                        } else {
                            ltResult.setRuleBaseInputValue(random(null,
                                    new BigDecimal(record.getRuleBaseMaxValue()),
                                    false,
                                    (record.getRuleBaseMaxOperator().equals("<=") ? true : false)).toString());
                        }
                        ltResult.setRuleBaseUnit(StringUtil.isNotBlank(record.getRuleBaseMaxUnit()) ? record.getRuleBaseMaxUnit() : "");
                        ltResult.setReferenceValue(record.getRuleBaseMaxOperator()
                                + record.getRuleBaseMaxValue()
                                + record.getRuleBaseMaxUnit());
                        tempList.add(ltResult);
                    } else if (StringUtil.isNotBlank(record.getRuleBaseEqOperator())) {
                        result.setReferenceValue(record.getRuleBaseEqOperator()
                                + record.getRuleBaseEqValue()
                                + record.getRuleBaseEqUnit());
                        result.setRuleBaseInputValue(record.getRuleBaseEqValue());
                        result.setRuleBaseUnit(StringUtil.isNotBlank(record.getRuleBaseEqUnit()) ? record.getRuleBaseEqUnit() : "");
                        tempList.add(result);
                    } else if (StringUtil.isBlank(record.getRuleBaseEqOperator())
                            && StringUtil.isNotBlank(record.getRuleBaseEqValue())) {
                        //文本内容
                        result.setReferenceValue(record.getRuleBaseEqValue());
                        result.setRuleBaseInputValue(record.getRuleBaseEqValue());
                        tempList.add(result);
                    } else {
                        tempList.add(result);
                    }
                } else {
                    tempList.add(result);
                }
                tempMap.put(num++, tempList);
            }
            inputList.addAll(addRuleOrderNo(tempMap, ruleOrderNo));
            ruleOrderNo = (long) inputList.size();
        }

        //药品分类扩展
        List<MedClassMedDTO> medClassMedList = klRelationFacade.getMedClassMedList();
        Map<String, List<String>> medClassMedMap =
                medClassMedList.stream().collect(Collectors.groupingBy(MedClassMedDTO::getMedClassLibName,
                        HashMap::new,
                        Collectors.mapping(MedClassMedDTO::getMedLibName, Collectors.toList())));
        Map<Long, List<ResultRule>> medMap = inputList
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        ruleOrderNo = 0L;
        for (Map.Entry<Long, List<ResultRule>> entry : medMap.entrySet()) {
            Map<Integer, List<ResultRule>> tempMap = new HashMap<>();
            Integer num = 1;
            for (ResultRule result : entry.getValue()) {
                List<ResultRule> tempList = Lists.newLinkedList();
                if (result.getHasSubCond() != null && result.getHasSubCond().equals(1)) {
                    if (result.getRuleBaseLibType().equals(LexiconEnum.MedChemClass.getKey())
                            || result.getRuleBaseLibType().equals(LexiconEnum.MedZhiLiaoClass.getKey())
                            || result.getRuleBaseLibType().equals(LexiconEnum.MedYaoLiClass.getKey())) {
                        List<String> medicines = medClassMedMap.get(result.getRuleBaseLibName());
                        if (ListUtil.isNotEmpty(medicines)) {
                            for (String med : medicines) {
                                ResultRule medResult = new ResultRule();
                                BeanUtil.copyProperties(result, medResult);
                                medResult.setReferenceValue("药品类别_" + result.getRuleBaseLibType() + "_" + result.getRuleBaseLibTypeName() + "_" + result.getRuleBaseLibName());
                                medResult.setRuleBaseLibName(med);
                                medResult.setRuleBaseLibType(LexiconEnum.Medicine.getKey());
                                medResult.setRuleBaseLibTypeName(LexiconEnum.Medicine.getName());
                                tempList.add(medResult);
                            }
                        } else {
                            result.setSuccess(0);
                            result.setMessage(result.getMessage() + result.getRuleBaseLibName() + "(" + result.getRuleBaseLibTypeName() + ")缺少相关药品通用名称；");
                            tempList.add(result);
                        }
                    } else {
                        tempList.add(result);
                    }
                } else {
                    tempList.add(result);
                }
                tempMap.put(num++, tempList);
            }
            medicineList.addAll(addRuleOrderNo(tempMap, ruleOrderNo));
            ruleOrderNo = (long) medicineList.size();
        }

        //禁忌条件“流产”转成包含流产的诊断
        ruleOrderNo = 0L;
        Map<Long, List<ResultRule>> containsDiseaseMap = medicineList
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        for (Map.Entry<Long, List<ResultRule>> entry : containsDiseaseMap.entrySet()) {
            Map<Integer, List<ResultRule>> tempMap = new HashMap<>();
            Integer num = 1;
            for (ResultRule result : entry.getValue()) {
                List<ResultRule> tempList = Lists.newLinkedList();
                if (result.getRuleType().equals(4)) {
                    if (StringUtil.isNotBlank(result.getRuleBaseLibName())
                            && result.getRuleBaseLibName().equals("流产")) {
                        List<String> conflictDiseases = diseaseConfigMap.keySet().stream()
                                .filter(i -> i.contains(result.getRuleBaseLibName())).collect(Collectors.toList());
                        for (String conflictDisease : conflictDiseases) {
                            ResultRule convertResult = new ResultRule();
                            BeanUtil.copyProperties(result, convertResult);
                            convertResult.setReferenceValue(convertResult.getRuleBaseLibType() + "_" + convertResult.getRuleBaseLibName() + "_" + convertResult.getRuleBaseLibName());
                            convertResult.setRuleBaseLibName(conflictDisease);
                            convertResult.setRuleBaseLibType(LexiconEnum.Disease.getKey());
                            convertResult.setRuleBaseLibTypeName(LexiconEnum.Disease.getName());
                            tempList.add(convertResult);
                        }
                    } else {
                        tempList.add(result);
                    }
                } else {
                    tempList.add(result);
                }
                tempMap.put(num++, tempList);
            }
            containsDiseaseList.addAll(addRuleOrderNo(tempMap, ruleOrderNo));
            ruleOrderNo = (long) containsDiseaseList.size();
        }

        //正常项目重复开立处理
        ruleOrderNo = 0L;
        Map<Long, List<ResultRule>> rePacsBillMap = containsDiseaseList
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        for (Map.Entry<Long, List<ResultRule>> entry : rePacsBillMap.entrySet()) {
            Map<Integer, List<ResultRule>> tempMap = new HashMap<>();
            Integer num = 1;
            for (ResultRule result : entry.getValue()) {
                List<ResultRule> tempList = Lists.newLinkedList();
                if (result.getRuleType().equals(6)) {
                    if (StringUtil.isNotBlank(result.getRuleBaseInputValue())) {
                        List<String> pacsResultArr = Arrays.asList(result.getRuleBaseInputValue().split("\\|"));
                        for (String pacsResult : pacsResultArr) {
                            ResultRule rePacsResult = new ResultRule();
                            BeanUtil.copyProperties(result, rePacsResult);
                            rePacsResult.setRuleBaseInputValue(pacsResult);
                            tempList.add(rePacsResult);
                        }
                    } else {
                        tempList.add(result);
                    }
                } else {
                    tempList.add(result);
                }
                tempMap.put(num++, tempList);
            }
            rePacsBillList.addAll(addRuleOrderNo(tempMap, ruleOrderNo));
            ruleOrderNo = (long) rePacsBillList.size();
        }

        //术语映射-开单项映射
        ruleOrderNo = 0L;
        Map<Long, List<ResultRule>> billMappingMap = rePacsBillList
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        for (Map.Entry<Long, List<ResultRule>> entry : billMappingMap.entrySet()) {
            Map<Integer, List<ResultRule>> tempMap = new HashMap<>();
            Integer num = 1;
            for (ResultRule result : entry.getValue()) {
                List<ResultRule> tempList = Lists.newLinkedList();
                if (result.getRuleLibType().equals(LexiconEnum.Medicine.getKey())) {
                    List<ResultRule> mappingList = getMappingResult_medicine(result, drugConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.Operation.getKey())) {
                    List<ResultRule> mappingList = getMappingResult(result, operationConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.LisName.getKey())) {
                    List<ResultRule> mappingList = getMappingResult_lis(result, lisConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                            item.setRuleBaseHisDetailName(item.getRuleHisDetailName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.LisSubName.getKey())) {
                    List<ResultRule> mappingList = getMappingResult_lis(result, lisConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                            item.setRuleBaseHisDetailName(item.getRuleHisDetailName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.PacsName.getKey())) {
                    List<ResultRule> mappingList = getMappingResult(result, pacsConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.PacsSubName.getKey())) {
                    List<ResultRule> mappingList = getMappingResult(result, pacsConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.Transfusion.getKey())) {
                    List<ResultRule> mappingList = getMappingResult(result, transfusionConfigMap, 1);
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        mappingList.forEach(item -> {
                            item.setRuleBaseHisName(item.getRuleHisName());
                        });
                    }
                    tempList.addAll(mappingList);
                } else if (result.getRuleLibType().equals(LexiconEnum.PacsResult.getKey())) {
                    tempList.add(result);
                } else {
                    tempList.add(result);
                }
                tempMap.put(num++, tempList);
            }
            billMappingList.addAll(addRuleOrderNo(tempMap, ruleOrderNo));
            ruleOrderNo = (long) billMappingList.size();
        }

        //术语映射-禁忌项映射
        ruleOrderNo = 0L;
        Map<Long, List<ResultRule>> conflictMappingMap = billMappingList
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        for (Map.Entry<Long, List<ResultRule>> entry : conflictMappingMap.entrySet()) {
            Map<Integer, List<ResultRule>> tempMap = new HashMap<>();
            Integer num = 1;
            for (ResultRule result : entry.getValue()) {
                List<ResultRule> tempList = Lists.newLinkedList();
                if (result.getHasSubCond() != null && result.getHasSubCond().equals(1)) {
                    if (result.getRuleLibType().equals(result.getRuleBaseLibType())
                            && result.getRuleLibName().equals(result.getRuleBaseLibName())) {
                        tempList.add(result);
                    } else {
                        if (result.getRuleBaseLibType().equals(LexiconEnum.Disease.getKey())) {
                            List<ResultRule> mappingList = getMappingResult(result, diseaseConfigMap, 2);
                            tempList.addAll(mappingList);
                        } else if (result.getRuleBaseLibType().equals(LexiconEnum.Medicine.getKey())) {
                            List<ResultRule> mappingList = getMappingResult_medicine(result, drugConfigMap, 2);
                            tempList.addAll(mappingList);
                        } else if (result.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey())) {
                            List<ResultRule> mappingList = getMappingResult_lis(result, lisConfigMap, 2);
                            tempList.addAll(mappingList);
                        } else if (result.getRuleBaseLibType().equals(LexiconEnum.PacsName.getKey())) {
                            List<ResultRule> mappingList = getMappingResult(result, pacsConfigMap, 2);
                            tempList.addAll(mappingList);
                        } else if (result.getRuleBaseLibType().equals(LexiconEnum.PacsSubName.getKey())) {
                            List<ResultRule> mappingList = getMappingResult(result, pacsConfigMap, 2);
                            tempList.addAll(mappingList);
                        } else if (result.getRuleBaseLibType().equals(LexiconEnum.Operation.getKey())) {
                            List<ResultRule> mappingList = getMappingResult(result, operationConfigMap, 2);
                            tempList.addAll(mappingList);
                        } else {
                            tempList.add(result);
                        }
                    }
                } else {
                    tempList.add(result);
                }
                tempMap.put(num++, tempList);
            }
            conflictMappingList.addAll(addRuleOrderNo(tempMap, ruleOrderNo));
            ruleOrderNo = (long) conflictMappingList.size();
        }

        //生成预期结果
        Map<Long, List<ResultRule>> ruleMap = conflictMappingList
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        for (Map.Entry<Long, List<ResultRule>> entry : ruleMap.entrySet()) {
            Boolean isSuccess = true;
            for (ResultRule i : entry.getValue()) {
                if (i.getSuccess() != null && i.getSuccess().equals(0)) {
                    isSuccess = false;
                    break;
                }
            }
            if (!isSuccess) {
                entry.getValue().forEach(i -> {
                    i.setSuccess(0);
                    i.setMessage(entry.getValue().stream()
                            .filter(k -> StringUtil.isNotBlank(k.getMessage()))
                            .map(k -> k.getMessage())
                            .collect(Collectors.joining("；")));
                });
                retList.addAll(entry.getValue());
                continue;
            }
            BillMsg billMsg = getBillMsg(entry.getValue());
            if (billMsg != null) {
                entry.getValue().forEach(item -> {
                    if (StringUtil.isBlank(item.getExpectedOutput())) {
                        item.setExpectedOutput(billMsg.getMsg());
                    }
                });
            } else {
                entry.getValue().forEach(item -> {
                    item.setSuccess(0);
                    item.setMessage(item.getMessage() + "未生成预期结果；");
                });
            }
            retList.addAll(entry.getValue());
        }
        return retList;
    }

    public List<ResultRule> addRuleOrderNo(Map<Integer, List<ResultRule>> map, Long ruleOrderNo) {
        List<ResultRule> retList = Lists.newLinkedList();
        ruleOrderNo++;
        if (map == null || map.size() == 0) {
            return retList;
        } else if (map.size() == 1) {
            List<ResultRule> list1 = map.get(1);
            for (ResultRule item : list1) {
                item.setRuleOrderNo(ruleOrderNo++);
                retList.add(item);
            }
        } else if (map.size() == 2) {
            List<ResultRule> list1 = map.get(1);
            List<ResultRule> list2 = map.get(2);
            for (ResultRule item1 : list1) {
                for (ResultRule item2 : list2) {
                    ResultRule addItem1 = new ResultRule();
                    BeanUtil.copyProperties(item1, addItem1);
                    ResultRule addItem2 = new ResultRule();
                    BeanUtil.copyProperties(item2, addItem2);
                    addItem1.setRuleOrderNo(ruleOrderNo);
                    addItem2.setRuleOrderNo(ruleOrderNo);
                    retList.add(addItem1);
                    retList.add(addItem2);
                    ruleOrderNo++;
                }
            }
        }
        return retList;
    }


    /**
     * 高危手术条件去重
     *
     * @param list
     * @return
     */
    public List<RuleDTO> highrisk_operation_distinct(List<RuleDTO> list) {
        List<RuleDTO> retList = Lists.newLinkedList();

        if (ListUtil.isEmpty(list)) {
            return list;
        }

        //没有子条件的规则
        retList.addAll(list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(0))
                .collect(Collectors.toList()));

        //带子条件的规则
        Map<Long, List<RuleDTO>> ruleIdMap = list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1))
                .collect(Collectors.groupingBy(RuleDTO::getRuleId));
        //Map<ruleId,Map<groupType_ruleBaseId,RuleDTO>>
        Map<Long, Map<String, RuleDTO>> nestMap = new HashMap<>();
        //Map<ruleId,groupType_ruleBaseId|groupType_ruleBaseId|groupType_ruleBaseId……>
        Map<Long, String> conditionMap = new HashMap<>();
        for (Map.Entry<Long, List<RuleDTO>> entry : ruleIdMap.entrySet()) {
            nestMap.put(entry.getKey(), entry.getValue().stream()
                    .collect(Collectors.toMap(k -> k.getGroupType() + "_" + k.getRuleBaseId(), v -> v)));
            conditionMap.put(entry.getKey(),
                    entry.getValue().stream()
                            .sorted(Comparator.comparing(RuleDTO::getRuleBaseId).thenComparing(RuleDTO::getGroupType))
                            .map(i -> i.getGroupType() + "_" + i.getRuleBaseId())
                            .collect(Collectors.joining("|")));
        }
        Map<String, List<Long>> reversedMap = new HashMap<>();
        for (Map.Entry<Long, String> entry : conditionMap.entrySet()) {
            List<Long> ruleIds = Lists.newLinkedList();
            if (reversedMap.containsKey(entry.getValue())) {
                ruleIds = reversedMap.get(entry.getValue());
            }
            ruleIds.add(entry.getKey());
            reversedMap.put(entry.getValue(), ruleIds);
        }

        for (Map.Entry<String, List<Long>> entry : reversedMap.entrySet()) {
            if (ListUtil.isEmpty(entry.getValue())) {
                continue;
            }
            Integer num = 0;
            List<String> baseIdList = Arrays.asList(entry.getKey().split("|"));
            for (Long ruleId : entry.getValue()) {
                retList.add(nestMap.get(ruleId).get(baseIdList.get(num++)));
                if (num == baseIdList.size()) {
                    num = 0;
                }
            }
        }

        return retList;
    }

    /**
     * @param result
     * @param configMap
     * @param type
     * @return
     */
    public List<ResultRule> getMappingResult(ResultRule result, Map<String, Map<String, List<Long>>> configMap, Integer type) {
        List<ResultRule> retList = Lists.newLinkedList();
        if (configMap == null || configMap.size() == 0) {
            result.setSuccess(0);
            if (type.equals(1)) {
                result.setMessage(result.getMessage() + "请维护术语(" + result.getRuleLibTypeName() + ")映射；");
            } else if (type.equals(2)) {
                result.setMessage(result.getMessage() + "请维护术语(" + result.getRuleBaseLibTypeName() + ")映射；");
            }
            retList.add(result);
            return retList;
        }
        String keyName = "";
        if (type.equals(1)) {
            keyName = result.getRuleLibName();
        } else if (type.equals(2)) {
            keyName = result.getRuleBaseLibName();
        }

        List<String> hisNames = configMap.containsKey(keyName)
                ? new ArrayList<>(configMap.get(keyName).keySet())
                : new ArrayList<>();
        if (ListUtil.isNotEmpty(hisNames)) {
            for (String hisName : hisNames) {
                ResultRule bmResult = new ResultRule();
                BeanUtil.copyProperties(result, bmResult);
                if (type.equals(1)) {
                    bmResult.setRuleHisName(hisName);
                } else if (type.equals(2)) {
                    bmResult.setRuleBaseHisName(hisName);
                }
                retList.add(bmResult);
            }
        } else {
            result.setSuccess(0);
            if (type.equals(1)) {
                result.setMessage(result.getMessage() + result.getRuleLibName() + "(" + result.getRuleLibTypeName() + ")缺少医院端映射；");
            } else if (type.equals(2)) {
                result.setMessage(result.getMessage() + result.getRuleBaseLibName() + "(" + result.getRuleBaseLibTypeName() + ")缺少医院端映射；");
            }
            retList.add(result);
        }
        return retList;
    }

    /**
     * @param result
     * @param configMap
     * @param type
     * @return
     */
    public List<ResultRule> getMappingResult_medicine(ResultRule result, Map<String, Map<String, Map<String, List<Long>>>> configMap, Integer type) {
        List<ResultRule> retList = Lists.newLinkedList();
        if (configMap == null || configMap.size() == 0) {
            result.setSuccess(0);
            if (type.equals(1)) {
                result.setMessage(result.getMessage() + "请维护术语(" + result.getRuleLibTypeName() + ")映射；");
            } else if (type.equals(2)) {
                result.setMessage(result.getMessage() + "请维护术语(" + result.getRuleBaseLibTypeName() + ")映射；");
            }
            retList.add(result);
            return retList;
        }
        String keyName = "";
        if (type.equals(1)) {
            keyName = result.getRuleLibName();
        } else if (type.equals(2)) {
            keyName = result.getRuleBaseLibName();
        }

        List<String> hisNames = configMap.containsKey(keyName)
                //? configMap.get(keyName).values().stream().map(i -> i.keySet()).flatMap(Collection::stream).distinct().collect(Collectors.toList())
                ? new ArrayList<>(configMap.get(keyName).keySet())
                : new ArrayList<>();
        if (ListUtil.isNotEmpty(hisNames)) {
            for (String hisName : hisNames) {
                ResultRule bmResult = new ResultRule();
                BeanUtil.copyProperties(result, bmResult);
                if (type.equals(1)) {
                    bmResult.setRuleHisName(hisName);
                } else if (type.equals(2)) {
                    bmResult.setRuleBaseHisName(hisName);
                }
                retList.add(bmResult);
            }
        } else {
            result.setSuccess(0);
            if (type.equals(1)) {
                result.setMessage(result.getMessage() + result.getRuleLibName() + "(" + result.getRuleLibTypeName() + ")缺少医院端映射；");
            } else if (type.equals(2)) {
                result.setMessage(result.getMessage() + result.getRuleBaseLibName() + "(" + result.getRuleBaseLibTypeName() + ")缺少医院端映射；");
            }
            retList.add(result);
        }
        return retList;
    }

    /**
     * @param result
     * @param configMap
     * @param type
     * @return
     */
    public List<ResultRule> getMappingResult_lis(ResultRule result, Map<String, Map<String, Map<String, List<Long>>>> configMap, Integer type) {
        List<ResultRule> retList = Lists.newLinkedList();
        if (configMap == null || configMap.size() == 0) {
            result.setSuccess(0);
            if (type.equals(1)) {
                result.setMessage(result.getMessage() + "请维护术语(" + result.getRuleLibTypeName() + ")映射；");
            } else if (type.equals(2)) {
                result.setMessage(result.getMessage() + "请维护术语(" + result.getRuleBaseLibTypeName() + ")映射；");
            }
            retList.add(result);
            return retList;
        }
        String keyName = "";
        if (type.equals(1)) {
            keyName = result.getRuleLibName();
        } else if (type.equals(2)) {
            keyName = result.getRuleBaseLibName();
        }

        Map<String, Map<String, List<Long>>> subConfigMap = configMap.get(keyName);
        if (subConfigMap == null || subConfigMap.size() == 0) {
            result.setSuccess(0);
            if (type.equals(1)) {
                result.setMessage(result.getMessage() + result.getRuleLibName() + "(" + result.getRuleLibTypeName() + ")缺少医院端映射；");
            } else if (type.equals(2)) {
                result.setMessage(result.getMessage() + result.getRuleBaseLibName() + "(" + result.getRuleBaseLibTypeName() + ")缺少医院端映射；");
            }
            retList.add(result);
            return retList;
        }

        for (Map.Entry<String, Map<String, List<Long>>> subEntry : subConfigMap.entrySet()) {
            subEntry.getValue().keySet().forEach(detailName -> {
                ResultRule bmResult = new ResultRule();
                BeanUtil.copyProperties(result, bmResult);
                if (type.equals(1)) {
                    bmResult.setRuleHisName(subEntry.getKey());
                    bmResult.setRuleHisDetailName(detailName);
                } else if (type.equals(2)) {
                    bmResult.setRuleBaseHisName(subEntry.getKey());
                    bmResult.setRuleBaseHisDetailName(detailName);
                }
                retList.add(bmResult);
            });
        }

        return retList;
    }

    /**
     * 模拟提示信息
     *
     * @param list
     * @return
     */
    public BillMsg getBillMsg(List<ResultRule> list) {
        BillMsg billMsg = new BillMsg();
        if (list.size() == 1) {
            ResultRule resultRule = list.get(0);
            String orderType = getOrderType(resultRule.getRuleLibTypeName());
            String conType = "";
            if (StringUtil.isNotBlank(resultRule.getRuleBaseLibTypeName())) {
                conType = getConType(resultRule.getRuleBaseLibTypeName(), resultRule.getRuleType(), resultRule.getRuleBaseType());
            }
            //开单合理性
            if (resultRule.getRuleType().equals(1)) {
                if (resultRule.getHasSubCond() != null && resultRule.getHasSubCond().equals(1)) {
                    if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Disease.getKey())) {
                        //禁忌疾病
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseHisName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Medicine.getKey())) {
                        //禁忌服用药品
                        if (resultRule.getRuleBaseType().equals(RuleBaseTypeEnum.Eq.getKey())) {
                            billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                    resultRule.getRuleLibName(),
                                    resultRule.getRuleBaseHisName(),
                                    conType,
                                    orderType);
                        } else if (resultRule.getRuleBaseType().equals(RuleBaseTypeEnum.Allergen.getKey())) {
                            billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                    resultRule.getRuleLibName(),
                                    resultRule.getRuleBaseHisName(),
                                    conType,
                                    orderType);
                        }
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Symptom.getKey())) {
                        //禁忌症状
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Vital.getKey())) {
                        String content = resultRule.getRuleBaseLibName();
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                content,
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.VitalResult.getKey())) {
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Operation.getKey())) {
                        //禁忌手术
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseHisName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey())) {
                        //禁忌实验室检查
                        String content = resultRule.getRuleBaseHisName();
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseHisDetailName())
                                && !resultRule.getRuleBaseHisName().equals(resultRule.getRuleBaseHisDetailName())) {
                            content += resultRule.getRuleBaseHisDetailName();
                        }
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                content,
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.PacsName.getKey())) {
                        //互斥项
                        if (resultRule.getRuleBaseType().equals(RuleBaseTypeEnum.BillItem.getKey())) {
                            billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                    resultRule.getRuleLibName(),
                                    resultRule.getRuleBaseHisName(),
                                    conType,
                                    orderType);
                        }
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.PacsResult.getKey())) {
                        //禁忌辅助检查结果
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Gender.getKey())) {
                        //禁忌性别
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Group.getKey())) {
                        //禁忌禁忌人群
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Allergen.getKey())) {
                        //其他过敏原
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Device.getKey())) {
                        //禁忌医疗器械及物品提示信息
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                resultRule.getRuleBaseLibName(),
                                conType,
                                orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Age.getKey())) {
                        //年龄
                        String content = resultRule.getRuleBaseLibName();
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                content,
                                conType,
                                orderType);
                    } else {
                        //通用
                        String content = resultRule.getRuleBaseLibName();
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(),
                                resultRule.getRuleLibName(),
                                content,
                                conType,
                                orderType);
                    }
                }
            } else if (resultRule.getRuleType().equals(2)) {
                if (resultRule.getHasSubCond() != null && resultRule.getHasSubCond().equals(1)) {
                    if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Disease.getKey())) {
                        billMsg = MsgUtil.getCommonHighRiskMsg(resultRule.getRuleBaseHisName(), resultRule.getRuleHisName(), orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Form.getKey())) {
                        billMsg = MsgUtil.getCommonHighRiskMsg(resultRule.getReferenceValue(), resultRule.getRuleHisName(), orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Vital.getKey())) {
                        String content = "";
                        if (resultRule.getRuleBaseLibName().equals("收缩压")) {
                            content = "血压" + resultRule.getRuleBaseInputValue() + "/80";
                        } else if (resultRule.getRuleBaseLibName().equals("舒张压")) {
                            content = "血压120/" + resultRule.getRuleBaseInputValue();
                        } else {
                            content = resultRule.getRuleBaseLibName();
                            if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                                content += trimZero(resultRule.getRuleBaseInputValue());
                            }
                        }
                        billMsg = MsgUtil.getCommonHighRiskMsg(content, resultRule.getRuleHisName(), orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.VitalResult.getKey())) {
                        billMsg = MsgUtil.getCommonHighRiskMsg(resultRule.getRuleBaseLibName(), resultRule.getRuleHisName(), orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey())) {
                        String content = "";
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseHisName())) {
                            content += resultRule.getRuleBaseHisName();
                        }
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseHisDetailName())
                                && !resultRule.getRuleBaseHisDetailName().equals(resultRule.getRuleBaseHisName())) {
                            content += resultRule.getRuleBaseHisDetailName();
                        }
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonHighRiskMsg(content, resultRule.getRuleHisName(), orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.PacsResult.getKey())) {
                        billMsg = MsgUtil.getCommonHighRiskMsg(resultRule.getRuleBaseLibName(), resultRule.getRuleHisName(), orderType);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Age.getKey())) {
                        String content = resultRule.getRuleBaseLibName() + ":";
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonHighRiskMsg(content, resultRule.getRuleHisName(), orderType);
                    } else {
                        billMsg = MsgUtil.getCommonHighRiskMsg(resultRule.getRuleBaseLibName(), resultRule.getRuleHisName(), orderType);
                    }
                } else {
                    billMsg = MsgUtil.getCommonHighRiskMsg("", resultRule.getRuleHisName(), orderType);
                }
            } else if (resultRule.getRuleType().equals(3)) {
                if (resultRule.getRuleLibType().equals(LexiconEnum.LisSubName.getKey())) {
                    String content = "";
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseHisName())) {
                        content += resultRule.getRuleBaseHisName();
                    }
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseHisDetailName())
                            && !resultRule.getRuleBaseHisName().equals(resultRule.getRuleBaseHisDetailName())) {
                        content += resultRule.getRuleBaseHisDetailName();
                    }
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                        content += trimZero(resultRule.getRuleBaseInputValue());
                    }
                    billMsg = MsgUtil.getCommonCriticalMsg(content, orderType);
                } else if (resultRule.getRuleLibType().equals(LexiconEnum.PacsResult.getKey())) {
                    billMsg = MsgUtil.getCommonCriticalMsg(resultRule.getRuleLibName(), orderType);
                }
            } else if (resultRule.getRuleType().equals(4)) {
                if (resultRule.getRuleLibType().equals(LexiconEnum.LisSubName.getKey())) {
                    Lis lis = new Lis();
                    lis.setName(resultRule.getRuleHisName());
                    lis.setDetailName(resultRule.getRuleHisDetailName());
                    lis.setUniqueName(resultRule.getRuleLibName());

                    if (resultRule.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey())) {
                        String content = resultRule.getRuleHisName();
                        if (StringUtil.isNotBlank(resultRule.getRuleHisDetailName())
                                && !resultRule.getRuleHisDetailName().equals(resultRule.getRuleHisName())) {
                            content += resultRule.getRuleHisDetailName();
                        }
                        if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                            content += trimZero(resultRule.getRuleBaseInputValue());
                        }
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, content, resultRule.getRuleHisName(), conType, lis);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Group.getKey())) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, resultRule.getRuleBaseLibName(), resultRule.getRuleHisName(), ConEnum.group.getName(), lis);
                    }
                } else if (resultRule.getRuleLibType().equals(LexiconEnum.PacsResult.getKey())) {
                    if (resultRule.getRuleBaseLibType().equals(LexiconEnum.PacsResult.getKey())) {
                        if (resultRule.getRuleBaseType() != null && resultRule.getRuleBaseType().equals(RuleBaseTypeEnum.Eq.getKey())) {
                            billMsg = MsgUtil.getCommonOtherMsg(orderType, resultRule.getExpectedOutput(), resultRule.getRuleLibName(), conType, null);
                        } else if (resultRule.getRuleBaseType() != null && resultRule.getRuleBaseType().equals(RuleBaseTypeEnum.Ne.getKey())) {
                            billMsg = MsgUtil.getCommonOtherMsg(orderType, resultRule.getExpectedOutput(), resultRule.getRuleLibName(), conType, null);
                        }
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Gender.getKey())) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, resultRule.getExpectedOutput(), resultRule.getRuleLibName(), conType, null);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Group.getKey())) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, resultRule.getExpectedOutput(), resultRule.getRuleLibName(), conType, null);
                    } else if (resultRule.getRuleBaseLibType().equals(LexiconEnum.Age.getKey())) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, resultRule.getExpectedOutput(), resultRule.getRuleLibName(), conType, null);
                    }
                }
            } else if (resultRule.getRuleType().equals(5)) {
                if (resultRule.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey())) {
                    String content = "";
                    content += resultRule.getRuleHisName();
                    if (StringUtil.isNotBlank(resultRule.getRuleHisDetailName())
                            && !resultRule.getRuleHisDetailName().equals(resultRule.getRuleHisName())) {
                        content += resultRule.getRuleHisDetailName();
                    }
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                        content += trimZero(resultRule.getRuleBaseInputValue());
                    }
                    billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(), content, resultRule.getRuleMsg(), ConEnum.lis.getName(), null);
                }
            } else if (resultRule.getRuleType().equals(6)) {
                billMsg = MsgUtil.getCommonBillMsg(resultRule.getRuleHisName(), resultRule.getRuleLibName(), resultRule.getRuleBaseHisName(), conType, orderType);
            }
        } else if (list.size() > 1) {
            Integer ruleType = list.get(0).getRuleType();
            Integer ruleLibType = list.get(0).getRuleLibType();
            String orderType = getOrderType(list.get(0).getRuleLibTypeName());
            if (ruleType.equals(3)) {
                //检验危急值
                if (ruleLibType.equals(LexiconEnum.LisSubName.getKey())) {
                    ResultRule resultRule = list.stream()
                            .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey()))
                            .collect(Collectors.toList())
                            .get(0);
                    String content = "";
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseHisName())) {
                        content += resultRule.getRuleBaseHisName();
                    }
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseHisDetailName())
                            && !resultRule.getRuleBaseHisName().equals(resultRule.getRuleBaseHisDetailName())) {
                        content += resultRule.getRuleBaseHisDetailName();
                    }
                    if (StringUtil.isNotBlank(resultRule.getRuleBaseInputValue())) {
                        content += trimZero(resultRule.getRuleBaseInputValue());
                    }
                    billMsg = MsgUtil.getCommonCriticalMsg(content, orderType);
                }
            } else if (ruleType.equals(4)) {
                //检验其他值
                if (ruleLibType.equals(LexiconEnum.LisSubName.getKey())) {
                    Lis lis = new Lis();
                    lis.setName(list.get(0).getRuleHisName());
                    lis.setDetailName(list.get(0).getRuleHisDetailName());
                    lis.setUniqueName(list.get(0).getRuleLibName());

                    ResultRule lisResult = list.stream()
                            .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey()))
                            .collect(Collectors.toList())
                            .get(0);
                    List<ResultRule> diseaseResults = list.stream()
                            .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.Disease.getKey()))
                            .collect(Collectors.toList());
                    List<ResultRule> operationResults = list.stream()
                            .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.Operation.getKey()))
                            .collect(Collectors.toList());
                    List<ResultRule> medicineResults = list.stream()
                            .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.Medicine.getKey()))
                            .collect(Collectors.toList());
                    List<ResultRule> groupResults = list.stream()
                            .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.Group.getKey()))
                            .collect(Collectors.toList());

                    String content = lisResult.getRuleHisName();
                    if (StringUtil.isNotBlank(lisResult.getRuleHisDetailName())
                            && !lisResult.getRuleHisDetailName().equals(lisResult.getRuleHisName())) {
                        content += lisResult.getRuleHisDetailName();
                    }
                    if (StringUtil.isNotBlank(lisResult.getRuleBaseInputValue())) {
                        content += trimZero(lisResult.getRuleBaseInputValue());
                    }

                    if (ListUtil.isNotEmpty(diseaseResults)) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, content, diseaseResults.get(0).getRuleBaseHisName(), TypeEnum.disease.getName(), lis);
                    } else if (ListUtil.isNotEmpty(operationResults)) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, content, operationResults.get(0).getRuleBaseHisName(), TypeEnum.operation.getName(), lis);
                    } else if (ListUtil.isNotEmpty(medicineResults)) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, content, medicineResults.get(0).getRuleBaseHisName(), TypeEnum.drug.getName(), lis);
                    } else if (ListUtil.isNotEmpty(groupResults)) {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, content, groupResults.get(0).getRuleBaseLibName(), ConEnum.group.getName(), lis);
                    } else {
                        billMsg = MsgUtil.getCommonOtherMsg(orderType, content, "", ConEnum.lis.getName(), lis);
                    }
                } else if (ruleLibType.equals(LexiconEnum.PacsResult.getKey())) {
                    billMsg = MsgUtil.getCommonOtherMsg(orderType, list.get(0).getExpectedOutput(), "", "", null);
                }
            } else if (ruleType.equals(5)) {
                ResultRule lisResult = list.stream()
                        .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey()))
                        .collect(Collectors.toList())
                        .get(0);
                List<ResultRule> diseaseResults = list.stream()
                        .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.Disease.getKey()))
                        .collect(Collectors.toList());
                List<ResultRule> operationResults = list.stream()
                        .filter(i -> i.getRuleBaseLibType().equals(LexiconEnum.Operation.getKey()))
                        .collect(Collectors.toList());
                String content = lisResult.getRuleHisName();
                if (StringUtil.isNotBlank(lisResult.getRuleHisDetailName())
                        && !lisResult.getRuleHisDetailName().equals(lisResult.getRuleHisName())) {
                    content += lisResult.getRuleHisDetailName();
                }
                if (StringUtil.isNotBlank(lisResult.getRuleBaseInputValue())) {
                    content += trimZero(lisResult.getRuleBaseInputValue());
                }

                if (ListUtil.isNotEmpty(diseaseResults)) {
                    ResultRule diseaseResult = diseaseResults.get(0);
                    content = "诊断" + diseaseResult.getRuleBaseHisName() + "，" + content;
                    billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(), content, lisResult.getRuleMsg(), ConEnum.disease.getName(), null);
                } else if (ListUtil.isNotEmpty(operationResults)) {
                    ResultRule operationResult = operationResults.get(0);
                    content +=  "，已开" + operationResult.getRuleBaseHisName();
                    billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(), content, lisResult.getRuleMsg(), ConEnum.operations.getName(), null);
                } else {
                    billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(), content, lisResult.getRuleMsg(), ConEnum.lis.getName(), null);
                }
            }
        }
        return billMsg;
    }

    /**
     * 生成随机数
     *
     * @param min
     * @param max
     * @return
     */
    public BigDecimal random(BigDecimal min, BigDecimal max, Boolean minBoundary, Boolean maxBoundary) {
        if (min == null) {
            min = BigDecimal.ZERO;
        }
        if (max == null) {
            max = min.add(BigDecimal.TEN);
        }
        BigDecimal randomVal = new BigDecimal(Math.random() * (max.doubleValue() - min.doubleValue()) + min.doubleValue());
        BigDecimal random_dec = randomVal.setScale(2, BigDecimal.ROUND_HALF_UP);
        //是否包含边界值
        if (!minBoundary || maxBoundary) {
            random_dec.add(new BigDecimal(0.01d));
        }
        random_dec = randomVal.setScale(2, BigDecimal.ROUND_HALF_UP);
        return random_dec;
    }

    /**
     * 生成随机-整数
     *
     * @param min
     * @param max
     * @param minBoundary
     * @param maxBoundary
     * @return
     */
    public Integer random_int(Integer min, Integer max, Boolean minBoundary, Boolean maxBoundary) {
        if (min == null) {
            min = 0;
        }
        if (max == null) {
            max = min + 10;
        }

        Random random = new Random();
        Integer random_int = random.nextInt(max - min) + min;

        //是否包含边界值
        if (!minBoundary || maxBoundary) {
            random_int++;
        }
        return random_int;
    }

    /**
     * 规则类型转换
     *
     * @param type
     * @return
     */
    public String getOrderType(String type) {
        String orderType = "";
        /**
         *     clinical(5, "临床表现"),
         *     vital(7, "查体"),
         *     age(9,"年龄"),
         *     group(10, "人群");
         */
        if (type.equals(LexiconEnum.LisName.getName())
                || type.equals(LexiconEnum.LisSubName.getName())) {
            orderType = TypeEnum.lis.getName();
        }
        if (type.equals(LexiconEnum.PacsName.getName())
                || type.equals(LexiconEnum.PacsSubName.getName())
                || type.equals(LexiconEnum.PacsResult.getName())) {
            orderType = TypeEnum.pacs.getName();
        }
        if (type.equals(LexiconEnum.Disease.getName())) {
            orderType = TypeEnum.disease.getName();
        }
        if (type.equals(LexiconEnum.Medicine.getName())) {
            orderType = TypeEnum.drug.getName();
        }
        if (type.equals(LexiconEnum.Operation.getName())) {
            orderType = TypeEnum.operation.getName();
        }
        if (type.equals(LexiconEnum.Transfusion.getName())) {
            orderType = TypeEnum.transfusion.getName();
        }
        return orderType;
    }

    /**
     * 基础规则类型转换
     *
     * @param type
     * @return
     */
    public String getConType(String type, Integer ruleType, Integer ruleBaseType) {
        String conType = "";
        if (type.equals(LexiconEnum.Gender.getName())) {
            conType = ConEnum.gender.getName();
        } else if (type.equals(LexiconEnum.Age.getName())) {
            conType = ConEnum.age.getName();
        } else if (type.equals(LexiconEnum.Medicine.getName())) {
            if (ruleBaseType.equals(RuleBaseTypeEnum.Eq.getKey())) {
                conType = ConEnum.oralmeds.getName();
            } else if (ruleBaseType.equals(RuleBaseTypeEnum.Allergen.getKey())) {
                conType = ConEnum.allergicmeds.getName();
            }
        } else if (type.equals(LexiconEnum.Operation.getName())) {
            conType = ConEnum.operations.getName();
        } else if (type.equals(LexiconEnum.LisSubName.getName())) {
            conType = ConEnum.lis.getName();
        } else if (type.equals(LexiconEnum.PacsName.getName()) || type.equals(LexiconEnum.PacsSubName.getName())) {
            if (ruleBaseType.equals(RuleBaseTypeEnum.BillItem.getKey())) {
                conType = ConEnum.exclusion.getName();
            } else {
                if (ruleType.equals(6)) {
                    conType = ConEnum.repeat.getName();
                } else {
                    conType = ConEnum.pacs.getName();
                }
            }
        } else if (type.equals(LexiconEnum.Disease.getName())) {
            conType = ConEnum.disease.getName();
        } else if (type.equals(LexiconEnum.Group.getName())) {
            conType = ConEnum.group.getName();
        } else if (type.equals(LexiconEnum.Vital.getName()) || type.equals(LexiconEnum.VitalResult.getName())) {
            conType = ConEnum.vitals.getName();
        } else if (type.equals(LexiconEnum.PacsResult.getName())) {
            conType = ConEnum.pacsDesc.getName();
        } else if (type.equals(LexiconEnum.Device.getName())) {
            conType = ConEnum.medEqu.getName();
        } else if (type.equals(LexiconEnum.Allergen.getName())) {
            conType = ConEnum.otherAllergy.getName();
        } else if (type.equals(LexiconEnum.Symptom.getName())) {
            conType = ConEnum.clinicfindings.getName();
        } else {
            /**
             * procedures(7, "禁忌操作"),
             * conflictmeds(12, "配伍禁忌"),
             * repeat24(18, "24小时重复开立")
             */
            conType = "禁忌" + type;
        }
        return conType;
    }

    /**
     * 去除小数点末位的0，如果都是0同时去除小数点
     *
     * @param s
     * @return
     */
    public String trimZero(String s) {
        if (s.indexOf(".") > 0) {
            //正则表达
            s = s.replaceAll("0+?$", "");//去掉后面无用的零
            s = s.replaceAll("[.]$", "");//如小数点后面全是零则去掉小数点
        }
        return s;
    }
}