package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.LexiconDTO;
import com.diagbot.dto.ResultRulePageDTO;
import com.diagbot.entity.ResultRule;
import com.diagbot.service.impl.ResultRuleServiceImpl;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ResultRulePageVO;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/3/9 11:06
 */
@Component
public class ResultRuleFacade extends ResultRuleServiceImpl {
    public IPage<ResultRulePageDTO> getPage(ResultRulePageVO resultRulePageVO) {
        resultRulePageVO.setExcludeMessage("映射");
        IPage<ResultRulePageDTO> page = super.getPage(resultRulePageVO);
        List<ResultRulePageDTO> records = page.getRecords();
        List<Long> ids = records.stream()
                .map(ResultRulePageDTO::getIds)
                .flatMap(i -> Arrays.stream(i.split(",")))
                .map(Long::parseLong)
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(ids)) {
            return page;
        }

        List<ResultRule> baseRules = this.list(new QueryWrapper<ResultRule>()
                .in("id", ids));
        if (ListUtil.isEmpty(baseRules)) {
            return page;
        }

        Map<Long, ResultRule> baseRuleMap = baseRules.stream().collect(Collectors.toMap(ResultRule::getId, v -> v));

        records.forEach(record -> {
            List<Long> resultIds = Arrays.asList(record.getIds().split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
            resultIds.forEach(id -> {
                if (ListUtil.isEmpty(record.getBaseRules())) {
                    record.setBaseRules(Lists.newLinkedList());
                }
                record.getBaseRules().add(baseRuleMap.get(id));
            });
        });
        page.setRecords(records);
        return page;
    }

    /**
     * 数据导出
     *
     * @param response
     * @param resultRulePageVO
     */
    public void exportExcel(HttpServletResponse response, ResultRulePageVO resultRulePageVO) {
        resultRulePageVO.setExcludeMessage("映射");
        resultRulePageVO.setSize(-1L);
        IPage<ResultRulePageDTO> page = super.getPage(resultRulePageVO);
        List<ResultRulePageDTO> records = page.getRecords();
        List<ResultRule> baseRules = this.list(new QueryWrapper<ResultRule>()
                .eq("result_id", resultRulePageVO.getResultId())
                .eq("success", resultRulePageVO.getSuccess()));

        Map<Long, ResultRule> baseRuleMap = baseRules.stream().collect(Collectors.toMap(ResultRule::getId, v -> v));

        records.forEach(record -> {
            if (ListUtil.isEmpty(record.getBaseRules())) {
                record.setBaseRules(Lists.newLinkedList());
            }
            List<Long> resultIds = Arrays.asList(record.getIds().split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
            resultIds.forEach(id -> {
                record.getBaseRules().add(baseRuleMap.get(id));
            });
        });

        String fileName = "规则测试结果导出.xls";
        ExcelUtils.exportExcelUser(records, null, "sheet1", ResultRulePageDTO.class, fileName, response);
    }

    /**
     * 动态获取下拉菜单选项
     *
     * @param resultRulePageVO
     * @return
     */
    public Map<String, List<LexiconDTO>> getDroplistData(ResultRulePageVO resultRulePageVO) {
        Map<String, List<LexiconDTO>> map = new HashMap<>();
        List<ResultRule> list = this.list(new QueryWrapper<ResultRule>()
                .eq("result_id", resultRulePageVO.getResultId())
                .eq("success", resultRulePageVO.getSuccess()));

        List<LexiconDTO> libTypeList = list.stream().map(item -> {
            LexiconDTO lexiconDTO = new LexiconDTO();
            lexiconDTO.setLibType(item.getRuleLibType());
            lexiconDTO.setLibTypeName(item.getRuleLibTypeName());
            return lexiconDTO;
        }).distinct()
                .sorted(Comparator.comparing(LexiconDTO::getLibType))
                .collect(Collectors.toList());

        List<LexiconDTO> baseLibTypeList = list.stream()
                .filter(i -> i.getRuleBaseLibType() != null)
                .map(item -> {
                    LexiconDTO lexiconDTO = new LexiconDTO();
                    lexiconDTO.setLibType(item.getRuleBaseLibType());
                    lexiconDTO.setLibTypeName(item.getRuleBaseLibTypeName());
                    return lexiconDTO;
                }).distinct()
                .sorted(Comparator.comparing(LexiconDTO::getLibType))
                .collect(Collectors.toList());

        map.put("ruleLibType", libTypeList);
        map.put("ruleBaseLibType", baseLibTypeList);
        return map;
    }
}
