package com.diagbot.facade;

import com.diagbot.aggregate.RuleTestAggregate;
import com.diagbot.dto.RuleDTO;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.DrugConfig;
import com.diagbot.entity.LisConfig;
import com.diagbot.entity.OperationConfig;
import com.diagbot.entity.PacsConfig;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.ExcelUtils;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.RuleTestVO;
import com.diagbot.vo.RuleVO;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/6 11:53
 */
@Component
public class RuleTestFacade {
    @Autowired
    private RuleTestAggregate ruleTestAggregate;
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;

    @Autowired
    private KlRuleFacade klRuleFacade;
    @Autowired
    private UnUsedMappingFacade unUsedMappingFacade;

    /**
     * 全规则测试
     *
     * @param hospitalIdVO
     */
    public Boolean allRuleTest(HospitalIdVO hospitalIdVO) {
        Boolean retFlag = false;
        //开单合理性规则-通用
        RuleTestVO billRuleVO = new RuleTestVO();
        billRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        billRuleVO.setCaseId(1L);
        //开单合理性规则-输血
        RuleTestVO billTransfusionRuleVO = new RuleTestVO();
        billTransfusionRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        billTransfusionRuleVO.setCaseId(2L);
        //危急值规则-检验
        RuleTestVO criticalLisRuleVO = new RuleTestVO();
        criticalLisRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        criticalLisRuleVO.setCaseId(3L);
        //危急值规则-检查
        RuleTestVO criticalPacsRuleVO = new RuleTestVO();
        criticalPacsRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        criticalPacsRuleVO.setCaseId(4L);
        //高危药品
        RuleTestVO highriskDrugRuleVO = new RuleTestVO();
        highriskDrugRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        highriskDrugRuleVO.setCaseId(5L);
        //高危手术
        RuleTestVO highriskOperationRuleVO = new RuleTestVO();
        highriskOperationRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        highriskOperationRuleVO.setCaseId(6L);
        //化验其他值提醒
        RuleTestVO otherLisRuleVO = new RuleTestVO();
        otherLisRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        otherLisRuleVO.setCaseId(7L);
        //辅检其他值提醒
        RuleTestVO otherPacsRuleVO = new RuleTestVO();
        otherPacsRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        otherPacsRuleVO.setCaseId(8L);
        //输血其他值提醒
        RuleTestVO otherTransfusionRuleVO = new RuleTestVO();
        otherTransfusionRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        otherTransfusionRuleVO.setCaseId(9L);
        //正常项重复开单
        RuleTestVO reBillRuleVO = new RuleTestVO();
        reBillRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        reBillRuleVO.setCaseId(15L);

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("billRuleVO", billRuleVO);
            invokeParams.put("billTransfusionRuleVO", billTransfusionRuleVO);
            invokeParams.put("criticalLisRuleVO", criticalLisRuleVO);
            invokeParams.put("criticalPacsRuleVO", criticalPacsRuleVO);
            invokeParams.put("highriskDrugRuleVO", highriskDrugRuleVO);
            invokeParams.put("highriskOperationRuleVO", highriskOperationRuleVO);
            invokeParams.put("otherLisRuleVO", otherLisRuleVO);
            invokeParams.put("otherPacsRuleVO", otherPacsRuleVO);
            invokeParams.put("otherTransfusionRuleVO", otherTransfusionRuleVO);
            invokeParams.put("reBillRuleVO", reBillRuleVO);

            retFlag
                    = dataBeanAggregateQueryFacade.get("ruleTest", invokeParams, Boolean.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }

        return retFlag;
    }

    /**
     * 开单合理性测试
     *
     * @param ruleTestVO
     */
    public Boolean billRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.billRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 开单合理性测试-输血
     *
     * @param ruleTestVO
     */
    public Boolean billTransfusionRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.billTransfusionRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 危急值规则测试-检验
     *
     * @param ruleTestVO
     */
    public Boolean criticalLisRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.criticalLisRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 危急值规则测试-检查
     *
     * @param ruleTestVO
     */
    public Boolean criticalPacsRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.criticalPacsRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 高危药品测试
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean highriskDrugRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.highriskDrugRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 高危手术测试
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean highriskOperationRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.highriskOperationRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-化验
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean otherLisRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.otherLisRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-辅检
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean otherPacsRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.otherPacsRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-输血
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean otherTransfusionRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.otherTransfusionRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 正常项重复开单
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean reBillRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.reBillRuleTest(ruleTestVO);
        return true;
    }


    /**
     * 数据导出
     *
     * @param response
     * @param hospitalIdVO
     */
    public void exportExcel(HttpServletResponse response, HospitalIdVO hospitalIdVO) {
        Long hospitalId = hospitalIdVO.getHospitalId();

        List<RuleDTO> list = klRuleFacade.getRules(new RuleVO());

        List<LisConfig> lisConfigList = lisConfigFacade.getListByHospitalId(hospitalId);
        List<PacsConfig> pacsConfigList = pacsConfigFacade.getListByHospitalId(hospitalId);
        List<DrugConfig> drugConfigList = drugConfigFacade.getListByHospitalId(hospitalId);
        List<DiseaseConfig> diseaseConfigList = diseaseConfigFacade.getListByHospitalId(hospitalId);
        List<OperationConfig> operationConfigList = operationConfigFacade.getListByHospitalId(hospitalId);
        List<TransfusionConfig> transfusionConfigList = transfusionConfigFacade.getListByHospitalId(hospitalId);

        //诊断映射关系
        Map<String, Map<String, List<Long>>> diseaseConfigMap = diseaseConfigFacade.getUniqueConfigMap(hospitalId, null, null);

        Map<String, List<String>> usedMap = unUsedMappingFacade.usedNames(list);

        List<String> lisNames = usedMap.get("lis");
        List<String> pacsNames = usedMap.get("pacs");
        List<String> diseaseNames = usedMap.get("disease");
        List<String> drugNames = usedMap.get("drug");
        List<String> operationNames = usedMap.get("operation");
        List<String> transfusionNames = usedMap.get("transfusion");

        diseaseNames.addAll(diseaseConfigMap.keySet().stream().filter(i -> i.contains("流产")).collect(Collectors.toList()));

        List<LisConfig> unUsedLisConfigs = unUsedMappingFacade.unUsedMapping(lisConfigList, lisNames);
        List<PacsConfig> unUsedPacsConfigs = unUsedMappingFacade.unUsedMapping(pacsConfigList, pacsNames);
        List<DiseaseConfig> unUsedDiseaseConfigs = unUsedMappingFacade.unUsedMapping(diseaseConfigList, diseaseNames);
        List<DrugConfig> unUsedDrugConfigs = unUsedMappingFacade.unUsedMapping(drugConfigList, drugNames);
        List<OperationConfig> unUsedOperationConfig = unUsedMappingFacade.unUsedMapping(operationConfigList, operationNames);
        List<TransfusionConfig> unUsedTransfusionConfigs = unUsedMappingFacade.unUsedMapping(transfusionConfigList, transfusionNames);

        String fileName = "规则未使用的映射关系.xls";
        Map<String, Map<Class<?>, List<?>>> map = new HashMap<>();
        Map<Class<?>, List<?>> lisMap = new HashMap<Class<?>, List<?>>();
        lisMap.put(LisConfig.class, unUsedLisConfigs);
        map.put("检验", lisMap);
        Map<Class<?>, List<?>> pacsMap = new HashMap<Class<?>, List<?>>();
        pacsMap.put(PacsConfig.class, unUsedPacsConfigs);
        map.put("检查", pacsMap);
        Map<Class<?>, List<?>> diseaseMap = new HashMap<Class<?>, List<?>>();
        diseaseMap.put(DiseaseConfig.class, unUsedDiseaseConfigs);
        map.put("诊断", diseaseMap);
        Map<Class<?>, List<?>> drugMap = new HashMap<Class<?>, List<?>>();
        drugMap.put(DrugConfig.class, unUsedDrugConfigs);
        map.put("药品", drugMap);
        Map<Class<?>, List<?>> operationMap = new HashMap<Class<?>, List<?>>();
        operationMap.put(OperationConfig.class, unUsedOperationConfig);
        map.put("手术和操作", operationMap);
        Map<Class<?>, List<?>> transfusionMap = new HashMap<Class<?>, List<?>>();
        transfusionMap.put(TransfusionConfig.class, unUsedTransfusionConfigs);
        map.put("输血", transfusionMap);

        ExcelUtils.exportExcel(map, null, fileName, response);
    }
}