package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.CaseInfo;
import com.diagbot.entity.RunningInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.CaseInfoService;
import com.diagbot.service.RunningInfoService;
import com.diagbot.service.impl.RunningInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.RunningInfoVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/22 15:05
 */
@Component
public class RunningInfoFacade extends RunningInfoServiceImpl {
    @Autowired
    @Qualifier("runningInfoServiceImpl")
    private RunningInfoService runningInfoService;

    @Autowired
    private CaseInfoService caseInfoService;

    /**
     * 更新运行状态
     *
     * @param runningInfoVO
     * @return
     */
    @Transactional
    public Boolean updateStatus(RunningInfoVO runningInfoVO) {
        Date now = new Date();
        QueryWrapper<RunningInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("case_id", runningInfoVO.getCaseId())
                .eq("hospital_id", runningInfoVO.getHospitalId());
        List<RunningInfo> list = this.list(queryWrapper);
        //先删除原状态
        if (ListUtil.isNotEmpty(list)) {
            list.forEach(item -> {
                item.setGmtModified(now);
                item.setIsDeleted(IsDeleteEnum.Y.getKey());
            });
            runningInfoService.saveOrUpdateBatch(list);
        }

        //记录新状态
        RunningInfo runningInfo = new RunningInfo();
        BeanUtil.copyProperties(runningInfoVO, runningInfo);
        runningInfo.setGmtCreate(now);
        runningInfo.setGmtModified(now);
        runningInfoService.save(runningInfo);

        return true;
    }

    /**
     * 查看运行状态
     *
     * @param runningInfoVO
     * @return
     */
    public Integer getStatus(RunningInfoVO runningInfoVO) {
        QueryWrapper<RunningInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("case_id", runningInfoVO.getCaseId())
                .eq("hospital_id", runningInfoVO.getHospitalId());
        List<RunningInfo> list = runningInfoService.list(queryWrapper);
        if (ListUtil.isEmpty(list)) {
            return 0;
        } else {
            return list.get(0).getStatus();
        }
    }

    /**
     * 查看所有测试用例执行状态
     *
     * @param hospitalIdVO
     * @return
     */
    public Map<Long, Integer> getStatusByHospitalId(HospitalIdVO hospitalIdVO) {
        Map<Long, Integer> retMap = new LinkedHashMap<>();
        List<CaseInfo> caseList = caseInfoService.list(new QueryWrapper<CaseInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        if (ListUtil.isEmpty(caseList)) {
            return retMap;
        }
        List<RunningInfo> list = runningInfoService.list(new QueryWrapper<RunningInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalIdVO.getHospitalId()));
        if (ListUtil.isEmpty(list)) {
            caseList.forEach(item -> {
                retMap.put(item.getId(), 0);
            });
            return retMap;
        }

        Map<Long, Integer> runningMap = EntityUtil.makeMapWithKeyValue(list, "caseId", "status");

        caseList.forEach(item -> {
            if (runningMap.containsKey(item.getId())) {
                retMap.put(item.getId(), runningMap.get(item.getId()));
            } else {
                retMap.put(item.getId(), 0);
            }
        });
        return retMap;
    }
}