package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.PermissionAllDTO;
import com.diagbot.entity.TokenPermission;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.TokenPermissionServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.PermissionTokenAllVO;
import com.diagbot.vo.PermissionTokenCancelVO;
import com.diagbot.vo.PermissionTokenUpadataVO;
import com.diagbot.vo.TokenPermissionAllVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-08-10 10:18
 */
@Component
public class TokenPermissionFacade extends TokenPermissionServiceImpl {

    @Autowired
    TokenHospitalFacade tokenHospitalFacade;

    public List<PermissionAllDTO> getPermissionAlls(TokenPermissionAllVO tokenPermissionAllVO) {
        List<PermissionAllDTO> data = getPermissionAll(tokenPermissionAllVO);
        return data;
    }

    /**
     * @param permissionTokenAllVO
     * @return
     */
    public Boolean boundPermissionTokens(PermissionTokenAllVO permissionTokenAllVO) {
        Date now = DateUtil.now();
        Long tokenId = tokenHospitalFacade.getHospitalTokenId(permissionTokenAllVO.getHospitalId());
        QueryWrapper<TokenPermission> tokenPermissionQuery = new QueryWrapper<>();
        tokenPermissionQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("token_id", tokenId)
                .eq("permission_id", permissionTokenAllVO.getPermissionId());
        int count = count(tokenPermissionQuery);
        if(count>0){
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该数据已存在");
        }
        TokenPermission tokenPermission = new TokenPermission();
        BeanUtil.copyProperties(permissionTokenAllVO, tokenPermission);
        tokenPermission.setCreator(UserUtils.getCurrentPrincipleID());
        tokenPermission.setModifier(UserUtils.getCurrentPrincipleID());
        tokenPermission.setTokenId(tokenId);
        tokenPermission.setGmtModified(now);
        tokenPermission.setGmtCreate(now);
        return save(tokenPermission);
    }

    /**
     * @param permissionTokenUpadataVO
     * @return
     */
    public Boolean updataPermissionToken(PermissionTokenUpadataVO permissionTokenUpadataVO) {
        Date now = DateUtil.now();
        checkTokenPermission(permissionTokenUpadataVO.getId());
        UpdateWrapper<TokenPermission> tokenPermission = new UpdateWrapper<>();
        tokenPermission.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", permissionTokenUpadataVO.getId())
                .eq("permission_id", permissionTokenUpadataVO.getPermissionId())
                .set("param_key", permissionTokenUpadataVO.getParamKey())
                .set("param_value", permissionTokenUpadataVO.getParamValue())
                .set("gmt_modified", now)
                .set("gmt_create", now)
                .set("creator", UserUtils.getCurrentPrincipleID())
                .set("modifier", UserUtils.getCurrentPrincipleID());
        return update(new TokenPermission(), tokenPermission);

    }

    /**
     * 判断数据是否存在有效
     *
     * @param id
     */
    private TokenPermission checkTokenPermission(Long id) {
        // 1.先判断数据是否存在有效
        QueryWrapper<TokenPermission> tokenPermissionData = new QueryWrapper<>();
        tokenPermissionData.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("id", id);
        TokenPermission tokenData = getOne(tokenPermissionData, false);
        if (null == tokenData) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该数据不存在");
        }
        return tokenData;
    }

    public Boolean cancelPermissionToken(PermissionTokenCancelVO permissionTokenCancelVO) {
        checkTokenPermission(permissionTokenCancelVO.getId());
        UpdateWrapper<TokenPermission> tokenPermission = new UpdateWrapper<>();
        tokenPermission.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", permissionTokenCancelVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("gmt_modified", DateUtil.now())
                .set("modifier", UserUtils.getCurrentPrincipleID());
        return update(new TokenPermission(), tokenPermission);
    }
}
