
package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.dto.MedClassMedDTO;
import com.diagbot.dto.RuleDTO;
import com.diagbot.entity.ResultRule;
import com.diagbot.enums.GraphLabelEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;


/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/7 15:32
 */

@Component
public class UnUsedMappingFacade {
    @Autowired
    private KlRelationFacade klRelationFacade;

    /**
     * 未使用映射关系
     *
     * @param items
     * @param usedNames
     * @param <T>
     * @return
     */

    public <T> List<T> unUsedMapping(List<T> items, List<String> usedNames) {
        List<T> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(usedNames)) {
            return items;
        }
        Map<String, List<T>> map = EntityUtil.makeEntityListMap(items, "uniqueName");
        for (Map.Entry<String, List<T>> entry : map.entrySet()) {
            if (!usedNames.contains(entry.getKey())) {
                retList.addAll(entry.getValue());
            }
        }
        return retList;
    }

    /**
     * 规则包含术语
     * @param list
     * @return
     */
    public Map<String, List<String>> usedNames(List<RuleDTO> list) {
        Map<String, List<String>> retMap = new HashMap<>();

        if (ListUtil.isEmpty(list)) {
            return retMap;
        }
        List<String> lisNames = list.stream()
                .filter(i -> i.getRuleLibType().equals(LexiconEnum.LisName.getKey()) || i.getRuleLibType().equals(LexiconEnum.LisSubName.getKey()))
                .map(RuleDTO::getRuleLibName)
                .collect(Collectors.toList());
        lisNames.addAll(list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1) && (i.getRuleBaseLibType().equals(LexiconEnum.LisName.getKey()) || i.getRuleBaseLibType().equals(LexiconEnum.LisSubName.getKey())))
                .map(RuleDTO::getRuleBaseLibName)
                .collect(Collectors.toList()));
        lisNames = lisNames.stream().distinct().collect(Collectors.toList());

        List<String> pacsNames = list.stream()
                .filter(i -> i.getRuleLibType().equals(LexiconEnum.PacsName.getKey()) || i.getRuleLibType().equals(LexiconEnum.PacsSubName.getKey()))
                .map(RuleDTO::getRuleLibName)
                .collect(Collectors.toList());
        pacsNames.addAll(list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1) && (i.getRuleBaseLibType().equals(LexiconEnum.PacsName.getKey()) || i.getRuleBaseLibType().equals(LexiconEnum.PacsSubName.getKey())))
                .map(RuleDTO::getRuleBaseLibName)
                .collect(Collectors.toList()));
        pacsNames = pacsNames.stream().distinct().collect(Collectors.toList());

        List<String> diseaseNames = list.stream()
                .filter(i -> i.getRuleLibType().equals(LexiconEnum.Disease.getKey()))
                .map(RuleDTO::getRuleLibName)
                .collect(Collectors.toList());
        diseaseNames.addAll(list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1) && (i.getRuleBaseLibType().equals(LexiconEnum.Disease.getKey())))
                .map(RuleDTO::getRuleBaseLibName)
                .collect(Collectors.toList()));
        diseaseNames = diseaseNames.stream().distinct().collect(Collectors.toList());

        //药品分类转换
        List<MedClassMedDTO> medClassMedList = klRelationFacade.getMedClassMedList();
        Map<String, List<String>> medClassMedMap =
                medClassMedList.stream().collect(Collectors.groupingBy(MedClassMedDTO::getMedClassLibName,
                        HashMap::new,
                        Collectors.mapping(MedClassMedDTO::getMedLibName, Collectors.toList())));
        List<RuleDTO> medList = Lists.newLinkedList();
        for (RuleDTO result : list) {
            if (result.getHasSubCond() != null && result.getHasSubCond().equals(1)) {
                if (result.getRuleBaseLibType().equals(LexiconEnum.MedChemClass.getKey())
                        || result.getRuleBaseLibType().equals(LexiconEnum.MedZhiLiaoClass.getKey())
                        || result.getRuleBaseLibType().equals(LexiconEnum.MedYaoLiClass.getKey())) {
                    List<String> medicines = medClassMedMap.get(result.getRuleBaseLibName());
                    if (ListUtil.isNotEmpty(medicines)) {
                        for (String med : medicines) {
                            RuleDTO medResult = new RuleDTO();
                            BeanUtil.copyProperties(result, medResult);
                            medResult.setRuleBaseLibName(med);
                            medResult.setRuleBaseLibType(LexiconEnum.Medicine.getKey());
                            medResult.setRuleBaseLibTypeName(LexiconEnum.Medicine.getName());
                            medList.add(medResult);
                        }
                    } else {
                        medList.add(result);
                    }
                } else {
                    medList.add(result);
                }
            } else {
                medList.add(result);
            }
        }

        List<String> drugNames = medList.stream()
                .filter(i -> i.getRuleLibType().equals(LexiconEnum.Medicine.getKey()))
                .map(RuleDTO::getRuleLibName)
                .collect(Collectors.toList());
        drugNames.addAll(medList.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1) && (i.getRuleBaseLibType().equals(LexiconEnum.Medicine.getKey())))
                .map(RuleDTO::getRuleBaseLibName)
                .collect(Collectors.toList()));
        drugNames = drugNames.stream().distinct().collect(Collectors.toList());

        List<String> operationNames = list.stream()
                .filter(i -> i.getRuleLibType().equals(LexiconEnum.Operation.getKey()))
                .map(RuleDTO::getRuleLibName)
                .collect(Collectors.toList());
        operationNames.addAll(list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1) && (i.getRuleBaseLibType().equals(LexiconEnum.Operation.getKey())))
                .map(RuleDTO::getRuleBaseLibName)
                .collect(Collectors.toList()));
        operationNames = operationNames.stream().distinct().collect(Collectors.toList());

        List<String> transfusionNames = list.stream()
                .filter(i -> i.getRuleLibType().equals(LexiconEnum.Transfusion.getKey()))
                .map(RuleDTO::getRuleLibName)
                .collect(Collectors.toList());
        transfusionNames.addAll(list.stream()
                .filter(i -> i.getHasSubCond() != null && i.getHasSubCond().equals(1) && (i.getRuleBaseLibType().equals(LexiconEnum.Transfusion.getKey())))
                .map(RuleDTO::getRuleBaseLibName)
                .collect(Collectors.toList()));
        transfusionNames = transfusionNames.stream().distinct().collect(Collectors.toList());

        retMap.put("lis", lisNames);
        retMap.put("pacs", pacsNames);
        retMap.put("disease", diseaseNames);
        retMap.put("drug", drugNames);
        retMap.put("operation", operationNames);
        retMap.put("transfusion", transfusionNames);
        return retMap;
    }
}
