package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.User;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.UserServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.UserInfoVO;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-08-11 10:13
 */
@Component
public class UserFacade extends UserServiceImpl {

    public User addUser(UserInfoVO userInfoVO) {
        Date now = DateUtil.now();
        QueryWrapper<User> userQuery = new QueryWrapper<>();
        userQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("username", userInfoVO.getUsername());
        int count = count(userQuery);
        if(count>0){
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该数据已存在");
        }
        User user = new User();
        BeanUtil.copyProperties(userInfoVO,user);
        user.setGmtModified(now);
        user.setGmtCreate(now);
        user.setCreator(UserUtils.getCurrentPrincipleID());
        user.setModifier(UserUtils.getCurrentPrincipleID());
        boolean res = save(user);
        return user;
    }
    public Map<String, String> getUserInfoByIds(List<String> ids) {
        List<Long> longList
                = ids.stream().map(r -> Long.valueOf(r)).distinct().collect(Collectors.toList());

        Map<String, String> map = new HashMap<>();
        if (ListUtil.isNotEmpty(longList)) {
            List<User> userList = this.getByIds(longList);
            if (ListUtil.isNotEmpty(userList)) {
                map = userList.stream()
                        .collect(Collectors.toMap(k -> k.getId().toString(), v -> v.getLinkman()));
            }
        }
        return map;
    }

}
