package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.facade.KlConceptStaticFacade;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 静态知识标准术语表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2020-08-18
 */
@RestController
@RequestMapping("/kl/conceptInfo")
@Api(value = "静态知识标准术语相关API", tags = { "静态知识标准术语相关API" })
@SuppressWarnings("unchecked")
public class ConceptInfoController {

    @Autowired
    private KlConceptStaticFacade klConceptStaticFacade;

    @ApiOperation(value = "医学术语检索-新增静态知识[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndexWithoutInfo")
    @SysLogger("staticKnowledgeIndexWithoutInfo")
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndexWithoutInfo(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        staticKnowledgeIndexVO.setHasInfo(0);
        List<StaticKnowledgeIndexDTO> data = klConceptStaticFacade.staticKnowledgeIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取静态知识列表[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作 <br>" +
                    "name： 术语名称<br>" +
                    "status： 启用状态：1-启用、0-禁用<br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<IPage<KlConceptStaticDTO>> getPage(@Valid @RequestBody KlConceptStaticPageVO klConceptStaticPageVO) {
        IPage<KlConceptStaticDTO> data = klConceptStaticFacade.getPage(klConceptStaticPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存静态知识-新增或修改[zhaops]",
            notes = "id： id <br>" +
                    "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作 <br>" +
                    "clinicalPathwayName： 临床路径名称<br>" +
                    "noticeName： 注意事项名称<br>" +
                    "details： 明细<br>")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        Boolean data = klConceptStaticFacade.saveOrUpdateRecord(klConceptStaticVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识启用禁用[zhaops]",
            notes = "id： 术语id <br>" +
                    "status： 启用状态：1-启用、0-禁用<br>")
    @PostMapping("/changeStatus")
    @SysLogger("changeStatus")
    @Transactional
    public RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        Boolean data = klConceptStaticFacade.changeStatus(changeStatusVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识是否存在[zhaops]",
            notes = "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作 <br>")
    @PostMapping("/isExist")
    @SysLogger("isExist")
    public RespDTO<Boolean> isExist(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        Boolean data = klConceptStaticFacade.isExist(klConceptStaticVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据术语id获取静态信息[zhaops]",
            notes = "id： 术语id <br>")
    @PostMapping("/getRecordById")
    @SysLogger("getRecordById")
    public RespDTO<KlConceptStaticDTO> getRecordById(@Valid @RequestBody IdVO idVO) {
        KlConceptStaticDTO data = klConceptStaticFacade.getRecordById(idVO);
        return RespDTO.onSuc(data);
    }
}