package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.DisclaimerInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.DisclaimerInfoFacade;
import com.diagbot.vo.DisclaimerInfoIdVO;
import com.diagbot.vo.DisclaimerInfoTitleVO;
import com.diagbot.vo.DisclaimerInfoVO;
import com.diagbot.vo.DisclaimerInformationAllVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 免责申明详情 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-08-05
 */
@RestController
@RequestMapping("/disclaimerInfo")
@Api(value = "免责申明详情API[by:wangfeng]", tags = { "免责申明详情API" })
@SuppressWarnings("unchecked")
public class DisclaimerInfoController {
    @Autowired
    DisclaimerInfoFacade disclaimerInfoFacade;

    @ApiOperation(value = "分页查询获取免责申明[by:wangfeng]", notes = "分页查询获取免责申明")
    @PostMapping("/getDisclaimerPages")
    @SysLogger("getDisclaimerPages")
    @Transactional
    public RespDTO<IPage<DisclaimerInfoDTO>> getDisclaimerInformations(@RequestBody DisclaimerInfoTitleVO disclaimerInfoTitleVO) {
        IPage<DisclaimerInfoDTO> data = disclaimerInfoFacade.getDisclaimerInformation(disclaimerInfoTitleVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "增加免责申明详情[by:wangfeng]", notes = "增加免责申明详情")
    @PostMapping("/addDisclaimerInfos")
    @SysLogger("addDisclaimerInfos")
    @Transactional
    public RespDTO<Boolean> addDisclaimerInformations(@Valid @RequestBody DisclaimerInfoVO disclaimerInfoVO) {
        boolean res = disclaimerInfoFacade.addDisclaimerInformation(disclaimerInfoVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "修改免责申明详情[by:wangfeng]", notes = "修改免责申明详情")
    @PostMapping("/updateDisclaimerInformations")
    @SysLogger("updateDisclaimerInformations")
    @Transactional
    public RespDTO<Boolean> updateDisclaimerInformations(@Valid @RequestBody DisclaimerInformationAllVO disclaimerInformationAllVO) {
        boolean res = disclaimerInfoFacade.updateDisclaimerInformation(disclaimerInformationAllVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除免责申明详情[by:wangfeng]", notes = "删除免责申明详情")
    @PostMapping("/cancelDisclaimerInformations")
    @SysLogger("cancelDisclaimerInformations")
    @Transactional
    public RespDTO<Boolean> cancelDisclaimerInformations(@Valid @RequestBody DisclaimerInfoIdVO disclaimerInfoIdVO) {
        boolean res = disclaimerInfoFacade.cancelDisclaimerInformation(disclaimerInfoIdVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "启用免责申明详情[by:wangfeng]", notes = "启用免责申明详情")
    @PostMapping("/startDisclaimerInformations")
    @SysLogger("startDisclaimerInformations")
    @Transactional
    public RespDTO<Boolean> startDisclaimerInformations(@Valid @RequestBody DisclaimerInfoIdVO disclaimerInfoIdVO) {
        boolean res = disclaimerInfoFacade.startDisclaimerInformations(disclaimerInfoIdVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "停用免责申明详情[by:wangfeng]", notes = "停用免责申明详情")
    @PostMapping("/endDisclaimerInformations")
    @SysLogger("endDisclaimerInformations")
    @Transactional
    public RespDTO<Boolean> endDisclaimerInformations(@Valid @RequestBody DisclaimerInfoIdVO disclaimerInfoIdVO) {
        boolean res = disclaimerInfoFacade.endDisclaimerInformations(disclaimerInfoIdVO);
        return RespDTO.onSuc(res);
    }
}
