package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptMappingDTO;
import com.diagbot.dto.HospitalInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.facade.HospitalInfoFacade;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.HospitalInfoListVO;
import com.diagbot.vo.HospitalInfoPageVO;
import com.diagbot.vo.HospitalSaveVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 医院信息表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-08-06
 */
@RestController
@RequestMapping("/hospitalInfo")
@Api(value = "医院信息API", tags = { "医院信息API" })
@SuppressWarnings("unchecked")
public class HospitalInfoController {

    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;

    @ApiOperation(value = "获取医院信息列表[by:wangfeng]", notes = "")
    @PostMapping("/getHospitalInfo")
    @SysLogger("getHospitalInfo")
    public RespDTO<List<HospitalInfoDTO>> getHospitalInfo() {
        List<HospitalInfoDTO> data = hospitalInfoFacade.getHospitalInfo();
        return RespDTO.onSuc(data);
    }

    /**
     * 保存或修改映射关系
     *
     * @param hospitalInfo
     * @return
     */
    @ApiOperation(value = "保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid HospitalInfo hospitalInfo) {
        Boolean data = hospitalInfoFacade.saveOrUpdateRecord(hospitalInfo);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量保存或修改映射关系
     *
     * @param hospitalInfoListVO
     * @return
     */
    @ApiOperation(value = "批量保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecords")
    @SysLogger("saveOrUpdateRecords")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecords(@RequestBody @Valid HospitalInfoListVO hospitalInfoListVO) {
        Boolean data = hospitalInfoFacade.saveOrUpdateRecords(hospitalInfoListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 删除映射关系
     *
     * @param idVO
     * @return
     */
    @ApiOperation(value = "删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = hospitalInfoFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量删除映射关系
     *
     * @param idListVO
     * @return
     */
    @ApiOperation(value = "批量删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = hospitalInfoFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 分页查询
     *
     * @param hospitalInfoPageVO
     * @return
     */
    @ApiOperation(value = "分页查询[by:zhaops]", notes = "")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<DiseaseConfig> getPage(@RequestBody @Valid HospitalInfoPageVO hospitalInfoPageVO) {
        IPage<DiseaseConfig> data = hospitalInfoFacade.getPage(hospitalInfoPageVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 获取术语映射统计列表
     *
     * @return
     */
    @ApiOperation(value = "获取术语映射统计列表[by:zhaops]", notes = "")
    @PostMapping("/getMappingListPage")
    @SysLogger("getMappingListPage")
    public RespDTO<ConceptMappingDTO> getMappingListPage(@RequestBody @Valid HospitalInfoPageVO hospitalInfoPageVO) {
        IPage<ConceptMappingDTO> data = hospitalInfoFacade.getMappingListPage(hospitalInfoPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医院信息——修改医院信息[by:zhaops]",
            notes = "id：医院id<br>" +
                    "connect：是否对接（0-不对接，1-对接）<br>")
    @PostMapping("/modifyConnectStatus")
    @SysLogger("modifyConnectStatus")
    @Transactional
    public RespDTO<Boolean> modifyConnectStatus(@RequestBody @Valid HospitalSaveVO hospitalSaveVO) {
        return RespDTO.onSuc(hospitalInfoFacade.modifyConnectStatus(hospitalSaveVO));
    }

    @ApiOperation(value = "医院信息——启用禁用[zhaops]",
            notes = "id： 医院id <br>" +
                    "status： 启用状态：1-启用、0-禁用<br>")
    @PostMapping("/changeStatus")
    @SysLogger("changeStatus")
    @Transactional
    public RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        Boolean data = hospitalInfoFacade.changeStatus(changeStatusVO);
        return RespDTO.onSuc(data);
    }
}