package com.diagbot.web;


import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.KlConceptAllDTO;
import com.diagbot.dto.KlConceptInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.StatusEnum;
import com.diagbot.facade.KlConceptFacade;
import com.diagbot.vo.GetAllForRelationVO;
import com.diagbot.vo.KlConceptAllVO;
import com.diagbot.vo.KlConceptClearVO;
import com.diagbot.vo.KlConceptInfoVO;
import com.diagbot.vo.KlConceptSatarOrdisaVO;
import com.diagbot.vo.KlConceptSaveVO;
import com.diagbot.vo.KlRuleSatartOrdisaVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 概念表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2021-03-01
 */
@RestController
@RequestMapping("/graph/klConcept")
@Api(value = "新-基础术语维护相关API", tags = { "基础术语维护相关API" })
@SuppressWarnings("unchecked")
public class KlConceptController {
    @Autowired
    KlConceptFacade klConceptFacade;

    @ApiOperation(value = "分页获取基础术语维护列表[by:wangfeng]")
    @PostMapping("/getConceptInfoPage")
    @SysLogger("getConceptInfoPage")
    public RespDTO<IPage<KlConceptInfoDTO>> getConceptInfoPages(@RequestBody KlConceptInfoVO klConceptInfoVO) {
        return RespDTO.onSuc(klConceptFacade.getConceptInfoPage(klConceptInfoVO));
    }

    @ApiOperation(value = "根据id获取基础术语详情[by:wangfeng]")
    @PostMapping("/getConceptAll")
    @SysLogger("getConceptAll")
    public RespDTO<KlConceptAllDTO> getConceptInfoAll(@RequestBody @Valid KlConceptAllVO klConceptAllVO) {
        return RespDTO.onSuc(klConceptFacade.getConceptAlls(klConceptAllVO));
    }
    @ApiOperation(value = "新增和修改基础术语详情[by:wangfeng]")
    @PostMapping("/saveConceptInfo")
    @SysLogger("saveConceptInfo")
    @DSTransactional
    public RespDTO<Boolean> saveConceptInfos(@RequestBody @Valid KlConceptSaveVO klConceptSaveVO) {
        return RespDTO.onSuc(klConceptFacade.saveConceptInfo(klConceptSaveVO));
    }
    @ApiOperation(value = "删除同义词[by:wangfeng]")
    @PostMapping("/clearConceptInfo")
    @SysLogger("clearConceptInfo")
    @DSTransactional
    public RespDTO<Boolean> clearConceptInfos(@RequestBody @Valid KlConceptClearVO klConceptClearVO) {
        return RespDTO.onSuc(klConceptFacade.clearConceptInfo(klConceptClearVO));
    }
    @ApiOperation(value = "停用术语[by:wangfeng]")
    @PostMapping("/disableConcept")
    @SysLogger("disableConcept")
    public RespDTO<Boolean> disableConcept(@RequestBody @Valid KlConceptSatarOrdisaVO klConceptSatarOrdisaVO) {
        return RespDTO.onSuc(klConceptFacade.startOrDisableRule(klConceptSatarOrdisaVO, StatusEnum.Disable.getKey()));
    }

    @ApiOperation(value = "启用术语[by:wangfeng]")
    @PostMapping("/startConcept")
    @SysLogger("startConcept")
    public RespDTO<Boolean> startRuleInfos(@RequestBody @Valid KlConceptSatarOrdisaVO klConceptSatarOrdisaVO) {
        return RespDTO.onSuc(klConceptFacade.startOrDisableRule(klConceptSatarOrdisaVO,StatusEnum.Enable.getKey()));
    }
}
