package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.*;
import com.diagbot.entity.KlConcept;
import com.diagbot.facade.KlConceptFacade;
import com.diagbot.facade.KlDiseaseFacade;
import com.diagbot.facade.RelationContactFacade;
import com.diagbot.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 疾病表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2021-03-01
 */
@RestController
@RequestMapping("/klDisease")
@SuppressWarnings("unchecked")
@Api(value = "疾病相关API", tags = {"疾病相关API"})
public class KlDiseaseController {
    @Autowired
    KlDiseaseFacade klDiseaseFacade;
    @Autowired
    KlConceptFacade klConceptFacade;
    @Autowired
    RelationContactFacade relationContactFacade;

    @ApiOperation(value = "分页获取诊断依据列表[by:kongwz]",
            notes = "disName: 要查询的疾病（可模糊查询）<br>" +
                    "current: 当前页 <br>" +
                    "size: 每页条数")
    @PostMapping("/getDiseasePage")
    @SysLogger("getDiseasePage")
    public RespDTO<IPage<DiseasePageDTO>> getDiseasePages(@RequestBody DiseasePageVO diseasePageVO) {
        return RespDTO.onSuc(klDiseaseFacade.getDiseasePageAll(diseasePageVO));
    }

    @ApiOperation(value = "诊断依据-获取详情[by:kongwz]",
            notes = "disName: 诊断名称<br>")
    @PostMapping("/getDetail")
    @SysLogger("getDetail")
    public RespDTO<DiseaseTreeDetailDTO> getDetail(@Valid @RequestBody GetDiseaseDetailVO getDiseaseDetailVO) {
        DiseaseTreeDetailDTO diseaseTreeDetailDTO = klDiseaseFacade.getDetail(getDiseaseDetailVO);
        return RespDTO.onSuc(diseaseTreeDetailDTO);
    }

    @ApiOperation(value = "查询诊断依据相关的类型术语[by:kongwz]",
            notes = "name: 查询术语的名称<br>" +
                    "libType: 查询术语的词性<br>" +
                    "excludedConceptIds: 需要排除的概念id集合")
    @PostMapping("/searchConcept")
    @SysLogger("searchConcept")
    public RespDTO<GetAllForRelationDTO> searchConcept(@Valid @RequestBody SearchConceptVO searchConceptVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = klConceptFacade.searchConceptByNameAndLibType(searchConceptVO);
        return RespDTO.onSuc(getAllForRelationDTOS);
    }

    @ApiOperation(value = "诊断依据保存[by:kongwz]",
            notes = "sid: 第一层的概念id <br>" +
                    "rid: 第一层的关系id <br>" +
                    "eids: 第三层的概念id")
    @PostMapping("/save")
    @SysLogger("save")
    public RespDTO<Boolean> save(@Valid @RequestBody List<KlRelationSaveVO> klRelationSaveVOs) {
        Boolean aBoolean = relationContactFacade.saveRelation(klRelationSaveVOs);
        return RespDTO.onSuc(aBoolean);
    }

    @ApiOperation(value = "诊断依据删除[by:kongwz]",
            notes = "disName： 选中的诊断名字 <br>")
    @PostMapping("/delete")
    @SysLogger("delete")
    public RespDTO<Boolean> delete(@Valid @RequestBody DiseaseDeleteVO diseaseDeleteVO) {
        Boolean aBoolean = relationContactFacade.deleteRelation(diseaseDeleteVO);
        return RespDTO.onSuc(aBoolean);
    }

    @ApiOperation(value = "新增疾病[by:kongwz]",
            notes = "disName： 选中的诊断名字 <br>")
    @PostMapping("/add")
    @SysLogger("add")
    public RespDTO<KlConcept> add(@Valid @RequestBody DiseaseDeleteVO diseaseDeleteVO) {
        List<KlConcept> klConcepts = relationContactFacade.addDisease(diseaseDeleteVO);
        return RespDTO.onSuc(klConcepts);
    }

}
