package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.PlanInfoDefaultDTO;
import com.diagbot.dto.PlanInfoPagesDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.PlanFacade;
import com.diagbot.vo.HospitalPlanCancelVO;
import com.diagbot.vo.HospitalPlanIdVO;
import com.diagbot.vo.HospitalPlanPageVO;
import com.diagbot.vo.HospitalPlanSaveVO;
import com.diagbot.vo.HospitalPlanVO;
import com.diagbot.vo.PlanRevStopVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-08-07
 */
@RestController
@RequestMapping("/plan")
@Api(value = "方案配置信息API", tags = { "方案配置信息API" })
@SuppressWarnings("unchecked")
public class PlanController {
    @Autowired
    PlanFacade planFacade;


    @ApiOperation(value = "分页获取方案配置信息[by:wangfeng]", notes = "分页获取方案配置信息[by:wangfeng]")
    @PostMapping("/getPlanInfoPages")
    @SysLogger("getPlanInfoPages")
    public RespDTO<IPage<PlanInfoPagesDTO>> getPlanInfoPages(@Valid @RequestBody HospitalPlanPageVO hospitalPlanPageVO) {
        IPage<PlanInfoPagesDTO> data = planFacade.getPlanInfoPage(hospitalPlanPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院获取方案配置信息[by:wangfeng]", notes = "hospitalid ：医院id  必填<br> ")
    @PostMapping("/getSysPlanInfoDatas")
    @SysLogger("getSysPlanInfoDatas")
    public RespDTO<List<PlanInfoPagesDTO>> getSysPlanInfoDatas(@Valid @RequestBody HospitalPlanVO hospitalPlanVO) {
        List<PlanInfoPagesDTO> data = planFacade.getSysPlanInfoData(hospitalPlanVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存更新方案配置信息[by:wangfeng]", notes = "保存更新方案配置信息")
    @PostMapping("/savePlanInfoDatas")
    @SysLogger("savePlanInfoDatas")
    @Transactional
    public RespDTO<Boolean> savePlanInfoDatas(@Valid @RequestBody HospitalPlanSaveVO hospitalPlanSaveVO) {
        boolean res = planFacade.savePlanInfoDatas(hospitalPlanSaveVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除方案配置信息[by:wangfeng]", notes = "删除方案配置信息")
    @PostMapping("/cancelPlanDatas")
    @SysLogger("cancelPlanDatas")
    @Transactional
    public RespDTO<Boolean> cancelPlanDatas(@Valid @RequestBody HospitalPlanCancelVO hospitalPlanCancelVO) {
        boolean res = planFacade.cancelPlanData(hospitalPlanCancelVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "停用启用方案配置[by:wangfeng]", notes = "停用启用方案配置")
    @PostMapping("/revStopPlans")
    @SysLogger("revStopPlans")
    @Transactional
    public RespDTO<Boolean> revStopPlans(@Valid @RequestBody PlanRevStopVO planRevStopVO) {
        boolean res = planFacade.revStopPlan(planRevStopVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "获取默认方案配置[by:wangfeng]", notes = "获取默认方案配置")
    @PostMapping("/getPlansDefault")
    @SysLogger("getPlansDefault")
    @Transactional
    public RespDTO<PlanInfoDefaultDTO> getPlansDefault() {
        PlanInfoDefaultDTO data = planFacade.getPlansDefaults();
        return RespDTO.onSuc(data);
    }
    @ApiOperation(value = "根据方案id获取配置信息[by:wangfeng]", notes = "id ：方案编号  <br> ")
    @PostMapping("/getPlanInfoIds")
    @SysLogger("getPlanInfoIds")
    public RespDTO<List<PlanInfoPagesDTO>> getPlanInfoIds(@Valid @RequestBody HospitalPlanIdVO hospitalPlanIdVO) {
        List<PlanInfoPagesDTO> data = planFacade.getPlanInfoIds(hospitalPlanIdVO);
        return RespDTO.onSuc(data);
    }
}

