package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.CaseResultDTO;
import com.diagbot.dto.LexiconDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.ResultRulePageDTO;
import com.diagbot.facade.ResultInfoFacade;
import com.diagbot.facade.ResultRuleFacade;
import com.diagbot.facade.RuleTestFacade;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.ResultRulePageVO;
import com.diagbot.vo.RuleTestVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/7 19:46
 */
@RestController
@RequestMapping("/test/rule")
@Api(value = "规则测试相关API", tags = { "规则测试相关API" })
public class RuleTestController {
    @Autowired
    private RuleTestFacade ruleTestFacade;
    @Autowired
    private ResultInfoFacade resultInfoFacade;
    @Autowired
    private ResultRuleFacade resultRuleFacade;


    @ApiOperation(value = "医院规则测试结果列表[zhaops]",
            notes = "hospitalId： 医院Id <br>")
    @PostMapping("/getCaseResultList")
    @SysLogger("getCaseResultList")
    public RespDTO<List<CaseResultDTO>> getCaseResultList(@Valid @RequestBody HospitalIdVO hospitalIdVO) {
        List<CaseResultDTO> data = resultInfoFacade.getCaseResultList(hospitalIdVO);
        return RespDTO.onSuc(data);
    }

    //region --------------------------规则测试------------------------------

    @ApiOperation(value = "全规则测试[zhaops]",
            notes = "hospitalId： 医院Id <br>")
    @PostMapping("/allRuleTest")
    @SysLogger("allRuleTest")
    public RespDTO<Boolean> allRuleTest(@Valid @RequestBody HospitalIdVO hospitalIdVO) {
        Boolean data = ruleTestFacade.allRuleTest(hospitalIdVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "规则测试[zhaops]",
            notes = "hospitalId： 医院Id <br>" +
                    "caseId： 测试用例id<br>")
    @PostMapping("/ruleTest")
    @SysLogger("ruleTest")
    public RespDTO<Boolean> ruleTest(@Valid @RequestBody RuleTestVO ruleTestVO) {
        Boolean data = ruleTestFacade.billRuleTest(ruleTestVO);
        return RespDTO.onSuc(data);
    }

    //endregion --------------------------规则测试------------------------------

    //region --------------------------分页查询------------------------------

    @ApiOperation(value = "分页查询列表[zhaops]",
            notes = "success： 成功标志（0：失败，1：成功） <br>" +
                    "resultId： 结果id <br>" +
                    "ruleLibType： 规则术语类型 <br>" +
                    "ruleLibName： 规则医学标准术语（模糊查询） <br>" +
                    "ruleBaseLibType： 基础规则术语类型 <br>" +
                    "ruleBaseLibName： 基础规则医学标准术语（模糊查询） <br>" +
                    "message： 失败原因（模糊查询 <br>")
    @PostMapping("/getResultRulePage")
    @SysLogger("getResultRulePage")
    public RespDTO<IPage<ResultRulePageDTO>> getResultRulePage(@Valid @RequestBody ResultRulePageVO resultRulePageVO) {
        IPage<ResultRulePageDTO> data = resultRuleFacade.getPage(resultRulePageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "动态加载下拉菜单选项[zhaops]",
            notes = "success： 成功标志（0：失败，1：成功） <br>" +
                    "resultId： 结果id <br>" )
    @PostMapping("/getDroplistData")
    @SysLogger("getDroplistData")
    public RespDTO<Map<String,List<LexiconDTO>>> getDroplistData(@Valid @RequestBody ResultRulePageVO resultRulePageVO) {
        Map<String, List<LexiconDTO>> data = resultRuleFacade.getDroplistData(resultRulePageVO);
        return RespDTO.onSuc(data);
    }

    //endregion --------------------------分页查询------------------------------

    //region --------------------------导出------------------------------

    @ApiOperation(value = "导出规则未覆盖映射关系[by:zhaops]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response, @RequestBody @Valid HospitalIdVO hospitalIdVO) {
        ruleTestFacade.exportExcel(response, hospitalIdVO);
    }

    @ApiOperation(value = "规则测试结果导出[by:zhaops]",
            notes = "success： 成功标志（0：失败，1：成功） <br>" +
                    "resultId： 结果id <br>" +
                    "ruleLibType： 规则术语类型 <br>" +
                    "ruleLibName： 规则医学标准术语（模糊查询） <br>" +
                    "ruleBaseLibType： 基础规则术语类型 <br>" +
                    "ruleBaseLibName： 基础规则医学标准术语（模糊查询） <br>" +
                    "message： 失败原因（模糊查询 <br>")
    @PostMapping("/ruleExportExcel")
    @SysLogger("ruleExportExcel")
    public void ruleExportExcel(HttpServletResponse response, @RequestBody @Valid ResultRulePageVO resultRulePageVO) {
        resultRuleFacade.exportExcel(response, resultRulePageVO);
    }

    //endregion --------------------------导出------------------------------
}