package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.RunningInfoFacade;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.RunningInfoVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/22 15:06
 */
@RestController
@RequestMapping("/test/running")
@Api(value = "执行测试相关API", tags = { "执行测试相关API" })
public class RunningInfoController {
    @Autowired
    private RunningInfoFacade runningInfoFacade;

    @ApiOperation(value = "查看测试用例运行状态[zhaops]",
            notes = "hospitalId： 医院Id <br>" +
                    "caseId： 测试用例id<br>")
    @PostMapping("/getStatus")
    @SysLogger("getStatus")
    public RespDTO<Integer> getStatus(@Valid @RequestBody RunningInfoVO runningInfoVO) {
        Integer data = runningInfoFacade.getStatus(runningInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "查拉面所有测试用例运行状态[zhaops]",
            notes = "hospitalId： 医院Id <br>")
    @PostMapping("/getStatusByHospitalId")
    @SysLogger("getStatusByHospitalId")
    public RespDTO<Map<Long, Integer>> getStatusByHospitalId(@Valid @RequestBody HospitalIdVO hospitalIdVO) {
        Map<Long, Integer> data = runningInfoFacade.getStatusByHospitalId(hospitalIdVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "更新测试用例运行状态[zhaops]",
            notes = "hospitalId： 医院Id <br>" +
                    "caseId： 测试用例id<br>" +
                    "status：状态（1：运行中，0：空闲）")
    @PostMapping("/updateStatus")
    @SysLogger("updateStatus")
    public RespDTO<Boolean> updateStatus(@Valid @RequestBody RunningInfoVO runningInfoVO) {
        Boolean data = runningInfoFacade.updateStatus(runningInfoVO);
        return RespDTO.onSuc(data);
    }
}