package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.CaseStaticResultDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ResultStaticKnowledge;
import com.diagbot.enums.StaticKnowledgeEnum;
import com.diagbot.facade.ResultStaticInfoFacade;
//import com.diagbot.facade.ResultStaticKnowledgeFacade;
import com.diagbot.facade.RunningInfoFacade;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.ResultStaticPageVO;
import com.diagbot.vo.RunningInfoVO;
import com.diagbot.vo.StaticKnowledgeTestVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/21 15:18
 */
@RestController
@RequestMapping("/test/staticKnowledge")
@Api(value = "静态知识测试相关API", tags = { "静态知识测试相关API" })
public class StaticKnowledgeTestController {

    @Autowired
    private ResultStaticInfoFacade resultStaticInfoFacade;
    /*@Autowired
    private ResultStaticKnowledgeFacade resultStaticKnowledgeFacade;*/
    @Autowired
    private RunningInfoFacade runningInfoFacade;

    @ApiOperation(value = "静态知识测试总览列表[zhaops]",
            notes = "hospitalId： 医院Id <br>")
    @PostMapping("/getCaseResultList")
    @SysLogger("getCaseResultList")
    public RespDTO<List<CaseStaticResultDTO>> getCaseResultList(@Valid @RequestBody HospitalIdVO hospitalIdVO) {
        List<CaseStaticResultDTO> data = resultStaticInfoFacade.getCaseResultList(hospitalIdVO);
        return RespDTO.onSuc(data);
    }

    /*@ApiOperation(value = "静态知识测试[zhaops]",
            notes = "hospitalId： 医院Id <br>" +
                    " type：术语类型（1：诊断、2：检验、3：检查、4：药品、5：手术/操作）<br>" +
                    "caseId： 测试用例id<br>")
    @PostMapping("/staticKnowledgeTest")
    @SysLogger("staticKnowledgeTest")
    public RespDTO<Boolean> staticKnowledgeTest(@RequestParam("file") MultipartFile file,
                                                @RequestParam("hospitalId") Long hospitalId,
                                                @RequestParam("type") Integer type,
                                                @RequestParam("caseId") Long caseId) {
        StaticKnowledgeTestVO testVO = new StaticKnowledgeTestVO();
        testVO.setHospitalId(hospitalId);
        testVO.setType(type);
        testVO.setCaseId(caseId);
        Boolean data = false;
        //开始运行
        RunningInfoVO runningInfoVO = new RunningInfoVO();
        runningInfoVO.setCaseId(caseId);
        runningInfoVO.setHospitalId(hospitalId);
        runningInfoVO.setStatus(1);
        runningInfoFacade.updateStatus(runningInfoVO);
        if (type.equals(StaticKnowledgeEnum.Lis.getKey())) {
            data = resultStaticKnowledgeFacade.lisTest(file, testVO);
        } else if (type.equals(StaticKnowledgeEnum.Pacs.getKey())) {
            data = resultStaticKnowledgeFacade.pacsTest(file, testVO);
        } else if (type.equals(StaticKnowledgeEnum.Drug.getKey())) {
            data = resultStaticKnowledgeFacade.drugTest(file, testVO);
        } else if (type.equals(StaticKnowledgeEnum.Disease.getKey())) {
            data = resultStaticKnowledgeFacade.diseaseTest(file, testVO);
        } else if (type.equals(StaticKnowledgeEnum.Operation.getKey())) {
            data = resultStaticKnowledgeFacade.operationTest(file, testVO);
        }
        //结束运行
        runningInfoVO.setStatus(0);
        runningInfoFacade.updateStatus(runningInfoVO);
        return RespDTO.onSuc(data);
    }*/

    /*@ApiOperation(value = "静态知识测试结果明细列表[zhaops]",
            notes = "resultId： 结果id <br>" +
                    "hasStandName： 是否映射到标准术语（0：否，1：是） <br>" +
                    "hasStaticKnowledge： 是否有静态知识（0：无，1：有） <br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<IPage<ResultStaticKnowledge>> getPage(@Valid @RequestBody ResultStaticPageVO resultStaticPageVO) {
        IPage<ResultStaticKnowledge> data = resultStaticKnowledgeFacade.getPage(resultStaticPageVO);
        return RespDTO.onSuc(data);
    }*/

    //region --------------------------导出------------------------------

    /*@ApiOperation(value = "通用未映射到标准术语测试结果导出[by:zhaops]",
            notes = "resultId： 结果id <br>" +
                    "hasStandName： 是否映射到标准术语（0：否，1：是） <br>" +
                    "hasStaticKnowledge： 是否有静态知识（0：无，1：有） <br>")
    @PostMapping("/exportCommonExcel")
    @SysLogger("exportCommonExcel")
    public void exportCommonExcel(HttpServletResponse response, @RequestBody @Valid ResultStaticPageVO resultStaticPageVO) {
        resultStaticKnowledgeFacade.exportCommonExcel(response, resultStaticPageVO);
    }*/

    /*@ApiOperation(value = "通用缺少静态知识测试结果导出[by:zhaops]",
            notes = "resultId： 结果id <br>" +
                    "hasStandName： 是否映射到标准术语（0：否，1：是） <br>" +
                    "hasStaticKnowledge： 是否有静态知识（0：无，1：有） <br>")
    @PostMapping("/exportCommonStaticExcel")
    @SysLogger("exportCommonStaticExcel")
    public void exportCommonStaticExcel(HttpServletResponse response, @RequestBody @Valid ResultStaticPageVO resultStaticPageVO) {
        resultStaticKnowledgeFacade.exportCommonStaticExcel(response, resultStaticPageVO);
    }*/

    /*@ApiOperation(value = "检验未映射到标准术语测试结果导出[by:zhaops]",
            notes = "resultId： 结果id <br>" +
                    "hasStandName： 是否映射到标准术语（0：否，1：是） <br>" +
                    "hasStaticKnowledge： 是否有静态知识（0：无，1：有） <br>")
    @PostMapping("/exportLisExcel")
    @SysLogger("exportLisExcel")
    public void exportLisExcel(HttpServletResponse response, @RequestBody @Valid ResultStaticPageVO resultStaticPageVO) {
        resultStaticKnowledgeFacade.exportLisExcel(response, resultStaticPageVO);
    }*/

    /*@ApiOperation(value = "检验缺少静态知识测试结果导出[by:zhaops]",
            notes = "resultId： 结果id <br>" +
                    "hasStandName： 是否映射到标准术语（0：否，1：是） <br>" +
                    "hasStaticKnowledge： 是否有静态知识（0：无，1：有） <br>")
    @PostMapping("/exportLisStaticExcel")
    @SysLogger("exportLisStaticExcel")
    public void exportLisStaticExcel(HttpServletResponse response, @RequestBody @Valid ResultStaticPageVO resultStaticPageVO) {
        resultStaticKnowledgeFacade.exportLisStaticExcel(response, resultStaticPageVO);
    }*/

    //endregion --------------------------导出------------------------------
}