package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TokenHospitaDTO;
import com.diagbot.facade.TokenHospitalFacade;
import com.diagbot.vo.TokenHospitalCancelVO;
import com.diagbot.vo.TokenHospitalInfoVO;
import com.diagbot.vo.TokenHospitalUpdataVO;
import com.diagbot.vo.TokenHospitalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-08-06
 */
@RestController
@RequestMapping("/tokenHospital")
@Api(value = "医院token信息API", tags = { "医院token信息API" })
@SuppressWarnings("unchecked")
public class TokenHospitalController {

    @Autowired
    private TokenHospitalFacade tokenHospitalFacade;

    @ApiOperation(value = "分页获取医院token信息[by:wangfeng]",
            notes = "userId: 医院id<br>" +
                    "HospitalId：医院id<br>")
    @PostMapping("/getTokenHospital")
    @SysLogger("getTokenHospital")
    public RespDTO<IPage<TokenHospitaDTO>> getTokenHospital(@RequestBody @Valid TokenHospitalVO tokenHospitalVO) {
        IPage<TokenHospitaDTO> data = tokenHospitalFacade.getTokenHospitals(tokenHospitalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "生成医院token信息[by:wangfeng]", notes = "")
    @PostMapping("/addTokenHospital")
    @SysLogger("addTokenHospital")
    @Transactional
    public RespDTO<Boolean> addTokenHospital(@RequestBody @Valid TokenHospitalInfoVO tokenHospitalInfoVO) {
        boolean res = tokenHospitalFacade.addTokenHospitals(tokenHospitalInfoVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "延续医院token有效时间[by:wangfeng]", notes = "")
    @PostMapping("/updataTokenHospital")
    @SysLogger("updataTokenHospital")
    @Transactional
    public RespDTO<Boolean> updataTokenHospital(@RequestBody @Valid TokenHospitalUpdataVO tokenHospitalUpdataVO) {
        boolean res = tokenHospitalFacade.updataTokenHospitals(tokenHospitalUpdataVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除医院token信息[by:wangfeng]", notes = "")
    @PostMapping("/cancelTokenHospital")
    @SysLogger("cancelTokenHospital")
    @Transactional
    public RespDTO<Boolean> cancelTokenHospital(@RequestBody @Valid TokenHospitalCancelVO tokenHospitalCancelVO) {
        boolean res = tokenHospitalFacade.cancelTokenHospitals(tokenHospitalCancelVO);
        return RespDTO.onSuc(res);
    }
}
