package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.PermissionAllDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.TokenPermissionFacade;
import com.diagbot.vo.PermissionTokenAllVO;
import com.diagbot.vo.PermissionTokenCancelVO;
import com.diagbot.vo.PermissionTokenUpadataVO;
import com.diagbot.vo.TokenPermissionAllVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-08-10
 */
@RestController
@RequestMapping("/tokenPermission")
@Api(value = "医院权限相关API", tags = { "医院权限相关API" })
@SuppressWarnings("unchecked")
public class TokenPermissionController {
    @Autowired
    private TokenPermissionFacade tokenPermissionFacade;

    @ApiOperation(value = "获取医院权限列表[by:wangfeng]",
            notes = "")
    @PostMapping("/getTokenPermissionAll")
    @SysLogger("getTokenPermissionAll")
    public RespDTO<List<PermissionAllDTO>> getPermissionAll(@RequestBody TokenPermissionAllVO tokenPermissionAllVO) {
        return RespDTO.onSuc(tokenPermissionFacade.getPermissionAlls(tokenPermissionAllVO));
    }

    @ApiOperation(value = "绑定系统资源Url和医院Token权限的关系[by:wangfeng]",
            notes = "")
    @PostMapping("/boundPermissionToken")
    @SysLogger("boundPermissionToken")
    public RespDTO<Boolean> boundPermissionToken(@RequestBody PermissionTokenAllVO permissionTokenAllVO) {
        return RespDTO.onSuc(tokenPermissionFacade.boundPermissionTokens(permissionTokenAllVO));
    }

    @ApiOperation(value = "修改医院Token系统资源参数[by:wangfeng]",
            notes = "")
    @PostMapping("/updataPermissionToken")
    @SysLogger("updataPermissionToken")
    public RespDTO<Boolean> updataPermissionToken(@RequestBody PermissionTokenUpadataVO permissionTokenUpadataVO) {
        return RespDTO.onSuc(tokenPermissionFacade.updataPermissionToken(permissionTokenUpadataVO));
    }

    @ApiOperation(value = "删除系统资源Url和医院Token权限的关系[by:wangfeng]",
            notes = "")
    @PostMapping("/cancelPermissionToken")
    @SysLogger("cancelPermissionToken")
    public RespDTO<Boolean> cancelPermissionToken(@RequestBody PermissionTokenCancelVO permissionTokenCancelVO) {
        return RespDTO.onSuc(tokenPermissionFacade.cancelPermissionToken(permissionTokenCancelVO));
    }

}
