package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.RelationNodeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TreeAllDTO;
import com.diagbot.dto.TreeDTO;
import com.diagbot.facade.KlConceptFacade;
import com.diagbot.facade.KlRelationFacade;
import com.diagbot.facade.RelationContactFacade;
import com.diagbot.vo.GetAllForRelationVO;
import com.diagbot.vo.KlRelationSaveVO;
import com.diagbot.vo.RelationContactDetailVO;
import com.diagbot.vo.TreeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @author kwz
 * @date 2021/3/3
 * @time 15:40
 */
@RestController
@RequestMapping("/multContact")
@SuppressWarnings("unchecked")
@Api(value = "树形结构维护相关API", tags = {"知识库标准化-树形结构维护相关API"})
public class TreeContactController {
    @Autowired
    private RelationContactFacade relationContactFacade;
    @Autowired
    private KlConceptFacade klConceptFacade;
    @Autowired
    private KlRelationFacade klRelationFacade;


    @ApiOperation(value = "知识库标准化-树形结构维护相关API-详情(根据类别查询)[by:kongwz]",
            notes = "type :类型 <br> " +
                    "{1:药品化学物质类别    2:药品治疗学类别   3:药品药理学类别   4:药品解剖学类别   5:症状类别" +
                    "6:手术和操作类别  7:实验室检查类别   8:辅助检查类别" +
                    "}")
    @PostMapping("/getTree")
    @SysLogger("getTree")
    public RespDTO<TreeAllDTO> getTree(@RequestBody TreeVO treeVO) {
        return RespDTO.onSuc(klRelationFacade.getTree(treeVO));
    }

    @ApiOperation(value = "知识库标准化-获取所有概念(术语关系维护时筛选使用)[by:kongwz]",
            notes = "excludedConceptIds: 要筛选掉的概念id <br>" +
                    "name: 要搜索的名字 <br>" +
                    "typeId: 要搜索的类型")
    @PostMapping("/getAllForRelation")
    @SysLogger("getAllForRelation")
    public RespDTO<List<GetAllForRelationDTO>> getAllForRelation(@RequestBody GetAllForRelationVO getAllForRelationVO) {
        return RespDTO.onSuc(klConceptFacade.getAllForRelation(getAllForRelationVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语多层关联维护-添加或者编辑[by:kongwz]",
            notes = "sid: 第一层的概念id <br>" +
                    "rid: 第一层的关系id <br>" +
                    "eid: 第二层的概念id <br>" +
                    "srid: 第二层的关系id <br>" +
                    "eids: 第三层的概念id")
    @PostMapping("/addRelation")
    @SysLogger("addRelation")
    public RespDTO<Boolean> addRelation(@Valid @RequestBody List<KlRelationSaveVO> klRelationSaveVOs) {
        return RespDTO.onSuc(relationContactFacade.addRelation(klRelationSaveVOs));
    }


    @ApiOperation(value = "知识库标准化-树形结构维护相关API-搜索定位[by:kongwz]",
            notes = "name: 要搜索的名字 <br>" +
                    "typeId: 要搜索的类型")
    @PostMapping("/searchAndPosition")
    @SysLogger("searchAndPosition")
    public RespDTO<List<String>> searchAndPosition(@RequestBody GetAllForRelationVO getAllForRelationVO){
        return RespDTO.onSuc(relationContactFacade.searchAndPosition(getAllForRelationVO));
    }


}
