package com.diagbot.client;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.ScaleContent;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.DisclaimerInformationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.DisclaimerInformationVO;
import com.diagbot.vo.GetStaticKnowledgeVO;
import com.diagbot.vo.VersionVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: 调用知识库服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {

    /**
     * 推理
     *
     * @param searchVO
     * @return
     */
    @PostMapping("/push/pushInner")
    RespDTO<PushDTO> pushInner(@RequestBody SearchVO searchVO);


    @PostMapping("/disclaimerInformation/getDisclaimerInformations")
    RespDTO<List<DisclaimerInformationDTO>> getDisclaimerInformations(@RequestBody DisclaimerInformationVO disclaimerInformationVO);

    /**
     * 计算接口-量表/公式
     *
     * @param calculateData
     * @return
     */
    @PostMapping(value = "/clinicaldata/scaleCalc")
    RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData);

    /**
     * 获取量表接口
     *
     * @param searchVO
     * @return
     */
    @PostMapping(value = "/clinicaldata/getScale")
    RespDTO<List<ScaleContent>> getScale(@RequestBody SearchVO searchVO);

    /**
     * 获取提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    @PostMapping(value = "/conceptDetail/getConceptDetail")
    RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO);

    /**
     * 批量获取提示信息
     *
     * @param conceptIntroduceVOList
     * @return
     */
    @PostMapping(value = "/conceptDetail/getConceptDetails")
    RespDTO<List<ConceptIntroduceDTO>> getConceptDetails(@Valid @RequestBody List<ConceptIntroduceVO> conceptIntroduceVOList);

    /**
     * 获取版本信息
     */
    @PostMapping("/versionInfo/getVersionInfoAlls")
    RespDTO<VersionWrapperDTO> getVersionInfoAlls(@RequestBody VersionVO versionVO);

    /**
     * 静态知识检索
     *
     * @param getStaticKnowledgeVO
     * @return
     */
    @PostMapping("/concept/getStaticKnowledge")
    RespDTO<List<RetrievalDTO>> getStaticKnowledge(@RequestBody GetStaticKnowledgeVO getStaticKnowledgeVO);

    /**
     * 静态知识检索【海南】
     *
     * @param getStaticKnowledgeVO
     * @return
     */
    @PostMapping("/concept/getStaticKnowledgeHaiNan")
    RespDTO<List<RetrievalDTO>> getStaticKnowledgeHaiNan(@RequestBody GetStaticKnowledgeVO getStaticKnowledgeVO);

//    /**
//     * 量表搜索
//     *
//     * @param scaleIndexVO
//     * @return
//     */
//    @PostMapping(value = "/scale/index")
//    RespDTO<List<ScaleIndexDTO>> index(@RequestBody ScaleIndexVO scaleIndexVO);

    /**
     * 获取字典信息
     *
     * @return
     */
    @PostMapping(value = "/dictionaryInfo/getList")
    RespDTO<Map<Long, List<DictionaryInfoDTO>>> getList();
}