package com.diagbot.client.hystrix;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.ScaleContent;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.DisclaimerInformationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.DisclaimerInformationVO;
import com.diagbot.vo.GetStaticKnowledgeVO;
import com.diagbot.vo.VersionVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: 调用知识库服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@Component
@Slf4j
public class AiptServiceHystrix implements AiptServiceClient {
    /**
     * 推理
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<PushDTO> pushInner(@RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "pushInner");
        return null;
    }


    @Override
    public RespDTO<List<DisclaimerInformationDTO>> getDisclaimerInformations(DisclaimerInformationVO disclaimerInformationVO) {
        log.error("【hystrix】调用{}异常", "getDisclaimerInformations");
        return null;
    }

    /**
     * 计算接口-量表/公式
     *
     * @param calculateData
     * @return
     */
    @Override
    public RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData) {
        log.error("【hystrix】调用{}异常", "scaleCalc");
        return null;
    }

    /**
     * 获取量表接口
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<List<ScaleContent>> getScale(@RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "getScale");
        return null;
    }

    /**
     * 获取提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    @Override
    public RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        log.error("【hystrix】调用{}异常", "getConceptDetail");
        return null;
    }

    /**
     * 批量获取提示信息
     *
     * @param conceptIntroduceVOList
     * @return
     */
    @Override
    public RespDTO<List<ConceptIntroduceDTO>> getConceptDetails(@Valid @RequestBody List<ConceptIntroduceVO> conceptIntroduceVOList) {
        log.error("【hystrix】调用{}异常", "getConceptDetails");
        return null;
    }

    /**
     * 获取版本信息
     */
    @Override
    public RespDTO<VersionWrapperDTO> getVersionInfoAlls(VersionVO versionVO) {
        log.error("【hystrix】调用{}异常", "getVersionInfoAlls");
        return null;
    }

    /**
     * 静态知识检索
     *
     * @param getStaticKnowledgeVO
     * @return
     */
    @Override
    public RespDTO<List<RetrievalDTO>> getStaticKnowledge(GetStaticKnowledgeVO getStaticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "getStaticKnowledge");
        return null;
    }

    @Override
    public RespDTO<List<RetrievalDTO>> getStaticKnowledgeHaiNan(GetStaticKnowledgeVO getStaticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "getStaticKnowledgeHaiNan");
        return null;
    }

    //    /**
//     * 量表搜索
//     *
//     * @param scaleIndexVO
//     * @return
//     */
//    @Override
//    public RespDTO<List<ScaleIndexDTO>> index(@RequestBody ScaleIndexVO scaleIndexVO) {
//        log.error("【hystrix】调用{}异常", "index");
//        return null;
//    }

    /**
     * 获取字典信息
     *
     * @return
     */
    @Override
    public RespDTO<Map<Long, List<DictionaryInfoDTO>>> getList() {
        log.error("【hystrix】调用{}异常", "getList");
        return null;
    }
}