package com.diagbot.facade;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.ScaleContent;
import com.diagbot.dto.ConceptPushDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.PushVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/13 17:47
 */
@Component
public class PushFacade {
    @Autowired
    AiptServiceClient aiptServiceClient;
    @Autowired
    AssembleFacade assembleFacade;

    /**
     * 推理接口
     *
     * @param pushVO
     * @return
     */
    public PushDTO push(PushVO pushVO) {
        PushDTO pushDTO = new PushDTO();
        SearchVO searchVO = assembleFacade.assembleData(pushVO);
        RespDTO<PushDTO> respDTO = aiptServiceClient.pushInner(searchVO);
        RespDTOUtil.respNGDeal(respDTO, "中间层没有返回结果");
        PushDTO data = respDTO.data;

        String featureType = searchVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        pushDTO.setHasIndications(data.getHasIndications());

        //症状
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Symptom.getKey()))) {
            List<ConceptPushDTO> symptom = data.getSymptom();
            pushDTO.setSymptom(symptom);
        }
        //查体
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Vital_Result.getKey())) ||
                featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Vital_Index.getKey()))) {
            List<ConceptPushDTO> vitals = data.getVital();
            pushDTO.setVital(vitals);
        }
        //化验
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Lis.getKey()))) {
            List<ConceptPushDTO> labs = data.getLab();
            pushDTO.setLab(labs);
        }
        //辅检
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Pacs.getKey()))) {
            pushDTO.setPacs(data.getPacs());
        }
        //诊断
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Disease.getKey()))) {
            pushDTO.setDis(data.getDis());
        }
        //推送指标
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Indication.getKey()))) {
            pushDTO.setMedicalIndications(data.getMedicalIndications());
        }
        //治疗方案
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Treat.getKey()))) {
            pushDTO.setTreat(data.getTreat());
        }
        return pushDTO;
    }

    /**
     * 治疗方案推送-不包含慢病内容
     *
     * @param pushVO
     * @return
     */
    public List<ScaleContent> pushScale(PushVO pushVO) {
        SearchVO searchVO = assembleFacade.assembleData(pushVO);
        RespDTO<List<ScaleContent>> res = aiptServiceClient.getScale(searchVO);
        RespDTOUtil.respNGDeal(res, "中间层没有返回量表");
        List<ScaleContent> data = res.data;
        return data;
    }
}