package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConceptDetailFacade;
import com.diagbot.vo.ConceptIntorducesVO;
import com.diagbot.vo.ConceptIntroduceVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/14 9:51
 */
@RestController
@RequestMapping("/conceptDetail")
@Api(value = "提示信息相关API", tags = { "数据服务模式-提示信息相关API" })
@SuppressWarnings("unchecked")
public class ConceptDetailController {
    @Autowired
    private ConceptDetailFacade conceptDetailFacade;

    @ApiOperation(value = "获取提示信息[by:zhaops]",
            notes = "name: 标签名称，必填<br>" +
                    "type：标签类型（1-症状，5-化验，6-辅检，7-诊断，8-药品，9-药品大类，10-不良反应，11-药品小类，12-化验公表项，22-指标，25-手术），单选必填<br>" +
                    "titles：提示信息标题列表，数组选填<br>" +
                    "position：1-摘要，2-全文，5-药品说明书，6-不良反应，单选选填")
    @PostMapping("/getConceptDetail")
    @SysLogger("getConceptDetail")
    public RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        ConceptIntroduceDTO data = conceptDetailFacade.getConceptDetail(conceptIntroduceVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "批量获取提示信息[by:zhaops]",
            notes = "conceptIntorduces:数组，必填<br>" +
                    "name: 标签名称，必填<br>" +
                    "type：标签类型（1-症状，5-化验，6-辅检，7-诊断，8-药品，9-药品大类，10-不良反应，11-药品小类，12-化验公表项，22-指标，25-手术），单选必填<br>" +
                    "titles：提示信息标题列表，数组选填<br>" +
                    "position：1-摘要，2-全文，5-药品说明书，6-不良反应，单选选填")
    @PostMapping("/getConceptDetails")
    @SysLogger("getConceptDetails")
    public RespDTO<List<ConceptIntroduceDTO>> getConceptDetails(@Valid @RequestBody ConceptIntorducesVO conceptIntorducesVO) {
        List<ConceptIntroduceDTO> data = conceptDetailFacade.getConceptDetails(conceptIntorducesVO.getConceptIntorduces());
        return RespDTO.onSuc(data);
    }
}