package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.PushFacade;
import com.diagbot.vo.PushVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/13 18:00
 */
@RestController
@RequestMapping("/push")
@Api(value = "推理相关API", tags = { "数据服务模式-推理相关API" })
@SuppressWarnings("unchecked")
public class PushController {
    @Autowired
    PushFacade pushFacade;

    @ApiOperation(value = "数据服务模式-推理[by:zhaops]",
            notes = "hosCode： 医院编码<br>" +
                    "featureType：类型(必填多选)，1：症状，4：查体结果，41：查体模板，42：查体指标，5：化验，6：辅检，7：诊断，8：治疗方案，22：病情提示（核心指标、危急值、开单合理项、异常值等）,多项用逗号分开，String<br>" +
                    "ruleType：规则类型，1：危急值提醒  2：开单合理性  3：管理评估  4：不良反应  5：药物推荐  6：异常值，多项用分号隔开,String<br>" +
                    "age： 年龄(必填)<br>" +
                    "sex：性别(必填)，1：男，2：女，Integer<br>" +
                    "chief：主诉,String<br>" +
                    "symptom：现病史,String<br>" +
                    "vital：查体,String<br>" +
                    "pasts：既往史,String<br>" +
                    "infectious：传染病史,String<br>" +
                    "operation：手术外伤史,String<br>" +
                    "allergy：过敏史,String<br>" +
                    "vaccination：接种史,String<br>" +
                    "personal：个人史,String<br>" +
                    "marital：婚育史,String<br>" +
                    "family：家族史,String<br>" +
                    "menstrual：月经史,String<br>" +
                    "other：其它史,String<br>" +
                    "lisString：化验文本输入,String<br>" +
                    "pacsString：辅检文本输入,String<br>" +
                    "diagString：诊断文本输入,String<br>" +
                    "drugString：药品文本输入,String<br>" +
                    "lis：化验,ArrayList<br>" +
                    "lis.name：套餐名称(必填),String<br>" +
                    "lis.detailName：化验项名称(必填),String<br>" +
                    "lis.uniqueName：公表名称,String<br>" +
                    "lis.value：值,Double<br>" +
                    "lis.otherValue：其他结果，包括阴性阳性，文字描述等,String<br>" +
                    "lis：units：单位,String<br>" +
                    "lis.minValue：最小值,Double<br>" +
                    "lis.maxValue：最大值,Double<br>" +
                    "lis.result：HIS比较结果,String<br>" +
                    "lis.dateValue：结果时间,String<br>" +
                    "lis：source：数据来源（必填）,0：内部来源，1：外部来源,Integer<br>" +
                    "pacs：辅检,ArrayList<br>" +
                    "pacs.name：辅检项目(外部),String<br>" +
                    "pacs.uniqueName：公表名辅检项目（内部）,String<br>" +
                    "pacs.desc：检查描述（描述信息）,String<br>" +
                    "pacs.result：检查结果（非描述信息）,String<br>" +
                    "pacs.dateValue：结果时间,String<br>" +
                    "diag：诊断,ArrayList<br>" +
                    "item.name：外部项目名称（诊断、药品等）,String<br>" +
                    "item.uniqueName：内部项目名称（诊断、药品等）,String<br>" +
                    "item.dateValue：结果时间，String<br>" +
                    "lisOrder：当前化验开单项，参考lis结构<br>" +
                    "pacsOrder：当前辅检开单项，参考pacs结构<br>" +
                    "diagOrder：当前诊断开单项，参考item结构<br>" +
                    "drugOrder：当前药品开单项，参考item结构<br>" +
                    "operationOrder：当前手术开单项，参考item结构<br>" +
                    "otherOrder：其他开单项，参考item结构<br>" +
                    "diseaseName：选中诊断，参考item结构<br>" +
                    "disType：诊断类型，1-慢病，2-急诊，普通病不填,Integer<br>")
    @PostMapping("/push")
    @SysLogger("push")
    public RespDTO<PushDTO> push(@RequestBody @Valid PushVO pushVO) {
        return RespDTO.onSuc(pushFacade.push(pushVO));
    }


    @ApiOperation(value = "数据服务模式-量表[by:zhaops]",
            notes = "hosCode： 医院编码<br>" +
                    "featureType：类型(必填多选)，1：症状，4：查体结果，41：查体模板，42：查体指标，5：化验，6：辅检，7：诊断，8：治疗方案，22：病情提示（核心指标、危急值、开单合理项、异常值等）,多项用逗号分开，String<br>" +
                    "ruleType：规则类型，1：危急值提醒  2：开单合理性  3：管理评估  4：不良反应  5：药物推荐  6：异常值，多项用分号隔开,String<br>" +
                    "age： 年龄(必填)<br>" +
                    "sex：性别(必填)，1：男，2：女，Integer<br>" +
                    "chief：主诉,String<br>" +
                    "symptom：现病史,String<br>" +
                    "vital：查体,String<br>" +
                    "pasts：既往史,String<br>" +
                    "infectious：传染病史,String<br>" +
                    "operation：手术外伤史,String<br>" +
                    "allergy：过敏史,String<br>" +
                    "vaccination：接种史,String<br>" +
                    "personal：个人史,String<br>" +
                    "marital：婚育史,String<br>" +
                    "family：家族史,String<br>" +
                    "menstrual：月经史,String<br>" +
                    "other：其它史,String<br>" +
                    "lisString：化验文本输入,String<br>" +
                    "pacsString：辅检文本输入,String<br>" +
                    "diagString：诊断文本输入,String<br>" +
                    "drugString：药品文本输入,String<br>" +
                    "lis：化验,ArrayList<br>" +
                    "lis.name：套餐名称(必填),String<br>" +
                    "lis.detailName：化验项名称(必填),String<br>" +
                    "lis.uniqueName：公表名称,String<br>" +
                    "lis.value：值,Double<br>" +
                    "lis.otherValue：其他结果，包括阴性阳性，文字描述等,String<br>" +
                    "lis：units：单位,String<br>" +
                    "lis.minValue：最小值,Double<br>" +
                    "lis.maxValue：最大值,Double<br>" +
                    "lis.result：HIS比较结果,String<br>" +
                    "lis.dateValue：结果时间,String<br>" +
                    "lis：source：数据来源（必填）,0：内部来源，1：外部来源,Integer<br>" +
                    "pacs：辅检,ArrayList<br>" +
                    "pacs.name：辅检项目(外部),String<br>" +
                    "pacs.uniqueName：公表名辅检项目（内部）,String<br>" +
                    "pacs.desc：检查描述（描述信息）,String<br>" +
                    "pacs.result：检查结果（非描述信息）,String<br>" +
                    "pacs.dateValue：结果时间,String<br>" +
                    "diag：诊断,ArrayList<br>" +
                    "item.name：外部项目名称（诊断、药品等）,String<br>" +
                    "item.uniqueName：内部项目名称（诊断、药品等）,String<br>" +
                    "item.dateValue：结果时间，String<br>" +
                    "lisOrder：当前化验开单项，参考lis结构<br>" +
                    "pacsOrder：当前辅检开单项，参考pacs结构<br>" +
                    "diagOrder：当前诊断开单项，参考item结构<br>" +
                    "drugOrder：当前药品开单项，参考item结构<br>" +
                    "operationOrder：当前手术开单项，参考item结构<br>" +
                    "otherOrder：其他开单项，参考item结构<br>" +
                    "diseaseName：选中诊断，参考item结构<br>" +
                    "disType：诊断类型，1-慢病，2-急诊，普通病不填,Integer<br>" +
                    "scaleName：量表名称,String<br>")
    @PostMapping("/pushScale")
    @SysLogger("pushScale")
    public RespDTO<Map<String, Object>> pushScale(@RequestBody @Valid PushVO pushVO) {
        return RespDTO.onSuc(pushFacade.pushScale(pushVO));
    }
}