package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.facade.SearchFacade;
import com.diagbot.vo.GetStaticVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author: gaodm
 * @time: 2019/5/14 9:51
 */
@RestController
@RequestMapping("/search")
@Api(value = "检索API", tags = { "数据服务模式-检索API" })
@SuppressWarnings("unchecked")
public class SearchController {
    @Autowired
    private SearchFacade searchFacade;

    /**
     * 静态知识检索
     *
     * @param getStaticVO
     * @return
     */
    @ApiOperation(value = "静态知识检索[by:zhoutg]",
            notes = "inputStr：检索内容，必填<br>" +
                    "types：指定类型，1：症状，10：药品，12：化验，16：辅检，18：诊断，25：手术，48：量表； 不指定：以上全部")
    @PostMapping("/getStaticKnowledge")
    @SysLogger("getStaticKnowledge")
    public RespDTO<List<RetrievalDTO>> getStaticKnowledge(@Valid @RequestBody GetStaticVO getStaticVO){
        return RespDTO.onSuc(searchFacade.getStaticKnowledge(getStaticVO));
    }

    /**
     * 静态知识检索【海南项目】
     *
     * @param getStaticVO
     * @return
     */
    @ApiOperation(value = "静态知识检索【海南项目】[by:zhoutg]",
            notes = "inputStr：检索内容，必填<br>" +
                    "types：指定类型，1：症状，10：药品，12：化验，16：辅检，18：诊断，25：手术，33：体征，" +
                    "72：诊疗指南（门诊）,73：诊疗指南（住院）,74：护理,75：临床路径,76：血液,77：麻醉,78：监护," +
                    "79：中医,80：入科流程,81：审查规则,82：电子书,83：视频， 不指定：以上全部")
    @PostMapping("/getStaticKnowledgeHaiNan")
    @SysLogger("getStaticKnowledgeHaiNan")
    public RespDTO<List<RetrievalDTO>> getStaticKnowledgeHaiNan(@Valid @RequestBody GetStaticVO getStaticVO){
        return RespDTO.onSuc(searchFacade.getStaticKnowledgeHaiNan(getStaticVO));
    }

//    /**
//     * 量表搜索
//     *
//     * @param scaleIndexVO
//     * @return
//     */
//    @ApiOperation(value = "量表搜索[by:zhoutg]",
//            notes = "name：检索内容，必填<br>" +
//                    "filterName：需要过滤的名称")
//    @PostMapping(value = "/getScale")
//    public RespDTO<List<ScaleIndexDTO>> index(@Valid @RequestBody ScaleIndexVO scaleIndexVO){
//        return RespDTO.onSuc(searchFacade.index(scaleIndexVO));
//    }
}
