package com.diagbot.util;

import com.diagbot.entity.IpEntity;
import org.lionsoul.ip2region.DataBlock;
import org.lionsoul.ip2region.DbConfig;
import org.lionsoul.ip2region.DbSearcher;
import org.lionsoul.ip2region.Util;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.util.FileCopyUtils;

import java.io.InputStream;
import java.lang.reflect.Method;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/3/24 18:29
 */
public class IPUtil {

    private static DbSearcher searcher;

    /**
     *  获取DbSearcher
     * @return
     */
    public static DbSearcher getInstance () {
        if (searcher != null) {
            return searcher;
        }
        try {
            DbConfig config = new DbConfig();
            Resource resource = new ClassPathResource("ip2region.db"); // 远程获取资源文件
            InputStream inputStream = resource.getInputStream();
            searcher = new DbSearcher(config, FileCopyUtils.copyToByteArray(inputStream));
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("获取DbSearcher错误");
        }
        return searcher;
    }


    //国家，区域，省份，城市，运营商
    public  static String getCityInfo(String ip) {
        //查询算法
        int algorithm = DbSearcher.MEMORY_ALGORITYM;
         //DbSearcher.BTREE_ALGORITHM; //B-tree
        //DbSearcher.BINARY_ALGORITHM //Binary
        //DbSearcher.MEMORY_ALGORITYM //Memory
        try {
            // 初始化
            getInstance();

            //define the method
            Method method = null;
            switch (algorithm) {
                case DbSearcher.BTREE_ALGORITHM:
                    method = searcher.getClass().getMethod("btreeSearch", String.class);
                    break;
                case DbSearcher.BINARY_ALGORITHM:
                    method = searcher.getClass().getMethod("binarySearch", String.class);
                    break;
                case DbSearcher.MEMORY_ALGORITYM:
                    method = searcher.getClass().getMethod("memorySearch", String.class);
                    break;
            }

            DataBlock dataBlock = null;
            if (Util.isIpAddress(ip) == false) {
                System.out.println("Error: Invalid ip address");
            }

            dataBlock = (DataBlock) method.invoke(searcher, ip);

            return dataBlock.getRegion();

        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("Ip解析错误");
        }
        return null;
    }


    /**
     * 返回ip实体类
     *
     * @param ip
     * @return ip实体类
     */
    public static IpEntity getCityInfoWithEntity(String ip) {
        String ipStr = getCityInfo(ip);
        String[] arr = ipStr.split("\\|");
        if (arr != null && arr.length == 5) {
            return new IpEntity(arr[0], arr[1], arr[2], arr[3], arr[4]);
        }
        return new IpEntity();
    }

    public static void main(String[] args) {
        System.out.println(getCityInfo("223.93.170.82"));
        System.out.println(getCityInfo("223.93.170.82"));
    }
}