use `qc`;
-- ----------------------------
-- Table structure for std_behospital_info
-- ----------------------------
DROP TABLE IF EXISTS `std_behospital_info`;
CREATE TABLE `std_behospital_info` (
  `behospital_code` varchar(16) NOT NULL COMMENT '病人住院ID',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '医院ID',
  `name` varchar(32) NOT NULL COMMENT '姓名',
  `sex` varchar(2) DEFAULT NULL COMMENT '性别（男，女）',
  `birthday` datetime DEFAULT NULL COMMENT '出生日期',
  `file_code` varchar(16) DEFAULT NULL COMMENT '档案号',
  `qc_type_id` bigint(20) DEFAULT '0' COMMENT '质控类型',
  `ward_code` varchar(16) DEFAULT NULL COMMENT '病区编码',
  `ward_name` varchar(32) DEFAULT NULL COMMENT '病区名称',
  `beh_dept_id` varchar(16) NOT NULL COMMENT '住院科室ID',
  `beh_dept_name` varchar(64) NOT NULL COMMENT '住院科室名称',
  `bed_code` varchar(12) DEFAULT NULL COMMENT '床位号',
  `bed_name` varchar(12) DEFAULT NULL COMMENT '床位名称',
  `insurance_name` varchar(50) DEFAULT NULL COMMENT '医保类别',
  `job_type` varchar(30) DEFAULT NULL COMMENT '职业',
  `behospital_date` datetime DEFAULT NULL COMMENT '入院时间',
  `leave_hospital_date` datetime DEFAULT NULL COMMENT '出院时间',
  `diagnose_icd` varchar(50) DEFAULT NULL COMMENT '疾病ICD编码',
  `diagnose` text COMMENT '疾病名称',
  `doctor_id` varchar(16) DEFAULT '' COMMENT '主治医生ID',
  `doctor_name` varchar(50) DEFAULT '' COMMENT '主治医生姓名',
  `beh_doctor_id` varchar(16) DEFAULT '' COMMENT '住院医生ID',
  `beh_doctor_name` varchar(50) DEFAULT '' COMMENT '住院医生姓名',
  `director_doctor_id` varchar(16) DEFAULT '' COMMENT '主任医生ID',
  `director_doctor_name` varchar(50) DEFAULT '' COMMENT '主任医生姓名',
  `is_placefile` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否归档（0：未归档，1：已归档）',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  PRIMARY KEY (`behospital_code`),
  KEY `idx_hospital_id_behospital_code` (`hospital_id`,`behospital_code`) USING BTREE,
  KEY `idx_leave_hospital_date` (`leave_hospital_date`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='标准住院病历信息';

-- ----------------------------
-- Table structure for std_qcresult_detail
-- ----------------------------
DROP TABLE IF EXISTS `std_qcresult_detail`;
CREATE TABLE `std_qcresult_detail` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '医院ID',
  `behospital_code` varchar(255) NOT NULL DEFAULT '' COMMENT '病人住院ID',
  `cases_id` bigint(20) DEFAULT NULL COMMENT '模块id',
  `cases_score` decimal(5,1) DEFAULT NULL COMMENT '模块总分',
  `cases_entry_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '条目ID',
  `score` decimal(5,1) DEFAULT NULL COMMENT '人工修改分值',
  `msg` varchar(255) DEFAULT NULL COMMENT '提示信息',
  `info` varchar(500) DEFAULT '' COMMENT '质控返回提示信息',
  `is_reject` int(3) DEFAULT '0' COMMENT '单项否决（1-单项否决 0-非）',
  `grade_type` tinyint(4) DEFAULT '1' COMMENT '初始类型(1：机器，2：人工)',
  `opt_type` tinyint(4) DEFAULT '1' COMMENT '操作类型(1：新增，2：删除，3：修改)',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  `satisfy` int(10) DEFAULT NULL COMMENT '是否满足',
  PRIMARY KEY (`id`),
  KEY `idx_hospital_id` (`hospital_id`) USING BTREE,
  KEY `idx_behospital_code` (`behospital_code`) USING BTREE,
  KEY `idx_hospital_id_behospital_code` (`hospital_id`,`behospital_code`) USING BTREE,
  KEY `idx_is_deleted` (`is_deleted`) USING BTREE,
  KEY `idx_msg` (`msg`) USING BTREE,
  KEY `idx_mix` (`hospital_id`,`behospital_code`,`is_deleted`) USING BTREE,
  KEY `idx_cases_entry_id` (`cases_entry_id`) USING BTREE,
  KEY `idx_cases_id` (`cases_id`) USING BTREE
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COMMENT='标准质控评分明细信息';

DROP TABLE IF EXISTS `std_mission_info`;
CREATE TABLE `std_mission_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `mission_name` varchar(20) DEFAULT NULL,
  `status` int(11) DEFAULT '0',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COMMENT='任务信息表';

DROP TABLE IF EXISTS `std_mission_detail`;
CREATE TABLE `std_mission_detail` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `mission_id` bigint(20) DEFAULT NULL,
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '医院ID',
  `cases_id` bigint(20) DEFAULT NULL COMMENT '模块id',
  `cases_name` varchar(255) DEFAULT NULL,
  `entry_code` varchar(255) NOT NULL DEFAULT '0' COMMENT '条目code',
  `entry_name` varchar(255) DEFAULT NULL COMMENT '提示信息',
  `amount` int(11) DEFAULT NULL,
  `behospitalCodeStr` varchar(255) DEFAULT NULL,
  `status` int(11) DEFAULT '0',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COMMENT='任务信息详情表';