package com.diagbot.facade;

import com.diagbot.dto.RespDTO;
import com.diagbot.dto.ServiceFilterDTO;
import com.diagbot.dto.ServiceFiltersDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: 产品服务过滤
 * @Author: ztg
 * @Date: 2018/9/18 14:45
 */
@Component
@Slf4j
public class ServiceFilterFacade {
    private static final String SERVICEFILTER = "ServiceFilter";

    @Autowired
    private RestTemplate restTemplate;

    /**
     * 获取需要appkey和secret的服务，未用
     *
     * @return 服务列表
     */
    @Cacheable(value = SERVICEFILTER, key = "'ServiceFilter'")
    public ServiceFilterDTO getServiceFilter() {
        ServiceFilterDTO serviceFilterDTO = new ServiceFilterDTO();
        ResponseEntity<RespDTO<List<ServiceFiltersDTO>>> respDTOResponseEntity
                = restTemplate.exchange("http://diagbotman-service/service_filter/getAll", HttpMethod.POST,
                null, new ParameterizedTypeReference<RespDTO<List<ServiceFiltersDTO>>>() {
                });
        RespDTO<List<ServiceFiltersDTO>> resFilter = respDTOResponseEntity.getBody();
        if (resFilter != null && RespDTO.TRUE_CODE.equals(resFilter.code)) {
            Set<String> apiMap = resFilter.data.stream().distinct().map(ServiceFiltersDTO::getName).collect(Collectors.toSet());
            Map<Long, String> productMap = resFilter.data.stream().collect(Collectors.toMap(ServiceFiltersDTO::getProductId, ServiceFiltersDTO::getName));
            serviceFilterDTO.setApiMap(apiMap);
            serviceFilterDTO.setProductMap(productMap);
            log.info("读取了拦截产品线缓存！");
        }
        return serviceFilterDTO;
    }
}
