package com.diagbot.aggregate;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.biz.push.entity.Treat;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.ConceptBaseVO;
import com.diagbot.client.bean.ConceptExistVO;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptPushDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.QuestionPushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.DeptVital;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.enums.TagTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.facade.DeptVitalFacade;
import com.diagbot.facade.QuestionFacade;
import com.diagbot.facade.TreatmentFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.QuestionIds2VO;
import com.diagbot.vo.QuestionVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/10/21 14:59
 */
@Component
public class PushItemAggregate {
    @Autowired
    private QuestionFacade questionFacade;
    @Autowired
    private AiptServiceClient aiptServiceClient;
    @Autowired
    private DeptVitalFacade deptVitalFacade;
    @Autowired
    private TreatmentFacade treatmentFacade;

    @DataProvider("setICSSAll")
    public PushDTO setICSSAll(
            @InvokeParameter("pushDTO") PushDTO pushDTO,
            @DataConsumer("setICSSSymptom") List<ConceptPushDTO> symptom,
            @DataConsumer("setICSSVitalIndex") List<ConceptPushDTO> vital,
            @DataConsumer("setICSSVitalModule") List<QuestionPushDTO> vitalModule,
            @DataConsumer("setICSSLab") List<QuestionPushDTO> lab,
            @DataConsumer("setICSSTreat") Treat treat) {

        if (ListUtil.isNotEmpty(symptom)) {
            pushDTO.setSymptom(symptom);
        }
        if (ListUtil.isNotEmpty(vital)) {
            pushDTO.setVital(vital);
            List<Long> vitalIds = vital
                    .stream()
                    .map(vitalItem -> vitalItem.getId())
                    .filter(vitalId -> vitalId != null)
                    .collect(Collectors.toList());
            pushDTO.setVitalIds(vitalIds);
        }
        if (ListUtil.isNotEmpty(vitalModule)) {
            pushDTO.setModuleVital(vitalModule);
        }
        if (ListUtil.isNotEmpty(lab)) {
            pushDTO.setLab(lab);
        }
        if (treat != null) {
            pushDTO.setTreat(treat);
        }
        return pushDTO;
    }

    @DataProvider("setICSSSymptom")
    public List<ConceptPushDTO> setSymptom(@InvokeParameter("featureTypeSet") Set<String> featureTypeSet,
                                           @InvokeParameter("data") PushDTO data) {
        List<ConceptPushDTO> symptomRes = ListUtil.newArrayList();
        //症状 概念列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Symptom.getKey()))) {
            symptomRes = data.getSymptom();
            if (ListUtil.isNotEmpty(symptomRes)) {
                symptomRes = addQuestionId(symptomRes, QuestionTypeEnum.Symptom.getKey());
            }
        }
        return symptomRes;
    }

    @DataProvider("setICSSVitalIndex")
    public List<ConceptPushDTO> setVitalIndex(@InvokeParameter("featureTypeSet") Set<String> featureTypeSet,
                                              @InvokeParameter("data") PushDTO data) {
        List<ConceptPushDTO> vitalRes = ListUtil.newArrayList();
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Vital_Index.getKey()))) {
            vitalRes = data.getVital();
            if (ListUtil.isNotEmpty(vitalRes)) {
                vitalRes = addQuestionId(vitalRes, QuestionTypeEnum.Vital.getKey());
            }
        }
        return vitalRes;
    }

    @DataProvider("setICSSVitalModule")
    public List<QuestionPushDTO> setVitalModule(@InvokeParameter("featureTypeSet") Set<String> featureTypeSet,
                                                @InvokeParameter("data") PushDTO data,
                                                @InvokeParameter("pushVO") PushVO pushVO) {
        List<QuestionPushDTO> moduleVitalDTO = Lists.newLinkedList();
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Vital_Module.getKey()))) {
            ConceptPushDTO dept = data.getDept();
            moduleVitalDTO = getVitalModule(dept, pushVO);
        }
        return moduleVitalDTO;
    }

    @DataProvider("setICSSLab")
    public List<QuestionPushDTO> setLab(@InvokeParameter("featureTypeSet") Set<String> featureTypeSet,
                                        @InvokeParameter("data") PushDTO data,
                                        @InvokeParameter("pushVO") PushVO pushVO) {
        List<QuestionPushDTO> labs = Lists.newLinkedList();
        List<QuestionPushDTO> lisDTO = Lists.newLinkedList();
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Lis.getKey()))) {
            labs = data.getLab();
            if (ListUtil.isNotEmpty(labs)) {
                Map<String, Long> lisMap = new LinkedHashMap<>();
                List<String> mealNameList = labs
                        .stream()
                        .map(lab -> lab.getName())
                        .collect(Collectors.toList());
                Map<String, QuestionDTO> questionDTOMap
                        = addLisQuestion(mealNameList, pushVO.getAge(), pushVO.getSex());
                for (String name : mealNameList) {
                    QuestionPushDTO questionPushDTO = new QuestionPushDTO();
                    QuestionDTO questionDTO = questionDTOMap.get(name);
                    if (questionDTO != null) {
                        BeanUtil.copyProperties(questionDTO, questionPushDTO);
                    } else {
                        questionPushDTO.setName(name);
                        questionPushDTO.setType(ConceptTypeEnum.Lis.getKey());
                    }
                    questionPushDTO.setLibType(12);
                    if (lisMap.containsKey(name) && lisMap.get(name) != null) {
                        questionPushDTO.setConceptId(lisMap.get(name));
                    }
                    lisDTO.add(questionPushDTO);
                }
            }
        }
        return lisDTO;
    }

    @DataProvider("setICSSTreat")
    public Treat setICSSTreat(@InvokeParameter("featureTypeSet") Set<String> featureTypeSet,
                              @InvokeParameter("data") PushDTO data,
                              @InvokeParameter("pushVO") PushVO pushVO) {
        Treat treat = data.getTreat();
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Treat.getKey()))) {
            treat = treatmentFacade.getTreatment(treat, pushVO.getPatientId());
        }
        return treat;
    }

    /**
     * 概念关联标签，添加标签id ——症状
     *
     * @param concepts
     * @param type
     * @return
     */
    public List<ConceptPushDTO> addQuestionId(List<ConceptPushDTO> concepts, Integer type) {
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
        List<String> nameList = concepts.stream()
                .map(concept -> concept.getName())
                .collect(Collectors.toList());
        questionInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("type", type)
                .ne("tag_type", TagTypeEnum.T8.getKey());
        //化验填写单匹配name
        if (ListUtil.isNotEmpty(nameList)) {
            if (type.equals(ConceptTypeEnum.Lis.getKey())) {
                questionInfoQueryWrapper.in("name", nameList);
            } else {
                questionInfoQueryWrapper.in("tag_name", nameList);
            }
        }
        List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);
        if (ListUtil.isNotEmpty(questionInfoList)) {
            Map<String, QuestionInfo> questionInfoMap
                    = EntityUtil.makeEntityMap(questionInfoList, "tagName");
            for (ConceptPushDTO concept : concepts) {
                if (questionInfoMap.containsKey(concept.getName())) {
                    concept.setId(questionInfoMap.get(concept.getName()).getId());
                }
            }
        }
        return concepts;
    }

    /**
     * 获取查体模板
     *
     * @param dept
     * @param pushVO
     * @return
     */
    public List<QuestionPushDTO> getVitalModule(ConceptPushDTO dept, PushVO pushVO) {
        List<QuestionPushDTO> vitalDTO = Lists.newLinkedList();
        if (dept.getConceptId() == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "全科信息未维护");
        }
        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("dept_id", dept.getConceptId())
                .orderByAsc("order_no");
        List<DeptVital> deptVitalList = deptVitalFacade.list(deptVitalQueryWrapper);
        //科室模板为空时，取全科模板
        if (deptVitalList.size() == 0 && dept.getName().equals("全科") == false) {
            ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
            conceptBaseVO.setName("全科");
            conceptBaseVO.setLibType(ConceptTypeEnum.DEPARTMENT.getKey());
            RespDTO<ConceptBaseDTO> res = aiptServiceClient.existConcept(conceptBaseVO);
            RespDTOUtil.respNGDeal(res, "全科信息未维护");
            BeanUtil.copyProperties(res.data, dept);
            deptVitalQueryWrapper = new QueryWrapper<>();
            deptVitalQueryWrapper
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("dept_id", dept.getConceptId())
                    .orderByAsc("order_no");
            deptVitalList = deptVitalFacade.list(deptVitalQueryWrapper);
        }
        //标签列表
        for (DeptVital deptVital : deptVitalList) {
            QuestionVO questionVO = new QuestionVO();
            QuestionPushDTO questionPushDTO = new QuestionPushDTO();
            questionVO.setAge(pushVO.getAge());
            questionVO.setSexType(pushVO.getSex());
            questionVO.setId(deptVital.getVitalId());
            QuestionDTO questionDTO = questionFacade.getById(questionVO);
            if (questionDTO != null && questionDTO.getId() != null) {
                BeanUtil.copyProperties(questionDTO, questionPushDTO);
                questionPushDTO.setLibType(ConceptTypeEnum.Vital.getKey());
                vitalDTO.add(questionPushDTO);
            }
        }
        //增加概念id
        if (ListUtil.isNotEmpty(vitalDTO)) {
            ConceptExistVO conceptExistVO = new ConceptExistVO();
            conceptExistVO.setType(ConceptTypeEnum.Vital.getKey());
            List<String> vitalNames = vitalDTO
                    .stream()
                    .map(vital -> vital.getTagName())
                    .collect(Collectors.toList());
            conceptExistVO.setNameList(vitalNames);
            RespDTO<Map<String, Long>> respDTO = aiptServiceClient.getConceptMapByNameAndType(conceptExistVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                Map<String, Long> vitalMap = respDTO.data;
                for (QuestionPushDTO questionPushDTO : vitalDTO) {
                    if (vitalMap.get(questionPushDTO.getTagName()) != null) {
                        questionPushDTO.setConceptId(vitalMap.get(questionPushDTO.getTagName()));
                    }
                }
            }
        }
        return vitalDTO;
    }

    /**
     * 获取化验填写单
     *
     * @param nameList
     * @param age
     * @param sex
     * @return
     */
    public Map<String, QuestionDTO> addLisQuestion(List<String> nameList, Integer age, Integer sex) {
        Map<String, QuestionDTO> mapDTO = new LinkedHashMap<>();
        if (ListUtil.isEmpty(nameList)) {
            return mapDTO;
        }
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
        questionInfoQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("name", nameList)
                .eq("tag_type", TagTypeEnum.T7.getKey());
        List<QuestionInfo> lisList = questionFacade.list(questionInfoQueryWrapper);
        if (ListUtil.isNotEmpty(lisList)) {
            List<Long> lisIds = lisList
                    .stream()
                    .map(lis -> lis.getId())
                    .collect(Collectors.toList());
            QuestionIds2VO questionIds2VO = new QuestionIds2VO();
            questionIds2VO.setAge(age);
            questionIds2VO.setSexType(sex);
            questionIds2VO.setIds(lisIds);
            Map<Long, Object> questionDTOMap = questionFacade.getByIds(questionIds2VO);
            for (Map.Entry<Long, Object> entry : questionDTOMap.entrySet()) {
                QuestionDTO questionDTO = new QuestionDTO();
                BeanUtil.copyProperties(entry.getValue(), questionDTO);
                mapDTO.put(questionDTO.getName(), questionDTO);
            }
        }
        return mapDTO;
    }
}