package com.diagbot.client;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.ConceptBaseVO;
import com.diagbot.client.bean.ConceptExistVO;
import com.diagbot.client.bean.DisTypeVO;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.DeptInfoDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.DisScaleDTO;
import com.diagbot.dto.DisTypeDTO;
import com.diagbot.dto.DisclaimerInformationDTO;
import com.diagbot.dto.DiseaseIcdDTO;
import com.diagbot.dto.EvaluationDTO;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.IndexConfigAndDataDTO;
import com.diagbot.dto.IndexConfigDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.ScaleIndexDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.entity.ScaleContent;
import com.diagbot.vo.ConceptFindVO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.DeptInfoVO;
import com.diagbot.vo.DisScaleVO;
import com.diagbot.vo.DisclaimerInformationVO;
import com.diagbot.vo.ExistListByConceptIdsVO;
import com.diagbot.vo.GetDiseaseIcdVO;
import com.diagbot.vo.GetLisMappingVO;
import com.diagbot.vo.GetStaticKnowledgeVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.IndexConfigdiseaseNameVO;
import com.diagbot.vo.IndexDataFindVO;
import com.diagbot.vo.IndexDataSaveVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.ScaleIndexVO;
import com.diagbot.vo.VersionVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:调用中间层服务
 * @Author: Mark Huang
 * @time: 2019/03/25 19:15
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {
    /**
     * 计算接口（量表、计算公式）
     *
     * @param calculateData
     * @return
     */
    @PostMapping(value = "/clinicaldata/scaleCalc")
    RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData);

    /**
     * 获取量表全文接口
     *
     * @param searchVO
     * @return
     */
    @PostMapping(value = "/clinicaldata/getScale")
    RespDTO<List<ScaleContent>> getScale(@RequestBody SearchVO searchVO);

    /**
     * 症状特征提取
     *
     * @param text
     * @return
     */
    @PostMapping(value = "/feature/symptomFeature")
    RespDTO<List<FeatureConceptDTO>> symptomFeature(@RequestParam("text") String text);


    /**
     * 推理
     *
     * @param searchVO
     * @return
     */
    @PostMapping("/push/pushInner")
    RespDTO<PushDTO> pushInner(@RequestBody SearchVO searchVO);


//    @PostMapping(value = "/scale/getList")
//    RespDTO<List<DisScaleDTO>> getList(@RequestBody DisScaleVO scaleVO);
//
//    @PostMapping(value = "/scale/index")
//    RespDTO<List<ScaleIndexDTO>> index(@RequestBody ScaleIndexVO scaleIndexVO);

    @PostMapping(value = "/disType/getDisType")
    RespDTO<DisTypeDTO> getDisType();


    @PostMapping(value = "/conceptDetail/getConceptDetail")
    RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO);

    @PostMapping(value = "/concept/exist")
    RespDTO<ConceptBaseDTO> existConcept(@RequestBody @Valid ConceptBaseVO conceptBaseVO);

    @PostMapping(value = "/concept/getConceptUsual")
    RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    @PostMapping(value = "/deptInfo/getDeptInfo")
    RespDTO<DeptInfoDTO> getDeptInfo(@Valid @RequestBody DeptInfoVO deptInfoVO);

    /**
     * 根据医院编号疾病概念名称获取各医院对应的诊断icd
     *
     * @param getDiseaseIcdVO
     * @return
     */
    @PostMapping("/diseaseIcd/getDiseaseIcds")
    RespDTO<List<DiseaseIcdDTO>> getDiseaseIcds(@RequestBody GetDiseaseIcdVO getDiseaseIcdVO);

    /**
     * 获取页面顶部病人医生科室信息
     *
     * @param getTopPatientInfoVO
     * @return
     */
    @PostMapping("/patientInfo/getTopPatientInfo")
    RespDTO<GetTopPatientInfoDTO> getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO);

    /**
     * 获取管理评估模板
     *
     * @param searchVO
     * @return
     */
    @PostMapping("/evaluation/getEvaluationModules")
    RespDTO<List<EvaluationDTO>> getEvaluationModules(@RequestBody SearchVO searchVO);

    /**
     * 获取指标配置
     *
     * @param indexConfigdiseaseNameVO
     * @return
     */
    @PostMapping("/indexConfig/getindexConfigs")
    RespDTO<List<IndexConfigDTO>> getindexConfigs(@RequestBody IndexConfigdiseaseNameVO indexConfigdiseaseNameVO);

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @PostMapping("/concept/getConceptMap")
    RespDTO<Map<Long, String>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO);


    /**
     * 根据概念Id,年龄,性别列表获取概念列表Map
     *
     * @param conceptFindVO 搜索参数
     * @return 术语id和术语 Map
     */
    @PostMapping("/concept/getConceptCondMap")
    RespDTO<Map<Long, String>> getConceptCondMap(@RequestBody ConceptFindVO conceptFindVO);


    /**
     * 是否有提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    @PostMapping("/conceptDetail/exist")
    RespDTO<Boolean> existConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO);

    /**
     * 验证诊断类型
     *
     * @param disTypeVO
     * @return
     */
    @PostMapping("/disType/verifyDisType")
    RespDTO<Boolean> verifyDisType(@Valid @RequestBody DisTypeVO disTypeVO);

    /**
     * 根据名称和类型获取概念列表Map
     *
     * @param conceptExistVO
     * @return
     */
    @PostMapping("/concept/getConceptMapByNameAndType")
    RespDTO<Map<String, Long>> getConceptMapByNameAndType(@RequestBody ConceptExistVO conceptExistVO);

    /**
     * 标签检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfo")
    RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@RequestBody @Valid RetrievalVO retrievalVO);

    /**
     * @param indexDataFindVO
     * @return
     */
    @PostMapping(value = "/indexData/getIndexDatas")
    RespDTO<IndexConfigAndDataDTO> getIndexDatas(@RequestBody IndexDataFindVO indexDataFindVO);

    /**
     * @param indexDataSaveVO
     * @return
     */
    @PostMapping(value = "/indexData/saveIndexDatas")
    RespDTO<Boolean> saveIndexDatas(@RequestBody IndexDataSaveVO indexDataSaveVO);

    @PostMapping("/disclaimerInformation/getDisclaimerInformations")
    RespDTO<List<DisclaimerInformationDTO>> getDisclaimerInformations(@RequestBody DisclaimerInformationVO disclaimerInformationVO);

    /**
     * 验证是否存在静态知识
     *
     * @param existListByConceptIdsVO
     * @return
     */
    @PostMapping("/conceptDetail/existListByConceptIds")
    RespDTO<List<Long>> existListByConceptIds(ExistListByConceptIdsVO existListByConceptIdsVO);

    /**
     * 静态知识检索
     *
     * @param getStaticKnowledgeVO
     * @return
     */
    @PostMapping("/concept/getStaticKnowledge")
    RespDTO<List<RetrievalDTO>> getStaticKnowledge(GetStaticKnowledgeVO getStaticKnowledgeVO);

    /**
     * 获取版本信息
     */
    @PostMapping("/versionInfo/getVersionInfoAlls")
    RespDTO<VersionWrapperDTO> getVersionInfoAlls(@RequestBody VersionVO versionVO);

    @PostMapping("/lisMapping/getLisMappingByQuestion")
    RespDTO<Map<String, Map<String, String>>> getLisMappingByQuestion(GetLisMappingVO getLisMappingVO);

    /**
     * 批量获取提示信息
     *
     * @param conceptIntroduceVOList
     * @return
     */
    @PostMapping(value = "/conceptDetail/getConceptDetails")
    RespDTO<List<ConceptIntroduceDTO>> getConceptDetails(@Valid @RequestBody List<ConceptIntroduceVO> conceptIntroduceVOList);

    /**
     * 获取字典信息
     *
     * @return
     */
    @PostMapping(value = "/dictionaryInfo/getList")
    RespDTO<Map<Long, List<DictionaryInfoDTO>>> getList();
}