package com.diagbot.client;

import com.diagbot.client.hystrix.TranServiceHystrix;
import com.diagbot.dto.DoctorInfoDTO;
import com.diagbot.dto.GetExportInquirysDTO;
import com.diagbot.dto.GetHospitalRankInfoDTO;
import com.diagbot.dto.HisInquirysForJzDTO;
import com.diagbot.dto.HospitalDeptInfoAllDTO;
import com.diagbot.dto.HospitalInfoDTO;
import com.diagbot.dto.LisConfigDTO;
import com.diagbot.dto.LisDataDTO;
import com.diagbot.dto.LisRetrieveListAllDTO;
import com.diagbot.dto.PacsDataDTO;
import com.diagbot.dto.PacsRetrieveListAllDTO;
import com.diagbot.dto.PatientInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.dto.TranFieldInfoDTO;
import com.diagbot.entity.DoctorInfo;
import com.diagbot.entity.HospitalDept;
import com.diagbot.entity.PatientInfo;
import com.diagbot.vo.AddDiagnoseSecondVO;
import com.diagbot.vo.DoctorInfoVO;
import com.diagbot.vo.GetHospitalRankInfoVO;
import com.diagbot.vo.HisInquirysForJzVO;
import com.diagbot.vo.HosCodeVO;
import com.diagbot.vo.HospitalCodeSetVO;
import com.diagbot.vo.HospitalCodeVo;
import com.diagbot.vo.HospitalInfoVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.LisArgumentsVO;
import com.diagbot.vo.LisGroupNameVO;
import com.diagbot.vo.LisHospitalCodeVO;
import com.diagbot.vo.PacsArgumentsVO;
import com.diagbot.vo.PacsCheckItemVO;
import com.diagbot.vo.PatientInfoVO;
import com.diagbot.vo.SaveInquiryToHisVO;
import com.diagbot.vo.TranFieldInfoVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: 调用信息对接层服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "tran-service", fallback = TranServiceHystrix.class)
public interface TranServiceClient {

    /**
     * 获取医生信息
     *
     * @param doctorInfoVo
     * @return
     */
    @PostMapping("/doctorInfo/getDoctorInfo")
    RespDTO<List<DoctorInfoDTO>> getDoctorInfo(@RequestBody DoctorInfoVO doctorInfoVo);

    /**
     * 根据ids获取医生信息map
     *
     * @param ids
     * @return
     */
    @PostMapping("/doctorInfo/doctorInfoMapByIds")
    RespDTO<Map<Long, DoctorInfo>> doctorInfoMapByIds(@RequestBody List<Long> ids);

    /**
     * 根据id获取医生信息
     *
     * @param id
     * @return
     */
    @PostMapping("/doctorInfo/doctorInfoById")
    RespDTO<DoctorInfoDTO> getDoctorInfoById(@RequestBody Long id);

    /**
     * 根据ids获取科室信息map
     *
     * @param ids
     * @return
     */
    @PostMapping("/hospitalDeptInfo/hospitalDeptInfoMapByIds")
    RespDTO<Map<Long, HospitalDept>> hospitalDeptInfoMapByIds(@RequestBody List<Long> ids);

    /**
     * 获取医院信息
     *
     * @param hospitalInfoVO
     * @return
     */
    @PostMapping("/hospitalInfo/getHospitalInfo")
    RespDTO<HospitalInfoDTO> getHospitalInfo(@RequestBody HospitalInfoVO hospitalInfoVO);

    /**
     * 匹配公表名
     *
     * @param lisHospitalCodeVO
     * @return
     */
    @PostMapping("/tranLisConfig/getLisConfigByhospitalId")
    RespDTO<List<LisConfigDTO>> getLisConfigByhospitalId(@RequestBody LisHospitalCodeVO lisHospitalCodeVO);

    /**
     * 获取患者信息
     *
     * @param patientInfoVO
     * @return
     */
    @PostMapping("/patientInfo/getPatientInfo")
    RespDTO<PatientInfoDTO> getPatientInfo(@RequestBody PatientInfoVO patientInfoVO);

    /**
     * 根据ids获取病人信息
     *
     * @param ids
     * @return
     */
    @PostMapping("/patientInfo/patientInfoMapByIds")
    RespDTO<Map<Long, PatientInfo>> patientInfoMapByIds(@RequestBody List<Long> ids);

    /**
     * 根据uuid获取相关信息
     *
     * @param tranFieldInfoVO
     * @return
     */
    @PostMapping("/tranFieldInfo/getInfoByUuid")
    RespDTO<List<TranFieldInfoDTO>> getInfoByUuid(@RequestBody TranFieldInfoVO tranFieldInfoVO);

    /**
     * 将问诊记录保存一份至对接层
     *
     * @param saveInquiryToHisVO
     * @return
     */
    @PostMapping("/inquiryInfo/saveInquiryToHis")
    RespDTO<Boolean> saveInquiryToHis(@RequestBody SaveInquiryToHisVO saveInquiryToHisVO);

    /**
     * 急诊-历史病历列表
     *
     * @param hisInquirysForJzVO
     * @return
     */
    @PostMapping("/inquiryInfo/hisInquirysForJz")
    RespDTO<HisInquirysForJzDTO> hisInquirysForJz(@RequestBody HisInquirysForJzVO hisInquirysForJzVO);

    /**
     * 急诊-添加二次诊断
     *
     * @param addDiagnoseSecondVO
     * @return
     */
    @PostMapping("/inquiryInfo/addDiagnoseSecond")
    RespDTO<Boolean> addDiagnoseSecond(@RequestBody AddDiagnoseSecondVO addDiagnoseSecondVO);

    /**
     * 急诊-获取需要导出的问诊数据
     *
     * @param inquiryIds
     * @return
     */
    @PostMapping("/inquiryInfo/getExportInquirys")
    RespDTO<List<GetExportInquirysDTO>> getExportInquirys(@RequestBody List<Long> inquiryIds);

    /**
     * 根据医院编码获取辅检公表映射关系
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/pacsConfig/getPacsConfigByHosCode")
    RespDTO<Map<String, String>> getPacsConfigByHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 根据医院编码获取提示信息标题映射关系
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/introduceTitleConfig/getTitleMappingHosCode")
    RespDTO<Map<String, String>> getTitleMappingHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 根据医院编码查询诊断icd映射
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/diseaseIcd/getDiseaseIcdByHosCode")
    RespDTO<Map<String, String>> getDiseaseIcdByHosCode(@RequestBody HosCodeVO hosCodeVO);

    /**
     * 根据医院编码查询辅检公表映射，公表项做key
     *
     * @param hosCodeVO
     * @return
     */
    @PostMapping("/pacsConfig/getPacsConfigByUniqueNameAndHosCode")
    RespDTO<Map<String, List<String>>> getPacsConfigByUniqueNameAndHosCode(@RequestBody HosCodeVO hosCodeVO);

    @PostMapping("/lisDockingImports/lisDockingImports")
    public RespDTO<LisRetrieveListAllDTO> getLisDatas(@RequestBody @Valid LisArgumentsVO lisArgumentsVO);

    @PostMapping("/lisDockingImports/getByGroupNameLisData")
    public RespDTO<LisDataDTO> getByGroupNameLisData(@RequestBody @Valid LisGroupNameVO lisGroupNameVO);

    @PostMapping("/lisDockingImports/pacsDockingImport")
    RespDTO<PacsRetrieveListAllDTO> pacsDockingImports(@RequestBody @Valid PacsArgumentsVO pacsArgumentsVO);

    @PostMapping("/lisDockingImports/getByCheckItemPacsData")
    RespDTO<PacsDataDTO> getByCheckItemPacsDatas(@RequestBody @Valid PacsCheckItemVO pacsGroupNameVO);

    @PostMapping("/sysSet/getSysSetInfoDatas")
    RespDTO<List<SysSetInfoDTO>> getSysSetInfoDatas(@Valid @RequestBody HospitalSetVO hospitalSetVO);

    @PostMapping("/sysSet/getHospitalRankInfo")
    RespDTO<GetHospitalRankInfoDTO> getHospitalRankInfo(@RequestBody GetHospitalRankInfoVO getHospitalRankInfoVO);

    @PostMapping("/sysSet/getAdminCheckout")
    RespDTO<Boolean> getAdminCheckout(@Valid @RequestBody HospitalCodeSetVO hospitalCodeSetVO);

    @PostMapping("/hospitalDeptInfo/getHospitalDeptInfoAll")
    RespDTO<List<HospitalDeptInfoAllDTO>> getHospitalDeptInfoAll(@RequestBody @Valid HospitalCodeVo hospitalCodeVo);
}