package com.diagbot.client.hystrix;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.ConceptBaseVO;
import com.diagbot.client.bean.ConceptExistVO;
import com.diagbot.client.bean.DisTypeVO;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.DeptInfoDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.DisScaleDTO;
import com.diagbot.dto.DisTypeDTO;
import com.diagbot.dto.DisclaimerInformationDTO;
import com.diagbot.dto.DiseaseIcdDTO;
import com.diagbot.dto.EvaluationDTO;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.IndexConfigAndDataDTO;
import com.diagbot.dto.IndexConfigDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.ScaleIndexDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.entity.ScaleContent;
import com.diagbot.vo.ConceptFindVO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.DeptInfoVO;
import com.diagbot.vo.DisScaleVO;
import com.diagbot.vo.DisclaimerInformationVO;
import com.diagbot.vo.ExistListByConceptIdsVO;
import com.diagbot.vo.GetDiseaseIcdVO;
import com.diagbot.vo.GetLisMappingVO;
import com.diagbot.vo.GetStaticKnowledgeVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.IndexConfigdiseaseNameVO;
import com.diagbot.vo.IndexDataFindVO;
import com.diagbot.vo.IndexDataSaveVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.ScaleIndexVO;
import com.diagbot.vo.VersionVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:调用中间层服务
 * @Author: Mark Huang
 * @time: 2019/03/25 19:15
 */
@Component
@Slf4j
public class AiptServiceHystrix implements AiptServiceClient {
    /**
     * 计算接口（量表、计算公式）
     *
     * @param calculateData
     * @return
     */
    @Override
    public RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData) {
        log.error("【hystrix】调用{}异常", "scaleCalc");
        return null;
    }

    /**
     * 获取量表全文接口
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<List<ScaleContent>> getScale(@RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "getScale");
        return null;
    }

    /**
     * 症状特征提取
     *
     * @param text
     * @return
     */
    @Override
    public RespDTO<List<FeatureConceptDTO>> symptomFeature(@RequestParam("text") String text) {
        log.error("【hystrix】调用{}异常", "symptomFeature");
        return null;
    }

    /**
     * 推理
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<PushDTO> pushInner(@RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "pushInner");
        return null;
    }


//    @Override
//    public RespDTO<List<DisScaleDTO>> getList(DisScaleVO scaleVO) {
//        log.error("【hystrix】调用{}异常", "getList");
//        return null;
//    }
//
//    @Override
//    public RespDTO<List<ScaleIndexDTO>> index(ScaleIndexVO scaleIndexVO) {
//        log.error("【hystrix】调用{}异常", "index");
//        return null;
//    }


    @Override
    public RespDTO<DisTypeDTO> getDisType() {
        log.error("【hystrix】调用{}异常", "getDisType");
        return null;
    }


    @Override
    public RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        log.error("【hystrix】调用{}异常", "getConceptDetail");
        return null;
    }

    @Override
    public RespDTO<ConceptBaseDTO> existConcept(@RequestBody @Valid ConceptBaseVO conceptBaseVO) {
        log.error("【hystrix】调用{}异常", "existConcept");
        return null;
    }

    @Override
    public RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO) {
        log.error("【hystrix】调用{}异常", "getConceptUsual");
        return null;
    }

    @Override
    public RespDTO<DeptInfoDTO> getDeptInfo(@Valid @RequestBody DeptInfoVO deptInfoVO) {
        log.error("【hystrix】调用{}异常", "getDeptInfo");
        return null;
    }

    /**
     * 根据医院编号疾病概念名称获取各医院对应的诊断icd
     *
     * @param getDiseaseIcdVO
     * @return
     */
    @Override
    public RespDTO<List<DiseaseIcdDTO>> getDiseaseIcds(@RequestBody GetDiseaseIcdVO getDiseaseIcdVO) {
        log.error("【hystrix】调用{}异常", "getDiseaseIcds");
        return null;
    }

    @Override
    public RespDTO<GetTopPatientInfoDTO> getTopPatientInfo(GetTopPatientInfoVO getTopPatientInfoVO) {
        log.error("【hystrix】调用{}异常", "getTopPatientInfo");
        return null;
    }

    @Override
    public RespDTO<List<EvaluationDTO>> getEvaluationModules(@Valid SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "getEvaluationModules");
        return null;
    }

    @Override
    public RespDTO<List<IndexConfigDTO>> getindexConfigs(IndexConfigdiseaseNameVO indexConfigdiseaseNameVO) {
        log.error("【hystrix】调用{}异常", "getindexConfigs");
        return null;
    }

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @Override
    public RespDTO<Map<Long, String>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO) {
        log.error("【hystrix】调用{}异常", "getConceptMap");
        return null;
    }

    /**
     * 根据概念Id,年龄,性别列表获取概念列表Map
     *
     * @param conceptFindVO 搜索参数
     * @return 术语id和术语 Map
     */
    @Override
    public RespDTO<Map<Long, String>> getConceptCondMap(@RequestBody ConceptFindVO conceptFindVO) {
        log.error("【hystrix】调用{}异常", "getConceptMap");
        return null;
    }


    /**
     * 是否有提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    @Override
    public RespDTO<Boolean> existConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO) {
        log.error("【hystrix】调用{}异常", "existConceptDetail");
        return null;
    }

    /**
     * 验证诊断类型
     *
     * @param disTypeVO
     * @return
     */
    @Override
    public RespDTO<Boolean> verifyDisType(@Valid @RequestBody DisTypeVO disTypeVO) {
        log.error("【hystrix】调用{}异常", "verifyDisType");
        return null;
    }

    /**
     * 根据名称和类型获取概念列表Map
     *
     * @param conceptExistVO
     * @return
     */
    @Override
    public RespDTO<Map<String, Long>> getConceptMapByNameAndType(@RequestBody ConceptExistVO conceptExistVO) {
        log.error("【hystrix】调用{}异常", "getConceptMapByNameAndType");
        return null;
    }

    /**
     * 标签检索
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@Valid RetrievalVO retrievalVO) {
        log.error("【hystrix】调用{}异常", "retrivelConceptInfo");
        return null;
    }

    /**
     * 指标获取
     *
     * @param indexDataFindVO
     * @return
     */
    @Override
    public RespDTO<IndexConfigAndDataDTO> getIndexDatas(@Valid IndexDataFindVO indexDataFindVO) {
        log.error("【hystrix】调用{}异常", "getIndexDatas");
        return null;
    }

    /**
     * 指标保存
     *
     * @param indexDataSaveVO
     * @return
     */
    @Override
    public RespDTO<Boolean> saveIndexDatas(@Valid IndexDataSaveVO indexDataSaveVO) {
        log.error("【hystrix】调用{}异常", "saveIndexDatas");
        return null;
    }

    @Override
    public RespDTO<List<DisclaimerInformationDTO>> getDisclaimerInformations(DisclaimerInformationVO disclaimerInformationVO) {
        log.error("【hystrix】调用{}异常", "getDisclaimerInformations");
        return null;
    }

    /**
     * 判断静态知识是否存在
     *
     * @param existListByConceptIdsVO
     * @return
     */
    @Override
    public RespDTO<List<Long>> existListByConceptIds(ExistListByConceptIdsVO existListByConceptIdsVO) {
        log.error("【hystrix】调用{}异常", "existListByConceptIds");
        return null;
    }

    @Override
    public RespDTO<List<RetrievalDTO>> getStaticKnowledge(GetStaticKnowledgeVO getStaticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "getStaticKnowledge");
        return null;
    }


    /**
     * 获取版本信息
     */
    @Override
    public RespDTO<VersionWrapperDTO> getVersionInfoAlls(VersionVO versionVO) {
        log.error("【hystrix】调用{}异常", "getVersionInfoAlls");
        return null;
    }

    @Override
    public RespDTO<Map<String, Map<String, String>>> getLisMappingByQuestion(GetLisMappingVO getLisMappingVO) {
        log.error("【hystrix】调用{}异常", "getLisMappingByQuestion");
        return null;
    }

    /**
     * 批量获取提示信息
     *
     * @param conceptIntroduceVOList
     * @return
     */
    @Override
    public RespDTO<List<ConceptIntroduceDTO>> getConceptDetails(@Valid @RequestBody List<ConceptIntroduceVO> conceptIntroduceVOList) {
        log.error("【hystrix】调用{}异常", "getConceptDetails");
        return null;
    }


    /**
     * 获取字典信息
     *
     * @return
     */
    @Override
    public RespDTO<Map<Long, List<DictionaryInfoDTO>>> getList() {
        log.error("【hystrix】调用{}异常", "getList");
        return null;
    }
}