package com.diagbot.client.hystrix;

import com.diagbot.client.TranServiceClient;
import com.diagbot.dto.DoctorInfoDTO;
import com.diagbot.dto.GetExportInquirysDTO;
import com.diagbot.dto.GetHospitalRankInfoDTO;
import com.diagbot.dto.HisInquirysForJzDTO;
import com.diagbot.dto.HospitalDeptInfoAllDTO;
import com.diagbot.dto.HospitalInfoDTO;
import com.diagbot.dto.LisConfigDTO;
import com.diagbot.dto.LisDataDTO;
import com.diagbot.dto.LisRetrieveListAllDTO;
import com.diagbot.dto.PacsDataDTO;
import com.diagbot.dto.PacsRetrieveListAllDTO;
import com.diagbot.dto.PatientInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.dto.TranFieldInfoDTO;
import com.diagbot.entity.DoctorInfo;
import com.diagbot.entity.HospitalDept;
import com.diagbot.entity.PatientInfo;
import com.diagbot.vo.AddDiagnoseSecondVO;
import com.diagbot.vo.DoctorInfoVO;
import com.diagbot.vo.GetHospitalRankInfoVO;
import com.diagbot.vo.HisInquirysForJzVO;
import com.diagbot.vo.HosCodeVO;
import com.diagbot.vo.HospitalCodeSetVO;
import com.diagbot.vo.HospitalCodeVo;
import com.diagbot.vo.HospitalInfoVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.LisArgumentsVO;
import com.diagbot.vo.LisGroupNameVO;
import com.diagbot.vo.LisHospitalCodeVO;
import com.diagbot.vo.PacsArgumentsVO;
import com.diagbot.vo.PacsCheckItemVO;
import com.diagbot.vo.PatientInfoVO;
import com.diagbot.vo.SaveInquiryToHisVO;
import com.diagbot.vo.TranFieldInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: 调用信息对接层服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@Component
@Slf4j
public class TranServiceHystrix implements TranServiceClient {
    @Override
    public RespDTO<List<DoctorInfoDTO>> getDoctorInfo(@Valid DoctorInfoVO doctorInfoVo) {
        log.error("【hystrix】调用{}异常", "getDoctorInfo");
        return null;
    }

    @Override
    public RespDTO<Map<Long, DoctorInfo>> doctorInfoMapByIds(List<Long> ids) {
        log.error("【hystrix】调用{}异常", "doctorInfoMapByIds");
        return null;
    }

    /**
     * 根据id获取医生信息
     *
     * @param id
     * @return
     */
    @Override
    public RespDTO<DoctorInfoDTO> getDoctorInfoById(@RequestBody Long id) {
        log.error("【hystrix】调用{}异常", "getDoctorInfoById");
        return null;
    }

    @Override
    public RespDTO<HospitalInfoDTO> getHospitalInfo(HospitalInfoVO hospitalInfoVO) {
        log.error("【hystrix】调用{}异常", "getHospitalInfo");
        return null;
    }

    @Override
    public RespDTO<Map<Long, HospitalDept>> hospitalDeptInfoMapByIds(List<Long> ids) {
        log.error("【hystrix】调用{}异常", "hospitalDeptInfoMapByIds");
        return null;
    }

    @Override
    public RespDTO<List<LisConfigDTO>> getLisConfigByhospitalId(LisHospitalCodeVO lisHospitalCodeVO) {
        log.error("【hystrix】调用{}异常", "getLisConfigByhospitalId");
        return null;
    }

    @Override
    public RespDTO<PatientInfoDTO> getPatientInfo(PatientInfoVO patientInfoVO) {
        log.error("【hystrix】调用{}异常", "getPatientInfo");
        return null;
    }

    @Override
    public RespDTO<Map<Long, PatientInfo>> patientInfoMapByIds(List<Long> ids) {
        log.error("【hystrix】调用{}异常", "patientInfoMapByIds");
        return null;
    }

    @Override
    public RespDTO<List<TranFieldInfoDTO>> getInfoByUuid(TranFieldInfoVO tranFieldInfoVO) {
        log.error("【hystrix】调用{}异常", "getInfoByUuid");
        return null;
    }

    @Override
    public RespDTO<Boolean> saveInquiryToHis(SaveInquiryToHisVO saveInquiryToHisVO) {
        log.error("【hystrix】调用{}异常", "saveInquiryToHis");
        return null;
    }

    @Override
    public RespDTO<HisInquirysForJzDTO> hisInquirysForJz(HisInquirysForJzVO hisInquirysForJzVO) {
        log.error("【hystrix】调用{}异常", "hisInquirysForJz");
        return null;
    }

    @Override
    public RespDTO<Boolean> addDiagnoseSecond(AddDiagnoseSecondVO addDiagnoseSecondVO) {
        log.error("【hystrix】调用{}异常", "addDiagnoseSecond");
        return null;
    }

    @Override
    public RespDTO<List<GetExportInquirysDTO>> getExportInquirys(List<Long> inquiryIds) {
        log.error("【hystrix】调用{}异常", "getExportInquirys");
        return null;
    }

    /**
     * 根据医院编码获取辅检公表映射关系
     *
     * @param hosCodeVO
     * @return
     */
    @Override
    public RespDTO<Map<String, String>> getPacsConfigByHosCode(@RequestBody HosCodeVO hosCodeVO) {
        log.error("【hystrix】调用{}异常", "getPacsConfigByHosCode");
        return null;
    }

    /**
     * 根据医院编码获取提示信息标题映射关系
     *
     * @param hosCodeVO
     * @return
     */
    @Override
    public RespDTO<Map<String, String>> getTitleMappingHosCode(@RequestBody HosCodeVO hosCodeVO) {
        log.error("【hystrix】调用{}异常", "getTitleMappingHosCode");
        return null;
    }

    /**
     * 根据医院编码查询诊断icd映射
     *
     * @param hosCodeVO
     * @return
     */
    @Override
    public RespDTO<Map<String, String>> getDiseaseIcdByHosCode(@RequestBody HosCodeVO hosCodeVO) {
        log.error("【hystrix】调用{}异常", "getDiseaseIcdByHosCode");
        return null;
    }

    /**
     * 根据医院编码查询辅检公表映射，公表项做key
     *
     * @param hosCodeVO
     * @return
     */
    @Override
    public RespDTO<Map<String, List<String>>> getPacsConfigByUniqueNameAndHosCode(@RequestBody HosCodeVO hosCodeVO) {
        log.error("【hystrix】调用{}异常", "getPacsConfigByUniqueNameAndHosCode");
        return null;
    }

    @Override
    public RespDTO<LisRetrieveListAllDTO> getLisDatas(@RequestBody @Valid LisArgumentsVO lisArgumentsVO) {
        log.error("【hystrix】调用{}异常", "getLisDatas");
        return null;
    }

    @Override
    public RespDTO<LisDataDTO> getByGroupNameLisData(@RequestBody @Valid LisGroupNameVO lisGroupNameVO) {
        log.error("【hystrix】调用{}异常", "getByGroupNameLisData");
        return null;
    }

    @Override
    public RespDTO<PacsRetrieveListAllDTO> pacsDockingImports(@RequestBody @Valid PacsArgumentsVO pacsArgumentsVO) {
        log.error("【hystrix】调用{}异常", "pacsDockingImports");
        return null;
    }

    @Override
    public RespDTO<PacsDataDTO> getByCheckItemPacsDatas(@RequestBody @Valid PacsCheckItemVO pacsGroupNameVO) {
        log.error("【hystrix】调用{}异常", "getByCheckItemPacsDatas");
        return null;
    }

    @Override
    public RespDTO<List<SysSetInfoDTO>> getSysSetInfoDatas(@Valid HospitalSetVO hospitalSetVO) {
        log.error("【hystrix】调用{}异常", "getSysSetInfoDatas");
        return null;
    }

    @Override
    public RespDTO<GetHospitalRankInfoDTO> getHospitalRankInfo(GetHospitalRankInfoVO getHospitalRankInfoVO) {
        log.error("【hystrix】调用{}异常", "getHospitalRankInfo");
        return null;
    }

    @Override
    public RespDTO<Boolean> getAdminCheckout(@Valid HospitalCodeSetVO hospitalCodeSetVO) {
        log.error("【hystrix】调用{}异常", "getAdminCheckout");
        return null;
    }

    @Override
    public RespDTO<List<HospitalDeptInfoAllDTO>> getHospitalDeptInfoAll(@Valid HospitalCodeVo hospitalCodeVo) {
        log.error("【hystrix】调用{}异常", "getHospitalDeptInfoAll");
        return null;
    }
}