package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.DoctorPageModeDTO;
import com.diagbot.entity.DoctorPageMode;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.DoctorPageModeServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.DoctorIdVO;
import com.diagbot.vo.DoctorPageModeAllVO;
import com.diagbot.vo.DoctorPageModeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.swing.text.Utilities;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangfeng
 * @Description: 医生页面模式设置
 * @date 2018年11月20日 下午2:56:01
 */
@Component
public class DoctorPageModeFacade extends DoctorPageModeServiceImpl {

    @Autowired
    DoctorPageModeServiceImpl doctorPageModeServiceImpl;
    /**
     * 保存医生页面结构设置信息
     *
     * @param doctorPageModeAllVO
     * @return
     */
    public boolean saveDoctorPageMode(DoctorPageModeAllVO doctorPageModeAllVO) {
        boolean res = false;
        if(ListUtil.isNotEmpty(doctorPageModeAllVO.getDoctorPageMode())) {
            QueryWrapper<DoctorPageMode> doctorPageModeWrapper = new QueryWrapper<>();
            doctorPageModeWrapper.eq("doctor_id", doctorPageModeAllVO.getDoctorId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey());
            res = remove(doctorPageModeWrapper);
            List<DoctorPageMode> doctorData = new ArrayList<DoctorPageMode>();
            Date dateTime = DateUtil.now();
            for (DoctorPageModeVO doctorPageModeData : doctorPageModeAllVO.getDoctorPageMode()) {
                DoctorPageMode data = new DoctorPageMode();
                data.setCreator(doctorPageModeAllVO.getDoctorId().toString());
                data.setGmtCreate(dateTime);
                data.setGmtModified(dateTime);
                data.setModifier(doctorPageModeAllVO.getDoctorId().toString());
                data.setModeClassify(doctorPageModeData.getModeClassify());
                data.setDoctorId(doctorPageModeAllVO.getDoctorId());
                data.setModeValue(doctorPageModeData.getModeValue());
                doctorData.add(data);
            }
            res = doctorPageModeServiceImpl.saveBatch(doctorData);
        }
        return res;
    }

    /**
     * 获取医生页面结构设置信息
     *
     * @param doctorIdVO
     * @return
     */
    public List<DoctorPageModeDTO> getDoctorPageMode(@Valid DoctorIdVO doctorIdVO) {
        QueryWrapper<DoctorPageMode> doctorPageModeWrapper = new QueryWrapper<>();
        Map<String, Object> mapAll = new HashMap<>();
        mapAll.put("doctor_id", doctorIdVO.getDoctorId());
        mapAll.put("is_deleted", IsDeleteEnum.N.getKey());
        doctorPageModeWrapper.allEq(mapAll);
        List<DoctorPageMode> doctorPageModeData = list(doctorPageModeWrapper);
        List<DoctorPageModeDTO> data = BeanUtil.listCopyTo(doctorPageModeData, DoctorPageModeDTO.class);
        return data;
    }

}
