package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/2/26 13:27
 */
@Component
public class FeatureFacade {
    @Autowired
    private AiptServiceClient aiptServiceClient;
    @Autowired
    private QuestionFacade questionFacade;

    /**
     * 症状关键词提取
     *
     * @param text
     * @return
     */
    public List<FeatureConceptDTO> getSymptomFeature(String text) {
        RespDTO<List<FeatureConceptDTO>> res = aiptServiceClient.symptomFeature(text);
        RespDTOUtil.respNGDealCover(res, res.msg == "" ? "AI没有返回结果" : res.msg);
        List<FeatureConceptDTO> symptomFeatureList = res.data;
        if (ListUtil.isNotEmpty(symptomFeatureList)) {
            if (!symptomFeatureList.get(0).getChronicLabel().equals(1)) {
                symptomFeatureList = addQuestionId(symptomFeatureList, ConceptTypeEnum.Symptom.getKey());
            }
        }
        return symptomFeatureList;
    }

    /**
     * 概念关联标签，添加标签id
     *
     * @param concepts
     * @param type
     * @return
     */
    public List<FeatureConceptDTO> addQuestionId(List<FeatureConceptDTO> concepts, Integer type) {
        List<String> nameList = concepts
                .stream()
                .map(concept -> concept.getName())
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(nameList)) {
            return concepts;
        }
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
        questionInfoQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("tag_name", nameList)
                .eq("type", type);
        List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);
        if (ListUtil.isNotEmpty(questionInfoList)) {
            Map<String, QuestionInfo> questionInfoMap
                    = EntityUtil.makeEntityMap(questionInfoList, "tagName");
            for (FeatureConceptDTO concept : concepts) {
                if (questionInfoMap.containsKey(concept.getName())) {
                    concept.setId(questionInfoMap.get(concept.getName()).getId());
                }
            }
        }
        return concepts;
    }
}