package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.ModuleDetailDTO;
import com.diagbot.dto.ModuleInfoDTO;
import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ModuleDetail;
import com.diagbot.entity.ModuleInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.service.impl.ModuleInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.ConceptFindVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ModuleVO;
import com.diagbot.vo.QuestionVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 模型业务层
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class ModuleFacade extends ModuleInfoServiceImpl {

    @Autowired
    ModuleDetailFacade moduleDetailFacade;
    @Autowired
    QuestionFacade questionFacade;
    @Autowired
    AiptServiceClient aiptServiceClient;

    /**
     * 返回所有的模型结构
     *
     * @return 模型结构
     */
    public List<ModuleInfoDTO> getAll(ModuleVO moduleVO) {
        ConceptFindVO conceptFindVO = new ConceptFindVO();
        ConceptSearchVO conceptSearchVO = new ConceptSearchVO();
        List<ModuleInfoDTO> data = new ArrayList<>();
        //取到所有模板信息
        List<ModuleInfo> list = new ArrayList<>();
        List<Long> ids = new ArrayList<>();
        List<Long> pacsConceptIds = new ArrayList<>();//辅检名称列表
        List<Long> pacModuleIds = new ArrayList<>();//模板中辅检id
        Boolean isDefault = false;
        Boolean isPacs = false;
        if (moduleVO.getMouduleType().intValue() == 1) {
            //根据科室筛选如果没有，返回通用模板
            list = getModuleInfoByDisType(moduleVO.getMouduleType(), moduleVO.getRelationId());
            if (ListUtil.isNotEmpty(list)) {
                isPacs = true;
            } else {
                isDefault = true;
            }
        } else if (moduleVO.getMouduleType().intValue() == 2) {
            //根据慢病筛选
            list = getModuleInfoByDisType(moduleVO.getMouduleType(), moduleVO.getRelationId());
            isPacs = true;
        } else {
            isDefault = true;
        }
        //科室没有内容，或者默认情况下，需要推测出默认模板
        if (isDefault) {
            //默认
            list = getModuleInfoByDisType(0, 0L);
            isPacs = true;
        }
        if (isPacs) {
            for (ModuleInfo moduleInfo : list) {
                if (Integer.parseInt(moduleInfo.getType()) == QuestionTypeEnum.Pacs.getKey()) {
                    pacModuleIds.add(moduleInfo.getId());
                } else {
                    ids.add(moduleInfo.getId());
                }
            }
        }
        data = BeanUtil.listCopyTo(list, ModuleInfoDTO.class);
        Map<Long, ModuleInfo> moduleInfoMap
                = list.stream().collect(Collectors.toMap(ModuleInfo::getId, moduleInfo -> moduleInfo));
        Map<Long, List<ModuleDetail>> moduleDetailMap = new LinkedHashMap<>();
        //取到所有模板明细
        if (ListUtil.isNotEmpty(ids)) {
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("ids", ids);
            paramMap.put("sexType", moduleVO.getSexType());
            paramMap.put("age", moduleVO.getAge());
            List<ModuleDetail> moduleDetailList = moduleDetailFacade.getDetailByModuleFac(paramMap);
            if (ListUtil.isNotEmpty(pacModuleIds)) {//获取类型是化验的标签id
                QueryWrapper<ModuleDetail> moduleDetailQueryWrapper = new QueryWrapper<>();
                moduleDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                        .in("module_id", pacModuleIds);
                List<ModuleDetail> moduleDetails = moduleDetailFacade.list(moduleDetailQueryWrapper);
                for (ModuleDetail moduleDetail : moduleDetails) {
                    moduleDetailList.add(moduleDetail);
                }
            }
            if (ListUtil.isNotEmpty(moduleDetailList)) {
                moduleDetailMap = EntityUtil.makeEntityListMap(moduleDetailList, "moduleId");
            }

        }
        //循环放入模板明细
        List<ModuleDetailDTO> moduleDetailDTOList = new ArrayList<>();
        for (ModuleInfoDTO bean : data) {
            if (moduleDetailMap.get(bean.getId()) != null) {
                moduleDetailDTOList = BeanUtil.listCopyTo(moduleDetailMap.get(bean.getId()), ModuleDetailDTO.class);
                for (ModuleDetailDTO detailDTO : moduleDetailDTOList) {
                    if (Integer.parseInt(bean.getType()) == 6) {
                        pacsConceptIds.add(detailDTO.getQuestionId());
                    } else if (null != detailDTO.getQuestionId()) {
                        QuestionVO questionVO = new QuestionVO();
                        questionVO.setId(detailDTO.getQuestionId());
                        questionVO.setSexType(moduleVO.getSexType());
                        questionVO.setAge(moduleVO.getAge());
                        QuestionDTO questionDTO = questionFacade.getById(questionVO);
                        BeanUtil.copyProperties(questionDTO, detailDTO);
                    }
                    if (null != detailDTO.getRelationModule() &&
                            moduleInfoMap.get(detailDTO.getRelationModule()) != null) {
                        detailDTO.setRelationModuleName(moduleInfoMap.get(detailDTO.getRelationModule()).getName());
                    }
                }
                bean.setModuleDetailDTOList(moduleDetailDTOList);
            }
        }
        RespDTO<Map<Long, String>> pacsRespDTO = null;
        //辅检
        if (ListUtil.isNotEmpty(pacsConceptIds)) {
            conceptFindVO = new ConceptFindVO();
            conceptFindVO.setConceptIds(pacsConceptIds);
            conceptFindVO.setAge(moduleVO.getAge());
            conceptFindVO.setSexType(moduleVO.getSexType());
            pacsRespDTO = aiptServiceClient.getConceptCondMap(conceptFindVO);
            RespDTOUtil.respNGDeal(pacsRespDTO, "获取辅检标签失败");
        }
        if (ListUtil.isNotEmpty(pacsConceptIds)) {//如果存在辅检相关信息添加数据返回
            //将辅检标签放入出参
            for (ModuleInfoDTO bean : data) {
                if (Integer.parseInt(bean.getType()) == QuestionTypeEnum.Pacs.getKey() && pacsRespDTO.data != null) {
                    for (ModuleDetailDTO detailDTO : bean.getModuleDetailDTOList()) {
                        detailDTO.setName(pacsRespDTO.data.get(detailDTO.getQuestionId()));
                    }
                }
            }
        }
        return data;
    }

    /**
     * 获取相应类型的模板信息
     *
     * @param moduleType
     * @return
     */
    public List<ModuleInfo> getModuleInfoByDisType(Integer moduleType, Long relationId) {
        QueryWrapper<ModuleInfo> queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("module_type", moduleType)
                .eq("relation_id", relationId);
        queryWrapper.orderByAsc("id");
        return this.list(queryWrapper);
    }
}
