package com.diagbot.facade;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.PushVO;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/20 11:18
 */
@Component
public class PushFacade {
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;
    @Autowired
    private AiptServiceClient aiptServiceClient;
    @Autowired
    private AssembleFacade assembleFacade;

    /**
     * 中间层推理-症状……诊断、病情提示、治疗方案
     *
     * @param searchVO
     * @return
     */
    public PushDTO pushInnerAipt(SearchVO searchVO) {
        RespDTO<PushDTO> res = aiptServiceClient.pushInner(searchVO);
        RespDTOUtil.respNGDeal(res, "中间层没有返回结果");
        return res.data;
    }

    /**
     * 推理接口
     *
     * @param pushVO
     * @return
     */
    public PushDTO pushInner(PushVO pushVO) {
        PushDTO pushDTO = new PushDTO();
        SearchVO searchVO = assembleFacade.assembleData(pushVO);
        PushDTO data = pushInnerAipt(searchVO);

        String featureType = pushVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        //推理结果返回科室
        pushDTO.setDept(data.getDept());
        pushDTO.setHasIndications(data.getHasIndications());

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("pushVO", pushVO);
            invokeParams.put("pushDTO", pushDTO);
            invokeParams.put("featureTypeSet", featureTypeSet);
            invokeParams.put("data", data);
            pushDTO
                    = dataBeanAggregateQueryFacade.get("setICSSAll", invokeParams, PushDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        } finally {
            if (pushDTO == null) {
                pushDTO = new PushDTO();
                pushDTO.setDept(data.getDept());
            }
        }

        //辅检  标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Pacs.getKey()))) {
            pushDTO.setPacs(data.getPacs());
        }
        //诊断 返回分类+标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Disease.getKey()))) {
            pushDTO.setDis(data.getDis());
        }
        //推送指标 featuretype=22
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Indication.getKey()))) {
            pushDTO.setMedicalIndications(data.getMedicalIndications());
        }
        return pushDTO;
    }
}