package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.FolderMapping;
import com.diagbot.entity.TemplateFolder;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.TemplateTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.FolderMappingService;
import com.diagbot.service.TemplateFolderService;
import com.diagbot.service.impl.TemplateFolderServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.HospitalCodeSetVO;
import com.diagbot.vo.TemplateFolderDelVO;
import com.diagbot.vo.TemplateFolderListVO;
import com.diagbot.vo.TemplateFolderSortVO;
import com.diagbot.vo.TemplateFolderVO;
import com.diagbot.vo.TemplateSortVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author zhoutg
 * @Description: 病历模板文件夹管理员
 * @date 2018年11月16日 上午11:24:36
 */
@Component
public class TemplateFolderAdminFacade extends TemplateFolderServiceImpl {

    @Autowired
    AdminCheckFacade adminCheckFacade;
    @Autowired
    FolderMappingFacade folderMappingFacade;
    @Autowired
    @Qualifier("templateFolderServiceImpl")
    TemplateFolderService templateFolderService;

    @Autowired
    @Qualifier("folderMappingServiceImpl")
    FolderMappingService folderMappingService;

    /**
     * 文件夹新增或更新
     *
     * @param templateFolderVO
     */
    public void saveOrUpdate(TemplateFolderVO templateFolderVO) {
        HospitalCodeSetVO hospitalCodeSetVO = new HospitalCodeSetVO();
        hospitalCodeSetVO.setDoctorId(templateFolderVO.getDoctorId());
        hospitalCodeSetVO.setHospitalId(templateFolderVO.getHospitalId());
        boolean res = adminCheckFacade.getadminCheck(hospitalCodeSetVO);
        if (!res) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "暂无修改权限");
        }
        boolean add = true;
        if (templateFolderVO.getId() != null) {
            add = false;
        }
        int count = this.count(new QueryWrapper<TemplateFolder>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("name", templateFolderVO.getName())
                .eq("hospital_id", templateFolderVO.getHospitalId())
                .eq("template_type", TemplateTypeEnum.AdminTemplate.getKey())
                // .eq("doctor_id", templateFolderVO.getDoctorId())
                .ne(!add, "id", templateFolderVO.getId())
        );
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该文件夹已存在");
        }
        TemplateFolder templateFolder = new TemplateFolder();
        BeanUtil.copyProperties(templateFolderVO, templateFolder);
        Date date = DateUtil.now();
        if (add) {
            templateFolder.setGmtCreate(date);
            templateFolder.setCreator(templateFolderVO.getDoctorId().toString());
        }
        templateFolder.setGmtModified(date);
        templateFolder.setModifier(templateFolderVO.getDoctorId().toString());
        //获取文件夹最大数
        Integer orderNo =templateFolderService.getMaxNo(templateFolderVO.getHospitalId());
        Integer num = orderNo == null ? 0 :  orderNo;
        //templateFolder.setOrderNo(num);
        this.saveOrUpdate(templateFolder);
        // 如果是新增，id改为负数,且文件夹放最后。
        if (add) {
            this.update(new UpdateWrapper<TemplateFolder>()
                    .eq("id", templateFolder.getId())
                    .set("id", Long.parseLong("-" + templateFolder.getId()))
                    .set("order_no",num)
            );
        }
    }


    /**
     * 文件夹删除
     *
     * @param templateFolderDelVO
     */
    public void delete(TemplateFolderDelVO templateFolderDelVO) {
        // 校验权限
        HospitalCodeSetVO hospitalCodeSetVO = new HospitalCodeSetVO();
        hospitalCodeSetVO.setDoctorId(templateFolderDelVO.getDoctorId());
        hospitalCodeSetVO.setHospitalId(templateFolderDelVO.getHospitalId());
        boolean res = adminCheckFacade.getadminCheck(hospitalCodeSetVO);
        if (!res) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "暂无删除权限");
        }
        int count = folderMappingFacade.count(new QueryWrapper<FolderMapping>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("folder_id", templateFolderDelVO.getFolderId())
        );
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "请先删除当前文件夹下的模板");
        }
        // 删除
        this.update(new UpdateWrapper<TemplateFolder>()
                .eq("id", templateFolderDelVO.getFolderId())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("gmt_modified", DateUtil.now())
                .set("modifier", templateFolderDelVO.getDoctorId())
                .set("doctor_id", templateFolderDelVO.getDoctorId())
        );
    }


    /**
     * 文件夹列表
     *
     * @param templateFolderListVO
     */
    public List<TemplateFolder> list(TemplateFolderListVO templateFolderListVO) {
        // 校验权限
        HospitalCodeSetVO hospitalCodeSetVO = new HospitalCodeSetVO();
        hospitalCodeSetVO.setDoctorId(templateFolderListVO.getDoctorId());
        hospitalCodeSetVO.setHospitalId(templateFolderListVO.getHospitalId());
        boolean res = adminCheckFacade.getadminCheck(hospitalCodeSetVO);
        if (!res) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "暂无获取文件夹列表权限");
        }
        int count = this.count(new QueryWrapper<TemplateFolder>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                //.eq("doctor_id", templateFolderListVO.getDoctorId())
                .eq("hospital_id", templateFolderListVO.getHospitalId())
                .eq("template_type", TemplateTypeEnum.AdminTemplate.getKey())
        );
        // 如果没有文件夹，新建一个“未分类文件夹”
        if (0 == count) {
            TemplateFolder templateFolder = new TemplateFolder();
            templateFolder.setIsDeleted(IsDeleteEnum.N.getKey());
            templateFolder.setCreator(templateFolderListVO.getDoctorId().toString());
            templateFolder.setModifier(templateFolderListVO.getDoctorId().toString());
            templateFolder.setDoctorId(templateFolderListVO.getDoctorId());
            templateFolder.setHospitalId(templateFolderListVO.getHospitalId());
            Date date = DateUtil.now();
            templateFolder.setGmtCreate(date);
            templateFolder.setGmtModified(date);
            templateFolder.setName("未分类文件夹");
            templateFolder.setOrderNo(99999);
            templateFolder.setTemplateType(TemplateTypeEnum.AdminTemplate.getKey());
            this.save(templateFolder);
            List<TemplateFolder> data = new ArrayList<>();
            data.add(templateFolder);
            return data;
        }
        return this.list(new QueryWrapper<TemplateFolder>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                //.eq("doctor_id", templateFolderListVO.getDoctorId())
                .eq("hospital_id", templateFolderListVO.getHospitalId())
                .orderByDesc("gmt_modified")
        );
    }


    /**
     * 文件夹排序
     *
     * @param templateFolderSortVOList
     */
    public void sort(List<TemplateFolderSortVO> templateFolderSortVOList) {
        if (ListUtil.isNotEmpty(templateFolderSortVOList)) {
            TemplateFolderSortVO templateFolderSortVO = templateFolderSortVOList.get(0);
            HospitalCodeSetVO hospitalCodeSetVO = new HospitalCodeSetVO();
            hospitalCodeSetVO.setDoctorId(templateFolderSortVO.getDoctorId());
            hospitalCodeSetVO.setHospitalId(templateFolderSortVO.getHospitalId());
            boolean res = adminCheckFacade.getadminCheck(hospitalCodeSetVO);
            if (!res) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "暂无文件夹排序权限");
            }
            List<TemplateFolder> list = new ArrayList<>();
            List<FolderMapping> folderMappingList = new ArrayList<>();
            Date date = DateUtil.now();
            for (TemplateFolderSortVO template : templateFolderSortVOList) {
                //文件夹排序
                TemplateFolder templateFolder = new TemplateFolder();
                BeanUtil.copyProperties(template, templateFolder);
                templateFolder.setGmtModified(date);
                templateFolder.setModifier(templateFolderSortVO.getDoctorId().toString());
                list.add(templateFolder);

                //模板排序
                if (ListUtil.isNotEmpty(template.getTemplateSortVOList())) {
                    List<TemplateSortVO> templateSortVOList = template.getTemplateSortVOList();
                    for (TemplateSortVO templateSortVO : templateSortVOList) {
                        FolderMapping folderMapping = new FolderMapping();
                        BeanUtil.copyProperties(templateSortVO, folderMapping);
                        folderMapping.setGmtModified(date);
                        folderMapping.setModifier(templateFolderSortVO.getDoctorId().toString());
                        folderMappingList.add(folderMapping);
                    }
                }
            }
            if (ListUtil.isNotEmpty(list)){
                templateFolderService.updateBatchById(list);
            }
            if (ListUtil.isNotEmpty(folderMappingList)){
                folderMappingService.updateBatchById(folderMappingList);
            }
        }
    }
}
