package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.TranServiceClient;
import com.diagbot.dto.DoctorInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.TemplateInfo;
import com.diagbot.entity.TemplateQuote;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.TemplateTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.TemplateQuoteServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.TemplateQuoteVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @author gaodm
 * @Description:
 * @date 2020-01-09 18:29
 */
@Component
public class TemplateQuoteFacade extends TemplateQuoteServiceImpl {

    @Autowired
    private TemplateInfoFacade templateInfoFacade;
    @Autowired
    private TranServiceClient tranServiceClient;

    /**
     * 添加模板引用
     *
     * @param templateQuoteVO
     * @return
     */
    public Boolean addTemplateQuote(TemplateQuoteVO templateQuoteVO) {
        //模板存在性验证
        QueryWrapper<TemplateInfo> templateInfoQuery = new QueryWrapper<>();
        templateInfoQuery
                .eq("id", templateQuoteVO.getTemplateId())
                .eq("is_deleted", IsDeleteEnum.N.getKey());
        TemplateInfo templateInfo = templateInfoFacade.getOne(templateInfoQuery, false);
        if (null == templateInfo) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该模板不存在！");
        }
        //医生存在性验证
        RespDTO<DoctorInfoDTO> res
                = tranServiceClient.getDoctorInfoById(templateQuoteVO.getDoctorId());
        RespDTOUtil.respNGDeal(res, "该医生不存在！");
        //逻辑验证
        if (templateInfo.getTemplateType().equals(TemplateTypeEnum.PersonalTemplate.getKey())) {
            if (!templateInfo.getDoctorId().equals(templateQuoteVO.getDoctorId())) {
                throw new CommonException(CommonErrorCode.NO_PERMISSION, "无权引用该模板！");
            }
        }

        //引用模板
        Date now = DateUtil.now();
        TemplateQuote templateQuote = new TemplateQuote();
        BeanUtil.copyProperties(templateQuoteVO, templateQuote);
        templateQuote.setCreator(templateQuoteVO.getDoctorId().toString());
        templateQuote.setModifier(templateQuoteVO.getDoctorId().toString());
        templateQuote.setGmtCreate(now);
        templateQuote.setGmtModified(now);

        boolean resSave = this.save(templateQuote);
        if (!resSave) {
            throw new CommonException(CommonErrorCode.INSERT_DATA_FAILED, "引用模板记录失败！");
        }

        return resSave;
    }
}
