package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.biz.push.entity.Medicition;
import com.diagbot.biz.push.entity.Treat;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.bean.DisTypeVO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.InquiryDrug;
import com.diagbot.entity.InquiryInfo;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ExistListByConceptIdsVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/12/17 13:41
 */
@Component
public class TreatmentFacade {
    @Autowired
    QuestionFacade questionFacade;
    @Autowired
    InquiryInfoFacade inquiryInfoFacade;
    @Autowired
    InquiryDrugFacade inquiryDrugFacade;
    @Autowired
    AiptServiceClient aiptServiceClient;

    /**
     * 获取治疗方案
     *
     * @param
     * @return
     */
    public Treat getTreatment(Treat treat, Long patientId) {
        //慢病 上次用药处理
        if (treat.getDisType() != null && treat.getDisType().equals(1) && patientId != null) {
            treat.setDrugHistory(getDrugHistory(patientId));
        } else {
            treat.setDrugHistory(null);
        }
        return treat;
    }


    /**
     * 根据病人id获取历史用药内容
     *
     * @param patientId
     * @return
     */
    public Map<String, List<Medicition>> getDrugHistory(Long patientId) {
        Map<String, List<Medicition>> drugHistoryMap = new LinkedHashMap<>();
        QueryWrapper<InquiryInfo> inquiryInfoQueryWrapper = new QueryWrapper<>();
        inquiryInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                eq("patient_id", patientId).
                orderByDesc("gmt_modified");
        List<InquiryInfo> inquiryInfoList = inquiryInfoFacade.list(inquiryInfoQueryWrapper);

        //是否慢病病历
        List<String> firstDiseaseNameList = Lists.newLinkedList();
        for (InquiryInfo inquiryInfo : inquiryInfoList) {
            String[] diseases = inquiryInfo.getDiagnose().split(",|，");
            if (diseases.length > 0) {
                firstDiseaseNameList.add(diseases[0]);
            }
        }
        Boolean chronicHistroy = false;
        Boolean not_chronicHistory = false;
        for (InquiryInfo inquiryInfo : inquiryInfoList) {
            //慢病和普通病都取到上次用药之后不再遍历
            if (chronicHistroy && not_chronicHistory) {
                break;
            }
            String[] diseases = inquiryInfo.getDiagnose().split(",|，");
            if (diseases.length > 0) {
                //判断是否慢病病历
                DisTypeVO disTypeVO = new DisTypeVO();
                disTypeVO.setName(diseases[0]);
                disTypeVO.setDisType(DisTypeEnum.CHRONIC.getKey());
                RespDTO<Boolean> resDisType = aiptServiceClient.verifyDisType(disTypeVO);
                Boolean isChronic = false;
                if (RespDTOUtil.respIsOK(resDisType) && resDisType.data == true) {
                    isChronic = true;
                }
                //当前病历是慢病病历且上次慢病用药已取到数据
                if (isChronic && chronicHistroy) {
                    continue;
                }
                //当前病历是普通病病历且上次普通病用药已取到数据
                if (!isChronic && not_chronicHistory) {
                    continue;
                }
                QueryWrapper<InquiryDrug> inquiryDrugQueryWrapper = new QueryWrapper<>();
                inquiryDrugQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("inquiry_id", inquiryInfo.getId());
                List<InquiryDrug> inquiryDrugList = inquiryDrugFacade.list(inquiryDrugQueryWrapper);
                //当前病历没有开药
                if (ListUtil.isEmpty(inquiryDrugList)) {
                    continue;
                }
                List<Medicition> medicitionList = Lists.newLinkedList();
                List<Long> conceptIds = inquiryDrugList
                        .stream()
                        .map(inquiryDrug -> inquiryDrug.getConceptId()).collect(Collectors.toList());
                ExistListByConceptIdsVO existListByConceptIds = new ExistListByConceptIdsVO();
                existListByConceptIds.setConceptIds(conceptIds);
                RespDTO<List<Long>> res = aiptServiceClient.existListByConceptIds(existListByConceptIds);
                List<Long> retConceptIds = Lists.newLinkedList();
                if (RespDTOUtil.respIsOK(res)) {
                    retConceptIds = res.data;
                }
                for (InquiryDrug inquiryDrug : inquiryDrugList) {
                    Medicition medicition = new Medicition();
                    medicition.setConceptId(inquiryDrug.getConceptId());
                    medicition.setMedicitionName(inquiryDrug.getConceptName());
                    medicition.setType(ConceptTypeEnum.Drug.getKey());
                    medicition.setIsShow("1");
                    ConceptIntroduceVO conceptIntroduceVO = new ConceptIntroduceVO();
                    conceptIntroduceVO.setName(medicition.getMedicitionName());
                    conceptIntroduceVO.setType(ConceptTypeEnum.Drug.getKey());
                    if (ListUtil.isNotEmpty(retConceptIds) && retConceptIds.contains(inquiryDrug.getConceptId())) {
                        medicition.setShowInfo("1");
                    } else {
                        medicition.setShowInfo("0");
                    }
                    medicitionList.add(medicition);
                }

                if (isChronic) {
                    //慢病上次用药
                    if (!chronicHistroy) {
                        drugHistoryMap.put("慢病用药内容", medicitionList);
                        chronicHistroy = true;
                    }
                } else {
                    //普通病上次用药
                    if (!not_chronicHistory) {
                        drugHistoryMap.put("普通病用药内容", medicitionList);
                        not_chronicHistory = true;
                    }
                }
            }
        }
        return drugHistoryMap;
    }
}