package com.diagbot.vo;

import java.util.Date;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import org.hibernate.validator.constraints.Range;
import org.springframework.format.annotation.DateTimeFormat;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * @Description:
 * @author: rengb
 * @time: 2018/11/19 18:58
 */
@ApiModel(value="历史病历列表接口传参")
@Getter
@Setter
public class HisInquirysVO {
	
    /**
     * 每页显示条数，默认 10
     */
    private long size = 10;
    
    /**
     * 当前页
     */
    private long current = 1;
	
	/**
     * 医院id
     */
	@ApiModelProperty(value="医院id",required=true)
	@NotNull(message="医院id必传")
    private Long hospitalId;

    /**
     * 科室id
     */
	@ApiModelProperty(value="科室id")
	//@NotNull(message="科室id必传")
    private Long hospitalDeptId;

    /**
     * 医生id
     */
	@ApiModelProperty(value="医生id")
	//@NotNull(message="医生id必传")
    private Long doctorId;

    /**
     * 患者id
     */
	@ApiModelProperty(value="患者id",required=true)
	@NotNull(message="患者id必传")
    private Long patientId;
    
    /**
     * 病历分类(1:门诊,2:住院)
     */
	@ApiModelProperty(value="病历分类(1:门诊,2:住院)")
    private Integer type;
    
    /**
     * 内容输入类型：0：结构化 1：文本模式
     */
	@ApiModelProperty(value="内容输入类型：0：结构化 1：文本模式")
    private Integer sign;
    
	/**
	 * 开始时间
	 */
	@ApiModelProperty(value="开始时间，格式为：2018-11-28 17:25:30")
	@DateTimeFormat(pattern="yyyy-MM-dd hh:mm:ss")
	private Date startDate;
	
	/**
	 * 结束时间
	 */
	@ApiModelProperty(value="结束时间，格式为：2018-11-28 17:25:30")
	@DateTimeFormat(pattern="yyyy-MM-dd hh:mm:ss")
	private Date endDate;
	
	/**
	 * 排序类型：1、时间降序；2、时间升序； 
	 */
	@ApiModelProperty(value="排序类型：1、时间降序；2、时间升序；")
	private Integer orderType=1;
	
	/**
	 * 诊断类型：1：慢病，2：急诊
	 */
	@NotNull(message="诊断类型必填")
	@ApiModelProperty(value="诊断类型：0：通用，1：慢病，2：急诊")
	@Range(min=0,max=2,message="诊断类型必须是0-2")
	private Integer disType;
	
	/**
	 * 诊断名称
	 */
	@NotBlank(message="诊断名称必填")
	@ApiModelProperty(value="诊断名称",required=true)
	private String disName;
	   
}
