package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.LisDataDTO;
import com.diagbot.dto.LisRetrieveListAllDTO;
import com.diagbot.dto.PacsDataDTO;
import com.diagbot.dto.PacsRetrieveListAllDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.DockingImportFacade;
import com.diagbot.vo.LisArgumentsVO;
import com.diagbot.vo.LisGroupNameVO;
import com.diagbot.vo.PacsArgumentsVO;
import com.diagbot.vo.PacsCheckItemVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @author wangfeng
 * @Description: TODO
 * @date 2019年5月27日 上午11:06:55
 */
@RestController
@RequestMapping("/dockingImport")
@Api(value = "对接导入API[by:wangfeng]", tags = { "ICSS三期-对接导入API" })
@SuppressWarnings("unchecked")
public class DockingImportController {

    @Autowired
    DockingImportFacade dockingImportFacade;


    @ApiOperation(value = "ICSS三期-导入化验数据检索:[by:wangfeng]", notes = "导入化验数据检索")
    @PostMapping("/lisDockingImports")
    @SysLogger("lisDockingImports")
    public RespDTO<LisRetrieveListAllDTO> lisDockingImports(@RequestBody @Valid LisArgumentsVO lisArgumentsVO) {
        return RespDTO.onSuc(dockingImportFacade.lisDockingImport(lisArgumentsVO));
    }

    @ApiOperation(value = "ICSS三期-根据组名称获取化验数据:[by:wangfeng]", notes = "根据组名称获取化验数据")
    @PostMapping("/getByGroupNameLisData")
    @SysLogger("getByGroupNameLisData")
    public RespDTO<LisDataDTO> getByGroupNameLisDatas(@RequestBody @Valid LisGroupNameVO lisGroupNameVO) {
        return RespDTO.onSuc(dockingImportFacade.getByGroupNameLisDatas(lisGroupNameVO));
    }

    @ApiOperation(value = "ICSS三期-导入辅件数据检索:[by:wangfeng]", notes = "导入辅件数据检索")
    @PostMapping("/pacsDockingImport")
    @SysLogger("pacsDockingImport")
    public RespDTO<PacsRetrieveListAllDTO> pacsDockingImports(@RequestBody @Valid PacsArgumentsVO pacsArgumentsVO) {
        return RespDTO.onSuc(dockingImportFacade.pacsDockingImports(pacsArgumentsVO));
    }

    @ApiOperation(value = "ICSS三期-根据组名称获取辅件数据:[by:wangfeng]", notes = "根据组名称获取辅件数据")
    @PostMapping("/getByCheckItemPacsData")
    @SysLogger("getByCheckItemPacsData")
    public RespDTO<PacsDataDTO> getByCheckItemPacsDatas(@RequestBody @Valid PacsCheckItemVO pacsGroupNameVO) {
        return RespDTO.onSuc(dockingImportFacade.getByCheckItemPacsDatas(pacsGroupNameVO));
    }

}
