package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.DoctorInfoDTO;
import com.diagbot.dto.HospitalDeptInfoAllDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.DoctorInfoFacade;
import com.diagbot.vo.DoctorInfoVO;
import com.diagbot.vo.HospitalCodeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 医生信息表 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2018-11-19
 */
@RestController
@RequestMapping("/doctorInfo")
@Api(value = "医生信息API", tags = { "医生信息API" })
@SuppressWarnings("unchecked")
//@ApiIgnore
//@Deprecated
public class DoctorInfoController {

    @Autowired
    private DoctorInfoFacade doctorInfoFacade;

    @ApiOperation(value = "医生信息——查询[by:wangyu]",
            notes = "deptCode：科室编号，必填<br>" +
                    "hospitalCode: 医院编号，必填<br>" +
                    "doctorCode: 医生编号，必填<br>")
    @PostMapping("/getDoctorInfo")
    @SysLogger("getDoctorInfo")
    @ApiIgnore
    public RespDTO<List<DoctorInfoDTO>> getDoctorInfo(@Valid @RequestBody DoctorInfoVO doctorInfoVo) {
        List<DoctorInfoDTO> data = doctorInfoFacade.getDoctorInfo(doctorInfoVo);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院编码搜索医院所有科室信息[by:wangfeng] ", notes = "hospitalCode：医院编码<br>")
    @PostMapping("/getHospitalDeptInfoAll")
    @SysLogger("getHospitalDeptInfoAll")
    public RespDTO<List<HospitalDeptInfoAllDTO>> getHospitalDeptInfoAll(@RequestBody @Valid HospitalCodeVo hospitalCodeVo) {
        List<HospitalDeptInfoAllDTO> data = doctorInfoFacade.getHospitalDeptInfoAll(hospitalCodeVo);
        return RespDTO.onSuc(data);
    }
}
