package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.facade.RetrievalFacade;
import com.diagbot.vo.GetStaticVO;
import com.diagbot.vo.RetrievalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 同义词检索检索表 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2018-11-16
 */
@RestController
@RequestMapping("/retrieval")
@Api(value = "页面检索API", tags = { "ICSS三期/知识库标准化-页面检索API" })
@SuppressWarnings("unchecked")
public class RetrievalController {
    @Autowired
    private RetrievalFacade retrievalFacade;

    @ApiOperation(value = "知识库标准化-页面检索——查询[by:wangyu]",
            notes = "type：类型(1：症状 3：其他史 4：查体，5：化验 6：辅检 7：诊断)，必填<br>" +
                    "age：年龄，必填<br>" +
                    "inputStr：检索内容，必填<br>" +
                    "sexType：性别，必填<br>" +
                    "inputIds：需要去重的conceptId<br>" +
                    "tagType: 标签标识<br>" +
                    "itemType: 元素类型<br>")
    @PostMapping("/getTagInfos")
    @SysLogger("getTagInfos")
    public RespDTO<List<RetrievalDTO>> getTagInfos(@Valid @RequestBody RetrievalVO retrievalVO) {
        List<RetrievalDTO> data = retrievalFacade.getTagInfos(retrievalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "ICSS三期-静态知识-页面检索[by:wangyu]",
            notes = "inputStr：检索内容，必填<br>" +
                    "inputIds：需要去重的id<br>" +
                    "types：指定类型，1：症状，10：药品，12：化验，16：辅检，18：诊断，25：手术, 48：量表；不指定：以上全部")
    @PostMapping("/getStaticKnowledge")
    @SysLogger("getStaticKnowledge")
    public RespDTO<List<RetrievalDTO>> getStaticKnowledge(@Valid @RequestBody GetStaticVO getStaticVO) {
        List<RetrievalDTO> data = retrievalFacade.getStaticKnowledge(getStaticVO);
        return RespDTO.onSuc(data);
    }
}
