package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.TemplateFolder;
import com.diagbot.facade.TemplateFolderAdminFacade;
import com.diagbot.facade.TemplateFolderFacade;
import com.diagbot.vo.TemplateFolderDelVO;
import com.diagbot.vo.TemplateFolderListVO;
import com.diagbot.vo.TemplateFolderSortObjVO;
import com.diagbot.vo.TemplateFolderSortVO;
import com.diagbot.vo.TemplateFolderVO;
import com.sun.org.apache.xpath.internal.operations.Bool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 模板文件夹表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2020-01-09
 */
@RestController
@RequestMapping("/templateFolderAdmin")
@Api(value = "病历模板文件夹管理员API", tags = { "病历模板文件夹管理员API" })
public class TemplateFolderAdminController {

    @Autowired
    TemplateFolderAdminFacade templateFolderAdminFacade;

    @ApiOperation(value = "文件夹新增或更新[by:zhoutg]",
            notes = "")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @Transactional
    public RespDTO<Boolean> saveOrUpdate(@Valid @RequestBody TemplateFolderVO templateFolderVO) {
        templateFolderAdminFacade.saveOrUpdate(templateFolderVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "文件夹删除[by:zhoutg]",
            notes = "folderId: 文件夹id<br>")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@RequestBody TemplateFolderDelVO templateFolderDelVO) {
        templateFolderAdminFacade.delete(templateFolderDelVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "文件夹列表[by:zhoutg]",
            notes = "")
    @PostMapping("/list")
    @SysLogger("list")
    public RespDTO<List<TemplateFolder>> list(@RequestBody TemplateFolderListVO templateFoldeListVO) {
        List<TemplateFolder> data = templateFolderAdminFacade.list(templateFoldeListVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "文件夹排序[by:zhoutg]",
            notes = "")
    @PostMapping("/sort")
    @SysLogger("sort")
    @Transactional
    public RespDTO<Boolean> sort(@RequestBody TemplateFolderSortObjVO templateFolderSortObjVO) {
        templateFolderAdminFacade.sort(templateFolderSortObjVO.getTemplateFolderSortVOList());
        return RespDTO.onSuc(true);
    }
}
