package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TemplateInfoDTO;
import com.diagbot.dto.TemplateInfoPageDTO;
import com.diagbot.facade.TemplateAdminFacade;
import com.diagbot.vo.TemplateAdminIdsVO;
import com.diagbot.vo.TemplateIdVO;
import com.diagbot.vo.TemplateInfoAdminPageVO;
import com.diagbot.vo.TemplateInfoRevampVO;
import com.diagbot.vo.TemplateInfosVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

/**
 * @author wangfeng
 * @Description:
 * @date 2019-12-09 18:37
 */
@RestController
@RequestMapping("/templateAdmin")
@Api(value = "管理员病历模板API[by:wangfeng]", tags = { "WF——管理员病历模板API" })
@SuppressWarnings("unchecked")
@ApiIgnore
public class TemplateInfoAdminController {

    @Autowired
    TemplateAdminFacade templateAdminFacade;

    @ApiOperation(value = "根据模板id修改模板名称[by:wangfeng]", notes = "id;//模板</br>modeName;"
            + "//模板名</br>doctorId;//医生id</br>hospitalId;//医院id</br>hospitalDeptId;//部门id</br>")
    @PostMapping("/updateByAdminNames")
    @SysLogger("updateByAdminNames")
    @Transactional
    public RespDTO<Boolean> updateByAdminNames(@Valid @RequestBody TemplateInfoRevampVO templateInfoRevampVO) {

        boolean res = templateAdminFacade.updateByIdUsName(templateInfoRevampVO);

        return RespDTO.onSuc(res);
    }


    @ApiOperation(value = "保存病历模板[by:wangfeng]", notes = "modeName;//模板名字</br>" +
            "doctorId;//医生id</br>hospitalId;//医院id</br>hospitalDeptId;//部门id</br>" +
            "modeType;//模板类型</br>preview;//预览文本</br>" +
            "{chief;//主诉present;//现病史other;//其他史vital;//查体lis;//化验pacs;//辅检diag;//诊断advice;//医嘱}</br>" +
            "dataJson;//模板数据</br>")
    @PostMapping("/saveTemplateAdmin")
    @SysLogger("saveTemplateAdmin")
    @Transactional
    public RespDTO<Boolean> saveTemplateAdmin(@Valid @RequestBody TemplateInfosVO templateInfosVO) {

        boolean res = templateAdminFacade.saveTemplateInfo(templateInfosVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "根据模板id批量删除病历模板[by:wangfeng]", notes = "ids: 模板ids<br>'ids':'1,2,3,4,5,6'")
    @PostMapping("/cancelTemplateAdminb")
    @SysLogger("cancelTemplateAdminb")
    @Transactional
    public RespDTO<Boolean> cancelTemplateAdminb(@Valid @RequestBody TemplateAdminIdsVO templateInfosIdVO) {

        boolean res = templateAdminFacade.cancelTemplateInfo(templateInfosIdVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "分页查询病历模板列表[by:wangfeng]",
            notes = "doctorId: 医生id<br>"
                    + "dospitalId：医院编码<br>"
                    + "deptId:部门id<br>"
                    + "type:模式<br>"
                    + "sex：性别<br>"
                    + "name：模板名称")
    @PostMapping("/getTemplatePageAlls")
    @SysLogger("getTemplatePageAlls")
    public RespDTO<IPage<TemplateInfoPageDTO>> getTemplatePageAll(@Valid @RequestBody TemplateInfoAdminPageVO templateInfoPageVO) {

        IPage<TemplateInfoPageDTO> data = templateAdminFacade.getTemplatePageAlls(templateInfoPageVO);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据模板id获取病历模板内容[by:wangfeng]", notes = "id: 请输入模板id")
    @PostMapping("/getTemplateIdAlls")
    @SysLogger("getTemplateIdAlls")
    public RespDTO<TemplateInfoDTO> getTemplateIdAll(@Valid @RequestBody TemplateIdVO templateIdVO) {

        TemplateInfoDTO data = templateAdminFacade.getTemplateIdAlls(templateIdVO);

        return RespDTO.onSuc(data);
    }
}
