package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TemplateFolderDTO;
import com.diagbot.dto.TemplateInfoDTO;
import com.diagbot.dto.TemplateInfoNormDTO;
import com.diagbot.facade.TemplateNormFacade;
import com.diagbot.vo.TemplateIdVO;
import com.diagbot.vo.TemplateNormUpVO;
import com.diagbot.vo.TemplateNormsFindVO;
import com.diagbot.vo.TemplateNormsIdsVO;
import com.diagbot.vo.TemplateNormsParamVO;
import com.diagbot.vo.TemplateNormsVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-01-10 14:21
 */
@RestController
@RequestMapping("/templateAdmin")
@Api(value = "管理员标准模板API[by:wangfeng]", tags = { "WF——管理员标准模板API" })
@SuppressWarnings("unchecked")
public class TemplateNormController {

    @Autowired
    TemplateNormFacade templateNormFacade;

    @ApiOperation(value = "根据模板id修改模板名称和模板名首拼及文件目录[by:wangfeng]")
    @PostMapping("/updateTemplateNorms")
    @SysLogger("updateTemplateNorms")
    @Transactional
    public RespDTO<Boolean> updateTemplateNorms(@Valid @RequestBody TemplateNormUpVO templateNormUpVO) {

        boolean res = templateNormFacade.updateTemplateNorm(templateNormUpVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "保存病历模板[by:wangfeng]")
    @PostMapping("/saveTemplateNorms")
    @SysLogger("saveTemplateNorms")
    @Transactional
    public RespDTO<Boolean> saveTemplateNorms(@Valid @RequestBody TemplateNormsVO templateNormsVO) {

        boolean res = templateNormFacade.saveTemplateNorm(templateNormsVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "根据模板id批量删除病历模板[by:wangfeng]", notes = "ids: 模板ids<br>'ids':'1,2,3,4,5,6'")
    @PostMapping("/cancelTemplateNorms")
    @SysLogger("cancelTemplateNorms")
    @Transactional
    public RespDTO<Boolean> cancelTemplateNorms(@Valid @RequestBody TemplateNormsIdsVO TemplateNormsIdsVO) {

        boolean res = templateNormFacade.cancelTemplateInfo(TemplateNormsIdsVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "查询标准模板列表[by:wangfeng]")
    @PostMapping("/getTemplateNorms")
    @SysLogger("getTemplateNorms")
    public RespDTO<List<TemplateFolderDTO>> getTemplateNorms() {

        List<TemplateFolderDTO> data = templateNormFacade.getTemplateNorm();

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据模板id获取病历模板内容[by:wangfeng]", notes = "id: 请输入模板id")
    @PostMapping("/getTemplateIdDatas")
    @SysLogger("getTemplateIdDatas")
    public RespDTO<TemplateInfoDTO> getTemplateIdDatas(@Valid @RequestBody TemplateIdVO templateIdVO) {

        TemplateInfoDTO data = templateNormFacade.getTemplateIdData(templateIdVO);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据名字和拼音查询标准模板列表[by:wangfeng]")
    @PostMapping("/findTemplateNorms")
    @SysLogger("findTemplateNorms")
    public RespDTO<List<TemplateInfoNormDTO>> findTemplateNorms(@Valid @RequestBody TemplateNormsFindVO templateNormsFindVO) {

        List<TemplateInfoNormDTO> data = templateNormFacade.findTemplateNorm(templateNormsFindVO);

        return RespDTO.onSuc(data);
    }
}
