package com.diagbot.client;

import com.diagbot.client.hystrix.KnowledgemanServiceHystrix;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.vo.ConceptExistVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptTypeVO;
import com.diagbot.vo.GetConceptPacInfosVO;
import com.diagbot.vo.IndexLexiconVO;
import com.diagbot.vo.IndexVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;
import java.util.Map;

/**
 * @Description:调用中间层服务
 * @Author: Mark Huang
 * @time: 2019/03/25 19:15
 */
@FeignClient(value = "knowledgeman-service", fallback = KnowledgemanServiceHystrix.class)
public interface KnowledgemanServiceClient {

    @PostMapping("/commonconcept/index")
    RespDTO<List<ConceptBaseDTO>> index(@RequestBody IndexVO indexVO);

    @PostMapping("/commonconcept/indexByLexicon")
    RespDTO<List<ConceptBaseDTO>> indexByLexiconFac(@RequestBody IndexLexiconVO indexLexiconVO);

    @PostMapping("/commonconcept/getConceptMapByNameAndType")
    RespDTO<Map<String, List<Integer>>> getConceptMap(@RequestBody ConceptExistVO conceptExistVO);

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @PostMapping("/concept/getConceptMap")
    RespDTO<Map<Long, String>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO);

    /**
     * 根据类型获取术语列表（科室、辅检、慢病）
     *
     * @param conceptTypeVO 类型 type: 1:科室、2:辅检、3:慢病
     * @return 术语列表
     */
    @PostMapping("/concept/getConceptListByType")
    RespDTO<List<ConceptBaseDTO>> getConceptListByType(@RequestBody ConceptTypeVO conceptTypeVO);

    /**
     * 模板维护获取辅检标签
     *
     * @param getConceptPacInfosVO
     * @return
     */
    @PostMapping("/concept/getConceptPacInfos")
    RespDTO<List<ConceptBaseDTO>> getConceptPacInfos(@RequestBody GetConceptPacInfosVO getConceptPacInfosVO);
}