package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.KnowledgemanServiceClient;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.DeptShortDTO;
import com.diagbot.dto.DeptVitalDTO;
import com.diagbot.dto.QuestionShortDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.DeptVital;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.DeptVitalService;
import com.diagbot.service.impl.DeptVitalServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptTypeVO;
import com.diagbot.vo.DVDetailVO;
import com.diagbot.vo.DeptVitalPageVO;
import com.diagbot.vo.DeptVitalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/22 11:45
 */
@Component
public class DeptVitalFacade extends DeptVitalServiceImpl {

    @Autowired
    private QuestionFacade questionFacade;
    @Autowired
    @Qualifier("deptVitalServiceImpl")
    private DeptVitalService deptVitalService;
    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private KnowledgemanServiceClient klmServiceClient;

    /**
     * 保存查体模板
     *
     * @param deptVitalVO
     * @return
     */
    public Boolean saveDeptVitals(DeptVitalVO deptVitalVO) {
        //科室存在性验证
        isExist(deptVitalVO.getDeptId());
        //验证模板是否已存在
        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper.eq("dept_id", deptVitalVO.getDeptId()).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        if (this.count(deptVitalQueryWrapper) > 0) {
            //先删除该科室原有模板
            UpdateWrapper<DeptVital> deptVitalUpdateWrapper = new UpdateWrapper<>();
            deptVitalUpdateWrapper.eq("dept_id", deptVitalVO.getDeptId()).
                    eq("is_deleted", IsDeleteEnum.N.getKey()).
                    set("is_deleted", IsDeleteEnum.Y.getKey()).
                    set("modifier", UserUtils.getCurrentPrincipleID()).
                    set("gmt_modified", DateUtil.now());
            this.update(new DeptVital(), deptVitalUpdateWrapper);
        }

        List<Long> vitalIds = deptVitalVO.getDvDetailVOList()
                .stream()
                .map(deDetailVO -> deDetailVO.getVitalId()).collect(Collectors.toList());
        if (ListUtil.isNotEmpty(vitalIds)) {
            //查找标签是否存在
            QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
            questionInfoQueryWrapper.in("id", vitalIds).
                    eq("type", QuestionTypeEnum.Vital.getKey()).
                    eq("is_deleted", IsDeleteEnum.N.getKey());
            List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);

            Map<Long, QuestionInfo> questionInfoMap = EntityUtil.makeEntityMap(questionInfoList, "id");
            for (Long vital : vitalIds) {
                if (questionInfoMap.get(vital) == null) {
                    throw new CommonException(CommonErrorCode.NOT_EXISTS, "id=" + vital + "的查体标签不存在");
                }
            }

            //插入新模板
            List<DeptVital> deptVitalList = Lists.newArrayList();
            Date now = DateUtil.now();
            String userId = UserUtils.getCurrentPrincipleID();
            for (DVDetailVO dvDetailVO : deptVitalVO.getDvDetailVOList()) {
                DeptVital deptVital = new DeptVital();
                deptVital.setDeptId(deptVitalVO.getDeptId());
                deptVital.setVitalId(dvDetailVO.getVitalId());
                deptVital.setRemark(dvDetailVO.getRemark());
                deptVital.setOrderNo(dvDetailVO.getOrderNo());
                deptVital.setCreator(userId);
                deptVital.setGmtCreate(now);
                deptVital.setModifier(userId);
                deptVital.setGmtModified(now);
                deptVitalList.add(deptVital);
            }

            deptVitalService.saveBatch(deptVitalList);
        }
        return true;
    }

    /**
     * 删除科室查体模板标签-全删除
     *
     * @param deptId
     * @return
     */
    public Boolean delAllDeptVitalList(Long deptId) {
        UpdateWrapper<DeptVital> deptVitalUpdateWrapper = new UpdateWrapper<>();
        deptVitalUpdateWrapper.eq("dept_id", deptId).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        this.update(new DeptVital(), deptVitalUpdateWrapper);
        return true;
    }

    /**
     * 根据部门获取查体模板
     *
     * @param deptId
     * @return
     */
    public DeptVitalDTO getModuleByDeptId(Long deptId) {
        //科室存在性验证
        Map<Long, String> deptMap = isExist(deptId);
        DeptVitalDTO deptVitalDTO = new DeptVitalDTO();
        deptVitalDTO.setDeptId(deptId);
        deptVitalDTO.setName(deptMap.get(deptId));

        //已关联查体
        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper.eq("dept_id", deptId).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                orderByAsc("order_no");
        List<DeptVital> deptVitalList = this.list(deptVitalQueryWrapper);
        List<Long> vitalIds = deptVitalList.stream()
                .map(deptVitals -> deptVitals.getVitalId())
                .collect(Collectors.toList());
        if (vitalIds.size() > 0) {
            QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
            questionInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("id", vitalIds);
            List<QuestionInfo> vitalList = questionFacade.list(questionInfoQueryWrapper);
            Map<Long, QuestionInfo> vitalMap = EntityUtil.makeEntityMap(vitalList, "id");
            List<QuestionShortDTO> vitals = Lists.newArrayList();
            for (Long vitalId : vitalIds) {
                if (!vitalMap.containsKey(vitalId)) {
                    continue;
                }
                QuestionShortDTO vital = new QuestionShortDTO();
                BeanUtil.copyProperties(vitalMap.get(vitalId), vital);
                vitals.add(vital);
            }
            deptVitalDTO.setVitals(vitals);
        }

        //未关联查体
        QueryWrapper<QuestionInfo> unSelectedVitalQueryWrapper = new QueryWrapper<>();
        unSelectedVitalQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                eq("sub_type", 0).
                eq("type", QuestionTypeEnum.Vital.getKey());
        if (vitalIds.size() > 0) {
            unSelectedVitalQueryWrapper.notIn("id", vitalIds);
        }
        List<QuestionInfo> unSelectedVitalList = questionFacade.list(unSelectedVitalQueryWrapper);
        List<QuestionShortDTO> unSelectedVitals = BeanUtil.listCopyTo(unSelectedVitalList, QuestionShortDTO.class);
        deptVitalDTO.setUnSelectedVitals(unSelectedVitals);
        return deptVitalDTO;
    }


    /**
     * 获取科室模板分页信息
     *
     * @param deptVitalPageVO
     * @return
     */
    public IPage<DeptVitalDTO> getDeptVitalPageByMap(DeptVitalPageVO deptVitalPageVO) {
        IPage<DeptVitalDTO> page = this.getDeptVitalPage(deptVitalPageVO);
        List<DeptVitalDTO> deptVitalDTOList = page.getRecords();
        //操作人信息
        List<String> userIds = deptVitalDTOList
                .stream()
                .map(deptVitalDTO -> deptVitalDTO.getOperator()).collect(Collectors.toList());
        RespDTO<Map<String, String>> data = userServiceClient.getUserInfoByIds(userIds);
        Map<String, String> userInfos = data.data;
        //科室信息
        List<Long> deptIds = deptVitalDTOList
                .stream()
                .map(deptVitalDTO -> deptVitalDTO.getDeptId()).collect(Collectors.toList());
        ConceptSearchVO conceptSearchVO = new ConceptSearchVO();
        conceptSearchVO.setConceptIds(deptIds);
        RespDTO<Map<Long, String>> deptMap = klmServiceClient.getConceptMap(conceptSearchVO);
        Map<Long, String> depts = deptMap.data;

        for (DeptVitalDTO deptVitalDTO : deptVitalDTOList) {
            if (null != userInfos.get(deptVitalDTO.getOperator())) {
                deptVitalDTO.setOperatorName(userInfos.get(deptVitalDTO.getOperator()));
            }
            if (null != depts.get(deptVitalDTO.getDeptId())) {
                deptVitalDTO.setName(depts.get(deptVitalDTO.getDeptId()));
            }
        }
        page.setRecords(deptVitalDTOList);
        return page;
    }

    /**
     * 获取科室下拉列表
     *
     * @return
     */
    public List<DeptShortDTO> getDeptShortList(List<Long> deptIds) {
        //获取所有科室
        ConceptTypeVO conceptTypeVO = new ConceptTypeVO();
        conceptTypeVO.setType(1);
        RespDTO<List<ConceptBaseDTO>> deptList = klmServiceClient.getConceptListByType(conceptTypeVO);
        RespDTOUtil.respNGDealCover(deptList, "科室不存在");
        //去除已经存在的科室
        List<DeptShortDTO> deptShortDTOList = new ArrayList<>();
        if (ListUtil.isNotEmpty(deptList.data)) {
            for (ConceptBaseDTO conceptBaseDTO : deptList.data) {
                if (!deptIds.contains(conceptBaseDTO.getConceptId())) {
                    DeptShortDTO deptShortDTO = new DeptShortDTO();
                    deptShortDTO.setId(conceptBaseDTO.getConceptId());
                    deptShortDTO.setName(conceptBaseDTO.getName());
                    deptShortDTOList.add(deptShortDTO);
                }
            }
        }
        return deptShortDTOList;
    }

    /**
     * 获取科室下拉列表-新增
     *
     * @return
     */
    public List<DeptShortDTO> getDeptShortLisCreate() {
        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                select("dept_id").
                groupBy("dept_id");
        List<DeptVital> deptVitalList = this.list(deptVitalQueryWrapper);
        List<Long> deptIds = deptVitalList
                .stream()
                .map(deptVital -> deptVital.getDeptId()).collect(Collectors.toList());
        return getDeptShortList(deptIds);
    }

    /**
     * 获取科室下拉列表-修改
     *
     * @return
     */
    public List<DeptShortDTO> getDeptShortLisModify(Long expId) {
        List<Long> deptIds = this.getAllExist();
        if (deptIds.contains(expId)) {
            deptIds.remove(expId);
            deptIds.size();
        }
        return getDeptShortList(deptIds);
    }

    /**
     * 获取科室下拉列表-查询
     *
     * @return
     */
    public List<DeptShortDTO> getDeptShortListSearch() {
        List<Long> deptIds = this.getAllExist();
        List<DeptShortDTO> res = ListUtil.newArrayList();
        if (ListUtil.isNotEmpty(deptIds)) {
            ConceptSearchVO conceptSearchVO = new ConceptSearchVO();
            conceptSearchVO.setConceptIds(deptIds);
            RespDTO<Map<Long, String>> deptMap = klmServiceClient.getConceptMap(conceptSearchVO);
            for (Long deptId : deptIds) {
                if (null != deptMap.data.get(deptId)) {
                    DeptShortDTO deptShortDTO = new DeptShortDTO();
                    deptShortDTO.setId(deptId);
                    deptShortDTO.setName(deptMap.data.get(deptId));
                    res.add(deptShortDTO);
                }
            }
        }

        return res;
    }

    /**
     * 获取所有已经存在的科室
     *
     * @return 已经存在的科室的Id列表
     */
    private List<Long> getAllExist() {
        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                select("dept_id").
                groupBy("dept_id");
        List<DeptVital> deptVitalList = this.list(deptVitalQueryWrapper);
        List<Long> deptIds = deptVitalList
                .stream()
                .map(deptVital -> deptVital.getDeptId()).collect(Collectors.toList());
        return deptIds;
    }

    /**
     * 存在性验证
     *
     * @param deptId 科室id
     * @return 科室信息Map
     */
    private Map<Long, String> isExist(Long deptId) {
        ConceptSearchVO conceptSearchVO = new ConceptSearchVO();
        List<Long> conceptIds = ListUtil.newArrayList();
        conceptIds.add(deptId);
        conceptSearchVO.setConceptIds(conceptIds);
        RespDTO<Map<Long, String>> deptMap = klmServiceClient.getConceptMap(conceptSearchVO);
        RespDTOUtil.respNGDealCover(deptMap, "科室不存在");
        if (null == deptMap.data.get(deptId)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "科室不存在");
        }
        return deptMap.data;
    }
}