package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.KnowledgemanServiceClient;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.GetQuestionInfoDTO;
import com.diagbot.dto.GetQuestiongAndModuleRelationsDTO;
import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.QuestionPageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.DeptVital;
import com.diagbot.entity.QuestionDetail;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.entity.QuestionMapping;
import com.diagbot.entity.wrapper.QuestionInfoWrapper;
import com.diagbot.entity.wrapper.QuestionMappingWrapper;
import com.diagbot.entity.wrapper.QuestionWrapper;
import com.diagbot.enums.InsertOrUpdateEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.enums.TagTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.QuestionDetailService;
import com.diagbot.service.QuestionInfoService;
import com.diagbot.service.QuestionMappingService;
import com.diagbot.service.impl.QuestionInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ConceptExistVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.DeleteQuestionVO;
import com.diagbot.vo.GetQuestionInfoVO;
import com.diagbot.vo.GetQuestiongAndModuleRelationsVO;
import com.diagbot.vo.IndexIcssVO;
import com.diagbot.vo.IndexLexiconVO;
import com.diagbot.vo.KLQuestionVO;
import com.diagbot.vo.QuestionIdsVO;
import com.diagbot.vo.QuestionIndexSubVO;
import com.diagbot.vo.QuestionIndexVO;
import com.diagbot.vo.QuestionPageVO;
import com.diagbot.vo.QuestionSaveVO;
import com.diagbot.vo.QuestionSpecVO;
import com.diagbot.vo.QuestionVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhoutg
 * @time: 2018/11/23 11:37
 */
@Component
public class QuestionFacade extends QuestionInfoServiceImpl {

    @Autowired
    QuestionDetailFacade questionDetailFacade;
    @Autowired
    @Qualifier("questionInfoServiceImpl")
    QuestionInfoService questionInfoService;
    @Autowired
    @Qualifier("questionDetailServiceImpl")
    QuestionDetailService questionDetailService;
    @Autowired
    @Qualifier("questionMappingServiceImpl")
    QuestionMappingService questionMappingService;
    @Autowired
    QuestionMappingFacade questionMappingFacade;
    @Autowired
    ModuleDetailFacade moduleDetailFacade;
    @Autowired
    UserServiceClient userServiceClient;
    @Autowired
    ModuleInfoFacade moduleInfoFacade;
    @Autowired
    DeptVitalFacade deptVitalFacade;
    @Autowired
    KnowledgemanServiceClient knowledgemanServiceClient;
    @Autowired
    CacheFacade cacheFacade;
    @Autowired
    private KnowledgemanServiceClient klmServiceClient;


    /**
     * 标签保存
     *
     * @param questionSaveVO
     */
    public void saveOrUpdate(QuestionSaveVO questionSaveVO) {
        QuestionWrapper questionWrapper = questionSaveVO.getQuestionWrapper();
        CommonParam param = initCommonParam();
        saveQuestionInfo(questionWrapper, param); //保存|修改主表信息
        switch (TagTypeEnum.getEnum(questionWrapper.getTagType())) {
            case T1:   //单项，例如：程度、体温
                saveQuestionDetail(questionWrapper, param);
                break;
            case T2:  //组合项，例如：杂音，修改主表和mapping
            case T3:  //组合项，例如：血压
            case T4:  //组合项，例如：咳嗽
            case T5:  //组合项，例如：有无治疗
            case T6:  // 组合项，例如：既往史
            case T7:  //化验
            case T11: //推送类型
                saveQuestionMapping(questionWrapper, param);
                break;
            case T8:  //文字，无单独维护
            case T9:  //空标签，无单独维护
            case T10: //组合，无单独维护
                break;
            default:
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "请选择标签显示类型");
        }

        // 删除标签缓存信息
        if (param.getInsertOrUpdate() == InsertOrUpdateEnum.Update.getKey()) {
            clearAllCacheByIds(questionWrapper.getId());
        }
    }


    /**
     * 删除关联的id信息
     *
     * @param ids
     */
    public void clearAllCacheByIds(List<Long> ids) {
        if (ListUtil.isNotEmpty(ids)) {
            for (Long id : ids) {
                cacheFacade.clearCache(id);
            }
        }
    }


    /**
     * 删除关联的id信息
     *
     * @param id
     */
    public void clearAllCacheByIds(Long id) {
        if (id != null) {
            List<Long> ids = getDeleteId(id);
            clearAllCacheByIds(ids);
        }
    }


    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        String person = UserUtils.getCurrentPrincipleID();
        param.setNow(DateUtil.now());
        param.setPerson(person);
        param.setInsertOrUpdate(InsertOrUpdateEnum.Insert.getKey());
        return param;
    }


    /**
     * 保存映射关系
     *
     * @param questionWrapper
     * @param param
     */
    public void saveQuestionMapping(QuestionWrapper questionWrapper, CommonParam param) {
        QuestionInfo questionInfo = param.getQuestionInfo();
        Date now = param.getNow();
        String person = param.getPerson();

        // 如果是既往史类型，考虑到整一列内容清空，原tagType=10下的映射关系就会遗留下来，
        // 变成脏数据，导致删除标签时报错
        // 将tagType=10下的所有映射关系删除
        if (InsertOrUpdateEnum.Update.getKey() == param.getInsertOrUpdate()
                && TagTypeEnum.T6.getKey() == questionWrapper.getTagType()) {
            Map<String, Object> map = new HashMap();
            map.put("id", questionInfo.getId());
            map.put("date", param.getNow());
            map.put("person", param.getPerson());
            deleteTagType10Mapping(map);
        }

        //先删除原明细，再插入新明细
        deleteMapping(questionInfo.getId(), param);//删除原映射关系
        List<QuestionMappingWrapper> questionMappings = questionWrapper.getQuestionMappings();
        if (ListUtil.isNotEmpty(questionMappings)) {
            for (QuestionMappingWrapper bean : questionMappings) {
                QuestionWrapper sonQues = bean.getQuestionMappingSon();
                if (ListUtil.isNotEmpty(sonQues.getQuestionMappings())) {
                    QuestionInfo ques = new QuestionInfo();
                    BeanUtil.copyProperties(sonQues, ques);
                    if (ques.getId() != null) {
                        ques = this.getOne(new QueryWrapper<QuestionInfo>()
                                .eq("is_deleted", IsDeleteEnum.N.getKey())
                                .eq("id", ques.getId()), false);
                        if (ques == null) {
                            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                                    "标签被修改，请刷新后重新保存");
                        }
                    } else {
                        String name = "组合项_" + DateUtil.format(now, DateUtil.FORMAT_FULL);
                        ques.setTagName(name);
                        ques.setName(name);
                        ques.setTagType(TagTypeEnum.T10.getKey());
                        ques.setGmtCreate(now);
                        ques.setCreator(person);
                        ques.setSubType(1);
                    }
                    ques.setType(questionInfo.getType());
                    ques.setGmtModified(now);
                    ques.setModifier(person);
                    this.saveOrUpdate(ques);

                    //删除原二级映射关系
                    deleteMapping(ques.getId(), param);

                    //添加二级映射关系
                    saveMapping(sonQues.getQuestionMappings(), new HashMap<String, Long>(), person, now, ques.getId());

                    //设置主映射关系
                    bean.setSonQuestion(ques.getId());
                    bean.setParentQuestion(questionInfo.getId());
                }
            }
        }
        if (ListUtil.isNotEmpty(questionMappings)) {
            //预处理文字标签，之后统一添加映射关系
            Map<String, Long> map = getNameMap(questionMappings, now, person, questionInfo.getType());
            saveMapping(questionMappings, map, person, now, questionInfo.getId());
        }
    }


    /**
     * 删除映射关系
     *
     * @param questionId
     * @param param
     */
    public void deleteMapping(Long questionId, CommonParam param) {
        questionMappingFacade.update(new QuestionMapping(),
                new UpdateWrapper<QuestionMapping>()
                        .eq("parent_question", questionId)
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("gmt_modified", param.getNow())
                        .set("modifier", param.getPerson())
                        .set("is_deleted", IsDeleteEnum.Y.getKey()));
    }


    /**
     * 保存映射关系
     *
     * @param questionMappings
     * @param map
     * @param person
     * @param now
     * @param id
     */
    public void saveMapping(List<QuestionMappingWrapper> questionMappings, Map<String, Long> map, String person,
                            Date now, Long id) {
        List<QuestionMapping> saveMapping = new ArrayList<>();
        int i = 1;
        for (QuestionMappingWrapper mapping : questionMappings) {
            String text = mapping.getText();
            if (StringUtil.isNotEmpty(text)) {
                mapping.setSonQuestion(map.get(text));
            }
            QuestionMapping bean = new QuestionMapping();
            BeanUtil.copyProperties(mapping, bean);
            bean.setId(null); //防止前端传参，将前端的id置空自动插入
            bean.setCreator(person);
            bean.setGmtCreate(now);
            bean.setModifier(person);
            bean.setGmtModified(now);
            bean.setParentQuestion(id);
            bean.setOrderNo(i++);
            saveMapping.add(bean);
        }
        questionMappingService.saveBatch(saveMapping);
    }


    /**
     * 获取name对应map列表，key为name，value为id
     *
     * @param questionMappings
     * @param now
     * @param person
     * @param type
     * @return
     */
    public Map<String, Long> getNameMap(List<QuestionMappingWrapper> questionMappings, Date now,
                                        String person, Integer type) {
        Map<String, Long> map = new HashMap<>();
        List<String> nameList = questionMappings.stream().filter(row -> StringUtil.isNotEmpty(row.getText()))
                .map(row -> row.getText()).collect(Collectors.toList());
        if (ListUtil.isNotEmpty(nameList)) {
            List<QuestionInfo> questionInfos = this.list(new QueryWrapper<QuestionInfo>()
                    .eq("type", type)
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("tag_type", TagTypeEnum.T8.getKey())
                    .in("name", nameList)
            );
            map = questionInfos.stream()
                    .collect(Collectors.toMap(row -> row.getName(), row -> row.getId(), (k1, k2) -> k1));
            List<QuestionInfo> addBatch = new ArrayList<>();
            List<String> nameExist = new ArrayList<>();
            for (String s : nameList) {
                if (map.get(s) == null && !nameExist.contains(s)) {
                    nameExist.add(s);
                    QuestionInfo bean = new QuestionInfo();
                    bean.setModifier(person);
                    bean.setCreator(person);
                    bean.setGmtCreate(now);
                    bean.setGmtModified(now);
                    bean.setTagType(TagTypeEnum.T8.getKey());
                    bean.setTagName(s);
                    bean.setName(s);
                    bean.setType(type);
                    bean.setSubType(1); //描述类型
                    addBatch.add(bean);
                }
            }
            questionInfoService.saveBatch(addBatch); //批量插入question
            for (QuestionInfo bean : addBatch) {
                if (map.get(bean.getName()) == null) {
                    map.put(bean.getName(), bean.getId());
                }
            }
        }
        return map;
    }


    /**
     * 保存明细
     *
     * @param questionWrapper
     * @param param
     */
    public void saveQuestionDetail(QuestionWrapper questionWrapper, CommonParam param) {
        QuestionInfo questionInfo = param.getQuestionInfo();
        Date now = param.getNow();
        String person = param.getPerson();
        //先删除原明细，再插入新明细
        questionDetailFacade.update(new QuestionDetail(), //删除原明细
                new UpdateWrapper<QuestionDetail>()
                        .eq("question_id", questionInfo.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("gmt_modified", now)
                        .set("modifier", person)
                        .set("is_deleted", IsDeleteEnum.Y.getKey()));
        List<QuestionDetail> questionDetails = questionWrapper.getQuestionDetails();
        if (ListUtil.isNotEmpty(questionDetails)) {
            List<QuestionDetail> saveDetail = new ArrayList<>();
            int i = 1;
            for (QuestionDetail detail : questionDetails) {
                QuestionDetail bean = new QuestionDetail();
                BeanUtil.copyProperties(detail, bean);
                bean.setId(null); //防止前端传参，将前端的id置空自动插入
                bean.setCreator(person);
                bean.setGmtCreate(now);
                bean.setModifier(person);
                bean.setGmtModified(now);
                bean.setQuestionId(questionInfo.getId());
                bean.setOrderNo(i++);
                saveDetail.add(bean);
            }
            questionDetailService.saveBatch(saveDetail);
        }
    }


    /**
     * 保存主表
     *
     * @param questionWrapper
     * @param param
     */
    public void saveQuestionInfo(QuestionWrapper questionWrapper, CommonParam param) {
        QuestionInfo questionInfo = new QuestionInfo();
        if (questionWrapper.getId() != null) {
            questionInfo = this.getOne(new QueryWrapper<QuestionInfo>()
                    .eq("id", questionWrapper.getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey()), false);
            if (questionInfo == null) { //校验
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标签不存在");
            }
            param.setInsertOrUpdate(InsertOrUpdateEnum.Update.getKey());
        }
        BeanUtil.copyProperties(questionWrapper, questionInfo);
        List<QuestionInfo> questionInfoList = this.list(new QueryWrapper<QuestionInfo>()
                .eq("tag_name", questionInfo.getTagName())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("type", questionInfo.getType())
                .ne("tag_type", TagTypeEnum.T8.getKey())
                .ne("id", questionInfo.getId() == null ? -1 : questionInfo.getId()));
        if (ListUtil.isNotEmpty(questionInfoList)) { //标签type、tagName唯一
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标签系统名称已重复，无法建立");
        }
        if (InsertOrUpdateEnum.Insert.getKey() == param.getInsertOrUpdate()) {
            questionInfo.setCreator(param.getPerson()); //创建人
            questionInfo.setGmtCreate(param.getNow());//创建时间
        }
        questionInfo.setGmtModified(param.getNow());//修改时间
        questionInfo.setModifier(param.getPerson());//修改人
        param.setQuestionInfo(questionInfo);
        this.saveOrUpdate(questionInfo);
    }


    /**
     * 标签列表
     *
     * @param questionPageVO
     * @return
     */
    public IPage<QuestionPageDTO> getListFac(QuestionPageVO questionPageVO) {
        IPage<QuestionPageDTO> res = this.getList(questionPageVO);
        if (res.getTotal() <= 0) {
            return res;
        }
        List<String> personIds = res.getRecords().stream().map(row -> row.getModifier()).collect(Collectors.toList());
        RespDTO<Map<String, String>> mapRespDTO = userServiceClient.getUserInfoByIds(personIds);
        if (mapRespDTO == null || !CommonErrorCode.OK.getCode().equals(mapRespDTO.code)) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取操作人失败");
        }
        for (QuestionPageDTO bean : res.getRecords()) {
            bean.setModifier(mapRespDTO.data.get(bean.getModifier()));
        }

        // 如果是化验明细项，获取name字段和术语库对应，其他用tagName
        List<String> nameList = new ArrayList<>();
        ConceptExistVO conceptExistVO = new ConceptExistVO();
        for (QuestionPageDTO bean : res.getRecords()) {
            if (QuestionTypeEnum.Lis.getKey() == bean.getType()
                    && TagTypeEnum.T7.getKey() != bean.getTagType().intValue()) {
                nameList.add(bean.getName());
            } else {
                nameList.add(bean.getTagName());
            }
        }
        conceptExistVO.setNameList(nameList);
        try {
            RespDTO<Map<String, List<Integer>>> resMap = knowledgemanServiceClient.getConceptMap(conceptExistVO);
            RespDTOUtil.respNGDeal(resMap, "远程调用根据名称和类型获取概念列表失败");
            Map<String, List<Integer>> map = resMap.data;
            for (QuestionPageDTO bean : res.getRecords()) {
                List<Integer> list = new ArrayList<>();
                Boolean isLisDetail = false;
                if (QuestionTypeEnum.Lis.getKey() == bean.getType()
                        && TagTypeEnum.T7.getKey() != bean.getTagType().intValue()) {
                    isLisDetail = true;
                }
                if (isLisDetail) {
                    list = map.get(bean.getName());
                } else {
                    list = map.get(bean.getTagName());
                }
                // 化验明细需要特殊处理，类型转换为明细
                if (ListUtil.isNotEmpty(list)) {
                    if (list.contains(bean.getType()) ||
                            (isLisDetail && list.contains(QuestionTypeEnum.LisDetail.getKey()))) {
                        bean.setExist(true);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return res;
    }


    /**
     * 根据id删除标签
     *
     * @param deleteQuestionVO
     * @return
     */
    public Boolean deleteByIdsFac(DeleteQuestionVO deleteQuestionVO) {
        String ids = deleteQuestionVO.getIds();
        if (StringUtil.isEmpty(ids)) {
            return true;
        }
        List<String> idList = Arrays.asList(ids.split(","));
        if (ListUtil.isEmpty(idList)) {
            return true;
        }
        StringBuffer errMsg = new StringBuffer();
        Date now = DateUtil.now();
        String person = UserUtils.getCurrentPrincipleID();
        for (String id : idList) {
            //删除缓存中的关联的标签信息
            clearAllCacheByIds(Long.parseLong(id));

            Map paramMap = new HashMap<>();
            paramMap.put("delete", IsDeleteEnum.Y.getKey());
            paramMap.put("ids", Arrays.asList(ids.split(",")));
            paramMap.put("gmtModified", now);
            paramMap.put("modifier", person);

            // 映射关系删除校验，如果有父项关联，不能删除
            List<Long> tagType10Ids = new ArrayList<>();
            List<QuestionInfo> questionInfoList = getParentQuestion(Long.parseLong(id));
            if (ListUtil.isNotEmpty(questionInfoList)) {
                boolean flag = false;
                StringBuffer sb = new StringBuffer();
                for (QuestionInfo ques : questionInfoList) {
                    if (ques.getTagType().equals(TagTypeEnum.T10.getKey())) {  // 既往史下特殊类型
                        tagType10Ids.add(ques.getId());
                        List<QuestionInfo> question2 = getParentQuestion(ques.getId());
                        if (ListUtil.isNotEmpty(question2)) {
                            sb.append("【" + question2.get(0).getTagName() + "】"); // 有且只有一个
                            flag = true;
                        }
                    } else {
                        sb.append("【" + ques.getTagName() + "】");
                        flag = true;
                    }
                }
                if (flag) {
                    throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                            sb.toString());
                }
            }
            //模板关联
            GetQuestiongAndModuleRelationsVO getQuestiongAndModuleRelationsVO = new GetQuestiongAndModuleRelationsVO();
            getQuestiongAndModuleRelationsVO.setQuestionId(Long.parseLong(id));
            GetQuestiongAndModuleRelationsDTO getQuestiongAndModuleRelationsDTO =
                    moduleInfoFacade.getQuestiongAndModuleRelations(getQuestiongAndModuleRelationsVO);
            if (null != getQuestiongAndModuleRelationsDTO &&
                    StringUtil.isNotBlank(getQuestiongAndModuleRelationsDTO.getRelationInfos().toString())) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                        getQuestiongAndModuleRelationsDTO.getRelationInfos().toString());
            }
            //查体模板
            QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
            deptVitalQueryWrapper.in("vital_id", Arrays.asList(ids.split(","))).
                    eq("is_deleted", IsDeleteEnum.N.getKey());
            List<DeptVital> deptVitals = deptVitalFacade.list(deptVitalQueryWrapper);
            if (ListUtil.isNotEmpty(deptVitals)) {
                //科室信息
                List<Long> deptIds = deptVitals.stream().map(deptVitalDTO -> deptVitalDTO.getDeptId())
                        .collect(Collectors.toList());
                ConceptSearchVO conceptSearchVO = new ConceptSearchVO();
                conceptSearchVO.setConceptIds(deptIds);
                RespDTO<Map<Long, String>> deptMap = klmServiceClient.getConceptMap(conceptSearchVO);
                Map<Long, String> depts = deptMap.data;
                //输入内容
                StringBuffer sb = new StringBuffer();
                sb.append("请先删除科室查体模板关联中的对应标签：");
                for (DeptVital deptVital : deptVitals) {
                    if (StringUtil.isNotBlank(depts.get(deptVital.getDeptId()))) {
                        sb.append("【" + depts.get(deptVital.getDeptId()) + "】").append("<br/>");
                    }
                }
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                        sb.toString());
            }
            //删除自身
            this.deleteByIds(paramMap);
            //删除tagType=10的无用标签
            if (ListUtil.isNotEmpty(tagType10Ids)) {
                this.update(new UpdateWrapper<QuestionInfo>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .in("id", tagType10Ids)
                        .set("is_deleted", IsDeleteEnum.Y.getKey())
                        .set("gmt_modified", now)
                        .set("modifier", person)
                );
            }
            //删除明细
            questionDetailFacade.deleteByQuestionIdFac(paramMap);
            //删除映射关系
            questionMappingFacade.deleteByIdsFac(paramMap);
            //删除查体模板
            deptVitalFacade.update(new DeptVital(), new UpdateWrapper<DeptVital>()
                    .eq("vital_id", id)
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .set("gmt_modified", now)
                    .set("modifier", person)
                    .set("is_deleted", IsDeleteEnum.Y.getKey()));

            //TODO 删除映射关系或检查抛异常

        }
        return true;
    }


    /**
     * 检索
     *
     * @param questionIndexVO
     * @return
     */
    public List<QuestionInfo> indexFac(QuestionIndexVO questionIndexVO) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("age", questionIndexVO.getAge());
        paramMap.put("sexType", questionIndexVO.getSexType());
        paramMap.put("tagName", questionIndexVO.getTagName());
        paramMap.put("type", questionIndexVO.getType());
        paramMap.put("controlType", questionIndexVO.getControlType());
        paramMap.put("notIds", questionIndexVO.getNotIds());
        paramMap.put("tagType", questionIndexVO.getTagType());
        paramMap.put("existName", questionIndexVO.getExistName());
        paramMap.put("notTagType", questionIndexVO.getNotTagType());
        paramMap.put("notControlType", questionIndexVO.getNotControlType());
        paramMap.put("name", questionIndexVO.getName());
        List<QuestionInfo> res = this.index(paramMap);
        return res;
    }

    /**
     * 知识库检索标签
     *
     * @param klQuestionVO
     * @return
     */
    public List<QuestionInfo> indexForKL(KLQuestionVO klQuestionVO) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("age", klQuestionVO.getAge());
        paramMap.put("sexType", klQuestionVO.getSexType());
        paramMap.put("tagNames", klQuestionVO.getTagNames());
        paramMap.put("type", klQuestionVO.getType());
        paramMap.put("tagType", klQuestionVO.getTagType());
        paramMap.put("names", klQuestionVO.getNames());
        List<QuestionInfo> res = this.indexForKL(paramMap);
        return res;
    }

    /**
     * 根据主标签id 检索子标签
     *
     * @param questionIndexSubVO
     * @return
     */
    public List<QuestionInfo> indexSub(QuestionIndexSubVO questionIndexSubVO) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("age", questionIndexSubVO.getAge());
        paramMap.put("sexType", questionIndexSubVO.getSexType());
        paramMap.put("tagName", questionIndexSubVO.getTagName());
        paramMap.put("type", questionIndexSubVO.getType());
        paramMap.put("controlType", questionIndexSubVO.getControlType());
        paramMap.put("notIds", questionIndexSubVO.getNotIds());
        paramMap.put("tagType", questionIndexSubVO.getTagType());
        paramMap.put("existName", questionIndexSubVO.getExistName());
        paramMap.put("notTagType", questionIndexSubVO.getNotTagType());
        paramMap.put("notControlType", questionIndexSubVO.getNotControlType());

        QueryWrapper<QuestionMapping> questionMappingQueryWrapper = new QueryWrapper<>();
        questionMappingQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("parent_question", questionIndexSubVO.getParentId());
        List<QuestionMapping> questionMappingList = questionMappingFacade.list(questionMappingQueryWrapper);
        if (ListUtil.isNotEmpty(questionMappingList)) {
            List<Long> sonIds = questionMappingList.stream().map(questionMapping -> questionMapping.getSonQuestion())
                    .collect(Collectors.toList());
            paramMap.put("ids", sonIds);
        }
        List<QuestionInfo> res = this.index(paramMap);
        return res;
    }

    /**
     * 获取特殊标签（同伴|无）
     *
     * @param questionSpecVO
     * @return
     */
    public List<QuestionInfo> getSpecialFac(QuestionSpecVO questionSpecVO) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("age", questionSpecVO.getAge());
        paramMap.put("sexType", questionSpecVO.getSexType());
        paramMap.put("type", questionSpecVO.getType());
        paramMap.put("notIds", questionSpecVO.getNotIds());
        return this.getSpecial(paramMap);
    }


    /**
     * 根据标签名获取标签信息
     *
     * @param getQuestionInfoVO
     * @return
     */
    public List<GetQuestionInfoDTO> getQuestiongInfosByName(GetQuestionInfoVO getQuestionInfoVO) {
        List<GetQuestionInfoDTO> getQuestionInfoDTOS = this.getQuestiongInfoByName(getQuestionInfoVO);
        return getQuestionInfoDTOS;
    }

    //    public List<Long> getQuestionIdsByType(GetQuestionIdsByTypeVO getQuestionIdsByTypeVO) {
    //        QueryWrapper<QuestionInfo> queryWrapper = new QueryWrapper<>();
    //        queryWrapper.in("id", getQuestionIdsByTypeVO.getQuestionIds())
    //                .eq("is_deleted", IsDeleteEnum.N.getKey())
    //                .eq("type", getQuestionIdsByTypeVO.getQuestionType());
    //        List<QuestionInfo> questionInfoList = this.list(queryWrapper);
    //        List<Long> questionIds = questionInfoList.stream().map(questionInfo -> questionInfo.getId())
    //                .collect(Collectors.toList());
    //        return questionIds;
    //    }

    /**
     * 根据参数返回标签内容
     *
     * @param questionVO 获取标签内容参数
     * @return 标签内容
     */
    public QuestionDTO getById(QuestionVO questionVO) {
        Map paramMap = new HashMap<>();
        paramMap.put("sexType", questionVO.getSexType());
        paramMap.put("age", questionVO.getAge());
        paramMap.put("id", questionVO.getId());
        QuestionInfo questionInfo = this.getByParam(paramMap);
        if (questionInfo == null) { //无数据直接返回，不通过报错的形式返回
            return new QuestionDTO();
        }
        QuestionDTO res = new QuestionDTO();
        BeanUtil.copyProperties(questionInfo, res);

        //获取明细项
        List<QuestionDetail> questionDetailList = questionDetailFacade.getByQuestionId(questionVO.getId());
        res.setQuestionDetailList(questionDetailList);

        //获取子标签
        List<QuestionInfoWrapper> questionMapping = this.getByQuestionMapping(paramMap);
        List<QuestionDTO> sonQuestionList = BeanUtil.listCopyTo(questionMapping, QuestionDTO.class);
        res.setQuestionMapping(sonQuestionList);
        //获取子标签的明细项
        if (ListUtil.isNotEmpty(sonQuestionList)) {
            Map paramMap1 = new HashMap<>();
            paramMap1.put("sexType", questionVO.getSexType());
            paramMap1.put("age", questionVO.getAge());
            List<Long> sonQuestionIds = sonQuestionList.stream()
                    .map(artist -> artist.getId())
                    .collect(Collectors.toList());
            Map<Long, List<QuestionDetail>> sonQuestionMap = questionDetailFacade.getByQuestionIds(sonQuestionIds);
            paramMap1.put("ids", sonQuestionIds);
            //获取第二级子标签
            List<QuestionInfoWrapper> questionMapping1 = this.getByQuestionMapping(paramMap1);
            Map<Long, List<QuestionInfoWrapper>> qm1 =
                    EntityUtil.makeEntityListMap(questionMapping1, "parentQuestion");
            for (QuestionDTO bean : sonQuestionList) {
                List<QuestionDetail> questionDetailList1 = sonQuestionMap.get(bean.getId());
                if (ListUtil.isEmpty(questionDetailList1)) {
                    questionDetailList1 = new ArrayList<>();
                }
                bean.setQuestionDetailList(questionDetailList1);
                List<QuestionDTO> sonQuestionList1 = new ArrayList<>();
                if (ListUtil.isNotEmpty(qm1.get(bean.getId()))) {
                    sonQuestionList1 = BeanUtil.listCopyTo(qm1.get(bean.getId()), QuestionDTO.class);
                }
                bean.setQuestionMapping(sonQuestionList1);

                List<Long> sonQuestionIds1 = sonQuestionList1.stream()
                        .map(artist -> artist.getId())
                        .collect(Collectors.toList());
                Map<Long, List<QuestionDetail>> sonQuestionMap1 = questionDetailFacade.getByQuestionIds(sonQuestionIds1);
                for (QuestionDTO questionDTO : sonQuestionList1) {
                    //获取二级子明细项
                    List<QuestionDetail> questionDetailList2 = sonQuestionMap1.get(questionDTO.getId());
                    if (ListUtil.isEmpty(questionDetailList2)) {
                        questionDetailList2 = new ArrayList<>();
                    }
                    questionDTO.setQuestionDetailList(questionDetailList2);
                }

            }
        }
        return res;
    }


    /**
     * 查询多个id返回标签内容
     *
     * @param questionIdsVO 获取标签内容参数
     * @return 标签内容
     */
    public Map<Long, Object> getByIds(QuestionIdsVO questionIdsVO) {
        Map<Long, Object> data = new HashMap<>();
        if (StringUtil.isEmpty(questionIdsVO.getIds())) {
            return data;
        }
        String[] ids = questionIdsVO.getIds().split(",");
        for (String id : ids) {
            QuestionVO questionVO = new QuestionVO();
            questionVO.setId(Long.parseLong(id));
            questionVO.setAge(questionIdsVO.getAge());
            questionVO.setSexType(questionIdsVO.getSexType());
            QuestionDTO item = this.getById(questionVO);
            data.put(item.getId(), item);
        }
        return data;
    }

    /**
     * 获取所有需要删除的缓存id列表
     *
     * @param id
     * @return
     */
    public List<Long> getDeleteId(Long id) {
        List<Long> list = new ArrayList<>();
        list.add(id); //添加自身id
        getParent(id, list); //添加父级
        return list;
    }


    public void getParent(Long id, List<Long> list) {
        List<QuestionMapping> questionMappingList = questionMappingFacade.list(
                new QueryWrapper<QuestionMapping>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("son_question", id)
        );
        if (ListUtil.isNotEmpty(questionMappingList)) {
            for (QuestionMapping questionMapping : questionMappingList) {
                Long parentId = questionMapping.getParentQuestion();
                if (!list.contains(parentId)) {
                    list.add(parentId);
                    getParent(parentId, list);
                }
            }
        }
    }


    public List<ConceptBaseDTO> indexByLexiconFac(IndexIcssVO indexIcssVO) {
        IndexLexiconVO vo = new IndexLexiconVO();
        BeanUtil.copyProperties(indexIcssVO, vo);
        RespDTO<List<ConceptBaseDTO>> res = knowledgemanServiceClient.indexByLexiconFac(vo);
        RespDTOUtil.respNGDeal(res, "【远程调用】搜索概念失败");

        // 过滤已添加的标签
        List<String> tagNames = res.data.stream().map(row -> row.getName()).collect(Collectors.toList());
        if (ListUtil.isNotEmpty(tagNames)){
            List<QuestionInfo> questionInfoList = this.list(new QueryWrapper<QuestionInfo>()
                    .in("tag_name", tagNames)
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("type", indexIcssVO.getType())
                    .ne("tag_type", TagTypeEnum.T8.getKey()));
            List<String> nameExist = questionInfoList.stream().map(row -> row.getTagName()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(nameExist)) {
                for (int i = 0; i < res.data.size(); i++) {
                    if (nameExist.contains(res.data.get(i).getName())) {
                        res.data.remove(i--);
                    }
                }
            }
        }

        return res.data;
    }
}