package com.diagbot.enums;

import com.diagbot.core.KeyedNamed;
import lombok.Setter;

/**
 * @Description: 术语分类枚举
 * @author: gaodm
 * @time: 2019/5/6 13:58
 */
public enum LexiconTypeEnum implements KeyedNamed {
    SYMPTOM(1, "症状"),
    TIME(2, "时间"),
    BODYPART(3, "部位"),
    NATURE(4, "性质"),
    CAUSE(5, "诱因"),
    DEGREE(6, "程度"),
    NEGATIVE(7, "阴性"),
    POSITIVE(8, "阳性"),
    COMPANY(9, "单位"),
    DRUGS(10, "药品通用名"),
    TREATMENT(11, "治疗"),
    LIS_PACKAGE(12, "化验套餐"),
    LIS_DETAILS(13, "化验明细"),
    LIS_RESULTS(14, "化验结果"),
    PACS_PACKAGE(15, "辅检套餐"),
    PACS_ITEMS(16, "辅检项目"),
    PACS_RESULTS(17, "辅检结果"),
    DIAGNOSIS(18, "诊断"),
    CONNECTIVES(19, "连接词"),
    POSITION(20, "方位"),
    RELATIVES(21, "亲属"),
    MARRIAGE(22, "婚姻"),
    FOOD(23, "食品"),
    HISTORY(24, "历史"),
    OPERATION(25, "手术"),
    SYMPTOM_MANIFESTATION(26, "症状表现"),
    DIGITAL_QUANTIFIER(27, "数字量词"),
    NUMBER(28, "数字"),
    DRUG_CATEGORY_BIG(29, "药品大类"),
    DRUG_CATEGORY_SMALL(30, "药品小类"),
    PACS_CATEGORY_BIG(31, "辅检大类"),
    PACS_CATEGORY_SMALL(32, "辅检小类"),
    VITAL_INDEX(33, "体征指标"),
    VITAL_INDEX_VALUE(34, "体征指标值"),
    VITAL_RESULT(35, "体征结果"),
    PLACE(36, "地点"),
    OCCUPATION(37, "职业"),
    PERSONAL_HISTORY(38, "个人史"),
    PERSONAL_HISTORY_DESCRIPTION(39, "个人史描述"),
    TIME_DESCRIPTION(40, "时间描述"),
    SYMPTOM_INDEX(41, "症状指标"),
    MEDICAL_SUPPLIES(42, "医用耗材"),
    DEGREE_CLASSIFIER(43, "程度量词"),
    OTHER(44, "其他"),
    DELETE(45, "删除"),
    LIS_TABLES(46, "化验公表项"),
    DEPARTMENT(47, "科室"),
    GAUGE(48, "量表"),
    SIDE_EFFECTS(49, "不良反应"),
    CORE_INDICATORS(50, "核心指标"),
    TYPES_OF_DISEASE(51, "疾病类型(慢病|急诊)"),
    PART_AREA(52, "部位区域"),
    LOCATION_AREA(52, "部位区域"),
    CHEMICAL_NAME_OF_DRUG(53, "药品化学名"),
    DRUG_TRADE_NAMES(54, "药品商品名"),
    HISTORY_OF_PAST_DISEASES(55, "既往疾病史"),
    HISTORY_OF_TRAUMA(56, "外伤史"),
    HISTORY_OF_SURGERY(57, "手术史"),
    HISTORY_OF_BLOOD_TRANSFUSION(58, "输血史"),
    INFECTIOUS_DISEASE_HISTORY(59, "传染病史"),
    BAD_HABITS(60, "不良习惯"),
    MARITAL_HISTORY(61, "冶游史"),
    HISTORY_OF_MARRIAGE_AND_CHILDBEARING(62, "婚育史"),
    BIRTH_HISTORY(63, "生育史"),
    FAMILY_HISTORY(64, "家族史"),
    FOOD_ALLERGY_HISTORY(65, "过敏史"),
    HISTORY_OF_DRUG_ALLERGY(66, "药物过敏史"),
    FAMILY_RELATED_INFECTIOUS_DISEASE_HISTORY(67, "家族相关传染病史"),
    FURTHER_CONSULTATION(68, "复诊"),
    DIAGNOSTIC_CRITERIA(69, "诊断依据条件"),
    OTHER_DIAGNOSTIC_CRITERIA(70, "诊断依据其他"),
    PACS_ROOT_DIRECTORY(71,"辅检根目录"),
    // GUIDELINES_OUTPATIENT(72,"诊疗指南（门诊）"),
    // GUIDELINES_HOS(73,"诊疗指南（住院）"),
    NURSE(74,"护理"),
    CLINICAL_PATHWAY(75,"临床路径"),
    BLOOD(76,"血液"),
    ANESTHESIA(77,"麻醉"),
    TUTELAGE(78,"监护"),
    TCM(79,"中医"),
    THE_DIVISION_PROCESS(80,"入科流程"),
    CENSORSHIP_RULES(81,"审查规则"),
    E_BOOK (82,"电子书"),
    VIDEO(83,"视频");

    @Setter
    private int key;

    @Setter
    private String name;

    LexiconTypeEnum(int key, String name) {
        this.key = key;
        this.name = name;
    }

    public static LexiconTypeEnum getEnum(int key) {
        for (LexiconTypeEnum item : LexiconTypeEnum.values()) {
            if (item.key == key) {
                return item;
            }
        }
        return null;
    }

    public static LexiconTypeEnum getEnum(String name) {
        for (LexiconTypeEnum item : LexiconTypeEnum.values()) {
            if (item.name.equals(name)) {
                return item;
            }
        }
        return null;
    }

    public static String getName(int key) {
        LexiconTypeEnum item = getEnum(key);
        return item != null ? item.name : null;
    }

    public static Integer getKey(String name) {
        LexiconTypeEnum item = getEnum(name);
        return item != null ? item.key : null;
    }

    @Override
    public int getKey() {
        return key;
    }

    @Override
    public String getName() {
        return name;
    }
}

