package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.NeoServiceClient;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.DiagnoseDetailSaveDTO;
import com.diagbot.dto.DiagnosePageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.VerifyDataDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.Concept;
import com.diagbot.entity.Diagnose;
import com.diagbot.entity.DiagnoseDetail;
import com.diagbot.entity.DiagnoseDetailSave;
import com.diagbot.entity.DiagnoseQuestion;
import com.diagbot.entity.LibraryInfo;
import com.diagbot.entity.wrapper.DiagnoseWrapper;
import com.diagbot.enums.DiagnoseFeatureTypeEnum;
import com.diagbot.enums.DiagnoseFieldEnum;
import com.diagbot.enums.DiagnoseTypeEnum;
import com.diagbot.enums.HasQuestionEnum;
import com.diagbot.enums.InsertOrUpdateEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.DiagnoseDetailService;
import com.diagbot.service.DiagnoseQuestionService;
import com.diagbot.service.DiagnoseService;
import com.diagbot.service.impl.DiagnoseServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.DiagnoseTypeConvertUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.VerifyUtil;
import com.diagbot.vo.DeleteDiagnoseVO;
import com.diagbot.vo.DiagnosePageVO;
import com.diagbot.vo.DiagnoseSearchVO;
import com.diagbot.vo.DiseaseIndexVO;
import com.diagbot.vo.GetDiagnoseDetailVO;
import com.diagbot.vo.NeoParamVO;
import com.diagbot.vo.UpdateDiagnoseVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Component
public class DiagnoseFacade extends DiagnoseServiceImpl {

    public static boolean verifyFlag = true;  // true:执行完毕，false：正在执行

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    @Qualifier("diagnoseServiceImpl")
    DiagnoseService diagnoseService;
    @Autowired
    @Qualifier("diagnoseDetailServiceImpl")
    DiagnoseDetailService diagnoseDetailService;
    @Autowired
    @Qualifier("diagnoseQuestionServiceImpl")
    DiagnoseQuestionService diagnoseQuestionService;
    @Autowired
    DiagnoseDetailFacade diagnoseDetailFacade;
    @Autowired
    LibraryInfoFacade libraryInfoFacade;
    @Autowired
    NeoServiceClient neoServiceClient;
    @Autowired
    LisMappingFacade lisMappingFacade;
    @Autowired
    CryptConfiguer cryptConfiguer;

    /**
     * 诊断依据分页
     *
     * @param diagnosePageVO
     * @return
     */
    public IPage<DiagnosePageDTO> getListFac(DiagnosePageVO diagnosePageVO) {
        IPage<DiagnosePageDTO> res = this.getList(diagnosePageVO);
        if (res.getTotal() <= 0) {
            return res;
        }
        return res;
    }


    /**
     * 诊断依据删除
     *
     * @param deleteDiagnoseVO
     * @return
     */
    public void delete(DeleteDiagnoseVO deleteDiagnoseVO) {
        if (!(IsDeleteEnum.N.getKey().equals(deleteDiagnoseVO.getIsDeleted())
                || IsDeleteEnum.Y.getKey().equals(deleteDiagnoseVO.getIsDeleted()))) {
            throw new CommonException(CommonErrorCode.STATUS_IS_ERROR, "状态值错误【Y】或【N】");
        }

        Date now = DateUtil.now();
        this.update(new Diagnose(), new UpdateWrapper<Diagnose>()
                .eq("id", deleteDiagnoseVO.getId())
                .set("gmt_modified", now)
                .set("modifier", deleteDiagnoseVO.getModifier())
                .set("is_deleted", deleteDiagnoseVO.getIsDeleted())
                .set("neo_update", now)
                .set("remark", deleteDiagnoseVO.getRemark()));

        // 删除明细（逻辑删除）
        diagnoseDetailFacade.update(new DiagnoseDetail(), new UpdateWrapper<DiagnoseDetail>()
                .eq("diagnose_id", deleteDiagnoseVO.getId())
                .set("gmt_modified", now)
                .set("modifier", deleteDiagnoseVO.getModifier())
                .set("is_deleted", deleteDiagnoseVO.getIsDeleted()));

        // 删除问题词列表（物理删除）
        diagnoseQuestionService.remove(new QueryWrapper<DiagnoseQuestion>()
                .eq("diagnose_id", deleteDiagnoseVO.getId()));

        // 删除图谱数据
        NeoParamVO neoParamVO = new NeoParamVO();
        neoParamVO.setId(deleteDiagnoseVO.getId());
        neoServiceClient.deleteNeo(neoParamVO);
        RespDTO<Boolean> respDTO = neoServiceClient.deleteNeo(neoParamVO);
        RespDTOUtil.respNGDeal(respDTO, "删除图谱数据失败【远程调用】");
    }


    /**
     * 诊断检索，过滤已添加的诊断
     *
     * @param diseaseIndexVO
     * @return
     */
    public List<ConceptBaseDTO> indexFac(DiseaseIndexVO diseaseIndexVO) {
        String name = diseaseIndexVO.getName();
        // 设置诊断类型
        diseaseIndexVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
        List<Diagnose> diagnoseList = this.list(
                new QueryWrapper<Diagnose>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .like(StringUtil.isNotEmpty(name), "dis_name", name)
        );
        List<String> disNameList = diagnoseList.stream().map(r -> r.getDisName()).distinct().collect(Collectors.toList());
        if (cryptConfiguer.cryptFlag) {
            CryptUtil.encryptList(disNameList);
            diseaseIndexVO.setName(CryptUtil.encrypt_char(name));
        }
        diseaseIndexVO.setDisNamList(disNameList);
        return this.index(diseaseIndexVO);
    }


    /**
     * 诊断依据获取详情
     *
     * @param getDiagnoseDetailVO
     * @return
     */
    public DiagnoseDetailSaveDTO getDetail(GetDiagnoseDetailVO getDiagnoseDetailVO) {
        DiagnoseDetailSaveDTO res = new DiagnoseDetailSaveDTO();
        List<DiagnoseDetail> detailList = diagnoseDetailFacade.list(new QueryWrapper<DiagnoseDetail>()
                .eq("diagnose_id", getDiagnoseDetailVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .orderByAsc("order_no")
        );
        for (DiagnoseDetail detail : detailList) {
            if (DiagnoseTypeEnum.Confirmed.getKey() == detail.getType() ||
                    DiagnoseTypeEnum.Examination.getKey() == detail.getType() ||
                    DiagnoseTypeEnum.Vigilant.getKey() == detail.getType()
            ) {
                res.getDisformula().add(detail);
            } else {
                res.getDisFeature().add(detail);
            }
        }
        if (ListUtil.isNotEmpty(detailList)) {
            String disName = detailList.get(0).getDisName();
            if (cryptConfiguer.cryptFlag) {
                disName = CryptUtil.encrypt_char(disName);
            }
            LibraryInfo libraryInfo = libraryInfoFacade.getOne(new QueryWrapper<LibraryInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("is_concept", 1)
                    .eq("type_id", LexiconTypeEnum.DIAGNOSIS.getKey())
                    .eq("name",
                            cryptConfiguer.cryptFlag ?
                                CryptUtil.encrypt_char(detailList.get(0).getDisName()) :
                                detailList.get(0).getDisName()
                    ),
                    false
            );
            if (libraryInfo == null) {
                res.setDisNameExist(false);
            }
        }
        return res;
    }


    /**
     * 诊断依据校验数据，新增或更新主表信息，覆盖明细表和问题词表
     *
     * @param diagnoseWrapper
     */
    public VerifyDataDTO verifyData(DiagnoseWrapper diagnoseWrapper) {
        VerifyDataDTO verifyDataDTO = new VerifyDataDTO();
        CommonParam param = initCommonParam();
        param.setPerson(diagnoseWrapper.getModifier());

        // 更新主表业务(同时删除明细表和问题词表)
        Long id = insertOrUpdateDiagnose(diagnoseWrapper, param);
        diagnoseWrapper.setId(id);
        verifyDataDTO.setId(id); //设置主键返回
        // 加载词库数据
//        List<LibraryForDiagnoseDTO> libraryForDiagnoseDTOS = libraryInfoFacade.getLibraryForDiagnose();
//        Set<String> set = libraryForDiagnoseDTOS.stream().map(row -> row.getName()+ "_"+row.getTypeId())
//                          .collect(Collectors.toSet());
        Set<String> libSet = libraryInfoFacade.getLibraryForDiagnose2(); // 加载词库数据
        // 明细数据
        List<DiagnoseDetail> diagnoseDetailList = new ArrayList<>();
        List<DiagnoseDetail> disFeature = new ArrayList<>(); // 诊断特征
        List<DiagnoseDetail> disformula = new ArrayList<>(); // 诊断公式

        // 问题词数据
        List<DiagnoseQuestion> diagnoseQuestionList = new ArrayList<>();

        // 处理诊断特征业务（校验编码|标准词|关联词|结果），单点校验不需要校验公表，因为同一按钮已校验
        doDisFuture(diagnoseWrapper, param, libSet, disFeature, diagnoseQuestionList);
        // 处理诊断公式业务（校验公式）
        doDisFormula(diagnoseWrapper, param, disformula, diagnoseQuestionList);

        diagnoseDetailList.addAll(disFeature);
        diagnoseDetailList.addAll(disformula);
        // 保存明细
        diagnoseDetailService.saveBatch(diagnoseDetailList);
        // 保存问题词
        if (ListUtil.isNotEmpty(diagnoseQuestionList)) {
            // 新增问题词数据
            diagnoseQuestionService.saveBatch(diagnoseQuestionList);
        }

        // 更新主表状态
        this.update(new UpdateWrapper<Diagnose>()
                .set("has_question", param.getFlag())
                .eq("id", diagnoseWrapper.getId())
        );
        verifyDataDTO.setHasQuestion(param.getFlag());
        verifyDataDTO.setDisFeature(disFeature);
        verifyDataDTO.setDisformula(disformula);
        verifyDataDTO.setDisNameExist(param.getDisNameExist());
        return verifyDataDTO;
    }


    /**
     * 诊断依据保存暂存数据，新增或更新主表信息，覆盖明细表，清空问题词
     *
     * @param diagnoseWrapper
     * @return 返回主表id
     */
    public Long tempSaveOrUpdateFac(DiagnoseWrapper diagnoseWrapper) {
        VerifyDataDTO verifyDataDTO = new VerifyDataDTO();
        CommonParam param = initCommonParam();
        param.setPerson(diagnoseWrapper.getModifier());
        param.setFlag(HasQuestionEnum.TEMP_DATA.getKey()); // 设置状态为暂存数据

        // 更新主表业务(同时删除明细表和问题词表)
        Long id = insertOrUpdateDiagnose(diagnoseWrapper, param);
        diagnoseWrapper.setId(id);
        verifyDataDTO.setId(id); //设置主键返回

        // 明细数据
        List<DiagnoseDetail> diagnoseDetailList = new ArrayList<>();
        int i = param.getI();

        // 合并疾病特征和诊断公式，统一遍历
        List<DiagnoseDetailSave> diagnoseDetailSaveList = new ArrayList<>();
        diagnoseDetailSaveList.addAll(diagnoseWrapper.getDisFeature());
        diagnoseDetailSaveList.addAll(diagnoseWrapper.getDisformula());
        for (DiagnoseDetailSave bean : diagnoseDetailSaveList) {
            DiagnoseDetail diagnoseDetail = new DiagnoseDetail();
            BeanUtil.copyProperties(bean, diagnoseDetail);
            diagnoseDetail.setDiagnoseId(diagnoseWrapper.getId());
            diagnoseDetail.setCreator(param.getPerson());
            diagnoseDetail.setGmtCreate(param.getNow());
            diagnoseDetail.setModifier(param.getPerson());
            diagnoseDetail.setGmtModified(param.getNow());
            diagnoseDetail.setDisName(diagnoseWrapper.getDisName());
            diagnoseDetail.setOrderNo(++i);
            diagnoseDetailList.add(diagnoseDetail);
        }
        // 保存明细
        diagnoseDetailService.saveBatch(diagnoseDetailList);
        return id;
    }


    /**
     * 诊断依据校验全部数据（不用修改操作人和时间）
     */
    public String verifyAllData() {
        StringBuffer res = new StringBuffer();
        Long time1 = System.currentTimeMillis();
        Set<String> diagnosetQuestionSet = new HashSet<>(); // 问题词去重变量
        Set<String> diagnoseName = new HashSet<>(); // 诊断依据有问题的诊断
        if (!verifyFlag) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "其他人正在执行此操作，请稍后再试！");
        }
        try {
            verifyFlag = false;
            CommonParam param = initCommonParam();

            // 获取所有主表信息
            List<Diagnose> diagnoseListWithDB = this.list(new QueryWrapper<Diagnose>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .and(wrapper -> wrapper.eq("has_question", HasQuestionEnum.NO_QUESTION.getKey())
                            .or()
                            .eq("has_question", HasQuestionEnum.HAS_QUESTION.getKey()))
                    );

            if (ListUtil.isEmpty(diagnoseListWithDB)) {
                return "无数据执行";
            }

            // 获取所有明细信息
            List<DiagnoseDetail> diagnoseDetailWithDB = diagnoseDetailFacade.list(new QueryWrapper<DiagnoseDetail>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("diagnose_id",
                            diagnoseListWithDB.stream().map(row -> row.getId()).collect(Collectors.toList()))
            );
            Map<String, List<DiagnoseDetail>> map =
                    EntityUtil.makeEntityListMap(diagnoseDetailWithDB, "disName");

            Long time2 = System.currentTimeMillis();
            res.append("加载全部主表和明细表：" + (time2 - time1) / 1000.0).append("秒,");
            // 明细数据
            List<DiagnoseDetail> diagnoseDetailList = new ArrayList<>();
            // 问题词数据
            List<DiagnoseQuestion> diagnoseQuestionList = new ArrayList<>();
            // 加载词库数据
//            List<LibraryForDiagnoseDTO> libraryForDiagnoseDTOS = libraryInfoFacade.getLibraryForDiagnose();
//            Set<String> set = libraryForDiagnoseDTOS.stream().map(row -> row.getName()+ "_"+row.getTypeId())
//                              .collect(Collectors.toSet());
            Set<String> libSet = libraryInfoFacade.getLibraryForDiagnose2(); // 加载词库
            Set<String> standardSet = libraryInfoFacade.getLibraryForStandard(); // 加载标准词词库数据
            Set<String> lisMappingSet = lisMappingFacade.loadLisMappingFac(); // 加载公表项，需要关联同义词
            Long time3 = System.currentTimeMillis();
            res.append("加载词库：" + (time3 - time2) / 1000.0).append("秒,");
            Boolean flag = true;
            List<Integer> typeList = new ArrayList<>();
            // 校验明细数据
            for (String key : map.keySet()) {
                List<DiagnoseDetail> detail = map.get(key);
                List<String> codeList = detail.stream().map(row -> row.getCode()).distinct().collect(Collectors.toList());
                codeList.add("拟诊"); // 公式校验手动添加拟诊文字
                int j = 0;
                if (ListUtil.isNotEmpty(detail)) {
                    // 判断诊断名称是否在标准词库中
                    flag = verifyExist(standardSet, detail.get(0).getDisName(),
                            Arrays.asList(LexiconTypeEnum.DIAGNOSIS.getKey()));
                    if (!flag) {
                        DiagnoseQuestion diagnoseQuestion1 = createDiagnoseQuestion(detail.get(0).getDiagnoseId(),
                                detail.get(0).getDisName(), DiagnoseFeatureTypeEnum.Dis.getKey(),
                                detail.get(0).getDisName(), DiagnoseFieldEnum.disName.getKey(), initCommonParam(), ++j);
                        diagnoseQuestionList.add(diagnoseQuestion1);
                        diagnoseName.add(diagnoseQuestion1.getDisName());
                    }

                    List<DiagnoseDetail> lisAndPacs = new ArrayList<>(); // 化验|辅检数据
                    List<DiagnoseDetail> formula = new ArrayList<>(); // 公式数据

                    for (DiagnoseDetail bean : detail) {
                        boolean updateFlag = false;
                        if (DiagnoseTypeEnum.Confirmed.getKey() == bean.getType()
                                || (DiagnoseTypeEnum.Examination.getKey() == bean.getType()
                                || (DiagnoseTypeEnum.Vigilant.getKey() == bean.getType()))) {
                            formula.add(bean); // 添加到公式列表
                            // 校验公式
                            String oldVerifyFormula = bean.getVerifyFormula();
                            String errMsg = VerifyUtil.verifyFormula(codeList, bean.getFormula());
                            if (StringUtil.isNotEmpty(errMsg)) {
                                bean.setVerifyFormula(errMsg);
                                diagnoseName.add(bean.getDisName());
                                // 不需要添加问题词
                            } else {
                                bean.setVerifyFormula("");
                            }
                            if (!oldVerifyFormula.equals(bean.getVerifyFormula())) {
                                updateFlag = true;
                            }
                        } else {
                            // 添加到化验|辅检列表
                            if (DiagnoseFeatureTypeEnum.Lis.getKey() == bean.getType()
                                    || DiagnoseFeatureTypeEnum.Pacs.getKey() == bean.getType()) {
                                lisAndPacs.add(bean);
                            }

                            // 校验编码
                            String oldVerifyCode = bean.getVerifyCode();
                            if (StringUtil.isNotEmpty(bean.getCode())) {
                                flag = VerifyUtil.verifyCode(String.valueOf(bean.getType()), bean.getCode());
                                if (!flag) {
                                    bean.setVerifyCode(bean.getCode());
                                    diagnoseName.add(bean.getDisName());
                                    // 不需要添加问题词
//                                    DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(bean.getDiagnoseId(),
//                                              bean.getDisName(), bean.getType(),
//                                              bean.getCode(), DiagnoseFieldEnum.code.getKey(), param, ++j);
//                                    diagnoseQuestionList.add(diagnoseQuestion);
                                } else {
                                    bean.setVerifyCode(""); // 将原值清空
                                }
                                if (!oldVerifyCode.equals(bean.getVerifyCode())) {
                                    updateFlag = true;
                                }
                            }

                            // 校验标准词
                            if (StringUtil.isNotEmpty(bean.getStandard())) {
                                String oldVerifyStandard = bean.getVerifyStandard();
                                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(),
                                        DiagnoseFieldEnum.standard.getKey());
                                flag = verifyExist(libSet, bean.getStandard(), typeList);
                                if (!flag) {
                                    bean.setVerifyStandard(bean.getStandard());
                                    DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(bean.getDiagnoseId(),
                                            bean.getDisName(), bean.getType(), bean.getStandard(),
                                            DiagnoseFieldEnum.standard.getKey(), param, ++j);
                                    if (addDiagnoseQuestion(diagnosetQuestionSet, diagnoseQuestion)) {
                                        diagnoseQuestionList.add(diagnoseQuestion);
                                    }
                                    diagnoseName.add(bean.getDisName());
                                } else {
                                    bean.setVerifyStandard(""); // 将原值清空
                                }
                                if (!oldVerifyStandard.equals(bean.getVerifyStandard())) {
                                    updateFlag = true;
                                }
                            }

                            // 校验关联词
                            if (StringUtil.isNotEmpty(bean.getRelation())) {
                                String oldVerifyRelation = bean.getVerifyRelation();
                                String[] arr = bean.getRelation().split("、");
                                StringBuffer sb = new StringBuffer();
                                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(),
                                        DiagnoseFieldEnum.relation.getKey());
                                for (String s : arr) {
                                    flag = verifyExist(libSet, s.trim(), typeList);
                                    if (!flag) {
                                        sb.append(s).append("、");
                                        DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(bean.getDiagnoseId(), bean.getDisName(), bean.getType(),
                                                s.trim(), DiagnoseFieldEnum.relation.getKey(), param, ++j);
                                        if (addDiagnoseQuestion(diagnosetQuestionSet, diagnoseQuestion)) {
                                            diagnoseQuestionList.add(diagnoseQuestion);
                                        }
                                        diagnoseName.add(bean.getDisName());
                                    }
                                }
                                String errWord = sb.toString();
                                if (StringUtil.isNotEmpty(errWord)) {
                                    bean.setVerifyRelation(errWord.substring(0, errWord.length() - 1));
                                } else {
                                    bean.setVerifyRelation(""); // 将原值清空
                                }
                                if (!oldVerifyRelation.equals(bean.getVerifyRelation())) {
                                    updateFlag = true;
                                }
                            }

                            // 校验公表，不需要插入问题词
                            if (DiagnoseFeatureTypeEnum.Lis.getKey() == bean.getType()) {
                                String oldVerifyUnique = bean.getVerifyUnique();
                                if (StringUtil.isNotEmpty(bean.getUniqueName())) { // 公表项数据不为空
                                    if (cryptConfiguer.cryptFlag) {
                                        if (!lisMappingSet.contains(CryptUtil.encrypt_char(bean.getStandard()) + "__" + CryptUtil.encrypt_char(bean.getRelation())
                                                + "__" + CryptUtil.encrypt_char(bean.getUniqueName()))) {
                                            bean.setVerifyUnique("无公表项");
                                            diagnoseName.add(bean.getDisName());
                                        } else {
                                            bean.setVerifyUnique(""); // 将原值清空
                                        }
                                    } else {
                                        if (!lisMappingSet.contains(bean.getStandard() + "__" + bean.getRelation()
                                                + "__" + bean.getUniqueName())) {
                                            bean.setVerifyUnique("无公表项");
                                            diagnoseName.add(bean.getDisName());
                                        } else {
                                            bean.setVerifyUnique(""); // 将原值清空
                                        }
                                    }
                                } else { // 公表项数据为空
                                    bean.setVerifyUnique("无公表项");
                                    diagnoseName.add(bean.getDisName());
                                }
                                if (!oldVerifyUnique.equals(bean.getVerifyUnique())) {
                                    updateFlag = true;
                                }
                            }

                            // 校验结果 2019/10/16 需求确认，不需要校验结果，代码注释
//                            if (StringUtil.isNotEmpty(bean.getResult())) {
//                                String oldVerifyResult = bean.getVerifyResult();
//                                String[] arr = bean.getResult().split("、");
//                                StringBuffer sb = new StringBuffer();
//                                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(),
//                                        DiagnoseFieldEnum.result.getKey());
//                                for (String s : arr) {
//                                    flag = verifyExist(libSet, s.trim(), typeList);
//                                    if (!flag) {
//                                        sb.append(s).append("、");
//                                        DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(bean.getDiagnoseId(),
//                                                bean.getDisName(), bean.getType(), s.trim(),
//                                                DiagnoseFieldEnum.result.getKey(), param, ++j);
//                                        if (addDiagnoseQuestion(diagnosetQuestionSet, diagnoseQuestion)) {
//                                            diagnoseQuestionList.add(diagnoseQuestion);
//                                        }
//                                        diagnoseName.add(bean.getDisName());
//                                    }
//                                }
//                                String errWord = sb.toString();
//                                if (StringUtil.isNotEmpty(errWord)) {
//                                    bean.setVerifyResult(errWord.substring(0, errWord.length() - 1));
//                                } else {
//                                    bean.setVerifyResult(""); // 将原值清空
//                                }
//
//                                if (!oldVerifyResult.equals(bean.getVerifyResult())) {
//                                    updateFlag = true;
//                                }
//                            }
                        }
                        if (updateFlag) {
                            diagnoseDetailList.add(bean);
                        }
                    }

                    // 当前诊断校验通过后再校验其他数据异常，例如：诊断公式里有化验和辅检编码，则化验和辅检数据必须完整
                    if (!diagnoseName.contains(key)) {
                        doLisAndPacs(diagnoseName, lisAndPacs, formula, diagnoseDetailList);
                    }
                }
            }
            Long time4 = System.currentTimeMillis();
            res.append("校验问题词：" + (time4 - time3) / 1000.0).append("秒,");

            // 更新明细
            if (ListUtil.isNotEmpty(diagnoseDetailList)) {
                diagnoseDetailService.updateBatch(diagnoseDetailList); // 效率比框架的updateBatchById高很多
            }

            Long time5 = System.currentTimeMillis();
            res.append("更新明细" + diagnoseDetailList.size() + "条：" + (time5 - time4) / 1000.0).append("秒,");

            // 删除全部的问题词数据
            diagnoseQuestionService.remove(new QueryWrapper<DiagnoseQuestion>());
            // 新增问题词
            if (ListUtil.isNotEmpty(diagnoseQuestionList)) {
                diagnoseQuestionService.insertBatch(diagnoseQuestionList); // 效率比框架的saveBatch高很多
            }
            Long time6 = System.currentTimeMillis();
            res.append("新增问题词" + diagnoseQuestionList.size() + "条：" + (time6 - time5) / 1000.0).append("秒,");

            // 修改问题词表的状态
            List<Diagnose> updateDiagnose = new ArrayList<>();
            for (Diagnose diagnose : diagnoseListWithDB) {
                String oldHasQueston = diagnose.getHasQuestion();
                if (diagnoseName.contains(diagnose.getDisName())) {
                    diagnose.setHasQuestion(String.valueOf(HasQuestionEnum.HAS_QUESTION.getKey()));
                } else {
                    diagnose.setHasQuestion(String.valueOf(HasQuestionEnum.NO_QUESTION.getKey()));
                }
                if (!oldHasQueston.equals(diagnose.getHasQuestion())) {
                    updateDiagnose.add(diagnose);
                }
            }
            // 更新主表信息
            if (ListUtil.isNotEmpty(updateDiagnose)) {
//                diagnoseService.updateBatchById(updateDiagnose);
                diagnoseService.updateBatch(updateDiagnose);
            }
            Long time7 = System.currentTimeMillis();
            res.append("更新主表" + updateDiagnose.size() + "条：" + (time7 - time6) / 1000.0).append("秒,");
            res.append("总计花费时间：" + (time7 - time1) / 1000.0).append("秒。");
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, e.getMessage());
        } finally {
            verifyFlag = true;
        }

        return res.toString();
    }


    /**
     * 校验化验和辅检数据异常
     *
     * @param diagnoseName       有问题的诊断名称
     * @param lisAndPacs         化验辅检数据
     * @param formula            公式数据
     * @param diagnoseDetailList 需要更新的明细
     */
    public void doLisAndPacs(Set<String> diagnoseName, List<DiagnoseDetail> lisAndPacs,
                             List<DiagnoseDetail> formula, List<DiagnoseDetail> diagnoseDetailList) {
        // 处理数据
        for (DiagnoseDetail det : lisAndPacs) {
            // 公式正则匹配化验和辅检的编码值，以编码值结尾或者编码值之后一位不能是数字
            // 避免3.1去匹配3.11成功；点号需要转义
            // 正则示例：(.*3.1)|(.*3.1\\D.*)
            String regex = "(.*" + det.getCode().replace(".", "\\.") + ")|(.*" +
                    det.getCode().replace(".", "\\.") + "\\D.*)";
            boolean verifyAll = false;
            for (DiagnoseDetail form : formula) {
                if (form.getFormula().matches(regex)) {
                    verifyAll = true;
                    break;
                }
            }
            if (verifyAll) { // 如果在公式里存在编码，则数据必须完整
                boolean needUpdate = false;
                if (StringUtil.isEmpty(det.getRelation())) { // 化验、辅检都有
                    det.setVerifyRelation("数据异常");
                    needUpdate = true;
                }
                if (DiagnoseFeatureTypeEnum.Lis.getKey() == det.getType()) { // 化验有
                    if (StringUtil.isEmpty(det.getResult())) {
                        det.setVerifyResult("数据异常");
                        needUpdate = true;
                    }
                }
                if (needUpdate) {
                    diagnoseName.add(det.getDisName());
                    diagnoseDetailList.add(det);
                }
            }
        }
    }

    /**
     * 校验是否已存在问题词数据
     *
     * @param set
     * @param diagnoseQuestion
     * @return
     */
    public Boolean addDiagnoseQuestion(Set<String> set, DiagnoseQuestion diagnoseQuestion) {
        String key = diagnoseQuestion.getDiagnoseId() + "_" + diagnoseQuestion.getType() + "_"
                + diagnoseQuestion.getQuestionType() + "_" + diagnoseQuestion.getQuestionName();
        if (!set.contains(key)) {
            set.add(key);
            return true;
        }
        return false;
    }


    /**
     * 处理诊断特征业务（校验诊断名称|编码|标准词|关联词|结果）
     *
     * @param diagnoseWrapper
     * @param param
     * @return
     */
    public void doDisFuture(DiagnoseWrapper diagnoseWrapper, CommonParam param, Set<String> set,
                            List<DiagnoseDetail> diagnoseDetailList, List<DiagnoseQuestion> diagnoseQuestionList) {
        int i = param.getI();
        int j = param.getJ();
        Set<String> questionSet = new HashSet<>(); // 问题词去重变量

        // 判断诊断名称是否在标准词库中
        boolean flag = true;
        int count = conceptFacade.count(new QueryWrapper<Concept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_type", LexiconTypeEnum.DIAGNOSIS.getKey())
                .eq("lib_name",
                        cryptConfiguer.cryptFlag ?
                            CryptUtil.encrypt_char(diagnoseWrapper.getDisName()) :
                            diagnoseWrapper.getDisName()
                        )
        );
        if (count == 0) {
            flag = false;
        }
        if (!flag) {
            DiagnoseQuestion diagnoseQuestion1 = createDiagnoseQuestion(diagnoseWrapper.getId(),
                    diagnoseWrapper.getDisName(), DiagnoseFeatureTypeEnum.Dis.getKey(),
                    diagnoseWrapper.getDisName(), DiagnoseFieldEnum.disName.getKey(), param, ++j);
            diagnoseQuestionList.add(diagnoseQuestion1);
            param.setDisNameExist(false); // 标识
        }

        // 遍历诊断特征
        for (DiagnoseDetailSave bean : diagnoseWrapper.getDisFeature()) {
            DiagnoseDetail diagnoseDetail = new DiagnoseDetail();
            BeanUtil.copyProperties(bean, diagnoseDetail);
            diagnoseDetail.setDiagnoseId(diagnoseWrapper.getId());
            diagnoseDetail.setCreator(param.getPerson());
            diagnoseDetail.setGmtCreate(param.getNow());
            diagnoseDetail.setModifier(param.getPerson());
            diagnoseDetail.setGmtModified(param.getNow());
            diagnoseDetail.setDisName(diagnoseWrapper.getDisName());
            diagnoseDetail.setOrderNo(++i);
            List<Integer> typeList = new ArrayList<>();

            // 校验编码
            if (StringUtil.isNotEmpty(bean.getCode())) {
                flag = VerifyUtil.verifyCode(String.valueOf(bean.getType()), bean.getCode());
                if (!flag) {
                    diagnoseDetail.setVerifyCode(bean.getCode());
                    param.setFlag(HasQuestionEnum.HAS_QUESTION.getKey());
                    // 不需要添加问题词
//                    DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnoseWrapper.getId(),
//                              diagnoseWrapper.getDisName(), bean.getType(),
//                              bean.getCode(), DiagnoseFieldEnum.code.getKey(), param, ++j);
//                    if (addDiagnoseQuestion(questionSet, diagnoseQuestion)) {
//                        diagnoseQuestionList.add(diagnoseQuestion);
//                    }
                }
            }

            // 校验标准词
            if (StringUtil.isNotEmpty(bean.getStandard())) {
                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(),
                        DiagnoseFieldEnum.standard.getKey());
                flag = verifyExist(set, bean.getStandard(), typeList);
                if (!flag) {
                    diagnoseDetail.setVerifyStandard(bean.getStandard());
                    DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnoseWrapper.getId(),
                            diagnoseWrapper.getDisName(), bean.getType(),
                            bean.getStandard(), DiagnoseFieldEnum.standard.getKey(), param, ++j);
                    if (addDiagnoseQuestion(questionSet, diagnoseQuestion)) {
                        diagnoseQuestionList.add(diagnoseQuestion);
                    }
                }
            }

            // 校验关联词
            if (StringUtil.isNotEmpty(bean.getRelation())) {
                String[] arr = bean.getRelation().split("、");
                StringBuffer sb = new StringBuffer();
                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(),
                        DiagnoseFieldEnum.relation.getKey());
                for (String s : arr) {
                    flag = verifyExist(set, s.trim(), typeList);
                    if (!flag) {
                        sb.append(s).append("、");
                        DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnoseWrapper.getId(),
                                diagnoseWrapper.getDisName(), bean.getType(), s.trim(),
                                DiagnoseFieldEnum.relation.getKey(), param, ++j);
                        if (addDiagnoseQuestion(questionSet, diagnoseQuestion)) {
                            diagnoseQuestionList.add(diagnoseQuestion);
                        }
                    }
                }
                String errWord = sb.toString();
                if (StringUtil.isNotEmpty(errWord)) {
                    diagnoseDetail.setVerifyRelation(errWord.substring(0, errWord.length() - 1));
                } else {
                    diagnoseDetail.setVerifyRelation(""); // 将原值清空
                }
            }

            // 校验结果 2019/10/16 需求确认，不需要校验结果，代码注释
//            if (StringUtil.isNotEmpty(bean.getResult())) {
//                String[] arr = bean.getResult().split("、");
//                StringBuffer sb = new StringBuffer();
//                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(),
//                        DiagnoseFieldEnum.result.getKey());
//                for (String s : arr) {
//                    flag = verifyExist(set, s.trim(), typeList);
//                    if (!flag) {
//                        sb.append(s).append("、");
//                        DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnoseWrapper.getId(),
//                                diagnoseWrapper.getDisName(), bean.getType(), s.trim(),
//                                DiagnoseFieldEnum.result.getKey(), param, ++j);
//                        if (addDiagnoseQuestion(questionSet, diagnoseQuestion)) {
//                            diagnoseQuestionList.add(diagnoseQuestion);
//                        }
//                    }
//                }
//                String errWord = sb.toString();
//                if (StringUtil.isNotEmpty(errWord)) {
//                    diagnoseDetail.setVerifyResult(errWord.substring(0, errWord.length() - 1));
//                } else {
//                    diagnoseDetail.setVerifyResult(""); // 将原值清空
//                }
//            }
            diagnoseDetailList.add(diagnoseDetail);
        }
        param.setI(i);
        param.setJ(j);
        if (ListUtil.isNotEmpty(diagnoseQuestionList)) {
            param.setFlag(HasQuestionEnum.HAS_QUESTION.getKey());
        }
    }


    /**
     * 处理诊断公式业务（校验公式）
     *
     * @param diagnoseWrapper
     * @param param
     * @return
     */
    public void doDisFormula(DiagnoseWrapper diagnoseWrapper, CommonParam param,
                             List<DiagnoseDetail> diagnoseDetailList, List<DiagnoseQuestion> diagnoseQuestionList) {
        List<String> codeList = diagnoseWrapper.getDisFeature().stream().map(row -> row.getCode()).distinct()
                .collect(Collectors.toList());
        codeList.add("拟诊");  // 添加拟诊的编码
        int i = param.getI();
        int j = param.getJ();
        // 遍历诊断公式
        for (DiagnoseDetailSave bean : diagnoseWrapper.getDisformula()) {
            DiagnoseDetail diagnoseDetail = new DiagnoseDetail();
            BeanUtil.copyProperties(bean, diagnoseDetail);
            diagnoseDetail.setDiagnoseId(diagnoseWrapper.getId());
            diagnoseDetail.setCreator(param.getPerson());
            diagnoseDetail.setGmtCreate(param.getNow());
            diagnoseDetail.setModifier(param.getPerson());
            diagnoseDetail.setGmtModified(param.getNow());
            diagnoseDetail.setDisName(diagnoseWrapper.getDisName());
            diagnoseDetail.setOrderNo(++i);
            // 校验公式
            String errMsg = VerifyUtil.verifyFormula(codeList, bean.getFormula());
            if (StringUtil.isNotEmpty(errMsg)) {
                diagnoseDetail.setVerifyFormula(errMsg);
                param.setFlag(HasQuestionEnum.HAS_QUESTION.getKey());
            } else {
                diagnoseDetail.setVerifyFormula("");
            }
            diagnoseDetailList.add(diagnoseDetail);
        }
        param.setI(i);
        param.setJ(j);
    }


    /**
     * 更新主表业务(同时删除明细表和问题词表)
     *
     * @param diagnoseWrapper
     * @param param
     * @return
     */
    public Long insertOrUpdateDiagnose(DiagnoseWrapper diagnoseWrapper, CommonParam param) {
        // 先校验诊断名称是否存在，多人同时点击
        QueryWrapper<Diagnose> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("dis_name", diagnoseWrapper.getDisName())
                .eq("is_deleted", IsDeleteEnum.N.getKey());
        if (diagnoseWrapper.getId() != null) {
            queryWrapper.ne("id", diagnoseWrapper.getId());
        }
//        Diagnose bean = this.getOne(queryWrapper, false);
        if (this.count(queryWrapper) > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "当前疾病的诊断依据已存在！");
        }

        Diagnose diagnose = new Diagnose();
        BeanUtil.copyProperties(diagnoseWrapper, diagnose);
        diagnose.setHasQuestion(String.valueOf(param.getFlag())); // 设置状态
        if (diagnoseWrapper.getId() == null) { //新增
            diagnose.setCreator(param.getPerson());
            diagnose.setGmtCreate(param.getNow());
        } else { // 修改
            // 明细删除
            diagnoseDetailService.remove(new QueryWrapper<DiagnoseDetail>()
                    .eq("diagnose_id", diagnoseWrapper.getId())
            );
            // 删除问题词
            diagnoseQuestionService.remove(new QueryWrapper<DiagnoseQuestion>()
                    .eq("diagnose_id", diagnoseWrapper.getId())
            );
            param.setInsertOrUpdate(InsertOrUpdateEnum.Update.getKey());
        }
        diagnose.setGmtModified(param.getNow());
        this.saveOrUpdate(diagnose);
        return diagnose.getId();
    }


    /**
     * 更新图谱(先校验，再更新)
     *
     * @param updateDiagnoseVO
     * @return
     */
    public void updateNeo(UpdateDiagnoseVO updateDiagnoseVO) {
        Date date = new Date();
        // 更新时间
        this.update(new UpdateWrapper<Diagnose>()
                .eq("id", updateDiagnoseVO.getId())
                .set("modifier", updateDiagnoseVO.getModifier())
                .set("neo_update", date)
                .set("gmt_modified", date)
        );

        // 更新图谱对接接口
        NeoParamVO neoParamVO = new NeoParamVO();
        neoParamVO.setId(updateDiagnoseVO.getId());
        RespDTO<Boolean> respDTO = neoServiceClient.updateNeo(neoParamVO);
        RespDTOUtil.respNGDeal(respDTO, "更新图谱数据失败【远程调用】");
    }


    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        param.setNow(DateUtil.now());
        param.setInsertOrUpdate(InsertOrUpdateEnum.Insert.getKey());
        return param;
    }


    /**
     * 校验词库中是否存在
     *
     * @param set  词库数据
     * @param name 界面输入
     * @param type 转换类型
     * @return 校验结果
     */
    public boolean verifyExist(Set<String> set, String name, List<Integer> type) {
        for (Integer integer : type) {
            if (cryptConfiguer.cryptFlag) {
                if (set.contains(CryptUtil.encrypt_char(name) + "_" + integer)) {
                    return true;
                }
            } else {
                if (set.contains(name + "_" + integer)) {
                    return true;
                }
            }
        }
        return false;
    }


    /**
     * 生成问题词
     *
     * @param disName
     * @param type
     * @param questionName
     * @param commonParam
     * @return
     */
    public DiagnoseQuestion createDiagnoseQuestion(Long diagnoseId, String disName, Integer type, String questionName,
                                                   Integer questionType, CommonParam commonParam, Integer orderNo) {
        DiagnoseQuestion dq = new DiagnoseQuestion();
        dq.setCreator(commonParam.getPerson());
        dq.setGmtCreate(commonParam.getNow());
        dq.setModifier(commonParam.getPerson());
        dq.setGmtModified(commonParam.getNow());
        dq.setDisName(disName);
        dq.setDiagnoseId(diagnoseId);
        dq.setType(type);
        dq.setQuestionName(questionName);
        dq.setQuestionType(questionType);
        dq.setOrderNo(orderNo);
        return dq;
    }


    /**
     * 诊断依据检索标准词和关联词
     */
    public List<String> searchFac(DiagnoseSearchVO diagnoseSearchVO) {
        List<String> res = new ArrayList<>();
        List<Integer> typeList =
                DiagnoseTypeConvertUtil.diagnoseType2LibType(diagnoseSearchVO.getType(), diagnoseSearchVO.getField());
        diagnoseSearchVO.setTypeList(typeList);
        if (ListUtil.isNotEmpty(typeList)) {
            List<LibraryInfo> libraryInfoList = this.search(diagnoseSearchVO);
            res = libraryInfoList.stream().map(row -> row.getName()).distinct().collect(Collectors.toList());
            if (cryptConfiguer.cryptFlag) {
                CryptUtil.decryptList(res);
            }
        }
        return res;
    }


    public static void main(String[] args) {
        String res = "1.2+1.3";
        String str = "1.3";
        String regex = "(.*" + str.replace(".", "\\.") + ")|(.*" +
                str.replace(".", "\\.") + "\\D.*)";
        System.out.println(regex);
        String regex1 = "(.*1\\.3)|(.*1\\.3\\D.*)";
        System.out.println(res.matches(regex));
        System.out.println(res.matches(regex1));
    }
}
