package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.IndexConfigAllDTO;
import com.diagbot.dto.IndexConfigDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.IndexConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.IndexConfigService;
import com.diagbot.service.impl.IndexConfigServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.IndexConfigDiseaseIdVO;
import com.diagbot.vo.IndexConfigDiseaseNameVO;
import com.diagbot.vo.IndexConfigListVO;
import com.diagbot.vo.IndexConfigVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description: TODO
 * @date 2019年3月25日 下午4:20:09
 */
@Component
public class IndexConfigFacade extends IndexConfigServiceImpl {

    @Autowired
    @Qualifier("indexConfigServiceImpl")
    IndexConfigService indexConfigService;
    @Autowired
    UserServiceClient userServiceClient;
    @Autowired
    ConceptFacade conceptFacade;

    /**
     * 分页模糊查询慢病指标值关联维护
     *
     * @param indexConfigDiseaseNameVO
     * @return
     */
    public IPage<IndexConfigDTO> queryIndexConfigPage(IndexConfigDiseaseNameVO indexConfigDiseaseNameVO) {
        IPage<IndexConfigDTO> datas = this.queryIndexConfigPageAlls(indexConfigDiseaseNameVO);
        List<IndexConfigDTO> data = datas.getRecords();
        List<String> modifierid = data.stream().map(ac -> ac.getModifier()).collect(Collectors.toList());
        Map<String, String> userNames = new HashMap<>();
        if (ListUtil.isNotEmpty(modifierid)) {
            RespDTO<Map<String, String>> userNamesDTO = userServiceClient.getUserInfoByIds(modifierid);
            if (RespDTOUtil.respIsNG(userNamesDTO)) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "获取所有用户信息失败");
            }
            userNames = userNamesDTO.data;
        }

        for (IndexConfigDTO evalua : data) {
            evalua.setModifier(userNames.get(evalua.getModifier()));
        }
        return datas;
    }

    /**
     * 获取慢病指标值关联维护
     *
     * @param indexConfigDiseaseIdVO
     * @return
     */
    public List<IndexConfigAllDTO> getIndexConfigLists(IndexConfigDiseaseIdVO indexConfigDiseaseIdVO) {
        QueryWrapper<IndexConfig> IndexConfigQuery = new QueryWrapper<IndexConfig>();
        IndexConfigQuery.eq("disease_id", indexConfigDiseaseIdVO.getDiseaseId()).eq("is_deleted",
                IsDeleteEnum.N.getKey());
        List<IndexConfig> datas = list(IndexConfigQuery);
        // 过滤
        List<IndexConfigAllDTO> data = BeanUtil.listCopyTo(datas, IndexConfigAllDTO.class);

        List<Long> conceptIds = data.stream().map(ac -> ac.getConceptId()).collect(Collectors.toList());
        ConceptSearchVO conceptSearchVO = new ConceptSearchVO();
        conceptSearchVO.setConceptIds(conceptIds);
        Map<Long, String> indexUniques = new HashMap<>();
        if (ListUtil.isNotEmpty(conceptIds)) {
            indexUniques = conceptFacade.getConceptNameMap(conceptSearchVO);
        }

        for (IndexConfigAllDTO evalua : data) {
            evalua.setIndexUnique(indexUniques.get(evalua.getConceptId()));
        }

        return data;
    }

    /**
     * 慢病指标值关联维护修改
     *
     * @param indexConfigListVO
     * @return
     */
    public boolean updateIndexConfigList(IndexConfigListVO indexConfigListVO) {
        boolean res = false;
        if (indexConfigListVO != null && ListUtil.isNotEmpty(indexConfigListVO.getIndexConfigData())) {
            IndexConfigDiseaseIdVO indexConfigDiseaseIdVO = new IndexConfigDiseaseIdVO();
            indexConfigDiseaseIdVO.setDiseaseId(indexConfigListVO.getDiseaseId());

            UpdateWrapper<IndexConfig> IndexConfigNew = new UpdateWrapper<>();
            IndexConfigNew.eq("disease_id", indexConfigListVO.getDiseaseId());
            res = indexConfigService.remove(IndexConfigNew);
            List<IndexConfig> IndexConfigLsit = new ArrayList<IndexConfig>();
            List<IndexConfigVO> data = indexConfigListVO.getIndexConfigData();
            Date optDate = DateUtil.now();
            String operator = UserUtils.getCurrentPrincipleID();
            for (IndexConfigVO dataNew : data) {
                IndexConfig IndexConfig = new IndexConfig();
                BeanUtil.copyProperties(dataNew, IndexConfig);
                IndexConfig.setGmtCreate(optDate);
                IndexConfig.setGmtModified(optDate);
                IndexConfig.setCreator(operator);
                IndexConfig.setModifier(operator);
                IndexConfigLsit.add(IndexConfig);
            }

            res = indexConfigService.saveBatch(IndexConfigLsit);

        }

        return res;
    }

    /**
     * 慢病指标值关联维护添加
     *
     * @param indexConfigListVO
     * @return
     */
    public boolean saveIndexConfigList(IndexConfigListVO indexConfigListVO) {
        boolean res = false;
        if (indexConfigListVO != null
                && ListUtil.isNotEmpty(indexConfigListVO.getIndexConfigData())) {
            List<IndexConfigVO> indexConfigData = indexConfigListVO.getIndexConfigData();
            List<Long> conceptId = new ArrayList<>();
            for (IndexConfigVO e : indexConfigData) {
                conceptId.add(e.getConceptId());
            }

            if(ListUtil.isNotEmpty(conceptId)){
                QueryWrapper<IndexConfig> IndexConfigQuery = new QueryWrapper<>();

                IndexConfigQuery
                        .eq("disease_id", indexConfigListVO.getDiseaseId())
                        .in("concept_id", conceptId);
                List<IndexConfig> dataUnique = list(IndexConfigQuery);
                if (ListUtil.isNotEmpty(dataUnique)) {
                    throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                            "该'" + dataUnique.get(0).getConceptId() + "'已存在");
                }
            }


            List<IndexConfig> IndexConfigLsit = new ArrayList<IndexConfig>();
            List<IndexConfigVO> data = indexConfigListVO.getIndexConfigData();
            Date optDate = DateUtil.now();
            String operator = UserUtils.getCurrentPrincipleID();
            for (IndexConfigVO dataNew : data) {
                IndexConfig IndexConfig = new IndexConfig();
                BeanUtil.copyProperties(dataNew, IndexConfig);
                IndexConfig.setGmtCreate(optDate);
                IndexConfig.setGmtModified(optDate);
                IndexConfig.setCreator(operator);
                IndexConfig.setModifier(operator);
                IndexConfigLsit.add(IndexConfig);
            }
            res = indexConfigService.saveBatch(IndexConfigLsit);
        }

        return res;
    }

    /**
     * 删除慢病指标值关联维护
     *
     * @param indexConfigDiseaseIdVO
     * @return
     */
    public boolean cancelIndexConfigAlls(IndexConfigDiseaseIdVO indexConfigDiseaseIdVO) {
        UpdateWrapper<IndexConfig> IndexConfigNew = new UpdateWrapper<>();
        if (indexConfigDiseaseIdVO.getState().equals(IsDeleteEnum.Y.getKey())) {
            IndexConfigNew
                    .eq("disease_id", indexConfigDiseaseIdVO.getDiseaseId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .set("is_deleted", IsDeleteEnum.Y.getKey())
                    .set("modifier", UserUtils.getCurrentPrincipleID())
                    .set("gmt_modified", DateUtil.now());
        } else if (indexConfigDiseaseIdVO.getState().equals(IsDeleteEnum.N.getKey())) {
            IndexConfigNew
                    .eq("disease_id", indexConfigDiseaseIdVO.getDiseaseId())
                    .eq("is_deleted", IsDeleteEnum.Y.getKey())
                    .set("is_deleted", IsDeleteEnum.N.getKey())
                    .set("modifier", UserUtils.getCurrentPrincipleID())
                    .set("gmt_modified", DateUtil.now());
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "state传参有误，应为'Y'或'N'");
        }
        return update(new IndexConfig(), IndexConfigNew);
    }

}
