package com.diagbot.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.LisSonContactListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.service.impl.RelationServiceImpl;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.LisSonContactListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 医学术语关联业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class LisSonContactFacade extends RelationServiceImpl {

    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 化验子项维护-列表
     *
     * @param lisSonContactListVO
     * @return
     */
    public IPage<LisSonContactListDTO> lisSonContactList(LisSonContactListVO lisSonContactListVO) {
        IPage<LisSonContactListDTO> ipage = this.baseMapper.lisSonContactList(lisSonContactListVO);
        if (ipage.getRecords().size() > 0) {
            List<String> ids = ipage.getRecords()
                    .stream().map(i -> i.getOperName()).distinct().collect(Collectors.toList());
            RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(ids);
            RespDTOUtil.respNGDealCover(respDTO, "获取用户信息失败");
            ipage.getRecords().forEach(i -> {
                i.setLibName(cryptConfiguer.cryptFlag ? CryptUtil.decrypt_char(i.getLibName()) : i.getLibName());
                i.setOperName(respDTO.data.get(i.getOperName()));
                i.setLibNameType(i.getLibName() + "(" + i.getLibType() + ")");
            });
        }
        return ipage;
    }


}
