package com.diagbot.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.OnlyByRootListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.service.impl.RelationServiceImpl;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.OnlyByRootListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 医学术语关联业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class MultContactFacade extends RelationServiceImpl {

    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    /**
     * 医学术语多层关联维护-列表
     *
     * @param onlyByRootListVO
     * @return
     */
    public IPage<OnlyByRootListDTO> multContactList(OnlyByRootListVO onlyByRootListVO) {
        IPage<OnlyByRootListDTO> ipage = this.baseMapper.onlyByRootList(onlyByRootListVO);

        if (ListUtil.isNotEmpty(ipage.getRecords())) {
            ipage.getRecords().forEach(i -> {
                i.setLibName(cryptConfiguer.cryptFlag ? CryptUtil.decrypt_char(i.getLibName()) : i.getLibName());
                i.setLibType(LexiconTypeEnum.getName(i.getLibTypeId().intValue()));
                i.setLibNameType(i.getLibName() + "(" + i.getLibType() + ")");
            });

            List<String> userIds = ipage.getRecords()
                    .stream().filter(i -> StringUtil.isNotBlank(i.getOperName()))
                    .map(i -> i.getOperName()).distinct().collect(Collectors.toList());

            if (ListUtil.isNotEmpty(userIds)) {
                RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(userIds);
                RespDTOUtil.respNGDealCover(respDTO, "获取用户信息失败");
                ipage.getRecords().forEach(i -> {
                    i.setOperName(respDTO.data.get(i.getOperName()));
                });
            }
        }

        return ipage;
    }

}
