package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.dto.RulePubDTO;
import com.diagbot.entity.RulePub;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.RulePubServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.IntegerUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RulePubIdVO;
import com.diagbot.vo.RulePubPageVO;
import com.diagbot.vo.RulePubSaveVO;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/1/30 15:44
 */
@Component
public class RulePubFacade extends RulePubServiceImpl {

    public IPage<RulePubDTO> pageRulePub(RulePubPageVO rulePubPageVO) {
        IPage<RulePubDTO> res = new Page<>(rulePubPageVO.getCurrent(), rulePubPageVO.getSize());
        QueryWrapper<RulePub> rulePubQueryWrapper = new QueryWrapper<>();
        rulePubQueryWrapper.eq(!IntegerUtil.isNull(rulePubPageVO.getId()), "id", rulePubPageVO.getId())
                .like(StringUtil.isNotBlank(rulePubPageVO.getPubName()), "pub_name", rulePubPageVO.getPubName())
                .orderByDesc("id");
        IPage<RulePub> page = this.page(rulePubPageVO, rulePubQueryWrapper);
        BeanUtil.copyProperties(page, res);
        if (ListUtil.isNotEmpty(page.getRecords())) {
            List<RulePubDTO> rulePubDTOList = ListUtil.newArrayList();
            for (RulePub rulePub : page.getRecords()) {
                RulePubDTO rulePubDTO = new RulePubDTO();
                BeanUtil.copyProperties(rulePub, rulePubDTO);
                rulePubDTOList.add(rulePubDTO);
            }
            res.setRecords(rulePubDTOList);
        }
        return res;
    }

    public Boolean removeRulePub(RulePubIdVO rulePubIdVO) {
        return this.removeById(rulePubIdVO.getId());
    }

    public RulePubDTO getRulePub(RulePubIdVO rulePubIdVO) {
        RulePub rulePub = this.getById(rulePubIdVO.getId());
        RulePubDTO rulePubDTO = new RulePubDTO();
        BeanUtil.copyProperties(rulePub, rulePubDTO);
        return rulePubDTO;
    }

    public Boolean saveRulePub(@Valid @RequestBody RulePubSaveVO rulePubSaveVO) {
        if (null == rulePubSaveVO) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL);
        }
        RulePub rulePub = new RulePub();
        BeanUtil.copyProperties(rulePubSaveVO, rulePub);
        return this.saveOrUpdate(rulePub);
    }
}
