package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.NLPServiceClient;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.SimilarLibraryDTO;
import com.diagbot.entity.LibraryInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/10/28 15:10
 */
@Component
public class SimilarFacade {
    @Autowired
    NLPServiceClient nlpServiceClient;
    @Autowired
    LibraryInfoFacade libraryInfoFacade;
    @Autowired
    private CryptConfiguer cryptConfiguer;

    public List<SimilarLibraryDTO> getSimilarList(String inputWord) {
        List<String> libraryNameList = nlpServiceClient.getSimilarList(inputWord);
        List<SimilarLibraryDTO> libraryList = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(libraryNameList)) {
            if (cryptConfiguer.cryptFlag){
                CryptUtil.encryptList(libraryNameList);
            }
            QueryWrapper<LibraryInfo> libraryInfoQueryWrapper = new QueryWrapper<>();
            libraryInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("name", libraryNameList)
                    .orderByDesc("is_concept")
                    .orderByAsc("type_id");
            List<LibraryInfo> libraryInfos = libraryInfoFacade.list(libraryInfoQueryWrapper);
            libraryList = BeanUtil.listCopyTo(libraryInfos, SimilarLibraryDTO.class);
            for (SimilarLibraryDTO similarLibraryDTO : libraryList) {
                if (similarLibraryDTO.getTypeId() != null) {
                    similarLibraryDTO.setLibType(LexiconTypeEnum.getName(similarLibraryDTO.getTypeId().intValue()));
                }
                if (similarLibraryDTO.getIsConcept().equals(1)) {
                    similarLibraryDTO.setIsConceptRemark("标准词");
                } else if (similarLibraryDTO.getIsConcept().equals(0)) {
                    similarLibraryDTO.setIsConceptRemark("同义词");
                }
            }
        }
        return libraryList;
    }
}