package com.diagbot.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.BIItemDTO;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptEnDeDTO;
import com.diagbot.dto.ConceptRes;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.dto.GetAllInformationDTO;
import com.diagbot.dto.GetAllLisConceptDTO;
import com.diagbot.dto.LibraryInfoEnDeDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.vo.GetAllInformationVO;
import com.diagbot.vo.GetAllLisConceptVO;
import com.diagbot.vo.IndexLexiconVO;
import com.diagbot.vo.IndexVO;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
public interface ConceptMapper extends BaseMapper<Concept> {
	
	IPage<GetAllInformationDTO> getAllInformation(GetAllInformationVO getAllInformationVO);

    /**
     * 获取所有化验公表项
     *
     * @param getAllLisConceptVO
     * @return
     */
    List<GetAllLisConceptDTO> getAllLisConcept(GetAllLisConceptVO getAllLisConceptVO);

    /**
     * 获取概念-关系-概念
     *
     * @param conceptWrapper 查询参数
     * @return 起点术语和终点术语
     */
    List<ConceptRes> getConcept(ConceptWrapper conceptWrapper);

    /**
     * 获取带排序概念-关系-概念
     *
     * @param conceptWrapper 查询参数
     * @return 起点术语和终点术语以及排序号
     */
    List<ConceptWithOrderRes> getConceptWithOrder(ConceptWrapper conceptWrapper);



    /**
     * 检索
     *
     * @param indexVO 查询参数
     * @return 检索结果
     */
    List<ConceptBaseDTO> index(IndexVO indexVO);

    /**
     * 根据名称和词性检索
     *
     * @param indexLexiconVO
     * @return
     */
    List<ConceptBaseDTO> indexByLexicon(IndexLexiconVO indexLexiconVO);

    /**
     * 知识库标准化-获取医学数据统计
     * @return
     */
    List<BIItemDTO> queryLibTypeCou();


    List<ConceptEnDeDTO> getConceptEnDe();

    void updateBatchEn(List<ConceptEnDeDTO> conceptEnDeDTOList);
    
}
