package com.diagbot.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @description: 校验工具类
 * @author: zhoutg
 * @time: 2019/8/12 14:44
 */
public class VerifyUtil {


    /**
     *  校验编码是否正确，
     *
     * @param type 诊断依据类型
     * @param val 值 ，如：1.1,1.99，点后最多2位
     * @return 校验结果
     */
    public static boolean verifyCode(String type, String val) {
        if (StringUtil.isEmpty(type)) {
            return false;
        }
//        String regex = type + "\\.[1-9]\\d?";
		String regex = type + "\\.(0|[1-9]\\d?)";
        return Pattern.matches(regex, val);
    }


    public static void main(String[] args) {
		System.out.println(verifyStartEnd("1.1+1.2"));
//    	String str = "1.99";
//		System.out.println(verifyCode("1", str));
//
//    	List<String> code = new ArrayList<>();
//    	code.add("拟诊");
//    	code.add("1.1");
//		System.out.println(verifyFormula(code, str));
//    	List<String> list = new ArrayList<>();
//    	list.add("1.1");
//    	list.add("1.2");
//		checkCodeEquation(list, "(1.1/1.2)任一+");
    }

    /**
     * 校验公式编码
     * @param coding 编码
     * @param str  公式
     * @return
     */
    public static boolean checkCodeEquation(List<String> coding, String str) {
		boolean res = true;
		// 分割出所有的数字
		String[] split = str.split("\\+");
		String[] digitalSplit = str.split("[+ ( / ) 任 一 二 三 四 五 六 七 八 九 十  ]");
		List<String> numberList = new ArrayList<String>();
		for (int i = 0; i < digitalSplit.length; i++) {
			if (!digitalSplit[i].equals("")) {
				numberList.add(digitalSplit[i]);
			}
		}

		res = coding.containsAll(numberList);
		if (res == true) {
			for (int i = 0; i < split.length; i++) {
				String str1 = split[i].substring(0, split[i].indexOf(")") + 1);
				// 检查是否以括号开头，以括号结尾
				boolean start = str1.startsWith("(");
				boolean end = str1.endsWith(")");

				if (start == false || end == false) {
					res = false;
					break;
					//throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "公式格式有误");
				}
				String str2 = split[i].substring(split[i].indexOf(")") + 1);
				String[] array = { "任一", "任二", "任三", "任四", "任五", "任六", "任七", "任八", "任九", "任十" };
				if (str2.length() > 0) {
					boolean pp = Arrays.asList(array).contains(str2);
					if (pp == false) {
						res = false;
						break;
					}
				} else {
					res = false;
					break;
				}
			}
		} else {
			res = false;
		}

		return res;
	}


	/**
	 * 校验括号中的值是否正确，大于等于任数，不能只有一个编码
	 *
	 * @param s
	 * @return
	 */
	public static boolean verifyValue(String s) {
		String regex = "\\(.*?\\)任"; // 非贪婪模式
		Pattern pattern = Pattern.compile(regex);
		Matcher matches = pattern.matcher(s);
		while(matches.find()) {
			String str = matches.group();
			String numStr = s.substring(matches.end(), matches.end()+1);

			int i = 0;
			switch (numStr) {
				case "一":
					i = 1;
					break;
				case "二":
					i = 2;
					break;
				case "三":
					i = 3;
					break;
				case "四":
					i = 4;
					break;
				case "五":
					i = 5;
					break;
				case "六":
					i = 6;
					break;
				case "七":
					i = 7;
					break;
				case "八":
					i = 8;
					break;
				case "九":
					i = 9;
					break;
				case "十":
					i = 10;
					break;
			}
			int num = str.split("/").length;
			if (i > num || 1 == num) { // 括号内只有一个编码
				return true;
			}
		}
		return false;
	}


	/**
	 * 1、校验括号是否匹配，是否嵌套
	 * 2、校验开头和结尾字符,开头不能是：)+/，结尾不能是：()+/
	 * 3、校验公式分隔符
	 * 4、校验公式中的编码值是否全部在编码中 （业务校验）
	 * 5、校验括号中的值是否正确，大于等于任数，不能只有一个编码（业务校验）
	 *
	 * @param s
	 * @return
	 */
	public static String verifyFormula(List<String> codeList, String s) {
		String errMsg = "公式格式错误";
		try {
			// 1、校验括号是否匹配，是否嵌套
			if (!verifyBracketNest(s)) {
				return errMsg;
			}

			// 2、校验开头和结尾字符,开头不能是：)+/，结尾不能是：()+/
			if (verifyStartEnd(s)) {
				return errMsg;
			}

			// 3、校验公式分隔符
			List<String> splitCode = verifySplit(s);
			int i = 0;
			for (String str : splitCode) {
				// 如果中间出现空字符串，表明出错了
				if (StringUtil.isEmpty(str) && i != 0 && i != splitCode.size() - 1) {
					return errMsg;
				}
				i++;
			}

			// 4、校验公式中的编码值是否全部在编码中
			List<String> codes = new ArrayList<>();
			for (String str : splitCode) {
				if (StringUtil.isNotEmpty(str)) {
					codes.add(str);
				}
			}
			if (!codeList.containsAll(codes)) {
				return errMsg;
			}

			// 5、校验括号中的值是否正确，大于等于任数，不能只有一个编码
			if (verifyValue(s)) {
				return errMsg;
			}

			// 6、校验拟诊是否放首位
			if (!verifyDiseaseType(s)) {
				return errMsg;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return errMsg;
		}
		return "";
	}


	/**
	 * 校验开头和结尾字符,开头不能是：)+/，结尾不能是：()+/
	 * @param s
	 * @return
	 */
	public static boolean verifyStartEnd(String s) {
		String startRegex =
				"(\\)|\\+|/).*" ;
		String endRegex =
				".*(\\(|\\)|\\+|/)" ;
		if (Pattern.matches(startRegex, s)) {
			return true;
		}
		if (Pattern.matches(endRegex, s)) {
			return true;
		}
		return false;
	}


	/**
	 * 以最大长度分隔符分隔
	 * @param s
	 * @return
	 */
	public static List<String> verifySplit(String s) {
		s = s.replaceAll("\\（", "(");
		s = s.replaceAll("\\）", ")");
		s = s.replaceAll(" ", "");
		String regex =
				"\\)任一\\+\\(" + "|\\)任二\\+\\(" + "|\\)任三\\+\\(" + "|\\)任四\\+\\(" + "|\\)任五\\+\\(" + "|\\)任六\\+\\(" + "|\\)任七\\+\\(" + "|\\)任八\\+\\(" + "|\\)任九\\+\\(" + "|\\)任十\\+\\(" +
						"|\\)任一\\+" + "|\\)任二\\+"  + "|\\)任三\\+" + "|\\)任四\\+" + "|\\)任五\\+" + "|\\)任六\\+" + "|\\)任七\\+" + "|\\)任八\\+" + "|\\)任九\\+" + "|\\)任十\\+" +
						"|\\)任一" + "|\\)任二" + "|\\)任三" + "|\\)任四" + "|\\)任五" + "|\\)任六" + "|\\)任七" + "|\\)任八" + "|\\)任九" + "|\\)任十" +
						"|\\+\\(" +
						"|\\(" +
						"|\\)" +
						"|\\+" +
						"|/" ;
		String[] arr = s.split(regex);
		return Arrays.asList(arr);
	}


	/**
	 * 校验括号是否匹配，是否嵌套
	 * @param s
	 * @return
	 */
	public static boolean verifyBracketNest(String s) {
		Stack<Character> sc=new Stack<Character>();
		char[] c=s.toCharArray();
		for (int i = 0; i < c.length; i++) {
			if (c[i]=='(') {
				if(sc.isEmpty()) {
					sc.push(c[i]);
				} else {
					if (sc.peek()=='(') {
						return false;
					}
				}
			}
			else if (c[i]==')') {
				if(sc.isEmpty()){
					return false;
				}
				else{
					if (sc.peek()=='(') {
						sc.pop();
					}
				}
			}
		}
		if (sc.empty()) {
			return true;
		}else {
			return false;
		}
	}


	/**
	 * 校验任字前面是否“）”
	 * @param s
	 * @return
	 */
	public static boolean verifyRen(String s) {
		int i = s.indexOf("任");
		while(-1 != i) {
			if(i == 0) {
				return false;
			} else {
				char ch = s.charAt(i-1);
				if (')' != ch) {
					return false;
				}
				i =  s.indexOf("任", i + 1);
			}
		}
		return true;
	}


	/**
	 * 获取编码
	 * @param s
	 * @return
	 */
	public static List<String> splitCode(String s) {
		List<String> res = new ArrayList<>();
		String resg1 = "\\(|/|\\)|任一|任二|任三|任四|任五|任六|任七|任八|任九|任十|\\+";
		String[] arr = s.split(resg1);
		for(String str : arr) {
			if (StringUtil.isNotEmpty(str)) {
				res.add(str);
			}
		}
		return res;
	}


	/**
	 * 是否出现重复分隔符
	 * @param s
	 * @return
	 */
	public static boolean isRepeat(String s) {
		String regex = ".*((\\()|(\\))|(\\+)|(//)|(任一)|(任二)|(任三)|(任四)|(任五)|(任六)|(任七)|(任八)|(任九)|(任十)){2,}.*";
		return Pattern.matches(regex, s);
	}


	/**
	 * 诊断类型（拟诊）匹配，拟诊两个字只能放首位
	 * @param s
	 * @return
	 */
	public static boolean verifyDiseaseType(String s) {
		int index = s.indexOf("拟诊");
		if (index > 0) {
			return false;
		}
		index = s.indexOf( "拟诊", index+1); // 再次校验，防止出现多个拟诊
		if (index > 0) {
			return false;
		}
		return true;
	}
}
