package com.diagbot.vo;

import com.diagbot.annotation.CryptField;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptCommon;
import com.diagbot.entity.LibraryInfo;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * @Description
 * @author rgb
 * @time 2019-03
 */
@Getter
@Setter
public class AddConceptInfoVO {
	
	/**
     * 概念id,当不为空时，即编辑操作
     */
	@ApiModelProperty(value="概念id")
    private Long conceptId;
	
	/**
     * 标准术语名称 提示使用
     */
	@CryptField
	@ApiModelProperty(value="标准术语名称",required=true)
	@NotBlank(message="标准术语名称必传")
    private String name;
    
    /**
     * 术语类型
     */
	@ApiModelProperty(value="术语类型",required=true)
	@NotBlank(message="术语类型必传")
    private String type;

	/**
     * 术语明细
     */
	@ApiModelProperty(value="术语明细",required=true)
	@NotEmpty(message="术语明细不能为空")
	private List<AddConceptInfoDetailVO> detailList;
	
	/**
	 * 诊断类型术语扩展
	 */
	@ApiModelProperty(value="诊断类型术语扩展")
	private AddCptDiseaseExtVO addCptDiseaseExtVO;

	/**
	 * 症状扩展属性
	 */
	@ApiModelProperty(value="症状扩展属性")
	private SymptomExtVO symptomExtVO;
	
	/**
     * 性别：1:男, 2:女, 3:通用
     */
	@ApiModelProperty(value="性别：1:男, 2:女, 3:通用")
	@Range(min=1,max=3,message="性别须是1-3")
    private Integer sexType=3;

    /**
     * 症状发生的最小年龄
     */
	@ApiModelProperty(value="症状发生的最小年龄")
	@Range(min=0,max=200,message="年龄须是0-200")
    private Integer minAge=0;

    /**
     * 症状发生的最大年龄
     */
	@ApiModelProperty(value="症状发生的最大年龄")
	@Range(min=0,max=200,message="年龄须是0-200")
    private Integer maxAge=200;
	
    /**
     * 是否推送：0-不推送；1-推送
     */
	@ApiModelProperty(value="是否推送：0-不推送；1-推送")
    private String isPush="0";
	
	/**
	 * 后端业务字段，前端不用
	 */
	private Integer lineNum;
	
	/**
	 * 后端业务字段，前端不用
	 */
	private Integer typeId;
	
	/**
	 * 后端业务字段，前端不用
	 */
	private Concept concept;
	
	/**
	 * 后端业务字段，前端不用
	 */
	private ConceptCommon conceptCommon;
	
	/**
	 * 后端业务字段，前端不用
	 */
	private List<LibraryInfo> libraryInfoList;
	
	

}
