package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.BIItemDTO;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.GetAllConceptDTO;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.GetAllInformationDTO;
import com.diagbot.dto.GetAllLisConceptDTO;
import com.diagbot.dto.GetConceptInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConceptFacade;
import com.diagbot.vo.AddConceptInfoVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptTypeVO;
import com.diagbot.vo.GetAllConceptVO;
import com.diagbot.vo.GetAllForRelationVO;
import com.diagbot.vo.GetAllInformationVO;
import com.diagbot.vo.GetAllLisConceptVO;
import com.diagbot.vo.GetConceptInfoDetailVO;
import com.diagbot.vo.GetConceptPacInfosVO;
import com.diagbot.vo.RemoveConceptInfoVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/concept")
@SuppressWarnings("unchecked")
@Api(value = "医学术语命名维护相关API", tags = { "知识库标准化-医学术语命名维护相关API" })
public class ConceptController {

    @Autowired
    private ConceptFacade conceptFacade;

    @ApiOperation(value = "知识库标准化-获取所有化验公表项[by:rengb]")
    @PostMapping("/getAllLisConcept")
    @SysLogger("getAllLisConcept")
    public RespDTO<List<GetAllLisConceptDTO>> getAllLisConcept(@RequestBody GetAllLisConceptVO getAllLisConceptVO) {
        return RespDTO.onSuc(conceptFacade.getAllLisConcept(getAllLisConceptVO));
    }

    @ApiOperation(value = "知识库标准化-获取所有医学术语命名[by:rengb]")
    @PostMapping("/getAllConcept")
    @SysLogger("getAllConcept")
    public RespDTO<List<GetAllConceptDTO>> getAllConcept(@RequestBody GetAllConceptVO getAllConceptVO) {
        return RespDTO.onSuc(conceptFacade.getAllConcept(getAllConceptVO));
    }

    @ApiOperation(value = "知识库标准化-获取所有概念(术语关系维护时筛选使用)[by:rengb]")
    @PostMapping("/getAllForRelation")
    @SysLogger("getAllForRelation")
    public RespDTO<List<GetAllForRelationDTO>> getAllForRelation(@RequestBody GetAllForRelationVO getAllForRelationVO) {
        return RespDTO.onSuc(conceptFacade.getAllForRelation(getAllForRelationVO));
    }

    @ApiOperation(value = "知识库标准化-获取医学术语命名列表[by:rengb]")
    @PostMapping("/getAllInformation")
    @SysLogger("getAllInformation")
    public RespDTO<IPage<GetAllInformationDTO>> getAllInformation(@RequestBody GetAllInformationVO getAllInformationVO) {
        return RespDTO.onSuc(conceptFacade.getAllInformation(getAllInformationVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语命名删除或者恢复[by:rengb]")
    @PostMapping("/removeConceptInfo")
    @SysLogger("removeConceptInfo")
    @Transactional
    public RespDTO<Boolean> removeConceptInfo(@Valid @RequestBody RemoveConceptInfoVO removeConceptInfoVO) {
        return RespDTO.onSuc(conceptFacade.removeConceptInfo(removeConceptInfoVO));
    }

    @ApiOperation(value = "知识库标准化-获取医学术语命名详情[by:rengb]")
    @PostMapping("/getConceptInfoDetail")
    @SysLogger("getConceptInfoDetail")
    public RespDTO<GetConceptInfoDTO> getConceptInfoDetail(@Valid @RequestBody GetConceptInfoDetailVO getConceptInfoDetailVO) {
        return RespDTO.onSuc(conceptFacade.getConceptInfoDetail(getConceptInfoDetailVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语命名添加或者编辑[by:rengb]")
    @PostMapping("/addConceptInfo")
    @SysLogger("addConceptInfo")
    @Transactional
    public RespDTO<Boolean> addConceptInfo(@Valid @RequestBody AddConceptInfoVO addConceptInfoVO) {
        return RespDTO.onSuc(conceptFacade.addConceptInfo(addConceptInfoVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语命名excel文件导入[by:rengb]")
    @PostMapping("/conceptInfoExcelIm")
    @Transactional
    public RespDTO<Boolean> conceptInfoExcelIm(@RequestParam("uploadfile") MultipartFile file, HttpServletRequest request) {
        return RespDTO.onSuc(conceptFacade.conceptInfoExcelIm(file));
    }

    @ApiOperation(value = "知识库标准化-获取医学数据统计[by:rengb]")
    @PostMapping("/queryLibTypeCou")
    @SysLogger("queryLibTypeCou")
    @ApiIgnore
    public RespDTO<List<BIItemDTO>> queryLibTypeCou() {
        return RespDTO.onSuc(conceptFacade.queryLibTypeCou());
    }

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @ApiOperation(value = "知识库标准化-根据概念Id列表获取概念列表Map[by:gaodm]",
            notes = "conceptIds: 术语Id列表")
    @PostMapping("/getConceptMap")
    @SysLogger("getConceptMap")
    @ApiIgnore
    public RespDTO<Map<String, Long>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO) {
        return RespDTO.onSuc(conceptFacade.getConceptMap(conceptSearchVO));
    }

    /**
     * 根据类型获取术语列表（科室、辅检、慢病）
     *
     * @param conceptTypeVO 类型
     * @return 术语列表
     */
    @ApiOperation(value = "知识库标准化-根据类型获取术语列表（科室、辅检、慢病）[by:gaodm]",
            notes = "type: 1:科室、2:辅检、3:慢病")
    @PostMapping("/getConceptListByType")
    @SysLogger("getConceptListByType")
    //    @ApiIgnore
    public RespDTO<List<ConceptBaseDTO>> getConceptListByType(@RequestBody ConceptTypeVO conceptTypeVO) {
        return RespDTO.onSuc(conceptFacade.getConceptListByType(conceptTypeVO));
    }

    @ApiOperation(value = "知识库标准化-批量更新拼音[by:gaodm]",
            notes = "")
    @PostMapping("/piyinUp")
    @SysLogger("piyinUp")
    public RespDTO<Boolean> piyinUp() {
        return RespDTO.onSuc(conceptFacade.piyinUp());
    }

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param getConceptPacInfosVO 搜索参数
     * @return 术语id和术语 name
     */
    @ApiOperation(value = "知识库标准化-获取辅检概念[by:wangyu]",
            notes = "inputStr: 输入内容")
    @PostMapping("/getConceptPacInfos")
    @SysLogger("getConceptPacInfos")
    @ApiIgnore
    public RespDTO<List<ConceptBaseDTO>> getConceptPacInfos(@RequestBody GetConceptPacInfosVO getConceptPacInfosVO) {
        return RespDTO.onSuc(conceptFacade.getConceptPacInfos(getConceptPacInfosVO));
    }
}
