package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.BIItemDTO;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptIndexDTO;
import com.diagbot.dto.GetConceptDetailListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConceptDetailFacade;
import com.diagbot.vo.AddConceptDetailVO;
import com.diagbot.vo.ConceptIndexVO;
import com.diagbot.vo.GetConceptDetailListVO;
import com.diagbot.vo.GetConceptDetailVO;
import com.diagbot.vo.RemoveConceptDetailVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-16
 */
@RestController
@RequestMapping("/conceptDetail")
@SuppressWarnings("unchecked")
@Api(value = "知识库标准化-医学术语静态知识维护相关API", tags = { "知识库标准化-医学术语静态知识维护相关API" })
public class ConceptDetailController {

    @Autowired
    private ConceptDetailFacade conceptDetailFacade;

    @ApiOperation(value = "知识库标准化-获取医学术语静态知识列表[by:rengb]")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<GetConceptDetailListDTO>> page(@RequestBody GetConceptDetailListVO getConceptDetailListVO) {
        return RespDTO.onSuc(conceptDetailFacade.getListFac(getConceptDetailListVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语静态知识删除|恢复[by:rengb]")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@Valid @RequestBody RemoveConceptDetailVO removeConceptDetailVO) {
        return RespDTO.onSuc(conceptDetailFacade.delete(removeConceptDetailVO));
    }

    @ApiOperation(value = "知识库标准化-获取医学术语静态知识详情[by:rengb]")
    @PostMapping("/getByConceptId")
    @SysLogger("getByConceptId")
    public RespDTO<ConceptDetailDTO> getByConceptId(@Valid @RequestBody GetConceptDetailVO getConceptDetailVO) {
        return RespDTO.onSuc(conceptDetailFacade.getByConceptId(getConceptDetailVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语静态知识添加或者编辑[by:rengb]")
    @PostMapping("/insertOrUpdate")
    @SysLogger("insertOrUpdate")
    @Transactional
    public RespDTO<Boolean> insertOrUpdate(@Valid @RequestBody List<AddConceptDetailVO> addConceptDetailVOList) {
        return RespDTO.onSuc(conceptDetailFacade.insertOrUpdate(addConceptDetailVOList));
    }


    @ApiOperation(value = "知识库标准化-内容检索[by:rengb]", notes = "")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<ConceptIndexDTO>> index(@Valid @RequestBody ConceptIndexVO conceptIndexVO) {
        List<ConceptIndexDTO> data = conceptDetailFacade.indexFac(conceptIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "数据统计-静态知识数据统计[by:zhaops]", notes = "")
    @PostMapping("/conceptDetailBI")
    @SysLogger("conceptDetailBI")
    @ApiIgnore
    public RespDTO<List<BIItemDTO>> conceptDetailBI() {
        List<BIItemDTO> data = conceptDetailFacade.conceptDetailBI();
        return RespDTO.onSuc(data);
    }
}