package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.DiagnoseDetailSaveDTO;
import com.diagbot.dto.DiagnosePageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.VerifyDataDTO;
import com.diagbot.entity.wrapper.DiagnoseWrapper;
import com.diagbot.facade.DiagnoseFacade;
import com.diagbot.facade.LibraryInfoFacade;
import com.diagbot.vo.DeleteDiagnoseVO;
import com.diagbot.vo.DiagnosePageVO;
import com.diagbot.vo.DiagnoseSearchVO;
import com.diagbot.vo.DiseaseIndexVO;
import com.diagbot.vo.GetDiagnoseDetailVO;
import com.diagbot.vo.UpdateDiagnoseVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 诊断依据表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2019-08-08
 */
@RestController
@RequestMapping("/diagnose")
@Api(value = "知识库标准化-诊断依据数据维护相关API", tags = { "知识库标准化-诊断依据数据维护相关API" })
public class DiagnoseController {

    @Autowired
    DiagnoseFacade diagnoseFacade;
    @Autowired
    LibraryInfoFacade libraryInfoFacade;

    @ApiOperation(value = "诊断依据-分页[by:zhoutg]", notes = "")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<DiagnosePageDTO>> page(@RequestBody DiagnosePageVO diagnosePageVO) {
        IPage<DiagnosePageDTO> data = diagnoseFacade.getListFac(diagnosePageVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "诊断依据-删除[by:zhoutg]", notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@Valid @RequestBody DeleteDiagnoseVO deleteDiagnoseVO) {
        diagnoseFacade.delete(deleteDiagnoseVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "诊断依据-诊断检索[by:zhoutg]", notes = "")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<ConceptBaseDTO>> index(@Valid @RequestBody DiseaseIndexVO diseaseIndexVO) {
        List<ConceptBaseDTO> data = diagnoseFacade.indexFac(diseaseIndexVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "诊断依据-校验并保存数据[by:zhoutg]")
    @PostMapping("/verifyData")
    @SysLogger("verifyData")
    @Transactional
    public RespDTO<VerifyDataDTO> verifyData(@RequestBody DiagnoseWrapper diagnoseWrapper) {
        VerifyDataDTO diagnoseDetailList = diagnoseFacade.verifyData(diagnoseWrapper);
        return RespDTO.onSuc(diagnoseDetailList);
    }


    @ApiOperation(value = "诊断依据-暂存数据[by:zhoutg]")
    @PostMapping("/tempSaveOrUpate")
    @SysLogger("tempSaveOrUpate")
    @Transactional
    public RespDTO<Long> tempSaveOrUpate(@RequestBody DiagnoseWrapper diagnoseWrapper) {
        Long id = diagnoseFacade.tempSaveOrUpdateFac(diagnoseWrapper);
        return RespDTO.onSuc(id);
    }


    @ApiOperation(value = "诊断依据-标准词和关联词搜索[by:zhoutg]")
    @PostMapping("/search")
    @SysLogger("search")
    public RespDTO<List<String>> search(@Valid @RequestBody DiagnoseSearchVO diagnoseSearchVO) {
        List<String> data = diagnoseFacade.searchFac(diagnoseSearchVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "诊断依据-校验所有数据[by:zhoutg]")
    @PostMapping("/verifyAllData")
    @SysLogger("verifyAllData")
    @Transactional
    public RespDTO<String> verifyAllData() {
        String str = diagnoseFacade.verifyAllData();
        return RespDTO.onSuc(str);
    }


    @ApiOperation(value = "诊断依据-更新图谱[by:zhoutg]")
    @PostMapping("/updateNeo")
    @SysLogger("updateNeo")
    @Transactional
    public RespDTO<Boolean> updateNeo4j(@RequestBody UpdateDiagnoseVO updateDiagnoseVO) {
        diagnoseFacade.updateNeo(updateDiagnoseVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "诊断依据-获取详情[by:zhoutg]")
    @PostMapping("/getDetail")
    @SysLogger("getDetail")
    public RespDTO<DiagnoseDetailSaveDTO> getDetail(@Valid @RequestBody GetDiagnoseDetailVO getDiagnoseDetailVO) {
        DiagnoseDetailSaveDTO diagnoseDetailSaveDTO = diagnoseFacade.getDetail(getDiagnoseDetailVO);
        return RespDTO.onSuc(diagnoseDetailSaveDTO);
    }


    @ApiOperation(value = "诊断依据-加载诊断依据所需词库[by:zhoutg]")
    @PostMapping("/loadRedisWithDiagnose")
    @SysLogger("loadRedisWithDiagnose")
    @ApiIgnore
    public RespDTO<Boolean> loadRedisWithDiagnose() {
        libraryInfoFacade.loadRedisWithDiagnose();
        return RespDTO.onSuc(true);
    }
}
