package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.DiseaseInfoDTO;
import com.diagbot.dto.IndexConfigAllDTO;
import com.diagbot.dto.IndexConfigDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.IndexConfigFacade;
import com.diagbot.vo.DiseaseNameVO;
import com.diagbot.vo.IndexConfigDiseaseIdVO;
import com.diagbot.vo.IndexConfigDiseaseNameVO;
import com.diagbot.vo.IndexConfigListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import javax.validation.Valid;

/**
 * <p>
 * 指标配置 前端控制器
 *
 * @author wangfeng
 * @since 2019-03-25
 */
@RestController
@RequestMapping("/indexConfig")
@Api(value = "慢病指标值关联维护API[by:wangfeng]", tags = { "知识库标准化-慢病指标值关联维护API" })
@SuppressWarnings("unchecked")
public class IndexConfigController {

    @Autowired
    IndexConfigFacade indexConfigFacade;

    @ApiOperation(value = "知识库标准化-分页模糊查询慢病指标值关联维护[by:wangfeng]", notes = "diseaseName：慢病名字")
    @PostMapping("/queryIndexConfigPages")
    @SysLogger("queryIndexConfigPages")
    @Transactional
    public RespDTO<IPage<IndexConfigDTO>> queryIndexConfigPages(
            @RequestBody IndexConfigDiseaseNameVO indexConfigDiseaseNameVO) {
        IPage<IndexConfigDTO> data = indexConfigFacade.queryIndexConfigPage(indexConfigDiseaseNameVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-获取慢病指标值关联维护[by:wangfeng]", notes = "diseaseId ： 慢病id")
    @PostMapping("/getIndexConfigLists")
    @SysLogger("getIndexConfigLists")
    @Transactional
    public RespDTO<List<IndexConfigAllDTO>> getIndexConfigLists(
            @RequestBody IndexConfigDiseaseIdVO indexConfigDiseaseIdVO) {
        List<IndexConfigAllDTO> data = indexConfigFacade.getIndexConfigLists(indexConfigDiseaseIdVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-慢病指标值关联维护——修改关联[by:wangfeng]", notes = "术语id+conceptId; 疾病id+diseaseId;指标描述+indexDesc; 显示顺序+ orderNo; 备注+remark;")
    @PostMapping("/updateIndexConfigList")
    @SysLogger("updateIndexConfigList")
    @Transactional
    public RespDTO<Boolean> updateIndexConfigList(@RequestBody IndexConfigListVO indexConfigListVO) {
        boolean res = indexConfigFacade.updateIndexConfigList(indexConfigListVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "知识库标准化-慢病指标值关联维护——添加关联[by:wangfeng]", notes = "术语id+conceptId; 疾病id+diseaseId;指标描述+indexDesc; 显示顺序+ orderNo; 备注+remark;")
    @PostMapping("/saveIndexConfigLists")
    @SysLogger("saveIndexConfigLists")
    @Transactional
    public RespDTO<Boolean> saveIndexConfigLists(@RequestBody IndexConfigListVO indexConfigListVO) {
        boolean res = indexConfigFacade.saveIndexConfigList(indexConfigListVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "知识库标准化-删除慢病指标值关联维护[by:wangfeng]", notes = "请输入疾病  id ：diseaseId;请输入状态:state")
    @PostMapping("/cancelIndexConfigAlls")
    @SysLogger("cancelIndexConfigAlls")
    @Transactional
    public RespDTO<Boolean> cancelIndexConfigAlls(@Valid @RequestBody IndexConfigDiseaseIdVO indexConfigDiseaseIdVO) {
        boolean res = indexConfigFacade.cancelIndexConfigAlls(indexConfigDiseaseIdVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "知识库标准化-慢病指标疾病标签查询[by:wangfeng]", notes = "请输 ：diseaseName;")
    @PostMapping("/queryIndexConfigDiseaseNames")
    @SysLogger("queryIndexConfigDiseaseNames")
    @Transactional
    public RespDTO<List<DiseaseInfoDTO>> queryIndexConfigDiseaseNames(@RequestBody DiseaseNameVO diseaseNameVO) {
        List<DiseaseInfoDTO> data = indexConfigFacade.queryIndexConfigDiseaseName(diseaseNameVO);
        return RespDTO.onSuc(data);
    }
}
