package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllLexiconDTO;
import com.diagbot.dto.GetLexiconListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.LexiconFacade;
import com.diagbot.vo.AddLexiconVO;
import com.diagbot.vo.GetAllLexiconVO;
import com.diagbot.vo.GetLexiconListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/lexicon")
@SuppressWarnings("unchecked")
@Api(value = "术语类型维护相关API", tags = { "知识库标准化-术语类型维护相关API" })
public class LexiconController {

    @Autowired
    private LexiconFacade lexiconFacade;

    @ApiOperation(value = "知识库标准化-获取所有术语类型[by:rengb]")
    @PostMapping("/getAllLexicon")
    @SysLogger("getAllLexicon")
    public RespDTO<List<GetAllLexiconDTO>> getAllLexicon(@RequestBody GetAllLexiconVO getAllLexiconVO) {
        return RespDTO.onSuc(lexiconFacade.getAllLexicon(getAllLexiconVO));
    }

    @ApiOperation(value = "获取术语类型列表[by:rengb]")
    @PostMapping("/getLexiconList")
    @SysLogger("getLexiconList")
    @ApiIgnore
    @Deprecated
    public RespDTO<IPage<GetLexiconListDTO>> getLexiconList(@RequestBody GetLexiconListVO getLexiconListVO) {
        return RespDTO.onSuc(lexiconFacade.getLexiconList(getLexiconListVO));
    }

    @ApiOperation(value = "术语类型添加[by:rengb]")
    @PostMapping("/addLexicon")
    @SysLogger("addLexicon")
    @Transactional
    @ApiIgnore
    @Deprecated
    public RespDTO<Boolean> addLexicon(@Valid @RequestBody AddLexiconVO addLexiconVO) {
        return RespDTO.onSuc(lexiconFacade.addLexicon(addLexiconVO));
    }


}
