package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.LisMappingFacade;
import com.diagbot.vo.GetUniqueNameVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.LisMappingPageVO;
import com.diagbot.vo.LisMappingSearchVO;
import com.diagbot.vo.LisMappingVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 化验项目公表映射 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2019-09-19
 */
@RestController
@RequestMapping("/lisMapping")
@Api(value = "化验大小项与公表项对应维护API", tags = { "知识库标准化-化验大小项与公表项对应维护API" })
public class LisMappingController {
    @Autowired
    LisMappingFacade lisMappingFacade;

    /**
     * 分页查询提示信息，带条件查询
     *
     * @param lisMappingPageVO
     * @return
     */
    @ApiOperation(value = "2期-分页查询提示信息，带条件[by:zhaops]",
            notes = "current:页码，必填<br>" +
                    "size:每页显示条数，必填<br>" +
                    "mealName:大项名称<br>" +
                    "itemName:小项名称<br>" +
                    "uniqueName:公表名称<br>")
    @PostMapping("/getLisMappingPage")
    @SysLogger("getLisMappingPage")
    public RespDTO getLisMappingPage(@RequestBody LisMappingPageVO lisMappingPageVO) {
        return RespDTO.onSuc(lisMappingFacade.getLisMappingPage(lisMappingPageVO));
    }

    @ApiOperation(value = "2期-化验公表项映射保存[by:zhaops]",
            notes = "mealName:大项名称，必填<br>" +
                    "itemName:小项名称<br>" +
                    "uniqueName:公表名称，必填<br>")
    @PostMapping("/saveLisMapping")
    @SysLogger("saveLisMapping")
    @Transactional
    public RespDTO<Boolean> saveLisMapping(@RequestBody @Valid LisMappingVO lisMappingVO) {
        return RespDTO.onSuc(lisMappingFacade.saveLisMapping(lisMappingVO));
    }

    @ApiOperation(value = "2期-化验公表项映射是否已存在[by:zhaops]",
            notes = "mealName:大项名称，必填<br>" +
                    "itemName:小项名称<br>" +
                    "uniqueName:公表名称，必填<br>")
    @PostMapping("/hasLisMapping")
    @SysLogger("hasLisMapping")
    @Transactional
    public RespDTO<Boolean> hasLisMapping(@RequestBody @Valid LisMappingVO lisMappingVO) {
        return RespDTO.onSuc(lisMappingFacade.hasLisMapping(lisMappingVO));
    }

    @ApiOperation(value = "2期-删除单条化验公表项映射[by:zhaops]",
            notes = "id:id，必填<br>")
    @PostMapping("/delLisMappingById")
    @SysLogger("delLisMappingById")
    @Transactional
    public RespDTO<Boolean> delLisMappingById(@RequestBody @Valid IdVO idVO) {
        return RespDTO.onSuc(lisMappingFacade.delLisMappingById(idVO));
    }

    @ApiOperation(value = "查找化验公表名[by:zhaops]",
            notes = "mealName:大项名称，必填<br>" +
                    "itemName:小项名称<br>")
    @PostMapping("/getUniqueName")
    @SysLogger("getUniqueName")
    @Transactional
    public RespDTO<String> getUniqueName(@RequestBody @Valid GetUniqueNameVO getUniqueNameVO) {
        return RespDTO.onSuc(lisMappingFacade.getLisUniqueName(getUniqueNameVO));
    }


    @ApiOperation(value = "查找化验公表名[by:zhoutg]",
            notes = "index:前端索引<br>" +
                    "mealName:大项名称<br>" +
                    "itemName:小项名称<br>" +
                    "uniqueName:返回结果<br>")
    @PostMapping("/getUniqueNameWithList")
    @SysLogger("getUniqueNameWithList")
    public RespDTO<String> getUniqueNameWithList(@RequestBody LisMappingSearchVO lisMappingSearchVO) {
        return RespDTO.onSuc(lisMappingFacade.getLisUniqueNameWithList(lisMappingSearchVO.getLisMappingSearchList()));
    }

}
