package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.LisSonContactListDTO;
import com.diagbot.dto.RelationNodeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.LisSonContactFacade;
import com.diagbot.facade.RelationContactFacade;
import com.diagbot.vo.LisSonContactListVO;
import com.diagbot.vo.RelationContactDetailVO;
import com.diagbot.vo.RelationNodeVO;
import com.diagbot.vo.RemoveRelationContactVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/lisSonContact")
@SuppressWarnings("unchecked")
@Api(value = "化验子项维护相关API", tags = { "知识库标准化-化验子项维护相关API" })
public class LisSonContactController {

    @Autowired
    private RelationContactFacade relationContactFacade;
    @Autowired
    private LisSonContactFacade lisSonContactFacade;

    @ApiOperation(value = "知识库标准化-化验子项维护-列表[by:rengb]")
    @PostMapping("/lisSonContactList")
    @SysLogger("lisSonContactList")
    public RespDTO<IPage<LisSonContactListDTO>> lisSonContactList(@RequestBody LisSonContactListVO lisSonContactListVO) {
        return RespDTO.onSuc(lisSonContactFacade.lisSonContactList(lisSonContactListVO));
    }

    @ApiOperation(value = "知识库标准化-化验子项维护-添加或者编辑[by:rengb]")
    @PostMapping("/addRelation")
    @SysLogger("addRelation")
    @Transactional
    public RespDTO<Boolean> addRelation(@Valid @RequestBody RelationNodeVO relationNodeVO) {
        return RespDTO.onSuc(relationContactFacade.addRelation(relationNodeVO));
    }

    @ApiOperation(value = "知识库标准化-化验子项维护-详情[by:rengb]")
    @PostMapping("/relationContactDetail")
    @SysLogger("relationContactDetail")
    public RespDTO<RelationNodeDTO> relationContactDetail(@Valid @RequestBody RelationContactDetailVO relationContactDetailVO) {
        return RespDTO.onSuc(relationContactFacade.relationContactDetail(relationContactDetailVO));
    }

    @ApiOperation(value = "知识库标准化-化验子项维护-删除或者恢复[by:rengb]")
    @PostMapping("/removeRelationContact")
    @SysLogger("removeRelationContact")
    @Transactional
    public RespDTO<Boolean> removeRelationContact(@Valid @RequestBody RemoveRelationContactVO removeRelationContactVO) {
        return RespDTO.onSuc(relationContactFacade.removeRelationContact(removeRelationContactVO));
    }

}
