package com.diagbot.web;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RelationNodeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SingleRelationListDTO;
import com.diagbot.facade.RelationContactFacade;
import com.diagbot.vo.DeleteRelationContactVO;
import com.diagbot.vo.RelationContactDetailVO;
import com.diagbot.vo.RelationNodeVO;
import com.diagbot.vo.SingleRelationListVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/relationContact")
@SuppressWarnings("unchecked")
@Api(value = "医学术语关联维护相关API", tags = { "知识库标准化-医学术语关联维护相关API" })
public class RelationContactController {

    @Autowired
    private RelationContactFacade relationContactFacade;

    @ApiOperation(value = "医学术语关联维护-列表[by:rengb]")
    @PostMapping("/relationContactList")
    @SysLogger("relationContactList")
    public RespDTO<IPage<SingleRelationListDTO>> relationContactList(@Valid @RequestBody SingleRelationListVO singleRelationListVO) {
        return RespDTO.onSuc(relationContactFacade.relationContactList(singleRelationListVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语关联维护-添加或者编辑[by:rengb]")
    @PostMapping("/addRelation")
    @SysLogger("addRelation")
    @Transactional
    public RespDTO<Boolean> addRelation(@Valid @RequestBody RelationNodeVO relationNodeVO) {
        return RespDTO.onSuc(relationContactFacade.addRelation(relationNodeVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语关联维护-详情[by:rengb]")
    @PostMapping("/relationContactDetail")
    @SysLogger("relationContactDetail")
    public RespDTO<RelationNodeDTO> relationContactDetail(@Valid @RequestBody RelationContactDetailVO relationContactDetailVO) {
        return RespDTO.onSuc(relationContactFacade.relationContactDetail(relationContactDetailVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语关联维护-删除或者恢复[by:rengb]")
    @PostMapping("/removeRelationContact")
    @SysLogger("removeRelationContact")
    @Transactional
    public RespDTO<Boolean> removeRelationContact(@Valid @RequestBody DeleteRelationContactVO deleteRelationContactVO) {
        return RespDTO.onSuc(relationContactFacade.deleteRelationContact(deleteRelationContactVO));
    }

}
