package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetRelationInfoDetailDTO;
import com.diagbot.dto.GetRelationInfoListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.RelationFacade;
import com.diagbot.vo.AddRelationInfoVO;
import com.diagbot.vo.GetRelationInfoDetailVO;
import com.diagbot.vo.GetRelationInfoListVO;
import com.diagbot.vo.RemoveRelationInfoVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/relation")
@SuppressWarnings("unchecked")
@Api(value = "医学术语关系维护相关API", tags = { "知识库标准化-医学术语关系维护相关API" })
public class RelationController {

    @Autowired
    private RelationFacade relationFacade;

    @ApiOperation(value = "知识库标准化-获取医学术语关系列表[by:rengb]")
    @PostMapping("/getRelationInfoList")
    @SysLogger("getRelationInfoList")
    public RespDTO<IPage<GetRelationInfoListDTO>> getRelationInfoList(@RequestBody GetRelationInfoListVO getRelationInfoListVO) {
        return RespDTO.onSuc(relationFacade.getRelationInfoList(getRelationInfoListVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语关系删除或者恢复[by:rengb]")
    @PostMapping("/removeRelationInfo")
    @SysLogger("removeRelationInfo")
    @Transactional
    public RespDTO<Boolean> removeRelationInfo(@Valid @RequestBody RemoveRelationInfoVO removeRelationInfoVO) {
        return RespDTO.onSuc(relationFacade.removeRelationInfo(removeRelationInfoVO));
    }

    @ApiOperation(value = "知识库标准化-获取医学术语关系详情[by:rengb]")
    @PostMapping("/getRelationInfoDetail")
    @SysLogger("getRelationInfoDetail")
    @ApiIgnore
    public RespDTO<GetRelationInfoDetailDTO> getRelationInfoDetail(@Valid @RequestBody GetRelationInfoDetailVO getRelationInfoDetailVO) {
        return RespDTO.onSuc(relationFacade.getRelationInfoDetail(getRelationInfoDetailVO));
    }

    @ApiOperation(value = "知识库标准化-医学术语关系添加或者编辑[by:rengb]")
    @PostMapping("/addRelationInfo")
    @SysLogger("addRelationInfo")
    @Transactional
    public RespDTO<Boolean> addRelationInfo(@Valid @RequestBody AddRelationInfoVO addRelationInfoVO) {
        return RespDTO.onSuc(relationFacade.addRelationInfo(addRelationInfoVO));
    }

    @ApiOperation(value = "医学术语关系excel文件导入[by:rengb]")
    @PostMapping("/relationInfoExcelIm")
    @Transactional
    public RespDTO<Boolean> relationInfoExcelIm(@RequestParam("uploadfile") MultipartFile file, HttpServletRequest request) {
        return RespDTO.onSuc(relationFacade.relationInfoExcelIm(file));
    }

}
