package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.ScaleContentDTO;
import com.diagbot.dto.ScaleContentIndexDTO;
import com.diagbot.dto.ScaleContentPageDTO;
import com.diagbot.facade.ScaleContentFacade;
import com.diagbot.vo.DeleteScaleVO;
import com.diagbot.vo.ScaleContentIndexVO;
import com.diagbot.vo.ScaleContentPageVO;
import com.diagbot.vo.ScaleContentParamVO;
import com.diagbot.vo.ScaleContentSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 量表内容表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2019-04-08
 */
@RestController
@RequestMapping("/scaleContent")
@Api(value = "量表内容相关API", tags = { "知识库标准化-量表内容相关API" })
public class ScaleContentController {

    @Autowired
    ScaleContentFacade scaleContentFacade;

    @ApiOperation(value = "知识库标准化-量表内容新增或更新[by:zhoutg]")
    @PostMapping("/insertOrUpdate")
    @SysLogger("insertOrUpdate")
    @Transactional
    public RespDTO<Boolean> insertOrUpdate(@RequestBody ScaleContentSaveVO scaleContentSaveVO) {
        scaleContentFacade.insertOrUpdate(scaleContentSaveVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "知识库标准化-量表内容分页[by:zhoutg]", notes = "")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<ScaleContentPageDTO>> page(@RequestBody ScaleContentPageVO questionPageVO) {
        IPage<ScaleContentPageDTO> data = scaleContentFacade.getListFac(questionPageVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "知识库标准化-量表内容检索[by:zhoutg]", notes = "")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<ScaleContentIndexDTO>> index(@Valid @RequestBody ScaleContentIndexVO scaleContentIndexVO) {
        List<ScaleContentIndexDTO> data = scaleContentFacade.indexFac(scaleContentIndexVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "知识库标准化-根据量表概念id获取内容[by:zhoutg]", notes = "")
    @PostMapping("/getByConceptId")
    @SysLogger("getByConceptId")
    public RespDTO<List<ScaleContentDTO>> getByConceptId(@Valid @RequestBody ScaleContentParamVO scaleContentParamVO) {
        List<ScaleContentDTO> data = scaleContentFacade.getByParamFac(scaleContentParamVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-量表内容删除|恢复[by:zhoutg]",
            notes = "ids：量表概念id<br>" +
                    "status: 删除传值'Y'<br>" +
                    "        恢复传值'N'")
    @PostMapping("/delete")
    @SysLogger("delete")
    public RespDTO<Boolean> delete(@Valid @RequestBody DeleteScaleVO deleteScaleVO) {
        scaleContentFacade.delete(deleteScaleVO);
        return RespDTO.onSuc(true);
    }
}
