package com.diagbot.aop;

import com.diagbot.enums.ProductTypeEnum;
import com.diagbot.biz.log.entity.BiRecord;
import com.diagbot.rabbit.MySender;
import com.diagbot.util.AopUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

/**
 * @Description: BI日志拦截切面
 * @author: gaodm
 * @time: 2019/11/12 17:26
 */
@Aspect
@Component
@ConditionalOnProperty(prefix = "bilog", value = { "enable" }, havingValue = "true")
public class BiLoggerAspect {
    @Autowired
    private MySender mySender;

    @Pointcut("@annotation(com.diagbot.annotation.BiLogger)")
    public void biPointCut() {
    }

    @Around(value = "biPointCut()")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
        BiRecord biRecord = new BiRecord();
        Object object = AopUtil.biLoggerAspect(biRecord, joinPoint, ProductTypeEnum.LTAPI.getKey());
        if (null == biRecord) {
            return object;
        }
        //保存BI日志
        mySender.outputBiLogSend(biRecord);
        return object;
    }

    @AfterThrowing(pointcut = "biPointCut()", throwing = "ex")
    public void saveBiRecordThrow(JoinPoint joinPoint, Throwable ex) {
        BiRecord biRecord = AopUtil.biLoggerAspectThrow(joinPoint, ex, ProductTypeEnum.LTAPI.getKey());
        if (null == biRecord) {
            return;
        }
        //保存BI日志
        mySender.outputBiLogSend(biRecord);
    }
}

