package com.diagbot.client;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.ScaleContent;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ConnectIntroduceVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: 调用知识库服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {
    /**
     * 推理
     *
     * @param searchVO
     * @return
     */
    @PostMapping("/push/pushInner")
    RespDTO<PushDTO> pushInner(@RequestBody SearchVO searchVO);

    /**
     * 计算接口-量表/公式
     *
     * @param calculateData
     * @return
     */
    @PostMapping(value = "/clinicaldata/scaleCalc")
    RespDTO<Map<String, Object>> scaleCalc(@RequestBody CalculateData calculateData);

    /**
     * 获取量表接口
     *
     * @param searchVO
     * @return
     */
    @PostMapping(value = "/clinicaldata/getScale")
    RespDTO<List<ScaleContent>> getScale(@RequestBody SearchVO searchVO);

    /**
     * 获取提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    @PostMapping(value = "/conceptDetail/getConceptDetail")
    RespDTO<ConceptIntroduceDTO> getConceptDetail(@Valid @RequestBody ConceptIntroduceVO conceptIntroduceVO);

    /**
     * 获取提示信息-对接
     *
     * @param connectIntroduceVO
     * @return
     */
    @PostMapping(value = "/conceptDetail/getConnectConceptDetails")
    RespDTO<List<ConceptIntroduceDTO>> getConnectConceptDetails(@Valid @RequestBody ConnectIntroduceVO connectIntroduceVO);
}