package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.BaseNodeRSDTO;
import com.diagbot.dto.CategorieDTO;
import com.diagbot.dto.Categories;
import com.diagbot.dto.GNodeDTO;
import com.diagbot.dto.GraDTO;
import com.diagbot.dto.GraphDTO;
import com.diagbot.dto.GraphLabelDTO;
import com.diagbot.dto.Link;
import com.diagbot.dto.LinkDTO;
import com.diagbot.dto.NextNodeDTO;
import com.diagbot.dto.Node;
import com.diagbot.dto.Tree;
import com.diagbot.dto.TreeDTO;
import com.diagbot.dto.TreeNodeDTO;
import com.diagbot.dto.Tu;
import com.diagbot.entity.PresetInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.KgServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.KgQueryVO;
import com.diagbot.vo.KgTreeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 用户日志业务层
 * @author: gaodm
 * @time: 2018/8/6 9:11
 */
@Component
public class KgFacade extends KgServiceImpl {
    @Autowired
    private PresetInfoFacade presetInfoFacade;

    public GraphLabelDTO getGraphFac(KgQueryVO kgQueryVO) {
        GraphLabelDTO graphLabelDTO = new GraphLabelDTO();
        List<CategorieDTO> categories = new ArrayList<>();
        List<GNodeDTO> node = new ArrayList<>();
        List<LinkDTO> links = new ArrayList<>();
        List<GraphDTO> res = this.getGraph(kgQueryVO);
        if (ListUtil.isEmpty(res)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS);
        } else {
            Integer nodeId = 0;
            categories.add(new CategorieDTO("中心词"));
            categories.add(new CategorieDTO("关系"));
            Map<String, Integer> cMap = new HashMap<>();
            cMap.put("中心词", 0);
            cMap.put("关系", 1);

            GraphDTO graphDTO = res.get(0);
            GNodeDTO gNodeDTO
                    = new GNodeDTO(graphDTO.getLabel(), 0, graphDTO.getName(), nodeId, "circle", 50);
            nodeId++;
            node.add(gNodeDTO);
            if (ListUtil.isNotEmpty(graphDTO.getENodeRSDTOS())) {
                Integer rsId = 2;
                for (BaseNodeRSDTO baseNodeRSDTO : graphDTO.getENodeRSDTOS()) {
                    if (null == cMap.get(baseNodeRSDTO.getRType())) {
                        cMap.put(baseNodeRSDTO.getRType(), rsId);
                        categories.add(new CategorieDTO(baseNodeRSDTO.getRType()));
                        rsId++;
                    }
                    GNodeDTO nNodeDTO
                            = new GNodeDTO("", 1, "", nodeId, "diamond", 10);
                    node.add(nNodeDTO);
                    links.add(new LinkDTO(gNodeDTO.getName(), nNodeDTO.getName(), baseNodeRSDTO.getRType()));
                    nodeId++;
                    if (ListUtil.isNotEmpty(baseNodeRSDTO.getENodeDTOS())) {
                        for (NextNodeDTO baseNodeDTO : baseNodeRSDTO.getENodeDTOS()) {
                            String symbol = "diamond";
                            if (baseNodeDTO.getPCount() > 0) {
                                symbol = "circle";
                            }
                            GNodeDTO eNodeDTO
                                    = new GNodeDTO(baseNodeDTO.getLabel(), cMap.get(baseNodeRSDTO.getRType()),
                                    baseNodeDTO.getName(), nodeId, symbol, 28);
                            nodeId++;
                            node.add(eNodeDTO);
                            links.add(new LinkDTO(nNodeDTO.getName(), eNodeDTO.getName(), ""));
                        }
                    }
                }
            }
            graphLabelDTO.setCategories(categories);
            graphLabelDTO.setNode(node);
            graphLabelDTO.setLinks(links);
        }

        return graphLabelDTO;
    }

    //    private String random() {
    //        //先随机产生一个下标再获取元素
    //        String random = "";
    //        String[] doc = { "circle", "diamond" };
    //        int index = (int) (Math.random() * doc.length);
    //        random = doc[index];
    //        return random;
    //    }

    public TreeDTO getTreeFac(KgTreeVO kgTreeVO) {
        TreeDTO treeDTO = new TreeDTO();
        if ((kgTreeVO.getType() == 1
                && (kgTreeVO.getSubType() == 1 || kgTreeVO.getSubType() == 2 || kgTreeVO.getSubType() == 3))
                || (kgTreeVO.getType() == 2 && (kgTreeVO.getSubType() == 0))
                || (kgTreeVO.getType() == 3 && (kgTreeVO.getSubType() == 0))
                || (kgTreeVO.getType() == 4 && (kgTreeVO.getSubType() == 0))
                || (kgTreeVO.getType() == 5 && (kgTreeVO.getSubType() == 0))
                || (kgTreeVO.getType() == 6 && (kgTreeVO.getSubType() == 0))) {
            // KgQueryVO kgQueryVO = new KgQueryVO();
            // kgQueryVO.setLabelName("诊断依据");
            // kgQueryVO.setInputStr("诊断依据");
            // String tree = this.getTree(kgQueryVO);
            PresetInfo presetInfo = presetInfoFacade.getOne(new QueryWrapper<PresetInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("type", (kgTreeVO.getType() * 10 + kgTreeVO.getSubType())), false);
            String tree = "";
            if (null != presetInfo){
                tree = presetInfo.getContent();
            }
            if (StringUtil.isBlank(tree)) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS);
            } else {
                treeDTO = getTreeDTO(tree);
            }
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR);
        }
        return treeDTO;
    }

    private TreeDTO getTreeDTO(String tree) {
        TreeDTO treeDTO = new TreeDTO();
        List<TreeNodeDTO> nodes = new ArrayList<>();
        Tree treeObj = FastJsonUtils.getJsonToBean(tree, Tree.class);
        if (null != treeObj) {
            Map<String, Integer> treeMap = new HashMap<>();
            treeMap.put("根节点", 0);
            addTree(nodes, treeObj, treeMap, "根节点");
        }

        treeDTO.setNodes(nodes);
        return treeDTO;
    }

    private void addTree(List<TreeNodeDTO> nodes, Tree treeObj, Map<String, Integer> treeMap, String pName) {
        if (null == treeMap.get(treeObj.getName())) {
            treeMap.put(treeObj.getName(), treeMap.size());
        }

        TreeNodeDTO node = new TreeNodeDTO("/images/icon.png",
                "/images/iconOpen.png",
                "/images/iconClose.png",
                treeMap.get(treeObj.getName()), treeObj.getName(), treeMap.get(pName));
        nodes.add(node);
        if (ListUtil.isNotEmpty(treeObj.getSNode())) {
            for (Tree tree : treeObj.getSNode()) {
                addTree(nodes, tree, treeMap, treeObj.getName());
            }
        } else {
            node.setIcon("/images/node.png");
            node.setIconOpen("");
            node.setIconClose("");
        }
    }

    public GraDTO getTuFac(KgQueryVO kgQueryVO) {
        GraDTO graDTO = new GraDTO();
        List<Tu> tu = this.getTu(kgQueryVO);
        Map<String, List<Tu>> map = EntityUtil.makeEntityListMap(tu, "rel");

        List<Categories> categoriesList = new ArrayList<>();
        List<Node> nodeList = new ArrayList<>();
        List<Link> linksList = new ArrayList<>();

        // 添加第一个节点：中心节点
        Node node = new Node(0, kgQueryVO.getInputStr(), 0L, 50);
        nodeList.add(node);

        // 设置categories
        Map<String, Integer> relMap = new LinkedHashMap<>();
        int i = 0; // 设置category
        Categories c1 = new Categories("中心词");
        categoriesList.add(c1);
        relMap.put("中心词", i++);
        Categories c2 = new Categories("关系");
        categoriesList.add(c2);
        relMap.put("关系", i++);
        Long j = 1L; // 设置节点编号
        for (String key : map.keySet()) {
            Categories bean = new Categories(key);
            categoriesList.add(bean);
            relMap.put(key, i);

            // 添加关系节点
            Node nodeRel = new Node(1, "", j, 10);
            nodeRel.setSymbol("diamond");
            nodeList.add(nodeRel);

            // 添加中心节点与关系的关联
            Link linkRel = new Link(0L, j, key);
            linksList.add(linkRel);

            List<Tu> inner = map.get(key);
            for (Tu t : inner) {
                // 添加关系下的节点
                Node nodeLabel = new Node(i, t.getSubName(), ++j, 28);
                if (t.getSubNextCount() == 0) {
                    nodeLabel.setSymbol("rect");
                }
                nodeLabel.setType(t.getSubLabel());
                nodeList.add(nodeLabel);

                // 添加关系和节点的关联
                Link linkLabel = new Link(linkRel.getTarget(), j, "");
                linksList.add(linkLabel);
            }
            i++;
            j++;
        }
        graDTO.setCategoriesList(categoriesList);
        graDTO.setNodeList(nodeList);
        graDTO.setLinksList(linksList);

        return graDTO;
    }


    private static final String jsonStr = "{\n" +
            "\t\"name\": \"疾病\",\n" +
            "\t\"sNode\": [\n" +
            "\t\t{\n" +
            "\t\t\t\"name\": \"A00-B99-某些传染病和寄生虫病\",\n" +
            "\t\t\t\"sNode\": [\n" +
            "\t\t\t\t{\n" +
            "\t\t\t\t\t\"name\": \"A00\",\n" +
            "\t\t\t\t\t\"sNode\": [\n" +
            "\t\t\t\t\t\t{\n" +
            "\t\t\t\t\t\t\t\"name\": \"A00.000 霍乱，由于O1群霍乱弧菌，霍乱生物型所致\",\n" +
            "\t\t\t\t\t\t\t\"sNode\": [\n" +
            "\t\t\t\t\t\t\t]\n" +
            "\t\t\t\t\t\t},\n" +
            "\t\t\t\t\t\t{\n" +
            "\t\t\t\t\t\t\t\"name\": \"A00.000x001 古典生物型霍乱\",\n" +
            "\t\t\t\t\t\t\t\"sNode\": [\n" +
            "\t\t\t\t\t\t\t]\n" +
            "\t\t\t\t\t\t}\n" +
            "\t\t\t\t\t]\n" +
            "\t\t\t\t}\n" +
            "\t\t\t]\n" +
            "\t\t},\n" +
            "\t\t{\n" +
            "\t\t\t\"name\": \"A22-B22-某些传染病和寄生虫病\",\n" +
            "\t\t\t\"sNode\": [\n" +
            "\t\t\t\t{\n" +
            "\t\t\t\t\t\"name\": \"A22\",\n" +
            "\t\t\t\t\t\"sNode\": [\n" +
            "\t\t\t\t\t\t{\n" +
            "\t\t\t\t\t\t\t\"name\": \"胆囊炎\",\n" +
            "\t\t\t\t\t\t\t\"sNode\": [\n" +
            "\t\t\t\t\t\t\t]\n" +
            "\t\t\t\t\t\t}\n" +
            "\t\t\t\t\t]\n" +
            "\t\t\t\t}\n" +
            "\t\t\t]\n" +
            "\t\t}\n" +
            "\t]\n" +
            "}";

}
