package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.GetAllByHospitalDTO;
import com.diagbot.dto.GetAllByHospitalDetialDTO;
import com.diagbot.dto.QcCasesEntryDTO;
import com.diagbot.entity.CasesEntryHospital;
import com.diagbot.entity.QcCases;
import com.diagbot.entity.QcCasesEntry;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.CasesEntryHospitalServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.GetAllByHospitalVO;
import com.diagbot.vo.InsertByHospitalVO;
import com.diagbot.vo.UpdateByHospitalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: wangyu
 * @time: 2020/3/31 11:13
 */
@Component
public class CasesEntryHospitalFacade extends CasesEntryHospitalServiceImpl {
    @Autowired
    @Qualifier("casesEntryHospitalServiceImpl")
    private CasesEntryHospitalServiceImpl casesEntryHospitalService;
    @Autowired
    private QcCacesEntryFacade qcCacesEntryFacade;
    @Autowired
    private QcCasesFacade qcCasesFacade;

    /**
     * 获取指定医院病例条目
     *
     * @param getAllByHospitalVO
     * @return
     */
    public List<GetAllByHospitalDTO> getAllByHospital(GetAllByHospitalVO getAllByHospitalVO){
        //获取病例条目关系
        QueryWrapper<CasesEntryHospital> casesEntryHospitalQueryWrapper = new QueryWrapper<>();
        casesEntryHospitalQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id",getAllByHospitalVO.getHospitalId());
        List<CasesEntryHospital> casesEntryHospitals = this.list(casesEntryHospitalQueryWrapper);
        Map<Long,CasesEntryHospital> casesEntryHospitalMap = casesEntryHospitals.stream().collect(Collectors.toMap(CasesEntryHospital::getCasesEntryId,casesEntryHospital -> casesEntryHospital));
        List<Long> casesEntryIds = new ArrayList<>();
        casesEntryIds = casesEntryHospitals.stream().map(CasesEntryHospital::getCasesEntryId).collect(Collectors.toList());
        //获取病例条目
        List<QcCasesEntryDTO> qcCasesEntries = new ArrayList<>();
        if(ListUtil.isNotEmpty(casesEntryIds)){
            qcCasesEntries = new ArrayList<QcCasesEntryDTO>(qcCacesEntryFacade.getByCasesEntryId(casesEntryIds));
        }
        //获取病例
        QueryWrapper<QcCases> qcCasesQueryWrapper = new QueryWrapper<>();
        qcCasesQueryWrapper.eq("is_deleted",IsDeleteEnum.N.getKey());
        List<QcCases> qcCases = qcCasesFacade.list(qcCasesQueryWrapper);
        List<GetAllByHospitalDTO> getAllByHospitalDTOS = new ArrayList<>();
        GetAllByHospitalDTO getAllByHospitalDTO = new GetAllByHospitalDTO();
        GetAllByHospitalDetialDTO getAllByHospitalDetialDTO = new GetAllByHospitalDetialDTO();
        List<GetAllByHospitalDetialDTO> getAllByHospitalDetialDTOS = new ArrayList<>();
        for (QcCases qcCases_new : qcCases) {
            getAllByHospitalDTO = new GetAllByHospitalDTO();
            getAllByHospitalDTO.setCasesId(qcCases_new.getId());
            getAllByHospitalDTO.setCasesName(qcCases_new.getName());
            getAllByHospitalDTO.setRemark(qcCases_new.getRemark());
            getAllByHospitalDetialDTOS = new ArrayList<>();
            for (QcCasesEntryDTO qcCasesEntry : qcCasesEntries) {
                if(getAllByHospitalDTO.getCasesName().equals(qcCasesEntry.getCasesName())){
                    getAllByHospitalDetialDTO = new GetAllByHospitalDetialDTO();
                    getAllByHospitalDetialDTO.setCasesEntryId(qcCasesEntry.getId());
                    getAllByHospitalDetialDTO.setCasesName(qcCasesEntry.getCasesName());
                    getAllByHospitalDetialDTO.setCode(qcCasesEntry.getCode());
                    getAllByHospitalDetialDTO.setModeId(qcCasesEntry.getModeId());
                    getAllByHospitalDetialDTO.setModeName(qcCasesEntry.getModeName());
                    getAllByHospitalDetialDTO.setMsg(casesEntryHospitalMap.get(qcCasesEntry.getId()).getMsg());
                    getAllByHospitalDetialDTO.setIsUsed(casesEntryHospitalMap.get(qcCasesEntry.getId()).getIsUsed());
                    getAllByHospitalDetialDTO.setIsUsedRun(casesEntryHospitalMap.get(qcCasesEntry.getId()).getIsUsedRun());
                    getAllByHospitalDetialDTO.setIsReject(casesEntryHospitalMap.get(qcCasesEntry.getId()).getIsReject());
                    getAllByHospitalDetialDTO.setScore(casesEntryHospitalMap.get(qcCasesEntry.getId()).getScore());
                    getAllByHospitalDetialDTO.setPrecond(qcCasesEntry.getPrecond());
                    getAllByHospitalDetialDTO.setScoreRun(casesEntryHospitalMap.get(qcCasesEntry.getId()).getScoreRun());
                    getAllByHospitalDetialDTO.setName(qcCasesEntry.getName());
                    getAllByHospitalDetialDTOS.add(getAllByHospitalDetialDTO);
                }
            }
            getAllByHospitalDTO.setGetAllByHospitalDetialDTOS(getAllByHospitalDetialDTOS);
            if(ListUtil.isNotEmpty(getAllByHospitalDTO.getGetAllByHospitalDetialDTOS())){
                getAllByHospitalDTOS.add(getAllByHospitalDTO);
            }
        }

        return getAllByHospitalDTOS;
    }

    /**
     * 修改指定医院病例条目
     *
     * @param updateByHospitalVO
     * @return
     */
    public Boolean updateByHospital(UpdateByHospitalVO updateByHospitalVO){
        QueryWrapper<CasesEntryHospital> casesEntryHospitalQueryWrapper = new QueryWrapper<>();
        casesEntryHospitalQueryWrapper
                .eq("is_deleted",IsDeleteEnum.N.getKey())
                .in("hospital_id",updateByHospitalVO.getHospitalIds());

        boolean res = this.remove(casesEntryHospitalQueryWrapper);
        //添加新的
        if(res){
           res = casesEntryHospitalService.saveAll(updateByHospitalVO.getCasesEntryHospitals());
        }

        return res;
    }

    /**
     * 新增指定医院条目
     * @param insertByHospitalVO
     * @return
     */
    public Boolean insertByHospital(InsertByHospitalVO insertByHospitalVO){

        QueryWrapper<QcCasesEntry> qcCasesEntryWrapper = new QueryWrapper<>();
        qcCasesEntryWrapper.eq("is_deleted",IsDeleteEnum.N.getKey())
                .ne("id",insertByHospitalVO.getId()==null? -1:insertByHospitalVO.getId())
                .eq("name",insertByHospitalVO.getName())
                .eq("cases_id",insertByHospitalVO.getCasesId());
        int sum = qcCacesEntryFacade.count(qcCasesEntryWrapper);
        if (sum != 0) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该条目名称存在");
        }
        QueryWrapper<QcCasesEntry> qcCasesEntryQueryWrapper = new QueryWrapper<>();
        qcCasesEntryQueryWrapper.eq("id",insertByHospitalVO.getId());
        List<QcCasesEntry> qcCasesEntries = qcCacesEntryFacade.list(qcCasesEntryQueryWrapper);
        QcCasesEntry qcCasesEntry = new QcCasesEntry();
        Date date = DateUtil.now();
        if(ListUtil.isNotEmpty(qcCasesEntries)){
            qcCasesEntry = new QcCasesEntry();
            BeanUtil.copyProperties(insertByHospitalVO,qcCasesEntry);
            qcCasesEntry.setGmtCreate(date);
            qcCasesEntry.setGmtModified(date);
            qcCacesEntryFacade.updateById(qcCasesEntry);
            //修改时添加明细前先删除
            List<Long> hospitalIds = new ArrayList<>();
            for (CasesEntryHospital casesEntryHospital : insertByHospitalVO.getCasesEntryHospitals()) {
                if(!hospitalIds.contains(casesEntryHospital.getHospitalId())){
                    hospitalIds.add(casesEntryHospital.getHospitalId());
                }
            }
            QueryWrapper<CasesEntryHospital> casesEntryHospitalQueryWrapper = new QueryWrapper<>();
            casesEntryHospitalQueryWrapper.eq("cases_entry_id",qcCasesEntry.getId());
            casesEntryHospitalService.remove(casesEntryHospitalQueryWrapper);
        }else {
            qcCasesEntry = new QcCasesEntry();
            BeanUtil.copyProperties(insertByHospitalVO,qcCasesEntry);
            qcCasesEntry.setGmtCreate(date);
            qcCasesEntry.setGmtModified(date);
            boolean res= qcCacesEntryFacade.save(qcCasesEntry);
            //先保存， 在更新Code
            if(res){
                Long id = qcCasesEntry.getId();
                String code = qcCasesEntry.getCode()+id;
                UpdateWrapper<QcCasesEntry> qcCasesEntryQueryUpdate = new UpdateWrapper<>();
                qcCasesEntryQueryUpdate
                        .eq("is_deleted",IsDeleteEnum.N.getKey())
                        .eq("id",id)
                        .set("code",code);
                qcCacesEntryFacade.update(qcCasesEntryQueryUpdate);
            }
        }
        //添加分值明细
        List<CasesEntryHospital> casesEntryHospitals = new ArrayList<>();
        CasesEntryHospital casesEntryHospital = new CasesEntryHospital();
        for (CasesEntryHospital casesEntryHospital_new : insertByHospitalVO.getCasesEntryHospitals()) {
            casesEntryHospital = new CasesEntryHospital();
            BeanUtil.copyProperties(casesEntryHospital_new,casesEntryHospital);
            casesEntryHospital.setCasesEntryId(qcCasesEntry.getId());
            casesEntryHospital.setGmtCreate(date);
            casesEntryHospital.setGmtModified(date);
            casesEntryHospitals.add(casesEntryHospital);
        }
        casesEntryHospitalService.saveBatch(casesEntryHospitals);
        return true;
    }
}
