package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.MedModuleMapping;
import com.diagbot.entity.QcModuleDetail;
import com.diagbot.service.MedModuleMappingService;
import com.diagbot.service.QcModuleDetailService;
import com.diagbot.service.impl.QcModuleDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddModuleDetailVO;
import com.diagbot.vo.AddModuleInfoVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhoutg
 * @time: 2018/11/23 11:37
 */
@Component
public class QcModuleDetailFacade extends QcModuleDetailServiceImpl {

    @Autowired
    QcModuleInfoFacade moduleInfoFacade;
    @Autowired
    @Qualifier("qcModuleDetailServiceImpl")
    QcModuleDetailService qcModuleDetailService;
    @Autowired
    @Qualifier("medModuleMappingServiceImpl")
    MedModuleMappingService medModuleMappingService;

    /**
     * 根据questionId删除明细
     *
     * @param map
     * @return
     */
    public void deleteByQuestionIdFac(Map map) {
        this.deleteByQuestionIds(map);
    }


    /**
     * 根据moduleId删除明细
     *
     * @param map
     * @return
     */
    public void deleteByModuleIdFac(Map map) {
        this.deleteByModuleIds(map);
    }

    /**
     * 添加模板明细
     *
     * @param addModuleInfoVO
     * @return
     */
    public Boolean saveOrUpdateInfos(AddModuleInfoVO addModuleInfoVO, Long moduleId) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
       //如果该模板添加过明细，先删除
        if (moduleId != null) {
            this.remove(new QueryWrapper<QcModuleDetail>()
                    .eq("module_id", moduleId)
            );
        }

        List<QcModuleDetail> moduleDetailList = new ArrayList<>();
        int i = 1;
        for (AddModuleDetailVO detailVO : addModuleInfoVO.getModuleDetail()) {
            QcModuleDetail qcModuleDetail = new QcModuleDetail();
            BeanUtil.copyProperties(detailVO, qcModuleDetail);
            qcModuleDetail.setOrderNo(i++);
            qcModuleDetail.setModuleId(moduleId);
            qcModuleDetail.setGmtCreate(now);
            qcModuleDetail.setGmtModified(now);
            qcModuleDetail.setCreator(userId);
            qcModuleDetail.setModifier(userId);
            moduleDetailList.add(qcModuleDetail);
        }
        //添加模板明细
        qcModuleDetailService.saveBatch(moduleDetailList);

        // 删除模板与文书的映射关系
        medModuleMappingService.remove(new UpdateWrapper<MedModuleMapping>()
                .eq("module_id", moduleId)
        );

        // 添加模板与文书的映射关系
        if (ListUtil.isNotEmpty(addModuleInfoVO.getRecordIdList())) {
            List<MedModuleMapping> medModuleMappingList = new ArrayList<>();
            for (Long recordId : addModuleInfoVO.getRecordIdList()) {
                MedModuleMapping medModuleMapping = new MedModuleMapping();
                medModuleMapping.setModuleId(moduleId);
                medModuleMapping.setRecordId(recordId);
                medModuleMapping.setHospitalId(addModuleInfoVO.getHospitalId());
                medModuleMapping.setGmtCreate(now);
                medModuleMapping.setGmtModified(now);
                medModuleMapping.setCreator(userId);
                medModuleMapping.setModifier(userId);
                medModuleMappingList.add(medModuleMapping);
            }
            medModuleMappingService.saveBatch(medModuleMappingList);
        }
        return true;
    }
}
