package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.*;
import com.diagbot.entity.MedModuleMapping;
import com.diagbot.entity.QcModuleDetail;
import com.diagbot.entity.QcModuleInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.QcModuleInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhoutg
 * @time: 2018/11/23 11:37
 */
@Component
public class QcModuleInfoFacade extends QcModuleInfoServiceImpl {

    @Autowired
    QcModuleDetailFacade qcModuleDetailFacade;
    @Autowired
    UserServiceClient userServiceClient;
    @Autowired
    DictionaryFacade dictionaryFacade;
    @Autowired
    QcQuestionFacade qcQuestionFacade;
    @Autowired
    MedModuleMappingFacade medModuleMappingFacade;
    @Autowired
    MedRecordAnalyzeFacade medRecordAnalyzeFacade;
    @Autowired
    CacheFacade cacheFacade;

    /**
     * 根据id删除标签模板
     *
     * @param ids
     * @return
     */
    public Boolean deleteByIdsFac(String ids) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date date = DateUtil.now();
        Map paramMap = new HashMap<>();
        List<String> idList = Arrays.asList(ids.split(","));
        paramMap.put("delete", IsDeleteEnum.Y.getKey());
        paramMap.put("ids", idList);
        paramMap.put("gmtModified", date);
        paramMap.put("modifier", userId);
        // 删除模板
        this.deleteByIds(paramMap);
        // 删除明细表
        if (ListUtil.isNotEmpty(idList)) {
            qcModuleDetailFacade.remove(new QueryWrapper<QcModuleDetail>()
                    .in("module_id", idList)
            );
        }
        // 删除模板与文书的映射关系
        if (ListUtil.isNotEmpty(idList)) {
            medModuleMappingFacade.remove(new QueryWrapper<MedModuleMapping>()
                    .in("module_id", idList)
            );
        }
        // 清除缓存
        cacheFacade.clear();
        return true;
    }

    /**
     * 模板列表首页
     *
     * @param defaultModuleVO
     * @return
     */
    public Object saveDefaultModuleInfo(DefaultModuleVO defaultModuleVO) {
        Boolean flag = false;
        //用户选择添加默认(标准)模块
        if (1 == defaultModuleVO.getDefaultModule()) {
            //用户同意更换默认(标准)模版   校验标准模版是否存在之前，提示信息用户操作默认为0 当用户点击提示信息确定则1
            if (1 == defaultModuleVO.getDefineType()) {
                //此时不会给出提示
                flag = true;
//            原有的标准模版设置为0 体现互斥行性
                UpdateWrapper<QcModuleInfo> serviceInfoWrapper = new UpdateWrapper<>();
                serviceInfoWrapper
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("mode_id", defaultModuleVO.getModeId())
                        .eq("hospital_id", defaultModuleVO.getHospitalId())
                        .eq("default_module", 1)
                        .set("default_module", "0");
                this.update(new QcModuleInfo(), serviceInfoWrapper);
            }
            //校验标准模版是否存在
            int sum = this.count(new QueryWrapper<QcModuleInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("mode_id", defaultModuleVO.getModeId())
                    .eq("hospital_id", defaultModuleVO.getHospitalId())
                    .eq("default_module", 1));
            if (sum > 0) {
                //发现标准模版已经存在，默认给出提示
                if (flag == false) {
                    return null;
                }
            }
            Boolean bac = false;
            //更新模版状态
            UpdateWrapper<QcModuleInfo> serviceInfoWrapper = new UpdateWrapper<>();
            serviceInfoWrapper
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("mode_id", defaultModuleVO.getModeId())
                    .eq("hospital_id", defaultModuleVO.getHospitalId())
                    .eq("name", defaultModuleVO.getName())
                    .set("default_module", "1");
            bac = this.update(new QcModuleInfo(), serviceInfoWrapper);
            return bac;

        } else {
            //用户直接取消,无视互斥
            UpdateWrapper<QcModuleInfo> serviceWrapper = new UpdateWrapper<>();
            serviceWrapper
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("mode_id", defaultModuleVO.getModeId())
                    .eq("hospital_id", defaultModuleVO.getHospitalId())
                    .eq("name", defaultModuleVO.getName())
                    .eq("default_module", 1)
                    .set("default_module", defaultModuleVO.getDefaultModule());
            this.update(new QcModuleInfo(), serviceWrapper);

        }
        return true;
    }

    /**
     * 添加模板
     *
     * @param addModulInfoVO
     * @return
     */
    public Object saveOrUpdateModuleInfo(AddModuleInfoVO addModulInfoVO) {
        Boolean flag = false;
        //用户选择添加默认(标准)模块
        if (1 == addModulInfoVO.getDefaultModule()) {
            //用户同意更换默认(标准)模版   校验标准模版是否存在之前，提示信息用户操作默认为0 当用户点击提示信息确定则1
            if (1 == addModulInfoVO.getDefineType()) {
                //此时不会给出提示
                flag = true;
//            原有的标准模版设置为0 互斥行性
                UpdateWrapper<QcModuleInfo> serviceInfoWrapper = new UpdateWrapper<>();
                serviceInfoWrapper
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("mode_id", addModulInfoVO.getModeId())
                        .eq("hospital_id", addModulInfoVO.getHospitalId())
                        .eq("default_module", 1)
                        .set("default_module", "0");
                this.update(new QcModuleInfo(), serviceInfoWrapper);
            }
            //校验标准模版是否存在
            List<QcModuleInfo> list = this.list(new QueryWrapper<QcModuleInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("mode_id", addModulInfoVO.getModeId())
                    .eq("hospital_id", addModulInfoVO.getHospitalId())
                    .eq("default_module", 1));
            if (list.size() > 0) {
                //发现标准模版已经存在
                if (flag == false) {
                    //修改页面
                    if (null != addModulInfoVO.getId()) {
                        //当默认唯一模版提交修改按钮不提示
                        if (!addModulInfoVO.getId().equals(list.get(0).getId())) {
                            return null;
                        }
                    } else if (null == addModulInfoVO.getId()) {
                        //新增时候互斥给出提示
                        return null;
                    }
                }
            }
        }
        // 校验名称是否相同
        int count = this.count(new QueryWrapper<QcModuleInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("mode_id", addModulInfoVO.getModeId())
                .eq("hospital_id", addModulInfoVO.getHospitalId())
                .eq("name", addModulInfoVO.getName())
                .ne("id", addModulInfoVO.getId() == null ? -1 : addModulInfoVO.getId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "相同【医院】【模块】下，模板名称已存在");
        }

        // 同一文书类型下只有一份
        //  QcModuleInfo existBean = this.getOne(new QueryWrapper<QcModuleInfo>()
        //         .eq("is_deleted", IsDeleteEnum.N.getKey())
        //         .eq("mode_id", addModulInfoVO.getModeId())
        //         .eq("hospital_id", addModulInfoVO.getHospitalId())
        //         .eq("record_module_id", addModulInfoVO.getRecordModuleId())
        //         .ne("id", addModulInfoVO.getId() == null ? -1 : addModulInfoVO.getId()));
        // if (existBean != null) {
        //     throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
        //             "相同【医院】【模块】下，模块类型已存在,模板名称【" + existBean.getName() + "】" );
        // }

        // 校验当前文书是否已添加，多人同时操作
        int countRecord = medModuleMappingFacade.count(new QueryWrapper<MedModuleMapping>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .ne("module_id", addModulInfoVO.getId() == null ? -1 : addModulInfoVO.getId())
                .in("record_id", addModulInfoVO.getRecordIdList())
        );
        if (countRecord > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "模块类型已添加，请刷新后重新选择");
        }

        Long moduleId = null;
        Boolean bac = false;
        if (addModulInfoVO.getId() == null) {
            String userId = UserUtils.getCurrentPrincipleID();
            Date now = DateUtil.now();
            QcModuleInfo moduleInfo = new QcModuleInfo();
            BeanUtil.copyProperties(addModulInfoVO, moduleInfo);
            moduleInfo.setCreator(userId);
            moduleInfo.setModifier(userId);
            moduleInfo.setGmtCreate(now);
            moduleInfo.setGmtModified(now);
            this.save(moduleInfo);
            moduleId = moduleInfo.getId();
            bac = qcModuleDetailFacade.saveOrUpdateInfos(addModulInfoVO, moduleId);
        } else {
            UpdateModuleInfoVO updateModuleInfoVO = new UpdateModuleInfoVO();
            BeanUtil.copyProperties(addModulInfoVO, updateModuleInfoVO);
            bac = this.updateModuleInfo(updateModuleInfoVO);
        }

        cacheFacade.clear();
        return bac;
    }

    /**
     * 修改模板
     *
     * @param updateModuleInfoVO
     * @return
     */
    public Boolean updateModuleInfo(UpdateModuleInfoVO updateModuleInfoVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        UpdateWrapper<QcModuleInfo> moduleInfoUpdateWrapper = new UpdateWrapper<>();
        moduleInfoUpdateWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", updateModuleInfoVO.getId())
                .set("name", updateModuleInfoVO.getName())
                .set("mode_id", updateModuleInfoVO.getModeId())
                .set("default_module", updateModuleInfoVO.getDefaultModule())
                // .set("record_module_id", updateModuleInfoVO.getRecordModuleId())
                .set("modifier", userId)
                .set("gmt_modified", DateUtil.now())
                .set("remark", updateModuleInfoVO.getRemark());
        this.update(new QcModuleInfo(), moduleInfoUpdateWrapper);
        AddModuleInfoVO addModuleInfoVO = new AddModuleInfoVO();
        BeanUtil.copyProperties(updateModuleInfoVO, addModuleInfoVO);
        return qcModuleDetailFacade.saveOrUpdateInfos(addModuleInfoVO, updateModuleInfoVO.getId());
    }


    /**
     * 模板维护页面获取模板列表
     *
     * @param getModuleInfoVO
     * @return
     */
    public IPage<ModuleInfoListDTO> getModuleInfoList(GetModuleInfoVO getModuleInfoVO) {
        //获取模板信息
        IPage<ModuleInfoListDTO> moduleInfoDTOIPage = this.getModuleInfoListByType(getModuleInfoVO);
        return moduleInfoDTOIPage;
    }

    /**
     * 根据id获取模板信息
     *
     * @param getModuleInfoOneVO
     * @return
     */
    public GetModuleInfoOneDTO getById(GetModuleInfoOneVO getModuleInfoOneVO) {
        Long moduleId = getModuleInfoOneVO.getModuleId();
        GetModuleInfoOneDTO getModuleInfoOneDTO = new GetModuleInfoOneDTO();
        //获取模板信息
        QueryWrapper<QcModuleInfo> moduleInfoQueryWrapper = new QueryWrapper<>();
        moduleInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", moduleId);
        QcModuleInfo moduleInfo = this.getOne(moduleInfoQueryWrapper, false);
        if (null == moduleInfo) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "模板已删除【" + moduleId + "】");
        }
        BeanUtil.copyProperties(moduleInfo, getModuleInfoOneDTO);
        List<QcModuleDetail> qcModuleDetailList = qcModuleDetailFacade.list(new QueryWrapper<QcModuleDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("module_id", getModuleInfoOneDTO.getId())
                .orderByAsc("order_no")
        );
        List<QcModuleDetailDTO> qcModuleDetailDTOList = BeanUtil.listCopyTo(qcModuleDetailList, QcModuleDetailDTO.class);

        // 根据questionId获取所有的标签
        List<Long> questionIds = qcModuleDetailList.stream().map(r -> r.getQuestionId()).collect(Collectors.toList());
        QuestionIdsVO questionIdsVO = new QuestionIdsVO();
        questionIdsVO.setIds(questionIds);
        Map<Long, QuestionDTO> questionDTOMap = qcQuestionFacade.getByIds(questionIdsVO);
        for (QcModuleDetailDTO bean : qcModuleDetailDTOList) {
            QuestionDTO questionDTO = questionDTOMap.get(bean.getQuestionId());
            if (questionDTO != null) {
                bean.setQuestionDTO(questionDTO);
            }
        }
        Map<Integer, List<QcModuleDetailDTO>> map = EntityUtil.makeEntityListMap(qcModuleDetailDTOList, "groupId");
        getModuleInfoOneDTO.setModuleDetail(map);

        // 返回模板和文书的映射关系
        List<MedRecordAnalyzeDTO> recordAnalyzeList = medRecordAnalyzeFacade.getByModuleId(moduleId);
        getModuleInfoOneDTO.setRecordAnalyzeList(recordAnalyzeList);
        return getModuleInfoOneDTO;
    }


    /**
     * 校验questionId是否在模板中使用
     *
     * @param map
     * @return
     */
    public List<QcModuleInfo> getModuleNameByQuestionIdFac(Map map) {
        return this.getModuleNameByQuestionId(map);
    }

    /**
     * 根据医院和模块获取文书信息
     *
     * @param moduleIndexVO
     * @return
     */
    public List<MedRecordAnalyzeDTO> indexFac(ModuleIndexVO moduleIndexVO) {
        return medRecordAnalyzeFacade.indexFac(moduleIndexVO);
    }
}
