package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.*;
import com.diagbot.entity.*;
import com.diagbot.enums.InsertOrUpdateEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.QcTypeCasesEntryService;
import com.diagbot.service.impl.QcTypeServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.*;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.*;



/**
 * @Description:
 * @Author:zhoutg
 * @time: 2018/11/23 11:37
 */
@Component
public class QcTypeFacade extends QcTypeServiceImpl {


    @Autowired
    @Qualifier("qcTypeCasesEntryServiceImpl")
    QcTypeCasesEntryService qcTypeCasesEntryService;
    @Autowired
    UserServiceClient userServiceClient;
    @Autowired
    QcCacesEntryFacade qcCacesEntryFacade;
    @Autowired
    QcTypeCasesEntryFacade qcTypeCasesEntryFacade;
    @Autowired
    CasesEntryHospitalFacade casesEntryHospitalFacade;


    /**
     * 关联质控条目检索
     *
     * @param qcTypeVO
     */
    public RespDTO<List<QcType>>  indexData(QcTypeVO qcTypeVO) {
        List<QcType> list = new ArrayList<>();
        Set<Long> parTypeIdSet = new HashSet<>();
        List<QcType> parTypelist = this.list(new QueryWrapper<QcType>()
                        .eq("hospital_id", qcTypeVO.getHospitalId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .isNotNull("parent_type_id")
                        .orderByAsc("name"));

        if(ListUtil.isNotEmpty(parTypelist)){
            parTypelist.forEach(qcType -> {
                parTypeIdSet.add(qcType.getParentTypeId());
            });
        };

            QueryWrapper<QcType> qcTypeQueryWrapper = new QueryWrapper<>();
            qcTypeQueryWrapper.eq("hospital_id", qcTypeVO.getHospitalId());
            qcTypeQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
            if(!CollectionUtils.isEmpty(parTypeIdSet)){
                qcTypeQueryWrapper.notIn("id",parTypeIdSet);
            }
            qcTypeQueryWrapper.isNull("parent_type_id");
        //修改 非此判断为新增
        if(null != qcTypeVO.getId()) {
            qcTypeQueryWrapper.ne("id", qcTypeVO.getId());
        }
            qcTypeQueryWrapper.orderByAsc("name");
            list = this.list(qcTypeQueryWrapper);
        return RespDTO.onSuc(list);
    }
    /**
     * 判断质控类型是否关联
     *
     * @param cancelTypeVO
     */
    public RespDTO getOrCancel(CancelTypeVO cancelTypeVO) {
        //修改和删除都会触发操作
        //子类判断关联关系
         QcType qcType = this.getOne(new QueryWrapper<QcType>()
                .eq("hospital_id", cancelTypeVO.getHospitalId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", cancelTypeVO.getId()));
        //父类判断关联关系
         List<QcType> parentList= this.list(new QueryWrapper<QcType>()
                .eq("hospital_id", cancelTypeVO.getHospitalId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("parent_type_id", cancelTypeVO.getId()));
        if (cancelTypeVO.getId() != null&& null == cancelTypeVO.getCancel()) {
        //子类存在关联  一对一给出提示
            if (null != qcType.getParentTypeId()) {
                QcType qcTypeMain= this.getOne(new QueryWrapper<QcType>()
                        .eq("hospital_id", cancelTypeVO.getHospitalId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("id", qcType.getParentTypeId()));
                RespDTO respDTO = new RespDTO();
                respDTO.code = "00000005";
                respDTO.msg = "该质控类型已被" + "\"" + qcTypeMain.getName() + "\"" + "类型绑定，需先将其解绑";
               return respDTO;
               //父类存在关联  一对多给出提示
            }else if(ListUtil.isNotEmpty(parentList)&&1==cancelTypeVO.getOper()){
                RespDTO respDTO = new RespDTO();
                respDTO.code = "00000005";
                respDTO.msg = "该质控类型存在绑定的子类型，需先将其解绑";
                return respDTO;
            }
    
        }
        //删除关联状态
        if(cancelTypeVO.getCancel() != null && 1 == cancelTypeVO.getCancel()){
            //子类存在关联  一对一删除
            if (null != qcType.getParentTypeId()) {
                this.update(new UpdateWrapper<QcType>()
                        .eq("hospital_id", cancelTypeVO.getHospitalId())
                        .eq("id", cancelTypeVO.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("parent_type_id", null)
                );
        //父类存在关联  一对多删除
            }else if(ListUtil.isNotEmpty(parentList)){
                this.update(new UpdateWrapper<QcType>()
                        .eq("hospital_id", cancelTypeVO.getHospitalId())
                        .eq("parent_type_id", cancelTypeVO.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("parent_type_id", null)
                );
            }
            RespDTO respDTO = new RespDTO();
            respDTO.code = "0";
            respDTO.msg = "解绑成功";
            return respDTO;

        }
            return new RespDTO();
    }


    /**
     * 保存
     *
     * @param qcTypeSaveVO
     */
    public RespDTO saveOrUpdate(QcTypeSaveVO qcTypeSaveVO) {
            //初始化参数
            CommonParam param = initCommonParam();
            // 保存主表
            saveQcType(qcTypeSaveVO, param);
            // 保存质控类型和质控条目映射关系
            saveQcTypeCasesEntry(qcTypeSaveVO, param);
            //质控类型进行关联
            getTypeConnections(qcTypeSaveVO, param);
        return RespDTO.onSuc("保存成功");

    }

    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        String person = UserUtils.getCurrentPrincipleID();
        param.setNow(DateUtil.now());
        param.setPerson(person);
        param.setInsertOrUpdate(InsertOrUpdateEnum.Insert.getKey());
        return param;
    }

    /**
     * 保存映射关系
     *
     * @param qcTypeSaveVO
     * @param param
     */
    public void saveQcTypeCasesEntry(QcTypeSaveVO qcTypeSaveVO, CommonParam param) {
        Date now = param.getNow();
        String person = param.getPerson();
        // 先删除原明细(物理删除)
        qcTypeCasesEntryService.remove(new QueryWrapper<QcTypeCasesEntry>()
                .eq("type_id", qcTypeSaveVO.getId())
        );
        // 再插入新明细
        List<QcTypeCasesEntryVO> qcTypeCasesEntryVOList = qcTypeSaveVO.getQcTypeCasesEntryVOList();
        if (ListUtil.isNotEmpty(qcTypeCasesEntryVOList)) {
            List<QcTypeCasesEntry> saveQcTypeCasesEntry = new ArrayList<>();
            for (QcTypeCasesEntryVO mapping : qcTypeCasesEntryVOList) {
                QcTypeCasesEntry bean = new QcTypeCasesEntry();
                bean.setCaseEntryId(mapping.getCaseEntryId());
                bean.setCreator(person);
                bean.setGmtCreate(now);
                bean.setModifier(person);
                bean.setGmtModified(now);
                bean.setTypeId(qcTypeSaveVO.getId());
                saveQcTypeCasesEntry.add(bean);
            }
            qcTypeCasesEntryService.saveBatch(saveQcTypeCasesEntry);
        }
    }

    /**
     * 保存主表
     *
     * @param qcTypeSaveVO
     * @param param
     */
    public void saveQcType(QcTypeSaveVO qcTypeSaveVO, CommonParam param) {
        QcType qcType = new QcType();
        Long id = qcTypeSaveVO.getId();

        // 保存主表
        if (qcTypeSaveVO.getId() != null) {
            qcType = this.getOne(new QueryWrapper<QcType>()
                    .eq("id", id)
                    .eq("is_deleted", IsDeleteEnum.N.getKey()), false);
            if (qcType == null) { //校验
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "当前质控类型不存在【id=" + id + "】");
            }
            param.setInsertOrUpdate(InsertOrUpdateEnum.Update.getKey());
        }
        BeanUtil.copyProperties(qcTypeSaveVO, qcType);
        //判断相同名称类型的质控id是否存在
        List<QcType> qcTypeList = this.list(new QueryWrapper<QcType>()
                .eq("name", qcTypeSaveVO.getName())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", qcTypeSaveVO.getHospitalId())
                .ne("id", id == null ? -1 : id));
        // 质控类型hospitalId、name唯一
        if (ListUtil.isNotEmpty(qcTypeList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "质控类型已存在");
        }
        if (InsertOrUpdateEnum.Insert.getKey() == param.getInsertOrUpdate()) {
            qcType.setCreator(param.getPerson()); //创建人
            qcType.setGmtCreate(param.getNow());//创建时间
        }
        qcType.setGmtModified(param.getNow());//修改时间
        qcType.setModifier(param.getPerson());//修改人
        this.saveOrUpdate(qcType);
        qcTypeSaveVO.setId(qcType.getId());
    }

    /**
     * 标签列表
     *
     * @param qcTypePageVO
     * @return
     */
    public IPage<QcTypePageDTO> getListFac(QcTypePageVO qcTypePageVO) {
        IPage<QcTypePageDTO> res = this.getList(qcTypePageVO);
//        if (res.getTotal() <= 0) {
//            return res;
//        }
//        List<String> personIds = res.getRecords().stream().map(row -> row.getModifier()).collect(Collectors.toList());
//        RespDTO<Map<String, String>> mapRespDTO = userServiceClient.getUserInfoByIds(personIds);
//        if (mapRespDTO == null || !CommonErrorCode.OK.getCode().equals(mapRespDTO.code)) {
//            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取操作人失败");
//        }
//        for (QuestionPageDTO bean : res.getRecords()) {
//            bean.setModifier(mapRespDTO.data.get(bean.getModifier()));
//        }
        return res;
    }

    /**
     * 质控类型维护关联
     * @param qcTypeSaveVO,param
     * @return
     */
    public void getTypeConnections(QcTypeSaveVO qcTypeSaveVO, CommonParam param) {
        QcTypeVO qcTypeVO  = new QcTypeVO();
        BeanUtil.copyProperties(qcTypeSaveVO, qcTypeVO);
        List<Long> connectionIds = qcTypeSaveVO.getTypeIdList();
        List<QcTypeCasesEntryVO> list = qcTypeSaveVO.getQcTypeCasesEntryVOList();
        //修改操作
        if(null !=qcTypeSaveVO.getId()){
                //删除父类id
                this.update(new UpdateWrapper<QcType>()
                        .eq("hospital_id", qcTypeVO.getHospitalId())
                        .eq("parent_type_id",qcTypeVO.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("parent_type_id",null)
                );
            }
        if(ListUtil.isNotEmpty(connectionIds)){
            for (Long id : connectionIds) {
                //删除绑定质控id所有case_entry_id
                qcTypeCasesEntryFacade.remove(new QueryWrapper<QcTypeCasesEntry>()
                        .eq("type_id", id)
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                );
                //该医院质控id下的质控条目case_entry_id与主关联的保持一致
                list.forEach(qcTypeCasesEntryVO->{
                                QcTypeCasesEntry qcTypeCaseEntry = new QcTypeCasesEntry();
                                qcTypeCaseEntry.setCaseEntryId(qcTypeCasesEntryVO.getCaseEntryId());
                                qcTypeCaseEntry.setTypeId(id);
                                qcTypeCaseEntry.setGmtCreate(param.getNow());
                                qcTypeCaseEntry.setGmtModified(param.getNow());
                                qcTypeCaseEntry.setCreator(param.getPerson());
                                qcTypeCaseEntry.setModifier(param.getPerson());
                                qcTypeCasesEntryFacade.save(qcTypeCaseEntry);
                            }
                );
                    //设置父质控id
                     this.update(new UpdateWrapper<QcType>()
                        .eq("hospital_id", qcTypeSaveVO.getHospitalId())
                        .eq("id", id)
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("parent_type_id",qcTypeVO.getId())
                     );
            }
        }
    }

    /**
     * 新增时质控条目分组信息
     *
     * @param qcEntryHospitalVO
     * @return
     */
    public Map<String, List<QcCasesEntrySimpleDTO>> getEntryByHospital(@RequestBody QcEntryHospitalVO qcEntryHospitalVO) {
        List<QcCasesEntrySimpleDTO> qcCasesEntrySimpleDTOList = qcCacesEntryFacade.getEntryByHospitalFac(qcEntryHospitalVO);
        return EntityUtil.makeEntityListMap(qcCasesEntrySimpleDTOList, "modeName");
    }

    /**
     * 检索
     *
     * @param qcTypeIndexVO
     * @return
     */
    public List<QcCasesEntrySimpleDTO> indexFac(QcTypeIndexVO qcTypeIndexVO) {
        List<QcCasesEntrySimpleDTO> res = qcCacesEntryFacade.index(qcTypeIndexVO);
        return res;
    }

    /**
     * 根据参数返回质控类型内容
     *
     * @param qcTypeVO 获取质控类型内容参数
     * @return 标签内容
     */
    public QcTypeDTO getById(QcTypeVO qcTypeVO) {
        QcTypeDTO qcTypeDTO = new QcTypeDTO();
        QcType qcType = this.getOne(new QueryWrapper<QcType>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", qcTypeVO.getId()), false
        );
        if (qcType == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "当前质控类型不存在或已删除【id=" + qcTypeVO.getId() + "】");
        }
        List<QcType> qcTypeList = this.list(new QueryWrapper<QcType>()
                .eq("parent_type_id", qcTypeVO.getId())
                .eq("is_deleted", "N"));
        List<QcTypeSimpDTO> qcTypeSimpDTOList = new ArrayList<>();
        qcTypeList.forEach( qcTypeOne->{
            QcTypeSimpDTO qcTypeSimpDTO = new QcTypeSimpDTO();
            qcTypeSimpDTO.setId(qcTypeOne.getId());
            qcTypeSimpDTO.setName(qcTypeOne.getName());
            qcTypeSimpDTOList.add(qcTypeSimpDTO);
        } );
        qcTypeDTO.setQcTypeSimpDTOList(qcTypeSimpDTOList);
        BeanUtil.copyProperties(qcType, qcTypeDTO);
        List<QcCasesEntrySimpleDTO> list = qcTypeCasesEntryFacade.getByTypeIdFac(qcTypeVO);
        Map<String, List<QcCasesEntrySimpleDTO>> entryMap = EntityUtil.makeEntityListMap(list, "modeName");
        qcTypeDTO.setEntryMap(entryMap);
        return qcTypeDTO;
    }

    /**
     * 根据id删除质控类型
     *
     * @param deleteQcTypeVO
     * @return
     */
    public RespDTO deleteByIdsFac(DeleteQcTypeVO deleteQcTypeVO) {
        Date now = DateUtil.now();
        String person = UserUtils.getCurrentPrincipleID();
        // 更新主表
        this.update(new UpdateWrapper<QcType>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("id", deleteQcTypeVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("gmt_modified", now)
                .set("modifier", person)
        );

        // 删除明细表
        if (ListUtil.isNotEmpty(deleteQcTypeVO.getIds())) {
            qcTypeCasesEntryService.remove(new QueryWrapper<QcTypeCasesEntry>()
                    .in("type_id", deleteQcTypeVO.getIds())
            );
        }
        return  RespDTO.onSuc("删除成功");
    }

}
