package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.StdMissionDetailPageDTO;
import com.diagbot.dto.SthospitalInfoStatiDTO;
import com.diagbot.entity.StdMissionDetail;
import com.diagbot.entity.StdMissionInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.StdMissionDetailService;
import com.diagbot.service.impl.StdMissionDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.vo.StdMissionDetailClVO;
import com.diagbot.vo.StdMissionDetailPageVO;
import com.diagbot.vo.StdMissionDetailUpVO;
import com.diagbot.vo.StdMissionInfoAddVO;
import com.diagbot.vo.SthospitalInfoStatiVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-06-12 16:50
 */
@Component
public class StdMissionDetailFacade extends StdMissionDetailServiceImpl {

    @Autowired
    StdMissionDetailService stdMissionDetailService;
    @Autowired
    StdBehospitalInfoFacade stdBehospitalInfoFacade;
    @Autowired
    StdMissionInfoFacade stdMissionInfoFacade;

    public IPage<StdMissionDetailPageDTO> getMissionDetailPages(StdMissionDetailPageVO stdMissionDetailPageVO) {
        return stdMissionDetailService.getMissionDetailPage(stdMissionDetailPageVO);
    }

    public boolean addMissionDetails(StdMissionInfoAddVO stdMissionInfoAddVO) {
        //验证任务名是否存在
        QueryWrapper<StdMissionInfo> stdQcresultTaskQuery = new QueryWrapper<>();
        stdQcresultTaskQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("mission_name", stdMissionInfoAddVO.getMissionName());
        int sum = stdMissionInfoFacade.count(stdQcresultTaskQuery);
        if (sum > 0) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该任务名已存在！");
        }
        //添加数据生成任务
        Date now = DateUtil.now();
        StdMissionInfo stdMissionInfo = new StdMissionInfo();
        BeanUtil.copyProperties(stdMissionInfoAddVO, stdMissionInfo);
        stdMissionInfo.setGmtCreate(now);
        stdMissionInfo.setGmtModified(now);
        boolean res = stdMissionInfoFacade.save(stdMissionInfo);
        if (res) {
            Long missionId = stdMissionInfo.getId();
            SthospitalInfoStatiVO sthospitalInfoPageVO = new SthospitalInfoStatiVO();
            sthospitalInfoPageVO.setPages(1);
            sthospitalInfoPageVO.setSize(5000);
            IPage<SthospitalInfoStatiDTO> sthospital = stdBehospitalInfoFacade.getStatistic(sthospitalInfoPageVO);
            List<SthospitalInfoStatiDTO> records = sthospital.getRecords();
            List<StdMissionDetail> missionDetailList = new ArrayList<>();
            for (SthospitalInfoStatiDTO data : records) {
                StdMissionDetail stdMissionDetail = new StdMissionDetail();
                BeanUtil.copyProperties(data, stdMissionDetail);
                stdMissionDetail.setMissionId(missionId);
                stdMissionDetail.setGmtCreate(now);
                stdMissionDetail.setGmtModified(now);
                missionDetailList.add(stdMissionDetail);
            }
            res = stdMissionDetailService.saveBatch(missionDetailList);
        }


        return res;
    }

    public boolean updateMissionDetails(StdMissionDetailUpVO stdMissionDetailUpVO) {
        Date now = DateUtil.now();
        return this.update(new StdMissionDetail(), new UpdateWrapper<StdMissionDetail>()
                .eq("id", stdMissionDetailUpVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("status", stdMissionDetailUpVO.getStatus())
                .set("remark", stdMissionDetailUpVO.getRemark()));
    }

    public boolean clearMissionDetails(StdMissionDetailClVO stdMissionDetailClVO) {
        Date now = DateUtil.now();
        return this.update(new StdMissionDetail(), new UpdateWrapper<StdMissionDetail>()
                .in("id", stdMissionDetailClVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("is_deleted", IsDeleteEnum.Y.getKey()));
    }
}
