package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.StdMissionInfoPageDTO;
import com.diagbot.entity.StdMissionDetail;
import com.diagbot.entity.StdMissionInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.StdMissionInfoService;
import com.diagbot.service.impl.StdMissionInfoServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.vo.StdMissionInfoClearVO;
import com.diagbot.vo.StdMissionInfoPageVO;
import com.diagbot.vo.StdMissionInfoUpVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-06-15 9:40
 */
@Component
public class StdMissionInfoFacade extends StdMissionInfoServiceImpl {
    @Autowired
    StdMissionInfoService stdMissionInfoService;
    @Autowired
    StdMissionDetailFacade stdMissionDetailFacade;

    public IPage<StdMissionInfoPageDTO> getMissionInfoPages(StdMissionInfoPageVO stdMissionInfoPageVO) {
        return stdMissionInfoService.getMissionInfoPages(stdMissionInfoPageVO);
    }

    public boolean updataMissionInfos(StdMissionInfoUpVO stdMissionInfoUpVO) {
        Date now = DateUtil.now();
        QueryWrapper<StdMissionInfo> missionInfoQuery = new QueryWrapper<>();
        missionInfoQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .ne("id", stdMissionInfoUpVO.getId() == null ? -1 : stdMissionInfoUpVO.getId())
                .eq("mission_name", stdMissionInfoUpVO.getMissionName());
        int count = stdMissionInfoService.count(missionInfoQuery);
        if (count > 0) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该任务名已存在！");
        }
        UpdateWrapper<StdMissionInfo> missionInfoUpdate = new UpdateWrapper<>();
        missionInfoUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", stdMissionInfoUpVO.getId())
                .set("mission_name", stdMissionInfoUpVO.getMissionName())
                .set("gmt_modified", now);

        return stdMissionInfoService.update(missionInfoUpdate);
    }

    public boolean clearMissionInfos(StdMissionInfoClearVO stdMissionInfoClearVO) {
        Date now = DateUtil.now();
        //删除该任务
        UpdateWrapper<StdMissionInfo> missionInfoUpdate = new UpdateWrapper<>();
        missionInfoUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", stdMissionInfoClearVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("gmt_modified", now);
        boolean res = stdMissionInfoService.update(missionInfoUpdate);
        //删除该任务所有明细
        UpdateWrapper<StdMissionDetail> missionDetailUpdate = new UpdateWrapper<>();
        missionDetailUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("mission_id", stdMissionInfoClearVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("gmt_modified", now);
        stdMissionDetailFacade.update(missionDetailUpdate);
        return res;
    }
}
