package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllByHospitalDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.CasesEntryHospitalFacade;
import com.diagbot.vo.GetAllByHospitalVO;
import com.diagbot.vo.InsertByHospitalVO;
import com.diagbot.vo.UpdateByHospitalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 病历条目 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2020-03-31
 */
@RestController
@Api(value = "病例条目医院关系", tags = { "病例条目医院关系" })
@RequestMapping("/casesEntryHospital")
public class CasesEntryHospitalController {
    @Autowired
    private CasesEntryHospitalFacade casesEntryHospitalFacade;

    @ApiOperation(value = "获取指定医院的病例条目[by:wangyu]",
            notes = "获取指定医院的病例条目")
    @PostMapping("/getAllByHospital")
    @SysLogger("getAllByHospital")
    public RespDTO<List<GetAllByHospitalDTO>> getAllByHospital(@RequestBody GetAllByHospitalVO getAllByHospitalVO) {
        List<GetAllByHospitalDTO> data = casesEntryHospitalFacade.getAllByHospital(getAllByHospitalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "修改指定医院的病例条目[by:wangyu]",
            notes = "修改指定医院的病例条目")
    @PostMapping("/updateByHospital")
    @SysLogger("updateByHospital")
    @Transactional
    public RespDTO<Boolean> updateByHospital(@RequestBody UpdateByHospitalVO updateByHospitalVO) {
        Boolean data = casesEntryHospitalFacade.updateByHospital(updateByHospitalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "新增指定医院的病例条目[by:wangyu]",
            notes = "新增指定医院的病例条目")
    @PostMapping("/insertByHospital")
    @SysLogger("insertByHospital")
    @Transactional
    public RespDTO<Boolean> insertByHospital(@RequestBody @Valid InsertByHospitalVO insertByHospitalVO) {
        Boolean data = casesEntryHospitalFacade.insertByHospital(insertByHospitalVO);
        return RespDTO.onSuc(data);
    }

}
