package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.QcCasesAllDTO;
import com.diagbot.dto.QcCasesDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.QcCasesFacade;
import com.diagbot.vo.GetAllByHospitalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 病历 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-03-10
 */
@RestController
@RequestMapping("/qcCases")
@Api(value = "病历模块相关接口", tags = { "病历模块相关接口" })
public class QcCasesController {

    @Autowired
    private QcCasesFacade qcCasesFacade;

    @ApiOperation(value = "获取全部模块及条目[by:wangfeng]",
            notes = "获取全部模块及条目")
    @PostMapping("/getAll")
    @SysLogger("getAll")
    public RespDTO<List<QcCasesAllDTO>> getAll() {
        List<QcCasesAllDTO> data =qcCasesFacade.getAlls();
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取指定医院模块及条目[by:wangyu]",
            notes = "获取全部模块及条目")
    @PostMapping("/getAllByHospital")
    @SysLogger("getAllByHospital")
    public RespDTO<List<QcCasesAllDTO>> getAllByHospital(@RequestBody GetAllByHospitalVO getAllByHospitalVO) {
        List<QcCasesAllDTO> data =qcCasesFacade.getAllByHospital(getAllByHospitalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取全部模块[by:wangfeng]",
            notes = "获取全部模块")
    @PostMapping("/getCases")
    @SysLogger("getCases")
    public RespDTO<List<QcCasesDTO>> getCasesAlls() {

        return RespDTO.onSuc(qcCasesFacade.getCasesAll());
    }
}
