package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetUpdateInfoDTO;
import com.diagbot.dto.QcCasesEntryAllDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.QcCacesEntryFacade;
import com.diagbot.vo.GetUpdateInfoVO;
import com.diagbot.vo.QcCasesEntryAllVO;
import com.diagbot.vo.QcCasesEntryIndexVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 病历条目 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-03-10
 */

@RestController
@RequestMapping("/qcCasesEntry")
@Api(value = "病历条目相关接口", tags = { "病历条目相关接口" })
public class QcCasesEntryController {
    @Autowired
    private QcCacesEntryFacade qcCacesEntryFacade;

    @ApiOperation(value = "获取全部条目[by:wangyu]",
            notes = "获取全部条目")
    @PostMapping("/getAll")
    @SysLogger("getAll")
    public RespDTO<IPage<QcCasesEntryAllDTO>> getAll(@RequestBody QcCasesEntryAllVO qcCasesEntryAllVO) {
        IPage<QcCasesEntryAllDTO> data = qcCacesEntryFacade.getAll(qcCasesEntryAllVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "搜索质控条目[by:zhoutg]",
            notes = "hospitalId：医院ID<br>" +
                    "code：编码<br>" +
                    "modeId：模块ID<br>" +
                    "casesId：病历模块<br>" +
                    "name：条目名称<br>" +
                    "ids：指定ids<br>" +
                    "notIds：排除ids<br>")
    @PostMapping("/indexCaseEntry")
    @SysLogger("indexCaseEntry")
    public RespDTO<IPage<QcCasesEntryAllDTO>> indexCaseEntry(@RequestBody QcCasesEntryIndexVO qcCasesEntryIndexVO) {
        IPage<QcCasesEntryAllDTO> data = qcCacesEntryFacade.getAllQcCasesEntryIndexFac(qcCasesEntryIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "修改时获取条目信息[by:wangyu]",
            notes = "修改时获取条目信息")
    @PostMapping("/getUpdateInfo")
    @SysLogger("getUpdateInfo")
    public RespDTO<List<GetUpdateInfoDTO>> getUpdateInfo(@RequestBody GetUpdateInfoVO getUpdateInfoVO) {
        List<GetUpdateInfoDTO> data =qcCacesEntryFacade.getUpdateInfo(getUpdateInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "删除指定条目[by:wangyu]",
            notes = "删除指定条目")
    @PostMapping("/deleteQcCasesEntry")
    @SysLogger("deleteQcCasesEntry")
    public RespDTO<Boolean> deleteQcCasesEntry(@RequestBody GetUpdateInfoVO getUpdateInfoVO) {
        Boolean data =qcCacesEntryFacade.deleteQcCasesEntry(getUpdateInfoVO);
        return RespDTO.onSuc(data);
    }
}
