package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.QcEntryTypeDTO;
import com.diagbot.dto.QcEntryTypeDetailDTO;
import com.diagbot.dto.QcTypeSimpDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.QcCacesEntryFacade;
import com.diagbot.vo.QcEntryTypeIndexVO;
import com.diagbot.vo.QcEntryTypePageVO;
import com.diagbot.vo.QcEntryTypeSaveVO;
import com.diagbot.vo.QcEntryTypeVO;
import com.diagbot.vo.QcTypeHospitalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 质控条目与质控类型映射 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2020-05-14
 */
@RestController
@RequestMapping("/qc/qcEntryType")
@SuppressWarnings("unchecked")
@Api(value = "质控条目与质控类型关系映射相关API", tags = { "质控条目与质控类型关系映射相关API" })
public class QcEntryTypeController {

    @Autowired
    QcCacesEntryFacade qcCacesEntryFacade;

    @ApiOperation(value = "保存[by:zhoutg]",
            notes = "// 医院id\n" +
                    " Long hospitalId;\n" +
                    "// 条目id\n" +
                    " Long entryId;\n" +
                    "// 质控类型id列表\n" +
                    " List<Long> typeIdList;")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @Transactional
    public RespDTO<Boolean> saveOrUpdate(@RequestBody QcEntryTypeSaveVO qcEntryTypeSaveVO) {
        qcCacesEntryFacade.saveOrUpdate(qcEntryTypeSaveVO);
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "分页列表[by:zhoutg]",
            notes = " // 医院id\n" +
                    " Long hospitalId;\n" +
                    "// 条目名称\n" +
                    " String entryName;\n" +
                    "// 质控类型名称\n" +
                    " String typeName;")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<QcEntryTypeDTO>> list(@RequestBody QcEntryTypePageVO qcEntryTypePageVO) {
        IPage<QcEntryTypeDTO> data = qcCacesEntryFacade.entryTypePageFac(qcEntryTypePageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院和条目检索未被选择的质控类型[by:zhoutg]",
            notes = "// 医院id\n" +
                    " Long hospitalId;\n" +
                    "// 条目Id\n" +
                    " Long entryId;")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<QcTypeSimpDTO>> index(@RequestBody QcEntryTypeIndexVO qcEntryTypeIndexVO) {
        List<QcTypeSimpDTO>data = qcCacesEntryFacade.indexFac(qcEntryTypeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据entryId和医院id返回内容[by:zhoutg]",
            notes = "// 条目id\n" +
                    " Long entryId;\n" +
                    "// 医院id\n" +
                    " Long hospitalId;")
    @PostMapping("/getById")
    @SysLogger("getById")
    public RespDTO<QcEntryTypeDetailDTO> getById(@RequestBody QcEntryTypeVO qcEntryTypeVO) {
        QcEntryTypeDetailDTO data = qcCacesEntryFacade.getByIdFac(qcEntryTypeVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院获取质控类型列表[by:zhoutg]",
            notes = "// 医院id\n" +
                    " Long hospitalId;")
    @PostMapping("/getTypeByHospital")
    @SysLogger("getTypeByHospital")
    public RespDTO<List<QcTypeSimpDTO>> getTypeByHospital(@RequestBody QcTypeHospitalVO qcTypeHospitalVO) {
        List<QcTypeSimpDTO> data = qcCacesEntryFacade.getTypeByHospitalFac(qcTypeHospitalVO);
        return RespDTO.onSuc(data);
    }
}
