package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.QcAllInputcasesDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.QcInputcasesFacade;
import com.diagbot.vo.QcInputcasesAllVO;
import com.diagbot.vo.QcInputcasesByIdVO;
import com.diagbot.vo.QcInputcasesSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 病历录入 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-03-10
 */
@RestController
@RequestMapping("/qcInputcases")
@Api(value = "病历录入相关接口", tags = { "病历录入相关接口" })
public class QcInputcasesController{

    @Autowired
    QcInputcasesFacade qcInputcasesFacade;

    /*@ApiOperation(value = "获取全部病历录入[by:wangfeng]",
            notes = "获取全部病历录入")
    @PostMapping("/getInputcasesAll")
    @SysLogger("getInputcasesAll")
    public RespDTO<List<QcInputcasesAllDTO>> getInputcasesAll(@Valid @RequestBody QcInputcasesAllVO qcInputcasesAllVO) {
        List<QcInputcasesAllDTO> data = qcInputcasesFacade.getInputcasesAlls(qcInputcasesAllVO.getTextId());
        return RespDTO.onSuc(data);
    }*/
    @ApiOperation(value = "保存和修改全部病历录入[by:wangfeng]",
            notes = "id; 病历id（保存不传，修改必传）医院Id：hospitalId;" +
                    "入院记录： text;" +
                    " 临床用血审核： clinicalBlood;" +
                    "会诊： consultation;" +
                    "死亡病例讨论： deathCaseDiscuss;" +
                    "死亡记录：deathRecord;" +
                    "疑难病例讨论：difficultCaseDiscuss;" +
                    "值班交接班：dutyShiftSystem;" +
                    "首次病程录： firstCourseRecord;" +
                    "出院小结： leaveHospital;" +
                    "分级护理：nursingSystem; " +
                    "手术期间讨论： operationDiscussion;" +
                    "术前讨论： preOperativeDiscussion;" +
                    "三级查房：threeLevelWard; " +
                    "危急值报告：criticalValue;" +
                    "qcInputMappingSaveVO{textId: 病历文本id ;casesId: 模块id;casesEntryId: 条目id ;pass:是否通过"+
                    "}")
    @PostMapping("/getInputcasesAlls")
    @SysLogger("getInputcasesAlls")
    public RespDTO<Boolean> saveAndUpdata(@Valid @RequestBody QcInputcasesSaveVO qcInputcasesSaveVO) {
        boolean res = qcInputcasesFacade.saveAndUpdatas(qcInputcasesSaveVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除病历录入[by:wangfeng]",
            notes = "删除病历录入病历录入")
    @PostMapping("/cancelById")
    @SysLogger("cancelById")
    public RespDTO<Boolean> cancelById(@Valid @RequestBody QcInputcasesByIdVO qcInputcasesByIdVO) {
        boolean res = qcInputcasesFacade.cancelByIds(qcInputcasesByIdVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除病历录入[by:wangyu]",
            notes = "删除病历录入病历录入")
    @PostMapping("/deleteById")
    @SysLogger("deleteById")
    public RespDTO<Boolean> deleteById(@Valid @RequestBody QcInputcasesByIdVO qcInputcasesByIdVO) {
        boolean res = qcInputcasesFacade.deleteById(qcInputcasesByIdVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "获取全部病历录入[by:wangyu]",
            notes = "获取全部病历录入")
    @PostMapping("/getAllInputcases")
    @SysLogger("getAllInputcases")
    public RespDTO<List<QcAllInputcasesDTO>> getAllInputcases(@Valid @RequestBody QcInputcasesAllVO qcInputcasesAllVO) {
        List<QcAllInputcasesDTO> data = qcInputcasesFacade.getAllInputcases(qcInputcasesAllVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取单份病例详情[by:wangyu]",
            notes = "获取单份病例详情")
    @PostMapping("/getInputcasesDetial")
    @SysLogger("getInputcasesDetial")
    public RespDTO<List<QcAllInputcasesDTO>> getInputcasesDetial(@Valid @RequestBody QcInputcasesAllVO qcInputcasesAllVO) {
        List<QcAllInputcasesDTO> data = qcInputcasesFacade.getInputcasesDetial(qcInputcasesAllVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存和修改全部病历录入[by:wangfeng]",
            notes = "id; 病历id（保存不传，修改必传）医院Id：hospitalId;" +
                    "入院记录： text;" +
                    " 临床用血审核： clinicalBlood;" +
                    "会诊： consultation;" +
                    "死亡病例讨论： deathCaseDiscuss;" +
                    "死亡记录：deathRecord;" +
                    "疑难病例讨论：difficultCaseDiscuss;" +
                    "值班交接班：dutyShiftSystem;" +
                    "首次病程录： firstCourseRecord;" +
                    "出院小结： leaveHospital;" +
                    "分级护理：nursingSystem; " +
                    "手术期间讨论： operationDiscussion;" +
                    "术前讨论： preOperativeDiscussion;" +
                    "三级查房：threeLevelWard; " +
                    "危急值报告：criticalValue;" +
                    "qcInputMappingSaveVO{textId: 病历文本id ;casesId: 模块id;casesEntryId: 条目id ;pass:是否通过"+
                    "}")
    @PostMapping("/saveOrUpdata")
    @SysLogger("saveOrUpdata")
    public RespDTO<Boolean> saveOrUpdata(@Valid @RequestBody QcInputcasesSaveVO qcInputcasesSaveVO) {
        boolean res = qcInputcasesFacade.saveOrUpdata(qcInputcasesSaveVO);
        return RespDTO.onSuc(res);
    }
}
