package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetModuleInfoOneDTO;
import com.diagbot.dto.MedRecordAnalyzeDTO;
import com.diagbot.dto.ModuleInfoListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.QcModuleInfoFacade;
import com.diagbot.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 模型表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2020-04-27
 */
@RestController
@RequestMapping("/qc/qcModuleInfo")
@SuppressWarnings("unchecked")
@Api(value = "模板相关API", tags = { "模板相关API" })
public class QcModuleInfoController {

    @Autowired
    QcModuleInfoFacade qcModuleInfoFacade;

    @ApiOperation(value = "模板新增或修改[by:zhoutg]",
            notes = "id，修改时传，新增不用传<br>" +
                    "name: 模板名称，必填<br>" +
                    "modeId: 数据模块ID，必填<br>" +
                    "defaultModule:默认模版是否开启(0-默认否 1-是)" +
                    "defineType: 校验时使用(0-默认不更换 1-更换)" +
                    "recordIdList: med_record_analyze的id列表<br>" +
                    "hospitalId: 医院ID，必填<br>" +
                    "moduleDetail: 标签列表<br>" +
                    "remark：备注")
    @PostMapping("/saveOrUpdateModuleInfo")
    @SysLogger("saveOrUpdateModuleInfo")
    @Transactional
    public RespDTO<Object> saveOrUpdateModuleInfo(@Valid @RequestBody AddModuleInfoVO addModuleInfoVO) {
        Object data = qcModuleInfoFacade.saveOrUpdateModuleInfo(addModuleInfoVO);
        if(null==data){
            return RespDTO.onError("默认标准类型已存在，是否替换");
        }
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "模板列表首页[by:cy]",
            notes = "id，模板id<br>" +
                    "name: 模板名称，必填<br>" +
                    "defaultModule:默认模版是否开启(0-默认否 1-是)" +
                    "defineType: 校验时使用(0-默认不更换 1-更换)" +
                    "modeId: 数据模块ID，必填<br>" +
                    "hospitalId: 医院ID，必填<br>")
    @PostMapping("/saveDefaultModuleInfo")
    @SysLogger("saveDefaultModuleInfo")
    @Transactional
    public RespDTO<Object> saveDefaultModuleInfo(@Valid @RequestBody DefaultModuleVO defaultModuleVO) {
        Object data = qcModuleInfoFacade.saveDefaultModuleInfo(defaultModuleVO);
        if(null==data){
            return RespDTO.onError("默认标准类型已存在，是否替换？");
        }
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "模板删除[by:zhoutg]",
            notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@RequestBody DeleteModuleVO deleteModuleVO) {
        qcModuleInfoFacade.deleteByIdsFac(deleteModuleVO.getIds());
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "模板分页列表[by:zhoutg]",
            notes = "name: 模板名称<br>" +
                    "modeId: 数据模块ID<br>" +
                    "id: 模板ID<br>" +
                    "hospitalId: 医院ID<br>")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<ModuleInfoListDTO>> getModuleInfoList(@RequestBody GetModuleInfoVO getModuleInfoVO) {
        IPage<ModuleInfoListDTO> data = qcModuleInfoFacade.getModuleInfoList(getModuleInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据id获取模板信息[by:zhoutg]",
            notes = "moduleId: 模板id<br>")
    @PostMapping("/getById")
    @SysLogger("getById")
    public RespDTO<GetModuleInfoOneDTO> getById(@RequestBody GetModuleInfoOneVO getModuleInfoVO) {
        GetModuleInfoOneDTO data = qcModuleInfoFacade.getById(getModuleInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院和模块获取文书信息[by:zhoutg]",
            notes = "// 医院ID\n" +
                    " Long hospitalId;\n" +
                    "// 数据模块ID\n" +
                    " Long modeId;\n" +
                    "// 过滤的id列表\n" +
                    " List<Long> notIdList;")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<MedRecordAnalyzeDTO>> index(@RequestBody ModuleIndexVO moduleIndexVO) {
        List<MedRecordAnalyzeDTO> data = qcModuleInfoFacade.indexFac(moduleIndexVO);
        return RespDTO.onSuc(data);
    }
}
