package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.QuestionPageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.QcQuestionInfo;
import com.diagbot.facade.QcQuestionFacade;
import com.diagbot.vo.DeleteQuestionVO;
import com.diagbot.vo.QuestionIdsVO;
import com.diagbot.vo.QuestionIndexVO;
import com.diagbot.vo.QuestionPageVO;
import com.diagbot.vo.QuestionSaveVO;
import com.diagbot.vo.QuestionVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 标签基础表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2020-04-27
 */
@RestController
@RequestMapping("/qc/qcQuestionInfo")
@SuppressWarnings("unchecked")
@Api(value = "标签相关API", tags = { "标签相关API" })
public class QcQuestionInfoController {

    @Autowired
    QcQuestionFacade qcQuestionFacade;

    @ApiOperation(value = "标签新增或更新[by:zhoutg]",
            notes = "")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @Transactional
    public RespDTO<Boolean> saveOrUpdate(@RequestBody QuestionSaveVO questionSaveVO) {
        qcQuestionFacade.saveOrUpdate(questionSaveVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "标签分页列表[by:zhoutg]",
            notes = "// 备注名称\n" +
                    " String tagName;\n" +
                    "// 标签标识(1：单标签 4：组合标签)\n" +
                    " List<Integer> tagTypeList;\n" +
                    "// 显示名称\n" +
                    " String name;\n" +
                    "// 医院ID\n" +
                    " Long hospitalId;\n" +
                    "// 模块id\n" +
                    " Long modeId;")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<QuestionPageDTO>> list(@RequestBody QuestionPageVO questionPageVO) {
        IPage<QuestionPageDTO> data = qcQuestionFacade.getListFac(questionPageVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "标签检索[by:zhoutg]",
            notes = "tagName：系统名称<br>" +
                    "hospitalId：医院id<br>" +
                    "modeId：模块id<br>" +
                    "notIds：过滤的id值<br>" +
                    "notControlType：不能为的控件类型<br>" +
                    "controlType：指定的控件类型<br>" +
                    "tagType：指定的标签类型<br>")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<QcQuestionInfo>> index(@RequestBody QuestionIndexVO questionIndexVO) {
        List<QcQuestionInfo> data = qcQuestionFacade.indexFac(questionIndexVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "标签删除[by:zhoutg]",
            notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@RequestBody DeleteQuestionVO deleteQuestionVO) {
        qcQuestionFacade.deleteByIdsFac(deleteQuestionVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "根据参数返回标签内容[by:zhoutg]",
            notes = "id: questionId<br>" )
    @PostMapping("/getById")
    public RespDTO<QuestionDTO> getById(@RequestBody QuestionVO questionVO) {
        QuestionDTO data = qcQuestionFacade.getById(questionVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "查询多个id返回标签内容[by:zhoutg]",
            notes = "ids: 多个questionId<br>")
    @PostMapping("/getByIds")
    @SysLogger("getByIds")
    public RespDTO<Map<Long, QuestionDTO>> getByIds(@RequestBody QuestionIdsVO questionIdsVO) {
        Map<Long, QuestionDTO> data = qcQuestionFacade.getByIds(questionIdsVO);
        return RespDTO.onSuc(data);
    }
}
