package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.QcCasesEntrySimpleDTO;
import com.diagbot.dto.QcTypeDTO;
import com.diagbot.dto.QcTypePageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.QcQuestionInfo;
import com.diagbot.entity.QcType;
import com.diagbot.facade.QcTypeFacade;
import com.diagbot.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 质控类型标准 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2020-05-14
 */
@RestController
@RequestMapping("/qc/qcType")
@SuppressWarnings("unchecked")
@Api(value = "质控类型相关API", tags = { "质控类型相关API" })
public class QcTypeController {

    @Autowired
    QcTypeFacade qcTypeFacade;

    @ApiOperation(value = "新增或更新[by:zhoutg]",
            notes = "")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @Transactional
    public RespDTO saveOrUpdate(@RequestBody QcTypeSaveVO qcTypeSaveVO) {
        return qcTypeFacade.saveOrUpdate(qcTypeSaveVO);
    }

    @ApiOperation(value = "关联质控条目检索[by:cy]",
            notes =  "id: 质控类型<br>" +
                     "hospitalId: 医院id;")
    @PostMapping("/indexData")
    @SysLogger("indexData")
    public RespDTO<List<QcType>> indexData(@RequestBody QcTypeVO qcTypeVO) {

        return  qcTypeFacade.indexData(qcTypeVO);}

    @ApiOperation(value = "判断质控类型是否关联[by:cy]",
            notes = "id: 质控类型<br>" +
                    "hospitalId: 医院id<br>" +
                    "cancel: 是否解绑;")
    @PostMapping("/getOrCancel")
    @SysLogger("getOrCancel")
    @Transactional
    public RespDTO getOrCancel(@RequestBody CancelTypeVO cancelTypeVO) {

        return qcTypeFacade.getOrCancel(cancelTypeVO);
    }

    @ApiOperation(value = "分页列表[by:zhoutg]",
            notes = "    // 名称\n" +
                    "     String name;\n" +
                    "    // 医院id\n" +
                    "     Long hospitalId;")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<QcTypePageDTO>> list(@RequestBody QcTypePageVO qcTypePageVO) {
        IPage<QcTypePageDTO> data = qcTypeFacade.getListFac(qcTypePageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "新增时质控条目分组信息[by:zhoutg]",
            notes = "")
    @PostMapping("/getEntryByHospital")
    @SysLogger("getEntryByHospital")
    public RespDTO<Map<String, List<QcCasesEntrySimpleDTO>>> getEntryByHospital(@RequestBody QcEntryHospitalVO qcEntryHospitalVO) {
        Map<String, List<QcCasesEntrySimpleDTO>> data = qcTypeFacade.getEntryByHospital(qcEntryHospitalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "质控条目检索[by:zhoutg]",
            notes = "    // 界面输入\n" +
                    "     String input;\n" +
                    "    // 医院id\n" +
                    "     Long hospitalId;\n" +
                    "    // 过滤caseEntryId\n" +
                    "     List<Long> notIds;")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<QcQuestionInfo>> index(@RequestBody QcTypeIndexVO qcTypeIndexVO) {
        List<QcCasesEntrySimpleDTO> data = qcTypeFacade.indexFac(qcTypeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "删除[by:zhoutg]",
            notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO delete(@Valid @RequestBody DeleteQcTypeVO deleteQcTypeVO) {
        return  qcTypeFacade.deleteByIdsFac(deleteQcTypeVO);
    }

    @ApiOperation(value = "根据id返回内容[by:zhoutg]",
            notes = "id: qcTypeId<br>" )
    @PostMapping("/getById")
    @SysLogger("getById")
    public RespDTO<QcTypeDTO> getById(@RequestBody QcTypeVO qcTypeVO) {
        QcTypeDTO data = qcTypeFacade.getById(qcTypeVO);
        return RespDTO.onSuc(data);
    }
}
