package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.AnalyzeMessageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SthospitalInfoFindDTO;
import com.diagbot.dto.SthospitalInfoPageDTO;
import com.diagbot.dto.SthospitalInfoStatiDTO;
import com.diagbot.facade.StdBehospitalInfoFacade;
import com.diagbot.vo.SthospitalInfoClearVO;
import com.diagbot.vo.SthospitalInfoFindVO;
import com.diagbot.vo.SthospitalInfoPageVO;
import com.diagbot.vo.SthospitalInfoStatiVO;
import com.diagbot.vo.SthospitalInfoUpdataVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 标准住院病历信息 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-06-10
 */
@RestController
@RequestMapping("/stdBehospitalInfo")
@Api(value = "标准住院病历信息相关接口", tags = { "标准住院病历信息相关接口" })
public class StdBehospitalInfoController {
    @Autowired
    StdBehospitalInfoFacade stdBehospitalInfoFacade;

    @ApiOperation(value = "分页获取所有标准病例[by:wangfeng]",
            notes = "分页获取所有标准病例")
    @PostMapping("/getSthospitalInfoPage")
    @SysLogger("getSthospitalInfoPage")
    public RespDTO<IPage<SthospitalInfoPageDTO>> getSthospitalInfoPage(@RequestBody SthospitalInfoPageVO sthospitalInfoPageVO) {
        IPage<SthospitalInfoPageDTO> data = stdBehospitalInfoFacade.getSthospitalInfoPages(sthospitalInfoPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "更新该条标准病例[by:wangfeng]",
            notes = "更新该条标准病例")
    @PostMapping("/updataSthospitalInfo")
    @SysLogger("updataSthospitalInfo")
    @Transactional
    public RespDTO<Boolean> updataSthospitalInfo(@RequestBody @Valid SthospitalInfoUpdataVO sthospitalInfoUpdataVO) {
        Boolean res = stdBehospitalInfoFacade.updataSthospitalInfos(sthospitalInfoUpdataVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "查找病例[by:wangfeng]",
            notes = "查找病例")
    @PostMapping("/findSthospitalInfo")
    @SysLogger("findSthospitalInfo")
    @Transactional
    public RespDTO<SthospitalInfoFindDTO> findSthospitalInfo(@RequestBody @Valid SthospitalInfoFindVO sthospitalInfoFindVO) {
        SthospitalInfoFindDTO data = stdBehospitalInfoFacade.findSthospitalInfos(sthospitalInfoFindVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据病例号删除标准病例[by:wangfeng]",
            notes = "根据病例号删除标准病例")
    @PostMapping("/clearSthospitalInfo")
    @SysLogger("clearSthospitalInfo")
    @Transactional
    public RespDTO<Boolean> clearSthospitalInfo(@RequestBody @Valid SthospitalInfoClearVO sthospitalInfoClearVO) {
        Boolean res = stdBehospitalInfoFacade.clearSthospitalInfos(sthospitalInfoClearVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "自动化测试统计[by:wangfeng]",
            notes = "自动化测试统计")
    @PostMapping("/getStatistic")
    @SysLogger("getStatistic")
    @Transactional
    public RespDTO<IPage<SthospitalInfoStatiDTO>> getStatistic(@RequestBody @Valid SthospitalInfoStatiVO sthospitalInfoStatiVO) {
        IPage<SthospitalInfoStatiDTO> data = stdBehospitalInfoFacade.getStatistics(sthospitalInfoStatiVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "标准病例重新评分[by:wangfeng]",
            notes = "标准病例重新评分")
    @PostMapping("/analyze")
    @SysLogger("analyze")
    @Transactional
    public RespDTO<AnalyzeMessageDTO> analyze() {
        AnalyzeMessageDTO data = stdBehospitalInfoFacade.analyze();
        return RespDTO.onSuc(data);
    }
}
