package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StdMissionDetailPageDTO;
import com.diagbot.facade.StdMissionDetailFacade;
import com.diagbot.vo.StdMissionDetailClVO;
import com.diagbot.vo.StdMissionDetailPageVO;
import com.diagbot.vo.StdMissionDetailUpVO;
import com.diagbot.vo.StdMissionInfoAddVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-06-12
 */

@RestController
@RequestMapping("/stdMissionDetail")
@Api(value = "任务详情信息相关接口", tags = { "任务详情信息相关接口" })
public class StdMissionDetailController {
    @Autowired
    StdMissionDetailFacade stdMissionDetailFacade;

    @ApiOperation(value = "分页获取任务详情信息[by:wangfeng]",
            notes = "分页获取任务详情信息")
    @PostMapping("/getMissionDetailPage")
    @SysLogger("getMissionDetailPage")
    public RespDTO<IPage<StdMissionDetailPageDTO>> getMissionDetailPage(@RequestBody @Valid StdMissionDetailPageVO stdQcresultTaskPageVO) {
        IPage<StdMissionDetailPageDTO> data = stdMissionDetailFacade.getMissionDetailPages(stdQcresultTaskPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "一键添加任务信息[by:wangfeng]",
            notes = "一键添加任务信息")
    @PostMapping("/addMissionDetail")
    @SysLogger("addMissionDetail")
    @Transactional
    public RespDTO<Boolean> addMissionDetail(@RequestBody @Valid StdMissionInfoAddVO stdMissionInfoAddVO) {
        boolean res = stdMissionDetailFacade.addMissionDetails(stdMissionInfoAddVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "修改任务信息[by:wangfeng]",
            notes = "修改任务信息")
    @PostMapping("/updateMissionDetail")
    @SysLogger("updateMissionDetail")
    @Transactional
    public RespDTO<Boolean> updateMissionDetail(@RequestBody @Valid StdMissionDetailUpVO stdQcresultTaskUpVO) {
        boolean res = stdMissionDetailFacade.updateMissionDetails(stdQcresultTaskUpVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除任务信息[by:wangfeng]",
            notes = "删除任务信息")
    @PostMapping("/clearMissionDetail")
    @SysLogger("clearMissionDetail")
    public RespDTO<Boolean> clearMissionDetail(@RequestBody @Valid StdMissionDetailClVO stdMissionDetailClVO) {
        boolean res = stdMissionDetailFacade.clearMissionDetails(stdMissionDetailClVO);
        return RespDTO.onSuc(res);
    }
}
