package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StdMissionInfoPageDTO;
import com.diagbot.facade.StdMissionInfoFacade;
import com.diagbot.vo.StdMissionInfoClearVO;
import com.diagbot.vo.StdMissionInfoPageVO;
import com.diagbot.vo.StdMissionInfoUpVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-06-12
 */
@RestController
@RequestMapping("/stdMissionInfo")
@Api(value = "任务信息相关接口", tags = { "任务信息相关接口" })
public class StdMissionInfoController {
    @Autowired
    StdMissionInfoFacade stdMissionInfoFacade;

    @ApiOperation(value = "分页获取任务信息[by:wangfeng]",
            notes = "分页获取任务信息")
    @PostMapping("/getMissionInfoPage")
    @SysLogger("getMissionInfoPage")
    public RespDTO<IPage<StdMissionInfoPageDTO>> getMissionInfoPage(@RequestBody StdMissionInfoPageVO stdMissionInfoPageVO) {
        IPage<StdMissionInfoPageDTO> data = stdMissionInfoFacade.getMissionInfoPages(stdMissionInfoPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "修改获取任务信息[by:wangfeng]",
            notes = "修改获取任务信息")
    @PostMapping("/updataMissionInfo")
    @SysLogger("updataMissionInfo")
    @Transactional
    public RespDTO<Boolean> updataMissionInfo(@RequestBody @Valid StdMissionInfoUpVO stdMissionInfoUpVO) {
        boolean res = stdMissionInfoFacade.updataMissionInfos(stdMissionInfoUpVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除任务信息[by:wangfeng]",
            notes = "删除任务信息")
    @PostMapping("/clearMissionInfo")
    @SysLogger("clearMissionInfo")
    public RespDTO<Boolean> clearMissionInfo(@RequestBody @Valid StdMissionInfoClearVO stdMissionInfoClearVO) {
        boolean res = stdMissionInfoFacade.clearMissionInfos(stdMissionInfoClearVO);
        return RespDTO.onSuc(res);
    }

}
