package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.FileDTO;
import com.diagbot.entity.MrqcToken;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.UploadService;
import com.diagbot.service.impl.MrqcTokenServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RSAEncrypt;
import com.diagbot.vo.DecryptVO;
import com.diagbot.vo.MrqcTokenContentVO;
import com.diagbot.vo.MrqcTokenDeleteVO;
import com.diagbot.vo.MrqcTokenIndexVO;
import com.diagbot.vo.MrqcTokenPageVO;
import com.diagbot.vo.MrqcTokenVO;
import org.bouncycastle.jcajce.provider.asymmetric.rsa.RSAUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Component
public class MrqcTokenFacade extends MrqcTokenServiceImpl {

    @Autowired
    UploadService uploadService;

    /**
     * token保存
     *
     * @param mrqcTokenVO
     */
    public void saveOrUpdate(MrqcTokenVO mrqcTokenVO) {
        Date date = DateUtil.now();
        MrqcToken mrqcToken = new MrqcToken();
        BeanUtil.copyProperties(mrqcTokenVO, mrqcToken);
        List<MrqcToken> list = this.list(new QueryWrapper<MrqcToken>()
                .eq("ip", mrqcTokenVO.getIp())
                .eq("cpu", mrqcTokenVO.getCpu())
//                .eq("server", mrqcTokenVO.getServer())
//                .eq("hospital", mrqcTokenVO.getHospital())
                .eq("mainboard", mrqcTokenVO.getMainboard())
                .eq("disk", mrqcTokenVO.getDisk())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .ne(mrqcTokenVO.getId() != null, "id", mrqcTokenVO.getId())
        );
        if (ListUtil.isNotEmpty(list)) {
            StringBuffer sb = new StringBuffer();
            sb.append("当前loginKey已生成!<br>" );
            for (MrqcToken bean : list) {
                sb.append("服务商【").append(bean.getServer())
                    .append("】，医院【").append(bean.getHospital())
                    .append("】<br>");
            }
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, sb.toString());
        }
        // id为空表示新增，只有新增时才生成loginKey
        if (mrqcTokenVO.getId() == null) {
            mrqcToken.setGmtCreate(date);
            mrqcToken.setLoginKey(UUID.randomUUID().toString()); // 生成登录获取token需要的key
        }
        mrqcToken.setGmtModified(date);
        mrqcToken.setModifier(mrqcTokenVO.getCreator());

        // 创建文件
        FileDTO fileDTO = null;
        try {
            StringBuffer sb = new StringBuffer();
            sb.append("server.cpu=" + mrqcToken.getCpu()) // cpu信息
                .append("$$")
                .append("server.key=" + mrqcToken.getLoginKey())  // 加密key信息
                .append("$$")
                .append("server.remainday=" + mrqcToken.getRemainDay()) // 剩余天数
                .append("$$")
                .append("server.validtime=" + mrqcToken.getValidTime()) // 有效时间
                .append("$$")
                .append("server.hospital=" + mrqcToken.getHospital()) // 医院
                .append("$$")
                .append("server.provider=" + mrqcToken.getServer()); // 服务商
            MultipartFile file = fileToMultipartFile(
                    RSAEncrypt.encrypt(sb.toString()),
                    "license.properties");
            fileDTO = uploadService.singleFileUpload(file);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "文件上传失败！");
        }
        mrqcToken.setUrl("{imageUrlPrefix}" + fileDTO.getUrl());
        this.saveOrUpdate(mrqcToken);
    }


    /**
     *
     * @param content 文件内容
     * @param fileName
     * @return
     * @throws Exception
     */
    public MultipartFile  fileToMultipartFile(String content, String fileName) throws Exception {
        InputStream  inputstream = new ByteArrayInputStream(content.getBytes());
        MultipartFile toMultipartFile = new MockMultipartFile(fileName, fileName, "text/plain", inputstream);
        return toMultipartFile;
    }


    /**
     * token删除
     *
     * @param mrqcTokenVO
     */
    public void deleteById(MrqcTokenDeleteVO mrqcTokenVO) {
        MrqcToken mrqcToken = new MrqcToken();
        this.update(new MrqcToken(),
                new UpdateWrapper<MrqcToken>()
                    .eq("id", mrqcTokenVO.getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .set("modifier", mrqcTokenVO.getCreator())
                    .set("gmt_modified", DateUtil.now())
                    .set("is_deleted", IsDeleteEnum.Y.getKey())
        );
    }


    /**
     * token列表
     *
     * @param mrqcTokenPageVO
     * @return
     */
    public IPage<MrqcToken> getListFac(MrqcTokenPageVO mrqcTokenPageVO) {
        return this.getList(mrqcTokenPageVO);
    }


    /**
     * 获取明细
     * @param mrqcTokenIndexVO
     * @return
     */
    public MrqcToken getById(MrqcTokenIndexVO mrqcTokenIndexVO) {
        return this.getOne(new QueryWrapper<MrqcToken>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("id", mrqcTokenIndexVO.getId())
                , false
        );
    }


    /**
     * 解析内容
     *
     * @param mrqcTokenContentVO
     * @return
     */
    public Map analysis(MrqcTokenContentVO mrqcTokenContentVO) {
        Map<String, String> map = new HashMap<>();
        try {
            String content = mrqcTokenContentVO.getContent();
            String contentDe = RSAEncrypt.decrypt(content);
            String[] item = contentDe.split("\\$\\$"); // $$作为分隔符
            for (String s : item) {
                String[] keyVal = s.split("=");
                map.put(keyVal[0], keyVal[1]);
            }
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "内容格式有误，不能解析！");
        }
        return map;
    }


    public String decrypt(DecryptVO decryptVO) {
        String contentDe = "";
        try {
            contentDe = RSAEncrypt.decrypt(decryptVO.getContent());
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "内容格式有误，不能解析！");
        }
        return contentDe;
    }

}
