package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.MrqcToken;
import com.diagbot.facade.MrqcTokenFacade;
import com.diagbot.vo.DecryptVO;
import com.diagbot.vo.MrqcTokenContentVO;
import com.diagbot.vo.MrqcTokenDeleteVO;
import com.diagbot.vo.MrqcTokenIndexVO;
import com.diagbot.vo.MrqcTokenPageVO;
import com.diagbot.vo.MrqcTokenVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * <p>
 * 病历质控token信息 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2019-12-23
 */
@RestController
@RequestMapping("/mrqcToken")
@Api(value = "token相关接口", tags = { "token相关接口" })
public class MrqcTokenController {

    @Autowired
    private MrqcTokenFacade mrqcTokenFacade;

    @ApiOperation(value = "token新增或更新[by:zhoutg]",
            notes = "id： 主键，修改传值<br>" +
                    "creator：创建人(修改人)姓名<br>" +
                    "server：服务商<br>" +
                    "hospital：医院<br>" +
                    "cpu：cpu<br>" +
                    "mainboard： 主板<br>" +
                    "disk：硬盘<br>" +
                    "ip： ip<br>" +
                    "loginKey：登录Key<br>" +
                    "remark：备注<br>" +
                    "content：内容<br>" +
                    "remainDay：剩余使用天数<br>" +
                    "validTime：有效时间(分钟)")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @Transactional
    RespDTO<Boolean> saveOrUpdate(@RequestBody MrqcTokenVO mrqcTokenVO) {
        mrqcTokenFacade.saveOrUpdate(mrqcTokenVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "token删除[by:zhoutg]",
            notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@RequestBody MrqcTokenDeleteVO mrqcTokenDeleteVO) {
        mrqcTokenFacade.deleteById(mrqcTokenDeleteVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "token列表[by:zhoutg]",
            notes = "")
    @PostMapping("/list")
    @SysLogger("list")
    public RespDTO<IPage<MrqcToken>> list(@RequestBody MrqcTokenPageVO mrqcTokenPageVO) {
        IPage<MrqcToken> data = mrqcTokenFacade.getListFac(mrqcTokenPageVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "token获取明细[by:zhoutg]",
            notes = "")
    @PostMapping("/getById")
    @SysLogger("getById")
    public RespDTO<MrqcToken> index(@RequestBody MrqcTokenIndexVO mrqcTokenIndexVO) {
        MrqcToken data = mrqcTokenFacade.getById(mrqcTokenIndexVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "解析配置信息[by:zhoutg]",
            notes = "server.cpu：cpu<br>" +
                    "server.mainboard：主板<br>" +
                    "server.disk：硬盘<br>")
    @PostMapping("/analysis")
    @SysLogger("analysis")
    public RespDTO<Map<String, String>> analysis(@RequestBody MrqcTokenContentVO mrqcTokenContentVO) {
        Map<String, String> map = mrqcTokenFacade.analysis(mrqcTokenContentVO);
        return RespDTO.onSuc(map);
    }


    @ApiOperation(value = "解密[by:zhoutg]",
            notes = "")
    @PostMapping("/decrypt")
    @SysLogger("decrypt")
    public RespDTO<String> decrypt(@RequestBody DecryptVO decryptVO) {
        String data = mrqcTokenFacade.decrypt(decryptVO);
        return RespDTO.onSuc(data);
    }
}
