package com.diagbot.client;

import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.ConceptRetrievalExtDTO;
import com.diagbot.dto.DisclaimerInformationDTO;
import com.diagbot.dto.GetTopPatientInfoDjDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.UsualDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.DisclaimerInformationVO;
import com.diagbot.vo.GetTopPatientInfoDjVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.VersionVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:调用中间层服务
 * @Author: Mark Huang
 * @time: 2019/03/25 19:15
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {
    /**
     * 推理
     *
     * @param searchVO
     * @return
     */
    @PostMapping("/push/pushInner")
    RespDTO<PushDTO> pushInner(@RequestBody SearchVO searchVO);

    /**
     * 对接-获取页面顶部病人医生科室信息
     *
     * @param getTopPatientInfoDjVO
     * @return
     */
    @PostMapping("/patientInfo/getTopPatientInfoDj")
    RespDTO<GetTopPatientInfoDjDTO> getTopPatientInfoDj(@RequestBody GetTopPatientInfoDjVO getTopPatientInfoDjVO);

    /**
     * 常见症状
     *
     * @param conceptUsualVO
     * @return
     */
    @PostMapping(value = "/concept/getConceptUsual")
    RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    /**
     * 获取常用标签附带通俗解释和详细说明
     *
     * @param conceptUsualVO
     * @return
     */
    @PostMapping(value = "/concept/getConceptExtUsual")
    RespDTO<List<UsualDTO>> getConceptExtUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    /**
     * 标签检索(附带通俗解释和详细说明, retrivelConceptInfo这个作废)
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfoExt")
    RespDTO<List<ConceptRetrievalExtDTO>> retrivelConceptInfoExt(@RequestBody @Valid RetrievalVO retrievalVO);


    /**
     * 标签检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfo")
    RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@RequestBody @Valid RetrievalVO retrievalVO);

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @PostMapping("/concept/getConceptMap")
    RespDTO<Map<Long, String>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO);


    /**
     * 获取版本信息
     */
    @PostMapping("/versionInfo/getVersionInfoAlls")
    RespDTO<VersionWrapperDTO> getVersionInfoAlls(@RequestBody VersionVO versionVO);

    @PostMapping("/disclaimerInformation/getDisclaimerInformations")
    RespDTO<List<DisclaimerInformationDTO>> getDisclaimerInformations(@RequestBody DisclaimerInformationVO disclaimerInformationVO);
}