package com.diagbot.client;

import com.diagbot.client.hystrix.TranServiceHystrix;
import com.diagbot.dto.GetHospitalRankInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SignInDTO;
import com.diagbot.dto.SysHospitalDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.vo.GetHospitalRankInfoVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.InquiryQuoteVO;
import com.diagbot.vo.SignInVO;
import com.diagbot.vo.SysHospitalCodeVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 调用信息对接层服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "tran-service", fallback = TranServiceHystrix.class)
public interface TranServiceClient {
    /**
     * @param hospitalSetVO
     * @return
     */
    @PostMapping("/sysSet/getSysSetInfoDatas")
    RespDTO<List<SysSetInfoDTO>> getSysSetInfoDatas(@Valid @RequestBody HospitalSetVO hospitalSetVO);
    
    /**
     * @param signInVO
     * @return
     */
    @PostMapping("/patientInfo/signIn")
    RespDTO<List<SignInDTO>> signIn(@RequestBody SignInVO signInVO);

    /**
     * @param inquiryQuoteVO
     * @return
     */
    @PostMapping("/patientInfo/inquiryQuote")
    RespDTO<InquiryQuoteVO> inquiryQuote(@RequestBody InquiryQuoteVO inquiryQuoteVO);

    /**
     *
     * @param sysHospitalCodeVO
     */
    @PostMapping("/sysSet/getHospitalDatas")
    RespDTO<SysHospitalDTO> getHospitalDeptInfoData(SysHospitalCodeVO sysHospitalCodeVO);

    @PostMapping("/sysSet/getHospitalRankInfo")
    RespDTO<GetHospitalRankInfoDTO> getHospitalRankInfo(@RequestBody GetHospitalRankInfoVO getHospitalRankInfoVO);

    @PostMapping("/sysSet/getParentHospitalCode")
    RespDTO<String> getParentHospitalCode(@RequestBody String hospitalCode);
}