package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.ConceptPushDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.PushVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/6/14 11:08
 */
@Component
public class PushFacade {
    @Autowired
    AiptServiceClient aiptServiceClient;
    @Autowired
    AssembleFacade assembleFacade;
    @Autowired
    QuestionFacade questionFacade;

    /**
     * 推理接口
     *
     * @param pushVO
     * @return
     */
    public PushDTO push(PushVO pushVO) {
        PushDTO pushDTO = new PushDTO();
        SearchVO searchVO = assembleFacade.assembleData(pushVO);
        RespDTO<PushDTO> respDTO = aiptServiceClient.pushInner(searchVO);
        RespDTOUtil.respNGDeal(respDTO, "中间层没有返回结果");
        PushDTO data = respDTO.data;

        String featureType = searchVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        //症状
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Feature_Type_Symptom.getKey()))) {
            List<ConceptPushDTO> symptom = data.getSymptom();
            if (ListUtil.isNotEmpty(symptom)) {
                //推理名称=系统名称=术语名称
                for (ConceptPushDTO symp : symptom) {
                    symp.setTagName(symp.getName());
                }
                List<ConceptPushDTO> symptomDTO = addQuestionId(symptom, 1);
                pushDTO.setSymptom(symptomDTO);
            }
        }
        return pushDTO;
    }

    /**
     * 概念关联标签，添加标签id ——症状
     *
     * @param concepts
     * @param type
     * @return
     */
    public List<ConceptPushDTO> addQuestionId(List<ConceptPushDTO> concepts, Integer type) {
        List<String> nameList = concepts
                .stream()
                .map(concept -> concept.getTagName())
                .collect(Collectors.toList());
        if (ListUtil.isNotEmpty(nameList)) {
            QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
            questionInfoQueryWrapper
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("tag_name", nameList)
                    .eq("type", type);
            List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);
            if (ListUtil.isNotEmpty(questionInfoList)) {
                Map<String, QuestionInfo> questionInfoMap
                        = EntityUtil.makeEntityMap(questionInfoList, "tagName");
                for (ConceptPushDTO concept : concepts) {
                    concept.setType(1);
                    if (questionInfoMap.containsKey(concept.getName())) {
                        QuestionInfo questionInfo = questionInfoMap.get(concept.getTagName());
                        concept.setId(questionInfo.getId());
                        concept.setName(questionInfo.getName());
                    }
                }
            }
        }
        return concepts;
    }
}