package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.ConceptRetrievalExtDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RetrievalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: wangyu
 * @time: 2018/11/27 14:54
 */
@Component
public class RetrievalFacade {

    @Autowired
    QuestionFacade questionFacade;
    @Autowired
    AiptServiceClient aiptServiceClient;

    /**
     * 获取标签信息
     *
     * @param retrievalVO
     * @return
     */
    public List<RetrievalDTO> getTagInfos(RetrievalVO retrievalVO) {
        retrievalVO.setType(QuestionTypeEnum.Symptom.getKey());
        List<RetrievalDTO> retrievalDTOS = new ArrayList<>();
        //调用aipt-服务
        RespDTO<List<ConceptRetrievalExtDTO>> conceptInfos = aiptServiceClient.retrivelConceptInfoExt(retrievalVO);
        RespDTOUtil.respNGDeal(conceptInfos, "获取检索信息失败");
        //获取questionId
        List<String> questionNames = new ArrayList<>();
        if (ListUtil.isNotEmpty(conceptInfos.data)) {
            for (ConceptRetrievalExtDTO conceptRetrievalDTO : conceptInfos.data) {
                if (!questionNames.contains(conceptRetrievalDTO.getSelfName())
                        && conceptRetrievalDTO.getSelfName() != null) {
                    questionNames.add(conceptRetrievalDTO.getSelfName());
                }
            }
            QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
            questionInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("tag_name", questionNames)
                    .eq("type", retrievalVO.getType())
                    .ne("tag_type", 8)
                    .eq("item_type", 0);
            List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);
            Map<String, QuestionInfo> questionInfoMap = questionInfoList.stream()
                    .collect(Collectors.toMap(QuestionInfo::getTagName, questionInfo -> questionInfo));
            RetrievalDTO retrievalDTO = new RetrievalDTO();
            //封装
            for (ConceptRetrievalExtDTO conceptRetrievalDTO : conceptInfos.data) {
                retrievalDTO = new RetrievalDTO();
                retrievalDTO.setConceptId(conceptRetrievalDTO.getSelfId());
                retrievalDTO.setTagName(conceptRetrievalDTO.getSelfName());
                if (null != questionInfoMap.get(conceptRetrievalDTO.getSelfName())) {//如果匹配到就添加questionId
                    retrievalDTO.setName(questionInfoMap.get(conceptRetrievalDTO.getSelfName()).getName());
                    retrievalDTO.setQuestionId(questionInfoMap.get(conceptRetrievalDTO.getSelfName()).getId());
                    retrievalDTO.setDescription(conceptRetrievalDTO.getDescription());
                    if (StringUtil.isNotBlank(conceptRetrievalDTO.getExplains())){
                        retrievalDTO.setHasExplain(1);
                        retrievalDTO.setExplains(conceptRetrievalDTO.getExplains());
                    }
                }
                if (StringUtil.isNotEmpty(conceptRetrievalDTO.getSameName())) {//同义词
                    retrievalDTO.setRetrievalName(conceptRetrievalDTO.getSameName());
                }
                retrievalDTO.setLibTypeId(conceptRetrievalDTO.getLibTypeId());
                retrievalDTO.setLibTypeName(conceptRetrievalDTO.getLibTypeName());
                retrievalDTO.setType(conceptRetrievalDTO.getType());
                retrievalDTO.setShowType(conceptRetrievalDTO.getShowType());
                retrievalDTOS.add(retrievalDTO);
            }
        }
        //如何没有匹配到questionId就移除
        Iterator<RetrievalDTO> retrievalDTOIterator = retrievalDTOS.iterator();
        while (retrievalDTOIterator.hasNext()) {
            RetrievalDTO retrievalDTO = retrievalDTOIterator.next();
            if (retrievalDTO.getQuestionId() == null) {
                retrievalDTOIterator.remove();
            }
        }
        return retrievalDTOS;
    }
}
