package com.diagbot.facade;

import com.diagbot.client.TranServiceClient;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysHospitalDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.SysHospitalCodeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author wangfeng
 * @Description: TODO
 * @date 2019年6月18日 上午10:24:58
 */
@Component
public class SysSetFacade {

    @Autowired
    TranServiceClient tranServiceClient;

    /**
     * 根据医院code获取配置信息
     *
     * @return
     */
    public List<SysSetInfoDTO> getSysSetInfoData(HospitalSetVO hospitalSetVO) {
        RespDTO<List<SysSetInfoDTO>> sysSetInfoLists = tranServiceClient.getSysSetInfoDatas(hospitalSetVO);
        RespDTOUtil.respNGDeal(sysSetInfoLists, "获取配置数据失败！");
        return sysSetInfoLists.data;
    }

    /**
     * @param sysHospitalCodeVO
     * @return
     */
    public SysHospitalDTO getHospitalDeptInfoData(SysHospitalCodeVO sysHospitalCodeVO) {
        RespDTO<SysHospitalDTO> sysHospitalDTO = tranServiceClient.getHospitalDeptInfoData(sysHospitalCodeVO);
        RespDTOUtil.respNGDeal(sysHospitalDTO, "医院编码获取医院信息及科室信息失败！");
        return sysHospitalDTO.data;
    }

    /**
     * 是否显示引用过的预问诊
     *
     * @param hospitalCode
     * @return
     */
    public Boolean isQuoteShow(String hospitalCode) {
        HospitalSetVO hospitalSetVO = new HospitalSetVO();
        hospitalSetVO.setHospitalCode(hospitalCode);
        hospitalSetVO.setCode("quote_show");

        RespDTO<List<SysSetInfoDTO>> retData = tranServiceClient.getSysSetInfoDatas(hospitalSetVO);
        RespDTOUtil.respNGDeal(retData, "获取配置数据失败！");

        if (ListUtil.isEmpty(retData.data)) {
            return false;
        }

        if (retData.data.size() > 1) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取配置数据失败！");
        }

        return StringUtil.isNotBlank(retData.data.get(0).getValue()) && retData.data.get(0).getValue().equals("1");
    }

}
