package com.diagbot.vo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import java.util.Date;

/**
 * @Description:
 * @author: rengb
 * @time: 2018/11/19 18:58
 */
@ApiModel(value = "病历引用接口传参-患者信息")
@Getter
@Setter
public class InquiryQuotePatientVO {

    /**
     * id
     */
    private Long id;

    /**
     * 患者编号
     */
    @ApiModelProperty(value = "患者编号", required = true)
    @NotBlank(message = "患者编号必传")
    private String code;

    /**
     * 患者姓名
     */
    @ApiModelProperty(value = "患者姓名", required = true)
    @NotBlank(message = "患者姓名必传")
    private String name;

    /**
     * 性别，0-未知、1-男、2-女
     */
    @ApiModelProperty(value = "性别，0-未知、1-男、2-女")
    private Integer sex;

    /**
     * 出生日期，格式为：2018-11-28 17:25:30
     */
    @ApiModelProperty(value = "出生日期，格式为：2018-11-28 17:25:30")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date birthday;

    /**
     * 证件类型
     */
    @ApiModelProperty(value = "证件类型")
    private String idType;

    /**
     * 证件号码
     */
    @ApiModelProperty(value = "证件号码", required = true)
    @NotBlank(message = "证件号码必传")
    private String idNo;

    /**
     * 家庭住址
     */
    @ApiModelProperty(value = "家庭住址")
    private String address;

    /**
     * 联系电话
     */
    @ApiModelProperty(value = "联系电话")
    private String phone;

    /**
     * 身份证号
     */
    @ApiModelProperty(value = "身份证号", required = true)
    @NotBlank(message = "身份证号必传")
    private String identityNum;

    /**
     * 家庭邮编
     */
    @ApiModelProperty(value = "家庭邮编")
    private String postcode;

    /**
     * 联系人
     */
    @ApiModelProperty(value = "联系人")
    private String contacts;

    /**
     * 联系人电话
     */
    @ApiModelProperty(value = "联系人电话")
    private String contactPhone;

    /**
     * 就职单位名称
     */
    @ApiModelProperty(value = "就职单位名称")
    private String workUnit;

    /**
     * 职业
     */
    @ApiModelProperty(value = "职业")
    private String operation;

    /**
     * 国籍
     */
    @ApiModelProperty(value = "国籍")
    private String country;

    /**
     * 民族
     */
    @ApiModelProperty(value = "民族")
    private String nationality;

    /**
     * 婚姻状况：0未婚，1已婚，2未知
     */
    @ApiModelProperty(value = "婚姻状况：0未婚，1已婚，2未知")
    private Integer matrimony;

}