package com.diagbot.vo;

import java.util.Date;
import java.util.List;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

import org.hibernate.validator.constraints.Range;
import org.springframework.format.annotation.DateTimeFormat;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * @Description:
 * @author: rengb
 * @time: 2018/11/19 18:58
 */
@ApiModel(value="问诊记录保存接口传参")
@Getter
@Setter
public class SaveInquiryVO {
	
	/**
     * 医院id
     */
	@ApiModelProperty(value="医院id",required=true)
	@NotNull(message="医院id必传")
    private Long hospitalId;
	
	/**
     * 医院编码
     */
	@ApiModelProperty(value="医院编码",required=true)
	@NotBlank(message="医院编码必传")
    private String hospitalCode;

    /**
     * 医院名称
     */
	@ApiModelProperty(value="医院名称",required=true)
	@NotBlank(message="医院名称必传")
    private String hospitalName;

	/**
	 * 子医院id
	 */
	@ApiModelProperty(value="子医院id")
	private Long sonHospitalId;

	/**
	 * 子医院编码
	 */
	@ApiModelProperty(value="子医院编码")
	private String sonHospitalCode;

	/**
	 * 子医院名称
	 */
	@ApiModelProperty(value="子医院名称")
	private String sonHospitalName;

    /**
     * 医院科室id
     */
	@ApiModelProperty(value="科室id",required=true)
	@NotNull(message="科室id必传")
    private Long hospitalDeptId;
	
	/**
     * 医院科室编码
     */
	@ApiModelProperty(value="医院科室编码",required=true)
	@NotBlank(message="医院科室编码必传")
    private String hospitalDeptCode;

    /**
     * 医院科室名称
     */
	@ApiModelProperty(value="医院科室名称",required=true)
	@NotBlank(message="医院科室名称必传")
    private String hospitalDeptName;

    /**
     * 医生id
     */
	@ApiModelProperty(value="医生id")
    private Long doctorId;
	
	/**
     * 医生编码
     */
	@ApiModelProperty(value="医生编码")
    private String doctorCode;

    /**
     * 医生姓名
     */
	@ApiModelProperty(value="医生姓名")
    private String doctorName;

    /**
     * 患者id
     */
	@ApiModelProperty(value="患者id",required=true)
	@NotNull(message="患者id必传")
    private Long patientId;
	
	/**
     * 患者编号
     */
	@ApiModelProperty(value="患者编号",required=true)
	@NotBlank(message="患者编号必传")
    private String patientCode;

    /**
     * 患者姓名
     */
	@ApiModelProperty(value="患者姓名",required=true)
	@NotBlank(message="患者姓名必传")
    private String patientName;

    /**
     * 患者性别：1男2女
     */
	@ApiModelProperty(value="患者性别：1男2女")
    private Integer patientSex;

    /**
     * 患者联系电话
     */
	@ApiModelProperty(value="患者联系电话")
    private String patientPhone;

    /**
     * 患者出生日期
     */
	@ApiModelProperty(value="患者出生日期，格式为：2018-11-28 17:25",required=true)
	@DateTimeFormat(pattern="yyyy-MM-dd HH:mm")
	@NotNull(message="患者出生日期必传")
    private Date patientBirthday;
    
    /**
     * 患者证件号码(病历号)
     */
	@ApiModelProperty(value="患者证件号码(病历号)",required=true)
	@NotBlank(message="患者证件号码(病历号)必传")
    private String patientIdNo;

    /**
     * 就诊序列号
     */
	@ApiModelProperty(value="就诊序列号")
    private String inquiryCode;

    /**
     * 就诊状态(0待接诊，1接诊中，2完成接诊)
     */
	@ApiModelProperty(value="就诊状态(0待接诊，1接诊中，2完成接诊)")
	@Range(min=0,max=2,message="就诊状态必须是0、1、2")
	private Integer regVisitedState=1;

    /**
     * 病历分类(1:门诊,2:住院)
     */
	@ApiModelProperty(value="病历分类(1:门诊,2:住院)")
	@Range(min=1,max=2,message="病历分类必须是1、2")
    private Integer type=1;

    /**
     * 内容JSON字符串
     */
	@ApiModelProperty(value="内容JSON字符串",required=true)
    @NotBlank(message="内容JSON字符串必传")
    private String dataJson;
    
	/**
	 * 问诊明细
	 */
    @Valid
    @ApiModelProperty(value="问诊明细",required=true)
    @NotEmpty(message="问诊明细不能为空")
    private List<SaveInquiryDetailVO> detailList;
    
	/**
	 * 治疗报告图片集合
	 */
    @ApiModelProperty(value="治疗报告图片集合")
    private List<SaveInquiryReportVO> reportList;
    
}
