package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.EvaluatorCalDTO;
import com.diagbot.dto.GetEvaluatorPageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.InquiryEvaluatorFacade;
import com.diagbot.vo.AddEvaluatorVO;
import com.diagbot.vo.EvaluatorCalVO;
import com.diagbot.vo.GetEvaluatorPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

@RestController
@RequestMapping("/inquiryEvaluator")
@SuppressWarnings("unchecked")
@Api(value = "问诊-儿童体质辨识API", tags = { "智能预问诊-儿童体质辨识API" })
public class InquiryEvaluatorController {

    @Autowired
    private InquiryEvaluatorFacade inquiryEvaluatorFacade;

    @ApiOperation(value = "儿童体质辨识--添加")
    @PostMapping("/addEvaluator")
    @SysLogger("addEvaluator")
    @Transactional
    public RespDTO<Boolean> addEvaluator(@Valid @RequestBody AddEvaluatorVO addEvaluatorVO) {
        return RespDTO.onSuc(inquiryEvaluatorFacade.addEvaluator(addEvaluatorVO));
    }

    @ApiOperation(value = "儿童体质辨识--体质计算")
    @PostMapping("/evaluatorCal")
    @SysLogger("evaluatorCal")
    public RespDTO<EvaluatorCalDTO> evaluatorCal(@Valid @RequestBody EvaluatorCalVO evaluatorCalVO) {
        return RespDTO.onSuc(inquiryEvaluatorFacade.evaluatorCal(evaluatorCalVO));
    }

    @ApiOperation(value = "儿童体质辨识--获取分页信息")
    @PostMapping("/getEvaluatorPage")
    @SysLogger("getEvaluatorPage")
    public RespDTO<IPage<GetEvaluatorPageDTO>> getEvaluatorPage(@Valid @RequestBody GetEvaluatorPageVO getEvaluatorPageVO) {
        return RespDTO.onSuc(inquiryEvaluatorFacade.getEvaluatorPage(getEvaluatorPageVO));
    }

}
