package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetInquiryDetailDTO;
import com.diagbot.dto.HisInquiryDTO;
import com.diagbot.dto.RecordCheckDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SaveInquiryDTO;
import com.diagbot.facade.InquiryInfoFacade;
import com.diagbot.vo.GetInquiryDetailVO;
import com.diagbot.vo.HisInquirysVO;
import com.diagbot.vo.InquiryQuoteForIcssVO;
import com.diagbot.vo.InquiryQuoteVO;
import com.diagbot.vo.RecordCheckVO;
import com.diagbot.vo.SaveInquiryVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @author rengb
 * @since 2018-11-23
 */
@RestController
@RequestMapping("/inquiryInfo")
@Api(value = "问诊记录API", tags = { "智能预问诊-问诊记录API" })
@SuppressWarnings("unchecked")
public class InquiryInfoController {

    @Autowired
    private InquiryInfoFacade inquiryInfoFacade;

    @ApiOperation(value = "问诊记录保存[by:rengb]")
    @PostMapping("/saveInquiry")
    @SysLogger("saveInquiry")
    @Transactional
    public RespDTO<SaveInquiryDTO> saveInquiry(@Valid @RequestBody SaveInquiryVO saveInquiryVO) {
        return RespDTO.onSuc(inquiryInfoFacade.saveInquiry(saveInquiryVO));
    }

    @ApiOperation(value = "问诊记录检查确认[by:rengb]")
    @PostMapping("/recordCheck")
    @SysLogger("recordCheck")
    public RespDTO<RecordCheckDTO> recordCheck(@Valid @RequestBody RecordCheckVO recordCheckVO) {
        return RespDTO.onSuc(inquiryInfoFacade.recordCheck(recordCheckVO));
    }

    @ApiOperation(value = "历史病历列表(医院对接用)[by:rengb]")
    @PostMapping("/hisInquirys")
    @SysLogger("hisInquirys")
    public RespDTO<List<HisInquiryDTO>> hisInquirys(@Valid @RequestBody HisInquirysVO hisInquirysVO) {
        return RespDTO.onSuc(inquiryInfoFacade.hisInquirys(hisInquirysVO));
    }

    @ApiOperation(value = "历史病历列表(ICSS调用)[by:rengb]")
    @PostMapping("/hisInquirysForIcss")
    @SysLogger("hisInquirysForIcss")
    public RespDTO<List<HisInquiryDTO>> hisInquirysForIcss(@Valid @RequestBody HisInquirysVO hisInquirysVO) {
        return RespDTO.onSuc(inquiryInfoFacade.hisInquirysForIcss(hisInquirysVO));
    }

    @ApiOperation(value = "病历详情(医院对接用)[by:rengb]")
    @PostMapping("/getInquiryDetail")
    @SysLogger("getInquiryDetail")
    public RespDTO<GetInquiryDetailDTO> getInquiryDetail(@Valid @RequestBody GetInquiryDetailVO getInquiryDetailVO) {
        return RespDTO.onSuc(inquiryInfoFacade.getInquiryDetail(getInquiryDetailVO));
    }

    @ApiOperation(value = "病历引用(医院对接用)[by:rengb]")
    @PostMapping("/inquiryQuote")
    @SysLogger("inquiryQuote")
    @Transactional
    public RespDTO<Boolean> inquiryQuote(@Valid @RequestBody InquiryQuoteVO inquiryQuoteVO) {
        return RespDTO.onSuc(inquiryInfoFacade.inquiryQuote(inquiryQuoteVO));
    }

    @ApiOperation(value = "icss引用预问诊(ICSS调用)[by:rengb]")
    @PostMapping("/inquiryQuoteForIcss")
    @SysLogger("inquiryQuoteForIcss")
    @Transactional
    public RespDTO<Boolean> inquiryQuoteForIcss(@Valid @RequestBody InquiryQuoteForIcssVO inquiryQuoteForIcssVO) {
        return RespDTO.onSuc(inquiryInfoFacade.inquiryQuoteForIcss(inquiryQuoteForIcssVO));
    }


}
