package com.diagbot.web;

import java.util.List;

import javax.validation.Valid;

import com.diagbot.dto.GetTopPatientInfoDjDTO;
import com.diagbot.vo.GetTopPatientInfoDjVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SignInDTO;
import com.diagbot.facade.PatientInfoFacade;
import com.diagbot.vo.SignInVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * 患者信息表 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2018-11-19
 */
@RestController
@RequestMapping("/patientInfo")
@Api(value = "患者信息API", tags = { "智能预问诊-患者信息API" })
@SuppressWarnings("unchecked")
public class PatientInfoController {

    @Autowired
    private PatientInfoFacade patientInfoFacade;

    @ApiOperation(value = "智能预问诊-页面顶部病人医生科室信息——查询[by:rengb]",
            notes = "hospitalCode：医院编号，必填<br>" +
                    "hospitalDeptCode：医院科室编号，必填<br>" +
                    "doctorCode：医院医生编号，必填<br>" +
                    "patientCode:医院患者编号，必填<br>")
    @PostMapping("/getTopPatientInfo")
    @SysLogger("getTopPatientInfo")
    public RespDTO<GetTopPatientInfoDjDTO> getTopPatientInfo(@Valid @RequestBody GetTopPatientInfoDjVO getTopPatientInfoDjVO) {
        return RespDTO.onSuc(patientInfoFacade.getTopPatientInfo(getTopPatientInfoDjVO));
    }
    
    @ApiOperation(value = "患者登录[by:rengb]")
    @PostMapping("/signIn")
    @SysLogger("signIn")
    public RespDTO<List<SignInDTO>> signIn(@Valid @RequestBody SignInVO signInVO) {
        return patientInfoFacade.signIn(signInVO);
    }

}
