package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.biz.push.enums.FeatureTypeEnum;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.PushFacade;
import com.diagbot.vo.PushVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description:推理接口
 * @Author:zhaops
 * @time: 2019/6/14 13:52
 */
@RestController
@RequestMapping("/push")
@Api(value = "推理相关API", tags = { "智能预问诊-推理相关API" })
@SuppressWarnings("unchecked")
public class PushController {
    @Autowired
    private PushFacade pushFacade;

    @ApiOperation(value = "智能预问诊-症状推理[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别(必填)，1：男，2：女<br>" +
                    "symptom：症状,String<br>" +
                    "hosCode：医院编码,String<br>")
    @PostMapping("/pushInner")
    @SysLogger("pushInner")
    public RespDTO<PushDTO> pushSymptom(@RequestBody @Valid PushVO pushVO) {
        pushVO.setFeatureType(String.valueOf(FeatureTypeEnum.Feature_Type_Symptom.getKey()));
        return RespDTO.onSuc(pushFacade.push(pushVO));
    }
}