package com.diagbot.web;

import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.QuestionFacade;
import com.diagbot.vo.QuestionIds2VO;
import com.diagbot.vo.QuestionNameVO;
import com.diagbot.vo.QuestionNamesVO;
import com.diagbot.vo.QuestionVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * <p>
 * 标签基础表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2018-11-15
 */
@RequestMapping("/questionInfo")
@RestController
@Api(value = "标签相关API", tags = { "智能预问诊-标签相关API" })
@SuppressWarnings("unchecked")
public class QuestionInfoController {

    @Autowired
    QuestionFacade questionFacade;

    @ApiOperation(value = "智能预问诊-根据参数返回标签内容[by:zhoutg]",
            notes = "id: questionId<br>" +
                    "sexType：性别，1：男，2：女<br>" +
                    "age：年龄")
    @PostMapping("/getById")
    public RespDTO<QuestionDTO> getById(@RequestBody QuestionVO questionVO) {
        QuestionDTO data = questionFacade.getById(questionVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "智能预问诊-根据参数返回标签内容[by:zhaops]",
            notes = "name: 标签名称<br>" +
                    "sexType：性别，1：男，2：女<br>" +
                    "type：标签类型，1：症状 3：其他史 51：诊疗情况 52：补充 <br>" +
                    "age：年龄")
    @PostMapping("/getByName")
    public RespDTO<QuestionDTO> getByName(@RequestBody QuestionNameVO questionNameVO) {
        QuestionDTO data = questionFacade.getByName(questionNameVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "智能预问诊-根据参数返回标签内容[by:zhaops]",
            notes = "name: 标签名称<br>" +
                    "sexType：性别，1：男，2：女<br>" +
                    "type：标签类型，1：症状 3：其他史 51：诊疗情况 52：补充 <br>" +
                    "age：年龄")
    @PostMapping("/getByNames")
    public RespDTO<Map<String, Object>> getByNames(@RequestBody QuestionNamesVO questionNamesVO) {
        Map<String, Object> data = questionFacade.getByNames(questionNamesVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "智能预问诊-查询多个id优化[by:zhoutg]",
            notes = "ids: 多个questionId<br>" +
                    "sexType：性别，1：男，2：女<br>" +
                    "age：年龄")
    @PostMapping("/getByIds")
    public RespDTO<Map<Long, Object>> getByIds(@RequestBody QuestionIds2VO questionIds2VO) {
        Map<Long, Object> data = questionFacade.getByIds(questionIds2VO);
        return RespDTO.onSuc(data);
    }
}
