package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysHospitalDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.facade.SysSetFacade;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.SysHospitalCodeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 医院所有配置信息 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2019-06-18
 */
@RestController
@RequestMapping("/sysSet")
@Api(value = "配置信息API", tags = { "智能预问诊-配置信息API" })
@SuppressWarnings("unchecked")
public class SysSetController {

    @Autowired
    SysSetFacade sysSetFacade;

    @ApiOperation(value = "智能预问诊-根据医院编码获取配置信息[by:wangfeng]", notes = "hospitalCode ：医院code  必填<br> ")
    @PostMapping("/getSysSetInfoDatas")
    @SysLogger("getSysSetInfoDatas")
    public RespDTO<List<SysSetInfoDTO>> getSysSetInfoDatas(@Valid @RequestBody HospitalSetVO hospitalSetVO) {
        List<SysSetInfoDTO> data = sysSetFacade.getSysSetInfoData(hospitalSetVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院编码获取医院信息及科室信息[by:wangfeng]", notes = "hospitalCode ：医院code  必填")
    @PostMapping("/getHospitalDatas")
    @SysLogger("getHospitalDatas")
    public RespDTO<SysHospitalDTO> getHospitalDatas(@Valid @RequestBody SysHospitalCodeVO sysHospitalCodeVO) {
        SysHospitalDTO data = sysSetFacade.getHospitalDeptInfoData(sysHospitalCodeVO);
        return RespDTO.onSuc(data);
    }
}
