package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllDeptAndDisInfoDTO;
import com.diagbot.dto.GetModuleInfoOneDTO;
import com.diagbot.dto.GetModuleTypeDTO;
import com.diagbot.dto.ModuleInfoListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ModuleInfoFacade;
import com.diagbot.vo.AddModuleInfoVO;
import com.diagbot.vo.DeleteModuleVO;
import com.diagbot.vo.GetAllDeptAndDisInfoVO;
import com.diagbot.vo.GetModuleInfoOneVO;
import com.diagbot.vo.GetModuleInfoVO;
import com.diagbot.vo.GetModuleTypeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 模型表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2018-12-05
 */
@RestController
@RequestMapping("/moduleInfo_prec")
@SuppressWarnings("unchecked")
@Api(value = "主诉-既往史模板相关API", tags = { "模板相关API" })
public class ModuleInfoController {


    @Autowired
    ModuleInfoFacade moduleInfoFacade;

    @ApiOperation(value = "模板新增或修改[by:wangyu]",
            notes = "id，修改时传，新增不用传<br>" +
                    "name: 模板名称，必填<br>" +
                    "type: 模板类型（症状，诊疗情况，其他），必填<br>" +
                    "questionId: 标签id，不是标点符号时需要填写<br>" +
                    "flag: 特殊标识类型<br>" +
                    "relationModule: 关联模板id<br>" +
                    "text: 特殊标签（如逗号句号）<br>" +
                    "textType: 特殊标签归属类型<br>" +
                    "moduleType: 模板类型（科室，通用），必填<br>" +
                    "relationId: 关联id，moduleType为0不用传，moduleType为1传科室id，moduleType为2传疾病id<br>" +
                    "remark：备注")
    @PostMapping("/saveOrUpdateModuleInfo")
    @SysLogger("saveOrUpdateModuleInfo")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateModuleInfo(@Valid @RequestBody AddModuleInfoVO addModuleInfoVO) {
        Boolean data = moduleInfoFacade.saveOrUpdateModuleInfo(addModuleInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "模板删除[by:zhoutg]",
            notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@RequestBody DeleteModuleVO deleteModuleVO) {
        moduleInfoFacade.deleteByIdsFac(deleteModuleVO.getIds());
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "获取模板列表（主页面查询）[by:wangyu]",
            notes = "type: 模板类型,1.症状情况 51.诊疗情况 3.其他病史 52.补充内容<br>" +
                    "moduleType: 模板类型,0.通用 1.根据科室划分 2.慢病<br>" +
                    "relationId: 关联id<br>" +
                    "name：模板名称")
    @PostMapping("/getModuleInfoList")
    @SysLogger("getModuleInfoList")
    public RespDTO<IPage<ModuleInfoListDTO>> getModuleInfoList(@Valid @RequestBody GetModuleInfoVO getModuleInfoVO) {
        IPage<ModuleInfoListDTO> date = moduleInfoFacade.getModuleInfoList(getModuleInfoVO);
        return RespDTO.onSuc(date);
    }

    @ApiOperation(value = "获取单个模板信息(修改用)[by:wangyu]",
            notes = "" +
                    "moduleId: 模板id<br>")
    @PostMapping("/getModuleInfoOne")
    @SysLogger("getModuleInfoOne")
    public RespDTO<GetModuleInfoOneDTO> getModuleInfoOne(@Valid @RequestBody GetModuleInfoOneVO getModuleInfoVO) {
        GetModuleInfoOneDTO date = moduleInfoFacade.getModuleInfoOne(getModuleInfoVO);
        return RespDTO.onSuc(date);
    }

    @ApiOperation(value = "获取模板Type[从症状情况到补充内容]（添加时的类型下拉）[by:wangyu]",
            notes = "moduleType: 模板mouduleType，必填<br>" +
                    "")
    @PostMapping("/getModuleType")
    @SysLogger("getModuleType")
    public RespDTO<List<GetModuleTypeDTO>> getModuleType(@Valid @RequestBody GetModuleTypeVO getModuleTypeVO) {
        List<GetModuleTypeDTO> date = moduleInfoFacade.getModuleType(getModuleTypeVO);
        return RespDTO.onSuc(date);
    }

    @ApiOperation(value = "获取科室所有信息（主页面下拉用）[by:wangyu]",
            notes = "")
    @PostMapping("/getAllDeptAndDisInfo")
    @SysLogger("getAllDeptAndDisInfo")
    public RespDTO<GetAllDeptAndDisInfoDTO> getAllDeptAndDisInfo(@RequestBody GetAllDeptAndDisInfoVO getAllDeptAndDisInfoVO) {
        GetAllDeptAndDisInfoDTO date = moduleInfoFacade.getAllDeptAndDisInfo(getAllDeptAndDisInfoVO);
        return RespDTO.onSuc(date);
    }

}
