package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.QuestionPageDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.facade.QuestionFacade;
import com.diagbot.vo.DeleteQuestionVO;
import com.diagbot.vo.QuestionIdsVO;
import com.diagbot.vo.QuestionIndexVO;
import com.diagbot.vo.QuestionPageVO;
import com.diagbot.vo.QuestionSaveVO;
import com.diagbot.vo.QuestionVO;
import com.diagbot.vo.SubQuestionVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 标签基础表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2018-11-23
 */
@RestController
@RequestMapping("/questionInfo_prec")
@SuppressWarnings("unchecked")
@Api(value = "标签相关API", tags = { "标签相关API" })
public class QuestionInfoController {

    @Autowired
    QuestionFacade questionFacade;

    @ApiOperation(value = "标签新增或更新[by:zhoutg]",
            notes = "")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @Transactional
    public RespDTO<Boolean> saveOrUpdate(@RequestBody QuestionSaveVO questionSaveVO) {
        questionFacade.saveOrUpdate(questionSaveVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "标签分页列表[by:zhoutg]",
            notes = "")
    @PostMapping("/page")
    @SysLogger("page")
    public RespDTO<IPage<QuestionPageDTO>> list(@RequestBody @Valid QuestionPageVO questionPageVO) {
        IPage<QuestionPageDTO> data = questionFacade.getListFac(questionPageVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "标签检索[by:zhoutg]",
            notes = "tagName：系统名称<br>" +
                    "sexType：性别：1：男，2：女，3：通用<br>" +
                    "age：年龄<br>" +
                    "type：归属<br>" +
                    "notIds：过滤的id值<br>" +
                    "notControlType：不能为的控件类型<br>" +
                    "controlType：指定的控件类型<br>" +
                    "tagType：指定的标签类型<br>" +
                    "noSubQuestion：明细字段subQuestion有值，1：过滤；其他值：不过滤<br>")
    @PostMapping("/index")
    @SysLogger("index")
    public RespDTO<List<QuestionInfo>> index(@RequestBody QuestionIndexVO questionIndexVO) {
        List<QuestionInfo> data = questionFacade.indexFac(questionIndexVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "标签删除[by:zhoutg]",
            notes = "")
    @PostMapping("/delete")
    @SysLogger("delete")
    @Transactional
    public RespDTO<Boolean> delete(@RequestBody DeleteQuestionVO deleteQuestionVO) {
        questionFacade.deleteByIdsFac(deleteQuestionVO);
        return RespDTO.onSuc(true);
    }


    @ApiOperation(value = "根据参数返回标签内容[by:zhoutg]",
            notes = "id: questionId<br>" +
                    "sexType：性别，1：男，2：女<br>" +
                    "age：年龄")
    @PostMapping("/getById")
    public RespDTO<QuestionDTO> getById(@RequestBody QuestionVO questionVO) {
        QuestionDTO data = questionFacade.getById(questionVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "查询多个id返回标签内容[by:zhoutg]",
            notes = "ids: 多个questionId<br>" +
                    "sexType：性别，1：男，2：女<br>" +
                    "age：年龄")
    @PostMapping("/getByIds")
    public RespDTO<Map<Long, Object>> getByIds(@RequestBody QuestionIdsVO questionIdsVO) {
        Map<Long, Object> data = questionFacade.getByIds(questionIdsVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "根据id返回被引用的父标签[by:zhoutg]",
            notes = "id: 标签id<br>")
    @PostMapping("/getBySubQuestionId")
    public RespDTO<List<QuestionInfo>> getBySubQuestionId(@RequestBody SubQuestionVO subQuestionVO) {
        List<QuestionInfo> data = questionFacade.getBySubQuestionId(subQuestionVO);
        return RespDTO.onSuc(data);
    }
}
