package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.DiseaseDeptDTO;
import com.diagbot.entity.DiseaseDept;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.service.impl.DiseaseDeptServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.DiseaseDeptVO;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/3/20 13:46
 */
@Component
public class DiseaseDeptFacade extends DiseaseDeptServiceImpl {

    /**
     * 根据内部诊断名称查询诊断科室映射关系
     *
     * @param diseaseDeptVO
     * @return
     */
    public List<DiseaseDeptDTO> getDiseaseDeptByConceptNames(DiseaseDeptVO diseaseDeptVO) {
        QueryWrapper<DiseaseDept> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("status", StatusEnum.Enable.getKey())
                .in(ListUtil.isNotEmpty(diseaseDeptVO.getConceptNames()), "concept_dis_name", diseaseDeptVO.getConceptNames());
        if (StringUtil.isNotBlank(diseaseDeptVO.getHospitalCode())) {
            queryWrapper.eq("hospital_code", diseaseDeptVO.getHospitalCode());
        }
        List<DiseaseDept> diseaseDeptList = this.list(queryWrapper);
        //过滤科室为空的数据和诊断分类为空的数据
        diseaseDeptList = diseaseDeptList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getDeptName()))
                .filter(i -> StringUtil.isNotBlank(i.getDiseaseName()))
                .collect(Collectors.toList());
        List<DiseaseDeptDTO> diseaseDeptDTOList = BeanUtil.listCopyTo(diseaseDeptList, DiseaseDeptDTO.class);
        return diseaseDeptDTOList;
    }
}