package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.DoctorInfoDTO;
import com.diagbot.dto.DoctorInfoFindDTO;
import com.diagbot.entity.DoctorInfo;
import com.diagbot.entity.SysSet;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.DoctorInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.vo.DoctorInfoVO;
import com.diagbot.vo.HospitalCodeSetVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: wangyu
 * @time: 2018/11/19 19:49
 */
@Component
public class DoctorInfoFacade extends DoctorInfoServiceImpl {
    @Autowired
    private SysSetFacade sysSetFacade;

    /**
     * 获取医生信息
     *
     * @param doctorInfoVo
     * @return
     */
    public List<DoctorInfoDTO> getDoctorInfo(DoctorInfoVO doctorInfoVo) {
        List<DoctorInfoDTO> doctorInfoDTOList = this.getDoctorInfos(doctorInfoVo.getDoctorCode(), doctorInfoVo.getHosptialCode(), doctorInfoVo.getDeptCode());
        return doctorInfoDTOList;
    }

    /**
     * 根据ids获取医生信息map
     *
     * @param ids 医生ids
     * @return 医生信息
     */
    public Map<Long, DoctorInfo> doctorInfoMapByIds(List<Long> ids) {
        Map<Long, DoctorInfo> doctorInfoMap
                = this.listByIds(ids).stream()
                .filter(i -> i.getIsDeleted().equals(IsDeleteEnum.N.getKey()))
                .collect(Collectors.toMap(DoctorInfo::getId, i -> i));
        return doctorInfoMap;
    }

    public DoctorInfoFindDTO findDoctorInfo(HospitalCodeSetVO hospitalCodeSetVO) {
        return this.baseMapper.findDoctorInfo(hospitalCodeSetVO);
    }


    public DoctorInfoDTO getDoctorInfoById(Long id) {
        DoctorInfo doctorInfo = this.getById(id);
        if (null == doctorInfo
                || doctorInfo.getIsDeleted().equals(IsDeleteEnum.Y.getKey())) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该医生不存在！");
        }

        if (doctorInfo.getHospitalCode().equals("LT")) {
            QueryWrapper<SysSet> sysSetInfo = new QueryWrapper<>();
            sysSetInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
            sysSetInfo.eq("hospital_code", doctorInfo.getHospitalCode());
            sysSetInfo.eq("code", "template_admin");
            sysSetInfo.eq("sys_type", SysTypeEnum.ICSS_SERVICE.getKey());
            sysSetInfo.apply("find_in_set({0}, value)", doctorInfo.getCode());
            if (sysSetFacade.count(sysSetInfo) > 0) {
                throw new CommonException(CommonErrorCode.NOPERMISSION_ERROR,
                        "该人员为管理员无需记录引用信息！");
            }
        }

        DoctorInfoDTO doctorInfoDTO = new DoctorInfoDTO();
        BeanUtil.copyProperties(doctorInfo, doctorInfoDTO);
        return doctorInfoDTO;
    }
}
