package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.HospitalDeptDTO;
import com.diagbot.dto.HospitalDeptInfoAllDTO;
import com.diagbot.dto.HospitalDeptInfoDTO;
import com.diagbot.entity.HospitalDept;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.HospitalDeptServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.HospitalCodeVo;
import com.diagbot.vo.HospitalDeptInfoVO;
import com.diagbot.vo.HospitalDeptVO;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author rgb
 * @Description
 * @time 2018年12月11日上午11:32:13
 */
@Component
public class HospitalDeptFacade extends HospitalDeptServiceImpl {

    /**
     * 获取科室信息
     *
     * @param hospitalDeptInfoVO
     * @return
     */
    public HospitalDeptInfoDTO getHospitalDeptInfo(HospitalDeptInfoVO hospitalDeptInfoVO) {
        HospitalDeptInfoDTO hospitalDeptInfoDTO = new HospitalDeptInfoDTO();
        QueryWrapper<HospitalDept> hospitalDeptQueryWrapper = new QueryWrapper<>();
        hospitalDeptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", hospitalDeptInfoVO.getHospitalCode())
                .eq("code", hospitalDeptInfoVO.getDeptCode());
        HospitalDept hospitalDept = this.getOne(hospitalDeptQueryWrapper, false);
        BeanUtil.copyProperties(hospitalDept, hospitalDeptInfoDTO);
        return hospitalDeptInfoDTO;
    }

    /**
     * 根据ids获取科室信息map
     *
     * @param ids 科室ids
     * @return 科室信息
     */
    public Map<Long, HospitalDept> hospitalDeptInfoMapByIds(List<Long> ids) {
        Map<Long, HospitalDept> hospitalDeptMap = this.listByIds(ids).stream().filter(i -> i.getIsDeleted().equals("N") && i.getStatus() == 1).collect(Collectors.toMap(HospitalDept::getId, i -> i));
        return hospitalDeptMap;
    }

    /**
     * 根据概念名称获取科室信息map
     *
     * @param hospitalDeptVO
     * @return
     */
    public Map<String, List<HospitalDeptDTO>> hospitalDeptByConceptNames(HospitalDeptVO hospitalDeptVO) {
        if (ListUtil.isEmpty(hospitalDeptVO.getConceptNames())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "请输入ICSS科室名称");
        }
        Map<String, List<HospitalDeptDTO>> retMap = new LinkedHashMap<>();
        QueryWrapper<HospitalDept> hospitalDeptQueryWrapper = new QueryWrapper<>();
        hospitalDeptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", hospitalDeptVO.getHospitalCode())
                .eq("status", StatusEnum.Enable.getKey())
                .in("concept_dept_name", hospitalDeptVO.getConceptNames())
                .orderByAsc("concept_dept_name", "order_no");
        List<HospitalDept> hospitalDeptList = this.list(hospitalDeptQueryWrapper);
        if (ListUtil.isNotEmpty(hospitalDeptList)) {
            Map<String, List<HospitalDept>> hospitalDeptMap
                    = EntityUtil.makeEntityListMap(hospitalDeptList, "conceptDeptName");
            for (Map.Entry<String, List<HospitalDept>> entry : hospitalDeptMap.entrySet()) {
                List<HospitalDeptDTO> hospitalDeptDTOList = Lists.newArrayList();
                for (HospitalDept hospitalDept : entry.getValue()) {
                    HospitalDeptDTO hospitalDeptDTO = new HospitalDeptDTO();
                    hospitalDeptDTO.setHospitalCode(hospitalDept.getHospitalCode());
                    hospitalDeptDTO.setHospitalName(hospitalDept.getHospitalName());
                    hospitalDeptDTO.setHospitalDeptCode(hospitalDept.getCode());
                    hospitalDeptDTO.setHospitalDeptName(hospitalDept.getName());
                    hospitalDeptDTO.setUrl(hospitalDept.getUrl());
                    hospitalDeptDTOList.add(hospitalDeptDTO);
                }
                retMap.put(entry.getKey(), hospitalDeptDTOList);
            }
        }
        return retMap;
    }

    /**
     * @param hospitalCodeVo
     * @return
     */
    public List<HospitalDeptInfoAllDTO> getHospitalDeptInfoAll(HospitalCodeVo hospitalCodeVo) {
        QueryWrapper<HospitalDept> hospitalDeptQueryWrapper = new QueryWrapper<>();
        hospitalDeptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", hospitalCodeVo.getHospitalCode())
                .eq("status", StatusEnum.Enable.getKey())
                .like(StringUtil.isNotBlank(hospitalCodeVo.getThirdpartyName()),
                        "name", hospitalCodeVo.getThirdpartyName())
                .like(StringUtil.isNotBlank(hospitalCodeVo.getDeptInfoName()),
                        "concept_dept_name", hospitalCodeVo.getDeptInfoName());
        List<HospitalDept> hospitalDeptList = this.list(hospitalDeptQueryWrapper);
        List<HospitalDeptInfoAllDTO> data = BeanUtil.listCopyTo(hospitalDeptList, HospitalDeptInfoAllDTO.class);
        return data;
    }
}