package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.GetExportInquirysDTO;
import com.diagbot.dto.HisInquirysForJzDTO;
import com.diagbot.dto.HisInquirysForJzDetailDTO;
import com.diagbot.entity.InquiryDetail;
import com.diagbot.entity.InquiryInfo;
import com.diagbot.entity.PatientInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.SexTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.InquiryDetailServiceImpl;
import com.diagbot.service.impl.InquiryInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.AddDiagnoseSecondVO;
import com.diagbot.vo.HisInquirysForJzVO;
import com.diagbot.vo.SaveInquiryToHisVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 对接层问诊信息业务逻辑
 * @author: rengb
 * @since 2019-3-18
 */
@Component
public class InquiryInfoFacade extends InquiryInfoServiceImpl {

    @Autowired
    private InquiryDetailFacade inquiryDetailFacade;
    @Autowired
    private PatientInfoFacade patientInfoFacade;
    @Autowired
    @Qualifier("inquiryDetailServiceImpl")
    private InquiryDetailServiceImpl inquiryDetailServiceImpl;

    /**
     * 问诊记录保存
     *
     * @param saveInquiryToHisVO
     * @return
     */
    public Boolean saveInquiryToHis(SaveInquiryToHisVO saveInquiryToHisVO) {
        Date now = new Date();

        QueryWrapper<InquiryInfo> inquiryInfoQe = new QueryWrapper<>();
        inquiryInfoQe.eq("is_deleted", IsDeleteEnum.N.getKey());
        inquiryInfoQe.eq("hospital_id", saveInquiryToHisVO.getHospitalId());
        inquiryInfoQe.eq("hospital_dept_id", saveInquiryToHisVO.getHospitalDeptId());
        inquiryInfoQe.eq("doctor_id", saveInquiryToHisVO.getDoctorId());
        inquiryInfoQe.eq("patient_id", saveInquiryToHisVO.getPatientId());
        inquiryInfoQe.eq("inquiry_code", saveInquiryToHisVO.getInquiryCode());
        InquiryInfo inquiryInfo = getOne(inquiryInfoQe, false);

        if (inquiryInfo == null) {
            inquiryInfo = new InquiryInfo();
            inquiryInfo.setGmtCreate(now);
        } else {
            QueryWrapper<InquiryDetail> inquiryDetailQe = new QueryWrapper<>();
            inquiryDetailQe.eq("inquiry_id", inquiryInfo.getId());
            inquiryDetailQe.ne("type", 9);
            inquiryDetailFacade.remove(inquiryDetailQe);
        }

        inquiryInfo.setGmtModified(now);
        BeanUtil.copyProperties(saveInquiryToHisVO, inquiryInfo);
        saveOrUpdate(inquiryInfo);

        List<InquiryDetail> inquiryDetailList = BeanUtil.listCopyTo(saveInquiryToHisVO.getDetailList(), InquiryDetail.class);
        for (InquiryDetail i : inquiryDetailList) {
            i.setInquiryId(inquiryInfo.getId());
            i.setGmtCreate(now);
            i.setGmtModified(now);
        }
        inquiryDetailServiceImpl.saveBatch(inquiryDetailList);

        return true;
    }

    /**
     * 急诊-历史病历列表
     *
     * @param hisInquirysForJzVO
     * @return
     */
    public HisInquirysForJzDTO hisInquirysForJz(HisInquirysForJzVO hisInquirysForJzVO) {
        HisInquirysForJzDTO hisInquirysForJzDTO = new HisInquirysForJzDTO();

        IPage<HisInquirysForJzDetailDTO> ipage = this.baseMapper.hisInquirysForJz(hisInquirysForJzVO);
        List<HisInquirysForJzDetailDTO> inquiryList = ipage.getRecords();

        if (ListUtil.isNotEmpty(inquiryList)) {
            List<Long> patientIdList = inquiryList.stream()
                    .map(i -> i.getPatientId()).distinct().collect(Collectors.toList());
            List<Long> inquiryIdList = inquiryList.stream()
                    .map(i -> i.getInquiryId()).distinct().collect(Collectors.toList());

            Map<Long, PatientInfo> patientInfoMap = patientInfoFacade.patientInfoMapByIds(patientIdList);
            Map<Long, List<InquiryDetail>> inquiryDetailListMap = inquiryDetailFacade.getDetailsByInquryIds(inquiryIdList);

            inquiryList.forEach(i -> {
                PatientInfo patientInfo = patientInfoMap.get(i.getPatientId());
                if (patientInfo != null) {
                    i.setPatientName(patientInfo.getName());
                    i.setPatientIdNo(patientInfo.getIdNo());
                    i.setPatientSex(SexTypeEnum.getName(patientInfo.getSex()));
                    i.setPatientAge(DateUtil.yearCompare(patientInfo.getBirthday(), DateUtil.now()));
                }

                List<InquiryDetail> detailList = inquiryDetailListMap.get(i.getInquiryId()).stream()
                        .filter(inquiryDetail -> inquiryDetail.getType() != 9).collect(Collectors.toList());
                i.setDetailList(detailList);
                String diagnose = detailList.stream().filter(inquiryDetail -> inquiryDetail.getType() == 7)
                        .map(inquiryDetail -> inquiryDetail.getContentValue()).collect(Collectors.toList()).get(0);
                i.setDiagnose(diagnose.split(";")[0]);
            });
        }

        hisInquirysForJzDTO.setInquiryList(inquiryList);
        hisInquirysForJzDTO.setTotal(ipage.getTotal());

        return hisInquirysForJzDTO;
    }


    /**
     * 急诊-添加二次诊断
     *
     * @param addDiagnoseSecondVO
     * @return
     */
    public Boolean addDiagnoseSecond(AddDiagnoseSecondVO addDiagnoseSecondVO) {
        QueryWrapper<InquiryDetail> inquiryDetailQe = new QueryWrapper<>();
        inquiryDetailQe.eq("inquiry_id", addDiagnoseSecondVO.getInquiryId());
        inquiryDetailQe.eq("type", 9);
        inquiryDetailFacade.remove(inquiryDetailQe);

        Date now = DateUtil.now();

        InquiryDetail inquiryDetail = new InquiryDetail();
        inquiryDetail.setContentValue(addDiagnoseSecondVO.getDiagnoseSecond());
        inquiryDetail.setType(9);
        inquiryDetail.setInquiryId(addDiagnoseSecondVO.getInquiryId());
        inquiryDetail.setGmtCreate(now);
        inquiryDetail.setGmtModified(now);
        inquiryDetail.setCreator(addDiagnoseSecondVO.getDoctorId().toString());
        inquiryDetail.setModifier(addDiagnoseSecondVO.getDoctorId().toString());

        return inquiryDetailFacade.save(inquiryDetail);
    }

    /**
     * 急诊-获取需要导出的问诊数据
     *
     * @param inquiryIds
     * @return
     */
    public List<GetExportInquirysDTO> getExportInquirys(List<Long> inquiryIds) {
        List<GetExportInquirysDTO> getExportInquirysDTOList = new ArrayList<>();

        if (inquiryIds.stream().distinct().count() != inquiryIds.size()) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR);
        }

        List<InquiryInfo> inquiryInfoList = Lists.newArrayList(listByIds(inquiryIds));
        if (inquiryInfoList.size() != inquiryIds.size()) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR);
        }

        List<Long> patientIdList = inquiryInfoList.stream()
                .map(i -> i.getPatientId()).distinct().collect(Collectors.toList());
        Map<Long, PatientInfo> patientInfoMap = patientInfoFacade.patientInfoMapByIds(patientIdList);
        Map<Long, InquiryInfo> inquiryInfoMap = inquiryInfoList.stream()
                .collect(Collectors.toMap(InquiryInfo::getId, i -> i));
        Map<Long, List<InquiryDetail>> inquiryDetailListMap = inquiryDetailFacade.getDetailsByInquryIds(inquiryIds);

        Integer lineNum = 1;
        PatientInfo patientInfo = null;
        String diagnose = null;
        for (Long inquiryId : inquiryIds) {
            GetExportInquirysDTO getExportInquirysDTO = new GetExportInquirysDTO();
            getExportInquirysDTO.setLineNum(lineNum);
            patientInfo = patientInfoMap.get(inquiryInfoMap.get(inquiryId).getPatientId());
            if (patientInfo != null) {
                getExportInquirysDTO.setPatientSex(SexTypeEnum.getName(patientInfo.getSex()));
                getExportInquirysDTO.setPatientAge(DateUtil.yearCompare(patientInfo.getBirthday(), DateUtil.now()));
            }

            Map<Integer, String> detailMap = inquiryDetailListMap.get(inquiryId).stream()
                    .collect(Collectors.toMap(InquiryDetail::getType, i -> i.getContentValue() == null ? "" : i.getContentValue()));
            diagnose = detailMap.get(7);
            detailMap.put(7, detailMap.get(8));
            if (detailMap.get(9) != null) {
                diagnose = "";
            }
            detailMap.put(8, diagnose);
            getExportInquirysDTO.setDetailMap(detailMap);

            getExportInquirysDTOList.add(getExportInquirysDTO);
            lineNum++;
        }

        return getExportInquirysDTOList;
    }


}
