package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.OperationConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.OperationConfigServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.OperationConfigVO;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/3/27 15:41
 */
@Component
public class OperationConfigFacade extends OperationConfigServiceImpl {

    /**
     * 获取辅检映射关系
     *
     * @param operationConfigVO
     * @return
     */
    public Map<String, List<String>> getOperationConfig(OperationConfigVO operationConfigVO) {
        Map<String, List<String>> map = new HashMap<>();
        Map<String, List<OperationConfig>> operationMap = new LinkedHashMap<>();
        QueryWrapper<OperationConfig> operationConfigQueryWrapper = new QueryWrapper<>();
        operationConfigQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", operationConfigVO.getHosCode());
        if (ListUtil.isNotEmpty(operationConfigVO.getNameList())) {
            operationConfigQueryWrapper.in("name", operationConfigVO.getNameList());
            List<OperationConfig> operationConfigList = this.list(operationConfigQueryWrapper);
            operationMap = EntityUtil.makeEntityListMap(operationConfigList, "name");
        } else if (ListUtil.isNotEmpty(operationConfigVO.getUniqueNameList())) {
            operationConfigQueryWrapper.in("unique_name", operationConfigVO.getUniqueNameList());
            List<OperationConfig> operationConfigList = this.list(operationConfigQueryWrapper);
            operationMap = EntityUtil.makeEntityListMap(operationConfigList, "uniqueName");
        }
        for (Map.Entry<String, List<OperationConfig>> entry : operationMap.entrySet()) {
            map.put(entry.getKey(), entry.getValue()
                    .stream()
                    .map(operationConfig -> operationConfig.getUniqueName())
                    .distinct()
                    .collect(Collectors.toList()));
        }
        return map;
    }
}
