package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.PacsConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.PacsConfigServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.PacsConfigVO;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/3/11 13:10
 */
@Component
public class PacsConfigFacade extends PacsConfigServiceImpl{

    /**
     * 根据医院编码获取辅检公表映射关系 Map<mealName,uniqueName>
     *
     * @return
     */
    public Map<String, String> getPacsConfigByHosCode(String hosCode) {
        QueryWrapper<PacsConfig> pacsConfigQueryWrapper = new QueryWrapper<>();
        pacsConfigQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", hosCode);
        List<PacsConfig> pacsConfigList = this.list(pacsConfigQueryWrapper);
        Map<String, String> retMap
                = EntityUtil.makeMapWithKeyValue(pacsConfigList, "mealName", "uniqueName");
        return retMap;
    }

    /**
     * 获取辅检映射关系
     * @param pacsConfigVO
     * @return
     */
    public Map<String,List<String>> getPacsConfig(PacsConfigVO pacsConfigVO) {
        Map<String, List<String>> map = new HashMap<>();
        Map<String, List<PacsConfig>> pacsMap = new LinkedHashMap<>();
        QueryWrapper<PacsConfig> pacsConfigQueryWrapper = new QueryWrapper<>();
        pacsConfigQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", pacsConfigVO.getHosCode());
        if (ListUtil.isNotEmpty(pacsConfigVO.getMealNameList())) {
            pacsConfigQueryWrapper.in("meal_name", pacsConfigVO.getMealNameList());
            List<PacsConfig> pacsConfigList = this.list(pacsConfigQueryWrapper);
            pacsMap = EntityUtil.makeEntityListMap(pacsConfigList, "mealName");
        } else if (ListUtil.isNotEmpty(pacsConfigVO.getUniqueNameList())) {
            pacsConfigQueryWrapper.in("unique_name", pacsConfigVO.getUniqueNameList());
            List<PacsConfig> pacsConfigList = this.list(pacsConfigQueryWrapper);
            pacsMap = EntityUtil.makeEntityListMap(pacsConfigList, "uniqueName");
        }
        for (Map.Entry<String, List<PacsConfig>> entry : pacsMap.entrySet()) {
            map.put(entry.getKey(), entry.getValue()
                    .stream()
                    .map(pacsConfig -> pacsConfig.getUniqueName())
                    .distinct()
                    .collect(Collectors.toList()));
        }
        return map;
    }

    /**
     * 根据医院编码获取辅检公表映射关系，公表项做key  Map<uniqueName,List<mealName>>
     *
     * @return
     */
    public Map<String, List<String>> getPacsConfigByUniqueNameAndHosCode(String hosCode) {
        Map<String, List<String>> retMap = new LinkedHashMap<>();
        QueryWrapper<PacsConfig> pacsConfigQueryWrapper = new QueryWrapper<>();
        pacsConfigQueryWrapper
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_code", hosCode);
        List<PacsConfig> pacsConfigList = this.list(pacsConfigQueryWrapper);
        Map<String, List<PacsConfig>> pacsConfigMap
                = EntityUtil.makeEntityListMap(pacsConfigList, "uniqueName");
        for (Map.Entry<String, List<PacsConfig>> entry : pacsConfigMap.entrySet()) {
            retMap.put(entry.getKey(), EntityUtil.makeListByFieldName(entry.getValue(), "mealName"));
        }
        return retMap;
    }
}