package com.diagbot.facade;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.GetTopPatientInfoDjDTO;
import com.diagbot.dto.RegisterInfoDTO;
import com.diagbot.dto.SignInDTO;
import com.diagbot.entity.DoctorInfo;
import com.diagbot.entity.HospitalDept;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.entity.PatientInfo;
import com.diagbot.entity.SysSet;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.enums.VisibleIdTypeEnum;
import com.diagbot.exception.CommonException;
import com.diagbot.exception.ServiceErrorCode;
import com.diagbot.idc.VisibleIdCreater;
import com.diagbot.service.DoctorInfoService;
import com.diagbot.service.HospitalDeptService;
import com.diagbot.service.PatientInfoService;
import com.diagbot.service.impl.PatientInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.IdCard;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.GetTopPatientInfoDjVO;
import com.diagbot.vo.InquiryQuoteVO;
import com.diagbot.vo.SignInVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 对接-患者业务逻辑
 * @author: wangyu
 * @time: 2018/11/19 13:19
 */
@Component
public class PatientInfoDjFacade extends PatientInfoServiceImpl {
    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;
    @Autowired
    private HospitalDeptFacade hospitalDeptFacade;
    @Autowired
    @Qualifier("hospitalDeptServiceImpl")
    private HospitalDeptService hospitalDeptService;
    @Autowired
    private DoctorInfoFacade doctorInfoFacade;
    @Autowired
    @Qualifier("doctorInfoServiceImpl")
    private DoctorInfoService doctorInfoService;
    @Autowired
    private SysSetFacade sysSetFacade;
    @Autowired
    private VisibleIdCreater visibleIdCreater;
    @Autowired
    @Qualifier("patientInfoServiceImpl")
    private PatientInfoService patientInfoService;

    /**
     * 对接-页面顶部病人医生科室信息——查询
     *
     * @param getTopPatientInfoDjVO
     * @return
     */
    public GetTopPatientInfoDjDTO getTopPatientInfoDj(GetTopPatientInfoDjVO getTopPatientInfoDjVO) {
        String hospitalCode = getTopPatientInfoDjVO.getHospitalCode();
        String sonHospitalCode =
                StringUtil.isNotBlank(getTopPatientInfoDjVO.getSonHospitalCode())
                        ? getTopPatientInfoDjVO.getSonHospitalCode()
                        : hospitalCode;
        getTopPatientInfoDjVO.setSonHospitalCode(sonHospitalCode);

        GetTopPatientInfoDjDTO getTopPatientInfoDjDTO = this.baseMapper.getTopPatientInfoDj(getTopPatientInfoDjVO);
        getTopPatientInfoDjDTO.setSystemTime(DateUtil.now());
        getTopPatientInfoDjDTO.setRecordId(getTopPatientInfoDjVO.getRecordId());
        getTopPatientInfoDjDTO.setPatientAge(DateUtil.yearCompare(getTopPatientInfoDjDTO.getBirthday(), DateUtil.now()));

        if (!hospitalCode.equals(sonHospitalCode)) {
            QueryWrapper<HospitalInfo> hospitalInfoQe = new QueryWrapper<>();
            hospitalInfoQe.eq("code", hospitalCode);
            HospitalInfo hospitalInfo = hospitalInfoFacade.getOne(hospitalInfoQe, false);
            getTopPatientInfoDjDTO.setHospitalId(hospitalInfo.getId());
            getTopPatientInfoDjDTO.setHospitalCode(hospitalInfo.getCode());
            getTopPatientInfoDjDTO.setHospitalName(hospitalInfo.getName());
        } else {
            getTopPatientInfoDjDTO.setSonHospitalId(null);
            getTopPatientInfoDjDTO.setSonHospitalCode(null);
            getTopPatientInfoDjDTO.setSonHospitalName(null);
        }

        QueryWrapper<HospitalDept> hospitalDeptQe = new QueryWrapper<>();
        hospitalDeptQe.eq("hospital_code", sonHospitalCode);
        hospitalDeptQe.eq("code", getTopPatientInfoDjVO.getHospitalDeptCode());
        HospitalDept hospitalDept = hospitalDeptFacade.getOne(hospitalDeptQe, false);
        if (hospitalDept != null) {
            getTopPatientInfoDjDTO.setHospitalDeptId(hospitalDept.getId());
            getTopPatientInfoDjDTO.setHospitalDeptCode(hospitalDept.getCode());
            getTopPatientInfoDjDTO.setHospitalDeptName(hospitalDept.getName());
            getTopPatientInfoDjDTO.setSelfDeptName(hospitalDept.getConceptDeptName());
        }

        return getTopPatientInfoDjDTO;
    }

    /**
     * 患者登录
     *
     * @param signInVO
     * @return
     */
    public List<SignInDTO> signIn(SignInVO signInVO) {
        List<SignInDTO> retList = null;

        List<RegisterInfoDTO> rifList = getRegisterInfo(signInVO);
        if (ListUtil.isNotEmpty(rifList)) {
            generateByRif(rifList);
            retList = rifConverSin(rifList);
        } else {
            retList = autoGenePatinfo(signInVO);
        }

        if (retList == null) {
            throw new CommonException(ServiceErrorCode.IQC_NOT_FOUND);
        }

        return retList;
    }

    /**
     * 自动生成患者信息
     *
     * @param signInVO
     * @return
     */
    private List<SignInDTO> autoGenePatinfo(SignInVO signInVO) {
        String hospitalCode = signInVO.getHospitalCode();
        String sonHospitalCode =
                StringUtil.isNotBlank(signInVO.getSonHospitalCode()) ? signInVO.getSonHospitalCode() : hospitalCode;
        QueryWrapper<SysSet> sysSetQe = new QueryWrapper<>();
        sysSetQe.eq("sys_type", SysTypeEnum.PREC_SERVICE.getKey());
        sysSetQe.and(qe1 -> qe1
                .and(qe2 -> qe2
                        .eq("hospital_code", hospitalCode)
                        .eq("code", "patientinfo"))
                .or(qe3 -> qe3
                        .eq("hospital_code", sonHospitalCode)
                        .eq("code", "default_dept")));
        Map<String, String> sysSetCodeValue = sysSetFacade.list(sysSetQe).stream()
                .collect(Collectors.toMap(SysSet::getCode, i -> i.getValue()));

        if (StringUtil.isBlank(sysSetCodeValue.get("patientinfo"))
                || !sysSetCodeValue.get("patientinfo").equals("1")) {
            return null;
        }
        if (!signInVO.getPatientInfoType().equals("101")) {
            throw new CommonException(ServiceErrorCode.IDCARD_LOGIN);
        }

        List<SignInDTO> retList = Lists.newArrayList();
        SignInDTO signInDTO = new SignInDTO();
        signInDTO.setHospitalCode(signInVO.getHospitalCode());
        signInDTO.setHospitalName(signInVO.getHospitalName());
        signInDTO.setSonHospitalCode(signInVO.getSonHospitalCode());
        signInDTO.setSonHospitalName(signInVO.getSonHospitalName());

        String hospitalDeptCode = sysSetCodeValue.get("default_dept");
        signInDTO.setHospitalDeptCode(hospitalDeptCode);

        QueryWrapper<HospitalDept> hospitalDeptQe = new QueryWrapper<>();
        hospitalDeptQe.eq("hospital_code", sonHospitalCode);
        hospitalDeptQe.eq("code", hospitalDeptCode);
        signInDTO.setHospitalDeptName(hospitalDeptFacade.getOne(hospitalDeptQe, false).getName());

        QueryWrapper<PatientInfo> patientInfoQe = new QueryWrapper<>();
        patientInfoQe.eq("hospital_code", sonHospitalCode);
        patientInfoQe.eq("name", signInVO.getPatientName());
        patientInfoQe.and(a ->
                a.eq("id_no", signInVO.getPatientInfo()).or().eq("identity_num", signInVO.getPatientInfo()));
        patientInfoQe.orderByAsc("remark");
        List<PatientInfo> patientInfoList = list(patientInfoQe);
        PatientInfo patientInfo = null;
        if (ListUtil.isEmpty(patientInfoList)) {
            patientInfo = new PatientInfo();
            patientInfo.setGmtCreate(DateUtil.now());
            patientInfo.setHospitalCode(sonHospitalCode);
            patientInfo.setIdNo(signInVO.getPatientInfo());
            patientInfo.setIdentityNum(signInVO.getPatientInfo());
            patientInfo.setName(signInVO.getPatientName());
            patientInfo.setSex(IdCard.getGenderByIdCard(signInVO.getPatientInfo()));
            patientInfo.setBirthday(DateUtil.parseDate(IdCard.getBirthByIdCard(signInVO.getPatientInfo()), "yyyyMMdd"));

            patientInfo.setCode("" + visibleIdCreater.getNextShortId(VisibleIdTypeEnum.PATIENT_NO.getKey()));
            patientInfo.setRemark("1");//自动生成标志

            save(patientInfo);
        } else {
            patientInfo = patientInfoList.get(0);
        }

        signInDTO.setPatientCode(patientInfo.getCode());
        signInDTO.setPatientName(patientInfo.getName());

        retList.add(signInDTO);
        return retList;
    }

    /**
     * his返回的挂号信息转化给前端
     *
     * @param rifList
     * @return
     */
    private List<SignInDTO> rifConverSin(List<RegisterInfoDTO> rifList) {
        List<SignInDTO> retList = Lists.newArrayList();
        rifList.forEach(rif -> {
            rif.getDetail().forEach(i -> {
                SignInDTO signInDTO = new SignInDTO(
                        rif.getHospitalCode(),
                        rif.getHospitalName(),
                        rif.getSonHospitalCode(),
                        rif.getSonHospitalName(),
                        i.getHospitalDeptCode(),
                        i.getHospitalDeptName(),
                        i.getDoctorInfo() == null ? null : i.getDoctorInfo().getCode(),
                        i.getDoctorInfo() == null ? null : i.getDoctorInfo().getName(),
                        rif.getPatientInfo().getCode(),
                        rif.getPatientInfo().getName(),
                        i.getRecordTime(),
                        i.getRecordId(),
                        i.getRegisterNum()
                );
                retList.add(signInDTO);
            });
        });
        return retList;
    }

    /**
     * 由挂号信息检查病人科室医生信息，有更新，无插入
     *
     * @param rifList
     */
    private void generateByRif(List<RegisterInfoDTO> rifList) {
        String hospitalCode = rifList.get(0).getHospitalCode();
        String hospitalName = rifList.get(0).getHospitalName();

        List<PatientInfo> patientSaveOrUpdateList = Lists.newArrayList();
        List<HospitalDept> deptSaveOrUpdateList = Lists.newArrayList();
        List<DoctorInfo> doctorSaveOrUpdateList = Lists.newArrayList();
        rifList.forEach(rif -> {
            String sonHospitalCode = StringUtil.isNotBlank(rif.getSonHospitalCode()) ? rif.getSonHospitalCode() : hospitalCode;
            String sonHospitalName = StringUtil.isNotBlank(rif.getSonHospitalCode()) ? rif.getSonHospitalName() : hospitalName;

            /********** 患者部分 **********/
            PatientInfo patientInfo = rif.getPatientInfo();
            if (!patientSaveOrUpdateList.stream()
                    .filter(patientSaveOrUpdate -> patientSaveOrUpdate.getHospitalCode().equals(sonHospitalCode))
                    .map(patientSaveOrUpdate -> patientSaveOrUpdate.getCode())
                    .collect(Collectors.toList())
                    .contains(patientInfo.getCode())) {
                QueryWrapper<PatientInfo> patientInfoQe = new QueryWrapper<>();
                patientInfoQe.eq("hospital_code", sonHospitalCode);
                //patientInfoQe.eq("name", patientInfo.getName());
                patientInfoQe.eq(StringUtil.isBlank(patientInfo.getIdentityNum())
                                && StringUtil.isNotBlank(patientInfo.getIdNo()),
                        "id_no", patientInfo.getIdNo());
                patientInfoQe.eq(StringUtil.isNotBlank(patientInfo.getIdentityNum()),
                        "identity_num", patientInfo.getIdentityNum());
                PatientInfo patientInfoQuery = getOne(patientInfoQe, false);
                PatientInfo saveOrUpat = new PatientInfo();
                Long patId = null;
                if (patientInfoQuery != null) {
                    patId = patientInfoQuery.getId();
                    BeanUtil.copyProperties(patientInfoQuery, saveOrUpat);
                }
                BeanUtil.copyProperties(patientInfo, saveOrUpat);
                saveOrUpat.setId(patId);
                saveOrUpat.setHospitalCode(sonHospitalCode);
                patientSaveOrUpdateList.add(saveOrUpat);
            }

            /********** 科室部分 **********/
            List<String> deptCodeList = rif.getDetail().stream()
                    .filter(i -> StringUtil.isNotBlank(i.getHospitalDeptCode())
                            && StringUtil.isNotBlank(i.getHospitalDeptName()))
                    .map(i -> i.getHospitalDeptCode())
                    .collect(Collectors.toList());
            if (ListUtil.isNotEmpty(deptCodeList)) {
                QueryWrapper<HospitalDept> hospitalDeptQe = new QueryWrapper<>();
                hospitalDeptQe.eq("hospital_code", sonHospitalCode);
                hospitalDeptQe.in("code", deptCodeList);
                Map<String, HospitalDept> deptMap = hospitalDeptFacade.list(hospitalDeptQe)
                        .stream()
                        .collect(Collectors.toMap(HospitalDept::getCode, i -> i));
                rif.getDetail().stream()
                        .filter(i -> StringUtil.isNotBlank(i.getHospitalDeptCode())
                                && StringUtil.isNotBlank(i.getHospitalDeptName()))
                        .forEach(i -> {
                            if (!deptSaveOrUpdateList.stream()
                                    .filter(deptSaveOrUpdate -> deptSaveOrUpdate.getHospitalCode().equals(sonHospitalCode))
                                    .map(deptSaveOrUpdate -> deptSaveOrUpdate.getCode())
                                    .collect(Collectors.toList())
                                    .contains(i.getHospitalDeptCode())) {
                                HospitalDept hospitalDeptQuery = deptMap.get(i.getHospitalDeptCode());
                                if (hospitalDeptQuery == null) {
                                    hospitalDeptQuery = new HospitalDept();
                                    hospitalDeptQuery.setHospitalCode(sonHospitalCode);
                                    hospitalDeptQuery.setHospitalName(sonHospitalName);
                                    hospitalDeptQuery.setCode(i.getHospitalDeptCode());
                                    hospitalDeptQuery.setName(i.getHospitalDeptName());
                                    hospitalDeptQuery.setConceptDeptName("全科");
                                } else {
                                    hospitalDeptQuery.setName(i.getHospitalDeptName());
                                }
                                deptSaveOrUpdateList.add(hospitalDeptQuery);
                            }
                        });
            }

            /********** 医生部分 **********/
            List<String> doctorCodeList = rif.getDetail().stream()
                    .filter(i -> i.getDoctorInfo() != null).map(i -> i.getDoctorInfo())
                    .filter(i -> StringUtil.isNotBlank(i.getCode()) && StringUtil.isNotBlank(i.getName()))
                    .map(i -> i.getCode())
                    .collect(Collectors.toList());
            if (ListUtil.isNotEmpty(doctorCodeList)) {
                QueryWrapper<DoctorInfo> doctorInfoQe = new QueryWrapper<>();
                doctorInfoQe.eq("hospital_code", sonHospitalCode);
                doctorInfoQe.in("code", doctorCodeList);
                Map<String, DoctorInfo> doctorMap = doctorInfoFacade.list(doctorInfoQe)
                        .stream()
                        .collect(Collectors.toMap(DoctorInfo::getCode, i -> i));
                rif.getDetail().stream()
                        .filter(i -> i.getDoctorInfo() != null).map(i -> i.getDoctorInfo())
                        .filter(i -> StringUtil.isNotBlank(i.getCode()) && StringUtil.isNotBlank(i.getName()))
                        .forEach(i -> {
                            if (!doctorSaveOrUpdateList.stream()
                                    .filter(doctorSaveOrUpdate -> doctorSaveOrUpdate.getHospitalCode().equals(sonHospitalCode))
                                    .map(doctorSaveOrUpdate -> doctorSaveOrUpdate.getCode())
                                    .collect(Collectors.toList())
                                    .contains(i.getCode())) {
                                DoctorInfo doctorInfoQuery = doctorMap.get(i.getCode());
                                DoctorInfo doctorInfo = new DoctorInfo();
                                Long docId = null;
                                if (doctorInfoQuery != null) {
                                    docId = doctorInfoQuery.getId();
                                    BeanUtil.copyProperties(doctorInfoQuery, doctorInfo);
                                }
                                BeanUtil.copyProperties(i, doctorInfo);
                                doctorInfo.setHospitalCode(sonHospitalCode);
                                doctorInfo.setId(docId);
                                doctorSaveOrUpdateList.add(doctorInfo);
                            }
                        });
            }
        });

        if (ListUtil.isNotEmpty(patientSaveOrUpdateList)) {
            patientInfoService.saveOrUpdateBatch(patientSaveOrUpdateList);
        }
        if (ListUtil.isNotEmpty(deptSaveOrUpdateList)) {
            hospitalDeptService.saveOrUpdateBatch(deptSaveOrUpdateList);
        }
        if (ListUtil.isNotEmpty(doctorSaveOrUpdateList)) {
            doctorInfoService.saveOrUpdateBatch(doctorSaveOrUpdateList);
        }
    }

    /**
     * 对接---获取挂号信息
     * 某一时刻患者只可能在一家医院，所以获取到的挂号信息只能是当前医院的，当出现江干的大小医院情况时可采用2种：
     * 1、预问诊必须选择小医院，对方接口小医院要求必传；2、his返回时间最近的一家医院挂号信息(可能不准确)。
     * 大小医院信息使用his返回来的(未选择小医院时有用)
     *
     * @param signInVO
     * @return
     */
    private List<RegisterInfoDTO> getRegisterInfo(SignInVO signInVO) {
        if (!signInVO.getHospitalCode().equals("33010400")) {
            return null;
        }
        String retJson = "  [{"
                + "\"hospitalCode\": \"X\","
                + "\"hospitalName\": \"江干区卫计局\","
                + "\"sonHospitalCode\": \"33010480\","
                + "\"sonHospitalName\": \"四季青街道社区卫生服务中心\","
                + "\"patientInfo\": {"
                + "	\"birthday\": \"1984-10-08 10:20:23\","
                + "\"code\": \"1759\","
                + "\"idNo\": \"binglihao001\","
                + "	\"hospitalCode\": \"33010480\","
                + "	\"sex\": 1,"
                + "	\"identityNum\": \"360827199603149901\","
                + "	\"name\": \"刘亦菲1\""
                + "},"
                + "\"detail\": [{"
                + " \"hospitalDeptCode\": \"2127\","
                + "   \"hospitalDeptName\": \"中医科\","
                //                + "  \"doctorInfo\": {"
                //                + "    \"code\": \"000\","
                //                + "     \"hospitalCode\": \"33010400\","
                //                + "    \"name\": \"普通\""
                //                + "},"
                //                + "     \"recordId\": \"9043516\","
                + "  \"recordTime\": \"2019-12-26 11:31:05\","
                + " \"registerNum\": \"15\""
                + "},{"
                + "\"hospitalDeptCode\": \"3005\","
                + " \"hospitalDeptName\": \"穴位埋线注射\","
                //                + " \"doctorInfo\": {"
                //                + "\"code\": \"000\","
                //                + " \"hospitalCode\": \"33010400\","
                //                + "\"name\": \"普通\""
                //                + "},"
                //                + "  \"recordId\": \"9043512\","
                + " \"recordTime\": \"2019-12-05 12:23:09\","
                + " \"registerNum\": \"1\""
                + "}]"
                + "},"
                + "{"
                + "  \"hospitalCode\": \"33010400\","
                + "\"hospitalName\": \"江干区卫计局\","
                + " \"sonHospitalCode\": \"33010450\","
                + " \"sonHospitalName\": \"九堡街道社区卫生服务中心\","
                + "\"patientInfo\": {"
                + " \"birthday\": \"1984-10-08 10:20:23\","
                + "\"code\": \"1759\","
                + " \"idNo\": \"binglihao001\","
                + " \"hospitalCode\": \"33010450\","
                + " \"sex\": 1,"
                + "  \"identityNum\": \"360827199603149901\","
                + "  \"name\": \"刘亦菲1\""
                + "},"
                + " \"detail\": [{"
                + "  \"hospitalDeptCode\": \"3005\","
                + "\"hospitalDeptName\": \"穴位埋线注射\","
                //                + " \"doctorInfo\": {"
                //                + "  \"code\": \"000\","
                //                + " \"hospitalCode\": \"33010400\","
                //                + "  \"name\": \"普通\""
                //                + "},"
                //                + " \"recordId\": \"9043516\","
                + " \"recordTime\": \"2019-12-05 12:29:55\","
                + "  \"registerNum\": \"1\""
                + "},{"
                + " \"hospitalDeptCode\": \"2284\","
                + "    \"hospitalDeptName\": \"中心全科\","
                //                + "   \"doctorInfo\": {"
                //                + "   \"code\": \"000\","
                //                + "  \"hospitalCode\": \"33010400\","
                //                + "  \"name\": \"普通\""
                //                + "},"
                //                + "\"recordId\": \"9043512\","
                + "   \"recordTime\": \"2019-12-05 12:23:09\","
                + "  \"registerNum\": \"1\""
                + "}]"
                + "}"
                + "]";

        //        List<RegisterInfoDTO> rifList = GsonUtil.toList(retJson, RegisterInfoDTO.class);
        List<RegisterInfoDTO> rifList = JSON.parseArray(retJson, RegisterInfoDTO.class);
        System.out.println(rifList);
        if (StringUtil.isBlank(signInVO.getSonHospitalCode())) {
            return rifList;
        }

        Map<String, List<RegisterInfoDTO>> registerInfoDTOListMap = rifList.stream().collect(Collectors.groupingBy(RegisterInfoDTO::getSonHospitalCode));

        return registerInfoDTOListMap.get(signInVO.getSonHospitalCode());
    }

    /**
     * 病历引用时，更新患者医生科室信息
     *
     * @param inquiryQuoteVO
     * @return
     */
    public InquiryQuoteVO inquiryQuote(InquiryQuoteVO inquiryQuoteVO) {
        List<String> hospitalCodeList = Lists.newArrayList();
        hospitalCodeList.add(inquiryQuoteVO.getHospitalCode());

        String hospitalCode = inquiryQuoteVO.getHospitalCode();
        String hospitalName = inquiryQuoteVO.getHospitalName();

        if (StringUtil.isNotBlank(inquiryQuoteVO.getSonHospitalCode())) {
            hospitalCodeList.add(inquiryQuoteVO.getSonHospitalCode());
            hospitalCode = inquiryQuoteVO.getSonHospitalCode();
            hospitalName = inquiryQuoteVO.getSonHospitalName();
        }

        QueryWrapper<HospitalInfo> hospitalInfoQe = new QueryWrapper<>();
        hospitalInfoQe.in("code", hospitalCodeList);
        Map<String, Long> hospitalInfoIdMap = hospitalInfoFacade.list(hospitalInfoQe)
                .stream().collect(Collectors.toMap(HospitalInfo::getCode, i -> i.getId()));

        inquiryQuoteVO.setHospitalId(hospitalInfoIdMap.get(inquiryQuoteVO.getHospitalCode()));
        if (StringUtil.isNotBlank(inquiryQuoteVO.getSonHospitalCode())) {
            inquiryQuoteVO.setSonHospitalId(hospitalInfoIdMap.get(inquiryQuoteVO.getSonHospitalCode()));
        }

        Date now = DateUtil.now();
        if (inquiryQuoteVO.getDeptInfo() != null) {
            QueryWrapper<HospitalDept> hospitalDeptQe = new QueryWrapper<>();
            hospitalDeptQe.eq("hospital_code", hospitalCode);
            hospitalDeptQe.eq("code", inquiryQuoteVO.getDeptInfo().getCode());
            HospitalDept hospitalDept = hospitalDeptFacade.getOne(hospitalDeptQe, false);
            Long deptId = null;
            if (hospitalDept == null) {
                hospitalDept = new HospitalDept();
                hospitalDept.setGmtCreate(now);
                hospitalDept.setHospitalCode(hospitalCode);
                hospitalDept.setHospitalName(hospitalName);
                hospitalDept.setConceptDeptName("全科");
            } else {
                deptId = hospitalDept.getId().longValue();
            }
            BeanUtil.copyProperties(inquiryQuoteVO.getDeptInfo(), hospitalDept);
            hospitalDept.setGmtModified(now);
            hospitalDept.setId(deptId);
            hospitalDeptFacade.saveOrUpdate(hospitalDept);
            inquiryQuoteVO.getDeptInfo().setId(hospitalDept.getId());
        }

        if (inquiryQuoteVO.getDoctorInfo() != null) {
            QueryWrapper<DoctorInfo> doctorInfoQe = new QueryWrapper<>();
            doctorInfoQe.eq("hospital_code", hospitalCode);
            doctorInfoQe.eq("code", inquiryQuoteVO.getDoctorInfo().getCode());
            DoctorInfo doctorInfo = doctorInfoFacade.getOne(doctorInfoQe, false);
            Long doctorId = null;
            if (doctorInfo == null) {
                doctorInfo = new DoctorInfo();
                doctorInfo.setGmtCreate(now);
                doctorInfo.setHospitalCode(hospitalCode);
            } else {
                doctorId = doctorInfo.getId().longValue();
            }
            BeanUtil.copyProperties(inquiryQuoteVO.getDoctorInfo(), doctorInfo);
            doctorInfo.setGmtModified(now);
            doctorInfo.setId(doctorId);
            doctorInfoFacade.saveOrUpdate(doctorInfo);
            inquiryQuoteVO.getDoctorInfo().setId(doctorInfo.getId());
        }

        QueryWrapper<PatientInfo> patientInfoQe = new QueryWrapper<>();
        patientInfoQe.eq("hospital_code", hospitalCode);
        patientInfoQe.eq("name", inquiryQuoteVO.getPatientInfo().getName());
        patientInfoQe.eq(StringUtil.isBlank(inquiryQuoteVO.getPatientInfo().getIdentityNum())
                        && StringUtil.isNotBlank(inquiryQuoteVO.getPatientInfo().getIdNo()),
                "id_no", inquiryQuoteVO.getPatientInfo().getIdNo());
        patientInfoQe.eq(StringUtil.isNotBlank(inquiryQuoteVO.getPatientInfo().getIdentityNum()),
                "identity_num", inquiryQuoteVO.getPatientInfo().getIdentityNum());
        PatientInfo patientInfo = getOne(patientInfoQe, false);
        Long patientId = null;
        if (patientInfo == null) {
            patientInfo = new PatientInfo();
            patientInfo.setGmtCreate(now);
            patientInfo.setHospitalCode(hospitalCode);
        } else {
            patientId = patientInfo.getId().longValue();
        }
        BeanUtil.copyProperties(inquiryQuoteVO.getPatientInfo(), patientInfo);
        patientInfo.setGmtModified(now);
        patientInfo.setId(patientId);
        saveOrUpdate(patientInfo);
        inquiryQuoteVO.getPatientInfo().setId(patientInfo.getId());

        return inquiryQuoteVO;
    }

}