package com.diagbot.facade;

import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.PatientInfoDTO;
import com.diagbot.entity.PatientInfo;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.PatientInfoServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.PatientInfoVO;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 患者业务逻辑
 * @author: wangyu
 * @time: 2018/11/19 13:19
 */
@Component
public class PatientInfoFacade extends PatientInfoServiceImpl {

    /**
     * 获取患者信息
     *
     * @param patientInfoVO
     * @return
     */
    public PatientInfoDTO getPatientInfo(PatientInfoVO patientInfoVO) {
        PatientInfoDTO patientInfoDTO = this.getPatientInfos(patientInfoVO.getPatientCode(),
                patientInfoVO.getHospitalCode());
        return patientInfoDTO;
    }

    /**
     * 页面顶部病人医生科室信息查询
     *
     * @param getTopPatientInfoVO
     * @return
     */
    public GetTopPatientInfoDTO getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO) {
        GetTopPatientInfoDTO getTopPatientInfoDTO = baseMapper.getTopPatientInfo(getTopPatientInfoVO);
        if (getTopPatientInfoDTO == null) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取医院信息失败");
        }
        if (StringUtil.isBlank(getTopPatientInfoDTO.getHospitalDeptCode())) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取科室信息失败");
        }
        if (StringUtil.isBlank(getTopPatientInfoDTO.getDoctorCode())) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取医生信息失败");
        }
        if (StringUtil.isBlank(getTopPatientInfoDTO.getPatientCode())) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "获取患者信息失败");
        }
        getTopPatientInfoDTO.setSystemTime(DateUtil.now());
        getTopPatientInfoDTO.setRecordId(getTopPatientInfoVO.getRecordId());
        getTopPatientInfoDTO.setPatientAge(DateUtil.yearCompare(getTopPatientInfoDTO.getBirthday(), DateUtil.now()));
        return getTopPatientInfoDTO;
    }

    /**
     * 根据ids获取病人信息
     *
     * @param ids 病人ids
     * @return 病人信息
     */
    public Map<Long, PatientInfo> patientInfoMapByIds(List<Long> ids) {
        Map<Long, PatientInfo> patientInfoMap = this.listByIds(ids).stream()
                .filter(i -> i.getIsDeleted().equals("N")).collect(Collectors.toMap(PatientInfo::getId, i -> i));
        return patientInfoMap;
    }

}