package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.DoctorInfoFindDTO;
import com.diagbot.dto.GetHospitalRankInfoDTO;
import com.diagbot.dto.SysHospitalDTO;
import com.diagbot.dto.SysHospitalDeptInfoDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.entity.SysSet;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.SysSetServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.IntegerUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.GetHospitalRankInfoVO;
import com.diagbot.vo.HospitalCodeSetVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.SysHospitalCodeVO;
import com.diagbot.vo.SysHospitalInfoVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description: TODO
 * @date 2019年6月12日 下午3:25:36
 */
@Component
public class SysSetFacade extends SysSetServiceImpl {

    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;
    @Autowired
    DoctorInfoFacade doctorInfoFacade;

    /**
     * @param hospitalSetVO
     * @return
     */
    public List<SysSetInfoDTO> getSysSetInfoData(HospitalSetVO hospitalSetVO) {

        if (IntegerUtil.isNull(hospitalSetVO.getPlan())) {
            hospitalSetVO.setPlan(0);
        }
        List<SysSet> sysSetData = querySysSetInfo(hospitalSetVO);
        List<SysSetInfoDTO> data = BeanUtil.listCopyTo(sysSetData, SysSetInfoDTO.class);
        return data;
    }

    /**
     * @param sysHospitalCodeVO
     * @return
     */
    public SysHospitalDTO getHospitalDeptInfoData(SysHospitalCodeVO sysHospitalCodeVO) {
        //取出下级医院Code
        QueryWrapper<SysSet> sysSetInfo = new QueryWrapper<>();
        sysSetInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
        sysSetInfo.eq("hospital_code", sysHospitalCodeVO.getHospitalCode());
        sysSetInfo.eq("code", "junior_hospital");
        sysSetInfo.eq("sys_type", SysTypeEnum.PREC_SERVICE.getKey());
        SysSet sysSetInfoData = getOne(sysSetInfo, false);
        SysHospitalInfoVO sysHospitalInfoVO = new SysHospitalInfoVO();
        List<String> listCode = Lists.newArrayList(sysSetInfoData.getValue().split(","));
        //追加集团code
        listCode.add(sysHospitalCodeVO.getHospitalCode());
        //查询下级医院的默认科室
        sysHospitalInfoVO.setHospitalCode(listCode);
        sysHospitalInfoVO.setCode("default_dept");
        sysHospitalInfoVO.setSysType(SysTypeEnum.PREC_SERVICE.getKey());
        List<SysHospitalDeptInfoDTO> juniorHospitalData = getjuniorHospitalData(sysHospitalInfoVO);
        SysHospitalDTO sysHospitalDTO = new SysHospitalDTO();
        List<SysHospitalDeptInfoDTO> hospitalDatas = new ArrayList<SysHospitalDeptInfoDTO>();
        for (SysHospitalDeptInfoDTO superiorsInfoData : juniorHospitalData) {
            if (superiorsInfoData.getHospitalCode().equals(sysHospitalCodeVO.getHospitalCode())) {
                //取出顶级集团信息
                sysHospitalDTO.setSuperiorsInfo(superiorsInfoData);
            } else {
                //下级医院信息
                hospitalDatas.add(superiorsInfoData);
            }
        }
        sysHospitalDTO.setJuniorHospital(hospitalDatas);
        return sysHospitalDTO;
    }

    /**
     * 获取大小医院信息
     *
     * @param getHospitalRankInfoVO
     * @return
     */
    public GetHospitalRankInfoDTO getHospitalRankInfo(GetHospitalRankInfoVO getHospitalRankInfoVO) {
        QueryWrapper<SysSet> sysSetQe = new QueryWrapper<>();
        sysSetQe.and(qe1 -> qe1
                .and(qe2 -> qe2
                        .eq("hospital_code", getHospitalRankInfoVO.getHospitalCode())
                        .eq("sys_type", SysTypeEnum.ICSS_SERVICE.getKey())
                        .eq("code", "connect_prec"))
                .or(qe3 -> qe3
                        .eq("sys_type", SysTypeEnum.PREC_SERVICE.getKey())
                        .eq("code", "junior_hospital")
                        .like("value", getHospitalRankInfoVO.getHospitalCode())));
        Map<String, SysSet> sysSetMap = list(sysSetQe).stream()
                .collect(Collectors.toMap(SysSet::getCode, i -> i));

        SysSet connectPrecSysSet = sysSetMap.get("connect_prec");
        if (getHospitalRankInfoVO.getIsCkConnectPrec()==1
                &&(connectPrecSysSet == null
                || StringUtil.isBlank(connectPrecSysSet.getValue())
                || connectPrecSysSet.getValue().equals("0"))) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "不支持同时对接预问诊");
        }

        String hospitalCode = getHospitalRankInfoVO.getHospitalCode(), sonHospitalCode = null;
        SysSet juniorHospitalSysSet = sysSetMap.get("junior_hospital");
        if (juniorHospitalSysSet != null) {
            if (!Lists.newArrayList(juniorHospitalSysSet.getValue().split(",")).contains(hospitalCode)) {
                throw new CommonException(CommonErrorCode.RPC_ERROR);
            }
            hospitalCode = juniorHospitalSysSet.getHospitalCode();
            sonHospitalCode = getHospitalRankInfoVO.getHospitalCode();
        }

        QueryWrapper<HospitalInfo> hospitalInfoQe = new QueryWrapper<>();
        hospitalInfoQe.eq("code", hospitalCode)
                .or(StringUtil.isNotBlank(sonHospitalCode),
                        i -> i.eq("code", getHospitalRankInfoVO.getHospitalCode()));
        Map<String, HospitalInfo> hospitalInfoMap = hospitalInfoFacade.list(hospitalInfoQe)
                .stream().collect(Collectors.toMap(HospitalInfo::getCode, i -> i));

        HospitalInfo hospitalInfo = hospitalInfoMap.get(hospitalCode);
        HospitalInfo sonHospitalInfo = hospitalInfoMap.get(sonHospitalCode);
        GetHospitalRankInfoDTO getHospitalRankInfoDTO = new GetHospitalRankInfoDTO();
        getHospitalRankInfoDTO.setHospitalId(hospitalInfo.getId());
        getHospitalRankInfoDTO.setHospitalCode(hospitalInfo.getCode());
        getHospitalRankInfoDTO.setHospitalName(hospitalInfo.getName());
        if (sonHospitalInfo != null) {
            getHospitalRankInfoDTO.setSonHospitalId(sonHospitalInfo.getId());
            getHospitalRankInfoDTO.setSonHospitalCode(sonHospitalInfo.getCode());
            getHospitalRankInfoDTO.setSonHospitalName(sonHospitalInfo.getName());
        }

        return getHospitalRankInfoDTO;
    }

    /**
     * 获取父级医院编码
     *
     * @param hospitalCode
     * @return
     */
    public String getParentHospitalCode(String hospitalCode) {
        if (StringUtil.isBlank(hospitalCode)) {
            return null;
        }

        QueryWrapper<SysSet> sysSetQe = new QueryWrapper<>();
        sysSetQe.eq("sys_type", SysTypeEnum.PREC_SERVICE.getKey())
                .eq("code", "junior_hospital")
                .like("value", hospitalCode);
        SysSet juniorHospitalSysSet = getOne(sysSetQe);

        if (juniorHospitalSysSet != null) {
            if (Lists.newArrayList(juniorHospitalSysSet.getValue().split(",")).contains(hospitalCode)) {
                return juniorHospitalSysSet.getHospitalCode();
            }
        }

        return null;
    }

    /**
     * @param hospitalCodeSetVO
     * @return
     */
    public Boolean getAdminCheckout(HospitalCodeSetVO hospitalCodeSetVO) {
        boolean res = false;
        DoctorInfoFindDTO data = doctorInfoFacade.findDoctorInfo(hospitalCodeSetVO);
        if (data == null) {
            throw new CommonException(CommonErrorCode.RPC_ERROR, "该用户不存在");
        }
        if (!"LT".equals(data.getHospitalCode())){
            throw new CommonException(CommonErrorCode.RPC_ERROR, "非法管理员用户");
        }
        QueryWrapper<SysSet> sysSetInfo = new QueryWrapper<>();
        sysSetInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
        sysSetInfo.eq("hospital_code", data.getHospitalCode());
        sysSetInfo.eq("code", "template_admin");
        sysSetInfo.eq("sys_type", SysTypeEnum.ICSS_SERVICE.getKey());
        sysSetInfo.apply("find_in_set({0}, value)", data.getDoctorCode());
        if (this.count(sysSetInfo) < 1) {
            res = false;
            if (hospitalCodeSetVO.getShowError()) {
                throw new CommonException(CommonErrorCode.RPC_ERROR, "该医院暂无模板管理者配置信息");
            }
        } else {
            res = true;
        }
        return res;
    }
}
