package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.LisConfigDTO;
import com.diagbot.entity.TranLisConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.TranLisConfigServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.LisConfigVO;
import com.diagbot.vo.LisHospitalCodeVO;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 
 * @author wangfeng
 * @Description: TODO
 * @date 2019年2月13日 下午1:34:31
 */
@Component
public class TranLisConfigFacade extends TranLisConfigServiceImpl{

	/**
	 * 根据医院编码查询公表映射配置
	 *
	 * @param lisHospitalCodeVO
	 * @return
	 */
	public List<LisConfigDTO> getLisConfigByhospitalIds(LisHospitalCodeVO lisHospitalCodeVO) {

		/*QueryWrapper<TranLisConfig> lisConfig = new QueryWrapper<>();
	    //System.out.println("hospitalCode==="+hospitalCode);
		lisConfig.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("hospital_code", hospitalCode);
		List<TranLisConfig> data = list(lisConfig);
		List<LisConfigDTO> datas = BeanUtil.listCopyTo(data, LisConfigDTO.class);
		//System.out.println("json=="+GsonUtil.toJson(datas));
		return datas;*/
		List<LisConfigDTO> datas = getLisConfigByhospitalId(lisHospitalCodeVO);
		return datas;
	}

	/**
	 * 根据医院编码获取化验公表映射关系 Map<mealName,Map<itemName,List<uniqueName>>>
	 *
	 * @return
	 */
	public Map<String, Map<String, List<String>>> getLisConfigByMealNameAndHosCode(LisConfigVO lisConfigVO) {
		Map<String, Map<String, List<String>>> retMap = new LinkedHashMap<>();
		QueryWrapper<TranLisConfig> lisConfigQueryWrapper = new QueryWrapper<>();
		lisConfigQueryWrapper
				.select("hospital_code,meal_name,ifnull(item_name,'') as item_name,unique_name,unique_id")
				.eq("is_deleted", IsDeleteEnum.N.getKey())
				.eq("hospital_code", lisConfigVO.getHosCode());
		if (ListUtil.isNotEmpty(lisConfigVO.getMealNameList())) {
			lisConfigQueryWrapper.in("meal_name", lisConfigVO.getMealNameList());
		}
		List<TranLisConfig> lisConfigList = this.list(lisConfigQueryWrapper);
		Map<String, List<TranLisConfig>> lisConfigMap
				= EntityUtil.makeEntityListMap(lisConfigList, "mealName");
		for (Map.Entry<String, List<TranLisConfig>> entry : lisConfigMap.entrySet()) {
			if (entry.getValue().size() > 0) {
				Map<String, List<TranLisConfig>> itemMap
						= EntityUtil.makeEntityListMap(entry.getValue(), "itemName");
				Map<String, List<String>> itemNameMap = new LinkedHashMap<>();
				for (Map.Entry<String, List<TranLisConfig>> itemEntry : itemMap.entrySet()) {
					if (ListUtil.isNotEmpty(itemEntry.getValue())) {
						itemNameMap.put(itemEntry.getKey(), itemEntry
								.getValue()
								.stream()
								.map(lisConfig -> lisConfig.getUniqueName())
								.collect(Collectors.toList()));
					}
				}
				retMap.put(entry.getKey(), itemNameMap);
			} else {
				retMap.put(entry.getKey(), null);
			}
		}
		return retMap;
	}


	/**
	 * 根据医院编码获取化验公表映射关系，公表项做key  Map<uniqueName,List<mealName>>
	 *
	 * @return
	 */
	public Map<String, List<String>> getLisConfigByUniqueNameAndHosCode(LisConfigVO lisConfigVO) {
		Map<String, List<String>> retMap = new LinkedHashMap<>();
		QueryWrapper<TranLisConfig> tranLisConfigQueryWrapper = new QueryWrapper<>();
		tranLisConfigQueryWrapper
				.eq("is_deleted", IsDeleteEnum.N.getKey())
				.eq("hospital_code", lisConfigVO.getHosCode());
		if (ListUtil.isNotEmpty(lisConfigVO.getUniqueNameList())) {
			tranLisConfigQueryWrapper.in("unique_name", lisConfigVO.getUniqueNameList());
		}
		List<TranLisConfig> tranLisConfigList = this.list(tranLisConfigQueryWrapper);
		Map<String, List<TranLisConfig>> lisConfigMap
				= EntityUtil.makeEntityListMap(tranLisConfigList, "uniqueName");
		for (Map.Entry<String, List<TranLisConfig>> entry : lisConfigMap.entrySet()) {
			retMap.put(entry.getKey(), EntityUtil.makeListByFieldName(entry.getValue(), "mealName"));
		}
		return retMap;
	}
}