package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetExportInquirysDTO;
import com.diagbot.dto.HisInquirysForJzDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.InquiryInfoFacade;
import com.diagbot.vo.AddDiagnoseSecondVO;
import com.diagbot.vo.HisInquirysForJzVO;
import com.diagbot.vo.SaveInquiryToHisVO;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * <p>
 * 对接层问诊信息控制器
 * </p>
 *
 * @author rengb
 * @since 2019-3-18
 */
@RestController
@RequestMapping("/inquiryInfo")
@Api(value = "对接层-问诊信息API", tags = { "对接层-问诊信息API" })
@SuppressWarnings("unchecked")
public class InquiryInfoController {

    @Autowired
    private InquiryInfoFacade inquiryInfoFacade;

    /**
     * 问诊记录保存
     *
     * @param saveInquiryToHisVO
     * @return
     */
    @PostMapping("/saveInquiryToHis")
    @SysLogger("saveInquiryToHis")
    @Transactional
    @ApiIgnore
    public RespDTO<Boolean> saveInquiryToHis(@RequestBody SaveInquiryToHisVO saveInquiryToHisVO) {
        return RespDTO.onSuc(inquiryInfoFacade.saveInquiryToHis(saveInquiryToHisVO));
    }

    /**
     * 急诊-历史病历列表
     *
     * @param hisInquirysForJzVO
     * @return
     */
    @PostMapping("/hisInquirysForJz")
    @SysLogger("hisInquirysForJz")
    @ApiIgnore
    public RespDTO<HisInquirysForJzDTO> hisInquirysForJz(@RequestBody HisInquirysForJzVO hisInquirysForJzVO) {
        return RespDTO.onSuc(inquiryInfoFacade.hisInquirysForJz(hisInquirysForJzVO));
    }

    /**
     * 急诊-添加二次诊断
     *
     * @param addDiagnoseSecondVO
     * @return
     */
    @PostMapping("/addDiagnoseSecond")
    @SysLogger("addDiagnoseSecond")
    @Transactional
    @ApiIgnore
    public RespDTO<Boolean> addDiagnoseSecond(@RequestBody AddDiagnoseSecondVO addDiagnoseSecondVO) {
        return RespDTO.onSuc(inquiryInfoFacade.addDiagnoseSecond(addDiagnoseSecondVO));
    }

    /**
     * 急诊-获取需要导出的问诊数据
     *
     * @param inquiryIds
     * @return
     */
    @PostMapping("/getExportInquirys")
    @SysLogger("getExportInquirys")
    @ApiIgnore
    public RespDTO<List<GetExportInquirysDTO>> getExportInquirys(@RequestBody List<Long> inquiryIds) {
        return RespDTO.onSuc(inquiryInfoFacade.getExportInquirys(inquiryIds));
    }


}
