package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.HospItemDTO;
import com.diagbot.dto.HospNameDTO;
import com.diagbot.dto.OptDTO;
import com.diagbot.dto.OptHospDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.OptInfoFacade;
import com.diagbot.vo.HospitalVO;
import com.diagbot.vo.OptHospPageVO;
import com.diagbot.vo.OptHospVO;
import com.diagbot.vo.OptVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 操作信息 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-03-20
 */
@RestController
@Api(value = "操作记录", tags = { "操作记录API" })
@SuppressWarnings("unchecked")
@RequestMapping("/optInfo")
public class OptInfoController {
    @Autowired
    private OptInfoFacade optInfoFacade;

    @ApiOperation(value = "操作记录保存[by:gaodm]")
    @PostMapping("/saveOptInfo")
    @SysLogger("saveOptInfo")
    @Transactional
    public RespDTO<Boolean> saveOptOnfo(@RequestBody HospitalVO hospitalVO) {
        return RespDTO.onSuc(optInfoFacade.saveOptOnfo(hospitalVO, "1"));
    }

    @ApiOperation(value = "获取合计数据不带医院名称[by:gaodm]",
            notes = "hospitalCode： 医院名称，String<br>" +
                    "startDate： 开始时间，Date<br>" +
                    "endDate： 结束时间，Date<br>")
    @PostMapping("/getOptSum")
    @SysLogger("getOptSum")
    public RespDTO<OptDTO> getOptSum(@RequestBody OptVO optVO) {
        return RespDTO.onSuc(optInfoFacade.getOptSum(optVO));
    }


    @ApiOperation(value = "获取合计数据带医院名称[by:gaodm]",
            notes = "hospitalCode： 医院名称，String<br>" +
                    "startDate： 开始时间，Date<br>" +
                    "endDate： 结束时间，Date<br>")
    @PostMapping("/getOptSumHosp")
    @SysLogger("getOptSumHosp")
    public RespDTO<OptHospDTO> getOptSumHosp(@RequestBody OptVO optVO) {
        return RespDTO.onSuc(optInfoFacade.getOptSumHosp(optVO));
    }

    @ApiOperation(value = "获取所有医院名称[by:gaodm]",
            notes = "")
    @PostMapping("/getAllHospitalName")
    @SysLogger("getAllHospitalName")
    public RespDTO<HospNameDTO> getAllHospitalName() {
        return RespDTO.onSuc(optInfoFacade.getAllHospitalName());
    }

    @ApiOperation(value = "获取所有医院名称,朗通通用排第一[by:zhoutg]",
            notes = "")
    @PostMapping("/getAllHospitalNameWithLtFrist")
    @SysLogger("getAllHospitalNameWithLtFrist")
    public RespDTO<HospNameDTO> getAllHospitalNameWithLtFrist() {
        return RespDTO.onSuc(optInfoFacade.getAllHospitalNameWithLtFrist());
    }


    @ApiOperation(value = "获取合计数据不带医院名称,只统计总数据，不统计每天数据[by:zhoutg]",
            notes = "hospitalCode： 医院名称，String<br>" +
                    "startDate： 开始时间，Date<br>" +
                    "endDate： 结束时间，Date<br>" +
                    "出参：<br>" +
                    "pvSum：访问总量<br>" +
                    "uvSum：用户总量<br>" +
                    "zzSum：完成问诊总量<br>")
    @PostMapping("/getOptSumNoDays")
    @SysLogger("getOptSumNoDays")
    public RespDTO<OptDTO> getOptSumNoDays(@RequestBody OptVO optVO) {
        return RespDTO.onSuc(optInfoFacade.getOptSumNoDays(optVO));
    }


    @ApiOperation(value = "获取合计数据带医院名称，只统计总数，不统计每天数据[by:zhoutg]",
            notes = "hospitalCodeList： 医院名称列表，List<br>" +
                    "startDate： 开始时间，Date<br>" +
                    "endDate： 结束时间，Date<br>")
    @PostMapping("/getOptSumHospNoDays")
    @SysLogger("getOptSumHospNoDays")
    public RespDTO<List<HospItemDTO>> getOptSumHospNoDays(@RequestBody OptHospVO optHospVO) {
        return RespDTO.onSuc(optInfoFacade.getOptSumHospNoDays(optHospVO));
    }

    @ApiOperation(value = "获取合计数据带医院名称，只统计总数，不统计每天数据，分页[by:zhoutg]",
            notes = "hospitalCodeList： 医院名称列表，List<br>" +
                    "startDate： 开始时间，Date<br>" +
                    "endDate： 结束时间，Date<br>" +
                    "desc：排序指标，[pvSum|uvSum|zzSum]<br>" +
                    "page；第几页<br>" +
                    "size：条数<br>" +
                    "typeList：类型数组，默认都会返回访问量，有uvSum：再返回用户量，有zzSum：再返回提交量")
    @PostMapping("/getOptSumHospNoDaysPage")
    @SysLogger("getOptSumHospNoDaysPage")
    public RespDTO<IPage<HospItemDTO>> getOptSumHospNoDaysPage(@RequestBody OptHospPageVO optHospPageVO) {
        IPage<HospItemDTO> data = optInfoFacade.getOptSumHospNoDaysPageFac(optHospPageVO);
        return RespDTO.onSuc(data);
    }
}
