package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.GetTopPatientInfoDjDTO;
import com.diagbot.dto.PatientInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SignInDTO;
import com.diagbot.entity.PatientInfo;
import com.diagbot.facade.PatientInfoDjFacade;
import com.diagbot.facade.PatientInfoFacade;
import com.diagbot.vo.GetTopPatientInfoDjVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.InquiryQuoteVO;
import com.diagbot.vo.PatientInfoVO;
import com.diagbot.vo.SignInVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 患者信息表 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2018-11-19
 */
@RestController
@RequestMapping("/patientInfo")
@Api(value = "患者信息API", tags = { "患者信息API" })
@SuppressWarnings("unchecked")
public class PatientInfoController {
    @Autowired
    private PatientInfoFacade patientInfoFacade;
    @Autowired
    private PatientInfoDjFacade patientInfoDjFacade;

    @ApiOperation(value = "患者信息——查询[by:wangyu]",
            notes = "patientCode：患者编号，必填<br>" +
                    "hospitalCode:医院编号，必填<br>")
    @PostMapping("/getPatientInfo")
    @SysLogger("getPatientInfo")
    @ApiIgnore
    public RespDTO<PatientInfoDTO> getPatientInfo(@Valid @RequestBody PatientInfoVO patientInfoVO) {
        PatientInfoDTO data = patientInfoFacade.getPatientInfo(patientInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "页面顶部病人医生科室信息——查询[by:rengb]",
            notes = "hospitalCode：医院编号，必填<br>" +
                    "hospitalDeptCode：医院科室编号，必填<br>" +
                    "doctorCode：医院医生编号，必填<br>" +
                    "patientCode:医院患者编号，必填<br>")
    @PostMapping("/getTopPatientInfo")
    @SysLogger("getTopPatientInfo")
    @ApiIgnore
    public RespDTO<GetTopPatientInfoDTO> getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO) {
        return RespDTO.onSuc(patientInfoFacade.getTopPatientInfo(getTopPatientInfoVO));
    }

    @PostMapping(value = "/patientInfoMapByIds")
    @SysLogger("patientInfoMapByIds")
    @ApiOperation(value = "根据ids获取病人信息[by:rengb]")
    @ApiIgnore
    RespDTO<Map<Long, PatientInfo>> patientInfoMapByIds(@RequestBody List<Long> ids) {
        Map<Long, PatientInfo> result = patientInfoFacade.patientInfoMapByIds(ids);
        return RespDTO.onSuc(result);
    }

    /**
     * 患者登录[by:rengb]
     */
    @PostMapping("/signIn")
    @SysLogger("signIn")
    @Transactional
    public RespDTO<List<SignInDTO>> signIn(@RequestBody SignInVO signInVO) {
        return RespDTO.onSuc(patientInfoDjFacade.signIn(signInVO));
    }

    /**
     * 对接-页面顶部病人医生科室信息——查询[by:rengb]
     *
     * @param getTopPatientInfoDjVO
     * @return
     */
    @PostMapping("/getTopPatientInfoDj")
    @SysLogger("getTopPatientInfoDj")
    public RespDTO<GetTopPatientInfoDjDTO> getTopPatientInfoDj(@RequestBody GetTopPatientInfoDjVO getTopPatientInfoDjVO) {
        return RespDTO.onSuc(patientInfoDjFacade.getTopPatientInfoDj(getTopPatientInfoDjVO));
    }

    /**
     * 病历引用时，更新患者医生科室信息
     *
     * @param inquiryQuoteVO
     * @return
     */
    @PostMapping("/inquiryQuote")
    @SysLogger("inquiryQuote")
    @Transactional
    public RespDTO<InquiryQuoteVO> inquiryQuote(@RequestBody InquiryQuoteVO inquiryQuoteVO) {
        return RespDTO.onSuc(patientInfoDjFacade.inquiryQuote(inquiryQuoteVO));
    }

}