package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.LisConfigDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.TranLisConfigFacade;
import com.diagbot.vo.LisConfigVO;
import com.diagbot.vo.LisHospitalCodeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 
 * @author wangfeng
 * @Description: 套餐字段映射公表 前端控制器
 * @date 2019年2月13日 上午11:29:43
 */
@RestController
@RequestMapping("/tranLisConfig")
@Api(value = "公表映射配置API[by:wangfeng]", tags = { "WF——公表映射配置API" })
@SuppressWarnings("unchecked")
public class TranLisConfigController {

	@Autowired
	TranLisConfigFacade tranLisConfigFacade;

	@ApiOperation(value = "根据医院编码查询公表映射配置:[by:wangfeng]", notes = "根据医院编码查询公表映射配置")
	@PostMapping("/getLisConfigByhospitalId")
	@SysLogger("getLisConfigByhospitalId")
	public RespDTO<List<LisConfigDTO>> getLisConfigByhospitalId(@RequestBody LisHospitalCodeVO lisHospitalCodeVO) {
		return RespDTO.onSuc(tranLisConfigFacade.getLisConfigByhospitalIds(lisHospitalCodeVO));
	}

	@ApiOperation(value = "根据医院编码和套餐名称获取化验公表映射关系:[by:zhaops]",
			notes = "hosCode: 医院编码，必填<br>")
	@PostMapping("/getLisConfigByMealNameAndHosCode")
	@SysLogger("getLisConfigByMealNameAndHosCode")
	public RespDTO<Map<String, Map<String, List<String>>>> getLisConfigByMealNameAndHosCode(@RequestBody LisConfigVO lisConfigVO) {
		return RespDTO.onSuc(tranLisConfigFacade.getLisConfigByMealNameAndHosCode(lisConfigVO));
	}

	@ApiOperation(value = "根据医院编码获取化验公表映射关系，公表项做key :[by:zhaops]",
			notes = "hosCode: 医院编码，必填<br>")
	@PostMapping("/getLisConfigByUniqueNameAndHosCode")
	@SysLogger("getLisConfigByUniqueNameAndHosCode")
	public RespDTO<Map<String, List<String>>> getLisConfigByUniqueNameAndHosCode(@RequestBody LisConfigVO lisConfigVO) {
		return RespDTO.onSuc(tranLisConfigFacade.getLisConfigByUniqueNameAndHosCode(lisConfigVO));
	}
}