package com.diagbot.client;

import com.diagbot.biz.push.entity.ResponseData;
import com.diagbot.biz.push.entity.ResponseDataWithExplain;
import com.diagbot.biz.push.entity.ResponseDataWithExplainV2;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.ConceptRetrievalExtDTO;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.FeatureDTO;
import com.diagbot.dto.PartSymptomDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.UsualDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.NLPVO;
import com.diagbot.vo.PartSymptomVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.VersionVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 调用知识库服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {

    @PostMapping(value = "/part/getSymptomByPartName")
    RespDTO<List<PartSymptomDTO>> getSymptomByPartName(@RequestBody PartSymptomVO partSymptomVO);

    /**
     * 获取常用标签
     *
     * @param conceptUsualVO
     * @return
     */
    @PostMapping(value = "/concept/getConceptUsual")
    RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    /**
     * 获取常用标签附带通俗解释和详细说明
     *
     * @param conceptUsualVO
     * @return
     */
    @PostMapping(value = "/concept/getConceptExtUsual")
    RespDTO<List<UsualDTO>> getConceptExtUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    /**
     * 标签检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfo")
    RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@RequestBody @Valid RetrievalVO retrievalVO);


    /**
     * 标签检索(附带通俗解释和详细说明, retrivelConceptInfo这个作废)
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfoExt")
    RespDTO<List<ConceptRetrievalExtDTO>> retrivelConceptInfoExt(@RequestBody @Valid RetrievalVO retrievalVO);


    @PostMapping(value = "/clinicaldata/processData")
    RespDTO<ResponseData> aiptData(@RequestBody SearchVO searchVO);

    /**
     * 推理返回大数据原始结果
     *
     * @param searchVO
     * @return
     */
    @PostMapping(value = "/pushV2/originalPush")
    RespDTO<ResponseData> originalPush(@Valid @RequestBody SearchVO searchVO);

    /**
     * 推理带名词解释
     *
     * @param searchVO
     * @return
     */
    @PostMapping(value = "/push/pushWithExplain")
    RespDTO<ResponseDataWithExplain> pushWithExplain(@Valid @RequestBody SearchVO searchVO);

    /**
     * 推理带名词解释
     *
     * @param searchVO
     * @return
     */
    @PostMapping(value = "/pushV2/pushWithExplainV2")
    RespDTO<ResponseDataWithExplainV2> pushWithExplainV2(@Valid @RequestBody SearchVO searchVO);

    /**
     * 获取版本信息
     */
    @PostMapping("/versionInfo/getVersionInfoAlls")
    RespDTO<VersionWrapperDTO> getVersionInfoAlls(@RequestBody VersionVO versionVO);

    /**
     * 症状特征提取
     *
     * @param text
     * @return
     */
    @PostMapping(value = "/feature/symptomFeature")
    RespDTO<List<FeatureConceptDTO>> symptomFeature(@RequestParam("text") String text);

    /**
     * 症状特征提取
     *
     * @param nlpvo
     * @return
     */
    @PostMapping(value = "/featureV2/symptomFeatureV2")
    RespDTO<List<FeatureDTO>> symptomFeatureV2(@RequestBody NLPVO nlpvo);
}