package com.diagbot.client;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.hystrix.TranServiceHystrix;
import com.diagbot.dto.DiseaseDeptDTO;
import com.diagbot.dto.HospItemDTO;
import com.diagbot.dto.HospNameDTO;
import com.diagbot.dto.HospitalDeptDTO;
import com.diagbot.dto.OptDTO;
import com.diagbot.dto.OptHospDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SaveInquiryTriageDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.vo.DiseaseDeptVO;
import com.diagbot.vo.HospitalDeptVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.HospitalVO;
import com.diagbot.vo.OptHospPageVO;
import com.diagbot.vo.OptHospVO;
import com.diagbot.vo.OptVO;
import com.diagbot.vo.SaveInquiryTriageVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/8/20 10:12
 */
@FeignClient(value = "tran-service", fallback = TranServiceHystrix.class)
public interface TranServiceClient {
    @PostMapping(value = "/hospitalDeptInfo/hospitalDeptByConceptNames")
    RespDTO<Map<String, List<HospitalDeptDTO>>> hospitalDeptByConceptNames(@RequestBody HospitalDeptVO hospitalDeptVO);

    @PostMapping("/sysSet/getSysSetInfoDatas")
    RespDTO<List<SysSetInfoDTO>> getSysSetInfoDatas(@Valid @RequestBody HospitalSetVO hospitalSetVO);

    /**
     * 根据诊断名称（内部名称）查询诊断科室映射关系
     *
     * @param diseaseDeptVO
     * @return
     */
    @PostMapping("/diseaseDept/getDiseaseDeptByConceptNames")
    RespDTO<List<DiseaseDeptDTO>> getDiseaseDeptByConceptNames(DiseaseDeptVO diseaseDeptVO);

    @PostMapping("/inquiryTriage/saveInquiry")
    public RespDTO<SaveInquiryTriageDTO> saveInquiry(@RequestBody SaveInquiryTriageVO saveInquiryVO);

    @PostMapping("/optInfo/saveOptInfo")
    public RespDTO<Boolean> saveOptOnfo(@RequestBody HospitalVO hospitalVO);

    @PostMapping("/optInfo/getOptSum")
    public RespDTO<OptDTO> getOptSum(@RequestBody OptVO optVO);

    @PostMapping("/optInfo/getOptSumHosp")
    public RespDTO<OptHospDTO> getOptSumHosp(@RequestBody OptVO optVO);

    @PostMapping("/optInfo/getAllHospitalName")
    public RespDTO<HospNameDTO> getAllHospitalName();

    @PostMapping("/optInfo/getAllHospitalNameWithLtFrist")
    public RespDTO<HospNameDTO> getAllHospitalNameWithLtFrist();

    @PostMapping("/optInfo/getOptSumNoDays")
    public RespDTO<OptDTO> getOptSumNoDays(@RequestBody OptVO optVO);

    @PostMapping("/optInfo/getOptSumHospNoDays")
    public RespDTO<List<HospItemDTO>> getOptSumHospNoDays(@RequestBody OptHospVO optHospVO);

    @PostMapping("/optInfo/getOptSumHospNoDaysPage")
    public RespDTO<IPage<HospItemDTO>> getOptSumHospNoDaysPage(@RequestBody OptHospPageVO optHospPageVO);
}