package com.diagbot.client.hystrix;

import com.diagbot.biz.push.entity.ResponseData;
import com.diagbot.biz.push.entity.ResponseDataWithExplain;
import com.diagbot.biz.push.entity.ResponseDataWithExplainV2;
import com.diagbot.biz.push.vo.SearchVO;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.ConceptRetrievalExtDTO;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.FeatureDTO;
import com.diagbot.dto.PartSymptomDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.UsualDTO;
import com.diagbot.dto.VersionWrapperDTO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.NLPVO;
import com.diagbot.vo.PartSymptomVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.VersionVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 调用知识库服务
 * @author: gaodm
 * @time: 2019/2/1 17:00
 */
@Component
@Slf4j
public class AiptServiceHystrix implements AiptServiceClient {
    @Override
    public RespDTO<List<PartSymptomDTO>> getSymptomByPartName(PartSymptomVO partSymptomVO) {
        log.error("【hystrix】调用{}异常", "getSymptomByPartName");
        return null;
    }

    @Override
    public RespDTO<List<ConceptBaseDTO>> getConceptUsual(@Valid ConceptUsualVO conceptUsualVO) {
        log.error("【hystrix】调用{}异常", "getConceptUsual");
        return null;
    }

    @Override
    public RespDTO<List<UsualDTO>> getConceptExtUsual(@Valid ConceptUsualVO conceptUsualVO) {
        log.error("【hystrix】调用{}异常", "getConceptExtUsual");
        return null;
    }

    @Override
    public RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@Valid RetrievalVO retrievalVO) {
        log.error("【hystrix】调用{}异常", "retrivelConceptInfo");
        return null;
    }

    @Override
    public RespDTO<List<ConceptRetrievalExtDTO>> retrivelConceptInfoExt(@Valid RetrievalVO retrievalVO) {
        log.error("【hystrix】调用{}异常", "retrivelConceptInfoExt");
        return null;
    }

    @Override
    public RespDTO<ResponseData> aiptData(SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "aiptData");
        return null;
    }

    /**
     * 推理返回大数据原始结果
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<ResponseData> originalPush(@Valid @RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "originalPush");
        return null;
    }

    /**
     * 推理带名词解释
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<ResponseDataWithExplain> pushWithExplain(@Valid @RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "pushWithExplain");
        return null;
    }

    /**
     * 推理带名词解释
     *
     * @param searchVO
     * @return
     */
    @Override
    public RespDTO<ResponseDataWithExplainV2> pushWithExplainV2(@Valid @RequestBody SearchVO searchVO) {
        log.error("【hystrix】调用{}异常", "pushWithExplainV2");
        return null;
    }

    /**
     * 获取版本信息
     */
    @Override
    public RespDTO<VersionWrapperDTO> getVersionInfoAlls(VersionVO versionVO) {
        log.error("【hystrix】调用{}异常", "getVersionInfoAlls");
        return null;
    }


    /**
     * 症状特征提取
     *
     * @param text
     * @return
     */
    @Override
    public RespDTO<List<FeatureConceptDTO>> symptomFeature(@RequestParam("text") String text) {
        log.error("【hystrix】调用{}异常", "symptomFeature");
        return null;
    }

    /**
     * 症状特征提取
     *
     * @param nlpvo
     * @return
     */
    @Override
    public RespDTO<List<FeatureDTO>> symptomFeatureV2(@RequestBody NLPVO nlpvo) {
        log.error("【hystrix】调用{}异常", "symptomFeatureV2");
        return null;
    }
}