package com.diagbot.facade;

import com.diagbot.biz.push.entity.FeatureRate;
import com.diagbot.biz.push.entity.FeatureRateWithExplain;
import com.diagbot.biz.push.entity.ResponseDataWithExplain;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.TranServiceClient;
import com.diagbot.dto.AIDTO;
import com.diagbot.dto.DiseaseDeptDTO;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.FeatureRateDTO;
import com.diagbot.dto.HospitalDeptDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SYFDTO;
import com.diagbot.dto.SYFDeptDTO;
import com.diagbot.dto.SYFDiseaseDeptDTO;
import com.diagbot.dto.SYFFeatureDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.AIVO;
import com.diagbot.vo.DiseaseDeptVO;
import com.diagbot.vo.HospitalDeptVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 调用AI服务业务实现
 * @author: gaodm
 * @time: 2018/10/25 10:09
 */
@Component
public class AIFacade {
    @Autowired
    private AiptServiceClient aiptServiceClient;
    @Autowired
    private TranServiceClient tranServiceClient;
    @Autowired
    private FeatureFacade featureFacade;
    @Autowired
    private AssembleFacade assembleFacade;

    /**
     * 调用ai接口业务逻辑
     *
     * @param aivo 搜索参数
     * @return AI接口返回结果
     */
    public AIDTO push(AIVO aivo, String type) {
        RespDTO<ResponseDataWithExplain> res = aiptServiceClient.pushWithExplain(assembleFacade.assembleData(aivo, type));
        RespDTOUtil.respNGDealCover(res, "中间层没有结果返回");
        AIDTO aidto = new AIDTO();
        switch (type) {
            case "1":
                List<FeatureRateWithExplain> symptom = res.data.getSymptom();
                if (ListUtil.isNotEmpty(symptom)) {
                    List<FeatureRateDTO> symptomDTO = BeanUtil.listCopyTo(symptom, FeatureRateDTO.class);
                    aidto.setItems(symptomDTO);
                }
                break;
            case "7":
                List<FeatureRateDTO> items = new ArrayList<>();
                if (ListUtil.isNotEmpty(res.data.getDis())) {
                    Map<String, List<FeatureRateDTO>> disFeatureMap = new LinkedHashMap<>();
                    List<FeatureRate> dis = res.data.getDis();
                    if (ListUtil.isEmpty(dis)) {
                        break;
                    }
                    List<FeatureRateDTO> disDTO = BeanUtil.listCopyTo(dis, FeatureRateDTO.class);
                    //诊断分类
                    for (FeatureRateDTO featureRateDTO : disDTO) {
                        if (StringUtil.isBlank(featureRateDTO.getDesc())) {
                            featureRateDTO.setDesc("{\"可能诊断\":\"\"}");
                        }
                        Map<String, Object> descMap = FastJsonUtils.getJsonToMap(featureRateDTO.getDesc());
                        for (String disClass : descMap.keySet()) {
                            List<FeatureRateDTO> featureRateDTOList = Lists.newLinkedList();
                            if (disFeatureMap.get(disClass) != null) {
                                featureRateDTOList = disFeatureMap.get(disClass);
                            }
                            featureRateDTOList.add(featureRateDTO);
                            disFeatureMap.put(disClass, featureRateDTOList);
                        }
                    }

                    FeatureRateDTO item;
                    if (disFeatureMap.get("确诊") != null) {
                        items = getItems(disFeatureMap, "确诊", 1);
                    }
                    if (disFeatureMap.get("拟诊") != null && ListUtil.isEmpty(items)) {
                        items = getItems(disFeatureMap, "拟诊", 2);
                    }
                    if (disFeatureMap.get("可能诊断") != null && ListUtil.isEmpty(items)) {
                        Double rate;
                        Map<String, Double> map = new HashMap<>();
                        List<FeatureRateDTO> featureRateList = disFeatureMap.get("可能诊断");
                        featureRateList = featureRateList
                                .stream()
                                .filter(i -> i.getExtraProperty() != null)
                                .filter(i -> i.getRate() != null)
                                .collect(Collectors.toList());
                        for (FeatureRate featureRate : featureRateList) {
                            if (featureRate.getRate() != null && Double.valueOf(featureRate.getRate()) > 0.05d) {
                                if (StringUtil.isNotBlank(featureRate.getExtraProperty())) {
                                    if (null != map.get(featureRate.getExtraProperty())) {
                                        rate = BigDecimal.valueOf(map.get(featureRate.getExtraProperty()))
                                                .add(BigDecimal.valueOf(Double.valueOf(featureRate.getRate())))
                                                .doubleValue();
                                        map.replace(featureRate.getExtraProperty(), rate);
                                    } else {
                                        map.put(featureRate.getExtraProperty(), Double.valueOf(featureRate.getRate()));
                                    }
                                }
                            }
                        }
                        List<Map.Entry<String, Double>> list = new ArrayList<Map.Entry<String, Double>>(map.entrySet());
                        Collections.sort(list, new Comparator<Map.Entry<String, Double>>() {
                            //降序排序
                            public int compare(Map.Entry<String, Double> o1,
                                               Map.Entry<String, Double> o2) {
                                return o2.getValue().compareTo(o1.getValue());
                            }
                        });
                        for (Map.Entry<String, Double> entry : list) {
                            if (items.size() < 2) {
                                item = new FeatureRateDTO();
                                item.setExtraProperty(entry.getKey());
                                items.add(item);
                            } else {
                                break;
                            }
                        }
                        if (ListUtil.isEmpty(items)) {
                            item = new FeatureRateDTO();
                            item.setExtraProperty(res.data.getDis().get(0).getExtraProperty());
                            items.add(item);
                        }
                    }
                }
                if (StringUtil.isNotBlank(aivo.getHospitalCode())) {
                    //获取对接科室
                    if (ListUtil.isNotEmpty(items)) {
                        HospitalDeptVO hospitalDeptVO = new HospitalDeptVO();
                        hospitalDeptVO.setHospitalCode(aivo.getHospitalCode());
                        hospitalDeptVO.setConceptNames(items
                                .stream()
                                .map(i -> i.getExtraProperty())
                                .collect(Collectors.toList()));
                        RespDTO<Map<String, List<HospitalDeptDTO>>> retDeptMap
                                = tranServiceClient.hospitalDeptByConceptNames(hospitalDeptVO);
                        if (RespDTOUtil.respIsOK(retDeptMap)) {
                            Map<String, List<HospitalDeptDTO>> deptMap = retDeptMap.data;
                            for (FeatureRateDTO item : items) {
                                item.setHospitalDepts(deptMap.get(item.getExtraProperty()));
                            }
                        }
                    }
                }
                aidto.setItems(items);
                break;
        }
        return aidto;
    }

    /**
     * 调用ai接口业务逻辑
     *
     * @param aivo 搜索参数
     * @return AI接口返回结果
     */
    public SYFDTO pushDisForSYF(AIVO aivo, String type) {
        if (StringUtil.isBlank(aivo.getHospitalCode())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入医院编码");
        }
        //判断症状个数
        List<FeatureConceptDTO> featureConceptDTOList = featureFacade.getSymptomFeature(aivo.getSymptom());
        Integer symptomSize = ListUtil.isEmpty(featureConceptDTOList) ? 0 : featureConceptDTOList.size();

        RespDTO<ResponseDataWithExplain> res = aiptServiceClient.pushWithExplain(assembleFacade.assembleData(aivo, type));
        RespDTOUtil.respNGDealCover(res, "中间层没有结果返回");
        SYFDTO syfdto = new SYFDTO();

        List<SYFDiseaseDeptDTO> items = new ArrayList<>();
        List<FeatureRate> dis = res.data.getDis();
        if (ListUtil.isNotEmpty(dis)) {
            Map<String, List<SYFFeatureDTO>> disFeatureMap = new LinkedHashMap<>();
            List<SYFFeatureDTO> disDTO = BeanUtil.listCopyTo(dis, SYFFeatureDTO.class);

            //关联邵逸夫医院映射
            DiseaseDeptVO diseaseDeptVO = new DiseaseDeptVO();
            diseaseDeptVO.setHospitalCode(aivo.getHospitalCode());
            List<String> disNames = dis.stream().map(i -> i.getFeatureName()).collect(Collectors.toList());
            diseaseDeptVO.setConceptNames(disNames);
            RespDTO<List<DiseaseDeptDTO>> diseaseDeptDTORes
                    = tranServiceClient.getDiseaseDeptByConceptNames(diseaseDeptVO);
            RespDTOUtil.respNGDealCover(diseaseDeptDTORes, "诊断科室映射关系未维护");
            List<DiseaseDeptDTO> diseaseDeptDTOList = diseaseDeptDTORes.data;
            Map<String, List<DiseaseDeptDTO>> diseaseDeptMap
                    = EntityUtil.makeEntityListMap(diseaseDeptDTOList, "conceptDisName");
            //诊断分类
            for (SYFFeatureDTO featureRateDTO : disDTO) {
                //关联外部诊断科室
                if (diseaseDeptMap != null
                        && ListUtil.isNotEmpty(diseaseDeptMap.get(featureRateDTO.getFeatureName()))) {
                    List<DiseaseDeptDTO> ddList = diseaseDeptMap.get(featureRateDTO.getFeatureName());
                    featureRateDTO.setDeptName(ddList.get(0).getDeptName());
                    featureRateDTO.setSubDeptName(ddList.get(0).getSubDeptName());
                    featureRateDTO.setDiseaseName(ddList.get(0).getDiseaseName());
                    featureRateDTO.setDeptCode(ddList.get(0).getDeptCode());
                    featureRateDTO.setSubDeptCode(ddList.get(0).getSubDeptCode());
                    featureRateDTO.setSuggestion(ddList.get(0).getSuggestion());
                    if (StringUtil.isNotBlank(ddList.get(0).getSubDeptName())
                            && (!ddList.get(0).getDeptName().equals(ddList.get(0).getSubDeptName()))) {
                        featureRateDTO
                                .setConcatDept(ddList.get(0).getDeptName() + "-" + ddList.get(0).getSubDeptName());
                    } else {
                        featureRateDTO.setConcatDept(ddList.get(0).getDeptName());
                    }
                    featureRateDTO.setUrl(ddList.get(0).getUrl());
                } else {
                    featureRateDTO.setDiseaseName(featureRateDTO.getFeatureName());
                }
                if (StringUtil.isBlank(featureRateDTO.getDesc())) {
                    featureRateDTO.setDesc("{\"可能诊断\":\"\"}");
                }
                Map<String, Object> descMap = FastJsonUtils.getJsonToMap(featureRateDTO.getDesc());
                for (String disClass : descMap.keySet()) {
                    List<SYFFeatureDTO> featureRateDTOList = Lists.newLinkedList();
                    if (disFeatureMap.get(disClass) != null) {
                        featureRateDTOList = disFeatureMap.get(disClass);
                    }
                    featureRateDTOList.add(featureRateDTO);
                    disFeatureMap.put(disClass, featureRateDTOList);
                }
            }

            if (disFeatureMap.get("确诊") != null) {
                items = getItemsForSYF(disFeatureMap, "确诊", 1);
            }
            if (disFeatureMap.get("拟诊") != null && ListUtil.isEmpty(items)) {
                items = getItemsForSYF(disFeatureMap, "拟诊", 2);
            }
            if (disFeatureMap.get("可能诊断") != null && ListUtil.isEmpty(items)) {
                List<SYFFeatureDTO> featureRateList = disFeatureMap.get("可能诊断");
                List<SYFDiseaseDeptDTO> syfDiseaseDeptList = Lists.newLinkedList();
                Map<String, List<SYFFeatureDTO>> syfFeatureMap = new HashMap<>();
                //根据阈值（0.05）过滤诊断
                featureRateList = featureRateList
                        .stream()
                        .filter(i -> (i.getRate() != null && Double.valueOf(i.getRate()) > 0.05d))
                        .collect(Collectors.toList());

                //对应后名称相同的项合并
                Map<String, List<SYFFeatureDTO>> mergeSYFFeatureMap
                        = EntityUtil.makeEntityListMap(featureRateList, "diseaseName");
                List<SYFFeatureDTO> mergeDisDTO = Lists.newArrayList();
                for (Map.Entry<String, List<SYFFeatureDTO>> entry : mergeSYFFeatureMap.entrySet()) {
                    if (ListUtil.isEmpty(entry.getValue())) {
                        continue;
                    }
                    SYFFeatureDTO syfFeatureDTO = new SYFFeatureDTO();
                    BeanUtil.copyProperties(entry.getValue().get(0), syfFeatureDTO);
                    Double rate = 0d;
                    String conceptDisName = "";
                    for (SYFFeatureDTO featureDTO : entry.getValue()) {
                        rate = BigDecimal.valueOf(rate)
                                .add(BigDecimal.valueOf(Double.valueOf(featureDTO.getRate())))
                                .doubleValue();
                        conceptDisName += featureDTO.getFeatureName() + ",";
                    }
                    syfFeatureDTO.setRate(rate.toString());
                    syfFeatureDTO.setFeatureName(conceptDisName.substring(0, conceptDisName.length() - 1));
                    mergeDisDTO.add(syfFeatureDTO);
                }

                for (SYFFeatureDTO syfFeatureDTO : mergeDisDTO) {
                    if (StringUtil.isNotBlank(syfFeatureDTO.getConcatDept())) {
                        List<SYFFeatureDTO> features = Lists.newLinkedList();
                        if (syfFeatureMap.containsKey(syfFeatureDTO.getConcatDept())) {
                            features = syfFeatureMap.get(syfFeatureDTO.getConcatDept());
                        }
                        features.add(syfFeatureDTO);
                        syfFeatureMap.put(syfFeatureDTO.getConcatDept(), features);

                    } else {
                        SYFDiseaseDeptDTO syfDiseaseDeptDTO = new SYFDiseaseDeptDTO();
                        syfDiseaseDeptDTO.setRate(Double.valueOf(syfFeatureDTO.getRate()));
                        List<SYFFeatureDTO> features = Lists.newLinkedList();
                        features.add(syfFeatureDTO);
                        //syfDiseaseDeptDTO.setFeatures(features);
                        syfDiseaseDeptDTO.setDiseaseNames(features
                                .stream()
                                .map(i -> i.getDiseaseName())
                                .distinct()
                                .collect(Collectors.toList()));
                        syfDiseaseDeptList.add(syfDiseaseDeptDTO);
                    }
                }
                if (syfFeatureMap != null) {
                    for (Map.Entry<String, List<SYFFeatureDTO>> entry : syfFeatureMap.entrySet()) {
                        SYFDiseaseDeptDTO syfDiseaseDeptDTO = new SYFDiseaseDeptDTO();
                        syfDiseaseDeptDTO.setDeptName(entry.getValue().get(0).getDeptName());
                        syfDiseaseDeptDTO.setSubDeptName(entry.getValue().get(0).getSubDeptName());
                        syfDiseaseDeptDTO.setDept(entry.getValue().get(0).getConcatDept());
                        //syfDiseaseDeptDTO.setUrl(entry.getValue().get(0).getUrl());
                        syfDiseaseDeptDTO.setDeptCode(entry.getValue().get(0).getDeptCode());
                        syfDiseaseDeptDTO.setSubDeptCode(entry.getValue().get(0).getSubDeptCode());
                        syfDiseaseDeptDTO.setSuggestion(entry.getValue().get(0).getSuggestion());
                        //syfDiseaseDeptDTO.setFeatures(entry.getValue());
                        syfDiseaseDeptDTO.setDiseaseNames(entry.getValue()
                                .stream()
                                .map(i -> i.getDiseaseName())
                                .distinct()
                                .collect(Collectors.toList()));
                        Double rate = 0d;
                        for (SYFFeatureDTO feature : entry.getValue()) {
                            rate = BigDecimal.valueOf(rate)
                                    .add(BigDecimal.valueOf(Double.valueOf(feature.getRate())))
                                    .doubleValue();
                        }
                        syfDiseaseDeptDTO.setRate(rate);
                        syfDiseaseDeptList.add(syfDiseaseDeptDTO);
                    }
                }
                Collections.sort(syfDiseaseDeptList, new Comparator<SYFDiseaseDeptDTO>() {
                    @Override
                    public int compare(SYFDiseaseDeptDTO o1, SYFDiseaseDeptDTO o2) {
                        return Double.compare(o2.getRate(), o1.getRate());
                    }
                });
                for (SYFDiseaseDeptDTO syfDiseaseDeptDTO : syfDiseaseDeptList) {
                    if (items.size() < 2) {
                        if (StringUtil.isBlank(syfDiseaseDeptDTO.getDept())) {
                            continue;
                        }
                        //输入症状不止一个
                        //第一个科室概率小于0.3，两个科室都推
                        //第一个科室概率大于0.3，第二个科室概率大于等于0.15，两个都推
                        //第一个科室概率大于0.3，第二个科室概率小于0.15，只推第一个
                        if (symptomSize != 1
                                && items.size() == 1
                                && Double.valueOf(items.get(0).getRate()) > 0.3d
                                && Double.valueOf(syfDiseaseDeptDTO.getRate()) < 0.15d) {
                            break;
                        }
                        items.add(syfDiseaseDeptDTO);
                    }
                }
            }
        }
        //按大科室分组展示
        Map<String, List<SYFDiseaseDeptDTO>> itemMap = EntityUtil.makeEntityListMap(items, "deptName");
        List<SYFDeptDTO> syfDeptDTOList = Lists.newLinkedList();
        itemMap.entrySet().forEach(item -> {
            SYFDeptDTO syfDeptDTO = new SYFDeptDTO();
            syfDeptDTO.setDeptName(item.getKey());
            syfDeptDTO.setDeptCode(item.getValue().get(0).getDeptCode());
            syfDeptDTO.setDiseaseDept(item.getValue());
            syfDeptDTOList.add(syfDeptDTO);
        });
        syfdto.setDiseaseDept(syfDeptDTOList);
        return syfdto;
    }

    private List<FeatureRateDTO> getItems(Map<String, List<FeatureRateDTO>> disFeatureMap,
                                          String type, int size) {
        List<FeatureRateDTO> items = new ArrayList<>();

        List<FeatureRateDTO> featureRateList = disFeatureMap.get(type);
        featureRateList = featureRateList
                .stream()
                .filter(i -> i.getExtraProperty() != null)
                .collect(Collectors.toList());
        if (ListUtil.isNotEmpty(featureRateList)) {
            Map<String, List<FeatureRateDTO>> disMap
                    = EntityUtil.makeEntityListMap(featureRateList, "extraProperty");
            List<Map.Entry<String, List<FeatureRateDTO>>> list = new ArrayList<Map.Entry<String, List<FeatureRateDTO>>>(disMap.entrySet());
            Collections.sort(list, new Comparator<Map.Entry<String, List<FeatureRateDTO>>>() {
                //降序排序
                public int compare(Map.Entry<String, List<FeatureRateDTO>> o1,
                                   Map.Entry<String, List<FeatureRateDTO>> o2) {
                    return Integer.compare(o2.getValue().size(), o1.getValue().size());
                }
            });
            FeatureRateDTO item;
            for (Map.Entry<String, List<FeatureRateDTO>> entry : list) {
                if (items.size() < size) {
                    item = new FeatureRateDTO();
                    item.setExtraProperty(entry.getKey());
                    items.add(item);
                } else {
                    break;
                }
            }
        }
        return items;
    }

    /**
     * 邵逸夫科室推理（确诊、拟诊）
     *
     * @param disFeatureMap
     * @param type
     * @param size
     * @return
     */
    private List<SYFDiseaseDeptDTO> getItemsForSYF(Map<String, List<SYFFeatureDTO>> disFeatureMap,
                                                   String type, int size) {
        List<SYFDiseaseDeptDTO> items = new ArrayList<>();
        List<SYFFeatureDTO> featureRateList = disFeatureMap.get(type);
        if (ListUtil.isEmpty(featureRateList)) {
            return items;
        }
        //推送结果根据diseaseName去重
        Iterator<SYFFeatureDTO> syfFeatureDTOIterator = featureRateList.iterator();
        Map<String, String> distinctMap = new HashMap<>();
        while (syfFeatureDTOIterator.hasNext()) {
            SYFFeatureDTO next = syfFeatureDTOIterator.next();
            if (StringUtil.isBlank(next.getDiseaseName())) {
                continue;
            }
            String featureName = "";
            if (distinctMap.containsKey(next.getDiseaseName())) {
                featureName = distinctMap.get(next.getDiseaseName());
                if (StringUtil.isNotBlank(featureName)) {
                    featureName += ",";
                }
                featureName += next.getFeatureName();
                syfFeatureDTOIterator.remove();
                distinctMap.put(next.getDiseaseName(), featureName);
            } else {
                distinctMap.put(next.getDiseaseName(), next.getFeatureName());
            }
        }
        featureRateList.forEach(i -> {
            if (distinctMap.containsKey(i.getDiseaseName())) {
                i.setFeatureName(distinctMap.get(i.getDiseaseName()));
            }
        });

        //根据featureName个数倒序排序
        Collections.sort(featureRateList, new Comparator<SYFFeatureDTO>() {
            @Override
            public int compare(SYFFeatureDTO o1, SYFFeatureDTO o2) {
                return Integer.compare(o2.getFeatureName().split(",").length,
                        o1.getFeatureName().split(",").length);
            }
        });

        List<SYFDiseaseDeptDTO> mergeFeatureList = Lists.newLinkedList();
        Map<String, List<SYFFeatureDTO>> syfFeatureMap = new HashMap<>();
        for (SYFFeatureDTO syfFeatureDTO : featureRateList) {
            if (StringUtil.isNotBlank(syfFeatureDTO.getConcatDept())) {
                SYFDiseaseDeptDTO syfDiseaseDeptDTO = new SYFDiseaseDeptDTO();
                syfDiseaseDeptDTO.setDept(syfFeatureDTO.getConcatDept());
                syfDiseaseDeptDTO.setDeptName(syfFeatureDTO.getDeptName());
                syfDiseaseDeptDTO.setSubDeptName(syfFeatureDTO.getSubDeptName());
                //syfDiseaseDeptDTO.setUrl(syfFeatureDTO.getUrl());
                syfDiseaseDeptDTO.setDeptCode(syfFeatureDTO.getDeptCode());
                syfDiseaseDeptDTO.setSubDeptCode(syfFeatureDTO.getSubDeptCode());
                syfDiseaseDeptDTO.setSuggestion(syfFeatureDTO.getSuggestion());
                List<SYFFeatureDTO> features = Lists.newLinkedList();
                if (syfFeatureMap.containsKey(syfFeatureDTO.getConcatDept())) {
                    features = syfFeatureMap.get(syfFeatureDTO.getConcatDept());
                } else {
                    mergeFeatureList.add(syfDiseaseDeptDTO);
                }
                features.add(syfFeatureDTO);

                //syfDiseaseDeptDTO.setFeatures(features);
                syfDiseaseDeptDTO.setDiseaseNames(features
                        .stream()
                        .map(i -> i.getDiseaseName())
                        .distinct()
                        .collect(Collectors.toList()));
                syfFeatureMap.put(syfFeatureDTO.getConcatDept(), features);
            } else {
                //科室为空的数据，不更改推理结果排序
                SYFDiseaseDeptDTO syfDiseaseDeptDTO = new SYFDiseaseDeptDTO();
                List<SYFFeatureDTO> features = Lists.newLinkedList();
                features.add(syfFeatureDTO);
                //syfDiseaseDeptDTO.setFeatures(features);
                syfDiseaseDeptDTO.setDiseaseNames(features
                        .stream()
                        .map(i -> i.getDiseaseName())
                        .distinct()
                        .collect(Collectors.toList()));
                mergeFeatureList.add(syfDiseaseDeptDTO);
            }
        }
        for (SYFDiseaseDeptDTO syfDiseaseDeptDTO : mergeFeatureList) {
            //更新诊断列表
            if (StringUtil.isNotBlank(syfDiseaseDeptDTO.getDept())) {
                if (syfFeatureMap.containsKey(syfDiseaseDeptDTO.getDept())) {
                    //syfDiseaseDeptDTO.setFeatures(syfFeatureMap.get(syfDiseaseDeptDTO.getDept()));
                    syfDiseaseDeptDTO.setDiseaseNames(syfFeatureMap.get(syfDiseaseDeptDTO.getDept())
                            .stream()
                            .map(i -> i.getDiseaseName())
                            .distinct()
                            .collect(Collectors.toList()));
                }
            }
        }

        //根据出现次数排序
        Collections.sort(mergeFeatureList, new Comparator<SYFDiseaseDeptDTO>() {
            @Override
            public int compare(SYFDiseaseDeptDTO o1, SYFDiseaseDeptDTO o2) {
                return Integer.compare(o2.getDiseaseNames().size(), o1.getDiseaseNames().size());
            }
        });

        for (SYFDiseaseDeptDTO syfDiseaseDeptDTO : mergeFeatureList) {
            if (items.size() < size) {
                //未匹配数据暂不返回
                if (StringUtil.isBlank(syfDiseaseDeptDTO.getDept())) {
                    continue;
                }
                items.add(syfDiseaseDeptDTO);
            } else {
                break;
            }
        }
        return items;
    }
}