package com.diagbot.facade;

import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.FeatureConceptDTO;
import com.diagbot.dto.FeatureDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.NLPVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/2/26 13:27
 */
@Component
public class FeatureFacade {
    @Autowired
    private AiptServiceClient aiptServiceClient;

    /**
     * 症状关键词提取
     *
     * @param text
     * @return
     */
    public List<FeatureConceptDTO> getSymptomFeature(String text) {
        RespDTO<List<FeatureConceptDTO>> res = aiptServiceClient.symptomFeature(text);
        RespDTOUtil.respNGDealCover(res, "AI没有返回结果");
        List<FeatureConceptDTO> symptomFeatureList = res.data;
        if (ListUtil.isNotEmpty(symptomFeatureList)) {
            symptomFeatureList = symptomFeatureList
                    .stream()
                    .filter(i -> i.getLibType().equals(1))
                    .collect(Collectors.toList());

        }
        return symptomFeatureList;
    }

    /**
     * 新版知识库症状特征词提取
     *
     * @param nlpvo
     * @return
     */
    public List<FeatureDTO> getSymptomFeatureV2(NLPVO nlpvo) {
        RespDTO<List<FeatureDTO>> respDTO = aiptServiceClient.symptomFeatureV2(nlpvo);
        RespDTOUtil.respNGDealCover(respDTO, "症状特征词提取失败");
        return respDTO.data;
    }
}