package com.diagbot.facade;

import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.PartSymptomDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.FixPartEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.PartSymptomVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: 部位facade
 * @Author: ztg
 * @Date: 2018/10/23 16:33
 */
@Component
public class PartFacade {

    @Autowired
    SymptomFacade symptomFacade;
    @Autowired
    AiptServiceClient aiptServiceClient;

    /**
     * 根据已选部位返回对应的症状
     *
     * @param partSymptomVO 参数
     * @return 部位症状关联信息
     */
    public List<PartSymptomDTO> getSymptomByPartName(PartSymptomVO partSymptomVO) {
        List<String> partList = partSymptomVO.getPartList();
        if (ListUtil.isEmpty(partList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择部位");
        }
        if (partSymptomVO.getSexType() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择性别");
        }
        if (partSymptomVO.getAge() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择年龄");
        }
        if (partSymptomVO.getAge().intValue() < 0 || partSymptomVO.getAge().intValue() > 200) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR,
                    "超出年龄范围【0 — 200】");
        }
        if (!partList.contains(FixPartEnum.PART.getPartName())) {
            partList.add(FixPartEnum.PART.getPartName());
        }
        RespDTO<List<PartSymptomDTO>> res = aiptServiceClient.getSymptomByPartName(partSymptomVO);

        RespDTOUtil.respNGDeal(res, "远程调用部位关联的症状失败");
        return res.data;
    }


}
