package com.diagbot.facade;

import com.diagbot.client.AiptServiceClient;
import com.diagbot.dto.ConceptRetrievalExtDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SymptomSearchDTO;
import com.diagbot.dto.UsualDTO;
import com.diagbot.entity.Symptom;
import com.diagbot.enums.ShowTypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.SymptomSearchVO;
import com.diagbot.vo.SymptomUsualVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 症状
 * @Author: ztg
 * @Date: 2018/10/23 16:33
 */
@Component
public class SymptomFacade {
    @Autowired
    AiptServiceClient aiptServiceClient;

    /**
     * 获取常用症状
     *
     * @return
     */
    public List<Symptom> getUsualFac(SymptomUsualVO symptomUsualVO) {
        ConceptUsualVO conceptUsualVO = new ConceptUsualVO();
        BeanUtil.copyProperties(symptomUsualVO, conceptUsualVO);
        conceptUsualVO.setDeptName("全科");
        conceptUsualVO.setType(1);
        //从知识库获取常用标签
        RespDTO<List<UsualDTO>> conceptBaseDTORespDTO = aiptServiceClient.getConceptExtUsual(conceptUsualVO);
        RespDTOUtil.respNGDeal(conceptBaseDTORespDTO, "获取常用标签失败");
        List<Symptom> symptomList = BeanUtil.listCopyTo(conceptBaseDTORespDTO.data, Symptom.class);
        return symptomList;
    }


    /**
     * 症状检索
     *
     * @param symptomSearchVO
     * @return 检索的症状
     */
    public List<SymptomSearchDTO> searchFac(SymptomSearchVO symptomSearchVO) {
        //调用aipt-service服务
        RetrievalVO retrievalVO = new RetrievalVO();
        retrievalVO.setType(1);
        retrievalVO.setInputStr(symptomSearchVO.getName());
        retrievalVO.setAge(symptomSearchVO.getAge());
        retrievalVO.setSexType(symptomSearchVO.getSexType());
        RespDTO<List<ConceptRetrievalExtDTO>> condeptRerievalS = aiptServiceClient.retrivelConceptInfoExt(retrievalVO);
        RespDTOUtil.respNGDeal(condeptRerievalS, "获取标签失败");
        List<SymptomSearchDTO> symptomList = new ArrayList<>();
        SymptomSearchDTO symptomSearchDTO = new SymptomSearchDTO();
        //封装
        for (ConceptRetrievalExtDTO conceptRetrievalDTO : condeptRerievalS.data) {
            symptomSearchDTO = new SymptomSearchDTO();
            symptomSearchDTO.setConceptId(conceptRetrievalDTO.getSelfId());
            symptomSearchDTO.setName(conceptRetrievalDTO.getSelfName());
            if (StringUtil.isNotEmpty(conceptRetrievalDTO.getSameName())) {//同义词
                symptomSearchDTO.setSearchName(conceptRetrievalDTO.getSameName());
            }
            symptomSearchDTO.setShowType(conceptRetrievalDTO.getShowType());
            symptomSearchDTO.setShowTypeCn(ShowTypeEnum.getName(conceptRetrievalDTO.getShowType().intValue()));
            symptomSearchDTO.setDescription(conceptRetrievalDTO.getDescription());
            symptomSearchDTO.setExplains(conceptRetrievalDTO.getExplains());
            symptomList.add(symptomSearchDTO);
        }
        return symptomList;
    }

}
