package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.AIDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SYFDTO;
import com.diagbot.facade.AIFacade;
import com.diagbot.facade.AIV2Facade;
import com.diagbot.vo.AIVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description: AI服务控制层
 * @author: gaodm
 * @time: 2018/10/25 10:09
 */
@RestController
@RequestMapping("/ai")
@Api(value = "AIAPI", tags = { "知识库标准化-AIAPI" })
@SuppressWarnings("unchecked")
public class AIController {

    @Autowired
    AIFacade aiFacade;
    @Autowired
    AIV2Facade aiv2Facade;

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 症状相关数据
     */
    @ApiOperation(value = "知识库标准化-症状推理症状接口[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别，1：男，2：女(必填)<br>" +
                    "symptom：症状自然文本描述(必填)")
    @PostMapping("/pushSymptom")
    @SysLogger("pushSymptom")
    public RespDTO<AIDTO> pushSymptom(@RequestBody @Valid AIVO aivo) {
        return RespDTO.onSuc(aiFacade.push(aivo, "1"));
    }

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 疾病相关数据
     */
    @ApiOperation(value = "知识库标准化-症状推理疾病接口[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别，1：男，2：女(必填)<br>" +
                    "symptom：症状自然文本描述(必填)")
    @PostMapping("/pushDis")
    @SysLogger("pushDis")
    public RespDTO<AIDTO> pushDis(@RequestBody @Valid AIVO aivo) {
        return RespDTO.onSuc(aiFacade.push(aivo, "7"));
    }


    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 症状相关数据
     */
    @ApiOperation(value = "知识库标准化-症状推理症状接口-邵逸夫[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别，1：男，2：女(必填)<br>" +
                    "symptom：症状自然文本描述(必填)")
    @PostMapping("/pushSymptomForSYF")
    @SysLogger("pushSymptomForSYF")
    public RespDTO<PushDTO> pushSymptomForSYF(@RequestBody @Valid AIVO aivo) {
        return RespDTO.onSuc(aiv2Facade.pushSymptomForSYFV2(aivo, "1"));
    }

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 疾病相关数据
     */
    @ApiOperation(value = "知识库标准化-症状推理疾病接口-邵逸夫[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别，1：男，2：女(必填)<br>" +
                    "symptom：症状自然文本描述(必填)")
    @PostMapping("/pushDisForSYF")
    @SysLogger("pushDisForSYF")
    public RespDTO<SYFDTO> pushDisForSYF(@RequestBody @Valid AIVO aivo) {
        return RespDTO.onSuc(aiv2Facade.pushDisForSYFV2(aivo, "7"));
    }

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 症状相关数据
     */
    @ApiOperation(value = "知识库标准化-症状推理症状接口-纳里[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别，1：男，2：女(必填)<br>" +
                    "symptom：症状自然文本描述(必填)")
    @PostMapping("/pushSymptomForNaliV2")
    @SysLogger("pushSymptomForNaliV2")
    public RespDTO<PushDTO> pushSymptomForNaliV2(@RequestBody @Valid AIVO aivo) {
        PushDTO data = aiv2Facade.pushSymptomForSYFV2(aivo, "1");
        return RespDTO.onSuc(data);
    }

    /**
     * 症状推理症状接口
     *
     * @param aivo ai输入参数
     * @return 疾病相关数据
     */
    @ApiOperation(value = "知识库标准化-症状推理疾病接口-纳里[by:zhaops]",
            notes = "age： 年龄(必填)<br>" +
                    "sex：性别，1：男，2：女(必填)<br>" +
                    "symptom：症状自然文本描述(必填)")
    @PostMapping("/pushDisForNaliV2")
    @SysLogger("pushDisForNaliV2")
    public RespDTO<AIDTO> pushDisForNaliV2(@RequestBody @Valid AIVO aivo) {
        AIDTO data = aiv2Facade.pushDisForNaliV2(aivo, "7");
        return RespDTO.onSuc(data);
    }
}