use `sys-tran`;

DROP TABLE IF EXISTS `tran_inquiry_triage`;

CREATE TABLE `tran_inquiry_triage` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `hospital_code` varchar(255) NOT NULL DEFAULT '朗通通用' COMMENT '医院名称',
  `ip` varchar(100) NOT NULL DEFAULT '' COMMENT '访问者的IP',
  `pat_name` varchar(50) NOT NULL DEFAULT '' COMMENT '姓名',
  `pat_age` int(8) NOT NULL DEFAULT '0' COMMENT '年龄',
  `pat_sex` tinyint(4) NOT NULL DEFAULT '0' COMMENT '性别(1：男 2：女)',
  `id_type` tinyint(4) NOT NULL DEFAULT '0' COMMENT '证件类型(1：身份证，2：护照)',
  `id_num` varchar(100) NOT NULL DEFAULT '' COMMENT '证件号',
  `symptom` varchar(255) NOT NULL DEFAULT '' COMMENT '症状',
  `supplement` varchar(1024) NOT NULL DEFAULT '' COMMENT '补充说明',
  `dis_dept` varchar(2048) NOT NULL DEFAULT '' COMMENT '诊断结果和疾病科室',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='智能分诊记录主表';

DROP TABLE IF EXISTS `tran_opt_info`;

CREATE TABLE `tran_opt_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `sys_type` int(11) DEFAULT NULL COMMENT '访问的系统类型(1, "user-service"),(2, "diagbotman-service"),(3, "uaa-service"),(4, "log-service"),(5, "bi-service"),(6, "ltapi-service"),(7, "feedback-service"),(8, "icss-old-service"),(9, "triage-service"),(10, "appkey"),(11, "icss-service"),(12, "icssman-service"),(13, "knowledgeman-service"),(14, "tran-service"),(15, "aipt-service"),(16, "data-service"),(17, "prec-service");',
  `hospital_code` varchar(255) NOT NULL DEFAULT '朗通通用' COMMENT '医院名称',
  `ip` varchar(100) NOT NULL DEFAULT '' COMMENT '访问者的IP',
  `opt_type` varchar(10) DEFAULT NULL COMMENT '操作类型：1 开始，2 结束',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  KEY `idx_gmt_create` (`gmt_create`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='操作信息';


/*
 Navicat Premium Data Transfer

 Source Server         : 192.168.2.236
 Source Server Type    : MySQL
 Source Server Version : 50725
 Source Host           : 192.168.2.236:3306
 Source Schema         : sys-tran

 Target Server Type    : MySQL
 Target Server Version : 50725
 File Encoding         : 65001

 Date: 26/03/2020 17:18:32
*/

SET NAMES utf8mb4;
SET FOREIGN_KEY_CHECKS = 0;

-- ----------------------------
-- Table structure for tran_disease_dept
-- ----------------------------
DROP TABLE IF EXISTS `tran_disease_dept`;
CREATE TABLE `tran_disease_dept`  (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `is_deleted` char(1) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime(0) NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime(0) NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(20) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(20) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `hospital_code` varchar(30) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '医院编码',
  `disease_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT 'HIS疾病名称',
  `icd10_code` varchar(30) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL COMMENT 'icd10编码',
  `icd10_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL COMMENT 'icd10名称',
  `concept_dis_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL COMMENT '朗通疾病名称',
  `sub_dept_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '小科室',
  `original_sub_dept_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '邵逸夫原始小科室数据',
  `sub_dept_code` varchar(20) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '小科室编码',
  `dept_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '大科室',
  `original_dept_name` varchar(200) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '邵逸夫原始大科室数据',
  `dept_code` varchar(20) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '大科室编码',
  `status` varchar(5) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '1' COMMENT '状态(1:有效)',
  `url` varchar(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '' COMMENT '医院科室地址',
  `remark` varchar(128) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB AUTO_INCREMENT = 3957 CHARACTER SET = utf8 COLLATE = utf8_general_ci ROW_FORMAT = Dynamic;

-- ----------------------------
-- Records of tran_disease_dept
-- ----------------------------
INSERT INTO `tran_disease_dept` VALUES (2641, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '酒精中毒', NULL, NULL, '酒精中毒', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2642, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '醉酒', NULL, NULL, '醉酒', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2643, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '休克', NULL, NULL, '休克', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2644, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '中暑', NULL, NULL, '中暑', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2645, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '意识丧失', NULL, NULL, '意识丧失', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2646, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '重症肌无力', NULL, NULL, '重症肌无力', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2647, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '晕厥和虚脱', NULL, NULL, '晕厥和虚脱', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2648, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '休克', NULL, NULL, '感染性休克', '急诊科', '急诊科', '80', '急诊科', '其他', '80', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2649, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '电解质代谢紊乱', NULL, NULL, '电解质代谢紊乱', '', '', '', '普内科', '普内科', '49', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2650, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '电解质代谢紊乱', NULL, NULL, '低钙血症', '', '', '', '普内科', '普内科', '49', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2651, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '电解质代谢紊乱', NULL, NULL, '低钾血症', '', '', '', '普内科', '普内科', '49', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2652, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '电解质代谢紊乱', NULL, NULL, '低钠血症', '', '', '', '普内科', '普内科', '49', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2653, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '感冒病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2654, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '普通感冒', NULL, NULL, '普通感冒', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2655, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '上呼吸道感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2656, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '上呼吸道疾病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2657, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '呼吸道感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2658, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '传染性单核细胞增多症', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2659, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺曲霉病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2660, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺隐球菌病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2661, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺真菌感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2662, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺部感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2663, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺部阴影', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2664, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺大泡', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2665, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肺结节相关疾病', NULL, NULL, '肺结节病', '肺结节MDT', '肺结节MDT', '8041', '多学科协作诊治门诊', '多学科协作诊治门诊', '8041', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2666, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肺结节相关疾病', NULL, NULL, '肺结节', '肺结节MDT', '肺结节MDT', '8041', '多学科协作诊治门诊', '多学科协作诊治门诊', '8041', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2667, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺气肿', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2668, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺栓塞', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2669, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2670, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺占位性病变', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2671, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺肿物', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2672, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '过敏性鼻炎伴哮喘', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2673, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '哮喘', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2674, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '呼吸衰竭', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2675, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '急性气管炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2676, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '气管炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2677, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '急性上呼吸道感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2678, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '急性支气管炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2679, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '间质性肺病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2680, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性阻塞性肺疾病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2681, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支气管哮喘', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2682, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '吸入性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2683, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺脓肿', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2684, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '特指呼吸性疾病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2685, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性肺源性心脏病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2686, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性支气管炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2687, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性支气管炎伴肺气肿', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2688, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性阻塞性肺病伴急性加重', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2689, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性阻塞性肺病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2690, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺恶性肿瘤', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2691, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '特指间质性肺病', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2692, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '胸膜炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2693, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支气管扩张', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2694, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支气管扩张伴感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2695, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支气管扩张伴咯血', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2696, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '咯血', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2697, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支气管炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2698, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支原体感染', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2699, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '胸腔积液', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2700, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '呼吸困难', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2701, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '慢性支气管炎急性发作', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2702, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '军团菌肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2703, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '大叶性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2704, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '尘肺', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2705, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '急性肺水肿', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2706, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '支原体肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2707, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '社区获得性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2708, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺水肿', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2709, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺炎球菌性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2710, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '病毒性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2711, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '葡萄球菌性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2712, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '过敏性哮喘', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2713, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '阿司匹林哮喘', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2714, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '高原性肺水肿', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2715, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '急性气管支气管炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2716, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '重症肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2717, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺癌', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2718, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '念珠菌性肺炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2719, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '原发性肺动脉高压', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2720, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸系统疾病', NULL, NULL, '肺结核', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2721, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胰腺占位性病变', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2722, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胰腺肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2723, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '黄疸', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2724, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '慢性肝病', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2725, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胰腺肿物', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2726, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '结肠肿物', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2727, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '结肠肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2728, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '结肠癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2729, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '乙状结肠肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2730, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '十二指肠恶性肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2731, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '乙状结肠癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2732, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '结肠息肉', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2733, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '沙门菌感染', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2734, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '感染性腹泻', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2735, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '腹泻', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2736, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '秋季腹泻', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2737, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '贲门失弛缓症', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2738, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2739, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肠易激综合征', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2740, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '便秘型肠易激综合征', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2741, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '螺旋杆菌感染', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2742, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '便血', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2743, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2744, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆囊结石(胆石症)', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2745, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆石症', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2746, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '反流性食管炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2747, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '复合性溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2748, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝功能不全', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2749, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝功能异常', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2750, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝硬化', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2751, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2752, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2753, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性胰腺炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2754, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '结肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2755, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '溃疡性结肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2756, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性肝炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2757, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃-食管反流病', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2758, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '食管反流', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2759, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性间质性肝炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2760, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性十二指肠球部溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2761, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '多发性复合性溃疡病', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2762, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性浅表性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2763, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性萎缩性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2764, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '萎缩性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2765, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2766, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃平滑肌肉瘤', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2767, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '残胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2768, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '十二指肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2769, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '十二指肠球炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2770, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '十二指肠肿物', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2771, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '糜烂性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2772, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性胃肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2773, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '不完全性肠梗阻', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2774, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肠道菌群失调', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2775, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肠功能紊乱', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2776, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '结肠溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2777, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '腹膜炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2778, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '腹腔感染', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2779, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '酒精性肝病', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2780, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '乙型肝炎后肝硬化失代偿期', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2781, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝炎后肝硬化', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2782, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝硬化伴食管静脉曲张', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2783, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胰腺囊肿', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2784, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '呕血', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2785, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃肠功能紊乱', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2786, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '上消化道出血', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2787, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '十二指肠溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2788, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '食管炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2789, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '特指急性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2790, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2791, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2792, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胃淋巴肉瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2793, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃痛', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2794, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2795, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '消化不良', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2796, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '消化道出血', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2797, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '消化性溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2798, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '呃逆', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2799, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '脂肪肝', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2800, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '直肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2801, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '克罗恩病', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2802, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '幽门螺旋杆菌感染', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2803, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '腹水', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2804, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '腹腔积液', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2805, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '十二指肠球部溃疡', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2806, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆汁反流性胃炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2807, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '下消化道出血', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2808, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '假膜性肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2809, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性结肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2810, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '反流性咽炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2811, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '功能性腹胀', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2812, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性水肿型胰腺炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2813, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性出血坏死型胰腺炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2814, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '便秘', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2815, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '特指慢性胰腺炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2816, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性胆源型胰腺炎，轻症', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2817, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '缺血性结肠炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2818, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝纤维化', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2819, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肠息肉', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2820, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '酒精性肝硬变', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2821, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '食物过敏', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2822, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胃泌素瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2823, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '原发性肝癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2824, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胃恶性肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2825, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胆管癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2826, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '壶腹部肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2827, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胆囊癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2828, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胰腺癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2829, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '肝恶性肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2830, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '食管恶性肿瘤', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2831, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '食管癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2832, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '低蛋白血症', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2833, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '倾倒综合征', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2834, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '消化道异物', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2835, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '食管肿物', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2836, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胃息肉', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2837, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胃占位性病变', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2838, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '胃肿物', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2839, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '食管裂孔疝', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2840, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '食道癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2841, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '食管内异物', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2842, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '食管异物穿孔', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2843, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '异常体重减轻', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2844, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性自身免疫性胰腺炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2845, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肠结核', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2846, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '细菌性痢疾', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2847, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '细菌性食物中毒', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2848, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '慢性胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2849, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝硬化腹水', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2850, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '酒精性脂肪肝', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2851, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '肝占位性病变', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2852, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化道肿瘤疾病', NULL, NULL, '肝细胞癌', '消化道肿瘤筛查门诊', '消化道肿瘤筛查门诊', '465', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2853, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝囊肿', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2854, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆囊结石', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2855, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆管结石', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2856, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆管炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2857, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '阿尔茨海默病', '记忆障碍门诊', '神经内科--记忆障碍门诊', '429', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2858, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '记忆力下降', '记忆障碍门诊', '神经内科--记忆障碍门诊', '429', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2859, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '下肢震颤', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2860, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '震颤', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2861, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '面神经麻痹', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2862, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '面瘫', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2863, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '面神经麻痹（面瘫）（Bell＇s）', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2864, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '贝尔面瘫', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2865, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '痴呆', '记忆障碍门诊', '神经内科--记忆障碍门诊', '429', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2866, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '低颅压综合征', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2867, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '血管性痴呆', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2868, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '脑炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2869, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '颅内感染', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2870, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '帕金森相关疾病', NULL, NULL, '帕金森叠加综合征', '帕金森病', '神经内科--帕金森病', '422', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2871, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '帕金森相关疾病', NULL, NULL, '帕金森综合征', '帕金森病', '神经内科--帕金森病', '422', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2872, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '不安腿综合征', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2873, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '混合型阿尔茨海默病性痴呆伴抑郁状态', '记忆障碍门诊', '神经内科--记忆障碍门诊', '429', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2874, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '阿尔茨海默病性痴呆', '记忆障碍门诊', '神经内科--记忆障碍门诊', '429', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2875, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头痛相关疾病', NULL, NULL, '枕神经痛', '头痛门诊', '神经内科--头痛门诊', '421', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2876, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头痛相关疾病', NULL, NULL, '药物性头痛', '头痛门诊', '神经内科--头痛门诊', '421', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2877, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '后循环缺血', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2878, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '面肌痉挛', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2879, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '脊髓病', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2880, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '良性发作性位置性眩晕', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2881, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '良性阵发性眩晕', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2882, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '腔隙性脑梗死', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2883, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '脑动脉粥样硬化', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2884, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '脑梗死后遗症', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2885, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头痛相关疾病', NULL, NULL, '紧张性头痛', '头痛门诊', '神经内科--头痛门诊', '421', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2886, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '梅尼埃病', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2887, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '面神经麻痹', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2888, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '脑膜炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2889, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '帕金森相关疾病', NULL, NULL, '帕金森病', '帕金森病', '神经内科--帕金森病', '422', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2890, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头痛相关疾病', NULL, NULL, '偏头痛', '头痛门诊', '神经内科--头痛门诊', '421', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2891, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头痛相关疾病', NULL, NULL, '紧张型头痛', '头痛门诊', '神经内科--头痛门诊', '421', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2892, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '前庭神经元炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2893, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '三叉神经痛', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2894, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '神经痛', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2895, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '原发性肌张力障碍', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2896, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '运动神经元病', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2897, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '枕大神经痛', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2898, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '周围神经病', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2899, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '周围性面神经麻痹', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2900, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '蛛网膜下腔出血', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2901, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '脑梗死个人史', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2902, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '癫痫', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2903, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '面神经炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2904, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '周期性麻痹', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2905, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '神经炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2906, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑血管疾病', NULL, NULL, '糖尿病性脑血管病', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2907, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '结核性脑膜炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2908, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '病毒性脑膜炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2909, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '急性硬膜下血肿', '', '', '50', '神经外科', '神经外科', '50', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2910, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '急性脊髓炎', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2911, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压脑病', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2912, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压性脑病', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2913, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '脑卒中', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2914, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '缺氧缺血性脑病', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2915, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头痛相关疾病', NULL, NULL, '血管性头痛', '头痛门诊', '神经内科--头痛门诊', '421', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2916, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '短暂性脑缺血发作', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2917, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '眩晕综合征', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2918, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '眩晕', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2919, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '周期性眩晕', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2920, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '睡眠相关疾病', NULL, NULL, '失眠症', '睡眠门诊', '神经内科--睡眠门诊', '4212', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2921, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '睡眠相关疾病', NULL, NULL, '失眠', '睡眠门诊', '神经内科--睡眠门诊', '4212', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2922, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '睡眠相关疾病', NULL, NULL, '睡眠障碍', '睡眠门诊', '神经内科--睡眠门诊', '4212', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2923, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '脑出血', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2924, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '神经系统疾病', NULL, NULL, '脑出血后遗症', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2925, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '脑梗塞', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2926, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '脑梗死', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2927, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '脑积水', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2928, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '颅内占位性病变', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2929, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '大面积脑梗死', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2930, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '小脑梗死', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2931, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '脑占位性病变', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2932, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '脑缺血灶', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2933, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑卒中相关疾病', NULL, NULL, '脑血管病', '卒中门诊', '卒中门诊', '426', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2934, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '脑垂体相关疾病', NULL, NULL, '垂体肿瘤', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2935, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颅脑相关疾病', NULL, NULL, '脑膜瘤', '', '', '', '神经内科', '神经内科', '42', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2936, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '不稳定性心绞痛', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2937, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '初发型劳力性心绞痛', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2938, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '低血压', NULL, NULL, '低血压', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2939, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心肌类疾病', NULL, NULL, '扩张型心肌病', '心肌病心脏影像门诊', '心肌病心脏影像门诊', '4318', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2940, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '恶化劳力性心绞痛', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2941, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类心脏病', NULL, NULL, '风湿性心脏病', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2942, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压Ⅱ期', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2943, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压Ⅲ期', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2944, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压I期', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2945, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压病', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2946, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2947, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压性心脏病', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2948, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '混合型心绞痛', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2949, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠状动脉粥样硬化', NULL, NULL, '冠状动脉粥样硬化', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2950, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '冠状动脉粥样硬化性心脏病', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2951, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '冠心病', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2952, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '陈旧性心肌梗死', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2953, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '肺动脉高压重度', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2954, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心脏瓣膜疾病', NULL, NULL, '主动脉瓣关闭不全', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2955, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '急性冠脉综合征', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2956, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '阵发性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2957, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2958, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '期前收缩', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2959, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '病态窦房结综合症', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2960, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心肌类疾病', NULL, NULL, '心肌炎', '心肌病心脏影像门诊', '心肌病心脏影像门诊', '4318', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2961, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '颈内动脉血栓形成', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2962, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '动脉粥样硬化', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2963, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '室上性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2964, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '心源性哮喘', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2965, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '颈动脉硬化', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2966, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '急性心肌梗死', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2967, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心肌类疾病', NULL, NULL, '肥厚型梗阻性心肌病', '心肌病心脏影像门诊', '心肌病心脏影像门诊', '4318', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2968, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心脏瓣膜疾病', NULL, NULL, '二尖瓣关闭不全', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2969, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, 'H型高血压', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2970, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '水肿', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2971, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '代谢相关疾病', NULL, NULL, '高甘油三酯血症', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2972, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '心包积液', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2973, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '心电图异常', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2974, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心肌类疾病', NULL, NULL, '心肌病', '心肌病心脏影像门诊', '心肌病心脏影像门诊', '4318', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2975, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '心绞痛', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2976, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '电解质代谢紊乱', NULL, NULL, '高钾血症', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2977, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '心力衰竭', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2978, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心脏瓣膜疾病', NULL, NULL, '心脏瓣膜病', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2979, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '主动脉夹层', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2980, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '右心衰竭', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2981, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '阵发性房室折返性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2982, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '阵发性心房纤颤', '房颤门诊', '房颤门诊', '432', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2983, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '左心衰竭', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2984, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '房室传导阻滞', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2985, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '房性期前收缩', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2986, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '室性期前收缩', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2987, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '心脏扩大', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2988, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '室性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2989, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心房颤动', '房颤门诊', '房颤门诊', '432', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2990, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '房颤', '房颤门诊', '房颤门诊', '432', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2991, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心房纤维颤动', '房颤门诊', '房颤门诊', '432', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2992, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '房颤（心房纤维颤动）', '房颤门诊', '房颤门诊', '432', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2993, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心房扑动', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2994, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '心功能不全', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2995, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心律失常', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2996, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心律失常（心律不齐）', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2997, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '窦性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2998, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '窦性心动过缓', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (2999, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '预激综合征', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3000, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '阵发性室上性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3001, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '心动过缓', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3002, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '心肌梗死', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3003, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '房性早搏', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3004, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '室性早搏', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3005, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '早搏', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3006, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '窦房结折返性心动过速', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3007, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '稳定型心绞痛', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3008, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心肌类疾病', NULL, NULL, '病毒性心肌炎', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3009, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '感染性心内膜炎', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3010, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '急性ST段抬高型心肌梗死', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3011, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '窦性停搏', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3012, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '冠心病类疾病', NULL, NULL, '急性非ST段抬高型心肌梗死', '冠心病门诊', '冠心病门诊', '435', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3013, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类心脏病', NULL, NULL, '风湿性二尖瓣狭窄', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3014, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心脏瓣膜疾病', NULL, NULL, '二尖瓣狭窄', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3015, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压危象', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3016, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '二尖瓣脱垂综合征', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3017, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心律失常类疾病', NULL, NULL, '窦房传导阻滞', '心律失常门诊', '心律失常门诊', '436', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3018, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '糖尿病性心脏病', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3019, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心脏瓣膜疾病', NULL, NULL, '主动脉瓣狭窄', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3020, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '原发性高血压', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3021, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '代谢相关疾病', NULL, NULL, '混合性高脂血症', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3022, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '代谢相关疾病', NULL, NULL, '高胆固醇血症', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3023, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '代谢相关疾病', NULL, NULL, '高脂血症', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3024, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '1型糖尿病性缺血性心肌病', '', '', '', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3025, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '老年性高血压', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3026, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心衰相关疾病', NULL, NULL, '下肢肿胀', '心衰联合门诊', '心衰联合门诊', '4302', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3027, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病性高渗性昏迷', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3028, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病性乳酸酸中毒', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3029, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病性视网膜病变', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3030, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病性周围神经病', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3031, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病足病', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3032, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '产后垂体功能减退症', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3033, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '亚临床甲状腺功能减退症', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3034, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '桥本氏病（慢性淋巴细胞性甲状腺炎）', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3035, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '慢性淋巴细胞性甲状腺炎', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3036, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '桥本氏病', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3037, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺肿', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3038, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '结节性甲状腺肿', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3039, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺相关性眼病', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3040, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '德奎尔万甲状腺炎', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3041, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '低血糖', NULL, NULL, '低血糖症', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3042, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病性肾病', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3043, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '糖尿病性足病', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3044, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '多汗症', '', '', '', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3045, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '2型糖尿病', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3046, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肥胖症', NULL, NULL, '肥胖症', '肥胖联合门诊', '肥胖联合门诊', '712', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3047, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '高泌乳素血症', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3048, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺功能异常', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3049, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺机能减退症', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3050, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺机能亢进症', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3051, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺机能亢进', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3052, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺功能亢进', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3053, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺疾患', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3054, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺结节', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3055, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺囊肿', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3056, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺炎', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3057, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺肿物', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3058, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '内分泌病性骨质疏松', '骨质疏松', '内分泌--骨质疏松', '4411', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3059, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '糖尿病', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3060, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '糖尿病酮症酸中毒', '糖尿病及其并发症', '糖尿病及其并发症', '4412', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3061, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '营养性消瘦', '', '', '', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3062, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '原发性甲状腺功能减退症', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3063, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '原发性肾上腺皮质功能减退症', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3064, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '口干', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3065, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '糖耐量异常', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3066, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '粘液性水肿', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3067, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '自身免疫性甲状腺炎', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3068, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '亚急性甲状腺炎', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3069, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '骨质疏松症', '骨质疏松', '内分泌--骨质疏松', '4411', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3070, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '血糖过多', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3071, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '高血糖', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3072, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '骨质疏松', '骨质疏松', '内分泌--骨质疏松', '4411', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3073, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '库欣综合征', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3074, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '1.5型糖尿病', '糖尿病联合门诊', '糖尿病联合门诊', '711', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3075, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '1型糖尿病', '1型糖尿病门诊', '1型糖尿病门诊', '441', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3076, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '糖尿病相关疾病', NULL, NULL, '低血糖昏迷', '', '', '', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3077, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '急性化脓性甲状腺炎', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3078, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '尿崩症', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3079, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺功能减退症', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3080, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '甲状腺相关疾病', NULL, NULL, '甲状腺功能减退', '甲状腺门诊', '甲状腺门诊', '449', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3081, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾上腺相关疾病', NULL, NULL, '原发性醛固酮过多症', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3082, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '鞍区肿物', '垂体肾上腺性腺门诊', '内分泌/垂体肾上腺性腺门诊', '447', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3083, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '内分泌系统疾病', NULL, NULL, '老年性骨质疏松', '骨质疏松', '内分泌--骨质疏松', '4411', '内分泌科', '内分泌专科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3084, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '紫癜性肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3085, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '蛋白尿', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3086, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾炎综合征', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3087, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '尿毒症', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3088, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾囊肿', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3089, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾病综合征', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3090, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾功能异常', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3091, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾病1期', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3092, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾病3期', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3093, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾上腺肿瘤', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3094, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾上腺肿物', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3095, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '狼疮性肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3096, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾盂肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3097, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾肿物', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3098, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '膜性肾病', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3099, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, 'IgA肾病', 'IgA肾病', 'IgA肾病', '476', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3100, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '急性肾盂肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3101, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾脏病5期', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3102, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '尿毒症性神经病变', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3103, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾衰竭', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3104, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '多囊肾', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3105, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '尿酸性肾病', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3106, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '间质性肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3107, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性间质性肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3108, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '高血压良性肾小动脉硬化症', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3109, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '急性肾小球肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3110, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '急性肾衰竭', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3111, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾盂肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3112, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '肾病', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3113, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '急性肾小管坏死', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3114, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '慢性肾炎', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3115, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '高血压相关疾病', NULL, NULL, '高血压性肾病', '高血压门诊', '高血压门诊', '433', '心内科', '心内科', '43', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3116, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3117, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '干燥综合征', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3118, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '继发性干燥综合征', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3119, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '结缔组织病', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3120, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '类风湿性关节炎', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3121, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '强直性脊柱炎', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3122, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '红斑狼疮', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3123, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '系统性硬化症', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3124, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '系统性红斑狼疮', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3125, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '血管炎', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3126, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '纤维肌痛', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3127, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '高尿酸血症', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3128, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '白塞病', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3129, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '痛风', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3130, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '肌痛', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3131, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '关节病', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3132, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '关节炎', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3133, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '风湿热', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3134, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '风湿类疾病', NULL, NULL, '过敏性紫癜', '风湿免疫科', '风湿免疫科', '239', '风湿免疫门诊', '风湿免疫门诊', '239', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3135, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '白细胞疾患', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3136, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '白细胞特指疾患', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3137, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '急性白血病', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3138, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '骨髓增生异常综合征', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3139, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '溶血性贫血', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3140, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '再生障碍性贫血', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3141, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '粒细胞缺乏', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3142, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '多发性骨髓瘤', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3143, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '非霍奇金淋巴瘤', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3144, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '急性粒细胞缺乏症', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3145, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '巨幼红细胞性贫血', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3146, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '慢性中性粒细胞白血病', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3147, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '缺铁性贫血', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3148, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '血小板减少症', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3149, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '贫血', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3150, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '慢性白血病', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3151, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '霍奇金淋巴瘤', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3152, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '地中海贫血', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3153, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '弥漫性大细胞淋巴瘤', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3154, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '特发性血小板减少性紫癜', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3155, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '血液系统疾病', NULL, NULL, '全血细胞减少', '', '', '', '血液内科', '血液内科', '41', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3156, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈部包块', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3157, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '软组织挫伤', NULL, NULL, '软组织挫伤', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3158, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '淋巴结相关疾病', NULL, NULL, '淋巴结肿大', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3159, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '甲状腺癌', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3160, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '甲状腺肿瘤', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3161, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '甲状腺恶性肿瘤', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3162, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肠疾病', NULL, NULL, '肠梗阻', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3163, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '淋巴管疾病', NULL, NULL, '丹毒', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3164, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腹部疾病', NULL, NULL, '腹腔肿瘤', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3165, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '感染相关疾病', NULL, NULL, '腹壁软组织感染', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3166, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腹部疾病', NULL, NULL, '腹壁疝', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3167, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腹部疾病', NULL, NULL, '腹部肿物', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3168, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '疝类疾病', NULL, NULL, '腹股沟疝', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3169, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '急性阑尾炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3170, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '急性阑尾炎伴腹膜炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3171, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '阑尾脓肿', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3172, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '阑尾炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3173, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '淋巴管疾病', NULL, NULL, '淋巴管炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3174, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '淋巴结相关疾病', NULL, NULL, '淋巴结炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3175, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '淋巴结相关疾病', NULL, NULL, '淋巴结增大', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3176, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈部肿物', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3177, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '慢性阑尾炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3178, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '疝类疾病', NULL, NULL, '脐疝', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3179, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '感染相关疾病', NULL, NULL, '软组织感染', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3180, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '软组织损伤', NULL, NULL, '软组织损伤', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3181, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '疝类疾病', NULL, NULL, '嵌顿性腹股沟斜疝', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3182, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '急性化脓性阑尾炎', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3183, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肠疾病', NULL, NULL, '乙状结肠扭转', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3184, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阑尾疾病', NULL, NULL, '急性阑尾炎伴穿孔', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3185, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '疝类疾病', NULL, NULL, '腹疝', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3186, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肠疾病', NULL, NULL, '肠绞痛', '', '', '', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3187, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '头部外伤', NULL, NULL, '头部外伤', '', '', '', '神经外科', '神经外科', '50', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3188, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '阿米巴肝脓肿', '肝胆胰外科', '肝胆胰外科', '5118', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3189, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊结石伴急性胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3190, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊结石伴慢性胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3191, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊结石伴特指胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3192, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊结石伴胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3193, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊息肉', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3194, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊腺肌症', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3195, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆囊占位性病变', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3196, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆总管结石', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3197, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆总管结石伴急性胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3198, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胆道系统疾病', NULL, NULL, '胆总管结石伴慢性胆囊炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3199, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '吞咽困难', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3200, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝胆管结石', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3201, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝内胆管结石', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3202, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '肝脓肿', '肝胆胰外科', '肝胆胰外科', '5118', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3203, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '肝血管瘤病', '肝胆胰外科', '肝胆胰外科', '5118', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3204, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '肝肿物', '肝胆胰外科', '肝胆胰外科', '5118', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3205, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '梗阻性黄疸', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3206, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆囊结石嵌顿', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3207, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性胆管炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3208, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '肝胆管狭窄', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3209, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '胆总管扩张', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3210, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '特指胆石症', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3211, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '原发性胆汁性肝硬变', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3212, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '肝脏破裂', '肝胆胰外科', '肝胆胰外科', '5118', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3213, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '消化系统疾病', NULL, NULL, '急性化脓性梗阻性胆管炎', '', '', '', '消化内科', '消化内科', '46', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3214, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝脏疾病', NULL, NULL, '肝继发恶性肿瘤', '肝胆胰外科', '肝胆胰外科', '5118', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3215, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '半月板损伤', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3216, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌腱类疾病', NULL, NULL, '肱骨外上髁炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3217, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨关节类疾病', NULL, NULL, '骨痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3218, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '骨质疏松伴病理性骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3219, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '踝关节扭伤', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3220, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌腱类疾病', NULL, NULL, '肌腱炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3221, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌腱类疾病', NULL, NULL, '肌筋膜炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3222, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌肉劳损', NULL, NULL, '肌肉劳损', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3223, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肩关节疾病', NULL, NULL, '肩袖损伤', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3224, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肩关节疾病', NULL, NULL, '肩周炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3225, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肩关节疾病', NULL, NULL, '肩（关节）周炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3226, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌腱类疾病', NULL, NULL, '腱鞘炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3227, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌筋膜疾病', NULL, NULL, '筋膜炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3228, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '颈椎骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3229, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨关节类疾病', NULL, NULL, '髋关节痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3230, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨关节类疾病', NULL, NULL, '骨关节病', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3231, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨关节类疾病', NULL, NULL, '关节痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3232, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肩关节疾病', NULL, NULL, '肩关节痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3233, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈椎病', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3234, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈椎病（颈椎骨关节炎）', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3235, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3236, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈椎骨质增生', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3237, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '颈部疾病', NULL, NULL, '颈椎骨质增生（颈椎骨关节炎）', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3238, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '手部疾病', NULL, NULL, '手损伤', '手足外科', '手足外科', '546', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3239, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '手部疾病', NULL, NULL, '手浅表损伤', '手足外科', '手足外科', '546', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3240, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '损伤', NULL, NULL, '损伤', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3241, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肩关节疾病', NULL, NULL, '特指肩损害', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3242, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '手部疾病', NULL, NULL, '腕关节痛', '手足外科', '手足外科', '546', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3243, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '手部疾病', NULL, NULL, '腕关节周围炎', '手足外科', '手足外科', '546', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3244, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '手部疾病', NULL, NULL, '腕管综合征', '手足外科', '手足外科', '546', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3245, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '手部疾病', NULL, NULL, '腕管综合症', '手足外科', '手足外科', '546', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3246, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '膝关节疾病', NULL, NULL, '膝关节病', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3247, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '膝关节疾病', NULL, NULL, '膝关节痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3248, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '膝关节疾病', NULL, NULL, '膝骨关节病', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3249, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰肌劳损', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3250, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎骨关节炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3251, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '腰椎骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3252, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎关节强硬', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3253, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎间盘突出', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3254, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰背痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3255, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '背痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3256, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎间盘脱出', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3257, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '椎间盘脱出伴脊髓病', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3258, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎间盘脱出伴脊髓炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3259, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎间盘突出伴脊髓病', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3260, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎管狭窄', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3261, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎椎管狭窄', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3262, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎骨关节炎', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3263, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '腰椎骨质增生（腰椎骨关节炎）', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3264, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '膝关节半月板撕裂', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3265, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '膝关节损伤', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3266, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '椎管狭窄', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3267, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '椎间盘疾患', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3268, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '腰痛相关疾病', NULL, NULL, '椎间盘疾患性腰痛伴坐骨神经痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3269, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '足部疾病', NULL, NULL, '足痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3270, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '足部疾病', NULL, NULL, '足损伤', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3271, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '周围神经疾病', NULL, NULL, '坐骨神经痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3272, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '髌骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3273, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '跟骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3274, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '肱骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3275, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '股骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3276, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3277, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '足部疾病', NULL, NULL, '踝关节痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3278, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '胫骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3279, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '肋骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3280, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '桡骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3281, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '颅骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3282, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '尺骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3283, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨折与损伤类疾病', NULL, NULL, '跖骨骨折', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3284, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '骨关节类疾病', NULL, NULL, '肘关节痛', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3285, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肌腱类疾病', NULL, NULL, '网球肘', '', '', '', '骨科', '骨科', '54', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3286, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛管炎', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3287, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛门周围感染', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3288, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛门搔痒症', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3289, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛周感染', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3290, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛周脓肿', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3291, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '混合痔', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3292, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '痔', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3293, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛门痛', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3294, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '血栓性外痔', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3295, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '血栓性痔', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3296, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '直肠静脉曲张破裂', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3297, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '直肠息肉', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3298, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '直肠肿瘤', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3299, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '直肠肿物', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3300, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛裂', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3301, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛瘘', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3302, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛乳头肥大', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3303, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '内痔', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3304, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '外痔', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3305, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '结肠恶性肿瘤个人史', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3306, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '直肠癌', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3307, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肛肠疾病', NULL, NULL, '肛门瘙痒症', '', '', '', '肛肠外科', '肛肠外科', '86', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3308, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '哺乳疾病', NULL, NULL, '哺乳疾病', '产后门诊', '产后门诊(下沙)', '363A', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3309, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺良性肿瘤', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3310, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺分叶状肿瘤', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3311, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳房纤维囊性乳腺病', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3312, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '男性乳房发育', '', '', '', '内分泌科', '内分泌科', '44', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3313, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳房疾患', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3314, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳痛症', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3315, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺恶性肿瘤', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3316, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺腺病', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3317, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺炎', '产后门诊', '产后门诊(下沙)', '363A', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3318, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺肿瘤', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3319, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '纤维囊性乳腺病', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3320, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺囊性增生病', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3321, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺囊性增生病(慢性囊性乳腺炎)', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3322, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳房结节', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3323, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳房肿块', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3324, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳房肿物', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3325, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳头溢液', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3326, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺纤维腺囊肿', '乳腺疾病门诊', '乳腺疾病门诊', '901', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3327, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '乳腺癌', '乳腺', '乳腺--肿瘤外科', '909', '肿瘤外科', '肿瘤外科', '90', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3328, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '乳腺疾病', NULL, NULL, '急性乳腺炎', '产后门诊', '产后门诊(下沙)', '363A', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3329, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胃十二指肠疾病', NULL, NULL, '胃癌', '胃肠外科', '胃肠外科(名医)(庆春)', '5116V', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3330, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢动脉相关疾病', NULL, NULL, '下肢动脉粥样硬化闭塞症', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3331, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢静脉相关疾病', NULL, NULL, '大隐静脉曲张', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3332, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢动脉相关疾病', NULL, NULL, '下肢动脉硬化闭塞症', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3333, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢动脉相关疾病', NULL, NULL, '下肢动脉粥样硬化', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3334, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢静脉相关疾病', NULL, NULL, '下肢静脉曲张', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3335, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢静脉相关疾病', NULL, NULL, '下肢静脉曲张伴静脉炎', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3336, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢静脉相关疾病', NULL, NULL, '下肢静脉曲张伴溃疡', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3337, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢静脉相关疾病', NULL, NULL, '下肢静脉曲张伴皮炎', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3338, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '下肢静脉相关疾病', NULL, NULL, '下肢深静脉血栓性静脉炎', '血管外科', '血管外科', '5109', '普外科', '普外科', '51', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3339, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '气管狭窄', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3340, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '胸腺良性肿瘤', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3341, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '纵隔淋巴结肿大', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3342, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '胸部损伤', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3343, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '纵隔肿瘤', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3344, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '纵隔肿物', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3345, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '创伤性气胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3346, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '开放性气胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3347, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '张力性气胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3348, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '脓胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3349, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '血气胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3350, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '闭合性气胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3351, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '胸腺恶性肿瘤', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3352, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '胸部疾病', NULL, NULL, '气胸', '', '', '', '胸外科', '胸外科', '529', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3353, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肾脏相关疾病', NULL, NULL, '单纯性肾囊肿', '', '', '', '肾内科', '肾内科', '47', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3354, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '睾丸鞘膜积液', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3355, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '慢性前列腺炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3356, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '泌尿道感染', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3357, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '泌尿系结石', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3358, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '尿道综合症', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3359, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '膀胱过度活动症', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3360, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '膀胱炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3361, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '前列腺炎性疾病', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3362, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '前列腺增生', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3363, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '输尿管结石', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3364, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾积水', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3365, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '尿道炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3366, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '压力性尿失禁', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3367, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '尿潴留', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3368, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '附睾炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3369, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '龟头包皮炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3370, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '龟头炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3371, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '精囊炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3372, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '精索静脉曲张', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3373, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '睾丸炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3374, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '阴囊肿物', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3375, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '包皮过长', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3376, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾挫伤', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3377, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '前列腺结石', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3378, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '睾丸扭转', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3379, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '急性膀胱炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3380, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '前列腺囊肿', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3381, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾癌', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3382, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '前列腺癌', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3383, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '膀胱癌', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3384, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '前列腺恶性肿瘤', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3385, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾恶性肿瘤', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3386, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '输尿管恶性肿瘤', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3387, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '膀胱恶性肿瘤', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3388, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾肿瘤', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3389, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾结石', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3390, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾积水伴肾结石', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3391, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '肾积水伴输尿管结石', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3392, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '早泄', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3393, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '男性勃起障碍', '男性疾病门诊', '男性疾病门诊', '621', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3394, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '勃起功能障碍', '男性疾病门诊', '男性疾病门诊', '621', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3395, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '性功能障碍，非由器质性障碍或疾病引起', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3396, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '性功能障碍', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3397, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '包皮炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3398, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '慢性非细菌性前列腺炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3399, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '慢性细菌性前列腺炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3400, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '急性细菌性前列腺炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3401, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '输精管炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3402, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '急性前列腺炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3403, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '包皮嵌顿', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3404, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '泌尿系统疾病', NULL, NULL, '慢性附睾炎', '', '', '', '泌尿外科', '泌尿外科', '62', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3405, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '滴虫性阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3406, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫颈癌', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3407, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3408, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '外阴炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3409, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '瘢痕子宫', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3410, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫粘膜下平滑肌瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3411, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '多囊卵巢综合征', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3412, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '附件炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3413, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈恶性肿瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3414, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈机能不全', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3415, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈上皮内肿瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3416, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫颈鳞状上皮不典型增生', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3417, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈息肉', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3418, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '后天性输卵管闭锁', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3419, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '急性外阴炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3420, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '急性阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3421, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '阴道出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3422, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '具有避孕装置(子宫内)', NULL, NULL, '具有避孕装置(子宫内)', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3423, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '宫内节育器', NULL, NULL, '宫内节育器', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3424, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '绝经后出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3425, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '绝经期出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3426, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '绝经期和围绝经期疾病', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3427, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '更年期疾病', NULL, NULL, '更年期综合症', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3428, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '老年性阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3429, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢过度刺激综合征', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3430, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢肿瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3431, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢子宫内膜异位囊肿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3432, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '慢性女性盆腔炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3433, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '慢性阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3434, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢恶性肿瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3435, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '念珠菌性阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3436, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '女性盆腔炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3437, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '盆腔炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3438, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '女性原发性不育', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3439, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '排卵期出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3440, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '盆腔囊肿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3441, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '盆腔肿物', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3442, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '输卵管积水', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3443, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '痛经', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3444, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '围绝经期综合征', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3445, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '慢性子宫内膜炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3446, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3447, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内膜息肉样增生', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3448, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '外阴肿物', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3449, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '月经过多', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3450, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '月经过少', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3451, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '月经不规则', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3452, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '经期延长', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3453, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '月经失调', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3454, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '功能障碍性子宫出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3455, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '女性更年期综合征', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3456, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '原发不孕', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3457, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '继发不孕', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3458, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '异常子宫出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3459, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '月经稀少', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3460, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫颈鳞状上皮增生', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3461, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内膜炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3462, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内膜异位症', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3463, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内膜增厚', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3464, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内粘连', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3465, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫平滑肌瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3466, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫腺肌症', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3467, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫纵隔', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3468, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢畸胎瘤', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3469, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '细菌性阴道炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3470, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫脱垂', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3471, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢囊肿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3472, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈糜烂', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3473, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内膜息肉', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3474, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢巧克力样囊肿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3475, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '女性继发性不育', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3476, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫颈息肉', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3477, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫颈机能不全', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3478, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫颈炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3479, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈炎性疾病', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3480, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '慢性盆腔炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3481, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '疤痕子宫', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3482, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢早衰', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3483, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢滤泡囊肿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3484, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢蒂扭转', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3485, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢黄体囊肿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3486, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '输卵管炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3487, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '绝经后骨质疏松', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3488, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '急性子宫内膜炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3489, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '绒毛膜癌', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3490, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '衣原体性宫颈炎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3491, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '闭经', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3492, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '黄体破裂', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3493, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '宫颈癌', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3494, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '子宫内膜癌', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3495, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '卵巢癌', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3496, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '单胎活产', '产科', '产科(名医)(庆春)', '36V', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3497, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠剧吐伴代谢紊乱', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3498, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠期肝内胆汁淤积症', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3499, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠期糖尿病', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3500, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '双胎妊娠', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3501, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '胎膜早破', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3502, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '臀位分娩', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3503, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '先兆流产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3504, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '难免流产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3505, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '羊水过少', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3506, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '稽留流产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3507, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠呕吐', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3508, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠期并发糖尿病', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3509, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '先兆临产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3510, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '先兆早产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3511, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '异位妊娠', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3512, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '早产儿', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3513, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '自然流产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3514, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '生育一般咨询和指导', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3515, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠状态', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3516, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '双胎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3517, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '羊水偏少', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3518, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '特指即刻产后出血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3519, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '习惯性流产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3520, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '子宫收缩乏力', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3521, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '妊娠引起高血压', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3522, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '葡萄胎', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3523, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '输卵管妊娠', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3524, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '弥漫性血管内凝血', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3525, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '宫颈机能不全产妇医疗', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3526, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '产科疾病', NULL, NULL, '难免性流产', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3527, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸科疾病', NULL, NULL, '急性腺样体炎', '', '', '', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3528, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '心血管系统疾病', NULL, NULL, '先天性心脏病', '', '', '', '心脏外科', '心脏外科', '528', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3529, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '耵聍栓塞', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3530, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '耵聍嵌塞', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3531, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '耳损伤', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3532, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '外耳道肿物', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3533, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '外耳道炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3534, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '耳痛', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3535, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '突发性聋', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3536, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '耳鸣', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3537, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '听力减退', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3538, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '急性分泌性中耳炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3539, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '分泌性中耳炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3540, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '慢性化脓性中耳炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3541, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '慢性化脓性中耳炎胆脂瘤型', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3542, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '突发性耳聋', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3543, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '外耳炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3544, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '中耳炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3545, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '鼓膜穿孔', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3546, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '听力检查', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3547, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '耳部疾病', NULL, NULL, '慢性中耳炎', '耳科', '耳科', '3310', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3548, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸相关疾病', NULL, NULL, '睡眠呼吸暂停', '鼾症--睡眠呼吸障碍专科门诊', '鼾症--睡眠呼吸障碍专科门诊（下沙）', '480A', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3549, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸相关疾病', NULL, NULL, '睡眠呼吸暂停综合症', '鼾症--睡眠呼吸障碍专科门诊', '鼾症--睡眠呼吸障碍专科门诊（下沙）', '480A', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3550, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '呼吸相关疾病', NULL, NULL, '阻塞性睡眠呼吸暂停综合征', '鼾症--睡眠呼吸障碍专科门诊', '鼾症--睡眠呼吸障碍专科门诊（下沙）', '480A', '呼吸内科', '呼吸内科', '48', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3551, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻出血', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3552, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻出血（鼻衄）', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3553, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻前庭炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3554, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻腔肿物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3555, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻息肉', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3556, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3557, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '急性鼻窦炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3558, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '过敏性鼻炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3559, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '过敏性鼻炎(变应性鼻炎)', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3560, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '慢性鼻窦炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3561, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '(慢性)鼻窦炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3562, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻窦炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3563, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '慢性鼻炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3564, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻部肿物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3565, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻骨骨折', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3566, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻中隔偏曲', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3567, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '急性鼻炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3568, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '慢性鼻咽炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3569, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼻咽癌', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3570, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '鼻部疾病', NULL, NULL, '鼾症', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3571, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '扁桃体肥大', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3572, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '扁桃体肿物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3573, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '肥大性咽炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3574, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '喉肿物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3575, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '会厌囊肿', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3576, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性扁桃体炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3577, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性喉炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3578, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性化脓性扁桃体炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3579, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性化脓性扁桃腺炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3580, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性会厌炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3581, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性咽喉炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3582, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性咽炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3583, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '慢性咽喉炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3584, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '慢性扁桃体炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3585, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '慢性喉炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3586, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '慢性咽炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3587, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '声带白斑', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3588, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '声带结节', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3589, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '咽部异物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3590, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '咽部肿物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3591, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '咽鼓管阻塞', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3592, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '声嘶', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3593, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '声带息肉', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3594, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '声带肿物', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3595, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '喉炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3596, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '急性病毒性咽喉炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3597, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '扁桃体炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3598, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '咽炎', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3599, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '喉恶性肿瘤', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3600, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '下咽恶性肿瘤', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3601, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '喉结核', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3602, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '茎突综合征', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3603, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '咽部疾病', NULL, NULL, '茎突过长', '', '', '', '耳鼻咽喉头颈外科', '耳鼻咽喉头颈外科', '33', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3604, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '泪道阻塞', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3605, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '睑板腺炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3606, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '倒睫', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3607, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '眶骨骨折', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3608, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '病毒性角膜炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3609, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '玻璃体出血', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3610, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '玻璃体混浊', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3611, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '眼睑痉挛', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3612, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '干眼症', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3613, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '干眼病（结膜干燥）', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3614, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '眼疲劳', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3615, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '高度近视', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3616, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '结膜结石', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3617, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '巩膜炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3618, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '角膜葡萄膜炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3619, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '老年性白内障', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3620, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '黄斑和后极变性', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3621, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '黄斑前膜', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3622, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '黄斑裂孔', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3623, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '睑板腺囊肿', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3624, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '睑腺炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3625, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜变性', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3626, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜干燥症', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3627, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜溃疡性穿孔', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3628, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜内皮失代偿', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3629, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜上皮脱落', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3630, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3631, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '角膜疾病', NULL, NULL, '角膜异物', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3632, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '结膜炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3633, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '近视', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3634, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '老视', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3635, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '泡性结膜炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3636, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '前房角狭窄', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3637, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '屈光不正', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3638, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '特指视网膜脱离', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3639, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '先天性倒睫', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3640, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '翼状胬肉', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3641, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '圆锥角膜', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3642, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '白内障', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3643, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '视物模糊', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3644, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '虹膜睫状体炎', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3645, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '结膜出血', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3646, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '青光眼', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3647, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '眼部疾病', NULL, NULL, '眼睑裂伤', '', '', '', '眼科', '眼科', '34', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3648, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '口干燥症', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3649, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '牙槽突不齐', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3650, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '龋齿', NULL, NULL, '龋齿', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3651, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '残冠', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3652, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '阻生齿', NULL, NULL, '阻生齿', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3653, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔粘膜溃疡', NULL, NULL, '口腔粘膜溃疡', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3654, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '颌下腺肿物', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3655, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '腮腺区肿物', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3656, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '腮腺肿瘤', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3657, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '甲状舌管囊肿', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3658, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '牙髓疾病', NULL, NULL, '牙髓炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3659, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '急性牙髓炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3660, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '慢性牙髓炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3661, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '牙龈疾病', NULL, NULL, '慢性龈炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3662, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '牙龈疾病', NULL, NULL, '牙龈出血', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3663, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '急性根尖周炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3664, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '单纯性牙周炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3665, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '急性牙周炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3666, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '牙周疾病', NULL, NULL, '牙周病', '牙周病门诊', '牙周病门诊', '321', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3667, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '慢性根尖周炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3668, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '口腔疾病', NULL, NULL, '增生性牙龈炎', '', '', '', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3669, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '牙周疾病', NULL, NULL, '慢性牙周炎', '牙周病门诊', '牙周病门诊', '321', '牙科', '牙科', '32', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3670, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮肤慢性溃疡', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3671, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮肤溃疡', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3672, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮肤感染', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3673, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '足癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3674, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '白癜风', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3675, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '寻常疣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3676, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '斑秃', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3677, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '瘢痕疙瘩', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3678, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3679, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '唇炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3680, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '带状疱疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3681, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '单纯疱疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3682, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '股癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3683, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤过敏性疾病', NULL, NULL, '过敏性皮炎', '皮肤过敏性疾病门诊', '皮肤过敏性疾病门诊', '232', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3684, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '汗疱疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3685, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '黑布拉痒疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3686, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '黑素细胞痣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3687, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '花斑癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3688, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '黄褐斑', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3689, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '甲沟炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3690, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '甲癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3691, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '脚癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3692, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '接触性皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3693, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '结节性红斑', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3694, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '疥疮', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3695, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '毛囊炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3696, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '玫瑰糠疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3697, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '疱疹病毒感染', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3698, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3699, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮脂腺囊肿', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3700, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '固定性药疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3701, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '神经性皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3702, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '丘疹性荨麻疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3703, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '日光性皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3704, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '褥疮', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3705, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '瘙痒症', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3706, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '手癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3707, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '水痘', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3708, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '体癣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3709, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '妇科疾病', NULL, NULL, '外阴瘙痒症', '', '', '', '妇科', '妇科', '61', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3710, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '荨麻疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3711, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '荨麻疹性血管炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3712, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '药物性皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3713, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '腋臭', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3714, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '阴囊湿疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3715, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '银屑病', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3716, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '疣', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3717, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '淤积性皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3718, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '脂溢性角化病', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3719, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '脂溢性皮炎', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3720, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '脂溢性脱发', '脱发门诊', '皮肤科--脱发门诊', '235', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3721, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '脱发', '脱发门诊', '皮肤科--脱发门诊', '235', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3722, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '瘢痕', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3723, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '痤疮', '痤疮门诊', '皮肤科--痤疮门诊', '236', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3724, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '湿疹', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3725, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '梅毒', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3726, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮肤肿物', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3727, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮下结节', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3728, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '脂肪瘤', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3729, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮肤脓肿', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3730, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '搔痒症', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3731, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '过敏反应', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3732, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '皮肤类疾病', NULL, NULL, '皮下肿物', '', '', '', '皮肤科', '皮肤科', '23', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3733, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '创伤后应激障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3734, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '恶劣心境', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3735, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '器质性精神障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3736, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '复发性抑郁障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3737, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '惊恐发作', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3738, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '焦虑障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3739, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '适应障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3740, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '特指分离转换性障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3741, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '分离[转换]性障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3742, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '广泛性焦虑障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3743, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '焦虑性神经症', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3744, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '惊恐障碍[间歇发作性焦虑]', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3745, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '精神分裂症', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3746, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '躯体化障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3747, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '躯体形式障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3748, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '双相性情感障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3749, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '分裂情感性障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3750, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '特指抑郁发作', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3751, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '抑郁发作', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3752, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '抑郁症', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3753, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '精神抑郁症', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3754, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '抑郁状态', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3755, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '自主神经功能紊乱', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3756, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '焦虑状态', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3757, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '精神障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3758, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '双相情感障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3759, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '特指的分裂情感性障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3760, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '强迫综合征', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3761, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '植物神经功能紊乱', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3762, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '精神卫生类疾病', NULL, NULL, '心境障碍', '', '', '', '精神卫生科', '精神卫生科', '38', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3763, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '慢性丙型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3764, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3765, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '病毒性肝炎病原携带者', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3766, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '急性病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3767, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '慢性活动型乙型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3768, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '慢性乙型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3769, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '急性丙型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3770, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '急性甲型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3771, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '乙型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3772, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3773, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '戊型病毒性肝炎', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3774, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肝病感染类疾病', NULL, NULL, '乙肝病毒携带者', '', '', '', '肝病感染内科', '肝病感染内科', '31', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3775, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肿瘤相关疾病', NULL, NULL, '肿瘤特殊筛查', '', '', '', '肿瘤内科', '肿瘤内科', '45', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3776, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肿瘤相关疾病', NULL, NULL, '肿瘤待排', '', '', '', '肿瘤内科', '肿瘤内科', '45', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3777, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '肿瘤相关疾病', NULL, NULL, '肿瘤', '', '', '', '肿瘤内科', '肿瘤内科', '45', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3778, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '发热', NULL, NULL, '感染性发热', '', '', '', '发热门诊', '全科医学', '58', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3779, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '发热', NULL, NULL, '发热', '', '', '', '发热门诊', '全科医学', '58', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3780, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '疼痛相关疾病', NULL, NULL, '肢体疼痛', '疼痛门诊', '疼痛门诊', '832', '疼痛专科', '疼痛专科', '832', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3781, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, '疼痛', '疼痛门诊', '疼痛门诊', '832', '疼痛专科', '疼痛专科', '832', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3782, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3783, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3784, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3785, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3786, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3787, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3788, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3789, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3790, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3791, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3792, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3793, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3794, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3795, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3796, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3797, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3798, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3799, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3800, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3801, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3802, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3803, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3804, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3805, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3806, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3807, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3808, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3809, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3810, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3811, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3812, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3813, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3814, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3815, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3816, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3817, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3818, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3819, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3820, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3821, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3822, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3823, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3824, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3825, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3826, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3827, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3828, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3829, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3830, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3831, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3832, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3833, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3834, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3835, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3836, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3837, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3838, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3839, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3840, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3841, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3842, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3843, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3844, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3845, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3846, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3847, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3848, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3849, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3850, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3851, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3852, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3853, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3854, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3855, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3856, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3857, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3858, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3859, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3860, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3861, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3862, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3863, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3864, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3865, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3866, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3867, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3868, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3869, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3870, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3871, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3872, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3873, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3874, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3875, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3876, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3877, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3878, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3879, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3880, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3881, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3882, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3883, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3884, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3885, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3886, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3887, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3888, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3889, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3890, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3891, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3892, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3893, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3894, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3895, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3896, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3897, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3898, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3899, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3900, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3901, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3902, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3903, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3904, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3905, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3906, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3907, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3908, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3909, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3910, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3911, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3912, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3913, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3914, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3915, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3916, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3917, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3918, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3919, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3920, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3921, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3922, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3923, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3924, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3925, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3926, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3927, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3928, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3929, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3930, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3931, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3932, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3933, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3934, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3935, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3936, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3937, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3938, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3939, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3940, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3941, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3942, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3943, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3944, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3945, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3946, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3947, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3948, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3949, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3950, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3951, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3952, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3953, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3954, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3955, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);
INSERT INTO `tran_disease_dept` VALUES (3956, 'N', '1970-01-01 12:00:00', '1970-01-01 12:00:00', '0', '0', '邵逸夫医院', '', NULL, NULL, NULL, '', '', '', '', '', '', '1', '', NULL);

SET FOREIGN_KEY_CHECKS = 1;
